/**
 * @namespace PIXI
 */
declare namespace PIXI {
  /**
   * String of the current PIXI version.
   *
   * @static
   * @constant
   * @memberof PIXI
   * @name VERSION
   * @type {string}
   */
  var VERSION: string;
  /**
   * This namespace contains WebGL-only display filters that can be applied
   * to DisplayObjects using the {@link PIXI.DisplayObject#filters filters} property.
   *
   * Since PixiJS only had a handful of built-in filters, additional filters
   * can be downloaded {@link https://github.com/pixijs/pixi-filters here} from the
   * PixiJS Filters repository.
   *
   * All filters must extend {@link PIXI.Filter}.
   *
   * @example
   * // Create a new application
   * const app = new PIXI.Application();
   *
   * // Draw a green rectangle
   * const rect = new PIXI.Graphics()
   *     .beginFill(0x00ff00)
   *     .drawRect(40, 40, 200, 200);
   *
   * // Add a blur filter
   * rect.filters = [new PIXI.filters.BlurFilter()];
   *
   * // Display rectangle
   * app.stage.addChild(rect);
   * document.body.appendChild(app.view);
   * @namespace PIXI.filters
   */
  namespace filters {
    /**
     * Simplest filter - applies alpha.
     *
     * Use this instead of Container's alpha property to avoid visual layering of individual elements.
     * AlphaFilter applies alpha evenly across the entire display object and any opaque elements it contains.
     * If elements are not opaque, they will blend with each other anyway.
     *
     * Very handy if you want to use common features of all filters:
     *
     * 1. Assign a blendMode to this filter, blend all elements inside display object with background.
     *
     * 2. To use clipping in display coordinates, assign a filterArea to the same container that has this filter.
     *
     * @class
     * @extends PIXI.Filter
     * @memberof PIXI.filters
     */
    class AlphaFilter extends PIXI.Filter {
      constructor(alpha?: number);

      /**
       * Coefficient for alpha multiplication
       *
       * @member {number}
       * @default 1
       */
      alpha: number;
      /**
       * The padding of the filter. Some filters require extra space to breath such as a blur.
       * Increasing this will add extra width and height to the bounds of the object that the
       * filter is applied to.
       *
       * @member {number} PIXI.Filter#padding
       */
      padding: number;
      /**
       * The resolution of the filter. Setting this to be lower will lower the quality but
       * increase the performance of the filter.
       *
       * @member {number} PIXI.Filter#resolution
       */
      resolution: number;
      /**
       * If enabled is true the filter is applied, if false it will not.
       *
       * @member {boolean} PIXI.Filter#enabled
       */
      enabled: boolean;
      /**
       * If enabled, PixiJS will fit the filter area into boundaries for better performance.
       * Switch it off if it does not work for specific shader.
       *
       * @member {boolean} PIXI.Filter#autoFit
       */
      autoFit: boolean;
      /**
       * Legacy filters use position and uvs from attributes
       * @member {boolean} PIXI.Filter#legacy
       * @readonly
       */
      readonly legacy: boolean;
      /**
       * The WebGL state the filter requires to render
       * @member {PIXI.State} PIXI.Filter#state
       */
      state: PIXI.State;

      /**
       * Applies the filter
       *
       * @param {PIXI.systems.FilterSystem} filterManager - The renderer to retrieve the filter from
       * @param {PIXI.RenderTexture} input - The input render target.
       * @param {PIXI.RenderTexture} output - The target to output to.
       * @param {boolean} clear - Should the output be cleared before rendering to it
       * @param {object} [currentState] - It's current state of filter.
       *        There are some useful properties in the currentState :
       *        target, filters, sourceFrame, destinationFrame, renderTarget, resolution
       */
      apply(
        filterManager: PIXI.systems.FilterSystem,
        input: PIXI.RenderTexture,
        output: PIXI.RenderTexture,
        clear: boolean,
        currentState?: any,
      ): void;

      /**
       * Sets the blendmode of the filter
       *
       * @member {number}
       * @default PIXI.BLEND_MODES.NORMAL
       */
      blendMode: number;
      /**
       * Program that the shader uses
       *
       * @member {PIXI.Program} PIXI.Shader#program
       */
      program: PIXI.Program;
      /**
       * Shader uniform values, shortcut for `uniformGroup.uniforms`
       * @readonly
       * @member {object}
       */
      readonly uniforms: any;
    }

    /**
     * The BlurFilter applies a Gaussian blur to an object.
     *
     * The strength of the blur can be set for the x-axis and y-axis separately.
     *
     * @class
     * @extends PIXI.Filter
     * @memberof PIXI.filters
     */
    class BlurFilter extends PIXI.Filter {
      constructor(strength?: number, quality?: number, resolution?: number, kernelSize?: number);

      /**
       * Applies the filter.
       *
       * @param {PIXI.systems.FilterSystem} filterManager - The manager.
       * @param {PIXI.RenderTexture} input - The input target.
       * @param {PIXI.RenderTexture} output - The output target.
       */
      apply(filterManager: PIXI.systems.FilterSystem, input: PIXI.RenderTexture, output: PIXI.RenderTexture): void;

      /**
       * Sets the strength of both the blurX and blurY properties simultaneously
       *
       * @member {number}
       * @default 2
       */
      blur: number;
      /**
       * Sets the number of passes for blur. More passes means higher quaility bluring.
       *
       * @member {number}
       * @default 1
       */
      quality: number;
      /**
       * Sets the strength of the blurX property
       *
       * @member {number}
       * @default 2
       */
      blurX: number;
      /**
       * Sets the strength of the blurY property
       *
       * @member {number}
       * @default 2
       */
      blurY: number;
      /**
       * Sets the blendmode of the filter
       *
       * @member {number}
       * @default PIXI.BLEND_MODES.NORMAL
       */
      blendMode: number;
      /**
       * If set to true the edge of the target will be clamped
       *
       * @member {bool}
       * @default false
       */
      repeatEdgePixels: boolean;
      /**
       * The padding of the filter. Some filters require extra space to breath such as a blur.
       * Increasing this will add extra width and height to the bounds of the object that the
       * filter is applied to.
       *
       * @member {number} PIXI.Filter#padding
       */
      padding: number;
      /**
       * The resolution of the filter. Setting this to be lower will lower the quality but
       * increase the performance of the filter.
       *
       * @member {number} PIXI.Filter#resolution
       */
      resolution: number;
      /**
       * If enabled is true the filter is applied, if false it will not.
       *
       * @member {boolean} PIXI.Filter#enabled
       */
      enabled: boolean;
      /**
       * If enabled, PixiJS will fit the filter area into boundaries for better performance.
       * Switch it off if it does not work for specific shader.
       *
       * @member {boolean} PIXI.Filter#autoFit
       */
      autoFit: boolean;
      /**
       * Legacy filters use position and uvs from attributes
       * @member {boolean} PIXI.Filter#legacy
       * @readonly
       */
      readonly legacy: boolean;
      /**
       * The WebGL state the filter requires to render
       * @member {PIXI.State} PIXI.Filter#state
       */
      state: PIXI.State;
      /**
       * Program that the shader uses
       *
       * @member {PIXI.Program} PIXI.Shader#program
       */
      program: PIXI.Program;
      /**
       * Shader uniform values, shortcut for `uniformGroup.uniforms`
       * @readonly
       * @member {object}
       */
      readonly uniforms: any;
    }

    /**
     * The BlurFilterPass applies a horizontal or vertical Gaussian blur to an object.
     *
     * @class
     * @extends PIXI.Filter
     * @memberof PIXI.filters
     */
    class BlurFilterPass extends PIXI.Filter {
      constructor(horizontal: boolean, strength: number, quality: number, resolution: number, kernelSize?: number);

      /**
       * Sets the strength of both the blur.
       *
       * @member {number}
       * @default 16
       */
      blur: number;
      /**
       * Sets the quality of the blur by modifying the number of passes. More passes means higher
       * quaility bluring but the lower the performance.
       *
       * @member {number}
       * @default 4
       */
      quality: number;
      /**
       * The padding of the filter. Some filters require extra space to breath such as a blur.
       * Increasing this will add extra width and height to the bounds of the object that the
       * filter is applied to.
       *
       * @member {number} PIXI.Filter#padding
       */
      padding: number;
      /**
       * The resolution of the filter. Setting this to be lower will lower the quality but
       * increase the performance of the filter.
       *
       * @member {number} PIXI.Filter#resolution
       */
      resolution: number;
      /**
       * If enabled is true the filter is applied, if false it will not.
       *
       * @member {boolean} PIXI.Filter#enabled
       */
      enabled: boolean;
      /**
       * If enabled, PixiJS will fit the filter area into boundaries for better performance.
       * Switch it off if it does not work for specific shader.
       *
       * @member {boolean} PIXI.Filter#autoFit
       */
      autoFit: boolean;
      /**
       * Legacy filters use position and uvs from attributes
       * @member {boolean} PIXI.Filter#legacy
       * @readonly
       */
      readonly legacy: boolean;
      /**
       * The WebGL state the filter requires to render
       * @member {PIXI.State} PIXI.Filter#state
       */
      state: PIXI.State;

      /**
       * Applies the filter
       *
       * @param {PIXI.systems.FilterSystem} filterManager - The renderer to retrieve the filter from
       * @param {PIXI.RenderTexture} input - The input render target.
       * @param {PIXI.RenderTexture} output - The target to output to.
       * @param {boolean} clear - Should the output be cleared before rendering to it
       * @param {object} [currentState] - It's current state of filter.
       *        There are some useful properties in the currentState :
       *        target, filters, sourceFrame, destinationFrame, renderTarget, resolution
       */
      apply(
        filterManager: PIXI.systems.FilterSystem,
        input: PIXI.RenderTexture,
        output: PIXI.RenderTexture,
        clear: boolean,
        currentState?: any,
      ): void;

      /**
       * Sets the blendmode of the filter
       *
       * @member {number}
       * @default PIXI.BLEND_MODES.NORMAL
       */
      blendMode: number;
      /**
       * Program that the shader uses
       *
       * @member {PIXI.Program} PIXI.Shader#program
       */
      program: PIXI.Program;
      /**
       * Shader uniform values, shortcut for `uniformGroup.uniforms`
       * @readonly
       * @member {object}
       */
      readonly uniforms: any;
    }

    /**
     * The ColorMatrixFilter class lets you apply a 5x4 matrix transformation on the RGBA
     * color and alpha values of every pixel on your displayObject to produce a result
     * with a new set of RGBA color and alpha values. It's pretty powerful!
     *
     * ```js
     *  let colorMatrix = new PIXI.filters.ColorMatrixFilter();
     *  container.filters = [colorMatrix];
     *  colorMatrix.contrast(2);
     * ```
     * @author Clément Chenebault <clement@goodboydigital.com>
     * @class
     * @extends PIXI.Filter
     * @memberof PIXI.filters
     */
    class ColorMatrixFilter extends PIXI.Filter {
      constructor();

      /**
       * Transforms current matrix and set the new one
       *
       * @param {number[]} matrix - 5x4 matrix
       * @param {boolean} multiply - if true, current matrix and matrix are multiplied. If false,
       *  just set the current matrix with @param matrix
       */
      _loadMatrix(matrix: number[], multiply: boolean): void;

      /**
       * Adjusts brightness
       *
       * @param {number} b - value of the brigthness (0-1, where 0 is black)
       * @param {boolean} multiply - if true, current matrix and matrix are multiplied. If false,
       *  just set the current matrix with @param matrix
       */
      brightness(b: number, multiply: boolean): void;

      /**
       * Set the matrices in grey scales
       *
       * @param {number} scale - value of the grey (0-1, where 0 is black)
       * @param {boolean} multiply - if true, current matrix and matrix are multiplied. If false,
       *  just set the current matrix with @param matrix
       */
      greyscale(scale: number, multiply: boolean): void;

      /**
       * Set the black and white matrice.
       *
       * @param {boolean} multiply - if true, current matrix and matrix are multiplied. If false,
       *  just set the current matrix with @param matrix
       */
      blackAndWhite(multiply: boolean): void;

      /**
       * Set the hue property of the color
       *
       * @param {number} rotation - in degrees
       * @param {boolean} multiply - if true, current matrix and matrix are multiplied. If false,
       *  just set the current matrix with @param matrix
       */
      hue(rotation: number, multiply: boolean): void;

      /**
       * Set the contrast matrix, increase the separation between dark and bright
       * Increase contrast : shadows darker and highlights brighter
       * Decrease contrast : bring the shadows up and the highlights down
       *
       * @param {number} amount - value of the contrast (0-1)
       * @param {boolean} multiply - if true, current matrix and matrix are multiplied. If false,
       *  just set the current matrix with @param matrix
       */
      contrast(amount: number, multiply: boolean): void;

      /**
       * Set the saturation matrix, increase the separation between colors
       * Increase saturation : increase contrast, brightness, and sharpness
       *
       * @param {number} amount - The saturation amount (0-1)
       * @param {boolean} multiply - if true, current matrix and matrix are multiplied. If false,
       *  just set the current matrix with @param matrix
       */
      saturate(amount: number, multiply: boolean): void;

      /**
       * Desaturate image (remove color)
       *
       * Call the saturate function
       *
       */
      desaturate(): void;

      /**
       * Negative image (inverse of classic rgb matrix)
       *
       * @param {boolean} multiply - if true, current matrix and matrix are multiplied. If false,
       *  just set the current matrix with @param matrix
       */
      negative(multiply: boolean): void;

      /**
       * Sepia image
       *
       * @param {boolean} multiply - if true, current matrix and matrix are multiplied. If false,
       *  just set the current matrix with @param matrix
       */
      sepia(multiply: boolean): void;

      /**
       * Color motion picture process invented in 1916 (thanks Dominic Szablewski)
       *
       * @param {boolean} multiply - if true, current matrix and matrix are multiplied. If false,
       *  just set the current matrix with @param matrix
       */
      technicolor(multiply: boolean): void;

      /**
       * Polaroid filter
       *
       * @param {boolean} multiply - if true, current matrix and matrix are multiplied. If false,
       *  just set the current matrix with @param matrix
       */
      polaroid(multiply: boolean): void;

      /**
       * Filter who transforms : Red -> Blue and Blue -> Red
       *
       * @param {boolean} multiply - if true, current matrix and matrix are multiplied. If false,
       *  just set the current matrix with @param matrix
       */
      toBGR(multiply: boolean): void;

      /**
       * Color reversal film introduced by Eastman Kodak in 1935. (thanks Dominic Szablewski)
       *
       * @param {boolean} multiply - if true, current matrix and matrix are multiplied. If false,
       *  just set the current matrix with @param matrix
       */
      kodachrome(multiply: boolean): void;

      /**
       * Brown delicious browni filter (thanks Dominic Szablewski)
       *
       * @param {boolean} multiply - if true, current matrix and matrix are multiplied. If false,
       *  just set the current matrix with @param matrix
       */
      browni(multiply: boolean): void;

      /**
       * Vintage filter (thanks Dominic Szablewski)
       *
       * @param {boolean} multiply - if true, current matrix and matrix are multiplied. If false,
       *  just set the current matrix with @param matrix
       */
      vintage(multiply: boolean): void;

      /**
       * We don't know exactly what it does, kind of gradient map, but funny to play with!
       *
       * @param {number} desaturation - Tone values.
       * @param {number} toned - Tone values.
       * @param {string} lightColor - Tone values, example: `0xFFE580`
       * @param {string} darkColor - Tone values, example: `0xFFE580`
       * @param {boolean} multiply - if true, current matrix and matrix are multiplied. If false,
       *  just set the current matrix with @param matrix
       */
      colorTone(desaturation: number, toned: number, lightColor: string, darkColor: string, multiply: boolean): void;

      /**
       * Night effect
       *
       * @param {number} intensity - The intensity of the night effect.
       * @param {boolean} multiply - if true, current matrix and matrix are multiplied. If false,
       *  just set the current matrix with @param matrix
       */
      night(intensity: number, multiply: boolean): void;

      /**
       * Predator effect
       *
       * Erase the current matrix by setting a new indepent one
       *
       * @param {number} amount - how much the predator feels his future victim
       * @param {boolean} multiply - if true, current matrix and matrix are multiplied. If false,
       *  just set the current matrix with @param matrix
       */
      predator(amount: number, multiply: boolean): void;

      /**
       * LSD effect
       *
       * Multiply the current matrix
       *
       * @param {boolean} multiply - if true, current matrix and matrix are multiplied. If false,
       *  just set the current matrix with @param matrix
       */
      lsd(multiply: boolean): void;

      /**
       * Erase the current matrix by setting the default one
       *
       */
      reset(): void;

      /**
       * The matrix of the color matrix filter
       *
       * @member {number[]}
       * @default [1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0]
       */
      matrix: number[];
      /**
       * The opacity value to use when mixing the original and resultant colors.
       *
       * When the value is 0, the original color is used without modification.
       * When the value is 1, the result color is used.
       * When in the range (0, 1) the color is interpolated between the original and result by this amount.
       *
       * @member {number}
       * @default 1
       */
      alpha: number;
      /**
       * The padding of the filter. Some filters require extra space to breath such as a blur.
       * Increasing this will add extra width and height to the bounds of the object that the
       * filter is applied to.
       *
       * @member {number} PIXI.Filter#padding
       */
      padding: number;
      /**
       * The resolution of the filter. Setting this to be lower will lower the quality but
       * increase the performance of the filter.
       *
       * @member {number} PIXI.Filter#resolution
       */
      resolution: number;
      /**
       * If enabled is true the filter is applied, if false it will not.
       *
       * @member {boolean} PIXI.Filter#enabled
       */
      enabled: boolean;
      /**
       * If enabled, PixiJS will fit the filter area into boundaries for better performance.
       * Switch it off if it does not work for specific shader.
       *
       * @member {boolean} PIXI.Filter#autoFit
       */
      autoFit: boolean;
      /**
       * Legacy filters use position and uvs from attributes
       * @member {boolean} PIXI.Filter#legacy
       * @readonly
       */
      readonly legacy: boolean;
      /**
       * The WebGL state the filter requires to render
       * @member {PIXI.State} PIXI.Filter#state
       */
      state: PIXI.State;

      /**
       * Applies the filter
       *
       * @param {PIXI.systems.FilterSystem} filterManager - The renderer to retrieve the filter from
       * @param {PIXI.RenderTexture} input - The input render target.
       * @param {PIXI.RenderTexture} output - The target to output to.
       * @param {boolean} clear - Should the output be cleared before rendering to it
       * @param {object} [currentState] - It's current state of filter.
       *        There are some useful properties in the currentState :
       *        target, filters, sourceFrame, destinationFrame, renderTarget, resolution
       */
      apply(
        filterManager: PIXI.systems.FilterSystem,
        input: PIXI.RenderTexture,
        output: PIXI.RenderTexture,
        clear: boolean,
        currentState?: any,
      ): void;

      /**
       * Sets the blendmode of the filter
       *
       * @member {number}
       * @default PIXI.BLEND_MODES.NORMAL
       */
      blendMode: number;
      /**
       * Program that the shader uses
       *
       * @member {PIXI.Program} PIXI.Shader#program
       */
      program: PIXI.Program;
      /**
       * Shader uniform values, shortcut for `uniformGroup.uniforms`
       * @readonly
       * @member {object}
       */
      readonly uniforms: any;
    }

    /**
     * The DisplacementFilter class uses the pixel values from the specified texture
     * (called the displacement map) to perform a displacement of an object.
     *
     * You can use this filter to apply all manor of crazy warping effects.
     * Currently the `r` property of the texture is used to offset the `x`
     * and the `g` property of the texture is used to offset the `y`.
     *
     * The way it works is it uses the values of the displacement map to look up the
     * correct pixels to output. This means it's not technically moving the original.
     * Instead, it's starting at the output and asking "which pixel from the original goes here".
     * For example, if a displacement map pixel has `red = 1` and the filter scale is `20`,
     * this filter will output the pixel approximately 20 pixels to the right of the original.
     *
     * @class
     * @extends PIXI.Filter
     * @memberof PIXI.filters
     */
    class DisplacementFilter extends PIXI.Filter {
      constructor(sprite: PIXI.Sprite, scale?: number);

      /**
       * scaleX, scaleY for displacements
       * @member {PIXI.Point} PIXI.filters.DisplacementFilter#scale
       */
      scale: PIXI.Point;

      /**
       * Applies the filter.
       *
       * @param {PIXI.systems.FilterSystem} filterManager - The manager.
       * @param {PIXI.RenderTexture} input - The input target.
       * @param {PIXI.RenderTexture} output - The output target.
       * @param {boolean} clear - Should the output be cleared before rendering to it.
       */
      apply(
        filterManager: PIXI.systems.FilterSystem,
        input: PIXI.RenderTexture,
        output: PIXI.RenderTexture,
        clear: boolean,
      ): void;

      /**
       * The texture used for the displacement map. Must be power of 2 sized texture.
       *
       * @member {PIXI.Texture}
       */
      map: PIXI.Texture;
      /**
       * The padding of the filter. Some filters require extra space to breath such as a blur.
       * Increasing this will add extra width and height to the bounds of the object that the
       * filter is applied to.
       *
       * @member {number} PIXI.Filter#padding
       */
      padding: number;
      /**
       * The resolution of the filter. Setting this to be lower will lower the quality but
       * increase the performance of the filter.
       *
       * @member {number} PIXI.Filter#resolution
       */
      resolution: number;
      /**
       * If enabled is true the filter is applied, if false it will not.
       *
       * @member {boolean} PIXI.Filter#enabled
       */
      enabled: boolean;
      /**
       * If enabled, PixiJS will fit the filter area into boundaries for better performance.
       * Switch it off if it does not work for specific shader.
       *
       * @member {boolean} PIXI.Filter#autoFit
       */
      autoFit: boolean;
      /**
       * Legacy filters use position and uvs from attributes
       * @member {boolean} PIXI.Filter#legacy
       * @readonly
       */
      readonly legacy: boolean;
      /**
       * The WebGL state the filter requires to render
       * @member {PIXI.State} PIXI.Filter#state
       */
      state: PIXI.State;
      /**
       * Sets the blendmode of the filter
       *
       * @member {number}
       * @default PIXI.BLEND_MODES.NORMAL
       */
      blendMode: number;
      /**
       * Program that the shader uses
       *
       * @member {PIXI.Program} PIXI.Shader#program
       */
      program: PIXI.Program;
      /**
       * Shader uniform values, shortcut for `uniformGroup.uniforms`
       * @readonly
       * @member {object}
       */
      readonly uniforms: any;
    }

    /**
     * Basic FXAA (Fast Approximate Anti-Aliasing) implementation based on the code on geeks3d.com
     * with the modification that the texture2DLod stuff was removed since it is unsupported by WebGL.
     *
     * @see https://github.com/mitsuhiko/webgl-meincraft
     *
     * @class
     * @extends PIXI.Filter
     * @memberof PIXI.filters
     *
     */
    class FXAAFilter extends PIXI.Filter {
      constructor();

      /**
       * The padding of the filter. Some filters require extra space to breath such as a blur.
       * Increasing this will add extra width and height to the bounds of the object that the
       * filter is applied to.
       *
       * @member {number} PIXI.Filter#padding
       */
      padding: number;
      /**
       * The resolution of the filter. Setting this to be lower will lower the quality but
       * increase the performance of the filter.
       *
       * @member {number} PIXI.Filter#resolution
       */
      resolution: number;
      /**
       * If enabled is true the filter is applied, if false it will not.
       *
       * @member {boolean} PIXI.Filter#enabled
       */
      enabled: boolean;
      /**
       * If enabled, PixiJS will fit the filter area into boundaries for better performance.
       * Switch it off if it does not work for specific shader.
       *
       * @member {boolean} PIXI.Filter#autoFit
       */
      autoFit: boolean;
      /**
       * Legacy filters use position and uvs from attributes
       * @member {boolean} PIXI.Filter#legacy
       * @readonly
       */
      readonly legacy: boolean;
      /**
       * The WebGL state the filter requires to render
       * @member {PIXI.State} PIXI.Filter#state
       */
      state: PIXI.State;

      /**
       * Applies the filter
       *
       * @param {PIXI.systems.FilterSystem} filterManager - The renderer to retrieve the filter from
       * @param {PIXI.RenderTexture} input - The input render target.
       * @param {PIXI.RenderTexture} output - The target to output to.
       * @param {boolean} clear - Should the output be cleared before rendering to it
       * @param {object} [currentState] - It's current state of filter.
       *        There are some useful properties in the currentState :
       *        target, filters, sourceFrame, destinationFrame, renderTarget, resolution
       */
      apply(
        filterManager: PIXI.systems.FilterSystem,
        input: PIXI.RenderTexture,
        output: PIXI.RenderTexture,
        clear: boolean,
        currentState?: any,
      ): void;

      /**
       * Sets the blendmode of the filter
       *
       * @member {number}
       * @default PIXI.BLEND_MODES.NORMAL
       */
      blendMode: number;
      /**
       * Program that the shader uses
       *
       * @member {PIXI.Program} PIXI.Shader#program
       */
      program: PIXI.Program;
      /**
       * Shader uniform values, shortcut for `uniformGroup.uniforms`
       * @readonly
       * @member {object}
       */
      readonly uniforms: any;
    }

    /**
     * A Noise effect filter.
     *
     * @class
     * @extends PIXI.Filter
     * @memberof PIXI.filters
     */
    class NoiseFilter extends PIXI.Filter {
      constructor(noise?: number, seed?: number);

      /**
       * The amount of noise to apply, this value should be in the range (0, 1].
       *
       * @member {number}
       * @default 0.5
       */
      noise: number;
      /**
       * A seed value to apply to the random noise generation. `Math.random()` is a good value to use.
       *
       * @member {number}
       */
      seed: number;
      /**
       * The padding of the filter. Some filters require extra space to breath such as a blur.
       * Increasing this will add extra width and height to the bounds of the object that the
       * filter is applied to.
       *
       * @member {number} PIXI.Filter#padding
       */
      padding: number;
      /**
       * The resolution of the filter. Setting this to be lower will lower the quality but
       * increase the performance of the filter.
       *
       * @member {number} PIXI.Filter#resolution
       */
      resolution: number;
      /**
       * If enabled is true the filter is applied, if false it will not.
       *
       * @member {boolean} PIXI.Filter#enabled
       */
      enabled: boolean;
      /**
       * If enabled, PixiJS will fit the filter area into boundaries for better performance.
       * Switch it off if it does not work for specific shader.
       *
       * @member {boolean} PIXI.Filter#autoFit
       */
      autoFit: boolean;
      /**
       * Legacy filters use position and uvs from attributes
       * @member {boolean} PIXI.Filter#legacy
       * @readonly
       */
      readonly legacy: boolean;
      /**
       * The WebGL state the filter requires to render
       * @member {PIXI.State} PIXI.Filter#state
       */
      state: PIXI.State;

      /**
       * Applies the filter
       *
       * @param {PIXI.systems.FilterSystem} filterManager - The renderer to retrieve the filter from
       * @param {PIXI.RenderTexture} input - The input render target.
       * @param {PIXI.RenderTexture} output - The target to output to.
       * @param {boolean} clear - Should the output be cleared before rendering to it
       * @param {object} [currentState] - It's current state of filter.
       *        There are some useful properties in the currentState :
       *        target, filters, sourceFrame, destinationFrame, renderTarget, resolution
       */
      apply(
        filterManager: PIXI.systems.FilterSystem,
        input: PIXI.RenderTexture,
        output: PIXI.RenderTexture,
        clear: boolean,
        currentState?: any,
      ): void;

      /**
       * Sets the blendmode of the filter
       *
       * @member {number}
       * @default PIXI.BLEND_MODES.NORMAL
       */
      blendMode: number;
      /**
       * Program that the shader uses
       *
       * @member {PIXI.Program} PIXI.Shader#program
       */
      program: PIXI.Program;
      /**
       * Shader uniform values, shortcut for `uniformGroup.uniforms`
       * @readonly
       * @member {object}
       */
      readonly uniforms: any;
    }
  }
  /**
   * This namespace contains an accessibility plugin for allowing interaction via the keyboard.
   *
   * Do not instantiate this plugin directly. It is available from the `renderer.plugins` property.
   * See {@link PIXI.CanvasRenderer#plugins} or {@link PIXI.Renderer#plugins}.
   * @namespace PIXI.accessibility
   */
  namespace accessibility {
    /**
     * The Accessibility manager recreates the ability to tab and have content read by screen readers.
     * This is very important as it can possibly help people with disabilities access PixiJS content.
     *
     * A DisplayObject can be made accessible just like it can be made interactive. This manager will map the
     * events as if the mouse was being used, minimizing the effort required to implement.
     *
     * An instance of this class is automatically created by default, and can be found at `renderer.plugins.accessibility`
     *
     * @class
     * @memberof PIXI.accessibility
     */
    class AccessibilityManager {
      constructor(renderer: PIXI.Renderer);

      /**
       * Setting this to true will visually show the divs.
       *
       * @type {boolean}
       */
      debug: boolean;
      /**
       * The renderer this accessibility manager works for.
       *
       * @member {PIXI.AbstractRenderer} PIXI.accessibility.AccessibilityManager#renderer
       */
      renderer: PIXI.AbstractRenderer;
      /**
       * A flag
       * @type {boolean}
       * @readonly
       */
      readonly isActive: boolean;
      /**
       * A flag
       * @type {boolean}
       * @readonly
       */
      readonly isMobileAccessibility: boolean;

      /**
       * private function that will visually add the information to the
       * accessability div
       *
       * @param {HTMLDivElement} div
       */
      updateDebugHTML(div: HTMLDivElement): void;

      /**
       * Adjust the hit area based on the bounds of a display object
       *
       * @param {PIXI.Rectangle} hitArea - Bounds of the child
       */
      capHitArea(hitArea: PIXI.Rectangle): void;

      /**
       * Destroys the accessibility manager
       *
       */
      destroy(): void;
    }
  }

  /**
   * Convenience class to create a new PIXI application.
   *
   * This class automatically creates the renderer, ticker and root container.
   *
   * @example
   * // Create the application
   * const app = new PIXI.Application();
   *
   * // Add the view to the DOM
   * document.body.appendChild(app.view);
   *
   * // ex, add display objects
   * app.stage.addChild(PIXI.Sprite.from('something.png'));
   *
   * @class
   * @memberof PIXI
   */
  class Application {
    constructor(options?: {
      autoStart?: boolean;
      width?: number;
      height?: number;
      view?: HTMLCanvasElement;
      transparent?: boolean;
      autoDensity?: boolean;
      antialias?: boolean;
      preserveDrawingBuffer?: boolean;
      resolution?: number;
      forceCanvas?: boolean;
      backgroundColor?: number;
      clearBeforeRender?: boolean;
      forceFXAA?: boolean;
      powerPreference?: string;
      sharedTicker?: boolean;
      sharedLoader?: boolean;
      resizeTo?: Window | HTMLElement;
    });

    /**
     * WebGL renderer if available, otherwise CanvasRenderer.
     * @member {PIXI.Renderer|PIXI.CanvasRenderer} PIXI.Application#renderer
     */
    renderer: PIXI.Renderer;
    /**
     * The root display container that's rendered.
     * @member {PIXI.Container} PIXI.Application#stage
     */
    stage: PIXI.Container;

    /**
     * Register a middleware plugin for the application
     * @static
     * @param {PIXI.Application.Plugin} plugin - Plugin being installed
     */
    static registerPlugin(plugin: PIXI.Application.Plugin): void;

    /**
     * Render the current stage.
     */
    render(): void;

    /**
     * Reference to the renderer's canvas element.
     * @member {HTMLCanvasElement}
     * @readonly
     */
    readonly view: HTMLCanvasElement;
    /**
     * Reference to the renderer's screen rectangle. Its safe to use as `filterArea` or `hitArea` for the whole screen.
     * @member {PIXI.Rectangle}
     * @readonly
     */
    readonly screen: PIXI.Rectangle;

    /**
     * Destroy and don't use after this.
     * @param {Boolean} [removeView=false] Automatically remove canvas from DOM.
     * @param {object|boolean} [stageOptions] - Options parameter. A boolean will act as if all options
     *  have been set to that value
     * @param {boolean} [stageOptions.children=false] - if set to true, all the children will have their destroy
     *  method called as well. 'stageOptions' will be passed on to those calls.
     * @param {boolean} [stageOptions.texture=false] - Only used for child Sprites if stageOptions.children is set
     *  to true. Should it destroy the texture of the child sprite
     * @param {boolean} [stageOptions.baseTexture=false] - Only used for child Sprites if stageOptions.children is set
     *  to true. Should it destroy the base texture of the child sprite
     */
    destroy(
      removeView?: boolean,
      stageOptions?: {
        children?: boolean;
        texture?: boolean;
        baseTexture?: boolean;
      },
    ): void;

    /**
     * The element or window to resize the application to.
     * @type {Window|HTMLElement}
     * @name resizeTo
     * @memberof PIXI.Application#
     */
    resizeTo: Window | HTMLElement;

    /**
     * If `resizeTo` is set, calling this function
     * will resize to the width and height of that element.
     * @method PIXI.Application#resize
     */
    resize(): void;

    /**
     * Loader instance to help with asset loading.
     * @name PIXI.Application#loader
     * @type {PIXI.Loader}
     * @readonly
     */
    readonly loader: PIXI.Loader;

    /**
     * Convenience method for stopping the render.
     *
     * @method PIXI.Application#stop
     */
    stop(): void;

    /**
     * Convenience method for starting the render.
     *
     * @method PIXI.Application#start
     */
    start(): void;

    /**
     * Ticker for doing render updates.
     *
     * @type {PIXI.Ticker}
     * @name ticker
     * @memberof PIXI.Application#
     * @default PIXI.Ticker.shared
     */
    ticker: PIXI.Ticker;
  }

  namespace Application {
    /**
     * @memberof PIXI.Application
     * @typedef {object} Plugin
     * @property {function} init - Called when Application is constructed, scoped to Application instance.
     *  Passes in `options` as the only argument, which are Application constructor options.
     * @property {function} destroy - Called when destroying Application, scoped to Application instance
     */
    type Plugin = {
      init: (...params: any[]) => any;
      destroy: (...params: any[]) => any;
    };
  }

  /**
   * Different types of environments for WebGL.
   *
   * @static
   * @memberof PIXI
   * @name ENV
   * @enum {number}
   * @property {number} WEBGL_LEGACY - Used for older v1 WebGL devices. PixiJS will aim to ensure compatibility
   *  with older / less advanced devices. If you experience unexplained flickering prefer this environment.
   * @property {number} WEBGL - Version 1 of WebGL
   * @property {number} WEBGL2 - Version 2 of WebGL
   */
  enum ENV {
    WEBGL_LEGACY,
    WEBGL,
    WEBGL2,
  }

  /**
   * Constant to identify the Renderer Type.
   *
   * @static
   * @memberof PIXI
   * @name RENDERER_TYPE
   * @enum {number}
   * @property {number} UNKNOWN - Unknown render type.
   * @property {number} WEBGL - WebGL render type.
   * @property {number} CANVAS - Canvas render type.
   */
  enum RENDERER_TYPE {
    UNKNOWN,
    WEBGL,
    CANVAS,
  }

  /**
   * Various blend modes supported by PIXI.
   *
   * IMPORTANT - The WebGL renderer only supports the NORMAL, ADD, MULTIPLY and SCREEN blend modes.
   * Anything else will silently act like NORMAL.
   *
   * @memberof PIXI
   * @name BLEND_MODES
   * @enum {number}
   * @property {number} NORMAL
   * @property {number} ADD
   * @property {number} MULTIPLY
   * @property {number} SCREEN
   * @property {number} OVERLAY
   * @property {number} DARKEN
   * @property {number} LIGHTEN
   * @property {number} COLOR_DODGE
   * @property {number} COLOR_BURN
   * @property {number} HARD_LIGHT
   * @property {number} SOFT_LIGHT
   * @property {number} DIFFERENCE
   * @property {number} EXCLUSION
   * @property {number} HUE
   * @property {number} SATURATION
   * @property {number} COLOR
   * @property {number} LUMINOSITY
   * @property {number} NORMAL_NPM
   * @property {number} ADD_NPM
   * @property {number} SCREEN_NPM
   * @property {number} NONE
   * @property {number} SRC_IN
   * @property {number} SRC_OUT
   * @property {number} SRC_ATOP
   * @property {number} DST_OVER
   * @property {number} DST_IN
   * @property {number} DST_OUT
   * @property {number} DST_ATOP
   * @property {number} SUBTRACT
   * @property {number} SRC_OVER
   * @property {number} ERASE
   * @property {number} XOR
   */
  enum BLEND_MODES {
    NORMAL,
    ADD,
    MULTIPLY,
    SCREEN,
    OVERLAY,
    DARKEN,
    LIGHTEN,
    COLOR_DODGE,
    COLOR_BURN,
    HARD_LIGHT,
    SOFT_LIGHT,
    DIFFERENCE,
    EXCLUSION,
    HUE,
    SATURATION,
    COLOR,
    LUMINOSITY,
    NORMAL_NPM,
    ADD_NPM,
    SCREEN_NPM,
    NONE,
    SRC_IN,
    SRC_OUT,
    SRC_ATOP,
    DST_OVER,
    DST_IN,
    DST_OUT,
    DST_ATOP,
    SUBTRACT,
    SRC_OVER,
    ERASE,
    XOR,
  }

  /**
   * Various webgl draw modes. These can be used to specify which GL drawMode to use
   * under certain situations and renderers.
   *
   * @memberof PIXI
   * @static
   * @name DRAW_MODES
   * @enum {number}
   * @property {number} POINTS
   * @property {number} LINES
   * @property {number} LINE_LOOP
   * @property {number} LINE_STRIP
   * @property {number} TRIANGLES
   * @property {number} TRIANGLE_STRIP
   * @property {number} TRIANGLE_FAN
   */
  enum DRAW_MODES {
    POINTS,
    LINES,
    LINE_LOOP,
    LINE_STRIP,
    TRIANGLES,
    TRIANGLE_STRIP,
    TRIANGLE_FAN,
  }

  /**
   * Various GL texture/resources formats.
   *
   * @memberof PIXI
   * @static
   * @name FORMATS
   * @enum {number}
   * @property {number} RGBA=6408
   * @property {number} RGB=6407
   * @property {number} ALPHA=6406
   * @property {number} LUMINANCE=6409
   * @property {number} LUMINANCE_ALPHA=6410
   * @property {number} DEPTH_COMPONENT=6402
   * @property {number} DEPTH_STENCIL=34041
   */
  enum FORMATS {
    RGBA,
    RGB,
    ALPHA,
    LUMINANCE,
    LUMINANCE_ALPHA,
    DEPTH_COMPONENT,
    DEPTH_STENCIL,
  }

  /**
   * Various GL target types.
   *
   * @memberof PIXI
   * @static
   * @name TARGETS
   * @enum {number}
   * @property {number} TEXTURE_2D=3553
   * @property {number} TEXTURE_CUBE_MAP=34067
   * @property {number} TEXTURE_2D_ARRAY=35866
   * @property {number} TEXTURE_CUBE_MAP_POSITIVE_X=34069
   * @property {number} TEXTURE_CUBE_MAP_NEGATIVE_X=34070
   * @property {number} TEXTURE_CUBE_MAP_POSITIVE_Y=34071
   * @property {number} TEXTURE_CUBE_MAP_NEGATIVE_Y=34072
   * @property {number} TEXTURE_CUBE_MAP_POSITIVE_Z=34073
   * @property {number} TEXTURE_CUBE_MAP_NEGATIVE_Z=34074
   */
  enum TARGETS {
    TEXTURE_2D,
    TEXTURE_CUBE_MAP,
    TEXTURE_2D_ARRAY,
    TEXTURE_CUBE_MAP_POSITIVE_X,
    TEXTURE_CUBE_MAP_NEGATIVE_X,
    TEXTURE_CUBE_MAP_POSITIVE_Y,
    TEXTURE_CUBE_MAP_NEGATIVE_Y,
    TEXTURE_CUBE_MAP_POSITIVE_Z,
    TEXTURE_CUBE_MAP_NEGATIVE_Z,
  }

  /**
   * Various GL data format types.
   *
   * @memberof PIXI
   * @static
   * @name TYPES
   * @enum {number}
   * @property {number} UNSIGNED_BYTE=5121
   * @property {number} UNSIGNED_SHORT=5123
   * @property {number} UNSIGNED_SHORT_5_6_5=33635
   * @property {number} UNSIGNED_SHORT_4_4_4_4=32819
   * @property {number} UNSIGNED_SHORT_5_5_5_1=32820
   * @property {number} FLOAT=5126
   * @property {number} HALF_FLOAT=36193
   */
  enum TYPES {
    UNSIGNED_BYTE,
    UNSIGNED_SHORT,
    UNSIGNED_SHORT_5_6_5,
    UNSIGNED_SHORT_4_4_4_4,
    UNSIGNED_SHORT_5_5_5_1,
    FLOAT,
    HALF_FLOAT,
  }

  /**
   * The scale modes that are supported by pixi.
   *
   * The {@link PIXI.settings.SCALE_MODE} scale mode affects the default scaling mode of future operations.
   * It can be re-assigned to either LINEAR or NEAREST, depending upon suitability.
   *
   * @memberof PIXI
   * @static
   * @name SCALE_MODES
   * @enum {number}
   * @property {number} LINEAR Smooth scaling
   * @property {number} NEAREST Pixelating scaling
   */
  enum SCALE_MODES {
    LINEAR,
    NEAREST,
  }

  /**
   * The wrap modes that are supported by pixi.
   *
   * The {@link PIXI.settings.WRAP_MODE} wrap mode affects the default wrapping mode of future operations.
   * It can be re-assigned to either CLAMP or REPEAT, depending upon suitability.
   * If the texture is non power of two then clamp will be used regardless as WebGL can
   * only use REPEAT if the texture is po2.
   *
   * This property only affects WebGL.
   *
   * @name WRAP_MODES
   * @memberof PIXI
   * @static
   * @enum {number}
   * @property {number} CLAMP - The textures uvs are clamped
   * @property {number} REPEAT - The texture uvs tile and repeat
   * @property {number} MIRRORED_REPEAT - The texture uvs tile and repeat with mirroring
   */
  enum WRAP_MODES {
    CLAMP,
    REPEAT,
    MIRRORED_REPEAT,
  }

  /**
   * Mipmap filtering modes that are supported by pixi.
   *
   * The {@link PIXI.settings.MIPMAP_TEXTURES} affects default texture filtering.
   * Mipmaps are generated for a baseTexture if its `mipmap` field is `ON`,
   * or its `POW2` and texture dimensions are powers of 2.
   * Due to platform restriction, `ON` option will work like `POW2` for webgl-1.
   *
   * This property only affects WebGL.
   *
   * @name MIPMAP_MODES
   * @memberof PIXI
   * @static
   * @enum {number}
   * @property {number} OFF - No mipmaps
   * @property {number} POW2 - Generate mipmaps if texture dimensions are pow2
   * @property {number} ON - Always generate mipmaps
   */
  enum MIPMAP_MODES {
    OFF,
    POW2,
    ON,
  }

  /**
   * How to treat textures with premultiplied alpha
   *
   * @name ALPHA_MODES
   * @memberof PIXI
   * @static
   * @enum {number}
   * @property {number} NO_PREMULTIPLIED_ALPHA - Source is not premultiplied, leave it like that.
   *  Option for compressed and data textures that are created from typed arrays.
   * @property {number} PREMULTIPLY_ON_UPLOAD - Source is not premultiplied, premultiply on upload.
   *  Default option, used for all loaded images.
   * @property {number} PREMULTIPLIED_ALPHA - Source is already premultiplied
   *  Example: spine atlases with `_pma` suffix.
   * @property {number} NPM - Alias for NO_PREMULTIPLIED_ALPHA.
   * @property {number} UNPACK - Default option, alias for PREMULTIPLY_ON_UPLOAD.
   * @property {number} PMA - Alias for PREMULTIPLIED_ALPHA.
   */
  enum ALPHA_MODES {
    NO_PREMULTIPLIED_ALPHA,
    PREMULTIPLY_ON_UPLOAD,
    PREMULTIPLIED_ALPHA,
    NPM,
    UNPACK,
    PMA,
  }

  /**
   * The gc modes that are supported by pixi.
   *
   * The {@link PIXI.settings.GC_MODE} Garbage Collection mode for PixiJS textures is AUTO
   * If set to GC_MODE, the renderer will occasionally check textures usage. If they are not
   * used for a specified period of time they will be removed from the GPU. They will of course
   * be uploaded again when they are required. This is a silent behind the scenes process that
   * should ensure that the GPU does not  get filled up.
   *
   * Handy for mobile devices!
   * This property only affects WebGL.
   *
   * @name GC_MODES
   * @enum {number}
   * @static
   * @memberof PIXI
   * @property {number} AUTO - Garbage collection will happen periodically automatically
   * @property {number} MANUAL - Garbage collection will need to be called manually
   */
  enum GC_MODES {
    AUTO,
    MANUAL,
  }

  /**
   * Constants that specify float precision in shaders.
   *
   * @name PRECISION
   * @memberof PIXI
   * @constant
   * @static
   * @enum {string}
   * @property {string} LOW='lowp'
   * @property {string} MEDIUM='mediump'
   * @property {string} HIGH='highp'
   */
  enum PRECISION {
    LOW,
    MEDIUM,
    HIGH,
  }

  /**
   * Constants for mask implementations.
   * We use `type` suffix because it leads to very different behaviours
   *
   * @name MASK_TYPES
   * @memberof PIXI
   * @static
   * @enum {number}
   * @property {number} NONE - Mask is ignored
   * @property {number} SCISSOR - Scissor mask, rectangle on screen, cheap
   * @property {number} STENCIL - Stencil mask, 1-bit, medium, works only if renderer supports stencil
   * @property {number} SPRITE - Mask that uses SpriteMaskFilter, uses temporary RenderTexture
   */
  enum MASK_TYPES {
    NONE,
    SCISSOR,
    STENCIL,
    SPRITE,
  }

  /**
   * The AbstractRenderer is the base for a PixiJS Renderer. It is extended by the {@link PIXI.CanvasRenderer}
   * and {@link PIXI.Renderer} which can be used for rendering a PixiJS scene.
   *
   * @abstract
   * @class
   * @extends PIXI.utils.EventEmitter
   * @memberof PIXI
   */
  class AbstractRenderer extends PIXI.utils.EventEmitter {
    constructor(
      system: string,
      options?: {
        width?: number;
        height?: number;
        view?: HTMLCanvasElement;
        transparent?: boolean;
        autoDensity?: boolean;
        antialias?: boolean;
        resolution?: number;
        preserveDrawingBuffer?: boolean;
        clearBeforeRender?: boolean;
        backgroundColor?: number;
      },
    );

    /**
     * The supplied constructor options.
     *
     * @member {Object} PIXI.AbstractRenderer#options
     * @readOnly
     */
    readonly options: any;
    /**
     * The type of the renderer.
     *
     * @member {number} PIXI.AbstractRenderer#type
     * @default PIXI.RENDERER_TYPE.UNKNOWN
     * @see PIXI.RENDERER_TYPE
     */
    type: number;
    /**
     * Measurements of the screen. (0, 0, screenWidth, screenHeight).
     *
     * Its safe to use as filterArea or hitArea for the whole stage.
     *
     * @member {PIXI.Rectangle} PIXI.AbstractRenderer#screen
     */
    screen: PIXI.Rectangle;
    /**
     * The canvas element that everything is drawn to.
     *
     * @member {HTMLCanvasElement} PIXI.AbstractRenderer#view
     */
    view: HTMLCanvasElement;
    /**
     * The resolution / device pixel ratio of the renderer.
     *
     * @member {number} PIXI.AbstractRenderer#resolution
     * @default 1
     */
    resolution: number;
    /**
     * Whether the render view is transparent.
     *
     * @member {boolean} PIXI.AbstractRenderer#transparent
     */
    transparent: boolean;
    /**
     * Whether CSS dimensions of canvas view should be resized to screen dimensions automatically.
     *
     * @member {boolean} PIXI.AbstractRenderer#autoDensity
     */
    autoDensity: boolean;
    /**
     * The value of the preserveDrawingBuffer flag affects whether or not the contents of
     * the stencil buffer is retained after rendering.
     *
     * @member {boolean} PIXI.AbstractRenderer#preserveDrawingBuffer
     */
    preserveDrawingBuffer: boolean;
    /**
     * This sets if the CanvasRenderer will clear the canvas or not before the new render pass.
     * If the scene is NOT transparent PixiJS will use a canvas sized fillRect operation every
     * frame to set the canvas background color. If the scene is transparent PixiJS will use clearRect
     * to clear the canvas every frame. Disable this by setting this to false. For example, if
     * your game has a canvas filling background image you often don't need this set.
     *
     * @member {boolean} PIXI.AbstractRenderer#clearBeforeRender
     * @default
     */
    clearBeforeRender: boolean;
    /**
     * The background color as a number.
     *
     * @member {number} PIXI.AbstractRenderer#_backgroundColor
     * @protected
     */
    protected _backgroundColor: number;
    /**
     * The background color as an [R, G, B] array.
     *
     * @member {number[]} PIXI.AbstractRenderer#_backgroundColorRgba
     * @protected
     */
    protected _backgroundColorRgba: number[];
    /**
     * The background color as a string.
     *
     * @member {string} PIXI.AbstractRenderer#_backgroundColorString
     * @protected
     */
    protected _backgroundColorString: string;
    /**
     * This temporary display object used as the parent of the currently being rendered item.
     *
     * @member {PIXI.DisplayObject} PIXI.AbstractRenderer#_tempDisplayObjectParent
     * @protected
     */
    protected _tempDisplayObjectParent: PIXI.DisplayObject;
    /**
     * The last root object that the renderer tried to render.
     *
     * @member {PIXI.DisplayObject} PIXI.AbstractRenderer#_lastObjectRendered
     * @protected
     */
    protected _lastObjectRendered: PIXI.DisplayObject;
    /**
     * Collection of plugins.
     * @readonly
     * @member {object} PIXI.AbstractRenderer#plugins
     */
    readonly plugins: any;

    /**
     * Initialize the plugins.
     *
     * @protected
     * @param {object} staticMap - The dictionary of statically saved plugins.
     */
    protected initPlugins(staticMap: any): void;

    /**
     * Same as view.width, actual number of pixels in the canvas by horizontal.
     *
     * @member {number}
     * @readonly
     * @default 800
     */
    readonly width: number;
    /**
     * Same as view.height, actual number of pixels in the canvas by vertical.
     *
     * @member {number}
     * @readonly
     * @default 600
     */
    readonly height: number;

    /**
     * Resizes the screen and canvas to the specified width and height.
     * Canvas dimensions are multiplied by resolution.
     *
     * @param {number} screenWidth - The new width of the screen.
     * @param {number} screenHeight - The new height of the screen.
     */
    resize(screenWidth: number, screenHeight: number): void;

    /**
     * Useful function that returns a texture of the display object that can then be used to create sprites
     * This can be quite useful if your displayObject is complicated and needs to be reused multiple times.
     *
     * @param {PIXI.DisplayObject} displayObject - The displayObject the object will be generated from.
     * @param {PIXI.SCALE_MODES} scaleMode - The scale mode of the texture.
     * @param {number} resolution - The resolution / device pixel ratio of the texture being generated.
     * @param {PIXI.Rectangle} [region] - The region of the displayObject, that shall be rendered,
     *        if no region is specified, defaults to the local bounds of the displayObject.
     * @return {PIXI.RenderTexture} A texture of the graphics object.
     */
    generateTexture(
      displayObject: PIXI.DisplayObject,
      scaleMode: PIXI.SCALE_MODES,
      resolution: number,
      region?: PIXI.Rectangle,
    ): PIXI.RenderTexture;

    /**
     * Removes everything from the renderer and optionally removes the Canvas DOM element.
     *
     * @param {boolean} [removeView=false] - Removes the Canvas element from the DOM.
     */
    destroy(removeView?: boolean): void;

    /**
     * The background color to fill if not transparent
     *
     * @member {number}
     */
    backgroundColor: number;
  }

  /**
   * The Renderer draws the scene and all its content onto a WebGL enabled canvas.
   *
   * This renderer should be used for browsers that support WebGL.
   *
   * This renderer works by automatically managing WebGLBatchesm, so no need for Sprite Batches or Sprite Clouds.
   * Don't forget to add the view to your DOM or you will not see anything!
   *
   * @class
   * @memberof PIXI
   * @extends PIXI.AbstractRenderer
   */
  class Renderer extends PIXI.AbstractRenderer {
    constructor(options?: {
      width?: number;
      height?: number;
      view?: HTMLCanvasElement;
      transparent?: boolean;
      autoDensity?: boolean;
      antialias?: boolean;
      forceFXAA?: boolean;
      resolution?: number;
      clearBeforeRender?: boolean;
      preserveDrawingBuffer?: boolean;
      backgroundColor?: number;
      powerPreference?: string;
      context?: any;
    });

    /**
     * The type of this renderer as a standardized const
     *
     * @member {number} PIXI.Renderer#type
     * @see PIXI.RENDERER_TYPE
     */
    type: number;
    /**
     * WebGL context, set by the contextSystem (this.context)
     *
     * @readonly
     * @member {WebGLRenderingContext} PIXI.Renderer#gl
     */
    readonly gl: WebGLRenderingContext;
    /**
     * Global uniforms
     * @member {PIXI.UniformGroup} PIXI.Renderer#globalUniforms
     */
    globalUniforms: PIXI.UniformGroup;
    /**
     * Mask system instance
     * @member {PIXI.systems.MaskSystem} mask
     * @memberof PIXI.Renderer#
     * @readonly
     */
    readonly mask: PIXI.systems.MaskSystem;
    /**
     * Context system instance
     * @member {PIXI.systems.ContextSystem} context
     * @memberof PIXI.Renderer#
     * @readonly
     */
    readonly context: PIXI.systems.ContextSystem;
    /**
     * State system instance
     * @member {PIXI.systems.StateSystem} state
     * @memberof PIXI.Renderer#
     * @readonly
     */
    readonly state: PIXI.systems.StateSystem;
    /**
     * Shader system instance
     * @member {PIXI.systems.ShaderSystem} shader
     * @memberof PIXI.Renderer#
     * @readonly
     */
    readonly shader: PIXI.systems.ShaderSystem;
    /**
     * Texture system instance
     * @member {PIXI.systems.TextureSystem} texture
     * @memberof PIXI.Renderer#
     * @readonly
     */
    readonly texture: PIXI.systems.TextureSystem;
    /**
     * Geometry system instance
     * @member {PIXI.systems.GeometrySystem} geometry
     * @memberof PIXI.Renderer#
     * @readonly
     */
    readonly geometry: PIXI.systems.GeometrySystem;
    /**
     * Framebuffer system instance
     * @member {PIXI.systems.FramebufferSystem} framebuffer
     * @memberof PIXI.Renderer#
     * @readonly
     */
    readonly framebuffer: PIXI.systems.FramebufferSystem;
    /**
     * Scissor system instance
     * @member {PIXI.systems.ScissorSystem} scissor
     * @memberof PIXI.Renderer#
     * @readonly
     */
    readonly scissor: PIXI.systems.ScissorSystem;
    /**
     * Stencil system instance
     * @member {PIXI.systems.StencilSystem} stencil
     * @memberof PIXI.Renderer#
     * @readonly
     */
    readonly stencil: PIXI.systems.StencilSystem;
    /**
     * Projection system instance
     * @member {PIXI.systems.ProjectionSystem} projection
     * @memberof PIXI.Renderer#
     * @readonly
     */
    readonly projection: PIXI.systems.ProjectionSystem;
    /**
     * Texture garbage collector system instance
     * @member {PIXI.systems.TextureGCSystem} textureGC
     * @memberof PIXI.Renderer#
     * @readonly
     */
    readonly textureGC: PIXI.systems.TextureGCSystem;
    /**
     * Filter system instance
     * @member {PIXI.systems.FilterSystem} filter
     * @memberof PIXI.Renderer#
     * @readonly
     */
    readonly filter: PIXI.systems.FilterSystem;
    /**
     * RenderTexture system instance
     * @member {PIXI.systems.RenderTextureSystem} renderTexture
     * @memberof PIXI.Renderer#
     * @readonly
     */
    readonly renderTexture: PIXI.systems.RenderTextureSystem;
    /**
     * Batch system instance
     * @member {PIXI.systems.BatchSystem} batch
     * @memberof PIXI.Renderer#
     * @readonly
     */
    readonly batch: PIXI.systems.BatchSystem;
    /**
     * Flag if we are rendering to the screen vs renderTexture
     * @member {boolean} PIXI.Renderer#renderingToScreen
     * @readonly
     * @default true
     */
    readonly renderingToScreen: boolean;

    /**
     * Add a new system to the renderer.
     * @param {Function} ClassRef - Class reference
     * @param {string} [name] - Property name for system, if not specified
     *        will use a static `name` property on the class itself. This
     *        name will be assigned as s property on the Renderer so make
     *        sure it doesn't collide with properties on Renderer.
     * @return {PIXI.Renderer} Return instance of renderer
     */
    addSystem(ClassRef: (...params: any[]) => any, name?: string): PIXI.Renderer;

    /**
     * Renders the object to its WebGL view
     *
     * @param {PIXI.DisplayObject} displayObject - The object to be rendered.
     * @param {PIXI.RenderTexture} [renderTexture] - The render texture to render to.
     * @param {boolean} [clear=true] - Should the canvas be cleared before the new render.
     * @param {PIXI.Matrix} [transform] - A transform to apply to the render texture before rendering.
     * @param {boolean} [skipUpdateTransform=false] - Should we skip the update transform pass?
     */
    render(
      displayObject: PIXI.DisplayObject,
      renderTexture?: PIXI.RenderTexture,
      clear?: boolean,
      transform?: PIXI.Matrix,
      skipUpdateTransform?: boolean,
    ): void;

    /**
     * Resizes the WebGL view to the specified width and height.
     *
     * @param {number} screenWidth - The new width of the screen.
     * @param {number} screenHeight - The new height of the screen.
     */
    resize(screenWidth: number, screenHeight: number): void;

    /**
     * Resets the WebGL state so you can render things however you fancy!
     *
     * @return {PIXI.Renderer} Returns itself.
     */
    reset(): PIXI.Renderer;

    /**
     * Clear the frame buffer
     */
    clear(): void;

    /**
     * Removes everything from the renderer (event listeners, spritebatch, etc...)
     *
     * @param {boolean} [removeView=false] - Removes the Canvas element from the DOM.
     *  See: https://github.com/pixijs/pixi.js/issues/2233
     */
    destroy(removeView?: boolean): void;

    /**
     * Collection of installed plugins. These are included by default in PIXI, but can be excluded
     * by creating a custom build. Consult the README for more information about creating custom
     * builds and excluding plugins.
     * @name PIXI.Renderer#plugins
     * @type {object}
     * @readonly
     * @property {PIXI.accessibility.AccessibilityManager} accessibility Support tabbing interactive elements.
     * @property {PIXI.Extract} extract Extract image data from renderer.
     * @property {PIXI.interaction.InteractionManager} interaction Handles mouse, touch and pointer events.
     * @property {PIXI.Prepare} prepare Pre-render display objects.
     */
    readonly plugins: {
      accessibility: PIXI.accessibility.AccessibilityManager;
      extract: PIXI.Extract;
      interaction: PIXI.interaction.InteractionManager;
      prepare: PIXI.Prepare;
    };

    /**
     * Adds a plugin to the renderer.
     *
     * @method
     * @param {string} pluginName - The name of the plugin.
     * @param {Function} ctor - The constructor function or class for the plugin.
     */
    static registerPlugin(pluginName: string, ctor: (...params: any[]) => any): void;

    /**
     * Collection of methods for extracting data (image, pixels, etc.) from a display object or render texture
     *
     * @member {PIXI.Extract} extract
     * @memberof PIXI.Renderer#
     * @see PIXI.Extract
     */
    extract: PIXI.Extract;
    /**
     * The supplied constructor options.
     *
     * @member {Object} PIXI.AbstractRenderer#options
     * @readOnly
     */
    readonly options: any;
    /**
     * Measurements of the screen. (0, 0, screenWidth, screenHeight).
     *
     * Its safe to use as filterArea or hitArea for the whole stage.
     *
     * @member {PIXI.Rectangle} PIXI.AbstractRenderer#screen
     */
    screen: PIXI.Rectangle;
    /**
     * The canvas element that everything is drawn to.
     *
     * @member {HTMLCanvasElement} PIXI.AbstractRenderer#view
     */
    view: HTMLCanvasElement;
    /**
     * The resolution / device pixel ratio of the renderer.
     *
     * @member {number} PIXI.AbstractRenderer#resolution
     * @default 1
     */
    resolution: number;
    /**
     * Whether the render view is transparent.
     *
     * @member {boolean} PIXI.AbstractRenderer#transparent
     */
    transparent: boolean;
    /**
     * Whether CSS dimensions of canvas view should be resized to screen dimensions automatically.
     *
     * @member {boolean} PIXI.AbstractRenderer#autoDensity
     */
    autoDensity: boolean;
    /**
     * The value of the preserveDrawingBuffer flag affects whether or not the contents of
     * the stencil buffer is retained after rendering.
     *
     * @member {boolean} PIXI.AbstractRenderer#preserveDrawingBuffer
     */
    preserveDrawingBuffer: boolean;
    /**
     * This sets if the CanvasRenderer will clear the canvas or not before the new render pass.
     * If the scene is NOT transparent PixiJS will use a canvas sized fillRect operation every
     * frame to set the canvas background color. If the scene is transparent PixiJS will use clearRect
     * to clear the canvas every frame. Disable this by setting this to false. For example, if
     * your game has a canvas filling background image you often don't need this set.
     *
     * @member {boolean} PIXI.AbstractRenderer#clearBeforeRender
     * @default
     */
    clearBeforeRender: boolean;
    /**
     * The background color as a number.
     *
     * @member {number} PIXI.AbstractRenderer#_backgroundColor
     * @protected
     */
    protected _backgroundColor: number;
    /**
     * The background color as an [R, G, B] array.
     *
     * @member {number[]} PIXI.AbstractRenderer#_backgroundColorRgba
     * @protected
     */
    protected _backgroundColorRgba: number[];
    /**
     * The background color as a string.
     *
     * @member {string} PIXI.AbstractRenderer#_backgroundColorString
     * @protected
     */
    protected _backgroundColorString: string;
    /**
     * This temporary display object used as the parent of the currently being rendered item.
     *
     * @member {PIXI.DisplayObject} PIXI.AbstractRenderer#_tempDisplayObjectParent
     * @protected
     */
    protected _tempDisplayObjectParent: PIXI.DisplayObject;
    /**
     * The last root object that the renderer tried to render.
     *
     * @member {PIXI.DisplayObject} PIXI.AbstractRenderer#_lastObjectRendered
     * @protected
     */
    protected _lastObjectRendered: PIXI.DisplayObject;

    /**
     * Initialize the plugins.
     *
     * @protected
     * @param {object} staticMap - The dictionary of statically saved plugins.
     */
    protected initPlugins(staticMap: any): void;

    /**
     * Same as view.width, actual number of pixels in the canvas by horizontal.
     *
     * @member {number}
     * @readonly
     * @default 800
     */
    readonly width: number;
    /**
     * Same as view.height, actual number of pixels in the canvas by vertical.
     *
     * @member {number}
     * @readonly
     * @default 600
     */
    readonly height: number;

    /**
     * Useful function that returns a texture of the display object that can then be used to create sprites
     * This can be quite useful if your displayObject is complicated and needs to be reused multiple times.
     *
     * @param {PIXI.DisplayObject} displayObject - The displayObject the object will be generated from.
     * @param {PIXI.SCALE_MODES} scaleMode - The scale mode of the texture.
     * @param {number} resolution - The resolution / device pixel ratio of the texture being generated.
     * @param {PIXI.Rectangle} [region] - The region of the displayObject, that shall be rendered,
     *        if no region is specified, defaults to the local bounds of the displayObject.
     * @return {PIXI.RenderTexture} A texture of the graphics object.
     */
    generateTexture(
      displayObject: PIXI.DisplayObject,
      scaleMode: PIXI.SCALE_MODES,
      resolution: number,
      region?: PIXI.Rectangle,
    ): PIXI.RenderTexture;

    /**
     * The background color to fill if not transparent
     *
     * @member {number}
     */
    backgroundColor: number;
  }

  /**
   * System is a base class used for extending systems used by the {@link PIXI.Renderer}
   *
   * @see PIXI.Renderer#addSystem
   * @class
   * @memberof PIXI
   */
  class System {
    constructor(renderer: PIXI.Renderer);

    /**
     * The renderer this manager works for.
     *
     * @member {PIXI.Renderer} PIXI.System#renderer
     */
    renderer: PIXI.Renderer;

    /**
     * Generic destroy methods to be overridden by the subclass
     */
    destroy(): void;
  }

  /**
   * This helper function will automatically detect which renderer you should be using.
   * WebGL is the preferred renderer as it is a lot faster. If WebGL is not supported by
   * the browser then this function will return a canvas renderer
   *
   * @memberof PIXI
   * @function autoDetectRenderer
   * @param {object} [options] - The optional renderer parameters
   * @param {number} [options.width=800] - the width of the renderers view
   * @param {number} [options.height=600] - the height of the renderers view
   * @param {HTMLCanvasElement} [options.view] - the canvas to use as a view, optional
   * @param {boolean} [options.transparent=false] - If the render view is transparent, default false
   * @param {boolean} [options.autoDensity=false] - Resizes renderer view in CSS pixels to allow for
   *   resolutions other than 1
   * @param {boolean} [options.antialias=false] - sets antialias
   * @param {boolean} [options.preserveDrawingBuffer=false] - enables drawing buffer preservation, enable this if you
   *  need to call toDataUrl on the webgl context
   * @param {number} [options.backgroundColor=0x000000] - The background color of the rendered area
   *  (shown if not transparent).
   * @param {boolean} [options.clearBeforeRender=true] - This sets if the renderer will clear the canvas or
   *   not before the new render pass.
   * @param {number} [options.resolution=1] - The resolution / device pixel ratio of the renderer, retina would be 2
   * @param {boolean} [options.forceCanvas=false] - prevents selection of WebGL renderer, even if such is present, this
   *   option only is available when using **pixi.js-legacy** or **@pixi/canvas-renderer** modules, otherwise
   *   it is ignored.
   * @param {boolean} [options.forceFXAA=false] - forces FXAA antialiasing to be used over native.
   *  FXAA is faster, but may not always look as great **webgl only**
   * @param {string} [options.powerPreference] - Parameter passed to webgl context, set to "high-performance"
   *  for devices with dual graphics card **webgl only**
   * @return {PIXI.Renderer|PIXI.CanvasRenderer} Returns WebGL renderer if available, otherwise CanvasRenderer
   */
  function autoDetectRenderer(options?: {
    width?: number;
    height?: number;
    view?: HTMLCanvasElement;
    transparent?: boolean;
    autoDensity?: boolean;
    antialias?: boolean;
    preserveDrawingBuffer?: boolean;
    backgroundColor?: number;
    clearBeforeRender?: boolean;
    resolution?: number;
    forceCanvas?: boolean;
    forceFXAA?: boolean;
    powerPreference?: string;
  }): PIXI.Renderer;

  /**
   * Renderer dedicated to drawing and batching sprites.
   *
   * This is the default batch renderer. It buffers objects
   * with texture-based geometries and renders them in
   * batches. It uploads multiple textures to the GPU to
   * reduce to the number of draw calls.
   *
   * @class
   * @protected
   * @memberof PIXI
   * @extends PIXI.ObjectRenderer
   */
  class AbstractBatchRenderer extends PIXI.ObjectRenderer {
    constructor(renderer: PIXI.Renderer);

    /**
     * This is used to generate a shader that can
     * color each vertex based on a `aTextureId`
     * attribute that points to an texture in `uSampler`.
     *
     * This enables the objects with different textures
     * to be drawn in the same draw call.
     *
     * You can customize your shader by creating your
     * custom shader generator.
     *
     * @member {PIXI.BatchShaderGenerator} PIXI.AbstractBatchRenderer#shaderGenerator
     * @protected
     */
    protected shaderGenerator: PIXI.BatchShaderGenerator;
    /**
     * The class that represents the geometry of objects
     * that are going to be batched with this.
     *
     * @member {object} PIXI.AbstractBatchRenderer#geometryClass
     * @default PIXI.BatchGeometry
     * @protected
     */
    protected geometryClass: any;
    /**
     * Size of data being buffered per vertex in the
     * attribute buffers (in floats). By default, the
     * batch-renderer plugin uses 6:
     *
     * | aVertexPosition | 2 |
     * |-----------------|---|
     * | aTextureCoords  | 2 |
     * | aColor          | 1 |
     * | aTextureId      | 1 |
     *
     * @member {number} PIXI.AbstractBatchRenderer#vertexSize
     * @readonly
     */
    readonly vertexSize: number;
    /**
     * The WebGL state in which this renderer will work.
     *
     * @member {PIXI.State} PIXI.AbstractBatchRenderer#state
     * @readonly
     */
    readonly state: PIXI.State;
    /**
     * The number of bufferable objects before a flush
     * occurs automatically.
     *
     * @member {number} PIXI.AbstractBatchRenderer#size
     * @default settings.SPRITE_BATCH_SIZE * 4
     */
    size: number;
    /**
     * This shader is generated by `this.shaderGenerator`.
     *
     * It is generated specifically to handle the required
     * number of textures being batched together.
     *
     * @member {PIXI.Shader} PIXI.AbstractBatchRenderer#_shader
     * @protected
     */
    protected _shader: PIXI.Shader;
    /**
     * Maximum number of textures that can be uploaded to
     * the GPU under the current context. It is initialized
     * properly in `this.contextChange`.
     *
     * @member {number} PIXI.AbstractBatchRenderer#MAX_TEXTURES
     * @see PIXI.AbstractBatchRenderer#contextChange
     * @readonly
     */
    readonly MAX_TEXTURES: number;

    /**
     * Handles the `contextChange` signal.
     *
     * It calculates `this.MAX_TEXTURES` and allocating the
     * packed-geometry object pool.
     */
    contextChange(): void;

    /**
     * Makes sure that static and dynamic flush pooled objects have correct dimensions
     */
    initFlushBuffers(): void;

    /**
     * Handles the `prerender` signal.
     *
     * It ensures that flushes start from the first geometry
     * object again.
     */
    onPrerender(): void;

    /**
     * Buffers the "batchable" object. It need not be rendered
     * immediately.
     *
     * @param {PIXI.DisplayObject} element - the element to render when
     *    using this renderer
     */
    render(element: PIXI.DisplayObject): void;

    /**
     * Populating drawcalls for rendering
     *
     * @param {PIXI.BatchTextureArray} texArray
     * @param {number} start
     * @param {number} finish
     */
    buildDrawCalls(texArray: PIXI.BatchTextureArray, start: number, finish: number): void;

    /**
     * Bind textures for current rendering
     *
     * @param {PIXI.BatchTextureArray} texArray
     */
    bindAndClearTexArray(texArray: PIXI.BatchTextureArray): void;

    /**
     * Renders the content _now_ and empties the current batch.
     */
    flush(): void;

    /**
     * Starts a new sprite batch.
     */
    start(): void;

    /**
     * Stops and flushes the current batch.
     */
    stop(): void;

    /**
     * Destroys this `AbstractBatchRenderer`. It cannot be used again.
     */
    destroy(): void;

    /**
     * Takes the four batching parameters of `element`, interleaves
     * and pushes them into the batching attribute/index buffers given.
     *
     * It uses these properties: `vertexData` `uvs`, `textureId` and
     * `indicies`. It also uses the "tint" of the base-texture, if
     * present.
     *
     * @param {PIXI.Sprite} element - element being rendered
     * @param {PIXI.ViewableBuffer} attributeBuffer - attribute buffer.
     * @param {Uint16Array} indexBuffer - index buffer
     * @param {number} aIndex - number of floats already in the attribute buffer
     * @param {number} iIndex - number of indices already in `indexBuffer`
     */
    packInterleavedGeometry(
      element: PIXI.Sprite,
      attributeBuffer: PIXI.ViewableBuffer,
      indexBuffer: Uint16Array,
      aIndex: number,
      iIndex: number,
    ): void;

    /**
     * Pool of `BatchDrawCall` objects that `flush` used
     * to create "batches" of the objects being rendered.
     *
     * These are never re-allocated again.
     * Shared between all batch renderers because it can be only one "flush" working at the moment.
     *
     * @static
     * @member {PIXI.BatchDrawCall[]}
     */
    static _drawCallPool: PIXI.BatchDrawCall[];
    /**
     * Pool of `BatchDrawCall` objects that `flush` used
     * to create "batches" of the objects being rendered.
     *
     * These are never re-allocated again.
     * Shared between all batch renderers because it can be only one "flush" working at the moment.
     *
     * @static
     * @member {PIXI.BatchTextureArray[]}
     */
    static _textureArrayPool: PIXI.BatchTextureArray[];
    /**
     * The renderer this manager works for.
     *
     * @member {PIXI.Renderer} PIXI.ObjectRenderer#renderer
     */
    renderer: PIXI.Renderer;
  }

  /**
   * Used by the batcher to draw batches.
   * Each one of these contains all information required to draw a bound geometry.
   *
   * @class
   * @memberof PIXI
   */
  class BatchDrawCall {
    constructor();

    /**
     * data for uniforms or custom webgl state
     * @member {object} PIXI.BatchDrawCall#data
     */
    data: any;
  }

  /**
   * Geometry used to batch standard PIXI content (e.g. Mesh, Sprite, Graphics objects).
   *
   * @class
   * @memberof PIXI
   */
  class BatchGeometry {
    constructor(_static?: boolean);

    /**
     * Buffer used for position, color, texture IDs
     *
     * @member {PIXI.Buffer} PIXI.BatchGeometry#_buffer
     * @protected
     */
    protected _buffer: PIXI.Buffer;
    /**
     * Index buffer data
     *
     * @member {PIXI.Buffer} PIXI.BatchGeometry#_indexBuffer
     * @protected
     */
    protected _indexBuffer: PIXI.Buffer;
  }

  /**
   * @class
   * @memberof PIXI
   * @hideconstructor
   */
  class BatchPluginFactory {
    /**
     * Create a new BatchRenderer plugin for Renderer. this convenience can provide an easy way
     * to extend BatchRenderer with all the necessary pieces.
     * @example
     * const fragment = `
     * varying vec2 vTextureCoord;
     * varying vec4 vColor;
     * varying float vTextureId;
     * uniform sampler2D uSamplers[%count%];
     *
     * void main(void){
     *     vec4 color;
     *     %forloop%
     *     gl_FragColor = vColor * vec4(color.a - color.rgb, color.a);
     * }
     * `;
     * const InvertBatchRenderer = PIXI.BatchPluginFactory.create({ fragment });
     * PIXI.Renderer.registerPlugin('invert', InvertBatchRenderer);
     * const sprite = new PIXI.Sprite();
     * sprite.pluginName = 'invert';
     *
     * @static
     * @param {object} [options]
     * @param {string} [options.vertex=PIXI.BatchPluginFactory.defaultVertexSrc] - Vertex shader source
     * @param {string} [options.fragment=PIXI.BatchPluginFactory.defaultFragmentTemplate] - Fragment shader template
     * @param {number} [options.vertexSize=6] - Vertex size
     * @param {object} [options.geometryClass=PIXI.BatchGeometry]
     * @return {*} New batch renderer plugin
     */
    static create(options?: { vertex?: string; fragment?: string; vertexSize?: number; geometryClass?: any }): any;

    /**
     * The default vertex shader source
     *
     * @static
     * @type {string}
     * @constant
     */
    static readonly defaultVertexSrc: string;
    /**
     * The default fragment shader source
     *
     * @static
     * @type {string}
     * @constant
     */
    static readonly defaultFragmentTemplate: string;
  }

  /**
   * Helper that generates batching multi-texture shader. Use it with your new BatchRenderer
   *
   * @class
   * @memberof PIXI
   */
  class BatchShaderGenerator {
    constructor(vertexSrc: string, fragTemplate: string);

    /**
     * Reference to the vertex shader source.
     *
     * @member {string} PIXI.BatchShaderGenerator#vertexSrc
     */
    vertexSrc: string;
    /**
     * Reference to the fragement shader template. Must contain "%count%" and "%forloop%".
     *
     * @member {string} PIXI.BatchShaderGenerator#fragTemplate
     */
    fragTemplate: string;
  }

  /**
   * Used by the batcher to build texture batches.
   * Holds list of textures and their respective locations.
   *
   * @class
   * @memberof PIXI
   */
  class BatchTextureArray {
    constructor();

    /**
     * inside textures array
     * @member {PIXI.BaseTexture[]} PIXI.BatchTextureArray#elements
     */
    elements: PIXI.BaseTexture[];
    /**
     * Respective locations for textures
     * @member {number[]} PIXI.BatchTextureArray#ids
     */
    ids: number[];
    /**
     * number of filled elements
     * @member {number} PIXI.BatchTextureArray#count
     */
    count: number;
  }

  /**
   * Base for a common object renderer that can be used as a
   * system renderer plugin.
   *
   * @class
   * @extends PIXI.System
   * @memberof PIXI
   */
  class ObjectRenderer extends PIXI.System {
    constructor(renderer: PIXI.Renderer);

    /**
     * The renderer this manager works for.
     *
     * @member {PIXI.Renderer} PIXI.ObjectRenderer#renderer
     */
    renderer: PIXI.Renderer;

    /**
     * Stub method that should be used to empty the current
     * batch by rendering objects now.
     */
    flush(): void;

    /**
     * Generic destruction method that frees all resources. This
     * should be called by subclasses.
     */
    destroy(): void;

    /**
     * Stub method that initializes any state required before
     * rendering starts. It is different from the `prerender`
     * signal, which occurs every frame, in that it is called
     * whenever an object requests _this_ renderer specifically.
     */
    start(): void;

    /**
     * Stops the renderer. It should free up any state and
     * become dormant.
     */
    stop(): void;

    /**
     * Keeps the object to render. It doesn't have to be
     * rendered immediately.
     *
     * @param {PIXI.DisplayObject} object - The object to render.
     */
    render(object: PIXI.DisplayObject): void;
  }

  /**
   * Filter is a special type of WebGL shader that is applied to the screen.
   *
   * {@link http://pixijs.io/examples/#/filters/blur-filter.js Example} of the
   * {@link PIXI.filters.BlurFilter BlurFilter}.
   *
   * ### Usage
   * Filters can be applied to any DisplayObject or Container.
   * PixiJS' `FilterSystem` renders the container into temporary Framebuffer,
   * then filter renders it to the screen.
   * Multiple filters can be added to the `filters` array property and stacked on each other.
   *
   * ```
   * const filter = new PIXI.Filter(myShaderVert, myShaderFrag, { myUniform: 0.5 });
   * const container = new PIXI.Container();
   * container.filters = [filter];
   * ```
   *
   * ### Previous Version Differences
   *
   * In PixiJS **v3**, a filter was always applied to _whole screen_.
   *
   * In PixiJS **v4**, a filter can be applied _only part of the screen_.
   * Developers had to create a set of uniforms to deal with coordinates.
   *
   * In PixiJS **v5** combines _both approaches_.
   * Developers can use normal coordinates of v3 and then allow filter to use partial Framebuffers,
   * bringing those extra uniforms into account.
   *
   * Also be aware that we have changed default vertex shader, please consult
   * {@link https://github.com/pixijs/pixi.js/wiki/v5-Creating-filters Wiki}.
   *
   * ### Built-in Uniforms
   *
   * PixiJS viewport uses screen (CSS) coordinates, `(0, 0, renderer.screen.width, renderer.screen.height)`,
   * and `projectionMatrix` uniform maps it to the gl viewport.
   *
   * **uSampler**
   *
   * The most important uniform is the input texture that container was rendered into.
   * _Important note: as with all Framebuffers in PixiJS, both input and output are
   * premultiplied by alpha._
   *
   * By default, input normalized coordinates are passed to fragment shader with `vTextureCoord`.
   * Use it to sample the input.
   *
   * ```
   * const fragment = `
   * varying vec2 vTextureCoord;
   * uniform sampler2D uSampler;
   * void main(void)
   * {
   *    gl_FragColor = texture2D(uSampler, vTextureCoord);
   * }
   * `;
   *
   * const myFilter = new PIXI.Filter(null, fragment);
   * ```
   *
   * This filter is just one uniform less than {@link PIXI.filters.AlphaFilter AlphaFilter}.
   *
   * **outputFrame**
   *
   * The `outputFrame` holds the rectangle where filter is applied in screen (CSS) coordinates.
   * It's the same as `renderer.screen` for a fullscreen filter.
   * Only a part of  `outputFrame.zw` size of temporary Framebuffer is used,
   * `(0, 0, outputFrame.width, outputFrame.height)`,
   *
   * Filters uses this quad to normalized (0-1) space, its passed into `aVertexPosition` attribute.
   * To calculate vertex position in screen space using normalized (0-1) space:
   *
   * ```
   * vec4 filterVertexPosition( void )
   * {
   *     vec2 position = aVertexPosition * max(outputFrame.zw, vec2(0.)) + outputFrame.xy;
   *     return vec4((projectionMatrix * vec3(position, 1.0)).xy, 0.0, 1.0);
   * }
   * ```
   *
   * **inputSize**
   *
   * Temporary framebuffer is different, it can be either the size of screen, either power-of-two.
   * The `inputSize.xy` are size of temporary framebuffer that holds input.
   * The `inputSize.zw` is inverted, it's a shortcut to evade division inside the shader.
   *
   * Set `inputSize.xy = outputFrame.zw` for a fullscreen filter.
   *
   * To calculate input normalized coordinate, you have to map it to filter normalized space.
   * Multiply by `outputFrame.zw` to get input coordinate.
   * Divide by `inputSize.xy` to get input normalized coordinate.
   *
   * ```
   * vec2 filterTextureCoord( void )
   * {
   *     return aVertexPosition * (outputFrame.zw * inputSize.zw); // same as /inputSize.xy
   * }
   * ```
   * **resolution**
   *
   * The `resolution` is the ratio of screen (CSS) pixels to real pixels.
   *
   * **inputPixel**
   *
   * `inputPixel.xy` is the size of framebuffer in real pixels, same as `inputSize.xy * resolution`
   * `inputPixel.zw` is inverted `inputPixel.xy`.
   *
   * It's handy for filters that use neighbour pixels, like {@link PIXI.filters.FXAAFilter FXAAFilter}.
   *
   * **inputClamp**
   *
   * If you try to get info from outside of used part of Framebuffer - you'll get undefined behaviour.
   * For displacements, coordinates has to be clamped.
   *
   * The `inputClamp.xy` is left-top pixel center, you may ignore it, because we use left-top part of Framebuffer
   * `inputClamp.zw` is bottom-right pixel center.
   *
   * ```
   * vec4 color = texture2D(uSampler, clamp(modifigedTextureCoord, inputClamp.xy, inputClamp.zw))
   * ```
   * OR
   * ```
   * vec4 color = texture2D(uSampler, min(modifigedTextureCoord, inputClamp.zw))
   * ```
   *
   * ### Additional Information
   *
   * Complete documentation on Filter usage is located in the
   * {@link https://github.com/pixijs/pixi.js/wiki/v5-Creating-filters Wiki}.
   *
   * Since PixiJS only had a handful of built-in filters, additional filters can be downloaded
   * {@link https://github.com/pixijs/pixi-filters here} from the PixiJS Filters repository.
   *
   * @class
   * @memberof PIXI
   * @extends PIXI.Shader
   */
  class Filter extends PIXI.Shader {
    constructor(vertexSrc?: string, fragmentSrc?: string, uniforms?: any);

    /**
     * The padding of the filter. Some filters require extra space to breath such as a blur.
     * Increasing this will add extra width and height to the bounds of the object that the
     * filter is applied to.
     *
     * @member {number} PIXI.Filter#padding
     */
    padding: number;
    /**
     * The resolution of the filter. Setting this to be lower will lower the quality but
     * increase the performance of the filter.
     *
     * @member {number} PIXI.Filter#resolution
     */
    resolution: number;
    /**
     * If enabled is true the filter is applied, if false it will not.
     *
     * @member {boolean} PIXI.Filter#enabled
     */
    enabled: boolean;
    /**
     * If enabled, PixiJS will fit the filter area into boundaries for better performance.
     * Switch it off if it does not work for specific shader.
     *
     * @member {boolean} PIXI.Filter#autoFit
     */
    autoFit: boolean;
    /**
     * Legacy filters use position and uvs from attributes
     * @member {boolean} PIXI.Filter#legacy
     * @readonly
     */
    readonly legacy: boolean;
    /**
     * The WebGL state the filter requires to render
     * @member {PIXI.State} PIXI.Filter#state
     */
    state: PIXI.State;

    /**
     * Applies the filter
     *
     * @param {PIXI.systems.FilterSystem} filterManager - The renderer to retrieve the filter from
     * @param {PIXI.RenderTexture} input - The input render target.
     * @param {PIXI.RenderTexture} output - The target to output to.
     * @param {boolean} clear - Should the output be cleared before rendering to it
     * @param {object} [currentState] - It's current state of filter.
     *        There are some useful properties in the currentState :
     *        target, filters, sourceFrame, destinationFrame, renderTarget, resolution
     */
    apply(
      filterManager: PIXI.systems.FilterSystem,
      input: PIXI.RenderTexture,
      output: PIXI.RenderTexture,
      clear: boolean,
      currentState?: any,
    ): void;

    /**
     * Sets the blendmode of the filter
     *
     * @member {number}
     * @default PIXI.BLEND_MODES.NORMAL
     */
    blendMode: number;
    /**
     * The default vertex shader source
     *
     * @static
     * @type {string}
     * @constant
     */
    static readonly defaultVertexSrc: string;
    /**
     * The default fragment shader source
     *
     * @static
     * @type {string}
     * @constant
     */
    static readonly defaultFragmentSrc: string;
    /**
     * Used for caching shader IDs
     *
     * @static
     * @type {object}
     * @protected
     */
    protected static SOURCE_KEY_MAP: any;
    /**
     * Program that the shader uses
     *
     * @member {PIXI.Program} PIXI.Shader#program
     */
    program: PIXI.Program;
    /**
     * Shader uniform values, shortcut for `uniformGroup.uniforms`
     * @readonly
     * @member {object}
     */
    readonly uniforms: any;
  }

  /**
   * This handles a Sprite acting as a mask, as opposed to a Graphic.
   *
   * WebGL only.
   *
   * @class
   * @extends PIXI.Filter
   * @memberof PIXI
   */
  class SpriteMaskFilter extends PIXI.Filter {
    constructor(sprite: PIXI.Sprite);

    /**
     * Sprite mask
     * @member {PIXI.Sprite} PIXI.SpriteMaskFilter#maskSprite
     */
    maskSprite: PIXI.Sprite;
    /**
     * Mask matrix
     * @member {PIXI.Matrix} PIXI.SpriteMaskFilter#maskMatrix
     */
    maskMatrix: PIXI.Matrix;

    /**
     * Applies the filter
     *
     * @param {PIXI.systems.FilterSystem} filterManager - The renderer to retrieve the filter from
     * @param {PIXI.RenderTexture} input - The input render target.
     * @param {PIXI.RenderTexture} output - The target to output to.
     * @param {boolean} clear - Should the output be cleared before rendering to it.
     */
    apply(
      filterManager: PIXI.systems.FilterSystem,
      input: PIXI.RenderTexture,
      output: PIXI.RenderTexture,
      clear: boolean,
    ): void;

    /**
     * The padding of the filter. Some filters require extra space to breath such as a blur.
     * Increasing this will add extra width and height to the bounds of the object that the
     * filter is applied to.
     *
     * @member {number} PIXI.Filter#padding
     */
    padding: number;
    /**
     * The resolution of the filter. Setting this to be lower will lower the quality but
     * increase the performance of the filter.
     *
     * @member {number} PIXI.Filter#resolution
     */
    resolution: number;
    /**
     * If enabled is true the filter is applied, if false it will not.
     *
     * @member {boolean} PIXI.Filter#enabled
     */
    enabled: boolean;
    /**
     * If enabled, PixiJS will fit the filter area into boundaries for better performance.
     * Switch it off if it does not work for specific shader.
     *
     * @member {boolean} PIXI.Filter#autoFit
     */
    autoFit: boolean;
    /**
     * Legacy filters use position and uvs from attributes
     * @member {boolean} PIXI.Filter#legacy
     * @readonly
     */
    readonly legacy: boolean;
    /**
     * The WebGL state the filter requires to render
     * @member {PIXI.State} PIXI.Filter#state
     */
    state: PIXI.State;
    /**
     * Sets the blendmode of the filter
     *
     * @member {number}
     * @default PIXI.BLEND_MODES.NORMAL
     */
    blendMode: number;
    /**
     * Program that the shader uses
     *
     * @member {PIXI.Program} PIXI.Shader#program
     */
    program: PIXI.Program;
    /**
     * Shader uniform values, shortcut for `uniformGroup.uniforms`
     * @readonly
     * @member {object}
     */
    readonly uniforms: any;
  }

  /**
   * Default vertex shader
   * @memberof PIXI
   * @member {string} defaultVertex
   */
  var defaultVertex: string;
  /**
   * Default filter vertex shader
   * @memberof PIXI
   * @member {string} defaultFilterVertex
   */
  var defaultFilterVertex: string;

  /**
   * Frame buffer used by the BaseRenderTexture
   *
   * @class
   * @memberof PIXI
   */
  class Framebuffer {
    constructor(width: number, height: number);

    /**
     * Reference to the colorTexture.
     *
     * @member {PIXI.Texture[]}
     * @readonly
     */
    readonly colorTexture: PIXI.Texture[];

    /**
     * Add texture to the colorTexture array
     *
     * @param {number} [index=0] - Index of the array to add the texture to
     * @param {PIXI.Texture} [texture] - Texture to add to the array
     */
    addColorTexture(index?: number, texture?: PIXI.Texture): void;

    /**
     * Add a depth texture to the frame buffer
     *
     * @param {PIXI.Texture} [texture] - Texture to add
     */
    addDepthTexture(texture?: PIXI.Texture): void;

    /**
     * Enable depth on the frame buffer
     */
    enableDepth(): void;

    /**
     * Enable stencil on the frame buffer
     */
    enableStencil(): void;

    /**
     * Resize the frame buffer
     *
     * @param {number} width - Width of the frame buffer to resize to
     * @param {number} height - Height of the frame buffer to resize to
     */
    resize(width: number, height: number): void;

    /**
     * disposes WebGL resources that are connected to this geometry
     */
    dispose(): void;
  }

  /**
   * Holds the information for a single attribute structure required to render geometry.
   *
   * This does not contain the actual data, but instead has a buffer id that maps to a {@link PIXI.Buffer}
   * This can include anything from positions, uvs, normals, colors etc.
   *
   * @class
   * @memberof PIXI
   */
  class Attribute {
    constructor(buffer: string, size?: number, normalized?: boolean, type?: number, stride?: number, start?: number);

    /**
     * Destroys the Attribute.
     */
    destroy(): void;

    /**
     * Helper function that creates an Attribute based on the information provided
     *
     * @static
     * @param {string} buffer  the id of the buffer that this attribute will look for
     * @param {Number} [size=0] the size of the attribute. If you have 2 floats per vertex (eg position x and y) this would be 2
     * @param {Boolean} [normalized=false] should the data be normalized.
     * @param {Number} [start=0] How far into the array to start reading values (used for interleaving data)
     * @param {Number} [type=PIXI.TYPES.FLOAT] what type of number is the attribute. Check {@link PIXI.TYPES} to see the ones available
     * @param {Number} [stride=0] How far apart (in floats) the start of each value is. (used for interleaving data)
     *
     * @returns {PIXI.Attribute} A new {@link PIXI.Attribute} based on the information provided
     */
    static from(
      buffer: string,
      size?: number,
      normalized?: boolean,
      start?: number,
      type?: number,
      stride?: number,
    ): PIXI.Attribute;
  }

  /**
   * A wrapper for data so that it can be used and uploaded by WebGL
   *
   * @class
   * @memberof PIXI
   */
  class Buffer {
    constructor(data: ArrayBuffer | SharedArrayBuffer | ArrayBufferView, _static?: boolean, index?: boolean);

    /**
     * The data in the buffer, as a typed array
     *
     * @member {ArrayBuffer| SharedArrayBuffer|ArrayBufferView} PIXI.Buffer#data
     */
    data: ArrayBuffer | SharedArrayBuffer | ArrayBufferView;

    /**
     * flags this buffer as requiring an upload to the GPU
     * @param {ArrayBuffer|SharedArrayBuffer|ArrayBufferView} [data] the data to update in the buffer.
     */
    update(data?: ArrayBuffer | SharedArrayBuffer | ArrayBufferView): void;

    /**
     * disposes WebGL resources that are connected to this geometry
     */
    dispose(): void;

    /**
     * Destroys the buffer
     */
    destroy(): void;

    /**
     * Helper function that creates a buffer based on an array or TypedArray
     *
     * @static
     * @param {ArrayBufferView | number[]} data the TypedArray that the buffer will store. If this is a regular Array it will be converted to a Float32Array.
     * @return {PIXI.Buffer} A new Buffer based on the data provided.
     */
    static from(data: ArrayBufferView | number[]): PIXI.Buffer;
  }

  /**
   * The Geometry represents a model. It consists of two components:
   * - GeometryStyle - The structure of the model such as the attributes layout
   * - GeometryData - the data of the model - this consists of buffers.
   * This can include anything from positions, uvs, normals, colors etc.
   *
   * Geometry can be defined without passing in a style or data if required (thats how I prefer!)
   *
   * ```js
   * let geometry = new PIXI.Geometry();
   *
   * geometry.addAttribute('positions', [0, 0, 100, 0, 100, 100, 0, 100], 2);
   * geometry.addAttribute('uvs', [0,0,1,0,1,1,0,1],2)
   * geometry.addIndex([0,1,2,1,3,2])
   *
   * ```
   * @class
   * @memberof PIXI
   */
  class Geometry {
    constructor(buffers?: PIXI.Buffer[], attributes?: any);

    /**
     * A map of renderer IDs to webgl VAOs
     *
     * @protected
     * @type {object}
     */
    protected glVertexArrayObjects: any;
    /**
     * Number of instances in this geometry, pass it to `GeometrySystem.draw()`
     * @member {number} PIXI.Geometry#instanceCount
     * @default 1
     */
    instanceCount: number;
    /**
     * Count of existing (not destroyed) meshes that reference this geometry
     * @member {number} PIXI.Geometry#refCount
     */
    refCount: number;

    /**
     *
     * Adds an attribute to the geometry
     *
     * @param {String} id - the name of the attribute (matching up to a shader)
     * @param {PIXI.Buffer|number[]} [buffer] the buffer that holds the data of the attribute . You can also provide an Array and a buffer will be created from it.
     * @param {Number} [size=0] the size of the attribute. If you have 2 floats per vertex (eg position x and y) this would be 2
     * @param {Boolean} [normalized=false] should the data be normalized.
     * @param {Number} [type=PIXI.TYPES.FLOAT] what type of number is the attribute. Check {PIXI.TYPES} to see the ones available
     * @param {Number} [stride=0] How far apart (in floats) the start of each value is. (used for interleaving data)
     * @param {Number} [start=0] How far into the array to start reading values (used for interleaving data)
     *
     * @return {PIXI.Geometry} returns self, useful for chaining.
     */
    addAttribute(
      id: string,
      buffer?: PIXI.Buffer | number[],
      size?: number,
      normalized?: boolean,
      type?: number,
      stride?: number,
      start?: number,
    ): PIXI.Geometry;

    /**
     * returns the requested attribute
     *
     * @param {String} id  the name of the attribute required
     * @return {PIXI.Attribute} the attribute requested.
     */
    getAttribute(id: string): PIXI.Attribute;

    /**
     * returns the requested buffer
     *
     * @param {String} id  the name of the buffer required
     * @return {PIXI.Buffer} the buffer requested.
     */
    getBuffer(id: string): PIXI.Buffer;

    /**
     *
     * Adds an index buffer to the geometry
     * The index buffer contains integers, three for each triangle in the geometry, which reference the various attribute buffers (position, colour, UV coordinates, other UV coordinates, normal, …). There is only ONE index buffer.
     *
     * @param {PIXI.Buffer|number[]} [buffer] the buffer that holds the data of the index buffer. You can also provide an Array and a buffer will be created from it.
     * @return {PIXI.Geometry} returns self, useful for chaining.
     */
    addIndex(buffer?: PIXI.Buffer | number[]): PIXI.Geometry;

    /**
     * returns the index buffer
     *
     * @return {PIXI.Buffer} the index buffer.
     */
    getIndex(): PIXI.Buffer;

    /**
     * this function modifies the structure so that all current attributes become interleaved into a single buffer
     * This can be useful if your model remains static as it offers a little performance boost
     *
     * @return {PIXI.Geometry} returns self, useful for chaining.
     */
    interleave(): PIXI.Geometry;

    /**
     * disposes WebGL resources that are connected to this geometry
     */
    dispose(): void;

    /**
     * Destroys the geometry.
     */
    destroy(): void;

    /**
     * returns a clone of the geometry
     *
     * @returns {PIXI.Geometry} a new clone of this geometry
     */
    clone(): PIXI.Geometry;

    /**
     * merges an array of geometries into a new single one
     * geometry attribute styles must match for this operation to work
     *
     * @param {PIXI.Geometry[]} geometries array of geometries to merge
     * @returns {PIXI.Geometry} shiny new geometry!
     */
    static merge(geometries: PIXI.Geometry[]): PIXI.Geometry;
  }

  /**
   * Flexible wrapper around `ArrayBuffer` that also provides
   * typed array views on demand.
   *
   * @class
   * @memberof PIXI
   */
  class ViewableBuffer {
    constructor(size: number);

    /**
     * Underlying `ArrayBuffer` that holds all the data
     * and is of capacity `size`.
     *
     * @member {ArrayBuffer} PIXI.ViewableBuffer#rawBinaryData
     */
    rawBinaryData: ArrayBuffer;
    /**
     * View on the raw binary data as a `Uint32Array`.
     *
     * @member {Uint32Array} PIXI.ViewableBuffer#uint32View
     */
    uint32View: Uint32Array;
    /**
     * View on the raw binary data as a `Float32Array`.
     *
     * @member {Float32Array} PIXI.ViewableBuffer#float32View
     */
    float32View: Float32Array;
    /**
     * View on the raw binary data as a `Int8Array`.
     *
     * @member {Int8Array}
     */
    int8View: Int8Array;
    /**
     * View on the raw binary data as a `Uint8Array`.
     *
     * @member {Uint8Array}
     */
    uint8View: Uint8Array;
    /**
     * View on the raw binary data as a `Int16Array`.
     *
     * @member {Int16Array}
     */
    int16View: Int16Array;
    /**
     * View on the raw binary data as a `Uint16Array`.
     *
     * @member {Uint16Array}
     */
    uint16View: Uint16Array;
    /**
     * View on the raw binary data as a `Int32Array`.
     *
     * @member {Int32Array}
     */
    int32View: Int32Array;

    /**
     * Returns the view of the given type.
     *
     * @param {string} type - One of `int8`, `uint8`, `int16`,
     *    `uint16`, `int32`, `uint32`, and `float32`.
     * @return {object} typed array of given type
     */
    view(type: string): any;

    /**
     * Destroys all buffer references. Do not use after calling
     * this.
     */
    destroy(): void;
  }

  /**
   * Component for masked elements
   *
   * Holds mask mode and temporary data about current mask
   *
   * @class
   * @memberof PIXI
   */
  class MaskData {
    constructor(maskObject?: PIXI.DisplayObject);

    /**
     * Mask type
     * @member {PIXI.MASK_TYPES} PIXI.MaskData#type
     */
    type: PIXI.MASK_TYPES;
    /**
     * Whether we know the mask type beforehand
     * @member {boolean} PIXI.MaskData#autoDetect
     * @default true
     */
    autoDetect: boolean;
    /**
     * Which element we use to mask
     * @member {PIXI.DisplayObject} PIXI.MaskData#maskObject
     */
    maskObject: PIXI.DisplayObject;
    /**
     * Whether it belongs to MaskSystem pool
     * @member {boolean} PIXI.MaskData#pooled
     */
    pooled: boolean;
    /**
     * Indicator of the type
     * @member {boolean} PIXI.MaskData#isMaskData
     */
    isMaskData: boolean;
    /**
     * Scissor operation above the mask in stack.
     * Null if _scissorCounter is zero, rectangle instance if positive.
     * @member {PIXI.Rectangle} PIXI.MaskData#_scissorRect
     */
    _scissorRect: PIXI.Rectangle;

    /**
     * resets the mask data after popMask()
     */
    reset(): void;

    /**
     * copies counters from maskData above, called from pushMask()
     * @param {PIXI.MaskData|null} maskAbove
     */
    copyCountersOrReset(maskAbove: PIXI.MaskData | null): void;
  }

  /**
   * A BaseRenderTexture is a special texture that allows any PixiJS display object to be rendered to it.
   *
   * __Hint__: All DisplayObjects (i.e. Sprites) that render to a BaseRenderTexture should be preloaded
   * otherwise black rectangles will be drawn instead.
   *
   * A BaseRenderTexture takes a snapshot of any Display Object given to its render method. The position
   * and rotation of the given Display Objects is ignored. For example:
   *
   * ```js
   * let renderer = PIXI.autoDetectRenderer();
   * let baseRenderTexture = new PIXI.BaseRenderTexture({ width: 800, height: 600 });
   * let renderTexture = new PIXI.RenderTexture(baseRenderTexture);
   * let sprite = PIXI.Sprite.from("spinObj_01.png");
   *
   * sprite.position.x = 800/2;
   * sprite.position.y = 600/2;
   * sprite.anchor.x = 0.5;
   * sprite.anchor.y = 0.5;
   *
   * renderer.render(sprite, renderTexture);
   * ```
   *
   * The Sprite in this case will be rendered using its local transform. To render this sprite at 0,0
   * you can clear the transform
   *
   * ```js
   *
   * sprite.setTransform()
   *
   * let baseRenderTexture = new PIXI.BaseRenderTexture({ width: 100, height: 100 });
   * let renderTexture = new PIXI.RenderTexture(baseRenderTexture);
   *
   * renderer.render(sprite, renderTexture);  // Renders to center of RenderTexture
   * ```
   *
   * @class
   * @extends PIXI.BaseTexture
   * @memberof PIXI
   */
  class BaseRenderTexture extends PIXI.BaseTexture {
    constructor(options?: { width?: number; height?: number; scaleMode?: PIXI.SCALE_MODES; resolution?: number });

    /**
     * A reference to the canvas render target (we only need one as this can be shared across renderers)
     *
     * @protected
     * @member {object} PIXI.BaseRenderTexture#_canvasRenderTarget
     */
    protected _canvasRenderTarget: any;
    /**
     * The data structure for the stencil masks.
     *
     * @member {PIXI.MaskData[]} PIXI.BaseRenderTexture#maskStack
     */
    maskStack: PIXI.MaskData[];
    /**
     * The data structure for the filters.
     *
     * @member {Object[]} PIXI.BaseRenderTexture#filterStack
     */
    filterStack: any[];

    /**
     * Resizes the BaseRenderTexture.
     *
     * @param {number} width - The width to resize to.
     * @param {number} height - The height to resize to.
     */
    resize(width: number, height: number): void;

    /**
     * Frees the texture and framebuffer from WebGL memory without destroying this texture object.
     * This means you can still use the texture later which will upload it to GPU
     * memory again.
     *
     * @fires PIXI.BaseTexture#dispose
     */
    dispose(): void;

    /**
     * Destroys this texture.
     *
     */
    destroy(): void;

    /**
     * The width of the base texture set when the image has loaded
     *
     * @readonly
     * @member {number} PIXI.BaseTexture#width
     */
    readonly width: number;
    /**
     * The height of the base texture set when the image has loaded
     *
     * @readonly
     * @member {number} PIXI.BaseTexture#height
     */
    readonly height: number;
    /**
     * The resolution / device pixel ratio of the texture
     *
     * @member {number} PIXI.BaseTexture#resolution
     * @default PIXI.settings.RESOLUTION
     */
    resolution: number;
    /**
     * Mipmap mode of the texture, affects downscaled images
     *
     * @member {PIXI.MIPMAP_MODES} PIXI.BaseTexture#mipmap
     * @default PIXI.settings.MIPMAP_TEXTURES
     */
    mipmap: PIXI.MIPMAP_MODES;
    /**
     * Anisotropic filtering level of texture
     *
     * @member {number} PIXI.BaseTexture#anisotropicLevel
     * @default PIXI.settings.ANISOTROPIC_LEVEL
     */
    anisotropicLevel: number;
    /**
     * How the texture wraps
     * @member {number} PIXI.BaseTexture#wrapMode
     */
    wrapMode: number;
    /**
     * The scale mode to apply when scaling this texture
     *
     * @member {PIXI.SCALE_MODES} PIXI.BaseTexture#scaleMode
     * @default PIXI.settings.SCALE_MODE
     */
    scaleMode: PIXI.SCALE_MODES;
    /**
     * The pixel format of the texture
     *
     * @member {PIXI.FORMATS} PIXI.BaseTexture#format
     * @default PIXI.FORMATS.RGBA
     */
    format: PIXI.FORMATS;
    /**
     * The type of resource data
     *
     * @member {PIXI.TYPES} PIXI.BaseTexture#type
     * @default PIXI.TYPES.UNSIGNED_BYTE
     */
    type: PIXI.TYPES;
    /**
     * The target type
     *
     * @member {PIXI.TARGETS} PIXI.BaseTexture#target
     * @default PIXI.TARGETS.TEXTURE_2D
     */
    target: PIXI.TARGETS;
    /**
     * How to treat premultiplied alpha, see {@link PIXI.ALPHA_MODES}.
     *
     * @member {PIXI.ALPHA_MODES} PIXI.BaseTexture#alphaMode
     * @default PIXI.ALPHA_MODES.UNPACK
     */
    alphaMode: PIXI.ALPHA_MODES;
    /**
     * Global unique identifier for this BaseTexture
     *
     * @member {string} PIXI.BaseTexture#uid
     * @protected
     */
    protected uid: string;
    /**
     * Used by automatic texture Garbage Collection, stores last GC tick when it was bound
     *
     * @member {number} PIXI.BaseTexture#touched
     * @protected
     */
    protected touched: number;
    /**
     * Whether or not the texture is a power of two, try to use power of two textures as much
     * as you can
     *
     * @readonly
     * @member {boolean} PIXI.BaseTexture#isPowerOfTwo
     * @default false
     */
    readonly isPowerOfTwo: boolean;
    /**
     * Used by TextureSystem to only update texture to the GPU when needed.
     * Please call `update()` to increment it.
     *
     * @readonly
     * @member {number} PIXI.BaseTexture#dirtyId
     */
    readonly dirtyId: number;
    /**
     * Used by TextureSystem to only update texture style when needed.
     *
     * @protected
     * @member {number} PIXI.BaseTexture#dirtyStyleId
     */
    protected dirtyStyleId: number;
    /**
     * Currently default cache ID.
     *
     * @member {string} PIXI.BaseTexture#cacheId
     */
    cacheId: string;
    /**
     * Generally speaking means when resource is loaded.
     * @readonly
     * @member {boolean} PIXI.BaseTexture#valid
     */
    readonly valid: boolean;
    /**
     * The collection of alternative cache ids, since some BaseTextures
     * can have more than one ID, short name and longer full URL
     *
     * @member {Array<string>} PIXI.BaseTexture#textureCacheIds
     * @readonly
     */
    readonly textureCacheIds: string[];
    /**
     * Flag if BaseTexture has been destroyed.
     *
     * @member {boolean} PIXI.BaseTexture#destroyed
     * @readonly
     */
    readonly destroyed: boolean;
    /**
     * The resource used by this BaseTexture, there can only
     * be one resource per BaseTexture, but textures can share
     * resources.
     *
     * @member {PIXI.resources.Resource} PIXI.BaseTexture#resource
     * @readonly
     */
    readonly resource: PIXI.resources.Resource;
    /**
     * Number of the texture batch, used by multi-texture renderers
     *
     * @member {number} PIXI.BaseTexture#_batchEnabled
     */
    _batchEnabled: number;
    /**
     * Location inside texture batch, used by multi-texture renderers
     *
     * @member {number} PIXI.BaseTexture#_batchLocation
     */
    _batchLocation: number;
    /**
     * Pixel width of the source of this texture
     *
     * @readonly
     * @member {number}
     */
    readonly realWidth: number;
    /**
     * Pixel height of the source of this texture
     *
     * @readonly
     * @member {number}
     */
    readonly realHeight: number;

    /**
     * Changes style options of BaseTexture
     *
     * @param {PIXI.SCALE_MODES} [scaleMode] - Pixi scalemode
     * @param {PIXI.MIPMAP_MODES} [mipmap] - enable mipmaps
     * @returns {PIXI.BaseTexture} this
     */
    setStyle(scaleMode?: PIXI.SCALE_MODES, mipmap?: PIXI.MIPMAP_MODES): PIXI.BaseTexture;

    /**
     * Changes w/h/resolution. Texture becomes valid if width and height are greater than zero.
     *
     * @param {number} width Visual width
     * @param {number} height Visual height
     * @param {number} [resolution] Optionally set resolution
     * @returns {PIXI.BaseTexture} this
     */
    setSize(width: number, height: number, resolution?: number): PIXI.BaseTexture;

    /**
     * Sets real size of baseTexture, preserves current resolution.
     *
     * @param {number} realWidth Full rendered width
     * @param {number} realHeight Full rendered height
     * @param {number} [resolution] Optionally set resolution
     * @returns {PIXI.BaseTexture} this
     */
    setRealSize(realWidth: number, realHeight: number, resolution?: number): PIXI.BaseTexture;

    /**
     * Changes resolution
     *
     * @param {number} [resolution] res
     * @returns {PIXI.BaseTexture} this
     */
    setResolution(resolution?: number): PIXI.BaseTexture;

    /**
     * Sets the resource if it wasn't set. Throws error if resource already present
     *
     * @param {PIXI.resources.Resource} resource - that is managing this BaseTexture
     * @returns {PIXI.BaseTexture} this
     */
    setResource(resource: PIXI.resources.Resource): PIXI.BaseTexture;

    /**
     * Invalidates the object. Texture becomes valid if width and height are greater than zero.
     */
    update(): void;
  }

  /**
   * A RenderTexture is a special texture that allows any PixiJS display object to be rendered to it.
   *
   * __Hint__: All DisplayObjects (i.e. Sprites) that render to a RenderTexture should be preloaded
   * otherwise black rectangles will be drawn instead.
   *
   * __Hint-2__: The actual memory allocation will happen on first render.
   * You shouldn't create renderTextures each frame just to delete them after, try to reuse them.
   *
   * A RenderTexture takes a snapshot of any Display Object given to its render method. For example:
   *
   * ```js
   * let renderer = PIXI.autoDetectRenderer();
   * let renderTexture = PIXI.RenderTexture.create({ width: 800, height: 600 });
   * let sprite = PIXI.Sprite.from("spinObj_01.png");
   *
   * sprite.position.x = 800/2;
   * sprite.position.y = 600/2;
   * sprite.anchor.x = 0.5;
   * sprite.anchor.y = 0.5;
   *
   * renderer.render(sprite, renderTexture);
   * ```
   *
   * The Sprite in this case will be rendered using its local transform. To render this sprite at 0,0
   * you can clear the transform
   *
   * ```js
   *
   * sprite.setTransform()
   *
   * let renderTexture = new PIXI.RenderTexture.create(100, 100);
   *
   * renderer.render(sprite, renderTexture);  // Renders to center of RenderTexture
   * ```
   *
   * @class
   * @extends PIXI.Texture
   * @memberof PIXI
   */
  class RenderTexture extends PIXI.Texture {
    constructor(baseRenderTexture: PIXI.BaseRenderTexture, frame?: PIXI.Rectangle);

    /**
     * This will let the renderer know if the texture is valid. If it's not then it cannot be rendered.
     *
     * @member {boolean} PIXI.RenderTexture#valid
     */
    valid: boolean;
    /**
     * Stores `sourceFrame` when this texture is inside current filter stack.
     * You can read it inside filters.
     *
     * @readonly
     * @member {PIXI.Rectangle} PIXI.RenderTexture#filterFrame
     */
    readonly filterFrame: PIXI.Rectangle;
    /**
     * The key for pooled texture of FilterSystem
     * @protected
     * @member {string} PIXI.RenderTexture#filterPoolKey
     */
    protected filterPoolKey: string;

    /**
     * Resizes the RenderTexture.
     *
     * @param {number} width - The width to resize to.
     * @param {number} height - The height to resize to.
     * @param {boolean} [resizeBaseTexture=true] - Should the baseTexture.width and height values be resized as well?
     */
    resize(width: number, height: number, resizeBaseTexture?: boolean): void;

    /**
     * Changes the resolution of baseTexture, but does not change framebuffer size.
     *
     * @param {number} resolution - The new resolution to apply to RenderTexture
     */
    setResolution(resolution: number): void;

    /**
     * A short hand way of creating a render texture.
     *
     * @param {object} [options] - Options
     * @param {number} [options.width=100] - The width of the render texture
     * @param {number} [options.height=100] - The height of the render texture
     * @param {number} [options.scaleMode=PIXI.settings.SCALE_MODE] - See {@link PIXI.SCALE_MODES} for possible values
     * @param {number} [options.resolution=1] - The resolution / device pixel ratio of the texture being generated
     * @return {PIXI.RenderTexture} The new render texture
     */
    static create(options?: {
      width?: number;
      height?: number;
      scaleMode?: number;
      resolution?: number;
    }): PIXI.RenderTexture;

    /**
     * Does this Texture have any frame data assigned to it?
     *
     * This mode is enabled automatically if no frame was passed inside constructor.
     *
     * In this mode texture is subscribed to baseTexture events, and fires `update` on any change.
     *
     * Beware, after loading or resize of baseTexture event can fired two times!
     * If you want more control, subscribe on baseTexture itself.
     *
     * ```js
     * texture.on('update', () => {});
     * ```
     *
     * Any assignment of `frame` switches off `noFrame` mode.
     *
     * @member {boolean} PIXI.Texture#noFrame
     */
    noFrame: boolean;
    /**
     * The base texture that this texture uses.
     *
     * @member {PIXI.BaseTexture} PIXI.Texture#baseTexture
     */
    baseTexture: PIXI.BaseTexture;
    /**
     * This is the area of the BaseTexture image to actually copy to the Canvas / WebGL when rendering,
     * irrespective of the actual frame size or placement (which can be influenced by trimmed texture atlases)
     *
     * @member {PIXI.Rectangle} PIXI.Texture#_frame
     */
    _frame: PIXI.Rectangle;
    /**
     * This is the trimmed area of original texture, before it was put in atlas
     * Please call `updateUvs()` after you change coordinates of `trim` manually.
     *
     * @member {PIXI.Rectangle} PIXI.Texture#trim
     */
    trim: PIXI.Rectangle;
    /**
     * This will let a renderer know that a texture has been updated (used mainly for WebGL uv updates)
     *
     * @member {boolean} PIXI.Texture#requiresUpdate
     */
    requiresUpdate: boolean;
    /**
     * The WebGL UV data cache. Can be used as quad UV
     *
     * @member {PIXI.TextureUvs} PIXI.Texture#_uvs
     * @protected
     */
    protected _uvs: PIXI.TextureUvs;
    /**
     * Default TextureMatrix instance for this texture
     * By default that object is not created because its heavy
     *
     * @member {PIXI.TextureMatrix} PIXI.Texture#uvMatrix
     */
    uvMatrix: PIXI.TextureMatrix;
    /**
     * This is the area of original texture, before it was put in atlas
     *
     * @member {PIXI.Rectangle} PIXI.Texture#orig
     */
    orig: PIXI.Rectangle;
    /**
     * Anchor point that is used as default if sprite is created with this texture.
     * Changing the `defaultAnchor` at a later point of time will not update Sprite's anchor point.
     * @member {PIXI.Point} PIXI.Texture#defaultAnchor
     * @default {0,0}
     */
    defaultAnchor: PIXI.Point;
    /**
     * Update ID is observed by sprites and TextureMatrix instances.
     * Call updateUvs() to increment it.
     *
     * @member {number} PIXI.Texture#_updateID
     * @protected
     */
    protected _updateID: number;
    /**
     * The ids under which this Texture has been added to the texture cache. This is
     * automatically set as long as Texture.addToCache is used, but may not be set if a
     * Texture is added directly to the TextureCache array.
     *
     * @member {string[]} PIXI.Texture#textureCacheIds
     */
    textureCacheIds: string[];

    /**
     * Updates this texture on the gpu.
     *
     * Calls the TextureResource update.
     *
     * If you adjusted `frame` manually, please call `updateUvs()` instead.
     *
     */
    update(): void;

    /**
     * Called when the base texture is updated
     *
     * @protected
     * @param {PIXI.BaseTexture} baseTexture - The base texture.
     */
    protected onBaseTextureUpdated(baseTexture: PIXI.BaseTexture): void;

    /**
     * Destroys this texture
     *
     * @param {boolean} [destroyBase=false] Whether to destroy the base texture as well
     */
    destroy(destroyBase?: boolean): void;

    /**
     * Creates a new texture object that acts the same as this one.
     *
     * @return {PIXI.Texture} The new texture
     */
    clone(): PIXI.Texture;

    /**
     * Updates the internal WebGL UV cache. Use it after you change `frame` or `trim` of the texture.
     * Call it after changing the frame
     */
    updateUvs(): void;

    /**
     * Returns resolution of baseTexture
     *
     * @member {number}
     * @readonly
     */
    readonly resolution: number;
    /**
     * The frame specifies the region of the base texture that this texture uses.
     * Please call `updateUvs()` after you change coordinates of `frame` manually.
     *
     * @member {PIXI.Rectangle}
     */
    frame: PIXI.Rectangle;
    /**
     * Indicates whether the texture is rotated inside the atlas
     * set to 2 to compensate for texture packer rotation
     * set to 6 to compensate for spine packer rotation
     * can be used to rotate or mirror sprites
     * See {@link PIXI.groupD8} for explanation
     *
     * @member {number}
     */
    rotate: number;
    /**
     * The width of the Texture in pixels.
     *
     * @member {number}
     */
    width: number;
    /**
     * The height of the Texture in pixels.
     *
     * @member {number}
     */
    height: number;
  }

  /**
   * Experimental!
   *
   * Texture pool, used by FilterSystem and plugins
   * Stores collection of temporary pow2 or screen-sized renderTextures
   *
   * If you use custom RenderTexturePool for your filters, you can use methods
   * `getFilterTexture` and `returnFilterTexture` same as in
   *
   * @class
   * @memberof PIXI
   */
  class RenderTexturePool {
    constructor(textureOptions?: { scaleMode?: PIXI.SCALE_MODES });

    /**
     * Allow renderTextures of the same size as screen, not just pow2
     *
     * Automatically sets to true after `setScreenSize`
     *
     * @member {boolean} PIXI.RenderTexturePool#enableFullScreen
     * @default false
     */
    enableFullScreen: boolean;

    /**
     * creates of texture with params that were specified in pool constructor
     *
     * @param {number} realWidth width of texture in pixels
     * @param {number} realHeight height of texture in pixels
     * @returns {RenderTexture}
     */
    createTexture(realWidth: number, realHeight: number): RenderTexture;

    /**
     * Gets a Power-of-Two render texture or fullScreen texture
     *
     * @protected
     * @param {number} minWidth - The minimum width of the render texture in real pixels.
     * @param {number} minHeight - The minimum height of the render texture in real pixels.
     * @param {number} [resolution=1] - The resolution of the render texture.
     * @return {PIXI.RenderTexture} The new render texture.
     */
    protected getOptimalTexture(minWidth: number, minHeight: number, resolution?: number): PIXI.RenderTexture;

    /**
     * Gets extra texture of the same size as input renderTexture
     *
     * `getFilterTexture(input, 0.5)` or `getFilterTexture(0.5, input)`
     *
     * @param {PIXI.RenderTexture} input renderTexture from which size and resolution will be copied
     * @param {number} [resolution] override resolution of the renderTexture
     *  It overrides, it does not multiply
     * @returns {PIXI.RenderTexture}
     */
    getFilterTexture(input: PIXI.RenderTexture, resolution?: number): PIXI.RenderTexture;

    /**
     * Place a render texture back into the pool.
     * @param {PIXI.RenderTexture} renderTexture - The renderTexture to free
     */
    returnTexture(renderTexture: PIXI.RenderTexture): void;

    /**
     * Alias for returnTexture, to be compliant with FilterSystem interface
     * @param {PIXI.RenderTexture} renderTexture - The renderTexture to free
     */
    returnFilterTexture(renderTexture: PIXI.RenderTexture): void;

    /**
     * Clears the pool
     *
     * @param {boolean} [destroyTextures=true] destroy all stored textures
     */
    clear(destroyTextures?: boolean): void;

    /**
     * If screen size was changed, drops all screen-sized textures,
     * sets new screen size, sets `enableFullScreen` to true
     *
     * Size is measured in pixels, `renderer.view` can be passed here, not `renderer.screen`
     *
     * @param {PIXI.ISize} size - Initial size of screen
     */
    setScreenSize(size: PIXI.ISize): void;

    /**
     * Key that is used to store fullscreen renderTextures in a pool
     *
     * @static
     * @const {string}
     */
    static readonly SCREEN_KEY: string;
  }

  /**
   * Helper class to create a WebGL Program
   *
   * @class
   * @memberof PIXI
   */
  class GLProgram {
    constructor(program: WebGLProgram, uniformData: any);

    /**
     * The shader program
     *
     * @member {WebGLProgram} PIXI.GLProgram#program
     */
    program: WebGLProgram;
    /**
     * holds the uniform data which contains uniform locations
     * and current uniform values used for caching and preventing unneeded GPU commands
     * @member {Object} PIXI.GLProgram#uniformData
     */
    uniformData: any;
    /**
     * uniformGroups holds the various upload functions for the shader. Each uniform group
     * and program have a unique upload function generated.
     * @member {Object} PIXI.GLProgram#uniformGroups
     */
    uniformGroups: any;

    /**
     * Destroys this program
     */
    destroy(): void;
  }

  /**
   * Helper class to create a shader program.
   *
   * @class
   * @memberof PIXI
   */
  class Program {
    constructor(vertexSrc?: string, fragmentSrc?: string, name?: string);

    /**
     * The vertex shader.
     *
     * @member {string} PIXI.Program#vertexSrc
     */
    vertexSrc: string;
    /**
     * The fragment shader.
     *
     * @member {string} PIXI.Program#fragmentSrc
     */
    fragmentSrc: string;

    /**
     * Extracts the data for a buy creating a small test program
     * or reading the src directly.
     * @protected
     *
     * @param {string} [vertexSrc] - The source of the vertex shader.
     * @param {string} [fragmentSrc] - The source of the fragment shader.
     */
    protected extractData(vertexSrc?: string, fragmentSrc?: string): void;

    /**
     * The default vertex shader source
     *
     * @static
     * @constant
     * @member {string}
     */
    static defaultVertexSrc: string;
    /**
     * The default fragment shader source
     *
     * @static
     * @constant
     * @member {string}
     */
    static defaultFragmentSrc: string;

    /**
     * A short hand function to create a program based of a vertex and fragment shader
     * this method will also check to see if there is a cached program.
     *
     * @param {string} [vertexSrc] - The source of the vertex shader.
     * @param {string} [fragmentSrc] - The source of the fragment shader.
     * @param {string} [name=pixi-shader] - Name for shader
     *
     * @returns {PIXI.Program} an shiny new Pixi shader!
     */
    static from(vertexSrc?: string, fragmentSrc?: string, name?: string): PIXI.Program;
  }

  /**
   * A helper class for shaders
   *
   * @class
   * @memberof PIXI
   */
  class Shader {
    constructor(program?: PIXI.Program, uniforms?: any);

    /**
     * Program that the shader uses
     *
     * @member {PIXI.Program} PIXI.Shader#program
     */
    program: PIXI.Program;
    /**
     * Shader uniform values, shortcut for `uniformGroup.uniforms`
     * @readonly
     * @member {object}
     */
    readonly uniforms: any;

    /**
     * A short hand function to create a shader based of a vertex and fragment shader
     *
     * @param {string} [vertexSrc] - The source of the vertex shader.
     * @param {string} [fragmentSrc] - The source of the fragment shader.
     * @param {object} [uniforms] - Custom uniforms to use to augment the built-in ones.
     *
     * @returns {PIXI.Shader} an shiny new Pixi shader!
     */
    static from(vertexSrc?: string, fragmentSrc?: string, uniforms?: any): PIXI.Shader;
  }

  /**
   * Uniform group holds uniform map and some ID's for work
   *
   * @class
   * @memberof PIXI
   */
  class UniformGroup {
    constructor(uniforms?: any, _static?: boolean);

    /**
     * uniform values
     * @member {object} PIXI.UniformGroup#uniforms
     * @readonly
     */
    readonly uniforms: any;
    /**
     * Its a group and not a single uniforms
     * @member {boolean} PIXI.UniformGroup#group
     * @readonly
     * @default true
     */
    readonly group: boolean;
    /**
     * dirty version
     * @protected
     * @member {number} PIXI.UniformGroup#dirtyId
     */
    protected dirtyId: number;
    /**
     * unique id
     * @protected
     * @member {number} PIXI.UniformGroup#id
     */
    protected id: number;
    /**
     * Uniforms wont be changed after creation
     * @member {boolean} PIXI.UniformGroup#static
     */
    static: boolean;
  }

  /**
   * This is a WebGL state, and is is passed The WebGL StateManager.
   *
   * Each mesh rendered may require WebGL to be in a different state.
   * For example you may want different blend mode or to enable polygon offsets
   *
   * @class
   * @memberof PIXI
   */
  class State {
    constructor();

    /**
     * Activates blending of the computed fragment color values
     *
     * @member {boolean}
     */
    blend: boolean;
    /**
     * Activates adding an offset to depth values of polygon's fragments
     *
     * @member {boolean}
     * @default false
     */
    offsets: boolean;
    /**
     * Activates culling of polygons.
     *
     * @member {boolean}
     * @default false
     */
    culling: boolean;
    /**
     * Activates depth comparisons and updates to the depth buffer.
     *
     * @member {boolean}
     * @default false
     */
    depthTest: boolean;
    /**
     * Specifies whether or not front or back-facing polygons can be culled.
     * @member {boolean}
     * @default false
     */
    clockwiseFrontFace: boolean;
    /**
     * The blend mode to be applied when this state is set. Apply a value of `PIXI.BLEND_MODES.NORMAL` to reset the blend mode.
     * Setting this mode to anything other than NO_BLEND will automatically switch blending on.
     *
     * @member {number}
     * @default PIXI.BLEND_MODES.NORMAL
     * @see PIXI.BLEND_MODES
     */
    blendMode: number;
    /**
     * The polygon offset. Setting this property to anything other than 0 will automatically enable polygon offset fill.
     *
     * @member {number}
     * @default 0
     */
    polygonOffset: number;
  }

  /**
   * Systems are individual components to the Renderer pipeline.
   * @namespace PIXI.systems
   */
  namespace systems {
    /**
     * System plugin to the renderer to manage batching.
     *
     * @class
     * @extends PIXI.System
     * @memberof PIXI.systems
     */
    class BatchSystem extends PIXI.System {
      constructor(renderer: PIXI.Renderer);

      /**
       * An empty renderer.
       *
       * @member {PIXI.ObjectRenderer} PIXI.systems.BatchSystem#emptyRenderer
       */
      emptyRenderer: PIXI.ObjectRenderer;
      /**
       * The currently active ObjectRenderer.
       *
       * @member {PIXI.ObjectRenderer} PIXI.systems.BatchSystem#currentRenderer
       */
      currentRenderer: PIXI.ObjectRenderer;

      /**
       * Changes the current renderer to the one given in parameter
       *
       * @param {PIXI.ObjectRenderer} objectRenderer - The object renderer to use.
       */
      setObjectRenderer(objectRenderer: PIXI.ObjectRenderer): void;

      /**
       * This should be called if you wish to do some custom rendering
       * It will basically render anything that may be batched up such as sprites
       */
      flush(): void;

      /**
       * Reset the system to an empty renderer
       */
      reset(): void;

      /**
       * Handy function for batch renderers: copies bound textures in first maxTextures locations to array
       * sets actual _batchLocation for them
       *
       * @param arr
       * @param maxTextures
       */
      copyBoundTextures(arr: any, maxTextures: any): void;

      /**
       * Assigns batch locations to textures in array based on boundTextures state.
       * All textures in texArray should have `_batchEnabled = _batchId`,
       * and their count should be less than `maxTextures`.
       *
       * @param {PIXI.BatchTextureArray} texArray textures to bound
       * @param {PIXI.BaseTexture[]} boundTextures current state of bound textures
       * @param {number} batchId marker for _batchEnabled param of textures in texArray
       * @param {number} maxTextures number of texture locations to manipulate
       */
      boundArray(
        texArray: PIXI.BatchTextureArray,
        boundTextures: PIXI.BaseTexture[],
        batchId: number,
        maxTextures: number,
      ): void;

      /**
       * The renderer this manager works for.
       *
       * @member {PIXI.Renderer} PIXI.System#renderer
       */
      renderer: PIXI.Renderer;

      /**
       * Generic destroy methods to be overridden by the subclass
       */
      destroy(): void;
    }

    /**
     * System plugin to the renderer to manage the context.
     *
     * @class
     * @extends PIXI.System
     * @memberof PIXI.systems
     */
    class ContextSystem extends PIXI.System {
      constructor(renderer: PIXI.Renderer);

      /**
       * Either 1 or 2 to reflect the WebGL version being used
       * @member {number} PIXI.systems.ContextSystem#webGLVersion
       * @readonly
       */
      readonly webGLVersion: number;
      /**
       * Extensions being used
       * @member {object} PIXI.systems.ContextSystem#extensions
       * @readonly
       * @property {WEBGL_draw_buffers} drawBuffers - WebGL v1 extension
       * @property {WEBGL_depth_texture} depthTexture - WebGL v1 extension
       * @property {OES_texture_float} floatTexture - WebGL v1 extension
       * @property {WEBGL_lose_context} loseContext - WebGL v1 extension
       * @property {OES_vertex_array_object} vertexArrayObject - WebGL v1 extension
       * @property {EXT_texture_filter_anisotropic} anisotropicFiltering - WebGL v1 and v2 extension
       */
      readonly extensions: {
        drawBuffers: WEBGL_draw_buffers;
        depthTexture: WEBGL_depth_texture;
        floatTexture: OES_texture_float;
        loseContext: WEBGL_lose_context;
        vertexArrayObject: OES_vertex_array_object;
        anisotropicFiltering: EXT_texture_filter_anisotropic;
      };
      /**
       * `true` if the context is lost
       * @member {boolean}
       * @readonly
       */
      readonly isLost: boolean;

      /**
       * Handle the context change event
       * @param {WebGLRenderingContext} gl new webgl context
       */
      contextChange(gl: WebGLRenderingContext): void;

      /**
       * Initialize the context
       *
       * @protected
       * @param {WebGLRenderingContext} gl - WebGL context
       */
      protected initFromContext(gl: WebGLRenderingContext): void;

      /**
       * Initialize from context options
       *
       * @protected
       * @see https://developer.mozilla.org/en-US/docs/Web/API/HTMLCanvasElement/getContext
       * @param {object} options - context attributes
       */
      protected initFromOptions(options: any): void;

      /**
       * Helper class to create a WebGL Context
       *
       * @param canvas {HTMLCanvasElement} the canvas element that we will get the context from
       * @param options {object} An options object that gets passed in to the canvas element containing the context attributes
       * @see https://developer.mozilla.org/en/docs/Web/API/HTMLCanvasElement/getContext
       * @return {WebGLRenderingContext} the WebGL context
       */
      createContext(canvas: HTMLCanvasElement, options: any): WebGLRenderingContext;

      /**
       * Auto-populate the extensions
       *
       * @protected
       */
      protected getExtensions(): void;

      /**
       * Handles a lost webgl context
       *
       * @protected
       * @param {WebGLContextEvent} event - The context lost event.
       */
      protected handleContextLost(event: WebGLContextEvent): void;

      /**
       * Handles a restored webgl context
       *
       * @protected
       */
      protected handleContextRestored(): void;

      /**
       * Handle the post-render runner event
       *
       * @protected
       */
      protected postrender(): void;

      /**
       * Validate context
       *
       * @protected
       * @param {WebGLRenderingContext} gl - Render context
       */
      protected validateContext(gl: WebGLRenderingContext): void;

      /**
       * The renderer this manager works for.
       *
       * @member {PIXI.Renderer} PIXI.System#renderer
       */
      renderer: PIXI.Renderer;

      /**
       * Generic destroy methods to be overridden by the subclass
       */
      destroy(): void;
    }

    /**
     * System plugin to the renderer to manage the filters.
     *
     * @class
     * @memberof PIXI.systems
     * @extends PIXI.System
     */
    class FilterSystem extends PIXI.System {
      constructor(renderer: PIXI.Renderer);

      /**
       * List of filters for the FilterSystem
       * @member {Object[]} PIXI.systems.FilterSystem#defaultFilterStack
       * @readonly
       */
      readonly defaultFilterStack: any[];
      /**
       * stores a bunch of PO2 textures used for filtering
       * @member {Object} PIXI.systems.FilterSystem#texturePool
       */
      texturePool: any;
      /**
       * a pool for storing filter states, save us creating new ones each tick
       * @member {Object[]} PIXI.systems.FilterSystem#statePool
       */
      statePool: any[];
      /**
       * A very simple geometry used when drawing a filter effect to the screen
       * @member {PIXI.Quad} PIXI.systems.FilterSystem#quad
       */
      quad: PIXI.Quad;
      /**
       * Quad UVs
       * @member {PIXI.QuadUv} PIXI.systems.FilterSystem#quadUv
       */
      quadUv: PIXI.QuadUv;
      /**
       * Temporary rect for maths
       * @type {PIXI.Rectangle}
       */
      tempRect: PIXI.Rectangle;
      /**
       * Active state
       * @member {object} PIXI.systems.FilterSystem#activeState
       */
      activeState: any;
      /**
       * This uniform group is attached to filter uniforms when used
       * @member {PIXI.UniformGroup} PIXI.systems.FilterSystem#globalUniforms
       * @property {PIXI.Rectangle} outputFrame
       * @property {Float32Array} inputSize
       * @property {Float32Array} inputPixel
       * @property {Float32Array} inputClamp
       * @property {Number} resolution
       * @property {Float32Array} filterArea
       * @property {Fload32Array} filterClamp
       */
      globalUniforms: PIXI.UniformGroup;
      /**
       * Whether to clear output renderTexture in AUTO/BLIT mode. See {@link PIXI.CLEAR_MODES}
       * @member {boolean} PIXI.systems.FilterSystem#forceClear
       */
      forceClear: boolean;
      /**
       * Old padding behavior is to use the max amount instead of sum padding.
       * Use this flag if you need the old behavior.
       * @member {boolean} PIXI.systems.FilterSystem#useMaxPadding
       * @default false
       */
      useMaxPadding: boolean;

      /**
       * Adds a new filter to the System.
       *
       * @param {PIXI.DisplayObject} target - The target of the filter to render.
       * @param {PIXI.Filter[]} filters - The filters to apply.
       */
      push(target: PIXI.DisplayObject, filters: PIXI.Filter[]): void;

      /**
       * Pops off the filter and applies it.
       *
       */
      pop(): void;

      /**
       * Draws a filter.
       *
       * @param {PIXI.Filter} filter - The filter to draw.
       * @param {PIXI.RenderTexture} input - The input render target.
       * @param {PIXI.RenderTexture} output - The target to output to.
       * @param {boolean} clear - Should the output be cleared before rendering to it
       */
      applyFilter(filter: PIXI.Filter, input: PIXI.RenderTexture, output: PIXI.RenderTexture, clear: boolean): void;

      /**
       * Multiply _input normalized coordinates_ to this matrix to get _sprite texture normalized coordinates_.
       *
       * Use `outputMatrix * vTextureCoord` in the shader.
       *
       * @param {PIXI.Matrix} outputMatrix - The matrix to output to.
       * @param {PIXI.Sprite} sprite - The sprite to map to.
       * @return {PIXI.Matrix} The mapped matrix.
       */
      calculateSpriteMatrix(outputMatrix: PIXI.Matrix, sprite: PIXI.Sprite): PIXI.Matrix;

      /**
       * Destroys this Filter System.
       */
      destroy(): void;

      /**
       * Gets a Power-of-Two render texture or fullScreen texture
       *
       * @protected
       * @param {number} minWidth - The minimum width of the render texture in real pixels.
       * @param {number} minHeight - The minimum height of the render texture in real pixels.
       * @param {number} [resolution=1] - The resolution of the render texture.
       * @return {PIXI.RenderTexture} The new render texture.
       */
      protected getOptimalFilterTexture(minWidth: number, minHeight: number, resolution?: number): PIXI.RenderTexture;

      /**
       * Gets extra render texture to use inside current filter
       * To be compliant with older filters, you can use params in any order
       *
       * @param {PIXI.RenderTexture} [input] renderTexture from which size and resolution will be copied
       * @param {number} [resolution] override resolution of the renderTexture
       * @returns {PIXI.RenderTexture}
       */
      getFilterTexture(input?: PIXI.RenderTexture, resolution?: number): PIXI.RenderTexture;

      /**
       * Frees a render texture back into the pool.
       *
       * @param {PIXI.RenderTexture} renderTexture - The renderTarget to free
       */
      returnFilterTexture(renderTexture: PIXI.RenderTexture): void;

      /**
       * Empties the texture pool.
       */
      emptyPool(): void;

      /**
       * calls `texturePool.resize()`, affects fullScreen renderTextures
       */
      resize(): void;

      /**
       * The renderer this manager works for.
       *
       * @member {PIXI.Renderer} PIXI.System#renderer
       */
      renderer: PIXI.Renderer;
    }

    /**
     * System plugin to the renderer to manage framebuffers.
     *
     * @class
     * @extends PIXI.System
     * @memberof PIXI.systems
     */
    class FramebufferSystem extends PIXI.System {
      constructor(renderer: PIXI.Renderer);

      /**
       * A list of managed framebuffers
       * @member {PIXI.Framebuffer[]} PIXI.systems.FramebufferSystem#managedFramebuffers
       * @readonly
       */
      readonly managedFramebuffers: PIXI.Framebuffer[];
      /**
       * Framebuffer value that shows that we don't know what is bound
       * @member {Framebuffer} PIXI.systems.FramebufferSystem#unknownFramebuffer
       * @readonly
       */
      readonly unknownFramebuffer: Framebuffer;

      /**
       * Sets up the renderer context and necessary buffers.
       */
      contextChange(): void;

      /**
       * Bind a framebuffer
       *
       * @param {PIXI.Framebuffer} framebuffer
       * @param {PIXI.Rectangle} [frame] frame, default is framebuffer size
       */
      bind(framebuffer: PIXI.Framebuffer, frame?: PIXI.Rectangle): void;

      /**
       * Set the WebGLRenderingContext's viewport.
       *
       * @param {Number} x - X position of viewport
       * @param {Number} y - Y position of viewport
       * @param {Number} width - Width of viewport
       * @param {Number} height - Height of viewport
       */
      setViewport(x: number, y: number, width: number, height: number): void;

      /**
       * Get the size of the current width and height. Returns object with `width` and `height` values.
       *
       * @member {object}
       * @readonly
       */
      readonly size: any;

      /**
       * Clear the color of the context
       *
       * @param {Number} r - Red value from 0 to 1
       * @param {Number} g - Green value from 0 to 1
       * @param {Number} b - Blue value from 0 to 1
       * @param {Number} a - Alpha value from 0 to 1
       */
      clear(r: number, g: number, b: number, a: number): void;

      /**
       * Initialize framebuffer
       *
       * @protected
       * @param {PIXI.Framebuffer} framebuffer
       */
      protected initFramebuffer(framebuffer: PIXI.Framebuffer): void;

      /**
       * Resize the framebuffer
       *
       * @protected
       * @param {PIXI.Framebuffer} framebuffer
       */
      protected resizeFramebuffer(framebuffer: PIXI.Framebuffer): void;

      /**
       * Update the framebuffer
       *
       * @protected
       * @param {PIXI.Framebuffer} framebuffer
       */
      protected updateFramebuffer(framebuffer: PIXI.Framebuffer): void;

      /**
       * Disposes framebuffer
       * @param {PIXI.Framebuffer} framebuffer framebuffer that has to be disposed of
       * @param {boolean} [contextLost=false] If context was lost, we suppress all delete function calls
       */
      disposeFramebuffer(framebuffer: PIXI.Framebuffer, contextLost?: boolean): void;

      /**
       * Disposes all framebuffers, but not textures bound to them
       * @param {boolean} [contextLost=false] If context was lost, we suppress all delete function calls
       */
      disposeAll(contextLost?: boolean): void;

      /**
       * resets framebuffer stored state, binds screen framebuffer
       *
       * should be called before renderTexture reset()
       */
      reset(): void;

      /**
       * The renderer this manager works for.
       *
       * @member {PIXI.Renderer} PIXI.System#renderer
       */
      renderer: PIXI.Renderer;

      /**
       * Generic destroy methods to be overridden by the subclass
       */
      destroy(): void;
    }

    /**
     * System plugin to the renderer to manage geometry.
     *
     * @class
     * @extends PIXI.System
     * @memberof PIXI.systems
     */
    class GeometrySystem extends PIXI.System {
      constructor(renderer: PIXI.Renderer);

      /**
       * `true` if we has `*_vertex_array_object` extension
       * @member {boolean} PIXI.systems.GeometrySystem#hasVao
       * @readonly
       */
      readonly hasVao: boolean;
      /**
       * `true` if has `ANGLE_instanced_arrays` extension
       * @member {boolean} PIXI.systems.GeometrySystem#hasInstance
       * @readonly
       */
      readonly hasInstance: boolean;
      /**
       * `true` if support `gl.UNSIGNED_INT` in `gl.drawElements` or `gl.drawElementsInstanced`
       * @member {boolean} PIXI.systems.GeometrySystem#canUseUInt32ElementIndex
       * @readonly
       */
      readonly canUseUInt32ElementIndex: boolean;
      /**
       * A cache of currently bound buffer,
       * contains only two members with keys ARRAY_BUFFER and ELEMENT_ARRAY_BUFFER
       * @member {Object.<number, PIXI.Buffer>} PIXI.systems.GeometrySystem#boundBuffers
       * @readonly
       */
      readonly boundBuffers: {
        [key: number]: PIXI.Buffer;
      };
      /**
       * Cache for all geometries by id, used in case renderer gets destroyed or for profiling
       * @member {object} PIXI.systems.GeometrySystem#managedGeometries
       * @readonly
       */
      readonly managedGeometries: any;
      /**
       * Cache for all buffers by id, used in case renderer gets destroyed or for profiling
       * @member {object} PIXI.systems.GeometrySystem#managedBuffers
       * @readonly
       */
      readonly managedBuffers: any;

      /**
       * Sets up the renderer context and necessary buffers.
       */
      contextChange(): void;

      /**
       * Binds geometry so that is can be drawn. Creating a Vao if required
       *
       * @param {PIXI.Geometry} geometry instance of geometry to bind
       * @param {PIXI.Shader} [shader] instance of shader to use vao for
       */
      bind(geometry: PIXI.Geometry, shader?: PIXI.Shader): void;

      /**
       * Reset and unbind any active VAO and geometry
       */
      reset(): void;

      /**
       * Update buffers
       * @protected
       */
      protected updateBuffers(): void;

      /**
       * Check compability between a geometry and a program
       * @protected
       * @param {PIXI.Geometry} geometry - Geometry instance
       * @param {PIXI.Program} program - Program instance
       */
      protected checkCompatibility(geometry: PIXI.Geometry, program: PIXI.Program): void;

      /**
       * Takes a geometry and program and generates a unique signature for them.
       *
       * @param {PIXI.Geometry} geometry to get signature from
       * @param {PIXI.Program} program to test geometry against
       * @returns {String} Unique signature of the geometry and program
       * @protected
       */
      protected getSignature(geometry: PIXI.Geometry, program: PIXI.Program): string;

      /**
       * Creates or gets Vao with the same structure as the geometry and stores it on the geometry.
       * If vao is created, it is bound automatically.
       *
       * @protected
       * @param {PIXI.Geometry} geometry - Instance of geometry to to generate Vao for
       * @param {PIXI.Program} program - Instance of program
       */
      protected initGeometryVao(geometry: PIXI.Geometry, program: PIXI.Program): void;

      /**
       * Disposes buffer
       * @param {PIXI.Buffer} buffer buffer with data
       * @param {boolean} [contextLost=false] If context was lost, we suppress deleteVertexArray
       */
      disposeBuffer(buffer: PIXI.Buffer, contextLost?: boolean): void;

      /**
       * Disposes geometry
       * @param {PIXI.Geometry} geometry Geometry with buffers. Only VAO will be disposed
       * @param {boolean} [contextLost=false] If context was lost, we suppress deleteVertexArray
       */
      disposeGeometry(geometry: PIXI.Geometry, contextLost?: boolean): void;

      /**
       * dispose all WebGL resources of all managed geometries and buffers
       * @param {boolean} [contextLost=false] If context was lost, we suppress `gl.delete` calls
       */
      disposeAll(contextLost?: boolean): void;

      /**
       * Activate vertex array object
       *
       * @protected
       * @param {PIXI.Geometry} geometry - Geometry instance
       * @param {PIXI.Program} program - Shader program instance
       */
      protected activateVao(geometry: PIXI.Geometry, program: PIXI.Program): void;

      /**
       * Draw the geometry
       *
       * @param {Number} type - the type primitive to render
       * @param {Number} [size] - the number of elements to be rendered
       * @param {Number} [start] - Starting index
       * @param {Number} [instanceCount] - the number of instances of the set of elements to execute
       */
      draw(type: number, size?: number, start?: number, instanceCount?: number): void;

      /**
       * Unbind/reset everything
       * @protected
       */
      protected unbind(): void;

      /**
       * The renderer this manager works for.
       *
       * @member {PIXI.Renderer} PIXI.System#renderer
       */
      renderer: PIXI.Renderer;

      /**
       * Generic destroy methods to be overridden by the subclass
       */
      destroy(): void;
    }

    /**
     * System plugin to the renderer to manage masks of certain type
     *
     * @class
     * @extends PIXI.System
     * @memberof PIXI.systems
     */
    class AbstractMaskSystem extends PIXI.System {
      constructor(renderer: PIXI.Renderer);

      /**
       * The mask stack
       * @member {PIXI.MaskData[]} PIXI.systems.AbstractMaskSystem#maskStack
       */
      maskStack: PIXI.MaskData[];

      /**
       * gets count of masks of certain type
       * @returns {number}
       */
      getStackLength(): number;

      /**
       * Changes the mask stack that is used by this System.
       *
       * @param {PIXI.MaskData[]} maskStack - The mask stack
       */
      setMaskStack(maskStack: PIXI.MaskData[]): void;

      /**
       * Destroys the mask stack.
       *
       */
      destroy(): void;

      /**
       * The renderer this manager works for.
       *
       * @member {PIXI.Renderer} PIXI.System#renderer
       */
      renderer: PIXI.Renderer;
    }

    /**
     * System plugin to the renderer to manage masks.
     *
     * @class
     * @extends PIXI.System
     * @memberof PIXI.systems
     */
    class MaskSystem extends PIXI.System {
      constructor(renderer: PIXI.Renderer);

      /**
       * Target to mask
       * @member {PIXI.DisplayObject} PIXI.systems.MaskSystem#scissorRenderTarget
       * @readonly
       */
      readonly scissorRenderTarget: PIXI.DisplayObject;
      /**
       * Enable scissor
       * @member {boolean} PIXI.systems.MaskSystem#enableScissor
       * @readonly
       */
      readonly enableScissor: boolean;
      /**
       * Pool of used sprite mask filters
       * @member {PIXI.SpriteMaskFilter[]} PIXI.systems.MaskSystem#alphaMaskPool
       * @readonly
       */
      readonly alphaMaskPool: PIXI.SpriteMaskFilter[];
      /**
       * Pool of mask data
       * @member {PIXI.MaskData[]} PIXI.systems.MaskSystem#maskDataPool
       * @readonly
       */
      readonly maskDataPool: PIXI.MaskData[];
      /**
       * Current index of alpha mask pool
       * @member {number} PIXI.systems.MaskSystem#alphaMaskIndex
       * @default 0
       * @readonly
       */
      readonly alphaMaskIndex: number;

      /**
       * Changes the mask stack that is used by this System.
       *
       * @param {PIXI.MaskData[]} maskStack - The mask stack
       */
      setMaskStack(maskStack: PIXI.MaskData[]): void;

      /**
       * Applies the Mask and adds it to the current filter stack.
       * Renderer batch must be flushed beforehand.
       *
       * @param {PIXI.DisplayObject} target - Display Object to push the mask to
       * @param {PIXI.MaskData|PIXI.Sprite|PIXI.Graphics|PIXI.DisplayObject} maskData - The masking data.
       */
      push(
        target: PIXI.DisplayObject,
        maskData: PIXI.MaskData | PIXI.Sprite | PIXI.Graphics | PIXI.DisplayObject,
      ): void;

      /**
       * Removes the last mask from the mask stack and doesn't return it.
       * Renderer batch must be flushed beforehand.
       *
       * @param {PIXI.DisplayObject} target - Display Object to pop the mask from
       */
      pop(target: PIXI.DisplayObject): void;

      /**
       * Sets type of MaskData based on its maskObject
       * @param {PIXI.MaskData} maskData
       */
      detect(maskData: PIXI.MaskData): void;

      /**
       * Applies the Mask and adds it to the current filter stack.
       *
       * @param {PIXI.MaskData} maskData - Sprite to be used as the mask
       */
      pushSpriteMask(maskData: PIXI.MaskData): void;

      /**
       * Removes the last filter from the filter stack and doesn't return it.
       */
      popSpriteMask(): void;

      /**
       * The renderer this manager works for.
       *
       * @member {PIXI.Renderer} PIXI.System#renderer
       */
      renderer: PIXI.Renderer;

      /**
       * Generic destroy methods to be overridden by the subclass
       */
      destroy(): void;
    }

    /**
     * System plugin to the renderer to manage scissor rects (used for masks).
     *
     * @class
     * @extends PIXI.System
     * @memberof PIXI.systems
     */
    class ScissorSystem extends PIXI.System {
      constructor(renderer: PIXI.Renderer);

      /**
       * Applies the Mask and adds it to the current stencil stack. @alvin
       *
       * @param {PIXI.MaskData} maskData - The mask data
       */
      push(maskData: PIXI.MaskData): void;

      /**
       * Pops scissor mask. MaskData is already removed from stack
       */
      pop(): void;

      /**
       * The renderer this manager works for.
       *
       * @member {PIXI.Renderer} PIXI.System#renderer
       */
      renderer: PIXI.Renderer;

      /**
       * Generic destroy methods to be overridden by the subclass
       */
      destroy(): void;
    }

    /**
     * System plugin to the renderer to manage stencils (used for masks).
     *
     * @class
     * @extends PIXI.System
     * @memberof PIXI.systems
     */
    class StencilSystem extends PIXI.System {
      constructor(renderer: PIXI.Renderer);

      /**
       * Applies the Mask and adds it to the current stencil stack.
       *
       * @param {PIXI.MaskData} maskData - The mask data
       */
      push(maskData: PIXI.MaskData): void;

      /**
       * Pops stencil mask. MaskData is already removed from stack
       *
       * @param {PIXI.DisplayObject} maskObject - object of popped mask data
       */
      pop(maskObject: PIXI.DisplayObject): void;

      /**
       * The renderer this manager works for.
       *
       * @member {PIXI.Renderer} PIXI.System#renderer
       */
      renderer: PIXI.Renderer;

      /**
       * Generic destroy methods to be overridden by the subclass
       */
      destroy(): void;
    }

    /**
     * System plugin to the renderer to manage the projection matrix.
     *
     * @class
     * @extends PIXI.System
     * @memberof PIXI.systems
     */
    class ProjectionSystem extends PIXI.System {
      constructor(renderer: PIXI.Renderer);

      /**
       * Destination frame
       * @member {PIXI.Rectangle} PIXI.systems.ProjectionSystem#destinationFrame
       * @readonly
       */
      readonly destinationFrame: PIXI.Rectangle;
      /**
       * Source frame
       * @member {PIXI.Rectangle} PIXI.systems.ProjectionSystem#sourceFrame
       * @readonly
       */
      readonly sourceFrame: PIXI.Rectangle;
      /**
       * Default destination frame
       * @member {PIXI.Rectangle} PIXI.systems.ProjectionSystem#defaultFrame
       * @readonly
       */
      readonly defaultFrame: PIXI.Rectangle;
      /**
       * Project matrix
       * @member {PIXI.Matrix} PIXI.systems.ProjectionSystem#projectionMatrix
       * @readonly
       */
      readonly projectionMatrix: PIXI.Matrix;
      /**
       * A transform that will be appended to the projection matrix
       * if null, nothing will be applied
       * @member {PIXI.Matrix} PIXI.systems.ProjectionSystem#transform
       */
      transform: PIXI.Matrix;

      /**
       * Updates the projection matrix based on a projection frame (which is a rectangle)
       *
       * @param {PIXI.Rectangle} destinationFrame - The destination frame.
       * @param {PIXI.Rectangle} sourceFrame - The source frame.
       * @param {Number} resolution - Resolution
       * @param {boolean} root - If is root
       */
      update(destinationFrame: PIXI.Rectangle, sourceFrame: PIXI.Rectangle, resolution: number, root: boolean): void;

      /**
       * Updates the projection matrix based on a projection frame (which is a rectangle)
       *
       * @param {PIXI.Rectangle} destinationFrame - The destination frame.
       * @param {PIXI.Rectangle} sourceFrame - The source frame.
       * @param {Number} resolution - Resolution
       * @param {boolean} root - If is root
       */
      calculateProjection(
        destinationFrame: PIXI.Rectangle,
        sourceFrame: PIXI.Rectangle,
        resolution: number,
        root: boolean,
      ): void;

      /**
       * Sets the transform of the active render target to the given matrix
       *
       * @param {PIXI.Matrix} matrix - The transformation matrix
       */
      setTransform(matrix: PIXI.Matrix): void;

      /**
       * The renderer this manager works for.
       *
       * @member {PIXI.Renderer} PIXI.System#renderer
       */
      renderer: PIXI.Renderer;

      /**
       * Generic destroy methods to be overridden by the subclass
       */
      destroy(): void;
    }

    /**
     * System plugin to the renderer to manage render textures.
     *
     * Should be added after FramebufferSystem
     *
     * @class
     * @extends PIXI.System
     * @memberof PIXI.systems
     */
    class RenderTextureSystem extends PIXI.System {
      constructor(renderer: PIXI.Renderer);

      /**
       * The clear background color as rgba
       * @member {number[]} PIXI.systems.RenderTextureSystem#clearColor
       */
      clearColor: number[];
      /**
       * List of masks for the StencilSystem
       * @member {PIXI.Graphics[]} PIXI.systems.RenderTextureSystem#defaultMaskStack
       * @readonly
       */
      readonly defaultMaskStack: PIXI.Graphics[];
      /**
       * Render texture
       * @member {PIXI.RenderTexture} PIXI.systems.RenderTextureSystem#current
       * @readonly
       */
      readonly current: PIXI.RenderTexture;
      /**
       * Source frame
       * @member {PIXI.Rectangle} PIXI.systems.RenderTextureSystem#sourceFrame
       * @readonly
       */
      readonly sourceFrame: PIXI.Rectangle;
      /**
       * Destination frame
       * @member {PIXI.Rectangle} PIXI.systems.RenderTextureSystem#destinationFrame
       * @readonly
       */
      readonly destinationFrame: PIXI.Rectangle;

      /**
       * Bind the current render texture
       * @param {PIXI.RenderTexture} [renderTexture] - RenderTexture to bind, by default its `null`, the screen
       * @param {PIXI.Rectangle} [sourceFrame] - part of screen that is mapped to the renderTexture
       * @param {PIXI.Rectangle} [destinationFrame] - part of renderTexture, by default it has the same size as sourceFrame
       */
      bind(renderTexture?: PIXI.RenderTexture, sourceFrame?: PIXI.Rectangle, destinationFrame?: PIXI.Rectangle): void;

      /**
       * Erases the render texture and fills the drawing area with a colour
       *
       * @param {number[]} [clearColor] - The color as rgba, default to use the renderer backgroundColor
       * @return {PIXI.Renderer} Returns itself.
       */
      clear(clearColor?: number[]): PIXI.Renderer;

      /**
       * Resets renderTexture state
       */
      reset(): void;

      /**
       * The renderer this manager works for.
       *
       * @member {PIXI.Renderer} PIXI.System#renderer
       */
      renderer: PIXI.Renderer;

      /**
       * Generic destroy methods to be overridden by the subclass
       */
      destroy(): void;
    }

    /**
     * System plugin to the renderer to manage shaders.
     *
     * @class
     * @memberof PIXI.systems
     * @extends PIXI.System
     */
    class ShaderSystem extends PIXI.System {
      constructor(renderer: PIXI.Renderer);

      /**
       * The current WebGL rendering context
       *
       * @member {WebGLRenderingContext} PIXI.systems.ShaderSystem#gl
       */
      gl: WebGLRenderingContext;

      /**
       * Changes the current shader to the one given in parameter
       *
       * @param {PIXI.Shader} shader - the new shader
       * @param {boolean} dontSync - false if the shader should automatically sync its uniforms.
       * @returns {PIXI.GLProgram} the glProgram that belongs to the shader.
       */
      bind(shader: PIXI.Shader, dontSync: boolean): PIXI.GLProgram;

      /**
       * Uploads the uniforms values to the currently bound shader.
       *
       * @param {object} uniforms - the uniforms values that be applied to the current shader
       */
      setUniforms(uniforms: any): void;

      /**
       *
       * syncs uniforms on the group
       * @param {*} group the uniform group to sync
       * @param {*} syncData this is data that is passed to the sync function and any nested sync functions
       */
      syncUniformGroup(group: any, syncData: any): void;

      /**
       * Returns the underlying GLShade rof the currently bound shader.
       * This can be handy for when you to have a little more control over the setting of your uniforms.
       *
       * @return {PIXI.GLProgram} the glProgram for the currently bound Shader for this context
       */
      getglProgram(): PIXI.GLProgram;

      /**
       * Resets ShaderSystem state, does not affect WebGL state
       */
      reset(): void;

      /**
       * Destroys this System and removes all its textures
       */
      destroy(): void;

      /**
       * The renderer this manager works for.
       *
       * @member {PIXI.Renderer} PIXI.System#renderer
       */
      renderer: PIXI.Renderer;
    }

    /**
     * System plugin to the renderer to manage WebGL state machines.
     *
     * @class
     * @extends PIXI.System
     * @memberof PIXI.systems
     */
    class StateSystem extends PIXI.System {
      constructor(renderer: PIXI.Renderer);

      /**
       * GL context
       * @member {WebGLRenderingContext} PIXI.systems.StateSystem#gl
       * @readonly
       */
      readonly gl: WebGLRenderingContext;
      /**
       * State ID
       * @member {number} PIXI.systems.StateSystem#stateId
       * @readonly
       */
      readonly stateId: number;
      /**
       * Polygon offset
       * @member {number} PIXI.systems.StateSystem#polygonOffset
       * @readonly
       */
      readonly polygonOffset: number;
      /**
       * Blend mode
       * @member {number} PIXI.systems.StateSystem#blendMode
       * @default PIXI.BLEND_MODES.NONE
       * @readonly
       */
      readonly blendMode: number;
      /**
       * Whether current blend equation is different
       * @member {boolean} PIXI.systems.StateSystem#_blendEq
       * @protected
       */
      protected _blendEq: boolean;
      /**
       * Collection of calls
       * @member {function[]} PIXI.systems.StateSystem#map
       * @readonly
       */
      readonly map: ((...params: any[]) => any)[];
      /**
       * Collection of check calls
       * @member {function[]} PIXI.systems.StateSystem#checks
       * @readonly
       */
      readonly checks: ((...params: any[]) => any)[];
      /**
       * Default WebGL State
       * @member {PIXI.State} PIXI.systems.StateSystem#defaultState
       * @readonly
       */
      readonly defaultState: PIXI.State;

      /**
       * Sets the current state
       *
       * @param {*} state - The state to set.
       */
      set(state: any): void;

      /**
       * Sets the state, when previous state is unknown
       *
       * @param {*} state - The state to set
       */
      forceState(state: any): void;

      /**
       * Enables or disabled blending.
       *
       * @param {boolean} value - Turn on or off webgl blending.
       */
      setBlend(value: boolean): void;

      /**
       * Enables or disable polygon offset fill
       *
       * @param {boolean} value - Turn on or off webgl polygon offset testing.
       */
      setOffset(value: boolean): void;

      /**
       * Sets whether to enable or disable depth test.
       *
       * @param {boolean} value - Turn on or off webgl depth testing.
       */
      setDepthTest(value: boolean): void;

      /**
       * Sets whether to enable or disable cull face.
       *
       * @param {boolean} value - Turn on or off webgl cull face.
       */
      setCullFace(value: boolean): void;

      /**
       * Sets the gl front face.
       *
       * @param {boolean} value - true is clockwise and false is counter-clockwise
       */
      setFrontFace(value: boolean): void;

      /**
       * Sets the blend mode.
       *
       * @param {number} value - The blend mode to set to.
       */
      setBlendMode(value: number): void;

      /**
       * Sets the polygon offset.
       *
       * @param {number} value - the polygon offset
       * @param {number} scale - the polygon offset scale
       */
      setPolygonOffset(value: number, scale: number): void;

      /**
       * Resets all the logic and disables the vaos
       */
      reset(): void;

      /**
       * checks to see which updates should be checked based on which settings have been activated.
       * For example, if blend is enabled then we should check the blend modes each time the state is changed
       * or if polygon fill is activated then we need to check if the polygon offset changes.
       * The idea is that we only check what we have too.
       *
       * @param {Function} func  the checking function to add or remove
       * @param {boolean} value  should the check function be added or removed.
       */
      updateCheck(func: (...params: any[]) => any, value: boolean): void;

      /**
       * The renderer this manager works for.
       *
       * @member {PIXI.Renderer} PIXI.System#renderer
       */
      renderer: PIXI.Renderer;

      /**
       * Generic destroy methods to be overridden by the subclass
       */
      destroy(): void;
    }

    /**
     * System plugin to the renderer to manage texture garbage collection on the GPU,
     * ensuring that it does not get clogged up with textures that are no longer being used.
     *
     * @class
     * @memberof PIXI.systems
     * @extends PIXI.System
     */
    class TextureGCSystem extends PIXI.System {
      constructor(renderer: PIXI.Renderer);

      /**
       * Count
       * @member {number} PIXI.systems.TextureGCSystem#count
       * @readonly
       */
      readonly count: number;
      /**
       * Check count
       * @member {number} PIXI.systems.TextureGCSystem#checkCount
       * @readonly
       */
      readonly checkCount: number;
      /**
       * Maximum idle time, in seconds
       * @member {number} PIXI.systems.TextureGCSystem#maxIdle
       * @see PIXI.settings.GC_MAX_IDLE
       */
      maxIdle: number;
      /**
       * Maximum number of item to check
       * @member {number} PIXI.systems.TextureGCSystem#checkCountMax
       * @see PIXI.settings.GC_MAX_CHECK_COUNT
       */
      checkCountMax: number;
      /**
       * Current garabage collection mode
       * @member {PIXI.GC_MODES} PIXI.systems.TextureGCSystem#mode
       * @see PIXI.settings.GC_MODE
       */
      mode: PIXI.GC_MODES;

      /**
       * Checks to see when the last time a texture was used
       * if the texture has not been used for a specified amount of time it will be removed from the GPU
       */
      postrender(): void;

      /**
       * Checks to see when the last time a texture was used
       * if the texture has not been used for a specified amount of time it will be removed from the GPU
       */
      run(): void;

      /**
       * Removes all the textures within the specified displayObject and its children from the GPU
       *
       * @param {PIXI.DisplayObject} displayObject - the displayObject to remove the textures from.
       */
      unload(displayObject: PIXI.DisplayObject): void;

      /**
       * The renderer this manager works for.
       *
       * @member {PIXI.Renderer} PIXI.System#renderer
       */
      renderer: PIXI.Renderer;

      /**
       * Generic destroy methods to be overridden by the subclass
       */
      destroy(): void;
    }

    /**
     * System plugin to the renderer to manage textures.
     *
     * @class
     * @extends PIXI.System
     * @memberof PIXI.systems
     */
    class TextureSystem extends PIXI.System {
      constructor(renderer: PIXI.Renderer);

      /**
       * Bound textures
       * @member {PIXI.BaseTexture[]} PIXI.systems.TextureSystem#boundTextures
       * @readonly
       */
      readonly boundTextures: PIXI.BaseTexture[];
      /**
       * Current location
       * @member {number} PIXI.systems.TextureSystem#currentLocation
       * @readonly
       */
      readonly currentLocation: number;
      /**
       * List of managed textures
       * @member {PIXI.BaseTexture[]} PIXI.systems.TextureSystem#managedTextures
       * @readonly
       */
      readonly managedTextures: PIXI.BaseTexture[];
      /**
       * BaseTexture value that shows that we don't know what is bound
       * @member {PIXI.BaseTexture} PIXI.systems.TextureSystem#unknownTexture
       * @readonly
       */
      readonly unknownTexture: PIXI.BaseTexture;

      /**
       * Sets up the renderer context and necessary buffers.
       */
      contextChange(): void;

      /**
       * Bind a texture to a specific location
       *
       * If you want to unbind something, please use `unbind(texture)` instead of `bind(null, textureLocation)`
       *
       * @param {PIXI.Texture|PIXI.BaseTexture} texture - Texture to bind
       * @param {number} [location=0] - Location to bind at
       */
      bind(texture: PIXI.Texture | PIXI.BaseTexture, location?: number): void;

      /**
       * Resets texture location and bound textures
       *
       * Actual `bind(null, i)` calls will be performed at next `unbind()` call
       */
      reset(): void;

      /**
       * Unbind a texture
       * @param {PIXI.Texture|PIXI.BaseTexture} texture - Texture to bind
       */
      unbind(texture: PIXI.Texture | PIXI.BaseTexture): void;

      /**
       * The renderer this manager works for.
       *
       * @member {PIXI.Renderer} PIXI.System#renderer
       */
      renderer: PIXI.Renderer;

      /**
       * Generic destroy methods to be overridden by the subclass
       */
      destroy(): void;
    }
  }

  /**
   * A Texture stores the information that represents an image.
   * All textures have a base texture, which contains information about the source.
   * Therefore you can have many textures all using a single BaseTexture
   *
   * @class
   * @extends PIXI.utils.EventEmitter
   * @memberof PIXI
   * @param {PIXI.resources.Resource|string|HTMLImageElement|HTMLCanvasElement|HTMLVideoElement} [resource=null]
   *        The current resource to use, for things that aren't Resource objects, will be converted
   *        into a Resource.
   * @param {Object} [options] - Collection of options
   * @param {PIXI.MIPMAP_MODES} [options.mipmap=PIXI.settings.MIPMAP_TEXTURES] - If mipmapping is enabled for texture
   * @param {number} [options.anisotropicLevel=PIXI.settings.ANISOTROPIC_LEVEL] - Anisotropic filtering level of texture
   * @param {PIXI.WRAP_MODES} [options.wrapMode=PIXI.settings.WRAP_MODE] - Wrap mode for textures
   * @param {PIXI.SCALE_MODES} [options.scaleMode=PIXI.settings.SCALE_MODE] - Default scale mode, linear, nearest
   * @param {PIXI.FORMATS} [options.format=PIXI.FORMATS.RGBA] - GL format type
   * @param {PIXI.TYPES} [options.type=PIXI.TYPES.UNSIGNED_BYTE] - GL data type
   * @param {PIXI.TARGETS} [options.target=PIXI.TARGETS.TEXTURE_2D] - GL texture target
   * @param {PIXI.ALPHA_MODES} [options.alphaMode=PIXI.ALPHA_MODES.UNPACK] - Pre multiply the image alpha
   * @param {number} [options.width=0] - Width of the texture
   * @param {number} [options.height=0] - Height of the texture
   * @param {number} [options.resolution] - Resolution of the base texture
   * @param {object} [options.resourceOptions] - Optional resource options,
   *        see {@link PIXI.resources.autoDetectResource autoDetectResource}
   */
  class BaseTexture extends PIXI.utils.EventEmitter {
    constructor(
      resource?: PIXI.resources.Resource | string | HTMLImageElement | HTMLCanvasElement | HTMLVideoElement,
      options?: {
        mipmap?: PIXI.MIPMAP_MODES;
        anisotropicLevel?: number;
        wrapMode?: PIXI.WRAP_MODES;
        scaleMode?: PIXI.SCALE_MODES;
        format?: PIXI.FORMATS;
        type?: PIXI.TYPES;
        target?: PIXI.TARGETS;
        alphaMode?: PIXI.ALPHA_MODES;
        width?: number;
        height?: number;
        resolution?: number;
        resourceOptions?: any;
      },
    );

    /**
     * The width of the base texture set when the image has loaded
     *
     * @readonly
     * @member {number} PIXI.BaseTexture#width
     */
    readonly width: number;
    /**
     * The height of the base texture set when the image has loaded
     *
     * @readonly
     * @member {number} PIXI.BaseTexture#height
     */
    readonly height: number;
    /**
     * The resolution / device pixel ratio of the texture
     *
     * @member {number} PIXI.BaseTexture#resolution
     * @default PIXI.settings.RESOLUTION
     */
    resolution: number;
    /**
     * Mipmap mode of the texture, affects downscaled images
     *
     * @member {PIXI.MIPMAP_MODES} PIXI.BaseTexture#mipmap
     * @default PIXI.settings.MIPMAP_TEXTURES
     */
    mipmap: PIXI.MIPMAP_MODES;
    /**
     * Anisotropic filtering level of texture
     *
     * @member {number} PIXI.BaseTexture#anisotropicLevel
     * @default PIXI.settings.ANISOTROPIC_LEVEL
     */
    anisotropicLevel: number;
    /**
     * How the texture wraps
     * @member {number} PIXI.BaseTexture#wrapMode
     */
    wrapMode: number;
    /**
     * The scale mode to apply when scaling this texture
     *
     * @member {PIXI.SCALE_MODES} PIXI.BaseTexture#scaleMode
     * @default PIXI.settings.SCALE_MODE
     */
    scaleMode: PIXI.SCALE_MODES;
    /**
     * The pixel format of the texture
     *
     * @member {PIXI.FORMATS} PIXI.BaseTexture#format
     * @default PIXI.FORMATS.RGBA
     */
    format: PIXI.FORMATS;
    /**
     * The type of resource data
     *
     * @member {PIXI.TYPES} PIXI.BaseTexture#type
     * @default PIXI.TYPES.UNSIGNED_BYTE
     */
    type: PIXI.TYPES;
    /**
     * The target type
     *
     * @member {PIXI.TARGETS} PIXI.BaseTexture#target
     * @default PIXI.TARGETS.TEXTURE_2D
     */
    target: PIXI.TARGETS;
    /**
     * How to treat premultiplied alpha, see {@link PIXI.ALPHA_MODES}.
     *
     * @member {PIXI.ALPHA_MODES} PIXI.BaseTexture#alphaMode
     * @default PIXI.ALPHA_MODES.UNPACK
     */
    alphaMode: PIXI.ALPHA_MODES;
    /**
     * Global unique identifier for this BaseTexture
     *
     * @member {string} PIXI.BaseTexture#uid
     * @protected
     */
    protected uid: string;
    /**
     * Used by automatic texture Garbage Collection, stores last GC tick when it was bound
     *
     * @member {number} PIXI.BaseTexture#touched
     * @protected
     */
    protected touched: number;
    /**
     * Whether or not the texture is a power of two, try to use power of two textures as much
     * as you can
     *
     * @readonly
     * @member {boolean} PIXI.BaseTexture#isPowerOfTwo
     * @default false
     */
    readonly isPowerOfTwo: boolean;
    /**
     * Used by TextureSystem to only update texture to the GPU when needed.
     * Please call `update()` to increment it.
     *
     * @readonly
     * @member {number} PIXI.BaseTexture#dirtyId
     */
    readonly dirtyId: number;
    /**
     * Used by TextureSystem to only update texture style when needed.
     *
     * @protected
     * @member {number} PIXI.BaseTexture#dirtyStyleId
     */
    protected dirtyStyleId: number;
    /**
     * Currently default cache ID.
     *
     * @member {string} PIXI.BaseTexture#cacheId
     */
    cacheId: string;
    /**
     * Generally speaking means when resource is loaded.
     * @readonly
     * @member {boolean} PIXI.BaseTexture#valid
     */
    readonly valid: boolean;
    /**
     * The collection of alternative cache ids, since some BaseTextures
     * can have more than one ID, short name and longer full URL
     *
     * @member {Array<string>} PIXI.BaseTexture#textureCacheIds
     * @readonly
     */
    readonly textureCacheIds: string[];
    /**
     * Flag if BaseTexture has been destroyed.
     *
     * @member {boolean} PIXI.BaseTexture#destroyed
     * @readonly
     */
    readonly destroyed: boolean;
    /**
     * The resource used by this BaseTexture, there can only
     * be one resource per BaseTexture, but textures can share
     * resources.
     *
     * @member {PIXI.resources.Resource} PIXI.BaseTexture#resource
     * @readonly
     */
    readonly resource: PIXI.resources.Resource;
    /**
     * Number of the texture batch, used by multi-texture renderers
     *
     * @member {number} PIXI.BaseTexture#_batchEnabled
     */
    _batchEnabled: number;
    /**
     * Location inside texture batch, used by multi-texture renderers
     *
     * @member {number} PIXI.BaseTexture#_batchLocation
     */
    _batchLocation: number;
    /**
     * Pixel width of the source of this texture
     *
     * @readonly
     * @member {number}
     */
    readonly realWidth: number;
    /**
     * Pixel height of the source of this texture
     *
     * @readonly
     * @member {number}
     */
    readonly realHeight: number;

    /**
     * Changes style options of BaseTexture
     *
     * @param {PIXI.SCALE_MODES} [scaleMode] - Pixi scalemode
     * @param {PIXI.MIPMAP_MODES} [mipmap] - enable mipmaps
     * @returns {PIXI.BaseTexture} this
     */
    setStyle(scaleMode?: PIXI.SCALE_MODES, mipmap?: PIXI.MIPMAP_MODES): PIXI.BaseTexture;

    /**
     * Changes w/h/resolution. Texture becomes valid if width and height are greater than zero.
     *
     * @param {number} width Visual width
     * @param {number} height Visual height
     * @param {number} [resolution] Optionally set resolution
     * @returns {PIXI.BaseTexture} this
     */
    setSize(width: number, height: number, resolution?: number): PIXI.BaseTexture;

    /**
     * Sets real size of baseTexture, preserves current resolution.
     *
     * @param {number} realWidth Full rendered width
     * @param {number} realHeight Full rendered height
     * @param {number} [resolution] Optionally set resolution
     * @returns {PIXI.BaseTexture} this
     */
    setRealSize(realWidth: number, realHeight: number, resolution?: number): PIXI.BaseTexture;

    /**
     * Changes resolution
     *
     * @param {number} [resolution] res
     * @returns {PIXI.BaseTexture} this
     */
    setResolution(resolution?: number): PIXI.BaseTexture;

    /**
     * Sets the resource if it wasn't set. Throws error if resource already present
     *
     * @param {PIXI.resources.Resource} resource - that is managing this BaseTexture
     * @returns {PIXI.BaseTexture} this
     */
    setResource(resource: PIXI.resources.Resource): PIXI.BaseTexture;

    /**
     * Invalidates the object. Texture becomes valid if width and height are greater than zero.
     */
    update(): void;

    /**
     * Destroys this base texture.
     * The method stops if resource doesn't want this texture to be destroyed.
     * Removes texture from all caches.
     */
    destroy(): void;

    /**
     * Frees the texture from WebGL memory without destroying this texture object.
     * This means you can still use the texture later which will upload it to GPU
     * memory again.
     *
     * @fires PIXI.BaseTexture#dispose
     */
    dispose(): void;

    /**
     * Helper function that creates a base texture based on the source you provide.
     * The source can be - image url, image element, canvas element. If the
     * source is an image url or an image element and not in the base texture
     * cache, it will be created and loaded.
     *
     * @static
     * @param {string|HTMLImageElement|HTMLCanvasElement|SVGElement|HTMLVideoElement} source - The
     *        source to create base texture from.
     * @param {object} [options] See {@link PIXI.BaseTexture}'s constructor for options.
     * @param {boolean} [strict] Enforce strict-mode, see {@link PIXI.settings.STRICT_TEXTURE_CACHE}.
     * @returns {PIXI.BaseTexture} The new base texture.
     */
    static from(
      source: string | HTMLImageElement | HTMLCanvasElement | SVGElement | HTMLVideoElement,
      options?: any,
      strict?: boolean,
    ): PIXI.BaseTexture;

    /**
     * Create a new BaseTexture with a BufferResource from a Float32Array.
     * RGBA values are floats from 0 to 1.
     * @static
     * @param {Float32Array|Uint8Array} buffer The optional array to use, if no data
     *        is provided, a new Float32Array is created.
     * @param {number} width - Width of the resource
     * @param {number} height - Height of the resource
     * @param {object} [options] See {@link PIXI.BaseTexture}'s constructor for options.
     * @return {PIXI.BaseTexture} The resulting new BaseTexture
     */
    static fromBuffer(
      buffer: Float32Array | Uint8Array,
      width: number,
      height: number,
      options?: any,
    ): PIXI.BaseTexture;

    /**
     * Adds a BaseTexture to the global BaseTextureCache. This cache is shared across the whole PIXI object.
     *
     * @static
     * @param {PIXI.BaseTexture} baseTexture - The BaseTexture to add to the cache.
     * @param {string} id - The id that the BaseTexture will be stored against.
     */
    static addToCache(baseTexture: PIXI.BaseTexture, id: string): void;

    /**
     * Remove a BaseTexture from the global BaseTextureCache.
     *
     * @static
     * @param {string|PIXI.BaseTexture} baseTexture - id of a BaseTexture to be removed, or a BaseTexture instance itself.
     * @return {PIXI.BaseTexture|null} The BaseTexture that was removed.
     */
    static removeFromCache(baseTexture: string | PIXI.BaseTexture): PIXI.BaseTexture | null;

    /**
     * Global number of the texture batch, used by multi-texture renderers
     *
     * @static
     * @member {number}
     */
    static _globalBatch: number;
  }

  /**
   * A Texture that depends on six other resources.
   *
   * @class
   * @extends PIXI.BaseTexture
   * @memberof PIXI
   */
  class CubeTexture extends PIXI.BaseTexture {
    static from(
      resources: string | HTMLImageElement | HTMLCanvasElement | SVGElement | HTMLVideoElement,
      options?: any,
    ): BaseTexture;
    /**
     * Generate a new CubeTexture.
     * @static
     * @param {string[]|PIXI.resources.Resource[]} resources - Collection of 6 URLs or resources
     * @param {object} [options] - Optional options passed to the resources being loaded.
     *        See {@PIXI.resources.autoDetectResource autoDetectResource} for more info
     *        on the options available to each resource.
     * @returns {PIXI.CubeTexture} new cube texture
     */
    static from(resources: string[] | PIXI.resources.Resource[], options?: any): PIXI.CubeTexture;

    /**
     * The width of the base texture set when the image has loaded
     *
     * @readonly
     * @member {number} PIXI.BaseTexture#width
     */
    readonly width: number;
    /**
     * The height of the base texture set when the image has loaded
     *
     * @readonly
     * @member {number} PIXI.BaseTexture#height
     */
    readonly height: number;
    /**
     * The resolution / device pixel ratio of the texture
     *
     * @member {number} PIXI.BaseTexture#resolution
     * @default PIXI.settings.RESOLUTION
     */
    resolution: number;
    /**
     * Mipmap mode of the texture, affects downscaled images
     *
     * @member {PIXI.MIPMAP_MODES} PIXI.BaseTexture#mipmap
     * @default PIXI.settings.MIPMAP_TEXTURES
     */
    mipmap: PIXI.MIPMAP_MODES;
    /**
     * Anisotropic filtering level of texture
     *
     * @member {number} PIXI.BaseTexture#anisotropicLevel
     * @default PIXI.settings.ANISOTROPIC_LEVEL
     */
    anisotropicLevel: number;
    /**
     * How the texture wraps
     * @member {number} PIXI.BaseTexture#wrapMode
     */
    wrapMode: number;
    /**
     * The scale mode to apply when scaling this texture
     *
     * @member {PIXI.SCALE_MODES} PIXI.BaseTexture#scaleMode
     * @default PIXI.settings.SCALE_MODE
     */
    scaleMode: PIXI.SCALE_MODES;
    /**
     * The pixel format of the texture
     *
     * @member {PIXI.FORMATS} PIXI.BaseTexture#format
     * @default PIXI.FORMATS.RGBA
     */
    format: PIXI.FORMATS;
    /**
     * The type of resource data
     *
     * @member {PIXI.TYPES} PIXI.BaseTexture#type
     * @default PIXI.TYPES.UNSIGNED_BYTE
     */
    type: PIXI.TYPES;
    /**
     * The target type
     *
     * @member {PIXI.TARGETS} PIXI.BaseTexture#target
     * @default PIXI.TARGETS.TEXTURE_2D
     */
    target: PIXI.TARGETS;
    /**
     * How to treat premultiplied alpha, see {@link PIXI.ALPHA_MODES}.
     *
     * @member {PIXI.ALPHA_MODES} PIXI.BaseTexture#alphaMode
     * @default PIXI.ALPHA_MODES.UNPACK
     */
    alphaMode: PIXI.ALPHA_MODES;
    /**
     * Global unique identifier for this BaseTexture
     *
     * @member {string} PIXI.BaseTexture#uid
     * @protected
     */
    protected uid: string;
    /**
     * Used by automatic texture Garbage Collection, stores last GC tick when it was bound
     *
     * @member {number} PIXI.BaseTexture#touched
     * @protected
     */
    protected touched: number;
    /**
     * Whether or not the texture is a power of two, try to use power of two textures as much
     * as you can
     *
     * @readonly
     * @member {boolean} PIXI.BaseTexture#isPowerOfTwo
     * @default false
     */
    readonly isPowerOfTwo: boolean;
    /**
     * Used by TextureSystem to only update texture to the GPU when needed.
     * Please call `update()` to increment it.
     *
     * @readonly
     * @member {number} PIXI.BaseTexture#dirtyId
     */
    readonly dirtyId: number;
    /**
     * Used by TextureSystem to only update texture style when needed.
     *
     * @protected
     * @member {number} PIXI.BaseTexture#dirtyStyleId
     */
    protected dirtyStyleId: number;
    /**
     * Currently default cache ID.
     *
     * @member {string} PIXI.BaseTexture#cacheId
     */
    cacheId: string;
    /**
     * Generally speaking means when resource is loaded.
     * @readonly
     * @member {boolean} PIXI.BaseTexture#valid
     */
    readonly valid: boolean;
    /**
     * The collection of alternative cache ids, since some BaseTextures
     * can have more than one ID, short name and longer full URL
     *
     * @member {Array<string>} PIXI.BaseTexture#textureCacheIds
     * @readonly
     */
    readonly textureCacheIds: string[];
    /**
     * Flag if BaseTexture has been destroyed.
     *
     * @member {boolean} PIXI.BaseTexture#destroyed
     * @readonly
     */
    readonly destroyed: boolean;
    /**
     * The resource used by this BaseTexture, there can only
     * be one resource per BaseTexture, but textures can share
     * resources.
     *
     * @member {PIXI.resources.Resource} PIXI.BaseTexture#resource
     * @readonly
     */
    readonly resource: PIXI.resources.Resource;
    /**
     * Number of the texture batch, used by multi-texture renderers
     *
     * @member {number} PIXI.BaseTexture#_batchEnabled
     */
    _batchEnabled: number;
    /**
     * Location inside texture batch, used by multi-texture renderers
     *
     * @member {number} PIXI.BaseTexture#_batchLocation
     */
    _batchLocation: number;
    /**
     * Pixel width of the source of this texture
     *
     * @readonly
     * @member {number}
     */
    readonly realWidth: number;
    /**
     * Pixel height of the source of this texture
     *
     * @readonly
     * @member {number}
     */
    readonly realHeight: number;

    /**
     * Changes style options of BaseTexture
     *
     * @param {PIXI.SCALE_MODES} [scaleMode] - Pixi scalemode
     * @param {PIXI.MIPMAP_MODES} [mipmap] - enable mipmaps
     * @returns {PIXI.BaseTexture} this
     */
    setStyle(scaleMode?: PIXI.SCALE_MODES, mipmap?: PIXI.MIPMAP_MODES): PIXI.BaseTexture;

    /**
     * Changes w/h/resolution. Texture becomes valid if width and height are greater than zero.
     *
     * @param {number} width Visual width
     * @param {number} height Visual height
     * @param {number} [resolution] Optionally set resolution
     * @returns {PIXI.BaseTexture} this
     */
    setSize(width: number, height: number, resolution?: number): PIXI.BaseTexture;

    /**
     * Sets real size of baseTexture, preserves current resolution.
     *
     * @param {number} realWidth Full rendered width
     * @param {number} realHeight Full rendered height
     * @param {number} [resolution] Optionally set resolution
     * @returns {PIXI.BaseTexture} this
     */
    setRealSize(realWidth: number, realHeight: number, resolution?: number): PIXI.BaseTexture;

    /**
     * Changes resolution
     *
     * @param {number} [resolution] res
     * @returns {PIXI.BaseTexture} this
     */
    setResolution(resolution?: number): PIXI.BaseTexture;

    /**
     * Sets the resource if it wasn't set. Throws error if resource already present
     *
     * @param {PIXI.resources.Resource} resource - that is managing this BaseTexture
     * @returns {PIXI.BaseTexture} this
     */
    setResource(resource: PIXI.resources.Resource): PIXI.BaseTexture;

    /**
     * Invalidates the object. Texture becomes valid if width and height are greater than zero.
     */
    update(): void;

    /**
     * Destroys this base texture.
     * The method stops if resource doesn't want this texture to be destroyed.
     * Removes texture from all caches.
     */
    destroy(): void;

    /**
     * Frees the texture from WebGL memory without destroying this texture object.
     * This means you can still use the texture later which will upload it to GPU
     * memory again.
     *
     * @fires PIXI.BaseTexture#dispose
     */
    dispose(): void;
  }

  /**
   * Internal texture for WebGL context
   * @class
   * @memberof PIXI
   */
  class GLTexture {
    constructor();

    /**
     * The WebGL texture
     * @member {WebGLTexture} PIXI.GLTexture#texture
     */
    texture: WebGLTexture;
    /**
     * Width of texture that was used in texImage2D
     * @member {number} PIXI.GLTexture#width
     */
    width: number;
    /**
     * Height of texture that was used in texImage2D
     * @member {number} PIXI.GLTexture#height
     */
    height: number;
    /**
     * Texture contents dirty flag
     * @member {number} PIXI.GLTexture#dirtyId
     */
    dirtyId: number;
    /**
     * Texture style dirty flag
     * @member {number} PIXI.GLTexture#dirtyStyleId
     */
    dirtyStyleId: number;
    /**
     * Whether mip levels has to be generated
     * @member {boolean} PIXI.GLTexture#mipmap
     */
    mipmap: boolean;
    /**
     * WrapMode copied from baseTexture
     * @member {number} PIXI.GLTexture#wrapMode
     */
    wrapMode: number;
    /**
     * Type copied from baseTexture
     * @member {number} PIXI.GLTexture#type
     */
    type: number;
    /**
     * Type copied from baseTexture
     * @member {number} PIXI.GLTexture#internalFormat
     */
    internalFormat: number;
  }

  /**
   * A texture stores the information that represents an image or part of an image.
   *
   * It cannot be added to the display list directly; instead use it as the texture for a Sprite.
   * If no frame is provided for a texture, then the whole image is used.
   *
   * You can directly create a texture from an image and then reuse it multiple times like this :
   *
   * ```js
   * let texture = PIXI.Texture.from('assets/image.png');
   * let sprite1 = new PIXI.Sprite(texture);
   * let sprite2 = new PIXI.Sprite(texture);
   * ```
   *
   * If you didnt pass the texture frame to constructor, it enables `noFrame` mode:
   * it subscribes on baseTexture events, it automatically resizes at the same time as baseTexture.
   *
   * Textures made from SVGs, loaded or not, cannot be used before the file finishes processing.
   * You can check for this by checking the sprite's _textureID property.
   * ```js
   * var texture = PIXI.Texture.from('assets/image.svg');
   * var sprite1 = new PIXI.Sprite(texture);
   * //sprite1._textureID should not be undefined if the texture has finished processing the SVG file
   * ```
   * You can use a ticker or rAF to ensure your sprites load the finished textures after processing. See issue #3068.
   *
   * @class
   * @extends PIXI.utils.EventEmitter
   * @memberof PIXI
   */
  class Texture extends PIXI.utils.EventEmitter {
    constructor(
      baseTexture: PIXI.BaseTexture,
      frame?: PIXI.Rectangle,
      orig?: PIXI.Rectangle,
      trim?: PIXI.Rectangle,
      rotate?: number,
      anchor?: PIXI.Point,
    );

    /**
     * Does this Texture have any frame data assigned to it?
     *
     * This mode is enabled automatically if no frame was passed inside constructor.
     *
     * In this mode texture is subscribed to baseTexture events, and fires `update` on any change.
     *
     * Beware, after loading or resize of baseTexture event can fired two times!
     * If you want more control, subscribe on baseTexture itself.
     *
     * ```js
     * texture.on('update', () => {});
     * ```
     *
     * Any assignment of `frame` switches off `noFrame` mode.
     *
     * @member {boolean} PIXI.Texture#noFrame
     */
    noFrame: boolean;
    /**
     * The base texture that this texture uses.
     *
     * @member {PIXI.BaseTexture} PIXI.Texture#baseTexture
     */
    baseTexture: PIXI.BaseTexture;
    /**
     * This is the area of the BaseTexture image to actually copy to the Canvas / WebGL when rendering,
     * irrespective of the actual frame size or placement (which can be influenced by trimmed texture atlases)
     *
     * @member {PIXI.Rectangle} PIXI.Texture#_frame
     */
    _frame: PIXI.Rectangle;
    /**
     * This is the trimmed area of original texture, before it was put in atlas
     * Please call `updateUvs()` after you change coordinates of `trim` manually.
     *
     * @member {PIXI.Rectangle} PIXI.Texture#trim
     */
    trim: PIXI.Rectangle;
    /**
     * This will let the renderer know if the texture is valid. If it's not then it cannot be rendered.
     *
     * @member {boolean} PIXI.Texture#valid
     */
    valid: boolean;
    /**
     * This will let a renderer know that a texture has been updated (used mainly for WebGL uv updates)
     *
     * @member {boolean} PIXI.Texture#requiresUpdate
     */
    requiresUpdate: boolean;
    /**
     * The WebGL UV data cache. Can be used as quad UV
     *
     * @member {PIXI.TextureUvs} PIXI.Texture#_uvs
     * @protected
     */
    protected _uvs: PIXI.TextureUvs;
    /**
     * Default TextureMatrix instance for this texture
     * By default that object is not created because its heavy
     *
     * @member {PIXI.TextureMatrix} PIXI.Texture#uvMatrix
     */
    uvMatrix: PIXI.TextureMatrix;
    /**
     * This is the area of original texture, before it was put in atlas
     *
     * @member {PIXI.Rectangle} PIXI.Texture#orig
     */
    orig: PIXI.Rectangle;
    /**
     * Anchor point that is used as default if sprite is created with this texture.
     * Changing the `defaultAnchor` at a later point of time will not update Sprite's anchor point.
     * @member {PIXI.Point} PIXI.Texture#defaultAnchor
     * @default {0,0}
     */
    defaultAnchor: PIXI.Point;
    /**
     * Update ID is observed by sprites and TextureMatrix instances.
     * Call updateUvs() to increment it.
     *
     * @member {number} PIXI.Texture#_updateID
     * @protected
     */
    protected _updateID: number;
    /**
     * The ids under which this Texture has been added to the texture cache. This is
     * automatically set as long as Texture.addToCache is used, but may not be set if a
     * Texture is added directly to the TextureCache array.
     *
     * @member {string[]} PIXI.Texture#textureCacheIds
     */
    textureCacheIds: string[];

    /**
     * Updates this texture on the gpu.
     *
     * Calls the TextureResource update.
     *
     * If you adjusted `frame` manually, please call `updateUvs()` instead.
     *
     */
    update(): void;

    /**
     * Called when the base texture is updated
     *
     * @protected
     * @param {PIXI.BaseTexture} baseTexture - The base texture.
     */
    protected onBaseTextureUpdated(baseTexture: PIXI.BaseTexture): void;

    /**
     * Destroys this texture
     *
     * @param {boolean} [destroyBase=false] Whether to destroy the base texture as well
     */
    destroy(destroyBase?: boolean): void;

    /**
     * Creates a new texture object that acts the same as this one.
     *
     * @return {PIXI.Texture} The new texture
     */
    clone(): PIXI.Texture;

    /**
     * Updates the internal WebGL UV cache. Use it after you change `frame` or `trim` of the texture.
     * Call it after changing the frame
     */
    updateUvs(): void;

    /**
     * Helper function that creates a new Texture based on the source you provide.
     * The source can be - frame id, image url, video url, canvas element, video element, base texture
     *
     * @static
     * @param {string|HTMLImageElement|HTMLCanvasElement|HTMLVideoElement|PIXI.BaseTexture} source
     *        Source to create texture from
     * @param {object} [options] See {@link PIXI.BaseTexture}'s constructor for options.
     * @param {boolean} [strict] Enforce strict-mode, see {@link PIXI.settings.STRICT_TEXTURE_CACHE}.
     * @return {PIXI.Texture} The newly created texture
     */
    static from(
      source: string | HTMLImageElement | HTMLCanvasElement | HTMLVideoElement | PIXI.BaseTexture,
      options?: any,
      strict?: boolean,
    ): PIXI.Texture;

    /**
     * Create a new Texture with a BufferResource from a Float32Array.
     * RGBA values are floats from 0 to 1.
     * @static
     * @param {Float32Array|Uint8Array} buffer The optional array to use, if no data
     *        is provided, a new Float32Array is created.
     * @param {number} width - Width of the resource
     * @param {number} height - Height of the resource
     * @param {object} [options] See {@link PIXI.BaseTexture}'s constructor for options.
     * @return {PIXI.Texture} The resulting new BaseTexture
     */
    static fromBuffer(buffer: Float32Array | Uint8Array, width: number, height: number, options?: any): PIXI.Texture;

    /**
     * Create a texture from a source and add to the cache.
     *
     * @static
     * @param {HTMLImageElement|HTMLCanvasElement} source - The input source.
     * @param {String} imageUrl - File name of texture, for cache and resolving resolution.
     * @param {String} [name] - Human readable name for the texture cache. If no name is
     *        specified, only `imageUrl` will be used as the cache ID.
     * @return {PIXI.Texture} Output texture
     */
    static fromLoader(source: HTMLImageElement | HTMLCanvasElement, imageUrl: string, name?: string): PIXI.Texture;

    /**
     * Adds a Texture to the global TextureCache. This cache is shared across the whole PIXI object.
     *
     * @static
     * @param {PIXI.Texture} texture - The Texture to add to the cache.
     * @param {string} id - The id that the Texture will be stored against.
     */
    static addToCache(texture: PIXI.Texture, id: string): void;

    /**
     * Remove a Texture from the global TextureCache.
     *
     * @static
     * @param {string|PIXI.Texture} texture - id of a Texture to be removed, or a Texture instance itself
     * @return {PIXI.Texture|null} The Texture that was removed
     */
    static removeFromCache(texture: string | PIXI.Texture): PIXI.Texture | null;

    /**
     * Returns resolution of baseTexture
     *
     * @member {number}
     * @readonly
     */
    readonly resolution: number;
    /**
     * The frame specifies the region of the base texture that this texture uses.
     * Please call `updateUvs()` after you change coordinates of `frame` manually.
     *
     * @member {PIXI.Rectangle}
     */
    frame: PIXI.Rectangle;
    /**
     * Indicates whether the texture is rotated inside the atlas
     * set to 2 to compensate for texture packer rotation
     * set to 6 to compensate for spine packer rotation
     * can be used to rotate or mirror sprites
     * See {@link PIXI.groupD8} for explanation
     *
     * @member {number}
     */
    rotate: number;
    /**
     * The width of the Texture in pixels.
     *
     * @member {number}
     */
    width: number;
    /**
     * The height of the Texture in pixels.
     *
     * @member {number}
     */
    height: number;
    /**
     * An empty texture, used often to not have to create multiple empty textures.
     * Can not be destroyed.
     *
     * @static
     * @constant
     * @member {PIXI.Texture}
     */
    static EMPTY: PIXI.Texture;
    /**
     * A white texture of 16x16 size, used for graphics and other things
     * Can not be destroyed.
     *
     * @static
     * @constant
     * @member {PIXI.Texture}
     */
    static WHITE: PIXI.Texture;
  }

  /**
   * Class controls uv mapping from Texture normal space to BaseTexture normal space.
   *
   * Takes `trim` and `rotate` into account. May contain clamp settings for Meshes and TilingSprite.
   *
   * Can be used in Texture `uvMatrix` field, or separately, you can use different clamp settings on the same texture.
   * If you want to add support for texture region of certain feature or filter, that's what you're looking for.
   *
   * Takes track of Texture changes through `_lastTextureID` private field.
   * Use `update()` method call to track it from outside.
   *
   * @see PIXI.Texture
   * @see PIXI.Mesh
   * @see PIXI.TilingSprite
   * @class
   * @memberof PIXI
   */
  class TextureMatrix {
    constructor(texture: PIXI.Texture, clampMargin?: number);

    /**
     * Matrix operation that converts texture region coords to texture coords
     * @member {PIXI.Matrix} PIXI.TextureMatrix#mapCoord
     * @readonly
     */
    readonly mapCoord: PIXI.Matrix;
    /**
     * Clamp region for normalized coords, left-top pixel center in xy , bottom-right in zw.
     * Calculated based on clampOffset.
     * @member {Float32Array} PIXI.TextureMatrix#uClampFrame
     * @readonly
     */
    readonly uClampFrame: Float32Array;
    /**
     * Normalized clamp offset.
     * Calculated based on clampOffset.
     * @member {Float32Array} PIXI.TextureMatrix#uClampOffset
     * @readonly
     */
    readonly uClampOffset: Float32Array;
    /**
     * Tracks Texture frame changes
     * @member {number} PIXI.TextureMatrix#_updateID
     * @protected
     */
    protected _updateID: number;
    /**
     * Changes frame clamping
     * Works with TilingSprite and Mesh
     * Change to 1.5 if you texture has repeated right and bottom lines, that leads to smoother borders
     *
     * @default 0
     * @member {number} PIXI.TextureMatrix#clampOffset
     */
    clampOffset: number;
    /**
     * Changes frame clamping
     * Works with TilingSprite and Mesh
     * Change to -0.5 to add a pixel to the edge, recommended for transparent trimmed textures in atlas
     *
     * @default 0.5
     * @member {number} PIXI.TextureMatrix#clampMargin
     */
    clampMargin: number;
    /**
     * If texture size is the same as baseTexture
     * @member {boolean} PIXI.TextureMatrix#isSimple
     * @default false
     * @readonly
     */
    readonly isSimple: boolean;
    /**
     * texture property
     * @member {PIXI.Texture}
     */
    texture: PIXI.Texture;

    /**
     * Multiplies uvs array to transform
     * @param {Float32Array} uvs mesh uvs
     * @param {Float32Array} [out=uvs] output
     * @returns {Float32Array} output
     */
    multiplyUvs(uvs: Float32Array, out?: Float32Array): Float32Array;

    /**
     * updates matrices if texture was changed
     * @param {boolean} [forceUpdate=false] if true, matrices will be updated any case
     * @returns {boolean} whether or not it was updated
     */
    update(forceUpdate?: boolean): boolean;
  }

  /**
   * Stores a texture's frame in UV coordinates, in
   * which everything lies in the rectangle `[(0,0), (1,0),
   * (1,1), (0,1)]`.
   *
   * | Corner       | Coordinates |
   * |--------------|-------------|
   * | Top-Left     | `(x0,y0)`   |
   * | Top-Right    | `(x1,y1)`   |
   * | Bottom-Right | `(x2,y2)`   |
   * | Bottom-Left  | `(x3,y3)`   |
   *
   * @class
   * @protected
   * @memberof PIXI
   */
  class TextureUvs {
    constructor();

    /**
     * X-component of top-left corner `(x0,y0)`.
     *
     * @member {number} PIXI.TextureUvs#x0
     */
    x0: number;
    /**
     * Y-component of top-left corner `(x0,y0)`.
     *
     * @member {number} PIXI.TextureUvs#y0
     */
    y0: number;
    /**
     * X-component of top-right corner `(x1,y1)`.
     *
     * @member {number} PIXI.TextureUvs#x1
     */
    x1: number;
    /**
     * Y-component of top-right corner `(x1,y1)`.
     *
     * @member {number} PIXI.TextureUvs#y1
     */
    y1: number;
    /**
     * X-component of bottom-right corner `(x2,y2)`.
     *
     * @member {number} PIXI.TextureUvs#x2
     */
    x2: number;
    /**
     * Y-component of bottom-right corner `(x2,y2)`.
     *
     * @member {number} PIXI.TextureUvs#y2
     */
    y2: number;
    /**
     * X-component of bottom-left corner `(x3,y3)`.
     *
     * @member {number} PIXI.TextureUvs#x3
     */
    x3: number;
    /**
     * Y-component of bottom-right corner `(x3,y3)`.
     *
     * @member {number} PIXI.TextureUvs#y3
     */
    y3: number;

    /**
     * Sets the texture Uvs based on the given frame information.
     *
     * @protected
     * @param {PIXI.Rectangle} frame - The frame of the texture
     * @param {PIXI.Rectangle} baseFrame - The base frame of the texture
     * @param {number} rotate - Rotation of frame, see {@link PIXI.groupD8}
     */
    protected set(frame: PIXI.Rectangle, baseFrame: PIXI.Rectangle, rotate: number): void;
  }

  /**
   * Collection of base resource types supported by PixiJS.
   *
   * Resources are used by {@link PIXI.BaseTexture} to handle different media types
   * such as images, video, SVG graphics, etc. In most use-cases, you should not
   * instantiate the resources directly. The easy thing is to use
   * {@link PIXI.BaseTexture.from}.
   * @example
   * const baseTexture = PIXI.BaseTexture.from('path/to/image.jpg');
   * @namespace PIXI.resources
   */
  namespace resources {
    /**
     * A resource that contains a number of sources.
     *
     * @class
     * @extends PIXI.resources.Resource
     * @memberof PIXI.resources
     * @param {number|Array<*>} source - Number of items in array or the collection
     *        of image URLs to use. Can also be resources, image elements, canvas, etc.
     * @param {object} [options] Options to apply to {@link PIXI.resources.autoDetectResource}
     * @param {number} [options.width] - Width of the resource
     * @param {number} [options.height] - Height of the resource
     */
    class ArrayResource extends PIXI.resources.Resource {
      constructor(
        source: number | any[],
        options?: {
          width?: number;
          height?: number;
        },
      );

      /**
       * Collection of resources.
       * @member {Array<PIXI.BaseTexture>} PIXI.resources.ArrayResource#items
       * @readonly
       */
      readonly items: PIXI.BaseTexture[];
      /**
       * Dirty IDs for each part
       * @member {Array<number>} PIXI.resources.ArrayResource#itemDirtyIds
       * @readonly
       */
      readonly itemDirtyIds: number[];
      /**
       * Number of elements in array
       *
       * @member {number} PIXI.resources.ArrayResource#length
       * @readonly
       */
      readonly length: number;

      /**
       * Set a resource by ID
       *
       * @param {PIXI.resources.Resource} resource
       * @param {number} index - Zero-based index of resource to set
       * @return {PIXI.resources.ArrayResource} Instance for chaining
       */
      addResourceAt(resource: PIXI.resources.Resource, index: number): PIXI.resources.ArrayResource;

      /**
       * Upload the resources to the GPU.
       * @param {PIXI.Renderer} renderer
       * @param {PIXI.BaseTexture} texture
       * @param {PIXI.GLTexture} glTexture
       * @returns {boolean} whether texture was uploaded
       */
      upload(renderer: PIXI.Renderer, texture: PIXI.BaseTexture, glTexture: PIXI.GLTexture): boolean;

      /**
       * Internal width of the resource
       * @member {number} PIXI.resources.Resource#_width
       * @protected
       */
      protected _width: number;
      /**
       * Internal height of the resource
       * @member {number} PIXI.resources.Resource#_height
       * @protected
       */
      protected _height: number;
      /**
       * If resource has been destroyed
       * @member {boolean} PIXI.resources.Resource#destroyed
       * @readonly
       * @default false
       */
      readonly destroyed: boolean;
      /**
       * `true` if resource is created by BaseTexture
       * useful for doing cleanup with BaseTexture destroy
       * and not cleaning up resources that were created
       * externally.
       * @member {boolean} PIXI.resources.Resource#internal
       * @protected
       */
      protected internal: boolean;

      /**
       * Bind to a parent BaseTexture
       *
       * @param {PIXI.BaseTexture} baseTexture - Parent texture
       */
      bind(baseTexture: PIXI.BaseTexture): void;

      /**
       * Unbind to a parent BaseTexture
       *
       * @param {PIXI.BaseTexture} baseTexture - Parent texture
       */
      unbind(baseTexture: PIXI.BaseTexture): void;

      /**
       * Trigger a resize event
       * @param {number} width X dimension
       * @param {number} height Y dimension
       */
      resize(width: number, height: number): void;

      /**
       * Has been validated
       * @readonly
       * @member {boolean}
       */
      readonly valid: boolean;

      /**
       * Has been updated trigger event
       */
      update(): void;

      /**
       * This can be overridden to start preloading a resource
       * or do any other prepare step.
       * @protected
       * @return {Promise<void>} Handle the validate event
       */
      protected load(): Promise<void>;

      /**
       * The width of the resource.
       *
       * @member {number}
       * @readonly
       */
      readonly width: number;
      /**
       * The height of the resource.
       *
       * @member {number}
       * @readonly
       */
      readonly height: number;

      /**
       * Set the style, optional to override
       *
       * @param {PIXI.Renderer} renderer - yeah, renderer!
       * @param {PIXI.BaseTexture} baseTexture - the texture
       * @param {PIXI.GLTexture} glTexture - texture instance for this webgl context
       * @returns {boolean} `true` is success
       */
      style(renderer: PIXI.Renderer, baseTexture: PIXI.BaseTexture, glTexture: PIXI.GLTexture): boolean;

      /**
       * Clean up anything, this happens when destroying is ready.
       *
       * @protected
       */
      protected dispose(): void;

      /**
       * Call when destroying resource, unbind any BaseTexture object
       * before calling this method, as reference counts are maintained
       * internally.
       */
      destroy(): void;
    }

    /**
     * Base for all the image/canvas resources
     * @class
     * @extends PIXI.resources.Resource
     * @memberof PIXI.resources
     */
    class BaseImageResource extends PIXI.resources.Resource {
      constructor(source: HTMLImageElement | HTMLCanvasElement | HTMLVideoElement | SVGElement);

      /**
       * The source element
       * @member {HTMLImageElement|HTMLCanvasElement|HTMLVideoElement|SVGElement} PIXI.resources.BaseImageResource#source
       * @readonly
       */
      readonly source: HTMLImageElement | HTMLCanvasElement | HTMLVideoElement | SVGElement;

      /**
       * Set cross origin based detecting the url and the crossorigin
       * @protected
       * @param {HTMLElement} element - Element to apply crossOrigin
       * @param {string} url - URL to check
       * @param {boolean|string} [crossorigin=true] - Cross origin value to use
       */
      protected static crossOrigin(element: HTMLElement, url: string, crossorigin?: boolean | string): void;

      /**
       * Upload the texture to the GPU.
       * @param {PIXI.Renderer} renderer Upload to the renderer
       * @param {PIXI.BaseTexture} baseTexture Reference to parent texture
       * @param {PIXI.GLTexture} glTexture
       * @param {HTMLImageElement|HTMLCanvasElement|HTMLVideoElement|SVGElement} [source] (optional)
       * @returns {boolean} true is success
       */
      upload(
        renderer: PIXI.Renderer,
        baseTexture: PIXI.BaseTexture,
        glTexture: PIXI.GLTexture,
        source?: HTMLImageElement | HTMLCanvasElement | HTMLVideoElement | SVGElement,
      ): boolean;

      /**
       * Checks if source width/height was changed, resize can cause extra baseTexture update.
       * Triggers one update in any case.
       */
      update(): void;

      /**
       * Internal width of the resource
       * @member {number} PIXI.resources.Resource#_width
       * @protected
       */
      protected _width: number;
      /**
       * Internal height of the resource
       * @member {number} PIXI.resources.Resource#_height
       * @protected
       */
      protected _height: number;
      /**
       * If resource has been destroyed
       * @member {boolean} PIXI.resources.Resource#destroyed
       * @readonly
       * @default false
       */
      readonly destroyed: boolean;
      /**
       * `true` if resource is created by BaseTexture
       * useful for doing cleanup with BaseTexture destroy
       * and not cleaning up resources that were created
       * externally.
       * @member {boolean} PIXI.resources.Resource#internal
       * @protected
       */
      protected internal: boolean;

      /**
       * Bind to a parent BaseTexture
       *
       * @param {PIXI.BaseTexture} baseTexture - Parent texture
       */
      bind(baseTexture: PIXI.BaseTexture): void;

      /**
       * Unbind to a parent BaseTexture
       *
       * @param {PIXI.BaseTexture} baseTexture - Parent texture
       */
      unbind(baseTexture: PIXI.BaseTexture): void;

      /**
       * Trigger a resize event
       * @param {number} width X dimension
       * @param {number} height Y dimension
       */
      resize(width: number, height: number): void;

      /**
       * Has been validated
       * @readonly
       * @member {boolean}
       */
      readonly valid: boolean;

      /**
       * This can be overridden to start preloading a resource
       * or do any other prepare step.
       * @protected
       * @return {Promise<void>} Handle the validate event
       */
      protected load(): Promise<void>;

      /**
       * The width of the resource.
       *
       * @member {number}
       * @readonly
       */
      readonly width: number;
      /**
       * The height of the resource.
       *
       * @member {number}
       * @readonly
       */
      readonly height: number;

      /**
       * Set the style, optional to override
       *
       * @param {PIXI.Renderer} renderer - yeah, renderer!
       * @param {PIXI.BaseTexture} baseTexture - the texture
       * @param {PIXI.GLTexture} glTexture - texture instance for this webgl context
       * @returns {boolean} `true` is success
       */
      style(renderer: PIXI.Renderer, baseTexture: PIXI.BaseTexture, glTexture: PIXI.GLTexture): boolean;

      /**
       * Clean up anything, this happens when destroying is ready.
       *
       * @protected
       */
      protected dispose(): void;

      /**
       * Call when destroying resource, unbind any BaseTexture object
       * before calling this method, as reference counts are maintained
       * internally.
       */
      destroy(): void;
    }

    /**
     * Buffer resource with data of typed array.
     * @class
     * @extends PIXI.resources.Resource
     * @memberof PIXI.resources
     */
    class BufferResource extends PIXI.resources.Resource {
      constructor(
        source: Float32Array | Uint8Array | Uint32Array,
        options: {
          width: number;
          height: number;
        },
      );

      /**
       * Source array
       * Cannot be ClampedUint8Array because it cant be uploaded to WebGL
       *
       * @member {Float32Array|Uint8Array|Uint32Array} PIXI.resources.BufferResource#data
       */
      data: Float32Array | Uint8Array | Uint32Array;

      /**
       * Upload the texture to the GPU.
       * @param {PIXI.Renderer} renderer Upload to the renderer
       * @param {PIXI.BaseTexture} baseTexture Reference to parent texture
       * @param {PIXI.GLTexture} glTexture glTexture
       * @returns {boolean} true is success
       */
      upload(renderer: PIXI.Renderer, baseTexture: PIXI.BaseTexture, glTexture: PIXI.GLTexture): boolean;

      /**
       * Used to auto-detect the type of resource.
       *
       * @static
       * @param {*} source - The source object
       * @return {boolean} `true` if <canvas>
       */
      static test(source: any): boolean;

      /**
       * Internal width of the resource
       * @member {number} PIXI.resources.Resource#_width
       * @protected
       */
      protected _width: number;
      /**
       * Internal height of the resource
       * @member {number} PIXI.resources.Resource#_height
       * @protected
       */
      protected _height: number;
      /**
       * If resource has been destroyed
       * @member {boolean} PIXI.resources.Resource#destroyed
       * @readonly
       * @default false
       */
      readonly destroyed: boolean;
      /**
       * `true` if resource is created by BaseTexture
       * useful for doing cleanup with BaseTexture destroy
       * and not cleaning up resources that were created
       * externally.
       * @member {boolean} PIXI.resources.Resource#internal
       * @protected
       */
      protected internal: boolean;

      /**
       * Bind to a parent BaseTexture
       *
       * @param {PIXI.BaseTexture} baseTexture - Parent texture
       */
      bind(baseTexture: PIXI.BaseTexture): void;

      /**
       * Unbind to a parent BaseTexture
       *
       * @param {PIXI.BaseTexture} baseTexture - Parent texture
       */
      unbind(baseTexture: PIXI.BaseTexture): void;

      /**
       * Trigger a resize event
       * @param {number} width X dimension
       * @param {number} height Y dimension
       */
      resize(width: number, height: number): void;

      /**
       * Has been validated
       * @readonly
       * @member {boolean}
       */
      readonly valid: boolean;

      /**
       * Has been updated trigger event
       */
      update(): void;

      /**
       * This can be overridden to start preloading a resource
       * or do any other prepare step.
       * @protected
       * @return {Promise<void>} Handle the validate event
       */
      protected load(): Promise<void>;

      /**
       * The width of the resource.
       *
       * @member {number}
       * @readonly
       */
      readonly width: number;
      /**
       * The height of the resource.
       *
       * @member {number}
       * @readonly
       */
      readonly height: number;

      /**
       * Set the style, optional to override
       *
       * @param {PIXI.Renderer} renderer - yeah, renderer!
       * @param {PIXI.BaseTexture} baseTexture - the texture
       * @param {PIXI.GLTexture} glTexture - texture instance for this webgl context
       * @returns {boolean} `true` is success
       */
      style(renderer: PIXI.Renderer, baseTexture: PIXI.BaseTexture, glTexture: PIXI.GLTexture): boolean;

      /**
       * Clean up anything, this happens when destroying is ready.
       *
       * @protected
       */
      protected dispose(): void;

      /**
       * Call when destroying resource, unbind any BaseTexture object
       * before calling this method, as reference counts are maintained
       * internally.
       */
      destroy(): void;
    }

    /**
     * Resource type for HTMLCanvasElement.
     * @class
     * @extends PIXI.resources.BaseImageResource
     * @memberof PIXI.resources
     * @param {HTMLCanvasElement} source - Canvas element to use
     */
    class CanvasResource extends PIXI.resources.BaseImageResource {
      constructor(source: HTMLCanvasElement);

      /**
       * Used to auto-detect the type of resource.
       *
       * @static
       * @param {HTMLCanvasElement|OffscreenCanvas} source - The source object
       * @return {boolean} `true` if source is HTMLCanvasElement or OffscreenCanvas
       */
      static test(source: HTMLCanvasElement | OffscreenCanvas): boolean;

      /**
       * The source element
       * @member {HTMLImageElement|HTMLCanvasElement|HTMLVideoElement|SVGElement} PIXI.resources.BaseImageResource#source
       * @readonly
       */
      readonly source: HTMLImageElement | HTMLCanvasElement | HTMLVideoElement | SVGElement;

      /**
       * Upload the texture to the GPU.
       * @param {PIXI.Renderer} renderer Upload to the renderer
       * @param {PIXI.BaseTexture} baseTexture Reference to parent texture
       * @param {PIXI.GLTexture} glTexture
       * @param {HTMLImageElement|HTMLCanvasElement|HTMLVideoElement|SVGElement} [source] (optional)
       * @returns {boolean} true is success
       */
      upload(
        renderer: PIXI.Renderer,
        baseTexture: PIXI.BaseTexture,
        glTexture: PIXI.GLTexture,
        source?: HTMLImageElement | HTMLCanvasElement | HTMLVideoElement | SVGElement,
      ): boolean;

      /**
       * Checks if source width/height was changed, resize can cause extra baseTexture update.
       * Triggers one update in any case.
       */
      update(): void;

      /**
       * Clean up anything, this happens when destroying is ready.
       *
       * @protected
       */
      protected dispose(): void;

      /**
       * Internal width of the resource
       * @member {number} PIXI.resources.Resource#_width
       * @protected
       */
      protected _width: number;
      /**
       * Internal height of the resource
       * @member {number} PIXI.resources.Resource#_height
       * @protected
       */
      protected _height: number;
      /**
       * If resource has been destroyed
       * @member {boolean} PIXI.resources.Resource#destroyed
       * @readonly
       * @default false
       */
      readonly destroyed: boolean;
      /**
       * `true` if resource is created by BaseTexture
       * useful for doing cleanup with BaseTexture destroy
       * and not cleaning up resources that were created
       * externally.
       * @member {boolean} PIXI.resources.Resource#internal
       * @protected
       */
      protected internal: boolean;

      /**
       * Bind to a parent BaseTexture
       *
       * @param {PIXI.BaseTexture} baseTexture - Parent texture
       */
      bind(baseTexture: PIXI.BaseTexture): void;

      /**
       * Unbind to a parent BaseTexture
       *
       * @param {PIXI.BaseTexture} baseTexture - Parent texture
       */
      unbind(baseTexture: PIXI.BaseTexture): void;

      /**
       * Trigger a resize event
       * @param {number} width X dimension
       * @param {number} height Y dimension
       */
      resize(width: number, height: number): void;

      /**
       * Has been validated
       * @readonly
       * @member {boolean}
       */
      readonly valid: boolean;

      /**
       * This can be overridden to start preloading a resource
       * or do any other prepare step.
       * @protected
       * @return {Promise<void>} Handle the validate event
       */
      protected load(): Promise<void>;

      /**
       * The width of the resource.
       *
       * @member {number}
       * @readonly
       */
      readonly width: number;
      /**
       * The height of the resource.
       *
       * @member {number}
       * @readonly
       */
      readonly height: number;

      /**
       * Set the style, optional to override
       *
       * @param {PIXI.Renderer} renderer - yeah, renderer!
       * @param {PIXI.BaseTexture} baseTexture - the texture
       * @param {PIXI.GLTexture} glTexture - texture instance for this webgl context
       * @returns {boolean} `true` is success
       */
      style(renderer: PIXI.Renderer, baseTexture: PIXI.BaseTexture, glTexture: PIXI.GLTexture): boolean;

      /**
       * Call when destroying resource, unbind any BaseTexture object
       * before calling this method, as reference counts are maintained
       * internally.
       */
      destroy(): void;
    }

    /**
     * Resource for a CubeTexture which contains six resources.
     *
     * @class
     * @extends PIXI.resources.ArrayResource
     * @memberof PIXI.resources
     * @param {Array<string|PIXI.resources.Resource>} [source] Collection of URLs or resources
     *        to use as the sides of the cube.
     * @param {object} [options] - ImageResource options
     * @param {number} [options.width] - Width of resource
     * @param {number} [options.height] - Height of resource
     */
    class CubeResource extends PIXI.resources.ArrayResource {
      constructor(
        source?: (string | PIXI.resources.Resource)[],
        options?: {
          width?: number;
          height?: number;
        },
      );

      /**
       * Upload the resource
       *
       * @returns {boolean} true is success
       */
      upload(): boolean;

      /**
       * Number of texture sides to store for CubeResources
       *
       * @name PIXI.resources.CubeResource.SIDES
       * @static
       * @member {number}
       * @default 6
       */
      static SIDES: number;
      /**
       * Collection of resources.
       * @member {Array<PIXI.BaseTexture>} PIXI.resources.ArrayResource#items
       * @readonly
       */
      readonly items: PIXI.BaseTexture[];
      /**
       * Dirty IDs for each part
       * @member {Array<number>} PIXI.resources.ArrayResource#itemDirtyIds
       * @readonly
       */
      readonly itemDirtyIds: number[];
      /**
       * Number of elements in array
       *
       * @member {number} PIXI.resources.ArrayResource#length
       * @readonly
       */
      readonly length: number;

      /**
       * Clean up anything, this happens when destroying is ready.
       *
       * @protected
       */
      protected dispose(): void;

      /**
       * Set a resource by ID
       *
       * @param {PIXI.resources.Resource} resource
       * @param {number} index - Zero-based index of resource to set
       * @return {PIXI.resources.ArrayResource} Instance for chaining
       */
      addResourceAt(resource: PIXI.resources.Resource, index: number): PIXI.resources.ArrayResource;

      /**
       * Bind to a parent BaseTexture
       *
       * @param {PIXI.BaseTexture} baseTexture - Parent texture
       */
      bind(baseTexture: PIXI.BaseTexture): void;

      /**
       * Unbind to a parent BaseTexture
       *
       * @param {PIXI.BaseTexture} baseTexture - Parent texture
       */
      unbind(baseTexture: PIXI.BaseTexture): void;

      /**
       * This can be overridden to start preloading a resource
       * or do any other prepare step.
       * @protected
       * @return {Promise<void>} Handle the validate event
       */
      protected load(): Promise<void>;

      /**
       * Internal width of the resource
       * @member {number} PIXI.resources.Resource#_width
       * @protected
       */
      protected _width: number;
      /**
       * Internal height of the resource
       * @member {number} PIXI.resources.Resource#_height
       * @protected
       */
      protected _height: number;
      /**
       * If resource has been destroyed
       * @member {boolean} PIXI.resources.Resource#destroyed
       * @readonly
       * @default false
       */
      readonly destroyed: boolean;
      /**
       * `true` if resource is created by BaseTexture
       * useful for doing cleanup with BaseTexture destroy
       * and not cleaning up resources that were created
       * externally.
       * @member {boolean} PIXI.resources.Resource#internal
       * @protected
       */
      protected internal: boolean;

      /**
       * Trigger a resize event
       * @param {number} width X dimension
       * @param {number} height Y dimension
       */
      resize(width: number, height: number): void;

      /**
       * Has been validated
       * @readonly
       * @member {boolean}
       */
      readonly valid: boolean;

      /**
       * Has been updated trigger event
       */
      update(): void;

      /**
       * The width of the resource.
       *
       * @member {number}
       * @readonly
       */
      readonly width: number;
      /**
       * The height of the resource.
       *
       * @member {number}
       * @readonly
       */
      readonly height: number;

      /**
       * Set the style, optional to override
       *
       * @param {PIXI.Renderer} renderer - yeah, renderer!
       * @param {PIXI.BaseTexture} baseTexture - the texture
       * @param {PIXI.GLTexture} glTexture - texture instance for this webgl context
       * @returns {boolean} `true` is success
       */
      style(renderer: PIXI.Renderer, baseTexture: PIXI.BaseTexture, glTexture: PIXI.GLTexture): boolean;

      /**
       * Call when destroying resource, unbind any BaseTexture object
       * before calling this method, as reference counts are maintained
       * internally.
       */
      destroy(): void;
    }

    /**
     * Resource type for DepthTexture.
     * @class
     * @extends PIXI.resources.BufferResource
     * @memberof PIXI.resources
     */
    class DepthResource extends PIXI.resources.BufferResource {
      /**
       * Upload the texture to the GPU.
       * @param {PIXI.Renderer} renderer Upload to the renderer
       * @param {PIXI.BaseTexture} baseTexture Reference to parent texture
       * @param {PIXI.GLTexture} glTexture glTexture
       * @returns {boolean} true is success
       */
      upload(renderer: PIXI.Renderer, baseTexture: PIXI.BaseTexture, glTexture: PIXI.GLTexture): boolean;

      /**
       * Source array
       * Cannot be ClampedUint8Array because it cant be uploaded to WebGL
       *
       * @member {Float32Array|Uint8Array|Uint32Array} PIXI.resources.BufferResource#data
       */
      data: Float32Array | Uint8Array | Uint32Array;

      /**
       * Clean up anything, this happens when destroying is ready.
       *
       * @protected
       */
      protected dispose(): void;

      /**
       * Internal width of the resource
       * @member {number} PIXI.resources.Resource#_width
       * @protected
       */
      protected _width: number;
      /**
       * Internal height of the resource
       * @member {number} PIXI.resources.Resource#_height
       * @protected
       */
      protected _height: number;
      /**
       * If resource has been destroyed
       * @member {boolean} PIXI.resources.Resource#destroyed
       * @readonly
       * @default false
       */
      readonly destroyed: boolean;
      /**
       * `true` if resource is created by BaseTexture
       * useful for doing cleanup with BaseTexture destroy
       * and not cleaning up resources that were created
       * externally.
       * @member {boolean} PIXI.resources.Resource#internal
       * @protected
       */
      protected internal: boolean;

      /**
       * Bind to a parent BaseTexture
       *
       * @param {PIXI.BaseTexture} baseTexture - Parent texture
       */
      bind(baseTexture: PIXI.BaseTexture): void;

      /**
       * Unbind to a parent BaseTexture
       *
       * @param {PIXI.BaseTexture} baseTexture - Parent texture
       */
      unbind(baseTexture: PIXI.BaseTexture): void;

      /**
       * Trigger a resize event
       * @param {number} width X dimension
       * @param {number} height Y dimension
       */
      resize(width: number, height: number): void;

      /**
       * Has been validated
       * @readonly
       * @member {boolean}
       */
      readonly valid: boolean;

      /**
       * Has been updated trigger event
       */
      update(): void;

      /**
       * This can be overridden to start preloading a resource
       * or do any other prepare step.
       * @protected
       * @return {Promise<void>} Handle the validate event
       */
      protected load(): Promise<void>;

      /**
       * The width of the resource.
       *
       * @member {number}
       * @readonly
       */
      readonly width: number;
      /**
       * The height of the resource.
       *
       * @member {number}
       * @readonly
       */
      readonly height: number;

      /**
       * Set the style, optional to override
       *
       * @param {PIXI.Renderer} renderer - yeah, renderer!
       * @param {PIXI.BaseTexture} baseTexture - the texture
       * @param {PIXI.GLTexture} glTexture - texture instance for this webgl context
       * @returns {boolean} `true` is success
       */
      style(renderer: PIXI.Renderer, baseTexture: PIXI.BaseTexture, glTexture: PIXI.GLTexture): boolean;

      /**
       * Call when destroying resource, unbind any BaseTexture object
       * before calling this method, as reference counts are maintained
       * internally.
       */
      destroy(): void;
    }

    /**
     * Resource type for ImageBitmap.
     * @class
     * @extends PIXI.resources.BaseImageResource
     * @memberof PIXI.resources
     * @param {ImageBitmap} source - Image element to use
     */
    class ImageBitmapResource extends PIXI.resources.BaseImageResource {
      constructor(source: ImageBitmap);

      /**
       * Used to auto-detect the type of resource.
       *
       * @static
       * @param {ImageBitmap} source - The source object
       * @return {boolean} `true` if source is an ImageBitmap
       */
      static test(source: ImageBitmap): boolean;

      /**
       * The source element
       * @member {HTMLImageElement|HTMLCanvasElement|HTMLVideoElement|SVGElement} PIXI.resources.BaseImageResource#source
       * @readonly
       */
      readonly source: HTMLImageElement | HTMLCanvasElement | HTMLVideoElement | SVGElement;

      /**
       * Upload the texture to the GPU.
       * @param {PIXI.Renderer} renderer Upload to the renderer
       * @param {PIXI.BaseTexture} baseTexture Reference to parent texture
       * @param {PIXI.GLTexture} glTexture
       * @param {HTMLImageElement|HTMLCanvasElement|HTMLVideoElement|SVGElement} [source] (optional)
       * @returns {boolean} true is success
       */
      upload(
        renderer: PIXI.Renderer,
        baseTexture: PIXI.BaseTexture,
        glTexture: PIXI.GLTexture,
        source?: HTMLImageElement | HTMLCanvasElement | HTMLVideoElement | SVGElement,
      ): boolean;

      /**
       * Checks if source width/height was changed, resize can cause extra baseTexture update.
       * Triggers one update in any case.
       */
      update(): void;

      /**
       * Clean up anything, this happens when destroying is ready.
       *
       * @protected
       */
      protected dispose(): void;

      /**
       * Internal width of the resource
       * @member {number} PIXI.resources.Resource#_width
       * @protected
       */
      protected _width: number;
      /**
       * Internal height of the resource
       * @member {number} PIXI.resources.Resource#_height
       * @protected
       */
      protected _height: number;
      /**
       * If resource has been destroyed
       * @member {boolean} PIXI.resources.Resource#destroyed
       * @readonly
       * @default false
       */
      readonly destroyed: boolean;
      /**
       * `true` if resource is created by BaseTexture
       * useful for doing cleanup with BaseTexture destroy
       * and not cleaning up resources that were created
       * externally.
       * @member {boolean} PIXI.resources.Resource#internal
       * @protected
       */
      protected internal: boolean;

      /**
       * Bind to a parent BaseTexture
       *
       * @param {PIXI.BaseTexture} baseTexture - Parent texture
       */
      bind(baseTexture: PIXI.BaseTexture): void;

      /**
       * Unbind to a parent BaseTexture
       *
       * @param {PIXI.BaseTexture} baseTexture - Parent texture
       */
      unbind(baseTexture: PIXI.BaseTexture): void;

      /**
       * Trigger a resize event
       * @param {number} width X dimension
       * @param {number} height Y dimension
       */
      resize(width: number, height: number): void;

      /**
       * Has been validated
       * @readonly
       * @member {boolean}
       */
      readonly valid: boolean;

      /**
       * This can be overridden to start preloading a resource
       * or do any other prepare step.
       * @protected
       * @return {Promise<void>} Handle the validate event
       */
      protected load(): Promise<void>;

      /**
       * The width of the resource.
       *
       * @member {number}
       * @readonly
       */
      readonly width: number;
      /**
       * The height of the resource.
       *
       * @member {number}
       * @readonly
       */
      readonly height: number;

      /**
       * Set the style, optional to override
       *
       * @param {PIXI.Renderer} renderer - yeah, renderer!
       * @param {PIXI.BaseTexture} baseTexture - the texture
       * @param {PIXI.GLTexture} glTexture - texture instance for this webgl context
       * @returns {boolean} `true` is success
       */
      style(renderer: PIXI.Renderer, baseTexture: PIXI.BaseTexture, glTexture: PIXI.GLTexture): boolean;

      /**
       * Call when destroying resource, unbind any BaseTexture object
       * before calling this method, as reference counts are maintained
       * internally.
       */
      destroy(): void;
    }

    /**
     * Resource type for HTMLImageElement.
     * @class
     * @extends PIXI.resources.BaseImageResource
     * @memberof PIXI.resources
     */
    class ImageResource extends PIXI.resources.BaseImageResource {
      constructor(source: HTMLImageElement | string);

      /**
       * URL of the image source
       * @member {string} PIXI.resources.ImageResource#url
       */
      url: string;
      /**
       * If the image should be disposed after upload
       * @member {boolean} PIXI.resources.ImageResource#preserveBitmap
       * @default false
       */
      preserveBitmap: boolean;
      /**
       * If capable, convert the image using createImageBitmap API
       * @member {boolean} PIXI.resources.ImageResource#createBitmap
       * @default PIXI.settings.CREATE_IMAGE_BITMAP
       */
      createBitmap: boolean;
      /**
       * Controls texture alphaMode field
       * Copies from options
       * Default is `null`, copies option from baseTexture
       *
       * @member {PIXI.ALPHA_MODES|null} PIXI.resources.ImageResource#alphaMode
       * @readonly
       */
      readonly alphaMode: PIXI.ALPHA_MODES | null;
      /**
       * The ImageBitmap element created for HTMLImageElement
       * @member {ImageBitmap} PIXI.resources.ImageResource#bitmap
       * @default null
       */
      bitmap: ImageBitmap;

      /**
       * returns a promise when image will be loaded and processed
       *
       * @param {boolean} [createBitmap=true] whether process image into bitmap
       * @returns {Promise<void>}
       */
      load(createBitmap?: boolean): Promise<void>;

      /**
       * Called when we need to convert image into BitmapImage.
       * Can be called multiple times, real promise is cached inside.
       *
       * @returns {Promise<void>} cached promise to fill that bitmap
       */
      process(): Promise<void>;

      /**
       * Upload the image resource to GPU.
       *
       * @param {PIXI.Renderer} renderer - Renderer to upload to
       * @param {PIXI.BaseTexture} baseTexture - BaseTexture for this resource
       * @param {PIXI.GLTexture} glTexture - GLTexture to use
       * @returns {boolean} true is success
       */
      upload(renderer: PIXI.Renderer, baseTexture: PIXI.BaseTexture, glTexture: PIXI.GLTexture): boolean;

      /**
       * The source element
       * @member {HTMLImageElement|HTMLCanvasElement|HTMLVideoElement|SVGElement} PIXI.resources.BaseImageResource#source
       * @readonly
       */
      readonly source: HTMLImageElement | HTMLCanvasElement | HTMLVideoElement | SVGElement;

      /**
       * Checks if source width/height was changed, resize can cause extra baseTexture update.
       * Triggers one update in any case.
       */
      update(): void;

      /**
       * Clean up anything, this happens when destroying is ready.
       *
       * @protected
       */
      protected dispose(): void;

      /**
       * Internal width of the resource
       * @member {number} PIXI.resources.Resource#_width
       * @protected
       */
      protected _width: number;
      /**
       * Internal height of the resource
       * @member {number} PIXI.resources.Resource#_height
       * @protected
       */
      protected _height: number;
      /**
       * If resource has been destroyed
       * @member {boolean} PIXI.resources.Resource#destroyed
       * @readonly
       * @default false
       */
      readonly destroyed: boolean;
      /**
       * `true` if resource is created by BaseTexture
       * useful for doing cleanup with BaseTexture destroy
       * and not cleaning up resources that were created
       * externally.
       * @member {boolean} PIXI.resources.Resource#internal
       * @protected
       */
      protected internal: boolean;

      /**
       * Bind to a parent BaseTexture
       *
       * @param {PIXI.BaseTexture} baseTexture - Parent texture
       */
      bind(baseTexture: PIXI.BaseTexture): void;

      /**
       * Unbind to a parent BaseTexture
       *
       * @param {PIXI.BaseTexture} baseTexture - Parent texture
       */
      unbind(baseTexture: PIXI.BaseTexture): void;

      /**
       * Trigger a resize event
       * @param {number} width X dimension
       * @param {number} height Y dimension
       */
      resize(width: number, height: number): void;

      /**
       * Has been validated
       * @readonly
       * @member {boolean}
       */
      readonly valid: boolean;
      /**
       * The width of the resource.
       *
       * @member {number}
       * @readonly
       */
      readonly width: number;
      /**
       * The height of the resource.
       *
       * @member {number}
       * @readonly
       */
      readonly height: number;

      /**
       * Set the style, optional to override
       *
       * @param {PIXI.Renderer} renderer - yeah, renderer!
       * @param {PIXI.BaseTexture} baseTexture - the texture
       * @param {PIXI.GLTexture} glTexture - texture instance for this webgl context
       * @returns {boolean} `true` is success
       */
      style(renderer: PIXI.Renderer, baseTexture: PIXI.BaseTexture, glTexture: PIXI.GLTexture): boolean;

      /**
       * Call when destroying resource, unbind any BaseTexture object
       * before calling this method, as reference counts are maintained
       * internally.
       */
      destroy(): void;
    }

    /**
     * Base resource class for textures that manages validation and uploading, depending on its type.
     *
     * Uploading of a base texture to the GPU is required.
     *
     * @class
     * @memberof PIXI.resources
     */
    class Resource {
      constructor(width?: number, height?: number);

      /**
       * Internal width of the resource
       * @member {number} PIXI.resources.Resource#_width
       * @protected
       */
      protected _width: number;
      /**
       * Internal height of the resource
       * @member {number} PIXI.resources.Resource#_height
       * @protected
       */
      protected _height: number;
      /**
       * If resource has been destroyed
       * @member {boolean} PIXI.resources.Resource#destroyed
       * @readonly
       * @default false
       */
      readonly destroyed: boolean;
      /**
       * `true` if resource is created by BaseTexture
       * useful for doing cleanup with BaseTexture destroy
       * and not cleaning up resources that were created
       * externally.
       * @member {boolean} PIXI.resources.Resource#internal
       * @protected
       */
      protected internal: boolean;

      /**
       * Bind to a parent BaseTexture
       *
       * @param {PIXI.BaseTexture} baseTexture - Parent texture
       */
      bind(baseTexture: PIXI.BaseTexture): void;

      /**
       * Unbind to a parent BaseTexture
       *
       * @param {PIXI.BaseTexture} baseTexture - Parent texture
       */
      unbind(baseTexture: PIXI.BaseTexture): void;

      /**
       * Trigger a resize event
       * @param {number} width X dimension
       * @param {number} height Y dimension
       */
      resize(width: number, height: number): void;

      /**
       * Has been validated
       * @readonly
       * @member {boolean}
       */
      readonly valid: boolean;

      /**
       * Has been updated trigger event
       */
      update(): void;

      /**
       * This can be overridden to start preloading a resource
       * or do any other prepare step.
       * @protected
       * @return {Promise<void>} Handle the validate event
       */
      protected load(): Promise<void>;

      /**
       * The width of the resource.
       *
       * @member {number}
       * @readonly
       */
      readonly width: number;
      /**
       * The height of the resource.
       *
       * @member {number}
       * @readonly
       */
      readonly height: number;

      /**
       * Uploads the texture or returns false if it cant for some reason. Override this.
       *
       * @param {PIXI.Renderer} renderer - yeah, renderer!
       * @param {PIXI.BaseTexture} baseTexture - the texture
       * @param {PIXI.GLTexture} glTexture - texture instance for this webgl context
       * @returns {boolean} true is success
       */
      upload(renderer: PIXI.Renderer, baseTexture: PIXI.BaseTexture, glTexture: PIXI.GLTexture): boolean;

      /**
       * Set the style, optional to override
       *
       * @param {PIXI.Renderer} renderer - yeah, renderer!
       * @param {PIXI.BaseTexture} baseTexture - the texture
       * @param {PIXI.GLTexture} glTexture - texture instance for this webgl context
       * @returns {boolean} `true` is success
       */
      style(renderer: PIXI.Renderer, baseTexture: PIXI.BaseTexture, glTexture: PIXI.GLTexture): boolean;

      /**
       * Clean up anything, this happens when destroying is ready.
       *
       * @protected
       */
      protected dispose(): void;

      /**
       * Call when destroying resource, unbind any BaseTexture object
       * before calling this method, as reference counts are maintained
       * internally.
       */
      destroy(): void;
    }

    /**
     * Resource type for SVG elements and graphics.
     * @class
     * @extends PIXI.resources.BaseImageResource
     * @memberof PIXI.resources
     * @param {string} source - Base64 encoded SVG element or URL for SVG file.
     * @param {object} [options] - Options to use
     * @param {number} [options.scale=1] Scale to apply to SVG. Overridden by...
     * @param {number} [options.width] Rasterize SVG this wide. Aspect ratio preserved if height not specified.
     * @param {number} [options.height] Rasterize SVG this high. Aspect ratio preserved if width not specified.
     * @param {boolean} [options.autoLoad=true] Start loading right away.
     */
    class SVGResource extends PIXI.resources.BaseImageResource {
      constructor(
        source: string,
        options?: {
          scale?: number;
          width?: number;
          height?: number;
          autoLoad?: boolean;
        },
      );

      /**
       * Base64 encoded SVG element or URL for SVG file
       * @readonly
       * @member {string} PIXI.resources.SVGResource#svg
       */
      readonly svg: string;
      /**
       * The source scale to apply when rasterizing on load
       * @readonly
       * @member {number} PIXI.resources.SVGResource#scale
       */
      readonly scale: number;
      /**
       * A width override for rasterization on load
       * @readonly
       * @member {number} PIXI.resources.SVGResource#_overrideWidth
       */
      readonly _overrideWidth: number;
      /**
       * A height override for rasterization on load
       * @readonly
       * @member {number} PIXI.resources.SVGResource#_overrideHeight
       */
      readonly _overrideHeight: number;

      /**
       * Get size from an svg string using regexp.
       *
       * @method
       * @param {string} svgString - a serialized svg element
       * @return {PIXI.ISize} image extension
       */
      static getSize(svgString: string): PIXI.ISize;

      /**
       * Used to auto-detect the type of resource.
       *
       * @static
       * @param {*} source - The source object
       * @param {string} extension - The extension of source, if set
       */
      static test(source: any, extension: string): void;

      /**
       * RegExp for SVG size.
       *
       * @static
       * @constant {RegExp|string} SVG_SIZE
       * @memberof PIXI.resources.SVGResource
       * @example &lt;svg width="100" height="100"&gt;&lt;/svg&gt;
       */
      static readonly SVG_SIZE: RegExp | string;
      /**
       * The source element
       * @member {HTMLImageElement|HTMLCanvasElement|HTMLVideoElement|SVGElement} PIXI.resources.BaseImageResource#source
       * @readonly
       */
      readonly source: HTMLImageElement | HTMLCanvasElement | HTMLVideoElement | SVGElement;

      /**
       * Upload the texture to the GPU.
       * @param {PIXI.Renderer} renderer Upload to the renderer
       * @param {PIXI.BaseTexture} baseTexture Reference to parent texture
       * @param {PIXI.GLTexture} glTexture
       * @param {HTMLImageElement|HTMLCanvasElement|HTMLVideoElement|SVGElement} [source] (optional)
       * @returns {boolean} true is success
       */
      upload(
        renderer: PIXI.Renderer,
        baseTexture: PIXI.BaseTexture,
        glTexture: PIXI.GLTexture,
        source?: HTMLImageElement | HTMLCanvasElement | HTMLVideoElement | SVGElement,
      ): boolean;

      /**
       * Checks if source width/height was changed, resize can cause extra baseTexture update.
       * Triggers one update in any case.
       */
      update(): void;

      /**
       * Clean up anything, this happens when destroying is ready.
       *
       * @protected
       */
      protected dispose(): void;

      /**
       * Internal width of the resource
       * @member {number} PIXI.resources.Resource#_width
       * @protected
       */
      protected _width: number;
      /**
       * Internal height of the resource
       * @member {number} PIXI.resources.Resource#_height
       * @protected
       */
      protected _height: number;
      /**
       * If resource has been destroyed
       * @member {boolean} PIXI.resources.Resource#destroyed
       * @readonly
       * @default false
       */
      readonly destroyed: boolean;
      /**
       * `true` if resource is created by BaseTexture
       * useful for doing cleanup with BaseTexture destroy
       * and not cleaning up resources that were created
       * externally.
       * @member {boolean} PIXI.resources.Resource#internal
       * @protected
       */
      protected internal: boolean;

      /**
       * Bind to a parent BaseTexture
       *
       * @param {PIXI.BaseTexture} baseTexture - Parent texture
       */
      bind(baseTexture: PIXI.BaseTexture): void;

      /**
       * Unbind to a parent BaseTexture
       *
       * @param {PIXI.BaseTexture} baseTexture - Parent texture
       */
      unbind(baseTexture: PIXI.BaseTexture): void;

      /**
       * Trigger a resize event
       * @param {number} width X dimension
       * @param {number} height Y dimension
       */
      resize(width: number, height: number): void;

      /**
       * Has been validated
       * @readonly
       * @member {boolean}
       */
      readonly valid: boolean;

      /**
       * This can be overridden to start preloading a resource
       * or do any other prepare step.
       * @protected
       * @return {Promise<void>} Handle the validate event
       */
      protected load(): Promise<void>;

      /**
       * The width of the resource.
       *
       * @member {number}
       * @readonly
       */
      readonly width: number;
      /**
       * The height of the resource.
       *
       * @member {number}
       * @readonly
       */
      readonly height: number;

      /**
       * Set the style, optional to override
       *
       * @param {PIXI.Renderer} renderer - yeah, renderer!
       * @param {PIXI.BaseTexture} baseTexture - the texture
       * @param {PIXI.GLTexture} glTexture - texture instance for this webgl context
       * @returns {boolean} `true` is success
       */
      style(renderer: PIXI.Renderer, baseTexture: PIXI.BaseTexture, glTexture: PIXI.GLTexture): boolean;

      /**
       * Call when destroying resource, unbind any BaseTexture object
       * before calling this method, as reference counts are maintained
       * internally.
       */
      destroy(): void;
    }

    /**
     * Resource type for HTMLVideoElement.
     * @class
     * @extends PIXI.resources.BaseImageResource
     * @memberof PIXI.resources
     * @param {HTMLVideoElement|object|string|Array<string|object>} source - Video element to use.
     * @param {object} [options] - Options to use
     * @param {boolean} [options.autoLoad=true] - Start loading the video immediately
     * @param {boolean} [options.autoPlay=true] - Start playing video immediately
     * @param {number} [options.updateFPS=0] - How many times a second to update the texture from the video.
     * Leave at 0 to update at every render.
     * @param {boolean} [options.crossorigin=true] - Load image using cross origin
     */
    class VideoResource extends PIXI.resources.BaseImageResource {
      constructor(
        source: HTMLVideoElement | any | string | (string | any)[],
        options?: {
          autoLoad?: boolean;
          autoPlay?: boolean;
          updateFPS?: number;
          crossorigin?: boolean;
        },
      );

      /**
       * When set to true will automatically play videos used by this texture once
       * they are loaded. If false, it will not modify the playing state.
       *
       * @member {boolean} PIXI.resources.VideoResource#autoPlay
       * @default true
       */
      autoPlay: boolean;

      /**
       * Trigger updating of the texture
       *
       * @param {number} [deltaTime=0] - time delta since last tick
       */
      update(deltaTime?: number): void;

      /**
       * Start preloading the video resource.
       *
       * @protected
       * @return {Promise<void>} Handle the validate event
       */
      protected load(): Promise<void>;

      /**
       * Should the base texture automatically update itself, set to true by default
       *
       * @member {boolean}
       */
      autoUpdate: boolean;
      /**
       * How many times a second to update the texture from the video. Leave at 0 to update at every render.
       * A lower fps can help performance, as updating the texture at 60fps on a 30ps video may not be efficient.
       *
       * @member {number}
       */
      updateFPS: number;

      /**
       * Used to auto-detect the type of resource.
       *
       * @static
       * @param {*} source - The source object
       * @param {string} extension - The extension of source, if set
       * @return {boolean} `true` if video source
       */
      static test(source: any, extension: string): boolean;

      /**
       * List of common video file extensions supported by VideoResource.
       * @constant
       * @member {Array<string>}
       * @static
       * @readonly
       */
      static readonly TYPES: string[];
      /**
       * The source element
       * @member {HTMLImageElement|HTMLCanvasElement|HTMLVideoElement|SVGElement} PIXI.resources.BaseImageResource#source
       * @readonly
       */
      readonly source: HTMLImageElement | HTMLCanvasElement | HTMLVideoElement | SVGElement;

      /**
       * Upload the texture to the GPU.
       * @param {PIXI.Renderer} renderer Upload to the renderer
       * @param {PIXI.BaseTexture} baseTexture Reference to parent texture
       * @param {PIXI.GLTexture} glTexture
       * @param {HTMLImageElement|HTMLCanvasElement|HTMLVideoElement|SVGElement} [source] (optional)
       * @returns {boolean} true is success
       */
      upload(
        renderer: PIXI.Renderer,
        baseTexture: PIXI.BaseTexture,
        glTexture: PIXI.GLTexture,
        source?: HTMLImageElement | HTMLCanvasElement | HTMLVideoElement | SVGElement,
      ): boolean;

      /**
       * Clean up anything, this happens when destroying is ready.
       *
       * @protected
       */
      protected dispose(): void;

      /**
       * Internal width of the resource
       * @member {number} PIXI.resources.Resource#_width
       * @protected
       */
      protected _width: number;
      /**
       * Internal height of the resource
       * @member {number} PIXI.resources.Resource#_height
       * @protected
       */
      protected _height: number;
      /**
       * If resource has been destroyed
       * @member {boolean} PIXI.resources.Resource#destroyed
       * @readonly
       * @default false
       */
      readonly destroyed: boolean;
      /**
       * `true` if resource is created by BaseTexture
       * useful for doing cleanup with BaseTexture destroy
       * and not cleaning up resources that were created
       * externally.
       * @member {boolean} PIXI.resources.Resource#internal
       * @protected
       */
      protected internal: boolean;

      /**
       * Bind to a parent BaseTexture
       *
       * @param {PIXI.BaseTexture} baseTexture - Parent texture
       */
      bind(baseTexture: PIXI.BaseTexture): void;

      /**
       * Unbind to a parent BaseTexture
       *
       * @param {PIXI.BaseTexture} baseTexture - Parent texture
       */
      unbind(baseTexture: PIXI.BaseTexture): void;

      /**
       * Trigger a resize event
       * @param {number} width X dimension
       * @param {number} height Y dimension
       */
      resize(width: number, height: number): void;

      /**
       * Has been validated
       * @readonly
       * @member {boolean}
       */
      readonly valid: boolean;
      /**
       * The width of the resource.
       *
       * @member {number}
       * @readonly
       */
      readonly width: number;
      /**
       * The height of the resource.
       *
       * @member {number}
       * @readonly
       */
      readonly height: number;

      /**
       * Set the style, optional to override
       *
       * @param {PIXI.Renderer} renderer - yeah, renderer!
       * @param {PIXI.BaseTexture} baseTexture - the texture
       * @param {PIXI.GLTexture} glTexture - texture instance for this webgl context
       * @returns {boolean} `true` is success
       */
      style(renderer: PIXI.Renderer, baseTexture: PIXI.BaseTexture, glTexture: PIXI.GLTexture): boolean;

      /**
       * Call when destroying resource, unbind any BaseTexture object
       * before calling this method, as reference counts are maintained
       * internally.
       */
      destroy(): void;
    }

    /**
     * Collection of installed resource types, class must extend {@link PIXI.resources.Resource}.
     * @example
     * class CustomResource extends PIXI.resources.Resource {
     *   // MUST have source, options constructor signature
     *   // for auto-detected resources to be created.
     *   constructor(source, options) {
     *     super();
     *   }
     *   upload(renderer, baseTexture, glTexture) {
     *     // upload with GL
     *     return true;
     *   }
     *   // used to auto-detect resource
     *   static test(source, extension) {
     *     return extension === 'xyz'|| source instanceof SomeClass;
     *   }
     * }
     * // Install the new resource type
     * PIXI.resources.INSTALLED.push(CustomResource);
     *
     * @name PIXI.resources.INSTALLED
     * @type {Array<*>}
     * @static
     * @readonly
     */
    var INSTALLED: any[];

    /**
     * Create a resource element from a single source element. This
     * auto-detects which type of resource to create. All resources that
     * are auto-detectable must have a static `test` method and a constructor
     * with the arguments `(source, options?)`. Currently, the supported
     * resources for auto-detection include:
     *  - {@link PIXI.resources.ImageResource}
     *  - {@link PIXI.resources.CanvasResource}
     *  - {@link PIXI.resources.VideoResource}
     *  - {@link PIXI.resources.SVGResource}
     *  - {@link PIXI.resources.BufferResource}
     * @static
     * @function PIXI.resources.autoDetectResource
     * @param {string|*} source - Resource source, this can be the URL to the resource,
     *        a typed-array (for BufferResource), HTMLVideoElement, SVG data-uri
     *        or any other resource that can be auto-detected. If not resource is
     *        detected, it's assumed to be an ImageResource.
     * @param {object} [options] - Pass-through options to use for Resource
     * @param {number} [options.width] - Width of BufferResource or SVG rasterization
     * @param {number} [options.height] - Height of BufferResource or SVG rasterization
     * @param {boolean} [options.autoLoad=true] - Image, SVG and Video flag to start loading
     * @param {number} [options.scale=1] - SVG source scale. Overridden by width, height
     * @param {boolean} [options.createBitmap=PIXI.settings.CREATE_IMAGE_BITMAP] - Image option to create Bitmap object
     * @param {boolean} [options.crossorigin=true] - Image and Video option to set crossOrigin
     * @param {boolean} [options.autoPlay=true] - Video option to start playing video immediately
     * @param {number} [options.updateFPS=0] - Video option to update how many times a second the
     *        texture should be updated from the video. Leave at 0 to update at every render
     * @return {PIXI.resources.Resource} The created resource.
     */
    function autoDetectResource(
      source: string | any,
      options?: {
        width?: number;
        height?: number;
        autoLoad?: boolean;
        scale?: number;
        createBitmap?: boolean;
        crossorigin?: boolean;
        autoPlay?: boolean;
        updateFPS?: number;
      },
    ): PIXI.resources.Resource;
  }

  /**
   * Helper class to create a quad
   *
   * @class
   * @memberof PIXI
   */
  class Quad {
    constructor();
  }

  /**
   * Helper class to create a quad with uvs like in v4
   *
   * @class
   * @memberof PIXI
   * @extends PIXI.Geometry
   */
  class QuadUv extends PIXI.Geometry {
    constructor();

    /**
     * An array of vertices
     *
     * @member {Float32Array} PIXI.QuadUv#vertices
     */
    vertices: Float32Array;
    /**
     * The Uvs of the quad
     *
     * @member {Float32Array} PIXI.QuadUv#uvs
     */
    uvs: Float32Array;

    /**
     * Maps two Rectangle to the quad.
     *
     * @param {PIXI.Rectangle} targetTextureFrame - the first rectangle
     * @param {PIXI.Rectangle} destinationFrame - the second rectangle
     * @return {PIXI.Quad} Returns itself.
     */
    map(targetTextureFrame: PIXI.Rectangle, destinationFrame: PIXI.Rectangle): PIXI.Quad;

    /**
     * legacy upload method, just marks buffers dirty
     * @returns {PIXI.QuadUv} Returns itself.
     */
    invalidate(): PIXI.QuadUv;

    /**
     * A map of renderer IDs to webgl VAOs
     *
     * @protected
     * @type {object}
     */
    protected glVertexArrayObjects: any;
    /**
     * Number of instances in this geometry, pass it to `GeometrySystem.draw()`
     * @member {number} PIXI.Geometry#instanceCount
     * @default 1
     */
    instanceCount: number;
    /**
     * Count of existing (not destroyed) meshes that reference this geometry
     * @member {number} PIXI.Geometry#refCount
     */
    refCount: number;

    /**
     *
     * Adds an attribute to the geometry
     *
     * @param {String} id - the name of the attribute (matching up to a shader)
     * @param {PIXI.Buffer|number[]} [buffer] the buffer that holds the data of the attribute . You can also provide an Array and a buffer will be created from it.
     * @param {Number} [size=0] the size of the attribute. If you have 2 floats per vertex (eg position x and y) this would be 2
     * @param {Boolean} [normalized=false] should the data be normalized.
     * @param {Number} [type=PIXI.TYPES.FLOAT] what type of number is the attribute. Check {PIXI.TYPES} to see the ones available
     * @param {Number} [stride=0] How far apart (in floats) the start of each value is. (used for interleaving data)
     * @param {Number} [start=0] How far into the array to start reading values (used for interleaving data)
     *
     * @return {PIXI.Geometry} returns self, useful for chaining.
     */
    addAttribute(
      id: string,
      buffer?: PIXI.Buffer | number[],
      size?: number,
      normalized?: boolean,
      type?: number,
      stride?: number,
      start?: number,
    ): PIXI.Geometry;

    /**
     * returns the requested attribute
     *
     * @param {String} id  the name of the attribute required
     * @return {PIXI.Attribute} the attribute requested.
     */
    getAttribute(id: string): PIXI.Attribute;

    /**
     * returns the requested buffer
     *
     * @param {String} id  the name of the buffer required
     * @return {PIXI.Buffer} the buffer requested.
     */
    getBuffer(id: string): PIXI.Buffer;

    /**
     *
     * Adds an index buffer to the geometry
     * The index buffer contains integers, three for each triangle in the geometry, which reference the various attribute buffers (position, colour, UV coordinates, other UV coordinates, normal, …). There is only ONE index buffer.
     *
     * @param {PIXI.Buffer|number[]} [buffer] the buffer that holds the data of the index buffer. You can also provide an Array and a buffer will be created from it.
     * @return {PIXI.Geometry} returns self, useful for chaining.
     */
    addIndex(buffer?: PIXI.Buffer | number[]): PIXI.Geometry;

    /**
     * returns the index buffer
     *
     * @return {PIXI.Buffer} the index buffer.
     */
    getIndex(): PIXI.Buffer;

    /**
     * this function modifies the structure so that all current attributes become interleaved into a single buffer
     * This can be useful if your model remains static as it offers a little performance boost
     *
     * @return {PIXI.Geometry} returns self, useful for chaining.
     */
    interleave(): PIXI.Geometry;

    /**
     * disposes WebGL resources that are connected to this geometry
     */
    dispose(): void;

    /**
     * Destroys the geometry.
     */
    destroy(): void;

    /**
     * returns a clone of the geometry
     *
     * @returns {PIXI.Geometry} a new clone of this geometry
     */
    clone(): PIXI.Geometry;
  }

  /**
   * 'Builder' pattern for bounds rectangles.
   *
   * This could be called an Axis-Aligned Bounding Box.
   * It is not an actual shape. It is a mutable thing; no 'EMPTY' or those kind of problems.
   *
   * @class
   * @memberof PIXI
   */
  class Bounds {
    constructor();

    /**
     * @member {number} PIXI.Bounds#minX
     * @default 0
     */
    minX: number;
    /**
     * @member {number} PIXI.Bounds#minY
     * @default 0
     */
    minY: number;
    /**
     * @member {number} PIXI.Bounds#maxX
     * @default 0
     */
    maxX: number;
    /**
     * @member {number} PIXI.Bounds#maxY
     * @default 0
     */
    maxY: number;

    /**
     * Checks if bounds are empty.
     *
     * @return {boolean} True if empty.
     */
    isEmpty(): boolean;

    /**
     * Clears the bounds and resets.
     *
     */
    clear(): void;

    /**
     * Can return Rectangle.EMPTY constant, either construct new rectangle, either use your rectangle
     * It is not guaranteed that it will return tempRect
     *
     * @param {PIXI.Rectangle} rect - temporary object will be used if AABB is not empty
     * @returns {PIXI.Rectangle} A rectangle of the bounds
     */
    getRectangle(rect: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * This function should be inlined when its possible.
     *
     * @param {PIXI.Point} point - The point to add.
     */
    addPoint(point: PIXI.Point): void;

    /**
     * Adds a quad, not transformed
     *
     * @param {Float32Array} vertices - The verts to add.
     */
    addQuad(vertices: Float32Array): void;

    /**
     * Adds sprite frame, transformed.
     *
     * @param {PIXI.Transform} transform - transform to apply
     * @param {number} x0 - left X of frame
     * @param {number} y0 - top Y of frame
     * @param {number} x1 - right X of frame
     * @param {number} y1 - bottom Y of frame
     */
    addFrame(transform: PIXI.Transform, x0: number, y0: number, x1: number, y1: number): void;

    /**
     * Adds sprite frame, multiplied by matrix
     *
     * @param {PIXI.Matrix} matrix - matrix to apply
     * @param {number} x0 - left X of frame
     * @param {number} y0 - top Y of frame
     * @param {number} x1 - right X of frame
     * @param {number} y1 - bottom Y of frame
     */
    addFrameMatrix(matrix: PIXI.Matrix, x0: number, y0: number, x1: number, y1: number): void;

    /**
     * Adds screen vertices from array
     *
     * @param {Float32Array} vertexData - calculated vertices
     * @param {number} beginOffset - begin offset
     * @param {number} endOffset - end offset, excluded
     */
    addVertexData(vertexData: Float32Array, beginOffset: number, endOffset: number): void;

    /**
     * Add an array of mesh vertices
     *
     * @param {PIXI.Transform} transform - mesh transform
     * @param {Float32Array} vertices - mesh coordinates in array
     * @param {number} beginOffset - begin offset
     * @param {number} endOffset - end offset, excluded
     */
    addVertices(transform: PIXI.Transform, vertices: Float32Array, beginOffset: number, endOffset: number): void;

    /**
     * Add an array of mesh vertices
     *
     * @param {PIXI.Matrix} matrix - mesh matrix
     * @param {Float32Array} vertices - mesh coordinates in array
     * @param {number} beginOffset - begin offset
     * @param {number} endOffset - end offset, excluded
     * @param {number} [padX] - x padding
     * @param {number} [padY] - y padding
     */
    addVerticesMatrix(
      matrix: PIXI.Matrix,
      vertices: Float32Array,
      beginOffset: number,
      endOffset: number,
      padX?: number,
      padY?: number,
    ): void;

    /**
     * Adds other Bounds
     *
     * @param {PIXI.Bounds} bounds - TODO
     */
    addBounds(bounds: PIXI.Bounds): void;

    /**
     * Adds other Bounds, masked with Bounds
     *
     * @param {PIXI.Bounds} bounds - TODO
     * @param {PIXI.Bounds} mask - TODO
     */
    addBoundsMask(bounds: PIXI.Bounds, mask: PIXI.Bounds): void;

    /**
     * Adds other Bounds, multiplied by matrix. Bounds shouldn't be empty
     *
     * @param {PIXI.Bounds} bounds other bounds
     * @param {PIXI.Matrix} matrix multiplicator
     */
    addBoundsMatrix(bounds: PIXI.Bounds, matrix: PIXI.Matrix): void;

    /**
     * Adds other Bounds, masked with Rectangle
     *
     * @param {PIXI.Bounds} bounds - TODO
     * @param {PIXI.Rectangle} area - TODO
     */
    addBoundsArea(bounds: PIXI.Bounds, area: PIXI.Rectangle): void;

    /**
     * Pads bounds object, making it grow in all directions.
     * If paddingY is omitted, both paddingX and paddingY will be set to paddingX.
     *
     * @param {number} [paddingX=0] - The horizontal padding amount.
     * @param {number} [paddingY=0] - The vertical padding amount.
     */
    pad(paddingX?: number, paddingY?: number): void;

    /**
     * Adds padded frame. (x0, y0) should be strictly less than (x1, y1)
     *
     * @param {number} x0 - left X of frame
     * @param {number} y0 - top Y of frame
     * @param {number} x1 - right X of frame
     * @param {number} y1 - bottom Y of frame
     * @param {number} padX - padding X
     * @param {number} padY - padding Y
     */
    addFramePad(x0: number, y0: number, x1: number, y1: number, padX: number, padY: number): void;
  }

  /**
   * A Container represents a collection of display objects.
   *
   * It is the base class of all display objects that act as a container for other objects (like Sprites).
   *
   *```js
   * let container = new PIXI.Container();
   * container.addChild(sprite);
   * ```
   *
   * @class
   * @extends PIXI.DisplayObject
   * @memberof PIXI
   */
  class Container extends PIXI.DisplayObject {
    constructor();

    /**
     * The array of children of this container.
     *
     * @member {PIXI.DisplayObject[]} PIXI.Container#children
     * @readonly
     */
    readonly children: PIXI.DisplayObject[];
    /**
     * If set to true, the container will sort its children by zIndex value
     * when updateTransform() is called, or manually if sortChildren() is called.
     *
     * This actually changes the order of elements in the array, so should be treated
     * as a basic solution that is not performant compared to other solutions,
     * such as @link https://github.com/pixijs/pixi-display
     *
     * Also be aware of that this may not work nicely with the addChildAt() function,
     * as the zIndex sorting may cause the child to automatically sorted to another position.
     *
     * @see PIXI.settings.SORTABLE_CHILDREN
     *
     * @member {boolean} PIXI.Container#sortableChildren
     */
    sortableChildren: boolean;
    /**
     * Should children be sorted by zIndex at the next updateTransform call.
     * Will get automatically set to true if a new child is added, or if a child's zIndex changes.
     *
     * @member {boolean} PIXI.Container#sortDirty
     */
    sortDirty: boolean;

    /**
     * Overridable method that can be used by Container subclasses whenever the children array is modified
     *
     * @protected
     */
    protected onChildrenChange(): void;

    /**
     * Adds one or more children to the container.
     *
     * Multiple items can be added like so: `myContainer.addChild(thingOne, thingTwo, thingThree)`
     *
     * @param {...PIXI.DisplayObject} child - The DisplayObject(s) to add to the container
     * @return {PIXI.DisplayObject} The first child that was added.
     */
    addChild<TChildren extends PIXI.DisplayObject[]>(...child: TChildren): TChildren[0];

    /**
     * Adds a child to the container at a specified index. If the index is out of bounds an error will be thrown
     *
     * @param {PIXI.DisplayObject} child - The child to add
     * @param {number} index - The index to place the child in
     * @return {PIXI.DisplayObject} The child that was added.
     */
    addChildAt<T extends PIXI.DisplayObject>(child: T, index: number): T;

    /**
     * Swaps the position of 2 Display Objects within this container.
     *
     * @param {PIXI.DisplayObject} child - First display object to swap
     * @param {PIXI.DisplayObject} child2 - Second display object to swap
     */
    swapChildren(child: PIXI.DisplayObject, child2: PIXI.DisplayObject): void;

    /**
     * Returns the index position of a child DisplayObject instance
     *
     * @param {PIXI.DisplayObject} child - The DisplayObject instance to identify
     * @return {number} The index position of the child display object to identify
     */
    getChildIndex(child: PIXI.DisplayObject): number;

    /**
     * Changes the position of an existing child in the display object container
     *
     * @param {PIXI.DisplayObject} child - The child DisplayObject instance for which you want to change the index number
     * @param {number} index - The resulting index number for the child display object
     */
    setChildIndex(child: PIXI.DisplayObject, index: number): void;

    /**
     * Returns the child at the specified index
     *
     * @param {number} index - The index to get the child at
     * @return {PIXI.DisplayObject} The child at the given index, if any.
     */
    getChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes one or more children from the container.
     *
     * @param {...PIXI.DisplayObject} child - The DisplayObject(s) to remove
     * @return {PIXI.DisplayObject} The first child that was removed.
     */
    removeChild<TChildren extends PIXI.DisplayObject[]>(...child: TChildren): TChildren[0];

    /**
     * Removes a child from the specified index position.
     *
     * @param {number} index - The index to get the child from
     * @return {PIXI.DisplayObject} The child that was removed.
     */
    removeChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes all children from this container that are within the begin and end indexes.
     *
     * @param {number} [beginIndex=0] - The beginning position.
     * @param {number} [endIndex=this.children.length] - The ending position. Default value is size of the container.
     * @returns {PIXI.DisplayObject[]} List of removed children
     */
    removeChildren(beginIndex?: number, endIndex?: number): PIXI.DisplayObject[];

    /**
     * Sorts children by zIndex. Previous order is mantained for 2 children with the same zIndex.
     */
    sortChildren(): void;

    /**
     * Updates the transform on all children of this container for rendering
     */
    updateTransform(): void;

    /**
     * Recalculates the bounds of the container.
     *
     */
    calculateBounds(): void;

    /**
     * Recalculates the bounds of the object. Override this to
     * calculate the bounds of the specific object (not including children).
     *
     * @protected
     */
    protected _calculateBounds(): void;

    /**
     * Renders the object using the WebGL renderer
     *
     * @param {PIXI.Renderer} renderer - The renderer
     */
    render(renderer: PIXI.Renderer): void;

    /**
     * Render the object using the WebGL renderer and advanced features.
     *
     * @protected
     * @param {PIXI.Renderer} renderer - The renderer
     */
    protected renderAdvanced(renderer: PIXI.Renderer): void;

    /**
     * To be overridden by the subclasses.
     *
     * @protected
     * @param {PIXI.Renderer} renderer - The renderer
     */
    protected _render(renderer: PIXI.Renderer): void;

    /**
     * Removes all internal references and listeners as well as removes children from the display list.
     * Do not use a Container after calling `destroy`.
     *
     * @param {object|boolean} [options] - Options parameter. A boolean will act as if all options
     *  have been set to that value
     * @param {boolean} [options.children=false] - if set to true, all the children will have their destroy
     *  method called as well. 'options' will be passed on to those calls.
     * @param {boolean} [options.texture=false] - Only used for child Sprites if options.children is set to true
     *  Should it destroy the texture of the child sprite
     * @param {boolean} [options.baseTexture=false] - Only used for child Sprites if options.children is set to true
     *  Should it destroy the base texture of the child sprite
     */
    destroy(options?: { children?: boolean; texture?: boolean; baseTexture?: boolean }): void;

    /**
     * The width of the Container, setting this will actually modify the scale to achieve the value set
     *
     * @member {number}
     */
    width: number;
    /**
     * The height of the Container, setting this will actually modify the scale to achieve the value set
     *
     * @member {number}
     */
    height: number;
    /**
     * Determines if the children to the displayObject can be clicked/touched
     * Setting this to false allows PixiJS to bypass a recursive `hitTest` function
     *
     * @member {boolean}
     * @memberof PIXI.Container#
     */
    interactiveChildren: boolean;

    /**
     * Returns the display object in the container.
     *
     * @method getChildByName
     * @memberof PIXI.Container#
     * @param {string} name - Instance name.
     * @return {PIXI.DisplayObject} The child with the specified name.
     */
    getChildByName(name: string): PIXI.DisplayObject;

    /**
     *  Flag for if the object is accessible. If true AccessibilityManager will overlay a
     *   shadow div with attributes set
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    accessible: boolean;
    /**
     * Sets the title attribute of the shadow div
     * If accessibleTitle AND accessibleHint has not been this will default to 'displayObject [tabIndex]'
     *
     * @member {?string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleTitle: string;
    /**
     * Sets the aria-label attribute of the shadow div
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleHint: string;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleActive: boolean;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleDiv: boolean;
    /**
     * Specify the type of div the accessible layer is. Screen readers treat the element differently
     * depending on this type. Defaults to button.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'button'
     */
    accessibleType: string;
    /**
     * Specify the pointer-events the accessible div will use
     * Defaults to auto.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'auto'
     */
    accessiblePointerEvents: string;
    /**
     * Setting to false will prevent any children inside this container to
     * be accessible. Defaults to true.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @default true
     */
    accessibleChildren: boolean;
    /**
     * World transform and local transform of this object.
     * This will become read-only later, please do not assign anything there unless you know what are you doing.
     *
     * @member {PIXI.Transform} PIXI.DisplayObject#transform
     */
    transform: PIXI.Transform;
    /**
     * The opacity of the object.
     *
     * @member {number} PIXI.DisplayObject#alpha
     */
    alpha: number;
    /**
     * The visibility of the object. If false the object will not be drawn, and
     * the updateTransform function will not be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds or call updateTransform manually.
     *
     * @member {boolean} PIXI.DisplayObject#visible
     */
    visible: boolean;
    /**
     * Can this object be rendered, if false the object will not be drawn but the updateTransform
     * methods will still be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds manually.
     *
     * @member {boolean} PIXI.DisplayObject#renderable
     */
    renderable: boolean;
    /**
     * The display object container that contains this display object.
     *
     * @member {PIXI.Container} PIXI.DisplayObject#parent
     * @readonly
     */
    readonly parent: PIXI.Container;
    /**
     * The multiplied alpha of the displayObject.
     *
     * @member {number} PIXI.DisplayObject#worldAlpha
     * @readonly
     */
    readonly worldAlpha: number;
    /**
     * Which index in the children array the display component was before the previous zIndex sort.
     * Used by containers to help sort objects with the same zIndex, by using previous array index as the decider.
     *
     * @member {number} PIXI.DisplayObject#_lastSortedIndex
     * @protected
     */
    protected _lastSortedIndex: number;
    /**
     * The zIndex of the displayObject.
     * A higher value will mean it will be rendered on top of other displayObjects within the same container.
     *
     * @member {number} PIXI.DisplayObject#_zIndex
     * @protected
     */
    protected _zIndex: number;
    /**
     * The area the filter is applied to. This is used as more of an optimization
     * rather than figuring out the dimensions of the displayObject each frame you can set this rectangle.
     *
     * Also works as an interaction mask.
     *
     * @member {?PIXI.Rectangle} PIXI.DisplayObject#filterArea
     */
    filterArea: PIXI.Rectangle;
    /**
     * Sets the filters for the displayObject.
     * * IMPORTANT: This is a WebGL only feature and will be ignored by the canvas renderer.
     * To remove filters simply set this property to `'null'`.
     *
     * @member {?PIXI.Filter[]} PIXI.DisplayObject#filters
     */
    filters: PIXI.Filter[];
    /**
     * The bounds object, this is used to calculate and store the bounds of the displayObject.
     *
     * @member {PIXI.Bounds} PIXI.DisplayObject#_bounds
     * @protected
     */
    protected _bounds: PIXI.Bounds;
    /**
     * The original, cached mask of the object.
     *
     * @member {PIXI.Graphics|PIXI.Sprite|null} PIXI.DisplayObject#_mask
     * @protected
     */
    protected _mask: PIXI.Graphics | PIXI.Sprite | null;
    /**
     * If the object has been destroyed via destroy(). If true, it should not be used.
     *
     * @member {boolean} PIXI.DisplayObject#_destroyed
     * @protected
     */
    protected _destroyed: boolean;
    /**
     * used to fast check if a sprite is.. a sprite!
     * @member {boolean} PIXI.DisplayObject#isSprite
     */
    isSprite: boolean;
    /**
     * Does any other displayObject use this object as a mask?
     * @member {boolean} PIXI.DisplayObject#isMask
     */
    isMask: boolean;
    /**
     * @protected
     * @member {PIXI.DisplayObject}
     */
    protected _tempDisplayObjectParent: PIXI.DisplayObject;

    /**
     * Recursively updates transform of all objects from the root to this one
     * internal function for toLocal()
     */
    _recursivePostUpdateTransform(): void;

    /**
     * Retrieves the bounds of the displayObject as a rectangle object.
     *
     * @param {boolean} [skipUpdate] - Setting to `true` will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @param {PIXI.Rectangle} [rect] - Optional rectangle to store the result of the bounds calculation.
     * @return {PIXI.Rectangle} The rectangular bounding area.
     */
    getBounds(skipUpdate?: boolean, rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Retrieves the local bounds of the displayObject as a rectangle object.
     *
     * @param {PIXI.Rectangle} [rect] - Optional rectangle to store the result of the bounds calculation.
     * @return {PIXI.Rectangle} The rectangular bounding area.
     */
    getLocalBounds(rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Calculates the global position of the display object.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform.
     * @return {PIXI.IPoint} A point object representing the position of this object.
     */
    toGlobal(position: PIXI.IPoint, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Calculates the local position of the display object relative to another point.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.DisplayObject} [from] - The DisplayObject to calculate the global position from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform
     * @return {PIXI.IPoint} A point object representing the position of this object
     */
    toLocal(position: PIXI.IPoint, from?: PIXI.DisplayObject, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Set the parent Container of this DisplayObject.
     *
     * @param {PIXI.Container} container - The Container to add this DisplayObject to.
     * @return {PIXI.Container} The Container that this DisplayObject was added to.
     */
    setParent(container: PIXI.Container): PIXI.Container;

    /**
     * Convenience function to set the position, scale, skew and pivot at once.
     *
     * @param {number} [x=0] - The X position
     * @param {number} [y=0] - The Y position
     * @param {number} [scaleX=1] - The X scale value
     * @param {number} [scaleY=1] - The Y scale value
     * @param {number} [rotation=0] - The rotation
     * @param {number} [skewX=0] - The X skew value
     * @param {number} [skewY=0] - The Y skew value
     * @param {number} [pivotX=0] - The X pivot value
     * @param {number} [pivotY=0] - The Y pivot value
     * @return {PIXI.DisplayObject} The DisplayObject instance
     */
    setTransform(
      x?: number,
      y?: number,
      scaleX?: number,
      scaleY?: number,
      rotation?: number,
      skewX?: number,
      skewY?: number,
      pivotX?: number,
      pivotY?: number,
    ): PIXI.DisplayObject;

    /**
     * The position of the displayObject on the x axis relative to the local coordinates of the parent.
     * An alias to position.x
     *
     * @member {number}
     */
    x: number;
    /**
     * The position of the displayObject on the y axis relative to the local coordinates of the parent.
     * An alias to position.y
     *
     * @member {number}
     */
    y: number;
    /**
     * Current transform of the object based on world (parent) factors.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly worldTransform: PIXI.Matrix;
    /**
     * Current transform of the object based on local factors: position, scale, other stuff.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly localTransform: PIXI.Matrix;
    /**
     * The coordinate of the object relative to the local coordinates of the parent.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    position: PIXI.IPoint;
    /**
     * The scale factor of the object.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    scale: PIXI.IPoint;
    /**
     * The pivot point of the displayObject that it rotates around.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    pivot: PIXI.IPoint;
    /**
     * The skew factor for the object in radians.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.ObservablePoint}
     */
    skew: PIXI.ObservablePoint;
    /**
     * The rotation of the object in radians.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    rotation: number;
    /**
     * The angle of the object in degrees.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    angle: number;
    /**
     * The zIndex of the displayObject.
     * If a container has the sortableChildren property set to true, children will be automatically
     * sorted by zIndex value; a higher value will mean it will be moved towards the end of the array,
     * and thus rendered on top of other displayObjects within the same container.
     *
     * @member {number}
     */
    zIndex: number;
    /**
     * Indicates if the object is globally visible.
     *
     * @member {boolean}
     * @readonly
     */
    readonly worldVisible: boolean;
    /**
     * Sets a mask for the displayObject. A mask is an object that limits the visibility of an
     * object to the shape of the mask applied to it. In PixiJS a regular mask must be a
     * {@link PIXI.Graphics} or a {@link PIXI.Sprite} object. This allows for much faster masking in canvas as it
     * utilities shape clipping. To remove a mask, set this property to `null`.
     *
     * For sprite mask both alpha and red channel are used. Black mask is the same as transparent mask.
     * @example
     * const graphics = new PIXI.Graphics();
     * graphics.beginFill(0xFF3300);
     * graphics.drawRect(50, 250, 100, 100);
     * graphics.endFill();
     *
     * const sprite = new PIXI.Sprite(texture);
     * sprite.mask = graphics;
     * @todo At the moment, PIXI.CanvasRenderer doesn't support PIXI.Sprite as mask.
     *
     * @member {PIXI.Container|PIXI.MaskData}
     */
    mask: PIXI.Container | PIXI.MaskData;

    /**
     * DisplayObject default updateTransform, does not update children of container.
     * Will crash if there's no parent element.
     *
     * @memberof PIXI.DisplayObject#
     * @function displayObjectUpdateTransform
     */
    displayObjectUpdateTransform(): void;

    /**
     * Enable interaction events for the DisplayObject. Touch, pointer and mouse
     * events will not be emitted unless `interactive` is set to `true`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.on('tap', (event) => {
     *    //handle event
     * });
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    interactive: boolean;
    /**
     * Interaction shape. Children will be hit first, then this shape will be checked.
     * Setting this will cause this shape to be checked in hit tests rather than the displayObject's bounds.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.hitArea = new PIXI.Rectangle(0, 0, 100, 100);
     * @member {PIXI.IHitArea}
     * @memberof PIXI.DisplayObject#
     */
    hitArea: PIXI.IHitArea;
    /**
     * If enabled, the mouse cursor use the pointer behavior when hovered over the displayObject if it is interactive
     * Setting this changes the 'cursor' property to `'pointer'`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.buttonMode = true;
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    buttonMode: boolean;
    /**
     * This defines what cursor mode is used when the mouse cursor
     * is hovered over the displayObject.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.cursor = 'wait';
     * @see https://developer.mozilla.org/en/docs/Web/CSS/cursor
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    cursor: string;
    /**
     * Set this to true if you want this display object to be cached as a bitmap.
     * This basically takes a snap shot of the display object as it is at that moment. It can
     * provide a performance benefit for complex static displayObjects.
     * To remove simply set this property to `false`
     *
     * IMPORTANT GOTCHA - Make sure that all your textures are preloaded BEFORE setting this property to true
     * as it will take a snapshot of what is currently there. If the textures have not loaded then they will not appear.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    cacheAsBitmap: boolean;
    /**
     * The instance name of the object.
     *
     * @memberof PIXI.DisplayObject#
     * @member {string} name
     */
    name: string;

    /**
     * Returns the global position of the displayObject. Does not depend on object scale, rotation and pivot.
     *
     * @method getGlobalPosition
     * @memberof PIXI.DisplayObject#
     * @param {PIXI.Point} [point=new PIXI.Point()] - The point to write the global value to.
     * @param {boolean} [skipUpdate=false] - Setting to true will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @return {PIXI.Point} The updated point.
     */
    getGlobalPosition(point?: PIXI.Point, skipUpdate?: boolean): PIXI.Point;
  }

  /**
   * The base class for all objects that are rendered on the screen.
   *
   * This is an abstract class and should not be used on its own; rather it should be extended.
   *
   * @class
   * @extends PIXI.utils.EventEmitter
   * @memberof PIXI
   */
  class DisplayObject extends PIXI.utils.EventEmitter {
    constructor();

    /**
     *  Flag for if the object is accessible. If true AccessibilityManager will overlay a
     *   shadow div with attributes set
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    accessible: boolean;
    /**
     * Sets the title attribute of the shadow div
     * If accessibleTitle AND accessibleHint has not been this will default to 'displayObject [tabIndex]'
     *
     * @member {?string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleTitle: string;
    /**
     * Sets the aria-label attribute of the shadow div
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleHint: string;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleActive: boolean;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleDiv: boolean;
    /**
     * Specify the type of div the accessible layer is. Screen readers treat the element differently
     * depending on this type. Defaults to button.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'button'
     */
    accessibleType: string;
    /**
     * Specify the pointer-events the accessible div will use
     * Defaults to auto.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'auto'
     */
    accessiblePointerEvents: string;
    /**
     * Setting to false will prevent any children inside this container to
     * be accessible. Defaults to true.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @default true
     */
    accessibleChildren: boolean;

    /**
     * Mixes all enumerable properties and methods from a source object to DisplayObject.
     *
     * @param {object} source The source of properties and methods to mix in.
     */
    static mixin(source: any): void;

    /**
     * World transform and local transform of this object.
     * This will become read-only later, please do not assign anything there unless you know what are you doing.
     *
     * @member {PIXI.Transform} PIXI.DisplayObject#transform
     */
    transform: PIXI.Transform;
    /**
     * The opacity of the object.
     *
     * @member {number} PIXI.DisplayObject#alpha
     */
    alpha: number;
    /**
     * The visibility of the object. If false the object will not be drawn, and
     * the updateTransform function will not be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds or call updateTransform manually.
     *
     * @member {boolean} PIXI.DisplayObject#visible
     */
    visible: boolean;
    /**
     * Can this object be rendered, if false the object will not be drawn but the updateTransform
     * methods will still be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds manually.
     *
     * @member {boolean} PIXI.DisplayObject#renderable
     */
    renderable: boolean;
    /**
     * The display object container that contains this display object.
     *
     * @member {PIXI.Container} PIXI.DisplayObject#parent
     * @readonly
     */
    readonly parent: PIXI.Container;
    /**
     * The multiplied alpha of the displayObject.
     *
     * @member {number} PIXI.DisplayObject#worldAlpha
     * @readonly
     */
    readonly worldAlpha: number;
    /**
     * Which index in the children array the display component was before the previous zIndex sort.
     * Used by containers to help sort objects with the same zIndex, by using previous array index as the decider.
     *
     * @member {number} PIXI.DisplayObject#_lastSortedIndex
     * @protected
     */
    protected _lastSortedIndex: number;
    /**
     * The zIndex of the displayObject.
     * A higher value will mean it will be rendered on top of other displayObjects within the same container.
     *
     * @member {number} PIXI.DisplayObject#_zIndex
     * @protected
     */
    protected _zIndex: number;
    /**
     * The area the filter is applied to. This is used as more of an optimization
     * rather than figuring out the dimensions of the displayObject each frame you can set this rectangle.
     *
     * Also works as an interaction mask.
     *
     * @member {?PIXI.Rectangle} PIXI.DisplayObject#filterArea
     */
    filterArea: PIXI.Rectangle;
    /**
     * Sets the filters for the displayObject.
     * * IMPORTANT: This is a WebGL only feature and will be ignored by the canvas renderer.
     * To remove filters simply set this property to `'null'`.
     *
     * @member {?PIXI.Filter[]} PIXI.DisplayObject#filters
     */
    filters: PIXI.Filter[];
    /**
     * The bounds object, this is used to calculate and store the bounds of the displayObject.
     *
     * @member {PIXI.Bounds} PIXI.DisplayObject#_bounds
     * @protected
     */
    protected _bounds: PIXI.Bounds;
    /**
     * The original, cached mask of the object.
     *
     * @member {PIXI.Graphics|PIXI.Sprite|null} PIXI.DisplayObject#_mask
     * @protected
     */
    protected _mask: PIXI.Graphics | PIXI.Sprite | null;
    /**
     * If the object has been destroyed via destroy(). If true, it should not be used.
     *
     * @member {boolean} PIXI.DisplayObject#_destroyed
     * @protected
     */
    protected _destroyed: boolean;
    /**
     * used to fast check if a sprite is.. a sprite!
     * @member {boolean} PIXI.DisplayObject#isSprite
     */
    isSprite: boolean;
    /**
     * Does any other displayObject use this object as a mask?
     * @member {boolean} PIXI.DisplayObject#isMask
     */
    isMask: boolean;
    /**
     * @protected
     * @member {PIXI.DisplayObject}
     */
    protected _tempDisplayObjectParent: PIXI.DisplayObject;

    /**
     * Updates the object transform for rendering.
     *
     * TODO - Optimization pass!
     */
    updateTransform(): void;

    /**
     * Recalculates the bounds of the display object.
     *
     * Does nothing by default and can be overwritten in a parent class.
     */
    calculateBounds(): void;

    /**
     * Recursively updates transform of all objects from the root to this one
     * internal function for toLocal()
     */
    _recursivePostUpdateTransform(): void;

    /**
     * Retrieves the bounds of the displayObject as a rectangle object.
     *
     * @param {boolean} [skipUpdate] - Setting to `true` will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @param {PIXI.Rectangle} [rect] - Optional rectangle to store the result of the bounds calculation.
     * @return {PIXI.Rectangle} The rectangular bounding area.
     */
    getBounds(skipUpdate?: boolean, rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Retrieves the local bounds of the displayObject as a rectangle object.
     *
     * @param {PIXI.Rectangle} [rect] - Optional rectangle to store the result of the bounds calculation.
     * @return {PIXI.Rectangle} The rectangular bounding area.
     */
    getLocalBounds(rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Calculates the global position of the display object.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform.
     * @return {PIXI.IPoint} A point object representing the position of this object.
     */
    toGlobal(position: PIXI.IPoint, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Calculates the local position of the display object relative to another point.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.DisplayObject} [from] - The DisplayObject to calculate the global position from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform
     * @return {PIXI.IPoint} A point object representing the position of this object
     */
    toLocal(position: PIXI.IPoint, from?: PIXI.DisplayObject, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Renders the object using the WebGL renderer.
     *
     * @param {PIXI.Renderer} renderer - The renderer.
     */
    render(renderer: PIXI.Renderer): void;

    /**
     * Set the parent Container of this DisplayObject.
     *
     * @param {PIXI.Container} container - The Container to add this DisplayObject to.
     * @return {PIXI.Container} The Container that this DisplayObject was added to.
     */
    setParent(container: PIXI.Container): PIXI.Container;

    /**
     * Convenience function to set the position, scale, skew and pivot at once.
     *
     * @param {number} [x=0] - The X position
     * @param {number} [y=0] - The Y position
     * @param {number} [scaleX=1] - The X scale value
     * @param {number} [scaleY=1] - The Y scale value
     * @param {number} [rotation=0] - The rotation
     * @param {number} [skewX=0] - The X skew value
     * @param {number} [skewY=0] - The Y skew value
     * @param {number} [pivotX=0] - The X pivot value
     * @param {number} [pivotY=0] - The Y pivot value
     * @return {PIXI.DisplayObject} The DisplayObject instance
     */
    setTransform(
      x?: number,
      y?: number,
      scaleX?: number,
      scaleY?: number,
      rotation?: number,
      skewX?: number,
      skewY?: number,
      pivotX?: number,
      pivotY?: number,
    ): PIXI.DisplayObject;

    /**
     * Base destroy method for generic display objects. This will automatically
     * remove the display object from its parent Container as well as remove
     * all current event listeners and internal references. Do not use a DisplayObject
     * after calling `destroy()`.
     *
     */
    destroy(): void;

    /**
     * The position of the displayObject on the x axis relative to the local coordinates of the parent.
     * An alias to position.x
     *
     * @member {number}
     */
    x: number;
    /**
     * The position of the displayObject on the y axis relative to the local coordinates of the parent.
     * An alias to position.y
     *
     * @member {number}
     */
    y: number;
    /**
     * Current transform of the object based on world (parent) factors.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly worldTransform: PIXI.Matrix;
    /**
     * Current transform of the object based on local factors: position, scale, other stuff.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly localTransform: PIXI.Matrix;
    /**
     * The coordinate of the object relative to the local coordinates of the parent.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    position: PIXI.IPoint;
    /**
     * The scale factor of the object.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    scale: PIXI.IPoint;
    /**
     * The pivot point of the displayObject that it rotates around.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    pivot: PIXI.IPoint;
    /**
     * The skew factor for the object in radians.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.ObservablePoint}
     */
    skew: PIXI.ObservablePoint;
    /**
     * The rotation of the object in radians.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    rotation: number;
    /**
     * The angle of the object in degrees.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    angle: number;
    /**
     * The zIndex of the displayObject.
     * If a container has the sortableChildren property set to true, children will be automatically
     * sorted by zIndex value; a higher value will mean it will be moved towards the end of the array,
     * and thus rendered on top of other displayObjects within the same container.
     *
     * @member {number}
     */
    zIndex: number;
    /**
     * Indicates if the object is globally visible.
     *
     * @member {boolean}
     * @readonly
     */
    readonly worldVisible: boolean;
    /**
     * Sets a mask for the displayObject. A mask is an object that limits the visibility of an
     * object to the shape of the mask applied to it. In PixiJS a regular mask must be a
     * {@link PIXI.Graphics} or a {@link PIXI.Sprite} object. This allows for much faster masking in canvas as it
     * utilities shape clipping. To remove a mask, set this property to `null`.
     *
     * For sprite mask both alpha and red channel are used. Black mask is the same as transparent mask.
     * @example
     * const graphics = new PIXI.Graphics();
     * graphics.beginFill(0xFF3300);
     * graphics.drawRect(50, 250, 100, 100);
     * graphics.endFill();
     *
     * const sprite = new PIXI.Sprite(texture);
     * sprite.mask = graphics;
     * @todo At the moment, PIXI.CanvasRenderer doesn't support PIXI.Sprite as mask.
     *
     * @member {PIXI.Container|PIXI.MaskData}
     */
    mask: PIXI.Container | PIXI.MaskData;

    /**
     * DisplayObject default updateTransform, does not update children of container.
     * Will crash if there's no parent element.
     *
     * @memberof PIXI.DisplayObject#
     * @function displayObjectUpdateTransform
     */
    displayObjectUpdateTransform(): void;

    /**
     * Enable interaction events for the DisplayObject. Touch, pointer and mouse
     * events will not be emitted unless `interactive` is set to `true`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.on('tap', (event) => {
     *    //handle event
     * });
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    interactive: boolean;
    /**
     * Interaction shape. Children will be hit first, then this shape will be checked.
     * Setting this will cause this shape to be checked in hit tests rather than the displayObject's bounds.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.hitArea = new PIXI.Rectangle(0, 0, 100, 100);
     * @member {PIXI.IHitArea}
     * @memberof PIXI.DisplayObject#
     */
    hitArea: PIXI.IHitArea;
    /**
     * If enabled, the mouse cursor use the pointer behavior when hovered over the displayObject if it is interactive
     * Setting this changes the 'cursor' property to `'pointer'`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.buttonMode = true;
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    buttonMode: boolean;
    /**
     * This defines what cursor mode is used when the mouse cursor
     * is hovered over the displayObject.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.cursor = 'wait';
     * @see https://developer.mozilla.org/en/docs/Web/CSS/cursor
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    cursor: string;
    /**
     * Set this to true if you want this display object to be cached as a bitmap.
     * This basically takes a snap shot of the display object as it is at that moment. It can
     * provide a performance benefit for complex static displayObjects.
     * To remove simply set this property to `false`
     *
     * IMPORTANT GOTCHA - Make sure that all your textures are preloaded BEFORE setting this property to true
     * as it will take a snapshot of what is currently there. If the textures have not loaded then they will not appear.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    cacheAsBitmap: boolean;
    /**
     * The instance name of the object.
     *
     * @memberof PIXI.DisplayObject#
     * @member {string} name
     */
    name: string;

    /**
     * Returns the global position of the displayObject. Does not depend on object scale, rotation and pivot.
     *
     * @method getGlobalPosition
     * @memberof PIXI.DisplayObject#
     * @param {PIXI.Point} [point=new PIXI.Point()] - The point to write the global value to.
     * @param {boolean} [skipUpdate=false] - Setting to true will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @return {PIXI.Point} The updated point.
     */
    getGlobalPosition(point?: PIXI.Point, skipUpdate?: boolean): PIXI.Point;
  }

  /**
   * This class provides renderer-specific plugins for exporting content from a renderer.
   * For instance, these plugins can be used for saving an Image, Canvas element or for exporting the raw image data (pixels).
   *
   * Do not instantiate these plugins directly. It is available from the `renderer.plugins` property.
   * See {@link PIXI.CanvasRenderer#plugins} or {@link PIXI.Renderer#plugins}.
   * @example
   * // Create a new app (will auto-add extract plugin to renderer)
   * const app = new PIXI.Application();
   *
   * // Draw a red circle
   * const graphics = new PIXI.Graphics()
   *     .beginFill(0xFF0000)
   *     .drawCircle(0, 0, 50);
   *
   * // Render the graphics as an HTMLImageElement
   * const image = app.renderer.plugins.extract.image(graphics);
   * document.body.appendChild(image);
   * @class
   * @memberof PIXI
   */
  class Extract {
    constructor(renderer: PIXI.Renderer);

    /**
     * Will return a HTML Image of the target
     *
     * @param {PIXI.DisplayObject|PIXI.RenderTexture} target - A displayObject or renderTexture
     *  to convert. If left empty will use the main renderer
     * @param {string} [format] - Image format, e.g. "image/jpeg" or "image/webp".
     * @param {number} [quality] - JPEG or Webp compression from 0 to 1. Default is 0.92.
     * @return {HTMLImageElement} HTML Image of the target
     */
    image(target: PIXI.DisplayObject | PIXI.RenderTexture, format?: string, quality?: number): HTMLImageElement;

    /**
     * Will return a a base64 encoded string of this target. It works by calling
     *  `Extract.getCanvas` and then running toDataURL on that.
     *
     * @param {PIXI.DisplayObject|PIXI.RenderTexture} target - A displayObject or renderTexture
     *  to convert. If left empty will use the main renderer
     * @param {string} [format] - Image format, e.g. "image/jpeg" or "image/webp".
     * @param {number} [quality] - JPEG or Webp compression from 0 to 1. Default is 0.92.
     * @return {string} A base64 encoded string of the texture.
     */
    base64(target: PIXI.DisplayObject | PIXI.RenderTexture, format?: string, quality?: number): string;

    /**
     * Creates a Canvas element, renders this target to it and then returns it.
     *
     * @param {PIXI.DisplayObject|PIXI.RenderTexture} target - A displayObject or renderTexture
     *  to convert. If left empty will use the main renderer
     * @return {HTMLCanvasElement} A Canvas element with the texture rendered on.
     */
    canvas(target: PIXI.DisplayObject | PIXI.RenderTexture): HTMLCanvasElement;

    /**
     * Will return a one-dimensional array containing the pixel data of the entire texture in RGBA
     * order, with integer values between 0 and 255 (included).
     *
     * @param {PIXI.DisplayObject|PIXI.RenderTexture} target - A displayObject or renderTexture
     *  to convert. If left empty will use the main renderer
     * @return {Uint8Array} One-dimensional array containing the pixel data of the entire texture
     */
    pixels(target: PIXI.DisplayObject | PIXI.RenderTexture): Uint8Array;

    /**
     * Destroys the extract
     *
     */
    destroy(): void;
  }

  /**
   * The Graphics class contains methods used to draw primitive shapes such as lines, circles and
   * rectangles to the display, and to color and fill them.
   *
   * Note that because Graphics can share a GraphicsGeometry with other instances,
   * it is necessary to call `destroy()` to properly dereference the underlying
   * GraphicsGeometry and avoid a memory leak. Alternatively, keep using the same
   * Graphics instance and call `clear()` between redraws.
   *
   * @class
   * @extends PIXI.Container
   * @memberof PIXI
   */
  class Graphics extends PIXI.Container {
    constructor(geometry?: PIXI.GraphicsGeometry);

    /**
     * Includes vertex positions, face indices, normals, colors, UVs, and
     * custom attributes within buffers, reducing the cost of passing all
     * this data to the GPU. Can be shared between multiple Mesh or Graphics objects.
     * @member {PIXI.GraphicsGeometry} PIXI.Graphics#geometry
     * @readonly
     */
    readonly geometry: PIXI.GraphicsGeometry;
    /**
     * Represents the vertex and fragment shaders that processes the geometry and runs on the GPU.
     * Can be shared between multiple Graphics objects.
     * @member {PIXI.Shader} PIXI.Graphics#shader
     */
    shader: PIXI.Shader;
    /**
     * Represents the WebGL state the Graphics required to render, excludes shader and geometry. E.g.,
     * blend mode, culling, depth testing, direction of rendering triangles, backface, etc.
     * @member {PIXI.State} PIXI.Graphics#state
     */
    state: PIXI.State;
    /**
     * Current fill style
     *
     * @member {PIXI.FillStyle} PIXI.Graphics#_fillStyle
     * @protected
     */
    protected _fillStyle: PIXI.FillStyle;
    /**
     * Current line style
     *
     * @member {PIXI.LineStyle} PIXI.Graphics#_lineStyle
     * @protected
     */
    protected _lineStyle: PIXI.LineStyle;
    /**
     * Current shape transform matrix.
     *
     * @member {PIXI.Matrix} PIXI.Graphics#_matrix
     * @protected
     */
    protected _matrix: PIXI.Matrix;
    /**
     * Current hole mode is enabled.
     *
     * @member {boolean} PIXI.Graphics#_holeMode
     * @default false
     * @protected
     */
    protected _holeMode: boolean;
    /**
     * Current path
     *
     * @member {PIXI.Polygon} PIXI.Graphics#currentPath
     * @protected
     */
    protected currentPath: PIXI.Polygon;
    /**
     * When cacheAsBitmap is set to true the graphics object will be rendered as if it was a sprite.
     * This is useful if your graphics element does not change often, as it will speed up the rendering
     * of the object in exchange for taking up texture memory. It is also useful if you need the graphics
     * object to be anti-aliased, because it will be rendered using canvas. This is not recommended if
     * you are constantly redrawing the graphics element.
     *
     * @name cacheAsBitmap
     * @member {boolean}
     * @memberof PIXI.Graphics#
     * @default false
     */
    cacheAsBitmap: boolean;
    /**
     * A collections of batches! These can be drawn by the renderer batch system.
     *
     * @protected
     * @member {object[]} PIXI.Graphics#batches
     */
    protected batches: any[];
    /**
     * Update dirty for limiting calculating tints for batches.
     *
     * @protected
     * @member {number} PIXI.Graphics#batchTint
     * @default -1
     */
    protected batchTint: number;
    /**
     * Copy of the object vertex data.
     *
     * @protected
     * @member {Float32Array} PIXI.Graphics#vertexData
     */
    protected vertexData: Float32Array;
    /**
     * Renderer plugin for batching
     *
     * @member {string} PIXI.Graphics#pluginName
     * @default 'batch'
     */
    pluginName: string;

    /**
     * Creates a new Graphics object with the same values as this one.
     * Note that the only the properties of the object are cloned, not its transform (position,scale,etc)
     *
     * @return {PIXI.Graphics} A clone of the graphics object
     */
    clone(): PIXI.Graphics;

    /**
     * The blend mode to be applied to the graphic shape. Apply a value of
     * `PIXI.BLEND_MODES.NORMAL` to reset the blend mode.
     *
     * @member {number}
     * @default PIXI.BLEND_MODES.NORMAL;
     * @see PIXI.BLEND_MODES
     */
    blendMode: number;
    /**
     * The tint applied to the graphic shape. This is a hex value. A value of
     * 0xFFFFFF will remove any tint effect.
     *
     * @member {number}
     * @default 0xFFFFFF
     */
    tint: number;
    /**
     * The current fill style.
     *
     * @member {PIXI.FillStyle}
     * @readonly
     */
    readonly fill: PIXI.FillStyle;
    /**
     * The current line style.
     *
     * @member {PIXI.LineStyle}
     * @readonly
     */
    readonly line: PIXI.LineStyle;

    /**
     * Specifies the line style used for subsequent calls to Graphics methods such as the lineTo()
     * method or the drawCircle() method.
     *
     * @method PIXI.Graphics#lineStyle
     * @param {number} [width=0] - width of the line to draw, will update the objects stored style
     * @param {number} [color=0x0] - color of the line to draw, will update the objects stored style
     * @param {number} [alpha=1] - alpha of the line to draw, will update the objects stored style
     * @param {number} [alignment=0.5] - alignment of the line to draw, (0 = inner, 0.5 = middle, 1 = outter)
     * @param {boolean} [native=false] - If true the lines will be draw using LINES instead of TRIANGLE_STRIP
     * @return {PIXI.Graphics} This Graphics object. Good for chaining method calls
     */
    lineStyle(width?: number, color?: number, alpha?: number, alignment?: number, native?: boolean): PIXI.Graphics;
    /**
     * Specifies the line style used for subsequent calls to Graphics methods such as the lineTo()
     * method or the drawCircle() method.
     *
     * @method PIXI.Graphics#lineStyle
     * @param {number} [width=0] - width of the line to draw, will update the objects stored style
     * @param {number} [color=0x0] - color of the line to draw, will update the objects stored style
     * @param {number} [alpha=1] - alpha of the line to draw, will update the objects stored style
     * @param {number} [alignment=0.5] - alignment of the line to draw, (0 = inner, 0.5 = middle, 1 = outter)
     * @param {boolean} [native=false] - If true the lines will be draw using LINES instead of TRIANGLE_STRIP
     * @return {PIXI.Graphics} This Graphics object. Good for chaining method calls
     */
    lineStyle(width?: number, color?: number, alpha?: number, alignment?: number, native?: boolean): PIXI.Graphics;

    /**
     * Like line style but support texture for line fill.
     *
     * @param {object} [options] - Collection of options for setting line style.
     * @param {number} [options.width=0] - width of the line to draw, will update the objects stored style
     * @param {PIXI.Texture} [options.texture=PIXI.Texture.WHITE] - Texture to use
     * @param {number} [options.color=0x0] - color of the line to draw, will update the objects stored style.
     *  Default 0xFFFFFF if texture present.
     * @param {number} [options.alpha=1] - alpha of the line to draw, will update the objects stored style
     * @param {PIXI.Matrix} [options.matrix=null] Texture matrix to transform texture
     * @param {number} [options.alignment=0.5] - alignment of the line to draw, (0 = inner, 0.5 = middle, 1 = outter)
     * @param {boolean} [options.native=false] - If true the lines will be draw using LINES instead of TRIANGLE_STRIP
     * @return {PIXI.Graphics} This Graphics object. Good for chaining method calls
     */
    lineTextureStyle(options?: {
      width?: number;
      texture?: PIXI.Texture;
      color?: number;
      alpha?: number;
      matrix?: PIXI.Matrix;
      alignment?: number;
      native?: boolean;
    }): PIXI.Graphics;

    /**
     * Start a polygon object internally
     * @protected
     */
    protected startPoly(): void;

    /**
     * Finish the polygon object.
     * @protected
     */
    protected finishPoly(): void;

    /**
     * Moves the current drawing position to x, y.
     *
     * @param {number} x - the X coordinate to move to
     * @param {number} y - the Y coordinate to move to
     * @return {PIXI.Graphics} This Graphics object. Good for chaining method calls
     */
    moveTo(x: number, y: number): PIXI.Graphics;

    /**
     * Draws a line using the current line style from the current drawing position to (x, y);
     * The current drawing position is then set to (x, y).
     *
     * @param {number} x - the X coordinate to draw to
     * @param {number} y - the Y coordinate to draw to
     * @return {PIXI.Graphics} This Graphics object. Good for chaining method calls
     */
    lineTo(x: number, y: number): PIXI.Graphics;

    /**
     * Initialize the curve
     *
     * @protected
     * @param {number} [x=0]
     * @param {number} [y=0]
     */
    protected _initCurve(x?: number, y?: number): void;

    /**
     * Calculate the points for a quadratic bezier curve and then draws it.
     * Based on: https://stackoverflow.com/questions/785097/how-do-i-implement-a-bezier-curve-in-c
     *
     * @param {number} cpX - Control point x
     * @param {number} cpY - Control point y
     * @param {number} toX - Destination point x
     * @param {number} toY - Destination point y
     * @return {PIXI.Graphics} This Graphics object. Good for chaining method calls
     */
    quadraticCurveTo(cpX: number, cpY: number, toX: number, toY: number): PIXI.Graphics;

    /**
     * Calculate the points for a bezier curve and then draws it.
     *
     * @param {number} cpX - Control point x
     * @param {number} cpY - Control point y
     * @param {number} cpX2 - Second Control point x
     * @param {number} cpY2 - Second Control point y
     * @param {number} toX - Destination point x
     * @param {number} toY - Destination point y
     * @return {PIXI.Graphics} This Graphics object. Good for chaining method calls
     */
    bezierCurveTo(cpX: number, cpY: number, cpX2: number, cpY2: number, toX: number, toY: number): PIXI.Graphics;

    /**
     * The arcTo() method creates an arc/curve between two tangents on the canvas.
     *
     * "borrowed" from https://code.google.com/p/fxcanvas/ - thanks google!
     *
     * @param {number} x1 - The x-coordinate of the first tangent point of the arc
     * @param {number} y1 - The y-coordinate of the first tangent point of the arc
     * @param {number} x2 - The x-coordinate of the end of the arc
     * @param {number} y2 - The y-coordinate of the end of the arc
     * @param {number} radius - The radius of the arc
     * @return {PIXI.Graphics} This Graphics object. Good for chaining method calls
     */
    arcTo(x1: number, y1: number, x2: number, y2: number, radius: number): PIXI.Graphics;

    /**
     * The arc method creates an arc/curve (used to create circles, or parts of circles).
     *
     * @param {number} cx - The x-coordinate of the center of the circle
     * @param {number} cy - The y-coordinate of the center of the circle
     * @param {number} radius - The radius of the circle
     * @param {number} startAngle - The starting angle, in radians (0 is at the 3 o'clock position
     *  of the arc's circle)
     * @param {number} endAngle - The ending angle, in radians
     * @param {boolean} [anticlockwise=false] - Specifies whether the drawing should be
     *  counter-clockwise or clockwise. False is default, and indicates clockwise, while true
     *  indicates counter-clockwise.
     * @return {PIXI.Graphics} This Graphics object. Good for chaining method calls
     */
    arc(
      cx: number,
      cy: number,
      radius: number,
      startAngle: number,
      endAngle: number,
      anticlockwise?: boolean,
    ): PIXI.Graphics;

    /**
     * Specifies a simple one-color fill that subsequent calls to other Graphics methods
     * (such as lineTo() or drawCircle()) use when drawing.
     *
     * @param {number} [color=0] - the color of the fill
     * @param {number} [alpha=1] - the alpha of the fill
     * @return {PIXI.Graphics} This Graphics object. Good for chaining method calls
     */
    beginFill(color?: number, alpha?: number): PIXI.Graphics;

    /**
     * Begin the texture fill
     *
     * @param {object} [options] - Object object.
     * @param {PIXI.Texture} [options.texture=PIXI.Texture.WHITE] - Texture to fill
     * @param {number} [options.color=0xffffff] - Background to fill behind texture
     * @param {number} [options.alpha=1] - Alpha of fill
     * @param {PIXI.Matrix} [options.matrix=null] - Transform matrix
     * @return {PIXI.Graphics} This Graphics object. Good for chaining method calls
     */
    beginTextureFill(options?: {
      texture?: PIXI.Texture;
      color?: number;
      alpha?: number;
      matrix?: PIXI.Matrix;
    }): PIXI.Graphics;

    /**
     * Applies a fill to the lines and shapes that were added since the last call to the beginFill() method.
     *
     * @return {PIXI.Graphics} This Graphics object. Good for chaining method calls
     */
    endFill(): PIXI.Graphics;

    /**
     * Draws a rectangle shape.
     *
     * @param {number} x - The X coord of the top-left of the rectangle
     * @param {number} y - The Y coord of the top-left of the rectangle
     * @param {number} width - The width of the rectangle
     * @param {number} height - The height of the rectangle
     * @return {PIXI.Graphics} This Graphics object. Good for chaining method calls
     */
    drawRect(x: number, y: number, width: number, height: number): PIXI.Graphics;

    /**
     * Draw a rectangle shape with rounded/beveled corners.
     *
     * @param {number} x - The X coord of the top-left of the rectangle
     * @param {number} y - The Y coord of the top-left of the rectangle
     * @param {number} width - The width of the rectangle
     * @param {number} height - The height of the rectangle
     * @param {number} radius - Radius of the rectangle corners
     * @return {PIXI.Graphics} This Graphics object. Good for chaining method calls
     */
    drawRoundedRect(x: number, y: number, width: number, height: number, radius: number): PIXI.Graphics;

    /**
     * Draws a circle.
     *
     * @param {number} x - The X coordinate of the center of the circle
     * @param {number} y - The Y coordinate of the center of the circle
     * @param {number} radius - The radius of the circle
     * @return {PIXI.Graphics} This Graphics object. Good for chaining method calls
     */
    drawCircle(x: number, y: number, radius: number): PIXI.Graphics;

    /**
     * Draws an ellipse.
     *
     * @param {number} x - The X coordinate of the center of the ellipse
     * @param {number} y - The Y coordinate of the center of the ellipse
     * @param {number} width - The half width of the ellipse
     * @param {number} height - The half height of the ellipse
     * @return {PIXI.Graphics} This Graphics object. Good for chaining method calls
     */
    drawEllipse(x: number, y: number, width: number, height: number): PIXI.Graphics;

    /**
     * Draws a polygon using the given path.
     *
     * @param {number[]|PIXI.Point[]|PIXI.Polygon} path - The path data used to construct the polygon.
     * @return {PIXI.Graphics} This Graphics object. Good for chaining method calls
     */
    drawPolygon(path: number[] | PIXI.Point[] | PIXI.Polygon): PIXI.Graphics;

    /**
     * Draw any shape.
     *
     * @param {PIXI.Circle|PIXI.Ellipse|PIXI.Polygon|PIXI.Rectangle|PIXI.RoundedRectangle} shape - Shape to draw
     * @return {PIXI.Graphics} This Graphics object. Good for chaining method calls
     */
    drawShape(shape: PIXI.Circle | PIXI.Ellipse | PIXI.Polygon | PIXI.Rectangle | PIXI.RoundedRectangle): PIXI.Graphics;

    /**
     * Draw a star shape with an arbitrary number of points.
     *
     * @param {number} x - Center X position of the star
     * @param {number} y - Center Y position of the star
     * @param {number} points - The number of points of the star, must be > 1
     * @param {number} radius - The outer radius of the star
     * @param {number} [innerRadius] - The inner radius between points, default half `radius`
     * @param {number} [rotation=0] - The rotation of the star in radians, where 0 is vertical
     * @return {PIXI.Graphics} This Graphics object. Good for chaining method calls
     */
    drawStar(
      x: number,
      y: number,
      points: number,
      radius: number,
      innerRadius?: number,
      rotation?: number,
    ): PIXI.Graphics;

    /**
     * Clears the graphics that were drawn to this Graphics object, and resets fill and line style settings.
     *
     * @return {PIXI.Graphics} This Graphics object. Good for chaining method calls
     */
    clear(): PIXI.Graphics;

    /**
     * True if graphics consists of one rectangle, and thus, can be drawn like a Sprite and
     * masked with gl.scissor.
     *
     * @returns {boolean} True if only 1 rect.
     */
    isFastRect(): boolean;

    /**
     * Renders the object using the WebGL renderer
     *
     * @protected
     * @param {PIXI.Renderer} renderer - The renderer
     */
    protected _render(renderer: PIXI.Renderer): void;

    /**
     * Populating batches for rendering
     *
     * @protected
     */
    protected _populateBatches(): void;

    /**
     * Renders the batches using the BathedRenderer plugin
     *
     * @protected
     * @param {PIXI.Renderer} renderer - The renderer
     */
    protected _renderBatched(renderer: PIXI.Renderer): void;

    /**
     * Renders the graphics direct
     *
     * @protected
     * @param {PIXI.Renderer} renderer - The renderer
     */
    protected _renderDirect(renderer: PIXI.Renderer): void;

    /**
     * Renders specific DrawCall
     *
     * @param {PIXI.Renderer} renderer
     * @param {PIXI.BatchDrawCall} drawCall
     */
    _renderDrawCallDirect(renderer: PIXI.Renderer, drawCall: PIXI.BatchDrawCall): void;

    /**
     * Resolves shader for direct rendering
     *
     * @protected
     * @param {PIXI.Renderer} renderer - The renderer
     */
    protected _resolveDirectShader(renderer: PIXI.Renderer): void;

    /**
     * Retrieves the bounds of the graphic shape as a rectangle object
     *
     * @protected
     */
    protected _calculateBounds(): void;

    /**
     * Tests if a point is inside this graphics object
     *
     * @param {PIXI.Point} point - the point to test
     * @return {boolean} the result of the test
     */
    containsPoint(point: PIXI.Point): boolean;

    /**
     * Recalcuate the tint by applying tin to batches using Graphics tint.
     * @protected
     */
    protected calculateTints(): void;

    /**
     * If there's a transform update or a change to the shape of the
     * geometry, recaculate the vertices.
     * @protected
     */
    protected calculateVertices(): void;

    /**
     * Closes the current path.
     *
     * @return {PIXI.Graphics} Returns itself.
     */
    closePath(): PIXI.Graphics;

    /**
     * Apply a matrix to the positional data.
     *
     * @param {PIXI.Matrix} matrix - Matrix to use for transform current shape.
     * @return {PIXI.Graphics} Returns itself.
     */
    setMatrix(matrix: PIXI.Matrix): PIXI.Graphics;

    /**
     * Begin adding holes to the last draw shape
     * IMPORTANT: holes must be fully inside a shape to work
     * Also weirdness ensues if holes overlap!
     * Ellipses, Circles, Rectangles and Rounded Rectangles cannot be holes or host for holes in CanvasRenderer,
     * please use `moveTo` `lineTo`, `quadraticCurveTo` if you rely on pixi-legacy bundle.
     * @return {PIXI.Graphics} Returns itself.
     */
    beginHole(): PIXI.Graphics;

    /**
     * End adding holes to the last draw shape
     * @return {PIXI.Graphics} Returns itself.
     */
    endHole(): PIXI.Graphics;

    /**
     * Destroys the Graphics object.
     *
     * @param {object|boolean} [options] - Options parameter. A boolean will act as if all
     *  options have been set to that value
     * @param {boolean} [options.children=false] - if set to true, all the children will have
     *  their destroy method called as well. 'options' will be passed on to those calls.
     * @param {boolean} [options.texture=false] - Only used for child Sprites if options.children is set to true
     *  Should it destroy the texture of the child sprite
     * @param {boolean} [options.baseTexture=false] - Only used for child Sprites if options.children is set to true
     *  Should it destroy the base texture of the child sprite
     */
    destroy(options?: { children?: boolean; texture?: boolean; baseTexture?: boolean }): void;

    /**
     * The array of children of this container.
     *
     * @member {PIXI.DisplayObject[]} PIXI.Container#children
     * @readonly
     */
    readonly children: PIXI.DisplayObject[];
    /**
     * If set to true, the container will sort its children by zIndex value
     * when updateTransform() is called, or manually if sortChildren() is called.
     *
     * This actually changes the order of elements in the array, so should be treated
     * as a basic solution that is not performant compared to other solutions,
     * such as @link https://github.com/pixijs/pixi-display
     *
     * Also be aware of that this may not work nicely with the addChildAt() function,
     * as the zIndex sorting may cause the child to automatically sorted to another position.
     *
     * @see PIXI.settings.SORTABLE_CHILDREN
     *
     * @member {boolean} PIXI.Container#sortableChildren
     */
    sortableChildren: boolean;
    /**
     * Should children be sorted by zIndex at the next updateTransform call.
     * Will get automatically set to true if a new child is added, or if a child's zIndex changes.
     *
     * @member {boolean} PIXI.Container#sortDirty
     */
    sortDirty: boolean;

    /**
     * Overridable method that can be used by Container subclasses whenever the children array is modified
     *
     * @protected
     */
    protected onChildrenChange(): void;

    /**
     * Adds one or more children to the container.
     *
     * Multiple items can be added like so: `myContainer.addChild(thingOne, thingTwo, thingThree)`
     *
     * @param {...PIXI.DisplayObject} child - The DisplayObject(s) to add to the container
     * @return {PIXI.DisplayObject} The first child that was added.
     */
    addChild<TChildren extends PIXI.DisplayObject[]>(...child: TChildren): TChildren[0];

    /**
     * Adds a child to the container at a specified index. If the index is out of bounds an error will be thrown
     *
     * @param {PIXI.DisplayObject} child - The child to add
     * @param {number} index - The index to place the child in
     * @return {PIXI.DisplayObject} The child that was added.
     */
    addChildAt<T extends PIXI.DisplayObject>(child: T, index: number): T;

    /**
     * Swaps the position of 2 Display Objects within this container.
     *
     * @param {PIXI.DisplayObject} child - First display object to swap
     * @param {PIXI.DisplayObject} child2 - Second display object to swap
     */
    swapChildren(child: PIXI.DisplayObject, child2: PIXI.DisplayObject): void;

    /**
     * Returns the index position of a child DisplayObject instance
     *
     * @param {PIXI.DisplayObject} child - The DisplayObject instance to identify
     * @return {number} The index position of the child display object to identify
     */
    getChildIndex(child: PIXI.DisplayObject): number;

    /**
     * Changes the position of an existing child in the display object container
     *
     * @param {PIXI.DisplayObject} child - The child DisplayObject instance for which you want to change the index number
     * @param {number} index - The resulting index number for the child display object
     */
    setChildIndex(child: PIXI.DisplayObject, index: number): void;

    /**
     * Returns the child at the specified index
     *
     * @param {number} index - The index to get the child at
     * @return {PIXI.DisplayObject} The child at the given index, if any.
     */
    getChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes one or more children from the container.
     *
     * @param {...PIXI.DisplayObject} child - The DisplayObject(s) to remove
     * @return {PIXI.DisplayObject} The first child that was removed.
     */
    removeChild<TChildren extends PIXI.DisplayObject[]>(...child: TChildren): TChildren[0];

    /**
     * Removes a child from the specified index position.
     *
     * @param {number} index - The index to get the child from
     * @return {PIXI.DisplayObject} The child that was removed.
     */
    removeChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes all children from this container that are within the begin and end indexes.
     *
     * @param {number} [beginIndex=0] - The beginning position.
     * @param {number} [endIndex=this.children.length] - The ending position. Default value is size of the container.
     * @returns {PIXI.DisplayObject[]} List of removed children
     */
    removeChildren(beginIndex?: number, endIndex?: number): PIXI.DisplayObject[];

    /**
     * Sorts children by zIndex. Previous order is mantained for 2 children with the same zIndex.
     */
    sortChildren(): void;

    /**
     * Updates the transform on all children of this container for rendering
     */
    updateTransform(): void;

    /**
     * Recalculates the bounds of the container.
     *
     */
    calculateBounds(): void;

    /**
     * Renders the object using the WebGL renderer
     *
     * @param {PIXI.Renderer} renderer - The renderer
     */
    render(renderer: PIXI.Renderer): void;

    /**
     * Render the object using the WebGL renderer and advanced features.
     *
     * @protected
     * @param {PIXI.Renderer} renderer - The renderer
     */
    protected renderAdvanced(renderer: PIXI.Renderer): void;

    /**
     * The width of the Container, setting this will actually modify the scale to achieve the value set
     *
     * @member {number}
     */
    width: number;
    /**
     * The height of the Container, setting this will actually modify the scale to achieve the value set
     *
     * @member {number}
     */
    height: number;
    /**
     * Determines if the children to the displayObject can be clicked/touched
     * Setting this to false allows PixiJS to bypass a recursive `hitTest` function
     *
     * @member {boolean}
     * @memberof PIXI.Container#
     */
    interactiveChildren: boolean;

    /**
     * Returns the display object in the container.
     *
     * @method getChildByName
     * @memberof PIXI.Container#
     * @param {string} name - Instance name.
     * @return {PIXI.DisplayObject} The child with the specified name.
     */
    getChildByName(name: string): PIXI.DisplayObject;

    /**
     *  Flag for if the object is accessible. If true AccessibilityManager will overlay a
     *   shadow div with attributes set
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    accessible: boolean;
    /**
     * Sets the title attribute of the shadow div
     * If accessibleTitle AND accessibleHint has not been this will default to 'displayObject [tabIndex]'
     *
     * @member {?string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleTitle: string;
    /**
     * Sets the aria-label attribute of the shadow div
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleHint: string;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleActive: boolean;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleDiv: boolean;
    /**
     * Specify the type of div the accessible layer is. Screen readers treat the element differently
     * depending on this type. Defaults to button.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'button'
     */
    accessibleType: string;
    /**
     * Specify the pointer-events the accessible div will use
     * Defaults to auto.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'auto'
     */
    accessiblePointerEvents: string;
    /**
     * Setting to false will prevent any children inside this container to
     * be accessible. Defaults to true.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @default true
     */
    accessibleChildren: boolean;
    /**
     * World transform and local transform of this object.
     * This will become read-only later, please do not assign anything there unless you know what are you doing.
     *
     * @member {PIXI.Transform} PIXI.DisplayObject#transform
     */
    transform: PIXI.Transform;
    /**
     * The opacity of the object.
     *
     * @member {number} PIXI.DisplayObject#alpha
     */
    alpha: number;
    /**
     * The visibility of the object. If false the object will not be drawn, and
     * the updateTransform function will not be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds or call updateTransform manually.
     *
     * @member {boolean} PIXI.DisplayObject#visible
     */
    visible: boolean;
    /**
     * Can this object be rendered, if false the object will not be drawn but the updateTransform
     * methods will still be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds manually.
     *
     * @member {boolean} PIXI.DisplayObject#renderable
     */
    renderable: boolean;
    /**
     * The display object container that contains this display object.
     *
     * @member {PIXI.Container} PIXI.DisplayObject#parent
     * @readonly
     */
    readonly parent: PIXI.Container;
    /**
     * The multiplied alpha of the displayObject.
     *
     * @member {number} PIXI.DisplayObject#worldAlpha
     * @readonly
     */
    readonly worldAlpha: number;
    /**
     * Which index in the children array the display component was before the previous zIndex sort.
     * Used by containers to help sort objects with the same zIndex, by using previous array index as the decider.
     *
     * @member {number} PIXI.DisplayObject#_lastSortedIndex
     * @protected
     */
    protected _lastSortedIndex: number;
    /**
     * The zIndex of the displayObject.
     * A higher value will mean it will be rendered on top of other displayObjects within the same container.
     *
     * @member {number} PIXI.DisplayObject#_zIndex
     * @protected
     */
    protected _zIndex: number;
    /**
     * The area the filter is applied to. This is used as more of an optimization
     * rather than figuring out the dimensions of the displayObject each frame you can set this rectangle.
     *
     * Also works as an interaction mask.
     *
     * @member {?PIXI.Rectangle} PIXI.DisplayObject#filterArea
     */
    filterArea: PIXI.Rectangle;
    /**
     * Sets the filters for the displayObject.
     * * IMPORTANT: This is a WebGL only feature and will be ignored by the canvas renderer.
     * To remove filters simply set this property to `'null'`.
     *
     * @member {?PIXI.Filter[]} PIXI.DisplayObject#filters
     */
    filters: PIXI.Filter[];
    /**
     * The bounds object, this is used to calculate and store the bounds of the displayObject.
     *
     * @member {PIXI.Bounds} PIXI.DisplayObject#_bounds
     * @protected
     */
    protected _bounds: PIXI.Bounds;
    /**
     * The original, cached mask of the object.
     *
     * @member {PIXI.Graphics|PIXI.Sprite|null} PIXI.DisplayObject#_mask
     * @protected
     */
    protected _mask: PIXI.Graphics | PIXI.Sprite | null;
    /**
     * If the object has been destroyed via destroy(). If true, it should not be used.
     *
     * @member {boolean} PIXI.DisplayObject#_destroyed
     * @protected
     */
    protected _destroyed: boolean;
    /**
     * used to fast check if a sprite is.. a sprite!
     * @member {boolean} PIXI.DisplayObject#isSprite
     */
    isSprite: boolean;
    /**
     * Does any other displayObject use this object as a mask?
     * @member {boolean} PIXI.DisplayObject#isMask
     */
    isMask: boolean;
    /**
     * @protected
     * @member {PIXI.DisplayObject}
     */
    protected _tempDisplayObjectParent: PIXI.DisplayObject;

    /**
     * Recursively updates transform of all objects from the root to this one
     * internal function for toLocal()
     */
    _recursivePostUpdateTransform(): void;

    /**
     * Retrieves the bounds of the displayObject as a rectangle object.
     *
     * @param {boolean} [skipUpdate] - Setting to `true` will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @param {PIXI.Rectangle} [rect] - Optional rectangle to store the result of the bounds calculation.
     * @return {PIXI.Rectangle} The rectangular bounding area.
     */
    getBounds(skipUpdate?: boolean, rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Retrieves the local bounds of the displayObject as a rectangle object.
     *
     * @param {PIXI.Rectangle} [rect] - Optional rectangle to store the result of the bounds calculation.
     * @return {PIXI.Rectangle} The rectangular bounding area.
     */
    getLocalBounds(rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Calculates the global position of the display object.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform.
     * @return {PIXI.IPoint} A point object representing the position of this object.
     */
    toGlobal(position: PIXI.IPoint, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Calculates the local position of the display object relative to another point.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.DisplayObject} [from] - The DisplayObject to calculate the global position from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform
     * @return {PIXI.IPoint} A point object representing the position of this object
     */
    toLocal(position: PIXI.IPoint, from?: PIXI.DisplayObject, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Set the parent Container of this DisplayObject.
     *
     * @param {PIXI.Container} container - The Container to add this DisplayObject to.
     * @return {PIXI.Container} The Container that this DisplayObject was added to.
     */
    setParent(container: PIXI.Container): PIXI.Container;

    /**
     * Convenience function to set the position, scale, skew and pivot at once.
     *
     * @param {number} [x=0] - The X position
     * @param {number} [y=0] - The Y position
     * @param {number} [scaleX=1] - The X scale value
     * @param {number} [scaleY=1] - The Y scale value
     * @param {number} [rotation=0] - The rotation
     * @param {number} [skewX=0] - The X skew value
     * @param {number} [skewY=0] - The Y skew value
     * @param {number} [pivotX=0] - The X pivot value
     * @param {number} [pivotY=0] - The Y pivot value
     * @return {PIXI.DisplayObject} The DisplayObject instance
     */
    setTransform(
      x?: number,
      y?: number,
      scaleX?: number,
      scaleY?: number,
      rotation?: number,
      skewX?: number,
      skewY?: number,
      pivotX?: number,
      pivotY?: number,
    ): PIXI.DisplayObject;

    /**
     * The position of the displayObject on the x axis relative to the local coordinates of the parent.
     * An alias to position.x
     *
     * @member {number}
     */
    x: number;
    /**
     * The position of the displayObject on the y axis relative to the local coordinates of the parent.
     * An alias to position.y
     *
     * @member {number}
     */
    y: number;
    /**
     * Current transform of the object based on world (parent) factors.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly worldTransform: PIXI.Matrix;
    /**
     * Current transform of the object based on local factors: position, scale, other stuff.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly localTransform: PIXI.Matrix;
    /**
     * The coordinate of the object relative to the local coordinates of the parent.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    position: PIXI.IPoint;
    /**
     * The scale factor of the object.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    scale: PIXI.IPoint;
    /**
     * The pivot point of the displayObject that it rotates around.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    pivot: PIXI.IPoint;
    /**
     * The skew factor for the object in radians.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.ObservablePoint}
     */
    skew: PIXI.ObservablePoint;
    /**
     * The rotation of the object in radians.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    rotation: number;
    /**
     * The angle of the object in degrees.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    angle: number;
    /**
     * The zIndex of the displayObject.
     * If a container has the sortableChildren property set to true, children will be automatically
     * sorted by zIndex value; a higher value will mean it will be moved towards the end of the array,
     * and thus rendered on top of other displayObjects within the same container.
     *
     * @member {number}
     */
    zIndex: number;
    /**
     * Indicates if the object is globally visible.
     *
     * @member {boolean}
     * @readonly
     */
    readonly worldVisible: boolean;
    /**
     * Sets a mask for the displayObject. A mask is an object that limits the visibility of an
     * object to the shape of the mask applied to it. In PixiJS a regular mask must be a
     * {@link PIXI.Graphics} or a {@link PIXI.Sprite} object. This allows for much faster masking in canvas as it
     * utilities shape clipping. To remove a mask, set this property to `null`.
     *
     * For sprite mask both alpha and red channel are used. Black mask is the same as transparent mask.
     * @example
     * const graphics = new PIXI.Graphics();
     * graphics.beginFill(0xFF3300);
     * graphics.drawRect(50, 250, 100, 100);
     * graphics.endFill();
     *
     * const sprite = new PIXI.Sprite(texture);
     * sprite.mask = graphics;
     * @todo At the moment, PIXI.CanvasRenderer doesn't support PIXI.Sprite as mask.
     *
     * @member {PIXI.Container|PIXI.MaskData}
     */
    mask: PIXI.Container | PIXI.MaskData;

    /**
     * DisplayObject default updateTransform, does not update children of container.
     * Will crash if there's no parent element.
     *
     * @memberof PIXI.DisplayObject#
     * @function displayObjectUpdateTransform
     */
    displayObjectUpdateTransform(): void;

    /**
     * Enable interaction events for the DisplayObject. Touch, pointer and mouse
     * events will not be emitted unless `interactive` is set to `true`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.on('tap', (event) => {
     *    //handle event
     * });
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    interactive: boolean;
    /**
     * Interaction shape. Children will be hit first, then this shape will be checked.
     * Setting this will cause this shape to be checked in hit tests rather than the displayObject's bounds.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.hitArea = new PIXI.Rectangle(0, 0, 100, 100);
     * @member {PIXI.IHitArea}
     * @memberof PIXI.DisplayObject#
     */
    hitArea: PIXI.IHitArea;
    /**
     * If enabled, the mouse cursor use the pointer behavior when hovered over the displayObject if it is interactive
     * Setting this changes the 'cursor' property to `'pointer'`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.buttonMode = true;
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    buttonMode: boolean;
    /**
     * This defines what cursor mode is used when the mouse cursor
     * is hovered over the displayObject.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.cursor = 'wait';
     * @see https://developer.mozilla.org/en/docs/Web/CSS/cursor
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    cursor: string;
    /**
     * The instance name of the object.
     *
     * @memberof PIXI.DisplayObject#
     * @member {string} name
     */
    name: string;

    /**
     * Returns the global position of the displayObject. Does not depend on object scale, rotation and pivot.
     *
     * @method getGlobalPosition
     * @memberof PIXI.DisplayObject#
     * @param {PIXI.Point} [point=new PIXI.Point()] - The point to write the global value to.
     * @param {boolean} [skipUpdate=false] - Setting to true will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @return {PIXI.Point} The updated point.
     */
    getGlobalPosition(point?: PIXI.Point, skipUpdate?: boolean): PIXI.Point;
  }

  /**
   * A class to contain data useful for Graphics objects
   *
   * @class
   * @memberof PIXI
   */
  class GraphicsData {
    constructor(
      shape: PIXI.Circle | PIXI.Rectangle | PIXI.Ellipse | PIXI.Polygon,
      fillStyle?: PIXI.FillStyle,
      lineStyle?: PIXI.LineStyle,
      matrix?: PIXI.Matrix,
    );

    /**
     * The shape object to draw.
     * @member {PIXI.Circle|PIXI.Ellipse|PIXI.Polygon|PIXI.Rectangle|PIXI.RoundedRectangle} PIXI.GraphicsData#shape
     */
    shape: PIXI.Circle | PIXI.Ellipse | PIXI.Polygon | PIXI.Rectangle | PIXI.RoundedRectangle;
    /**
     * The style of the line.
     * @member {PIXI.LineStyle} PIXI.GraphicsData#lineStyle
     */
    lineStyle: PIXI.LineStyle;
    /**
     * The style of the fill.
     * @member {PIXI.FillStyle} PIXI.GraphicsData#fillStyle
     */
    fillStyle: PIXI.FillStyle;
    /**
     * The transform matrix.
     * @member {PIXI.Matrix} PIXI.GraphicsData#matrix
     */
    matrix: PIXI.Matrix;
    /**
     * The type of the shape, see the Const.Shapes file for all the existing types,
     * @member {number} PIXI.GraphicsData#type
     */
    type: number;
    /**
     * The collection of points.
     * @member {number[]} PIXI.GraphicsData#points
     */
    points: number[];
    /**
     * The collection of holes.
     * @member {PIXI.GraphicsData[]} PIXI.GraphicsData#holes
     */
    holes: PIXI.GraphicsData[];

    /**
     * Creates a new GraphicsData object with the same values as this one.
     *
     * @return {PIXI.GraphicsData} Cloned GraphicsData object
     */
    clone(): PIXI.GraphicsData;

    /**
     * Destroys the Graphics data.
     */
    destroy(): void;
  }

  /**
   * The Graphics class contains methods used to draw primitive shapes such as lines, circles and
   * rectangles to the display, and to color and fill them.
   *
   * GraphicsGeometry is designed to not be continually updating the geometry since it's expensive
   * to re-tesselate using **earcut**. Consider using {@link PIXI.Mesh} for this use-case, it's much faster.
   *
   * @class
   * @extends PIXI.BatchGeometry
   * @memberof PIXI
   */
  class GraphicsGeometry extends PIXI.BatchGeometry {
    constructor();

    /**
     * An array of points to draw, 2 numbers per point
     *
     * @member {number[]} PIXI.GraphicsGeometry#points
     * @protected
     */
    protected points: number[];
    /**
     * The collection of colors
     *
     * @member {number[]} PIXI.GraphicsGeometry#colors
     * @protected
     */
    protected colors: number[];
    /**
     * The UVs collection
     *
     * @member {number[]} PIXI.GraphicsGeometry#uvs
     * @protected
     */
    protected uvs: number[];
    /**
     * The indices of the vertices
     *
     * @member {number[]} PIXI.GraphicsGeometry#indices
     * @protected
     */
    protected indices: number[];
    /**
     * Reference to the texture IDs.
     *
     * @member {number[]} PIXI.GraphicsGeometry#textureIds
     * @protected
     */
    protected textureIds: number[];
    /**
     * The collection of drawn shapes.
     *
     * @member {PIXI.GraphicsData[]} PIXI.GraphicsGeometry#graphicsData
     * @protected
     */
    protected graphicsData: PIXI.GraphicsData[];
    /**
     * Used to detect if the graphics object has changed.
     *
     * @member {number} PIXI.GraphicsGeometry#dirty
     * @protected
     */
    protected dirty: number;
    /**
     * Batches need to regenerated if the geometry is updated.
     *
     * @member {number} PIXI.GraphicsGeometry#batchDirty
     * @protected
     */
    protected batchDirty: number;
    /**
     * Used to check if the cache is dirty.
     *
     * @member {number} PIXI.GraphicsGeometry#cacheDirty
     * @protected
     */
    protected cacheDirty: number;
    /**
     * Used to detect if we cleared the graphicsData.
     *
     * @member {number} PIXI.GraphicsGeometry#clearDirty
     * @default 0
     * @protected
     */
    protected clearDirty: number;
    /**
     * List of current draw calls drived from the batches.
     *
     * @member {object[]} PIXI.GraphicsGeometry#drawCalls
     * @protected
     */
    protected drawCalls: any[];
    /**
     * Intermediate abstract format sent to batch system.
     * Can be converted to drawCalls or to batchable objects.
     *
     * @member {PIXI.graphicsUtils.BatchPart[]} PIXI.GraphicsGeometry#batches
     * @protected
     */
    protected batches: PIXI.graphicsUtils.BatchPart[];
    /**
     * Index of the last batched shape in the stack of calls.
     *
     * @member {number} PIXI.GraphicsGeometry#shapeIndex
     * @protected
     */
    protected shapeIndex: number;
    /**
     * Cached bounds.
     *
     * @member {PIXI.Bounds} PIXI.GraphicsGeometry#_bounds
     * @protected
     */
    protected _bounds: PIXI.Bounds;
    /**
     * The bounds dirty flag.
     *
     * @member {number} PIXI.GraphicsGeometry#boundsDirty
     * @protected
     */
    protected boundsDirty: number;
    /**
     * Padding to add to the bounds.
     *
     * @member {number} PIXI.GraphicsGeometry#boundsPadding
     * @default 0
     */
    boundsPadding: number;
    /**
     * Minimal distance between points that are considered different.
     * Affects line tesselation.
     *
     * @member {number} PIXI.GraphicsGeometry#closePointEps
     */
    closePointEps: number;
    /**
     * Get the current bounds of the graphic geometry.
     *
     * @member {PIXI.Bounds}
     * @readonly
     */
    readonly bounds: PIXI.Bounds;

    /**
     * Call if you changed graphicsData manually.
     * Empties all batch buffers.
     */
    invalidate(): void;

    /**
     * Clears the graphics that were drawn to this Graphics object, and resets fill and line style settings.
     *
     * @return {PIXI.GraphicsGeometry} This GraphicsGeometry object. Good for chaining method calls
     */
    clear(): PIXI.GraphicsGeometry;

    /**
     * Draws the given shape to this Graphics object. Can be any of Circle, Rectangle, Ellipse, Line or Polygon.
     *
     * @param {PIXI.Circle|PIXI.Ellipse|PIXI.Polygon|PIXI.Rectangle|PIXI.RoundedRectangle} shape - The shape object to draw.
     * @param {PIXI.FillStyle} fillStyle - Defines style of the fill.
     * @param {PIXI.LineStyle} lineStyle - Defines style of the lines.
     * @param {PIXI.Matrix} matrix - Transform applied to the points of the shape.
     * @return {PIXI.GraphicsGeometry} Returns geometry for chaining.
     */
    drawShape(
      shape: PIXI.Circle | PIXI.Ellipse | PIXI.Polygon | PIXI.Rectangle | PIXI.RoundedRectangle,
      fillStyle: PIXI.FillStyle,
      lineStyle: PIXI.LineStyle,
      matrix: PIXI.Matrix,
    ): PIXI.GraphicsGeometry;

    /**
     * Draws the given shape to this Graphics object. Can be any of Circle, Rectangle, Ellipse, Line or Polygon.
     *
     * @param {PIXI.Circle|PIXI.Ellipse|PIXI.Polygon|PIXI.Rectangle|PIXI.RoundedRectangle} shape - The shape object to draw.
     * @param {PIXI.Matrix} matrix - Transform applied to the points of the shape.
     * @return {PIXI.GraphicsGeometry} Returns geometry for chaining.
     */
    drawHole(
      shape: PIXI.Circle | PIXI.Ellipse | PIXI.Polygon | PIXI.Rectangle | PIXI.RoundedRectangle,
      matrix: PIXI.Matrix,
    ): PIXI.GraphicsGeometry;

    /**
     * Destroys the Graphics object.
     *
     * @param {object|boolean} [options] - Options parameter. A boolean will act as if all
     *  options have been set to that value
     * @param {boolean} [options.children=false] - if set to true, all the children will have
     *  their destroy method called as well. 'options' will be passed on to those calls.
     * @param {boolean} [options.texture=false] - Only used for child Sprites if options.children is set to true
     *  Should it destroy the texture of the child sprite
     * @param {boolean} [options.baseTexture=false] - Only used for child Sprites if options.children is set to true
     *  Should it destroy the base texture of the child sprite
     */
    destroy(options?: { children?: boolean; texture?: boolean; baseTexture?: boolean }): void;

    /**
     * Check to see if a point is contained within this geometry.
     *
     * @param {PIXI.Point} point - Point to check if it's contained.
     * @return {Boolean} `true` if the point is contained within geometry.
     */
    containsPoint(point: PIXI.Point): boolean;

    /**
     * Generates intermediate batch data. Either gets converted to drawCalls
     * or used to convert to batch objects directly by the Graphics object.
     */
    updateBatches(): void;

    /**
     * Affinity check
     *
     * @param {PIXI.FillStyle | PIXI.LineStyle} styleA
     * @param {PIXI.FillStyle | PIXI.LineStyle} styleB
     */
    _compareStyles(styleA: PIXI.FillStyle | PIXI.LineStyle, styleB: PIXI.FillStyle | PIXI.LineStyle): void;

    /**
     * Test geometry for batching process.
     *
     * @protected
     */
    protected validateBatching(): void;

    /**
     * Offset the indices so that it works with the batcher.
     *
     * @protected
     */
    protected packBatches(): void;

    /**
     * Checks to see if this graphics geometry can be batched.
     * Currently it needs to be small enough and not contain any native lines.
     *
     * @protected
     */
    protected isBatchable(): void;

    /**
     * Converts intermediate batches data to drawCalls.
     *
     * @protected
     */
    protected buildDrawCalls(): void;

    /**
     * Packs attributes to single buffer.
     *
     * @protected
     */
    protected packAttributes(): void;

    /**
     * Process fill part of Graphics.
     *
     * @param {PIXI.GraphicsData} data
     * @protected
     */
    protected processFill(data: PIXI.GraphicsData): void;

    /**
     * Process line part of Graphics.
     *
     * @param {PIXI.GraphicsData} data
     * @protected
     */
    protected processLine(data: PIXI.GraphicsData): void;

    /**
     * Process the holes data.
     *
     * @param {PIXI.GraphicsData[]} holes - Holes to render
     * @protected
     */
    protected processHoles(holes: PIXI.GraphicsData[]): void;

    /**
     * Update the local bounds of the object. Expensive to use performance-wise.
     *
     * @protected
     */
    protected calculateBounds(): void;

    /**
     * Transform points using matrix.
     *
     * @protected
     * @param {number[]} points - Points to transform
     * @param {PIXI.Matrix} matrix - Transform matrix
     */
    protected transformPoints(points: number[], matrix: PIXI.Matrix): void;

    /**
     * Add colors.
     *
     * @protected
     * @param {number[]} colors - List of colors to add to
     * @param {number} color - Color to add
     * @param {number} alpha - Alpha to use
     * @param {number} size - Number of colors to add
     */
    protected addColors(colors: number[], color: number, alpha: number, size: number): void;

    /**
     * Add texture id that the shader/fragment wants to use.
     *
     * @protected
     * @param {number[]} textureIds
     * @param {number} id
     * @param {number} size
     */
    protected addTextureIds(textureIds: number[], id: number, size: number): void;

    /**
     * Generates the UVs for a shape.
     *
     * @protected
     * @param {number[]} verts - Vertices
     * @param {number[]} uvs - UVs
     * @param {PIXI.Texture} texture - Reference to Texture
     * @param {number} start - Index buffer start index.
     * @param {number} size - The size/length for index buffer.
     * @param {PIXI.Matrix} [matrix] - Optional transform for all points.
     */
    protected addUvs(
      verts: number[],
      uvs: number[],
      texture: PIXI.Texture,
      start: number,
      size: number,
      matrix?: PIXI.Matrix,
    ): void;

    /**
     * Modify uvs array according to position of texture region
     * Does not work with rotated or trimmed textures
     *
     * @param {number[]} uvs array
     * @param {PIXI.Texture} texture region
     * @param {number} start starting index for uvs
     * @param {number} size how many points to adjust
     */
    adjustUvs(uvs: number[], texture: PIXI.Texture, start: number, size: number): void;

    /**
     * The maximum number of points to consider an object "batchable",
     * able to be batched by the renderer's batch system.
     *
     * @memberof PIXI.GraphicsGeometry
     * @static
     * @member {number} BATCHABLE_SIZE
     * @default 100
     */
    static BATCHABLE_SIZE: number;
    /**
     * Buffer used for position, color, texture IDs
     *
     * @member {PIXI.Buffer} PIXI.BatchGeometry#_buffer
     * @protected
     */
    protected _buffer: PIXI.Buffer;
    /**
     * Index buffer data
     *
     * @member {PIXI.Buffer} PIXI.BatchGeometry#_indexBuffer
     * @protected
     */
    protected _indexBuffer: PIXI.Buffer;
  }

  /**
   * Graphics curves resolution settings. If `adaptive` flag is set to `true`,
   * the resolution is calculated based on the curve's length to ensure better visual quality.
   * Adaptive draw works with `bezierCurveTo` and `quadraticCurveTo`.
   *
   * @static
   * @constant
   * @memberof PIXI
   * @name GRAPHICS_CURVES
   * @type {object}
   * @property {boolean} adaptive=false - flag indicating if the resolution should be adaptive
   * @property {number} maxLength=10 - maximal length of a single segment of the curve (if adaptive = false, ignored)
   * @property {number} minSegments=8 - minimal number of segments in the curve (if adaptive = false, ignored)
   * @property {number} maxSegments=2048 - maximal number of segments in the curve (if adaptive = false, ignored)
   */
  var GRAPHICS_CURVES: {
    adaptive: boolean;
    maxLength: number;
    minSegments: number;
    maxSegments: number;
  };

  /**
   * Fill style object for Graphics.
   *
   * @class
   * @memberof PIXI
   */
  class FillStyle {
    constructor();

    /**
     * Clones the object
     *
     * @return {PIXI.FillStyle}
     */
    clone(): PIXI.FillStyle;

    /**
     * Reset
     */
    reset(): void;

    /**
     * The hex color value used when coloring the Graphics object.
     *
     * @member {number} PIXI.FillStyle#color
     * @default 1
     */
    color: number;
    /**
     * The alpha value used when filling the Graphics object.
     *
     * @member {number} PIXI.FillStyle#alpha
     * @default 1
     */
    alpha: number;
    /**
     * The texture to be used for the fill.
     *
     * @member {string} PIXI.FillStyle#texture
     * @default 0
     */
    texture: string;
    /**
     * The transform aplpied to the texture.
     *
     * @member {string} PIXI.FillStyle#matrix
     * @default 0
     */
    matrix: string;
    /**
     * If the current fill is visible.
     *
     * @member {boolean} PIXI.FillStyle#visible
     * @default false
     */
    visible: boolean;

    /**
     * Destroy and don't use after this
     */
    destroy(): void;
  }

  /**
   * Represents the line style for Graphics.
   * @memberof PIXI
   * @class
   * @extends PIXI.FillStyle
   */
  class LineStyle extends PIXI.FillStyle {
    /**
     * Clones the object
     *
     * @return {PIXI.LineStyle}
     */
    clone(): PIXI.LineStyle;

    /**
     * Reset the line style to default.
     */
    reset(): void;

    /**
     * The width (thickness) of any lines drawn.
     *
     * @member {number} PIXI.LineStyle#width
     * @default 0
     */
    width: number;
    /**
     * The alignment of any lines drawn (0.5 = middle, 1 = outer, 0 = inner).
     *
     * @member {number} PIXI.LineStyle#alignment
     * @default 0.5
     */
    alignment: number;
    /**
     * If true the lines will be draw using LINES instead of TRIANGLE_STRIP
     *
     * @member {boolean} PIXI.LineStyle#native
     * @default false
     */
    native: boolean;
    /**
     * The hex color value used when coloring the Graphics object.
     *
     * @member {number} PIXI.FillStyle#color
     * @default 1
     */
    color: number;
    /**
     * The alpha value used when filling the Graphics object.
     *
     * @member {number} PIXI.FillStyle#alpha
     * @default 1
     */
    alpha: number;
    /**
     * The texture to be used for the fill.
     *
     * @member {string} PIXI.FillStyle#texture
     * @default 0
     */
    texture: string;
    /**
     * The transform aplpied to the texture.
     *
     * @member {string} PIXI.FillStyle#matrix
     * @default 0
     */
    matrix: string;
    /**
     * If the current fill is visible.
     *
     * @member {boolean} PIXI.FillStyle#visible
     * @default false
     */
    visible: boolean;

    /**
     * Destroy and don't use after this
     */
    destroy(): void;
  }

  /**
   * Draw a star shape with an arbitrary number of points.
   *
   * @class
   * @extends PIXI.Polygon
   * @memberof PIXI
   * @param {number} x - Center X position of the star
   * @param {number} y - Center Y position of the star
   * @param {number} points - The number of points of the star, must be > 1
   * @param {number} radius - The outer radius of the star
   * @param {number} [innerRadius] - The inner radius between points, default half `radius`
   * @param {number} [rotation=0] - The rotation of the star in radians, where 0 is vertical
   * @return {PIXI.Graphics} This Graphics object. Good for chaining method calls
   */
  class Star extends PIXI.Polygon {
    constructor(x: number, y: number, points: number, radius: number, innerRadius?: number, rotation?: number);

    /**
     * An array of the points of this polygon
     *
     * @member {number[]} PIXI.Polygon#points
     */
    points: number[];
    /**
     * The type of the object, mainly used to avoid `instanceof` checks
     *
     * @member {number} PIXI.Polygon#type
     * @readOnly
     * @default PIXI.SHAPES.POLY
     * @see PIXI.SHAPES
     */
    readonly type: number;
    /**
     * `false` after moveTo, `true` after `closePath`. In all other cases it is `true`.
     * @member {boolean} PIXI.Polygon#closeStroke
     * @default true
     */
    closeStroke: boolean;

    /**
     * Creates a clone of this polygon
     *
     * @return {PIXI.Polygon} a copy of the polygon
     */
    clone(): PIXI.Polygon;

    /**
     * Checks whether the x and y coordinates passed to this function are contained within this polygon
     *
     * @param {number} x - The X coordinate of the point to test
     * @param {number} y - The Y coordinate of the point to test
     * @return {boolean} Whether the x/y coordinates are within this polygon
     */
    contains(x: number, y: number): boolean;
  }

  /**
   * Generalized convenience utilities for Graphics.
   *
   * @namespace PIXI.graphicsUtils
   */
  namespace graphicsUtils {
    /**
     * A structure to hold interim batch objects for Graphics.
     * @class
     * @memberof PIXI.graphicsUtils
     */
    class BatchPart {
      constructor();

      /**
       * Begin batch part
       *
       * @param {PIXI.FillStyle | PIXI.LineStyle} style
       * @param {number} startIndex
       * @param {number} attribStart
       */
      begin(style: PIXI.FillStyle | PIXI.LineStyle, startIndex: number, attribStart: number): void;

      /**
       * End batch part
       *
       * @param {number} endIndex
       * @param {number} endAttrib
       */
      end(endIndex: number, endAttrib: number): void;
    }

    /**
     * Map of fill commands for each shape type.
     *
     * @memberof PIXI.graphicsUtils
     * @member {Object}
     */
    var FILL_COMMANDS: any;
    /**
     * Batch pool, stores unused batches for preventing allocations.
     *
     * @memberof PIXI.graphicsUtils
     * @type {Array<PIXI.graphicsUtils.BatchPart>}
     */
    var BATCH_POOL: PIXI.graphicsUtils.BatchPart[];
    /**
     * Draw call pool, stores unused draw calls for preventing allocations.
     *
     * @memberof PIXI.graphicsUtils
     * @type {Array<PIXI.BatchDrawCall>}
     */
    var DRAW_CALL_POOL: PIXI.BatchDrawCall[];
  }
  /**
   * This namespace contains a renderer plugin for handling mouse, pointer, and touch events.
   *
   * Do not instantiate this plugin directly. It is available from the `renderer.plugins` property.
   * See {@link PIXI.CanvasRenderer#plugins} or {@link PIXI.Renderer#plugins}.
   * @namespace PIXI.interaction
   */
  namespace interaction {
    /**
     * Holds all information related to an Interaction event
     *
     * @class
     * @memberof PIXI.interaction
     */
    class InteractionData {
      constructor();

      /**
       * This point stores the global coords of where the touch/mouse event happened
       *
       * @member {PIXI.Point} PIXI.interaction.InteractionData#global
       */
      global: PIXI.Point;
      /**
       * The target Sprite that was interacted with
       *
       * @member {PIXI.Sprite} PIXI.interaction.InteractionData#target
       */
      target: PIXI.Sprite;
      /**
       * When passed to an event handler, this will be the original DOM Event that was captured
       *
       * @see https://developer.mozilla.org/en-US/docs/Web/API/MouseEvent
       * @see https://developer.mozilla.org/en-US/docs/Web/API/TouchEvent
       * @see https://developer.mozilla.org/en-US/docs/Web/API/PointerEvent
       * @member {MouseEvent|TouchEvent|PointerEvent} PIXI.interaction.InteractionData#originalEvent
       */
      originalEvent: MouseEvent | TouchEvent | PointerEvent;
      /**
       * Unique identifier for this interaction
       *
       * @member {number} PIXI.interaction.InteractionData#identifier
       */
      identifier: number;
      /**
       * Indicates whether or not the pointer device that created the event is the primary pointer.
       * @see https://developer.mozilla.org/en-US/docs/Web/API/PointerEvent/isPrimary
       * @type {Boolean}
       */
      isPrimary: boolean;
      /**
       * Indicates which button was pressed on the mouse or pointer device to trigger the event.
       * @see https://developer.mozilla.org/en-US/docs/Web/API/MouseEvent/button
       * @type {number}
       */
      button: number;
      /**
       * Indicates which buttons are pressed on the mouse or pointer device when the event is triggered.
       * @see https://developer.mozilla.org/en-US/docs/Web/API/MouseEvent/buttons
       * @type {number}
       */
      buttons: number;
      /**
       * The width of the pointer's contact along the x-axis, measured in CSS pixels.
       * radiusX of TouchEvents will be represented by this value.
       * @see https://developer.mozilla.org/en-US/docs/Web/API/PointerEvent/width
       * @type {number}
       */
      width: number;
      /**
       * The height of the pointer's contact along the y-axis, measured in CSS pixels.
       * radiusY of TouchEvents will be represented by this value.
       * @see https://developer.mozilla.org/en-US/docs/Web/API/PointerEvent/height
       * @type {number}
       */
      height: number;
      /**
       * The angle, in degrees, between the pointer device and the screen.
       * @see https://developer.mozilla.org/en-US/docs/Web/API/PointerEvent/tiltX
       * @type {number}
       */
      tiltX: number;
      /**
       * The angle, in degrees, between the pointer device and the screen.
       * @see https://developer.mozilla.org/en-US/docs/Web/API/PointerEvent/tiltY
       * @type {number}
       */
      tiltY: number;
      /**
       * The type of pointer that triggered the event.
       * @see https://developer.mozilla.org/en-US/docs/Web/API/PointerEvent/pointerType
       * @type {string}
       */
      pointerType: string;
      /**
       * Pressure applied by the pointing device during the event. A Touch's force property
       * will be represented by this value.
       * @see https://developer.mozilla.org/en-US/docs/Web/API/PointerEvent/pressure
       * @type {number}
       */
      pressure: number;
      /**
       * From TouchEvents (not PointerEvents triggered by touches), the rotationAngle of the Touch.
       * @see https://developer.mozilla.org/en-US/docs/Web/API/Touch/rotationAngle
       * @type {number}
       */
      rotationAngle: number;
      /**
       * Twist of a stylus pointer.
       * @see https://w3c.github.io/pointerevents/#pointerevent-interface
       * @type {number}
       */
      twist: number;
      /**
       * Barrel pressure on a stylus pointer.
       * @see https://w3c.github.io/pointerevents/#pointerevent-interface
       * @type {number}
       */
      tangentialPressure: number;
      /**
       * The unique identifier of the pointer. It will be the same as `identifier`.
       * @readonly
       * @member {number}
       * @see https://developer.mozilla.org/en-US/docs/Web/API/PointerEvent/pointerId
       */
      readonly pointerId: number;

      /**
       * This will return the local coordinates of the specified displayObject for this InteractionData
       *
       * @param {PIXI.DisplayObject} displayObject - The DisplayObject that you would like the local
       *  coords off
       * @param {PIXI.Point} [point] - A Point object in which to store the value, optional (otherwise
       *  will create a new point)
       * @param {PIXI.Point} [globalPos] - A Point object containing your custom global coords, optional
       *  (otherwise will use the current global coords)
       * @return {PIXI.Point} A point containing the coordinates of the InteractionData position relative
       *  to the DisplayObject
       */
      getLocalPosition(displayObject: PIXI.DisplayObject, point?: PIXI.Point, globalPos?: PIXI.Point): PIXI.Point;

      /**
       * Copies properties from normalized event data.
       *
       * @param {Touch|MouseEvent|PointerEvent} event The normalized event data
       */
      copyEvent(event: Touch | MouseEvent | PointerEvent): void;

      /**
       * Resets the data for pooling.
       */
      reset(): void;
    }

    /**
     * Event class that mimics native DOM events.
     *
     * @class
     * @memberof PIXI.interaction
     */
    class InteractionEvent {
      constructor();

      /**
       * Whether this event will continue propagating in the tree.
       *
       * Remaining events for the {@link stopsPropagatingAt} object
       * will still be dispatched.
       *
       * @member {boolean} PIXI.interaction.InteractionEvent#stopped
       */
      stopped: boolean;
      /**
       * The object which caused this event to be dispatched.
       * For listener callback see {@link PIXI.interaction.InteractionEvent.currentTarget}.
       *
       * @member {PIXI.DisplayObject} PIXI.interaction.InteractionEvent#target
       */
      target: PIXI.DisplayObject;
      /**
       * The object whose event listener’s callback is currently being invoked.
       *
       * @member {PIXI.DisplayObject} PIXI.interaction.InteractionEvent#currentTarget
       */
      currentTarget: PIXI.DisplayObject;
      /**
       * Type of the event
       *
       * @member {string} PIXI.interaction.InteractionEvent#type
       */
      type: string;
      /**
       * InteractionData related to this event
       *
       * @member {PIXI.interaction.InteractionData} PIXI.interaction.InteractionEvent#data
       */
      data: PIXI.interaction.InteractionData;

      /**
       * Prevents event from reaching any objects other than the current object.
       *
       */
      stopPropagation(): void;

      /**
       * Resets the event.
       */
      reset(): void;
    }

    /**
     * The interaction manager deals with mouse, touch and pointer events.
     *
     * Any DisplayObject can be interactive if its `interactive` property is set to true.
     *
     * This manager also supports multitouch.
     *
     * An instance of this class is automatically created by default, and can be found at `renderer.plugins.interaction`
     *
     * @class
     * @extends PIXI.utils.EventEmitter
     * @memberof PIXI.interaction
     */
    class InteractionManager extends PIXI.utils.EventEmitter {
      constructor(
        renderer: PIXI.Renderer,
        options?: {
          autoPreventDefault?: boolean;
          interactionFrequency?: number;
          useSystemTicker?: number;
        },
      );

      /**
       * The renderer this interaction manager works for.
       *
       * @member {PIXI.AbstractRenderer} PIXI.interaction.InteractionManager#renderer
       */
      renderer: PIXI.AbstractRenderer;
      /**
       * Should default browser actions automatically be prevented.
       * Does not apply to pointer events for backwards compatibility
       * preventDefault on pointer events stops mouse events from firing
       * Thus, for every pointer event, there will always be either a mouse of touch event alongside it.
       *
       * @member {boolean} PIXI.interaction.InteractionManager#autoPreventDefault
       * @default true
       */
      autoPreventDefault: boolean;
      /**
       * Maximum requency in milliseconds at which pointer over/out states will be checked by {@link tickerUpdate}.
       *
       * @member {number} PIXI.interaction.InteractionManager#interactionFrequency
       * @default 10
       */
      interactionFrequency: number;
      /**
       * The mouse data
       *
       * @member {PIXI.interaction.InteractionData} PIXI.interaction.InteractionManager#mouse
       */
      mouse: PIXI.interaction.InteractionData;
      /**
       * An event data object to handle all the event tracking/dispatching
       *
       * @member {object} PIXI.interaction.InteractionManager#eventData
       */
      eventData: any;
      /**
       * The DOM element to bind to.
       *
       * @protected
       * @member {HTMLElement} PIXI.interaction.InteractionManager#interactionDOMElement
       */
      protected interactionDOMElement: HTMLElement;
      /**
       * This property determines if mousemove and touchmove events are fired only when the cursor
       * is over the object.
       * Setting to true will make things work more in line with how the DOM version works.
       * Setting to false can make things easier for things like dragging
       * It is currently set to false as this is how PixiJS used to work. This will be set to true in
       * future versions of pixi.
       *
       * @member {boolean} PIXI.interaction.InteractionManager#moveWhenInside
       * @default false
       */
      moveWhenInside: boolean;
      /**
       * Have events been attached to the dom element?
       *
       * @protected
       * @member {boolean} PIXI.interaction.InteractionManager#eventsAdded
       */
      protected eventsAdded: boolean;
      /**
       * Has the system ticker been added?
       *
       * @protected
       * @member {boolean} PIXI.interaction.InteractionManager#tickerAdded
       */
      protected tickerAdded: boolean;
      /**
       * Is the mouse hovering over the renderer?
       *
       * @protected
       * @member {boolean} PIXI.interaction.InteractionManager#mouseOverRenderer
       */
      protected mouseOverRenderer: boolean;
      /**
       * Does the device support touch events
       * https://www.w3.org/TR/touch-events/
       *
       * @readonly
       * @member {boolean} PIXI.interaction.InteractionManager#supportsTouchEvents
       */
      readonly supportsTouchEvents: boolean;
      /**
       * Does the device support pointer events
       * https://www.w3.org/Submission/pointer-events/
       *
       * @readonly
       * @member {boolean} PIXI.interaction.InteractionManager#supportsPointerEvents
       */
      readonly supportsPointerEvents: boolean;
      /**
       * Dictionary of how different cursor modes are handled. Strings are handled as CSS cursor
       * values, objects are handled as dictionaries of CSS values for interactionDOMElement,
       * and functions are called instead of changing the CSS.
       * Default CSS cursor values are provided for 'default' and 'pointer' modes.
       * @member {Object.<string, Object>} PIXI.interaction.InteractionManager#cursorStyles
       */
      cursorStyles: {
        [key: string]: any;
      };
      /**
       * The mode of the cursor that is being used.
       * The value of this is a key from the cursorStyles dictionary.
       *
       * @member {string} PIXI.interaction.InteractionManager#currentCursorMode
       */
      currentCursorMode: string;
      /**
       * The current resolution / device pixel ratio.
       *
       * @member {number} PIXI.interaction.InteractionManager#resolution
       * @default 1
       */
      resolution: number;
      /**
       * Should the InteractionManager automatically add {@link tickerUpdate} to {@link PIXI.Ticker.system}.
       *
       * @member {boolean}
       * @default true
       */
      useSystemTicker: boolean;

      /**
       * Hit tests a point against the display tree, returning the first interactive object that is hit.
       *
       * @param {PIXI.Point} globalPoint - A point to hit test with, in global space.
       * @param {PIXI.Container} [root] - The root display object to start from. If omitted, defaults
       * to the last rendered root of the associated renderer.
       * @return {PIXI.DisplayObject} The hit display object, if any.
       */
      hitTest(globalPoint: PIXI.Point, root?: PIXI.Container): PIXI.DisplayObject;

      /**
       * Sets the DOM element which will receive mouse/touch events. This is useful for when you have
       * other DOM elements on top of the renderers Canvas element. With this you'll be bale to delegate
       * another DOM element to receive those events.
       *
       * @param {HTMLElement} element - the DOM element which will receive mouse and touch events.
       * @param {number} [resolution=1] - The resolution / device pixel ratio of the new element (relative to the canvas).
       */
      setTargetElement(element: HTMLElement, resolution?: number): void;

      /**
       * Updates the state of interactive objects if at least {@link interactionFrequency}
       * milliseconds have passed since the last invocation.
       *
       * Invoked by a throttled ticker update from {@link PIXI.Ticker.system}.
       *
       * @param {number} deltaTime - time delta since the last call
       */
      tickerUpdate(deltaTime: number): void;

      /**
       * Updates the state of interactive objects.
       */
      update(): void;

      /**
       * Sets the current cursor mode, handling any callbacks or CSS style changes.
       *
       * @param {string} mode - cursor mode, a key from the cursorStyles dictionary
       */
      setCursorMode(mode: string): void;

      /**
       * Maps x and y coords from a DOM object and maps them correctly to the PixiJS view. The
       * resulting value is stored in the point. This takes into account the fact that the DOM
       * element could be scaled and positioned anywhere on the screen.
       *
       * @param  {PIXI.Point} point - the point that the result will be stored in
       * @param  {number} x - the x coord of the position to map
       * @param  {number} y - the y coord of the position to map
       */
      mapPositionToPoint(point: PIXI.Point, x: number, y: number): void;

      /**
       * This function is provides a neat way of crawling through the scene graph and running a
       * specified function on all interactive objects it finds. It will also take care of hit
       * testing the interactive objects and passes the hit across in the function.
       *
       * @protected
       * @param {PIXI.interaction.InteractionEvent} interactionEvent - event containing the point that
       *  is tested for collision
       * @param {PIXI.Container|PIXI.Sprite|PIXI.TilingSprite} displayObject - the displayObject
       *  that will be hit test (recursively crawls its children)
       * @param {Function} [func] - the function that will be called on each interactive object. The
       *  interactionEvent, displayObject and hit will be passed to the function
       * @param {boolean} [hitTest] - indicates whether we want to calculate hits
       *  or just iterate through all interactive objects
       */
      protected processInteractive(
        interactionEvent: PIXI.interaction.InteractionEvent,
        displayObject: PIXI.Container | PIXI.Sprite | PIXI.TilingSprite,
        func?: (...params: any[]) => any,
        hitTest?: boolean,
      ): void;

      /**
       * Destroys the interaction manager
       *
       */
      destroy(): void;
    }
  }

  /**
   * Interface for classes that represent a hit area.
   *
   * It is implemented by the following classes:
   * - {@link PIXI.Circle}
   * - {@link PIXI.Ellipse}
   * - {@link PIXI.Polygon}
   * - {@link PIXI.RoundedRectangle}
   *
   * @interface IHitArea
   * @memberof PIXI
   */
  interface IHitArea {
    /**
     * Checks whether the x and y coordinates given are contained within this area
     *
     * @method
     * @name contains
     * @memberof PIXI.IHitArea#
     * @param {number} x - The X coordinate of the point to test
     * @param {number} y - The Y coordinate of the point to test
     * @return {boolean} Whether the x/y coordinates are within this area
     */
    contains(x: number, y: number): boolean;
  }

  /**
   * Application plugin for supporting loader option. Installing the LoaderPlugin
   * is not necessary if using **pixi.js** or **pixi.js-legacy**.
   * @example
   * import {AppLoaderPlugin} from '@pixi/loaders';
   * import {Application} from '@pixi/app';
   * Application.registerPlugin(AppLoaderPlugin);
   * @class
   * @memberof PIXI
   */
  class AppLoaderPlugin {}

  /**
   * Plugin to be installed for handling specific Loader resources.
   *
   * @memberof PIXI
   * @typedef ILoaderPlugin
   * @property {function} [add] - Function to call immediate after registering plugin.
   * @property {PIXI.Loader.loaderMiddleware} [pre] - Middleware function to run before load, the
   *           arguments for this are `(resource, next)`
   * @property {PIXI.Loader.loaderMiddleware} [use] - Middleware function to run after load, the
   *           arguments for this are `(resource, next)`
   */
  type ILoaderPlugin = {
    add?: (...params: any[]) => any;
    pre?: PIXI.Loader.loaderMiddleware;
    use?: PIXI.Loader.loaderMiddleware;
  };
  namespace Loader {
    /**
     * @memberof PIXI.Loader
     * @callback loaderMiddleware
     * @param {PIXI.LoaderResource} resource
     * @param {function} next
     */
    type loaderMiddleware = (resource: PIXI.LoaderResource, next: (...params: any[]) => any) => void;
  }

  /**
   * The new loader, extends Resource Loader by Chad Engler: https://github.com/englercj/resource-loader
   *
   * ```js
   * const loader = PIXI.Loader.shared; // PixiJS exposes a premade instance for you to use.
   * //or
   * const loader = new PIXI.Loader(); // you can also create your own if you want
   *
   * const sprites = {};
   *
   * // Chainable `add` to enqueue a resource
   * loader.add('bunny', 'data/bunny.png')
   *       .add('spaceship', 'assets/spritesheet.json');
   * loader.add('scoreFont', 'assets/score.fnt');
   *
   * // Chainable `pre` to add a middleware that runs for each resource, *before* loading that resource.
   * // This is useful to implement custom caching modules (using filesystem, indexeddb, memory, etc).
   * loader.pre(cachingMiddleware);
   *
   * // Chainable `use` to add a middleware that runs for each resource, *after* loading that resource.
   * // This is useful to implement custom parsing modules (like spritesheet parsers, spine parser, etc).
   * loader.use(parsingMiddleware);
   *
   * // The `load` method loads the queue of resources, and calls the passed in callback called once all
   * // resources have loaded.
   * loader.load((loader, resources) => {
   *     // resources is an object where the key is the name of the resource loaded and the value is the resource object.
   *     // They have a couple default properties:
   *     // - `url`: The URL that the resource was loaded from
   *     // - `error`: The error that happened when trying to load (if any)
   *     // - `data`: The raw data that was loaded
   *     // also may contain other properties based on the middleware that runs.
   *     sprites.bunny = new PIXI.TilingSprite(resources.bunny.texture);
   *     sprites.spaceship = new PIXI.TilingSprite(resources.spaceship.texture);
   *     sprites.scoreFont = new PIXI.TilingSprite(resources.scoreFont.texture);
   * });
   *
   * // throughout the process multiple signals can be dispatched.
   * loader.onProgress.add(() => {}); // called once per loaded/errored file
   * loader.onError.add(() => {}); // called once per errored file
   * loader.onLoad.add(() => {}); // called once per loaded file
   * loader.onComplete.add(() => {}); // called once when the queued resources all load.
   * ```
   *
   * @see https://github.com/englercj/resource-loader
   *
   * @class Loader
   * @memberof PIXI
   * @param {string} [baseUrl=''] - The base url for all resources loaded by this loader.
   * @param {number} [concurrency=10] - The number of resources to load concurrently.
   */
  class Loader {
    constructor(baseUrl?: string, concurrency?: number);

    /**
     * @memberof PIXI.Loader#
     * @member {object} onStart
     */
    onStart: any;
    /**
     * @memberof PIXI.Loader#
     * @member {object} onProgress
     */
    onProgress: any;
    /**
     * @memberof PIXI.Loader#
     * @member {object} onError
     */
    onError: any;
    /**
     * @memberof PIXI.Loader#
     * @member {object} onLoad
     */
    onLoad: any;
    /**
     * @memberof PIXI.Loader#
     * @member {object} onComplete
     */
    onComplete: any;
    /**
     * A premade instance of the loader that can be used to load resources.
     * @name shared
     * @type {PIXI.Loader}
     * @static
     * @memberof PIXI.Loader
     */
    static shared: PIXI.Loader;

    /**
     * Adds a Loader plugin for the global shared loader and all
     * new Loader instances created.
     *
     * @static
     * @method registerPlugin
     * @memberof PIXI.Loader
     * @param {PIXI.ILoaderPlugin} plugin - The plugin to add
     * @return {PIXI.Loader} Reference to PIXI.Loader for chaining
     */
    static registerPlugin(plugin: PIXI.ILoaderPlugin): PIXI.Loader;
  }

  interface TextureLoader extends PIXI.ILoaderPlugin {}

  /**
   * Loader plugin for handling Texture resources.
   * @class
   * @memberof PIXI
   * @implements PIXI.ILoaderPlugin
   */
  class TextureLoader implements PIXI.ILoaderPlugin {
    /**
     * Called after a resource is loaded.
     * @see PIXI.Loader.loaderMiddleware
     * @param {PIXI.LoaderResource} resource
     * @param {function} next
     */
    static use(resource: PIXI.LoaderResource, next: (...params: any[]) => any): void;
  }

  /**
   * Reference to **{@link https://github.com/englercj/resource-loader
   * resource-loader}**'s Resource class.
   * @see http://englercj.github.io/resource-loader/Resource.html
   * @class LoaderResource
   * @memberof PIXI
   */
  class LoaderResource {}

  /**
   * Common interface for points. Both Point and ObservablePoint implement it
   * @memberof PIXI
   * @interface IPoint
   */
  interface IPoint {
    /**
     * X coord
     * @memberof PIXI.IPoint#
     * @member {number} x
     */
    x: number;
    /**
     * Y coord
     * @memberof PIXI.IPoint#
     * @member {number} y
     */
    y: number;

    /**
     * Sets the point to a new x and y position.
     * If y is omitted, both x and y will be set to x.
     *
     * @method set
     * @memberof PIXI.IPoint#
     * @param {number} [x=0] - position of the point on the x axis
     * @param {number} [y=x] - position of the point on the y axis
     */
    set(x?: number, y?: number): void;

    /**
     * Copies x and y from the given point
     * @method copyFrom
     * @memberof PIXI.IPoint#
     * @param {PIXI.IPoint} p - The point to copy from
     * @returns {this} Returns itself.
     */
    copyFrom(p: PIXI.IPoint): this;

    /**
     * Copies x and y into the given point
     * @method copyTo
     * @memberof PIXI.IPoint#
     * @param {PIXI.IPoint} p - The point to copy.
     * @returns {PIXI.IPoint} Given point with values updated
     */
    copyTo(p: PIXI.IPoint): PIXI.IPoint;

    /**
     * Returns true if the given point is equal to this point
     *
     * @method equals
     * @memberof PIXI.IPoint#
     * @param {PIXI.IPoint} p - The point to check
     * @returns {boolean} Whether the given point equal to this point
     */
    equals(p: PIXI.IPoint): boolean;
  }

  /**
   * The PixiJS Matrix as a class makes it a lot faster.
   *
   * Here is a representation of it:
   * ```js
   * | a | c | tx|
   * | b | d | ty|
   * | 0 | 0 | 1 |
   * ```
   * @class
   * @memberof PIXI
   */
  class Matrix {
    constructor(a?: number, b?: number, c?: number, d?: number, tx?: number, ty?: number);

    /**
     * @member {number} PIXI.Matrix#a
     * @default 1
     */
    a: number;
    /**
     * @member {number} PIXI.Matrix#b
     * @default 0
     */
    b: number;
    /**
     * @member {number} PIXI.Matrix#c
     * @default 0
     */
    c: number;
    /**
     * @member {number} PIXI.Matrix#d
     * @default 1
     */
    d: number;
    /**
     * @member {number} PIXI.Matrix#tx
     * @default 0
     */
    tx: number;
    /**
     * @member {number} PIXI.Matrix#ty
     * @default 0
     */
    ty: number;

    /**
     * Creates a Matrix object based on the given array. The Element to Matrix mapping order is as follows:
     *
     * a = array[0]
     * b = array[1]
     * c = array[3]
     * d = array[4]
     * tx = array[2]
     * ty = array[5]
     *
     * @param {number[]} array - The array that the matrix will be populated from.
     */
    fromArray(array: number[]): void;

    /**
     * sets the matrix properties
     *
     * @param {number} a - Matrix component
     * @param {number} b - Matrix component
     * @param {number} c - Matrix component
     * @param {number} d - Matrix component
     * @param {number} tx - Matrix component
     * @param {number} ty - Matrix component
     *
     * @return {PIXI.Matrix} This matrix. Good for chaining method calls.
     */
    set(a: number, b: number, c: number, d: number, tx: number, ty: number): PIXI.Matrix;

    /**
     * Creates an array from the current Matrix object.
     *
     * @param {boolean} transpose - Whether we need to transpose the matrix or not
     * @param {Float32Array} [out=new Float32Array(9)] - If provided the array will be assigned to out
     * @return {number[]} the newly created array which contains the matrix
     */
    toArray(transpose: boolean, out?: Float32Array): number[];

    /**
     * Get a new position with the current transformation applied.
     * Can be used to go from a child's coordinate space to the world coordinate space. (e.g. rendering)
     *
     * @param {PIXI.Point} pos - The origin
     * @param {PIXI.Point} [newPos] - The point that the new position is assigned to (allowed to be same as input)
     * @return {PIXI.Point} The new point, transformed through this matrix
     */
    apply(pos: PIXI.Point, newPos?: PIXI.Point): PIXI.Point;

    /**
     * Get a new position with the inverse of the current transformation applied.
     * Can be used to go from the world coordinate space to a child's coordinate space. (e.g. input)
     *
     * @param {PIXI.Point} pos - The origin
     * @param {PIXI.Point} [newPos] - The point that the new position is assigned to (allowed to be same as input)
     * @return {PIXI.Point} The new point, inverse-transformed through this matrix
     */
    applyInverse(pos: PIXI.Point, newPos?: PIXI.Point): PIXI.Point;

    /**
     * Translates the matrix on the x and y.
     *
     * @param {number} x How much to translate x by
     * @param {number} y How much to translate y by
     * @return {PIXI.Matrix} This matrix. Good for chaining method calls.
     */
    translate(x: number, y: number): PIXI.Matrix;

    /**
     * Applies a scale transformation to the matrix.
     *
     * @param {number} x The amount to scale horizontally
     * @param {number} y The amount to scale vertically
     * @return {PIXI.Matrix} This matrix. Good for chaining method calls.
     */
    scale(x: number, y: number): PIXI.Matrix;

    /**
     * Applies a rotation transformation to the matrix.
     *
     * @param {number} angle - The angle in radians.
     * @return {PIXI.Matrix} This matrix. Good for chaining method calls.
     */
    rotate(angle: number): PIXI.Matrix;

    /**
     * Appends the given Matrix to this Matrix.
     *
     * @param {PIXI.Matrix} matrix - The matrix to append.
     * @return {PIXI.Matrix} This matrix. Good for chaining method calls.
     */
    append(matrix: PIXI.Matrix): PIXI.Matrix;

    /**
     * Sets the matrix based on all the available properties
     *
     * @param {number} x - Position on the x axis
     * @param {number} y - Position on the y axis
     * @param {number} pivotX - Pivot on the x axis
     * @param {number} pivotY - Pivot on the y axis
     * @param {number} scaleX - Scale on the x axis
     * @param {number} scaleY - Scale on the y axis
     * @param {number} rotation - Rotation in radians
     * @param {number} skewX - Skew on the x axis
     * @param {number} skewY - Skew on the y axis
     * @return {PIXI.Matrix} This matrix. Good for chaining method calls.
     */
    setTransform(
      x: number,
      y: number,
      pivotX: number,
      pivotY: number,
      scaleX: number,
      scaleY: number,
      rotation: number,
      skewX: number,
      skewY: number,
    ): PIXI.Matrix;

    /**
     * Prepends the given Matrix to this Matrix.
     *
     * @param {PIXI.Matrix} matrix - The matrix to prepend
     * @return {PIXI.Matrix} This matrix. Good for chaining method calls.
     */
    prepend(matrix: PIXI.Matrix): PIXI.Matrix;

    /**
     * Decomposes the matrix (x, y, scaleX, scaleY, and rotation) and sets the properties on to a transform.
     *
     * @param {PIXI.Transform} transform - The transform to apply the properties to.
     * @return {PIXI.Transform} The transform with the newly applied properties
     */
    decompose(transform: PIXI.Transform): PIXI.Transform;

    /**
     * Inverts this matrix
     *
     * @return {PIXI.Matrix} This matrix. Good for chaining method calls.
     */
    invert(): PIXI.Matrix;

    /**
     * Resets this Matrix to an identity (default) matrix.
     *
     * @return {PIXI.Matrix} This matrix. Good for chaining method calls.
     */
    identity(): PIXI.Matrix;

    /**
     * Creates a new Matrix object with the same values as this one.
     *
     * @return {PIXI.Matrix} A copy of this matrix. Good for chaining method calls.
     */
    clone(): PIXI.Matrix;

    /**
     * Changes the values of the given matrix to be the same as the ones in this matrix
     *
     * @param {PIXI.Matrix} matrix - The matrix to copy to.
     * @return {PIXI.Matrix} The matrix given in parameter with its values updated.
     */
    copyTo(matrix: PIXI.Matrix): PIXI.Matrix;

    /**
     * Changes the values of the matrix to be the same as the ones in given matrix
     *
     * @param {PIXI.Matrix} matrix - The matrix to copy from.
     * @return {PIXI.Matrix} this
     */
    copyFrom(matrix: PIXI.Matrix): PIXI.Matrix;

    /**
     * A default (identity) matrix
     *
     * @static
     * @const
     * @member {PIXI.Matrix}
     */
    static IDENTITY: PIXI.Matrix;
    /**
     * A temp matrix
     *
     * @static
     * @const
     * @member {PIXI.Matrix}
     */
    static TEMP_MATRIX: PIXI.Matrix;
  }

  interface ObservablePoint extends IPoint {}

  /**
   * The Point object represents a location in a two-dimensional coordinate system, where x represents
   * the horizontal axis and y represents the vertical axis.
   *
   * An ObservablePoint is a point that triggers a callback when the point's position is changed.
   *
   * @class
   * @memberof PIXI
   * @implements IPoint
   */
  class ObservablePoint implements IPoint {
    constructor(cb: (...params: any[]) => any, scope: any, x?: number, y?: number);

    /**
     * Creates a clone of this point.
     * The callback and scope params can be overidden otherwise they will default
     * to the clone object's values.
     *
     * @override
     * @param {Function} [cb=null] - callback when changed
     * @param {object} [scope=null] - owner of callback
     * @return {PIXI.ObservablePoint} a copy of the point
     */
    clone(cb?: (...params: any[]) => any, scope?: any): PIXI.ObservablePoint;

    /**
     * Sets the point to a new x and y position.
     * If y is omitted, both x and y will be set to x.
     *
     * @param {number} [x=0] - position of the point on the x axis
     * @param {number} [y=x] - position of the point on the y axis
     * @returns {this} Returns itself.
     */
    set(x?: number, y?: number): this;

    /**
     * Copies x and y from the given point
     *
     * @param {PIXI.IPoint} p - The point to copy from.
     * @returns {this} Returns itself.
     */
    copyFrom(p: PIXI.IPoint): this;

    /**
     * Copies x and y into the given point
     *
     * @param {PIXI.IPoint} p - The point to copy.
     * @returns {PIXI.IPoint} Given point with values updated
     */
    copyTo(p: PIXI.IPoint): PIXI.IPoint;

    /**
     * Returns true if the given point is equal to this point
     *
     * @param {PIXI.IPoint} p - The point to check
     * @returns {boolean} Whether the given point equal to this point
     */
    equals(p: PIXI.IPoint): boolean;

    /**
     * The position of the displayObject on the x axis relative to the local coordinates of the parent.
     *
     * @member {number}
     */
    x: number;
    /**
     * The position of the displayObject on the x axis relative to the local coordinates of the parent.
     *
     * @member {number}
     */
    y: number;
  }

  interface Point extends IPoint {}

  /**
   * The Point object represents a location in a two-dimensional coordinate system, where x represents
   * the horizontal axis and y represents the vertical axis.
   *
   * @class
   * @memberof PIXI
   * @implements IPoint
   */
  class Point implements IPoint {
    constructor(x?: number, y?: number);

    /**
     * @member {number} PIXI.Point#x
     * @default 0
     */
    x: number;
    /**
     * @member {number} PIXI.Point#y
     * @default 0
     */
    y: number;

    /**
     * Creates a clone of this point
     *
     * @return {PIXI.Point} a copy of the point
     */
    clone(): PIXI.Point;

    /**
     * Copies x and y from the given point
     *
     * @param {PIXI.IPoint} p - The point to copy from
     * @returns {this} Returns itself.
     */
    copyFrom(p: PIXI.IPoint): this;

    /**
     * Copies x and y into the given point
     *
     * @param {PIXI.IPoint} p - The point to copy.
     * @returns {PIXI.IPoint} Given point with values updated
     */
    copyTo(p: PIXI.IPoint): PIXI.IPoint;

    /**
     * Returns true if the given point is equal to this point
     *
     * @param {PIXI.IPoint} p - The point to check
     * @returns {boolean} Whether the given point equal to this point
     */
    equals(p: PIXI.IPoint): boolean;

    /**
     * Sets the point to a new x and y position.
     * If y is omitted, both x and y will be set to x.
     *
     * @param {number} [x=0] - position of the point on the x axis
     * @param {number} [y=x] - position of the point on the y axis
     * @returns {this} Returns itself.
     */
    set(x?: number, y?: number): this;
  }

  /**
   * Transform that takes care about its versions
   *
   * @class
   * @memberof PIXI
   */
  class Transform {
    constructor();

    /**
     * The world transformation matrix.
     *
     * @member {PIXI.Matrix} PIXI.Transform#worldTransform
     */
    worldTransform: PIXI.Matrix;
    /**
     * The local transformation matrix.
     *
     * @member {PIXI.Matrix} PIXI.Transform#localTransform
     */
    localTransform: PIXI.Matrix;
    /**
     * The coordinate of the object relative to the local coordinates of the parent.
     *
     * @member {PIXI.ObservablePoint} PIXI.Transform#position
     */
    position: PIXI.ObservablePoint;
    /**
     * The scale factor of the object.
     *
     * @member {PIXI.ObservablePoint} PIXI.Transform#scale
     */
    scale: PIXI.ObservablePoint;
    /**
     * The pivot point of the displayObject that it rotates around.
     *
     * @member {PIXI.ObservablePoint} PIXI.Transform#pivot
     */
    pivot: PIXI.ObservablePoint;
    /**
     * The skew amount, on the x and y axis.
     *
     * @member {PIXI.ObservablePoint} PIXI.Transform#skew
     */
    skew: PIXI.ObservablePoint;
    /**
     * The rotation amount.
     *
     * @protected
     * @member {number} PIXI.Transform#_rotation
     */
    protected _rotation: number;
    /**
     * The X-coordinate value of the normalized local X axis,
     * the first column of the local transformation matrix without a scale.
     *
     * @protected
     * @member {number} PIXI.Transform#_cx
     */
    protected _cx: number;
    /**
     * The Y-coordinate value of the normalized local X axis,
     * the first column of the local transformation matrix without a scale.
     *
     * @protected
     * @member {number} PIXI.Transform#_sx
     */
    protected _sx: number;
    /**
     * The X-coordinate value of the normalized local Y axis,
     * the second column of the local transformation matrix without a scale.
     *
     * @protected
     * @member {number} PIXI.Transform#_cy
     */
    protected _cy: number;
    /**
     * The Y-coordinate value of the normalized local Y axis,
     * the second column of the local transformation matrix without a scale.
     *
     * @protected
     * @member {number} PIXI.Transform#_sy
     */
    protected _sy: number;
    /**
     * The locally unique ID of the local transform.
     *
     * @protected
     * @member {number} PIXI.Transform#_localID
     */
    protected _localID: number;
    /**
     * The locally unique ID of the local transform
     * used to calculate the current local transformation matrix.
     *
     * @protected
     * @member {number} PIXI.Transform#_currentLocalID
     */
    protected _currentLocalID: number;
    /**
     * The locally unique ID of the world transform.
     *
     * @protected
     * @member {number} PIXI.Transform#_worldID
     */
    protected _worldID: number;
    /**
     * The locally unique ID of the parent's world transform
     * used to calculate the current world transformation matrix.
     *
     * @protected
     * @member {number} PIXI.Transform#_parentID
     */
    protected _parentID: number;

    /**
     * Called when a value changes.
     *
     * @protected
     */
    protected onChange(): void;

    /**
     * Called when the skew or the rotation changes.
     *
     * @protected
     */
    protected updateSkew(): void;

    /**
     * Updates the local transformation matrix.
     */
    updateLocalTransform(): void;

    /**
     * Updates the local and the world transformation matrices.
     *
     * @param {PIXI.Transform} parentTransform - The parent transform
     */
    updateTransform(parentTransform: PIXI.Transform): void;

    /**
     * Decomposes a matrix and sets the transforms properties based on it.
     *
     * @param {PIXI.Matrix} matrix - The matrix to decompose
     */
    setFromMatrix(matrix: PIXI.Matrix): void;

    /**
     * The rotation of the object in radians.
     *
     * @member {number}
     */
    rotation: number;
    /**
     * A default (identity) transform
     *
     * @static
     * @constant
     * @member {PIXI.Transform}
     */
    static IDENTITY: PIXI.Transform;
  }

  /**
   * Constants that identify shapes, mainly to prevent `instanceof` calls.
   *
   * @static
   * @constant
   * @name SHAPES
   * @memberof PIXI
   * @type {enum}
   * @property {number} POLY Polygon
   * @property {number} RECT Rectangle
   * @property {number} CIRC Circle
   * @property {number} ELIP Ellipse
   * @property {number} RREC Rounded Rectangle
   * @enum {number}
   */
  enum SHAPES {
    POLY,
    RECT,
    CIRC,
    ELIP,
    RREC,
  }

  /**
   * Two Pi.
   *
   * @static
   * @constant {number} PI_2
   * @memberof PIXI
   */
  var PI_2: number;
  /**
   * Conversion factor for converting radians to degrees.
   *
   * @static
   * @constant {number} RAD_TO_DEG
   * @memberof PIXI
   */
  var RAD_TO_DEG: number;
  /**
   * Conversion factor for converting degrees to radians.
   *
   * @static
   * @constant {number} DEG_TO_RAD
   * @memberof PIXI
   */
  var DEG_TO_RAD: number;
  /**
   * @memberof PIXI
   * @typedef {number} GD8Symmetry
   * @see PIXI.groupD8
   */
  type GD8Symmetry = number;
  /**
   * Implements the dihedral group D8, which is similar to
   * [group D4]{@link http://mathworld.wolfram.com/DihedralGroupD4.html};
   * D8 is the same but with diagonals, and it is used for texture
   * rotations.
   *
   * The directions the U- and V- axes after rotation
   * of an angle of `a: GD8Constant` are the vectors `(uX(a), uY(a))`
   * and `(vX(a), vY(a))`. These aren't necessarily unit vectors.
   *
   * **Origin:**<br>
   *  This is the small part of gameofbombs.com portal system. It works.
   *
   * @see PIXI.groupD8.E
   * @see PIXI.groupD8.SE
   * @see PIXI.groupD8.S
   * @see PIXI.groupD8.SW
   * @see PIXI.groupD8.W
   * @see PIXI.groupD8.NW
   * @see PIXI.groupD8.N
   * @see PIXI.groupD8.NE
   * @author Ivan @ivanpopelyshev
   * @namespace PIXI.groupD8
   * @memberof PIXI
   */
  namespace groupD8 {
    /**
     * | Rotation | Direction |
     * |----------|-----------|
     * | 0°       | East      |
     *
     * @memberof PIXI.groupD8
     * @constant {PIXI.GD8Symmetry}
     */
    var E: PIXI.GD8Symmetry;
    /**
     * | Rotation | Direction |
     * |----------|-----------|
     * | 45°↻     | Southeast |
     *
     * @memberof PIXI.groupD8
     * @constant {PIXI.GD8Symmetry}
     */
    var SE: PIXI.GD8Symmetry;
    /**
     * | Rotation | Direction |
     * |----------|-----------|
     * | 90°↻     | South     |
     *
     * @memberof PIXI.groupD8
     * @constant {PIXI.GD8Symmetry}
     */
    var S: PIXI.GD8Symmetry;
    /**
     * | Rotation | Direction |
     * |----------|-----------|
     * | 135°↻    | Southwest |
     *
     * @memberof PIXI.groupD8
     * @constant {PIXI.GD8Symmetry}
     */
    var SW: PIXI.GD8Symmetry;
    /**
     * | Rotation | Direction |
     * |----------|-----------|
     * | 180°     | West      |
     *
     * @memberof PIXI.groupD8
     * @constant {PIXI.GD8Symmetry}
     */
    var W: PIXI.GD8Symmetry;
    /**
     * | Rotation    | Direction    |
     * |-------------|--------------|
     * | -135°/225°↻ | Northwest    |
     *
     * @memberof PIXI.groupD8
     * @constant {PIXI.GD8Symmetry}
     */
    var NW: PIXI.GD8Symmetry;
    /**
     * | Rotation    | Direction    |
     * |-------------|--------------|
     * | -90°/270°↻  | North        |
     *
     * @memberof PIXI.groupD8
     * @constant {PIXI.GD8Symmetry}
     */
    var N: PIXI.GD8Symmetry;
    /**
     * | Rotation    | Direction    |
     * |-------------|--------------|
     * | -45°/315°↻  | Northeast    |
     *
     * @memberof PIXI.groupD8
     * @constant {PIXI.GD8Symmetry}
     */
    var NE: PIXI.GD8Symmetry;
    /**
     * Reflection about Y-axis.
     *
     * @memberof PIXI.groupD8
     * @constant {PIXI.GD8Symmetry}
     */
    var MIRROR_VERTICAL: PIXI.GD8Symmetry;
    /**
     * Reflection about the main diagonal.
     *
     * @memberof PIXI.groupD8
     * @constant {PIXI.GD8Symmetry}
     */
    var MAIN_DIAGONAL: PIXI.GD8Symmetry;
    /**
     * Reflection about X-axis.
     *
     * @memberof PIXI.groupD8
     * @constant {PIXI.GD8Symmetry}
     */
    var MIRROR_HORIZONTAL: PIXI.GD8Symmetry;
    /**
     * Reflection about reverse diagonal.
     *
     * @memberof PIXI.groupD8
     * @constant {PIXI.GD8Symmetry}
     */
    var REVERSE_DIAGONAL: PIXI.GD8Symmetry;

    /**
     * @memberof PIXI.groupD8
     * @param {PIXI.GD8Symmetry} ind - sprite rotation angle.
     * @return {PIXI.GD8Symmetry} The X-component of the U-axis
     *    after rotating the axes.
     */
    function uX(ind: PIXI.GD8Symmetry): PIXI.GD8Symmetry;

    /**
     * @memberof PIXI.groupD8
     * @param {PIXI.GD8Symmetry} ind - sprite rotation angle.
     * @return {PIXI.GD8Symmetry} The Y-component of the U-axis
     *    after rotating the axes.
     */
    function uY(ind: PIXI.GD8Symmetry): PIXI.GD8Symmetry;

    /**
     * @memberof PIXI.groupD8
     * @param {PIXI.GD8Symmetry} ind - sprite rotation angle.
     * @return {PIXI.GD8Symmetry} The X-component of the V-axis
     *    after rotating the axes.
     */
    function vX(ind: PIXI.GD8Symmetry): PIXI.GD8Symmetry;

    /**
     * @memberof PIXI.groupD8
     * @param {PIXI.GD8Symmetry} ind - sprite rotation angle.
     * @return {PIXI.GD8Symmetry} The Y-component of the V-axis
     *    after rotating the axes.
     */
    function vY(ind: PIXI.GD8Symmetry): PIXI.GD8Symmetry;

    /**
     * @memberof PIXI.groupD8
     * @param {PIXI.GD8Symmetry} rotation - symmetry whose opposite
     *   is needed. Only rotations have opposite symmetries while
     *   reflections don't.
     * @return {PIXI.GD8Symmetry} The opposite symmetry of `rotation`
     */
    function inv(rotation: PIXI.GD8Symmetry): PIXI.GD8Symmetry;

    /**
     * Composes the two D8 operations.
     *
     * Taking `^` as reflection:
     *
     * |       | E=0 | S=2 | W=4 | N=6 | E^=8 | S^=10 | W^=12 | N^=14 |
     * |-------|-----|-----|-----|-----|------|-------|-------|-------|
     * | E=0   | E   | S   | W   | N   | E^   | S^    | W^    | N^    |
     * | S=2   | S   | W   | N   | E   | S^   | W^    | N^    | E^    |
     * | W=4   | W   | N   | E   | S   | W^   | N^    | E^    | S^    |
     * | N=6   | N   | E   | S   | W   | N^   | E^    | S^    | W^    |
     * | E^=8  | E^  | N^  | W^  | S^  | E    | N     | W     | S     |
     * | S^=10 | S^  | E^  | N^  | W^  | S    | E     | N     | W     |
     * | W^=12 | W^  | S^  | E^  | N^  | W    | S     | E     | N     |
     * | N^=14 | N^  | W^  | S^  | E^  | N    | W     | S     | E     |
     *
     * [This is a Cayley table]{@link https://en.wikipedia.org/wiki/Cayley_table}
     * @memberof PIXI.groupD8
     * @param {PIXI.GD8Symmetry} rotationSecond - Second operation, which
     *   is the row in the above cayley table.
     * @param {PIXI.GD8Symmetry} rotationFirst - First operation, which
     *   is the column in the above cayley table.
     * @return {PIXI.GD8Symmetry} Composed operation
     */
    function add(rotationSecond: PIXI.GD8Symmetry, rotationFirst: PIXI.GD8Symmetry): PIXI.GD8Symmetry;

    /**
     * Reverse of `add`.
     *
     * @memberof PIXI.groupD8
     * @param {PIXI.GD8Symmetry} rotationSecond - Second operation
     * @param {PIXI.GD8Symmetry} rotationFirst - First operation
     * @return {PIXI.GD8Symmetry} Result
     */
    function sub(rotationSecond: PIXI.GD8Symmetry, rotationFirst: PIXI.GD8Symmetry): PIXI.GD8Symmetry;

    /**
     * Adds 180 degrees to rotation, which is a commutative
     * operation.
     *
     * @memberof PIXI.groupD8
     * @param {number} rotation - The number to rotate.
     * @returns {number} Rotated number
     */
    function rotate180(rotation: number): number;

    /**
     * Checks if the rotation angle is vertical, i.e. south
     * or north. It doesn't work for reflections.
     *
     * @memberof PIXI.groupD8
     * @param {PIXI.GD8Symmetry} rotation - The number to check.
     * @returns {boolean} Whether or not the direction is vertical
     */
    function isVertical(rotation: PIXI.GD8Symmetry): boolean;

    /**
     * Approximates the vector `V(dx,dy)` into one of the
     * eight directions provided by `groupD8`.
     *
     * @memberof PIXI.groupD8
     * @param {number} dx - X-component of the vector
     * @param {number} dy - Y-component of the vector
     * @return {PIXI.GD8Symmetry} Approximation of the vector into
     *  one of the eight symmetries.
     */
    function byDirection(dx: number, dy: number): PIXI.GD8Symmetry;

    /**
     * Helps sprite to compensate texture packer rotation.
     *
     * @memberof PIXI.groupD8
     * @param {PIXI.Matrix} matrix - sprite world matrix
     * @param {PIXI.GD8Symmetry} rotation - The rotation factor to use.
     * @param {number} tx - sprite anchoring
     * @param {number} ty - sprite anchoring
     */
    function matrixAppendRotationInv(matrix: PIXI.Matrix, rotation: PIXI.GD8Symmetry, tx: number, ty: number): void;
  }

  /**
   * The Circle object is used to help draw graphics and can also be used to specify a hit area for displayObjects.
   *
   * @class
   * @memberof PIXI
   */
  class Circle {
    constructor(x?: number, y?: number, radius?: number);

    /**
     * @member {number} PIXI.Circle#x
     * @default 0
     */
    x: number;
    /**
     * @member {number} PIXI.Circle#y
     * @default 0
     */
    y: number;
    /**
     * @member {number} PIXI.Circle#radius
     * @default 0
     */
    radius: number;
    /**
     * The type of the object, mainly used to avoid `instanceof` checks
     *
     * @member {number} PIXI.Circle#type
     * @readOnly
     * @default PIXI.SHAPES.CIRC
     * @see PIXI.SHAPES
     */
    readonly type: number;

    /**
     * Creates a clone of this Circle instance
     *
     * @return {PIXI.Circle} a copy of the Circle
     */
    clone(): PIXI.Circle;

    /**
     * Checks whether the x and y coordinates given are contained within this circle
     *
     * @param {number} x - The X coordinate of the point to test
     * @param {number} y - The Y coordinate of the point to test
     * @return {boolean} Whether the x/y coordinates are within this Circle
     */
    contains(x: number, y: number): boolean;

    /**
     * Returns the framing rectangle of the circle as a Rectangle object
     *
     * @return {PIXI.Rectangle} the framing rectangle
     */
    getBounds(): PIXI.Rectangle;
  }

  /**
   * The Ellipse object is used to help draw graphics and can also be used to specify a hit area for displayObjects.
   *
   * @class
   * @memberof PIXI
   */
  class Ellipse {
    constructor(x?: number, y?: number, halfWidth?: number, halfHeight?: number);

    /**
     * @member {number} PIXI.Ellipse#x
     * @default 0
     */
    x: number;
    /**
     * @member {number} PIXI.Ellipse#y
     * @default 0
     */
    y: number;
    /**
     * @member {number} PIXI.Ellipse#width
     * @default 0
     */
    width: number;
    /**
     * @member {number} PIXI.Ellipse#height
     * @default 0
     */
    height: number;
    /**
     * The type of the object, mainly used to avoid `instanceof` checks
     *
     * @member {number} PIXI.Ellipse#type
     * @readOnly
     * @default PIXI.SHAPES.ELIP
     * @see PIXI.SHAPES
     */
    readonly type: number;

    /**
     * Creates a clone of this Ellipse instance
     *
     * @return {PIXI.Ellipse} a copy of the ellipse
     */
    clone(): PIXI.Ellipse;

    /**
     * Checks whether the x and y coordinates given are contained within this ellipse
     *
     * @param {number} x - The X coordinate of the point to test
     * @param {number} y - The Y coordinate of the point to test
     * @return {boolean} Whether the x/y coords are within this ellipse
     */
    contains(x: number, y: number): boolean;

    /**
     * Returns the framing rectangle of the ellipse as a Rectangle object
     *
     * @return {PIXI.Rectangle} the framing rectangle
     */
    getBounds(): PIXI.Rectangle;
  }

  /**
   * A class to define a shape via user defined co-orinates.
   *
   * @class
   * @memberof PIXI
   */
  class Polygon {
    constructor(...points: (PIXI.Point[] | number[] | number[][])[]);

    /**
     * An array of the points of this polygon
     *
     * @member {number[]} PIXI.Polygon#points
     */
    points: number[];
    /**
     * The type of the object, mainly used to avoid `instanceof` checks
     *
     * @member {number} PIXI.Polygon#type
     * @readOnly
     * @default PIXI.SHAPES.POLY
     * @see PIXI.SHAPES
     */
    readonly type: number;
    /**
     * `false` after moveTo, `true` after `closePath`. In all other cases it is `true`.
     * @member {boolean} PIXI.Polygon#closeStroke
     * @default true
     */
    closeStroke: boolean;

    /**
     * Creates a clone of this polygon
     *
     * @return {PIXI.Polygon} a copy of the polygon
     */
    clone(): PIXI.Polygon;

    /**
     * Checks whether the x and y coordinates passed to this function are contained within this polygon
     *
     * @param {number} x - The X coordinate of the point to test
     * @param {number} y - The Y coordinate of the point to test
     * @return {boolean} Whether the x/y coordinates are within this polygon
     */
    contains(x: number, y: number): boolean;
  }

  /**
   * Size object, contains width and height
   *
   * @memberof PIXI
   * @typedef {object} ISize
   * @property {number} width - Width component
   * @property {number} height - Height component
   */
  type ISize = {
    width: number;
    height: number;
  };

  /**
   * Rectangle object is an area defined by its position, as indicated by its top-left corner
   * point (x, y) and by its width and its height.
   *
   * @class
   * @memberof PIXI
   */
  class Rectangle {
    constructor(x?: number, y?: number, width?: number, height?: number);

    /**
     * @member {number} PIXI.Rectangle#x
     * @default 0
     */
    x: number;
    /**
     * @member {number} PIXI.Rectangle#y
     * @default 0
     */
    y: number;
    /**
     * @member {number} PIXI.Rectangle#width
     * @default 0
     */
    width: number;
    /**
     * @member {number} PIXI.Rectangle#height
     * @default 0
     */
    height: number;
    /**
     * The type of the object, mainly used to avoid `instanceof` checks
     *
     * @member {number} PIXI.Rectangle#type
     * @readOnly
     * @default PIXI.SHAPES.RECT
     * @see PIXI.SHAPES
     */
    readonly type: number;
    /**
     * returns the left edge of the rectangle
     *
     * @member {number}
     */
    left: number;
    /**
     * returns the right edge of the rectangle
     *
     * @member {number}
     */
    right: number;
    /**
     * returns the top edge of the rectangle
     *
     * @member {number}
     */
    top: number;
    /**
     * returns the bottom edge of the rectangle
     *
     * @member {number}
     */
    bottom: number;
    /**
     * A constant empty rectangle.
     *
     * @static
     * @constant
     * @member {PIXI.Rectangle}
     * @return {PIXI.Rectangle} An empty rectangle
     */
    static EMPTY: PIXI.Rectangle;

    /**
     * Creates a clone of this Rectangle
     *
     * @return {PIXI.Rectangle} a copy of the rectangle
     */
    clone(): PIXI.Rectangle;

    /**
     * Copies another rectangle to this one.
     *
     * @param {PIXI.Rectangle} rectangle - The rectangle to copy from.
     * @return {PIXI.Rectangle} Returns itself.
     */
    copyFrom(rectangle: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Copies this rectangle to another one.
     *
     * @param {PIXI.Rectangle} rectangle - The rectangle to copy to.
     * @return {PIXI.Rectangle} Returns given parameter.
     */
    copyTo(rectangle: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Checks whether the x and y coordinates given are contained within this Rectangle
     *
     * @param {number} x - The X coordinate of the point to test
     * @param {number} y - The Y coordinate of the point to test
     * @return {boolean} Whether the x/y coordinates are within this Rectangle
     */
    contains(x: number, y: number): boolean;

    /**
     * Pads the rectangle making it grow in all directions.
     * If paddingY is omitted, both paddingX and paddingY will be set to paddingX.
     *
     * @param {number} [paddingX=0] - The horizontal padding amount.
     * @param {number} [paddingY=0] - The vertical padding amount.
     * @return {PIXI.Rectangle} Returns itself.
     */
    pad(paddingX?: number, paddingY?: number): PIXI.Rectangle;

    /**
     * Fits this rectangle around the passed one.
     *
     * @param {PIXI.Rectangle} rectangle - The rectangle to fit.
     * @return {PIXI.Rectangle} Returns itself.
     */
    fit(rectangle: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Enlarges rectangle that way its corners lie on grid
     *
     * @param {number} [resolution=1] resolution
     * @param {number} [eps=0.001] precision
     * @return {PIXI.Rectangle} Returns itself.
     */
    ceil(resolution?: number, eps?: number): PIXI.Rectangle;

    /**
     * Enlarges this rectangle to include the passed rectangle.
     *
     * @param {PIXI.Rectangle} rectangle - The rectangle to include.
     * @return {PIXI.Rectangle} Returns itself.
     */
    enlarge(rectangle: PIXI.Rectangle): PIXI.Rectangle;
  }

  /**
   * The Rounded Rectangle object is an area that has nice rounded corners, as indicated by its
   * top-left corner point (x, y) and by its width and its height and its radius.
   *
   * @class
   * @memberof PIXI
   */
  class RoundedRectangle {
    constructor(x?: number, y?: number, width?: number, height?: number, radius?: number);

    /**
     * @member {number} PIXI.RoundedRectangle#x
     * @default 0
     */
    x: number;
    /**
     * @member {number} PIXI.RoundedRectangle#y
     * @default 0
     */
    y: number;
    /**
     * @member {number} PIXI.RoundedRectangle#width
     * @default 0
     */
    width: number;
    /**
     * @member {number} PIXI.RoundedRectangle#height
     * @default 0
     */
    height: number;
    /**
     * @member {number} PIXI.RoundedRectangle#radius
     * @default 20
     */
    radius: number;
    /**
     * The type of the object, mainly used to avoid `instanceof` checks
     *
     * @member {number} PIXI.RoundedRectangle#type
     * @readonly
     * @default PIXI.SHAPES.RREC
     * @see PIXI.SHAPES
     */
    readonly type: number;

    /**
     * Creates a clone of this Rounded Rectangle
     *
     * @return {PIXI.RoundedRectangle} a copy of the rounded rectangle
     */
    clone(): PIXI.RoundedRectangle;

    /**
     * Checks whether the x and y coordinates given are contained within this Rounded Rectangle
     *
     * @param {number} x - The X coordinate of the point to test
     * @param {number} y - The Y coordinate of the point to test
     * @return {boolean} Whether the x/y coordinates are within this Rounded Rectangle
     */
    contains(x: number, y: number): boolean;
  }

  /**
   * Base mesh class.
   *
   * This class empowers you to have maximum flexibility to render any kind of WebGL visuals you can think of.
   * This class assumes a certain level of WebGL knowledge.
   * If you know a bit this should abstract enough away to make you life easier!
   *
   * Pretty much ALL WebGL can be broken down into the following:
   * - Geometry - The structure and data for the mesh. This can include anything from positions, uvs, normals, colors etc..
   * - Shader - This is the shader that PixiJS will render the geometry with (attributes in the shader must match the geometry)
   * - State - This is the state of WebGL required to render the mesh.
   *
   * Through a combination of the above elements you can render anything you want, 2D or 3D!
   *
   * @class
   * @extends PIXI.Container
   * @memberof PIXI
   */
  class Mesh extends PIXI.Container {
    constructor(
      geometry: PIXI.Geometry,
      shader: PIXI.Shader | PIXI.MeshMaterial,
      state?: PIXI.State,
      drawMode?: number,
    );

    /**
     * Includes vertex positions, face indices, normals, colors, UVs, and
     * custom attributes within buffers, reducing the cost of passing all
     * this data to the GPU. Can be shared between multiple Mesh objects.
     * @member {PIXI.Geometry} PIXI.Mesh#geometry
     * @readonly
     */
    readonly geometry: PIXI.Geometry;
    /**
     * Represents the vertex and fragment shaders that processes the geometry and runs on the GPU.
     * Can be shared between multiple Mesh objects.
     * @member {PIXI.Shader|PIXI.MeshMaterial} PIXI.Mesh#shader
     */
    shader: PIXI.Shader | PIXI.MeshMaterial;
    /**
     * Represents the WebGL state the Mesh required to render, excludes shader and geometry. E.g.,
     * blend mode, culling, depth testing, direction of rendering triangles, backface, etc.
     * @member {PIXI.State} PIXI.Mesh#state
     */
    state: PIXI.State;
    /**
     * The way the Mesh should be drawn, can be any of the {@link PIXI.DRAW_MODES} constants.
     *
     * @member {number} PIXI.Mesh#drawMode
     * @see PIXI.DRAW_MODES
     */
    drawMode: number;
    /**
     * Typically the index of the IndexBuffer where to start drawing.
     * @member {number} PIXI.Mesh#start
     * @default 0
     */
    start: number;
    /**
     * How much of the geometry to draw, by default `0` renders everything.
     * @member {number} PIXI.Mesh#size
     * @default 0
     */
    size: number;
    /**
     * To change mesh uv's, change its uvBuffer data and increment its _updateID.
     * @member {PIXI.Buffer}
     * @readonly
     */
    readonly uvBuffer: PIXI.Buffer;
    /**
     * To change mesh vertices, change its uvBuffer data and increment its _updateID.
     * Incrementing _updateID is optional because most of Mesh objects do it anyway.
     * @member {PIXI.Buffer}
     * @readonly
     */
    readonly verticesBuffer: PIXI.Buffer;
    /**
     * Alias for {@link PIXI.Mesh#shader}.
     * @member {PIXI.Shader|PIXI.MeshMaterial}
     */
    material: PIXI.Shader | PIXI.MeshMaterial;
    /**
     * The blend mode to be applied to the Mesh. Apply a value of
     * `PIXI.BLEND_MODES.NORMAL` to reset the blend mode.
     *
     * @member {number}
     * @default PIXI.BLEND_MODES.NORMAL;
     * @see PIXI.BLEND_MODES
     */
    blendMode: number;
    /**
     * If true PixiJS will Math.floor() x/y values when rendering, stopping pixel interpolation.
     * Advantages can include sharper image quality (like text) and faster rendering on canvas.
     * The main disadvantage is movement of objects may appear less smooth.
     * To set the global default, change {@link PIXI.settings.ROUND_PIXELS}
     *
     * @member {boolean}
     * @default false
     */
    roundPixels: boolean;
    /**
     * The multiply tint applied to the Mesh. This is a hex value. A value of
     * `0xFFFFFF` will remove any tint effect.
     *
     * @member {number}
     * @default 0xFFFFFF
     */
    tint: number;
    /**
     * The texture that the Mesh uses.
     *
     * @member {PIXI.Texture}
     */
    texture: PIXI.Texture;

    /**
     * Standard renderer draw.
     * @protected
     * @param {PIXI.Renderer} renderer - Instance to renderer.
     */
    protected _render(renderer: PIXI.Renderer): void;

    /**
     * Standard non-batching way of rendering.
     * @protected
     * @param {PIXI.Renderer} renderer - Instance to renderer.
     */
    protected _renderDefault(renderer: PIXI.Renderer): void;

    /**
     * Rendering by using the Batch system.
     * @protected
     * @param {PIXI.Renderer} renderer - Instance to renderer.
     */
    protected _renderToBatch(renderer: PIXI.Renderer): void;

    /**
     * Updates vertexData field based on transform and vertices
     */
    calculateVertices(): void;

    /**
     * Updates uv field based on from geometry uv's or batchUvs
     */
    calculateUvs(): void;

    /**
     * Updates the bounds of the mesh as a rectangle. The bounds calculation takes the worldTransform into account.
     * there must be a aVertexPosition attribute present in the geometry for bounds to be calculated correctly.
     *
     * @protected
     */
    protected _calculateBounds(): void;

    /**
     * Tests if a point is inside this mesh. Works only for PIXI.DRAW_MODES.TRIANGLES.
     *
     * @param {PIXI.Point} point the point to test
     * @return {boolean} the result of the test
     */
    containsPoint(point: PIXI.Point): boolean;

    /**
     * Destroys the Mesh object.
     *
     * @param {object|boolean} [options] - Options parameter. A boolean will act as if all
     *  options have been set to that value
     * @param {boolean} [options.children=false] - if set to true, all the children will have
     *  their destroy method called as well. 'options' will be passed on to those calls.
     */
    destroy(options?: { children?: boolean }): void;

    /**
     * The maximum number of vertices to consider batchable. Generally, the complexity
     * of the geometry.
     * @memberof PIXI.Mesh
     * @static
     * @member {number} BATCHABLE_SIZE
     */
    static BATCHABLE_SIZE: number;
    /**
     * The array of children of this container.
     *
     * @member {PIXI.DisplayObject[]} PIXI.Container#children
     * @readonly
     */
    readonly children: PIXI.DisplayObject[];
    /**
     * If set to true, the container will sort its children by zIndex value
     * when updateTransform() is called, or manually if sortChildren() is called.
     *
     * This actually changes the order of elements in the array, so should be treated
     * as a basic solution that is not performant compared to other solutions,
     * such as @link https://github.com/pixijs/pixi-display
     *
     * Also be aware of that this may not work nicely with the addChildAt() function,
     * as the zIndex sorting may cause the child to automatically sorted to another position.
     *
     * @see PIXI.settings.SORTABLE_CHILDREN
     *
     * @member {boolean} PIXI.Container#sortableChildren
     */
    sortableChildren: boolean;
    /**
     * Should children be sorted by zIndex at the next updateTransform call.
     * Will get automatically set to true if a new child is added, or if a child's zIndex changes.
     *
     * @member {boolean} PIXI.Container#sortDirty
     */
    sortDirty: boolean;

    /**
     * Overridable method that can be used by Container subclasses whenever the children array is modified
     *
     * @protected
     */
    protected onChildrenChange(): void;

    /**
     * Adds one or more children to the container.
     *
     * Multiple items can be added like so: `myContainer.addChild(thingOne, thingTwo, thingThree)`
     *
     * @param {...PIXI.DisplayObject} child - The DisplayObject(s) to add to the container
     * @return {PIXI.DisplayObject} The first child that was added.
     */
    addChild<TChildren extends PIXI.DisplayObject[]>(...child: TChildren): TChildren[0];

    /**
     * Adds a child to the container at a specified index. If the index is out of bounds an error will be thrown
     *
     * @param {PIXI.DisplayObject} child - The child to add
     * @param {number} index - The index to place the child in
     * @return {PIXI.DisplayObject} The child that was added.
     */
    addChildAt<T extends PIXI.DisplayObject>(child: T, index: number): T;

    /**
     * Swaps the position of 2 Display Objects within this container.
     *
     * @param {PIXI.DisplayObject} child - First display object to swap
     * @param {PIXI.DisplayObject} child2 - Second display object to swap
     */
    swapChildren(child: PIXI.DisplayObject, child2: PIXI.DisplayObject): void;

    /**
     * Returns the index position of a child DisplayObject instance
     *
     * @param {PIXI.DisplayObject} child - The DisplayObject instance to identify
     * @return {number} The index position of the child display object to identify
     */
    getChildIndex(child: PIXI.DisplayObject): number;

    /**
     * Changes the position of an existing child in the display object container
     *
     * @param {PIXI.DisplayObject} child - The child DisplayObject instance for which you want to change the index number
     * @param {number} index - The resulting index number for the child display object
     */
    setChildIndex(child: PIXI.DisplayObject, index: number): void;

    /**
     * Returns the child at the specified index
     *
     * @param {number} index - The index to get the child at
     * @return {PIXI.DisplayObject} The child at the given index, if any.
     */
    getChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes one or more children from the container.
     *
     * @param {...PIXI.DisplayObject} child - The DisplayObject(s) to remove
     * @return {PIXI.DisplayObject} The first child that was removed.
     */
    removeChild<TChildren extends PIXI.DisplayObject[]>(...child: TChildren): TChildren[0];

    /**
     * Removes a child from the specified index position.
     *
     * @param {number} index - The index to get the child from
     * @return {PIXI.DisplayObject} The child that was removed.
     */
    removeChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes all children from this container that are within the begin and end indexes.
     *
     * @param {number} [beginIndex=0] - The beginning position.
     * @param {number} [endIndex=this.children.length] - The ending position. Default value is size of the container.
     * @returns {PIXI.DisplayObject[]} List of removed children
     */
    removeChildren(beginIndex?: number, endIndex?: number): PIXI.DisplayObject[];

    /**
     * Sorts children by zIndex. Previous order is mantained for 2 children with the same zIndex.
     */
    sortChildren(): void;

    /**
     * Updates the transform on all children of this container for rendering
     */
    updateTransform(): void;

    /**
     * Recalculates the bounds of the container.
     *
     */
    calculateBounds(): void;

    /**
     * Renders the object using the WebGL renderer
     *
     * @param {PIXI.Renderer} renderer - The renderer
     */
    render(renderer: PIXI.Renderer): void;

    /**
     * Render the object using the WebGL renderer and advanced features.
     *
     * @protected
     * @param {PIXI.Renderer} renderer - The renderer
     */
    protected renderAdvanced(renderer: PIXI.Renderer): void;

    /**
     * The width of the Container, setting this will actually modify the scale to achieve the value set
     *
     * @member {number}
     */
    width: number;
    /**
     * The height of the Container, setting this will actually modify the scale to achieve the value set
     *
     * @member {number}
     */
    height: number;
    /**
     * Determines if the children to the displayObject can be clicked/touched
     * Setting this to false allows PixiJS to bypass a recursive `hitTest` function
     *
     * @member {boolean}
     * @memberof PIXI.Container#
     */
    interactiveChildren: boolean;

    /**
     * Returns the display object in the container.
     *
     * @method getChildByName
     * @memberof PIXI.Container#
     * @param {string} name - Instance name.
     * @return {PIXI.DisplayObject} The child with the specified name.
     */
    getChildByName(name: string): PIXI.DisplayObject;

    /**
     *  Flag for if the object is accessible. If true AccessibilityManager will overlay a
     *   shadow div with attributes set
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    accessible: boolean;
    /**
     * Sets the title attribute of the shadow div
     * If accessibleTitle AND accessibleHint has not been this will default to 'displayObject [tabIndex]'
     *
     * @member {?string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleTitle: string;
    /**
     * Sets the aria-label attribute of the shadow div
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleHint: string;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleActive: boolean;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleDiv: boolean;
    /**
     * Specify the type of div the accessible layer is. Screen readers treat the element differently
     * depending on this type. Defaults to button.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'button'
     */
    accessibleType: string;
    /**
     * Specify the pointer-events the accessible div will use
     * Defaults to auto.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'auto'
     */
    accessiblePointerEvents: string;
    /**
     * Setting to false will prevent any children inside this container to
     * be accessible. Defaults to true.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @default true
     */
    accessibleChildren: boolean;
    /**
     * World transform and local transform of this object.
     * This will become read-only later, please do not assign anything there unless you know what are you doing.
     *
     * @member {PIXI.Transform} PIXI.DisplayObject#transform
     */
    transform: PIXI.Transform;
    /**
     * The opacity of the object.
     *
     * @member {number} PIXI.DisplayObject#alpha
     */
    alpha: number;
    /**
     * The visibility of the object. If false the object will not be drawn, and
     * the updateTransform function will not be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds or call updateTransform manually.
     *
     * @member {boolean} PIXI.DisplayObject#visible
     */
    visible: boolean;
    /**
     * Can this object be rendered, if false the object will not be drawn but the updateTransform
     * methods will still be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds manually.
     *
     * @member {boolean} PIXI.DisplayObject#renderable
     */
    renderable: boolean;
    /**
     * The display object container that contains this display object.
     *
     * @member {PIXI.Container} PIXI.DisplayObject#parent
     * @readonly
     */
    readonly parent: PIXI.Container;
    /**
     * The multiplied alpha of the displayObject.
     *
     * @member {number} PIXI.DisplayObject#worldAlpha
     * @readonly
     */
    readonly worldAlpha: number;
    /**
     * Which index in the children array the display component was before the previous zIndex sort.
     * Used by containers to help sort objects with the same zIndex, by using previous array index as the decider.
     *
     * @member {number} PIXI.DisplayObject#_lastSortedIndex
     * @protected
     */
    protected _lastSortedIndex: number;
    /**
     * The zIndex of the displayObject.
     * A higher value will mean it will be rendered on top of other displayObjects within the same container.
     *
     * @member {number} PIXI.DisplayObject#_zIndex
     * @protected
     */
    protected _zIndex: number;
    /**
     * The area the filter is applied to. This is used as more of an optimization
     * rather than figuring out the dimensions of the displayObject each frame you can set this rectangle.
     *
     * Also works as an interaction mask.
     *
     * @member {?PIXI.Rectangle} PIXI.DisplayObject#filterArea
     */
    filterArea: PIXI.Rectangle;
    /**
     * Sets the filters for the displayObject.
     * * IMPORTANT: This is a WebGL only feature and will be ignored by the canvas renderer.
     * To remove filters simply set this property to `'null'`.
     *
     * @member {?PIXI.Filter[]} PIXI.DisplayObject#filters
     */
    filters: PIXI.Filter[];
    /**
     * The bounds object, this is used to calculate and store the bounds of the displayObject.
     *
     * @member {PIXI.Bounds} PIXI.DisplayObject#_bounds
     * @protected
     */
    protected _bounds: PIXI.Bounds;
    /**
     * The original, cached mask of the object.
     *
     * @member {PIXI.Graphics|PIXI.Sprite|null} PIXI.DisplayObject#_mask
     * @protected
     */
    protected _mask: PIXI.Graphics | PIXI.Sprite | null;
    /**
     * If the object has been destroyed via destroy(). If true, it should not be used.
     *
     * @member {boolean} PIXI.DisplayObject#_destroyed
     * @protected
     */
    protected _destroyed: boolean;
    /**
     * used to fast check if a sprite is.. a sprite!
     * @member {boolean} PIXI.DisplayObject#isSprite
     */
    isSprite: boolean;
    /**
     * Does any other displayObject use this object as a mask?
     * @member {boolean} PIXI.DisplayObject#isMask
     */
    isMask: boolean;
    /**
     * @protected
     * @member {PIXI.DisplayObject}
     */
    protected _tempDisplayObjectParent: PIXI.DisplayObject;

    /**
     * Recursively updates transform of all objects from the root to this one
     * internal function for toLocal()
     */
    _recursivePostUpdateTransform(): void;

    /**
     * Retrieves the bounds of the displayObject as a rectangle object.
     *
     * @param {boolean} [skipUpdate] - Setting to `true` will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @param {PIXI.Rectangle} [rect] - Optional rectangle to store the result of the bounds calculation.
     * @return {PIXI.Rectangle} The rectangular bounding area.
     */
    getBounds(skipUpdate?: boolean, rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Retrieves the local bounds of the displayObject as a rectangle object.
     *
     * @param {PIXI.Rectangle} [rect] - Optional rectangle to store the result of the bounds calculation.
     * @return {PIXI.Rectangle} The rectangular bounding area.
     */
    getLocalBounds(rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Calculates the global position of the display object.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform.
     * @return {PIXI.IPoint} A point object representing the position of this object.
     */
    toGlobal(position: PIXI.IPoint, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Calculates the local position of the display object relative to another point.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.DisplayObject} [from] - The DisplayObject to calculate the global position from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform
     * @return {PIXI.IPoint} A point object representing the position of this object
     */
    toLocal(position: PIXI.IPoint, from?: PIXI.DisplayObject, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Set the parent Container of this DisplayObject.
     *
     * @param {PIXI.Container} container - The Container to add this DisplayObject to.
     * @return {PIXI.Container} The Container that this DisplayObject was added to.
     */
    setParent(container: PIXI.Container): PIXI.Container;

    /**
     * Convenience function to set the position, scale, skew and pivot at once.
     *
     * @param {number} [x=0] - The X position
     * @param {number} [y=0] - The Y position
     * @param {number} [scaleX=1] - The X scale value
     * @param {number} [scaleY=1] - The Y scale value
     * @param {number} [rotation=0] - The rotation
     * @param {number} [skewX=0] - The X skew value
     * @param {number} [skewY=0] - The Y skew value
     * @param {number} [pivotX=0] - The X pivot value
     * @param {number} [pivotY=0] - The Y pivot value
     * @return {PIXI.DisplayObject} The DisplayObject instance
     */
    setTransform(
      x?: number,
      y?: number,
      scaleX?: number,
      scaleY?: number,
      rotation?: number,
      skewX?: number,
      skewY?: number,
      pivotX?: number,
      pivotY?: number,
    ): PIXI.DisplayObject;

    /**
     * The position of the displayObject on the x axis relative to the local coordinates of the parent.
     * An alias to position.x
     *
     * @member {number}
     */
    x: number;
    /**
     * The position of the displayObject on the y axis relative to the local coordinates of the parent.
     * An alias to position.y
     *
     * @member {number}
     */
    y: number;
    /**
     * Current transform of the object based on world (parent) factors.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly worldTransform: PIXI.Matrix;
    /**
     * Current transform of the object based on local factors: position, scale, other stuff.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly localTransform: PIXI.Matrix;
    /**
     * The coordinate of the object relative to the local coordinates of the parent.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    position: PIXI.IPoint;
    /**
     * The scale factor of the object.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    scale: PIXI.IPoint;
    /**
     * The pivot point of the displayObject that it rotates around.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    pivot: PIXI.IPoint;
    /**
     * The skew factor for the object in radians.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.ObservablePoint}
     */
    skew: PIXI.ObservablePoint;
    /**
     * The rotation of the object in radians.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    rotation: number;
    /**
     * The angle of the object in degrees.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    angle: number;
    /**
     * The zIndex of the displayObject.
     * If a container has the sortableChildren property set to true, children will be automatically
     * sorted by zIndex value; a higher value will mean it will be moved towards the end of the array,
     * and thus rendered on top of other displayObjects within the same container.
     *
     * @member {number}
     */
    zIndex: number;
    /**
     * Indicates if the object is globally visible.
     *
     * @member {boolean}
     * @readonly
     */
    readonly worldVisible: boolean;
    /**
     * Sets a mask for the displayObject. A mask is an object that limits the visibility of an
     * object to the shape of the mask applied to it. In PixiJS a regular mask must be a
     * {@link PIXI.Graphics} or a {@link PIXI.Sprite} object. This allows for much faster masking in canvas as it
     * utilities shape clipping. To remove a mask, set this property to `null`.
     *
     * For sprite mask both alpha and red channel are used. Black mask is the same as transparent mask.
     * @example
     * const graphics = new PIXI.Graphics();
     * graphics.beginFill(0xFF3300);
     * graphics.drawRect(50, 250, 100, 100);
     * graphics.endFill();
     *
     * const sprite = new PIXI.Sprite(texture);
     * sprite.mask = graphics;
     * @todo At the moment, PIXI.CanvasRenderer doesn't support PIXI.Sprite as mask.
     *
     * @member {PIXI.Container|PIXI.MaskData}
     */
    mask: PIXI.Container | PIXI.MaskData;

    /**
     * DisplayObject default updateTransform, does not update children of container.
     * Will crash if there's no parent element.
     *
     * @memberof PIXI.DisplayObject#
     * @function displayObjectUpdateTransform
     */
    displayObjectUpdateTransform(): void;

    /**
     * Enable interaction events for the DisplayObject. Touch, pointer and mouse
     * events will not be emitted unless `interactive` is set to `true`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.on('tap', (event) => {
     *    //handle event
     * });
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    interactive: boolean;
    /**
     * Interaction shape. Children will be hit first, then this shape will be checked.
     * Setting this will cause this shape to be checked in hit tests rather than the displayObject's bounds.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.hitArea = new PIXI.Rectangle(0, 0, 100, 100);
     * @member {PIXI.IHitArea}
     * @memberof PIXI.DisplayObject#
     */
    hitArea: PIXI.IHitArea;
    /**
     * If enabled, the mouse cursor use the pointer behavior when hovered over the displayObject if it is interactive
     * Setting this changes the 'cursor' property to `'pointer'`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.buttonMode = true;
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    buttonMode: boolean;
    /**
     * This defines what cursor mode is used when the mouse cursor
     * is hovered over the displayObject.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.cursor = 'wait';
     * @see https://developer.mozilla.org/en/docs/Web/CSS/cursor
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    cursor: string;
    /**
     * Set this to true if you want this display object to be cached as a bitmap.
     * This basically takes a snap shot of the display object as it is at that moment. It can
     * provide a performance benefit for complex static displayObjects.
     * To remove simply set this property to `false`
     *
     * IMPORTANT GOTCHA - Make sure that all your textures are preloaded BEFORE setting this property to true
     * as it will take a snapshot of what is currently there. If the textures have not loaded then they will not appear.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    cacheAsBitmap: boolean;
    /**
     * The instance name of the object.
     *
     * @memberof PIXI.DisplayObject#
     * @member {string} name
     */
    name: string;

    /**
     * Returns the global position of the displayObject. Does not depend on object scale, rotation and pivot.
     *
     * @method getGlobalPosition
     * @memberof PIXI.DisplayObject#
     * @param {PIXI.Point} [point=new PIXI.Point()] - The point to write the global value to.
     * @param {boolean} [skipUpdate=false] - Setting to true will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @return {PIXI.Point} The updated point.
     */
    getGlobalPosition(point?: PIXI.Point, skipUpdate?: boolean): PIXI.Point;
  }

  /**
   * Class controls cache for UV mapping from Texture normal space to BaseTexture normal space.
   *
   * @class
   * @memberof PIXI
   */
  class MeshBatchUvs {
    constructor(uvBuffer: PIXI.Buffer, uvMatrix: PIXI.TextureMatrix);

    /**
     * Buffer with normalized UV's
     * @member {PIXI.Buffer} PIXI.MeshBatchUvs#uvBuffer
     */
    uvBuffer: PIXI.Buffer;
    /**
     * Material UV matrix
     * @member {PIXI.TextureMatrix} PIXI.MeshBatchUvs#uvMatrix
     */
    uvMatrix: PIXI.TextureMatrix;
    /**
     * UV Buffer data
     * @member {Float32Array} PIXI.MeshBatchUvs#data
     * @readonly
     */
    readonly data: Float32Array;

    /**
     * updates
     *
     * @param {boolean} forceUpdate - force the update
     */
    update(forceUpdate: boolean): void;
  }

  /**
   * Standard 2D geometry used in PixiJS.
   *
   * Geometry can be defined without passing in a style or data if required.
   *
   * ```js
   * const geometry = new PIXI.Geometry();
   *
   * geometry.addAttribute('positions', [0, 0, 100, 0, 100, 100, 0, 100], 2);
   * geometry.addAttribute('uvs', [0,0,1,0,1,1,0,1], 2);
   * geometry.addIndex([0,1,2,1,3,2]);
   *
   * ```
   * @class
   * @memberof PIXI
   * @extends PIXI.Geometry
   */
  class MeshGeometry extends PIXI.Geometry {
    constructor(vertices: Float32Array | number[], uvs: Float32Array | number[], index: Uint16Array | number[]);

    /**
     * A map of renderer IDs to webgl VAOs
     *
     * @protected
     * @type {object}
     */
    protected glVertexArrayObjects: any;
    /**
     * Number of instances in this geometry, pass it to `GeometrySystem.draw()`
     * @member {number} PIXI.Geometry#instanceCount
     * @default 1
     */
    instanceCount: number;
    /**
     * Count of existing (not destroyed) meshes that reference this geometry
     * @member {number} PIXI.Geometry#refCount
     */
    refCount: number;

    /**
     *
     * Adds an attribute to the geometry
     *
     * @param {String} id - the name of the attribute (matching up to a shader)
     * @param {PIXI.Buffer|number[]} [buffer] the buffer that holds the data of the attribute . You can also provide an Array and a buffer will be created from it.
     * @param {Number} [size=0] the size of the attribute. If you have 2 floats per vertex (eg position x and y) this would be 2
     * @param {Boolean} [normalized=false] should the data be normalized.
     * @param {Number} [type=PIXI.TYPES.FLOAT] what type of number is the attribute. Check {PIXI.TYPES} to see the ones available
     * @param {Number} [stride=0] How far apart (in floats) the start of each value is. (used for interleaving data)
     * @param {Number} [start=0] How far into the array to start reading values (used for interleaving data)
     *
     * @return {PIXI.Geometry} returns self, useful for chaining.
     */
    addAttribute(
      id: string,
      buffer?: PIXI.Buffer | number[],
      size?: number,
      normalized?: boolean,
      type?: number,
      stride?: number,
      start?: number,
    ): PIXI.Geometry;

    /**
     * returns the requested attribute
     *
     * @param {String} id  the name of the attribute required
     * @return {PIXI.Attribute} the attribute requested.
     */
    getAttribute(id: string): PIXI.Attribute;

    /**
     * returns the requested buffer
     *
     * @param {String} id  the name of the buffer required
     * @return {PIXI.Buffer} the buffer requested.
     */
    getBuffer(id: string): PIXI.Buffer;

    /**
     *
     * Adds an index buffer to the geometry
     * The index buffer contains integers, three for each triangle in the geometry, which reference the various attribute buffers (position, colour, UV coordinates, other UV coordinates, normal, …). There is only ONE index buffer.
     *
     * @param {PIXI.Buffer|number[]} [buffer] the buffer that holds the data of the index buffer. You can also provide an Array and a buffer will be created from it.
     * @return {PIXI.Geometry} returns self, useful for chaining.
     */
    addIndex(buffer?: PIXI.Buffer | number[]): PIXI.Geometry;

    /**
     * returns the index buffer
     *
     * @return {PIXI.Buffer} the index buffer.
     */
    getIndex(): PIXI.Buffer;

    /**
     * this function modifies the structure so that all current attributes become interleaved into a single buffer
     * This can be useful if your model remains static as it offers a little performance boost
     *
     * @return {PIXI.Geometry} returns self, useful for chaining.
     */
    interleave(): PIXI.Geometry;

    /**
     * disposes WebGL resources that are connected to this geometry
     */
    dispose(): void;

    /**
     * Destroys the geometry.
     */
    destroy(): void;

    /**
     * returns a clone of the geometry
     *
     * @returns {PIXI.Geometry} a new clone of this geometry
     */
    clone(): PIXI.Geometry;
  }

  /**
   * Slightly opinionated default shader for PixiJS 2D objects.
   * @class
   * @memberof PIXI
   * @extends PIXI.Shader
   */
  class MeshMaterial extends PIXI.Shader {
    constructor(
      uSampler: PIXI.Texture,
      options?: {
        alpha?: number;
        tint?: number;
        pluginName?: string;
        program?: PIXI.Program;
        uniforms?: any;
      },
    );

    /**
     * TextureMatrix instance for this Mesh, used to track Texture changes
     *
     * @member {PIXI.TextureMatrix} PIXI.MeshMaterial#uvMatrix
     * @readonly
     */
    readonly uvMatrix: PIXI.TextureMatrix;
    /**
     * `true` if shader can be batch with the renderer's batch system.
     * @member {boolean} PIXI.MeshMaterial#batchable
     * @default true
     */
    batchable: boolean;
    /**
     * Renderer plugin for batching
     *
     * @member {string} PIXI.MeshMaterial#pluginName
     * @default 'batch'
     */
    pluginName: string;
    /**
     * Reference to the texture being rendered.
     * @member {PIXI.Texture}
     */
    texture: PIXI.Texture;
    /**
     * This gets automatically set by the object using this.
     *
     * @default 1
     * @member {number}
     */
    alpha: number;
    /**
     * Multiply tint for the material.
     * @member {number}
     * @default 0xFFFFFF
     */
    tint: number;

    /**
     * Gets called automatically by the Mesh. Intended to be overridden for custom
     * MeshMaterial objects.
     */
    update(): void;

    /**
     * Program that the shader uses
     *
     * @member {PIXI.Program} PIXI.Shader#program
     */
    program: PIXI.Program;
    /**
     * Shader uniform values, shortcut for `uniformGroup.uniforms`
     * @readonly
     * @member {object}
     */
    readonly uniforms: any;
  }

  /**
   * The NineSlicePlane allows you to stretch a texture using 9-slice scaling. The corners will remain unscaled (useful
   * for buttons with rounded corners for example) and the other areas will be scaled horizontally and or vertically
   *
   *```js
   * let Plane9 = new PIXI.NineSlicePlane(PIXI.Texture.from('BoxWithRoundedCorners.png'), 15, 15, 15, 15);
   *  ```
   * <pre>
   *      A                          B
   *    +---+----------------------+---+
   *  C | 1 |          2           | 3 |
   *    +---+----------------------+---+
   *    |   |                      |   |
   *    | 4 |          5           | 6 |
   *    |   |                      |   |
   *    +---+----------------------+---+
   *  D | 7 |          8           | 9 |
   *    +---+----------------------+---+

   *  When changing this objects width and/or height:
   *     areas 1 3 7 and 9 will remain unscaled.
   *     areas 2 and 8 will be stretched horizontally
   *     areas 4 and 6 will be stretched vertically
   *     area 5 will be stretched both horizontally and vertically
   * </pre>
   *
   * @class
   * @extends PIXI.SimplePlane
   * @memberof PIXI
   *
   */
  class NineSlicePlane extends PIXI.SimplePlane {
    constructor(
      texture: PIXI.Texture,
      leftWidth?: number,
      topHeight?: number,
      rightWidth?: number,
      bottomHeight?: number,
    );

    /**
     * The width of the NineSlicePlane, setting this will actually modify the vertices and UV's of this plane
     *
     * @member {number} PIXI.NineSlicePlane#_width
     * @override
     */
    _width: number;
    /**
     * The height of the NineSlicePlane, setting this will actually modify the vertices and UV's of this plane
     *
     * @member {number} PIXI.NineSlicePlane#_height
     * @override
     */
    _height: number;

    /**
     * Updates the horizontal vertices.
     *
     */
    updateHorizontalVertices(): void;

    /**
     * Updates the vertical vertices.
     *
     */
    updateVerticalVertices(): void;

    /**
     * The width of the NineSlicePlane, setting this will actually modify the vertices and UV's of this plane
     *
     * @member {number}
     */
    width: number;
    /**
     * The height of the NineSlicePlane, setting this will actually modify the vertices and UV's of this plane
     *
     * @member {number}
     */
    height: number;
    /**
     * The width of the left column
     *
     * @member {number}
     */
    leftWidth: number;
    /**
     * The width of the right column
     *
     * @member {number}
     */
    rightWidth: number;
    /**
     * The height of the top row
     *
     * @member {number}
     */
    topHeight: number;
    /**
     * The height of the bottom row
     *
     * @member {number}
     */
    bottomHeight: number;

    /**
     * Refreshes NineSlicePlane coords. All of them.
     */
    _refresh(): void;

    /**
     * Method used for overrides, to do something in case texture frame was changed.
     * Meshes based on plane can override it and change more details based on texture.
     */
    textureUpdated(): void;

    /**
     * Includes vertex positions, face indices, normals, colors, UVs, and
     * custom attributes within buffers, reducing the cost of passing all
     * this data to the GPU. Can be shared between multiple Mesh objects.
     * @member {PIXI.Geometry} PIXI.Mesh#geometry
     * @readonly
     */
    readonly geometry: PIXI.Geometry;
    /**
     * Represents the vertex and fragment shaders that processes the geometry and runs on the GPU.
     * Can be shared between multiple Mesh objects.
     * @member {PIXI.Shader|PIXI.MeshMaterial} PIXI.Mesh#shader
     */
    shader: PIXI.Shader | PIXI.MeshMaterial;
    /**
     * Represents the WebGL state the Mesh required to render, excludes shader and geometry. E.g.,
     * blend mode, culling, depth testing, direction of rendering triangles, backface, etc.
     * @member {PIXI.State} PIXI.Mesh#state
     */
    state: PIXI.State;
    /**
     * The way the Mesh should be drawn, can be any of the {@link PIXI.DRAW_MODES} constants.
     *
     * @member {number} PIXI.Mesh#drawMode
     * @see PIXI.DRAW_MODES
     */
    drawMode: number;
    /**
     * Typically the index of the IndexBuffer where to start drawing.
     * @member {number} PIXI.Mesh#start
     * @default 0
     */
    start: number;
    /**
     * How much of the geometry to draw, by default `0` renders everything.
     * @member {number} PIXI.Mesh#size
     * @default 0
     */
    size: number;
    /**
     * To change mesh uv's, change its uvBuffer data and increment its _updateID.
     * @member {PIXI.Buffer}
     * @readonly
     */
    readonly uvBuffer: PIXI.Buffer;
    /**
     * To change mesh vertices, change its uvBuffer data and increment its _updateID.
     * Incrementing _updateID is optional because most of Mesh objects do it anyway.
     * @member {PIXI.Buffer}
     * @readonly
     */
    readonly verticesBuffer: PIXI.Buffer;
    /**
     * Alias for {@link PIXI.Mesh#shader}.
     * @member {PIXI.Shader|PIXI.MeshMaterial}
     */
    material: PIXI.Shader | PIXI.MeshMaterial;
    /**
     * The blend mode to be applied to the Mesh. Apply a value of
     * `PIXI.BLEND_MODES.NORMAL` to reset the blend mode.
     *
     * @member {number}
     * @default PIXI.BLEND_MODES.NORMAL;
     * @see PIXI.BLEND_MODES
     */
    blendMode: number;
    /**
     * If true PixiJS will Math.floor() x/y values when rendering, stopping pixel interpolation.
     * Advantages can include sharper image quality (like text) and faster rendering on canvas.
     * The main disadvantage is movement of objects may appear less smooth.
     * To set the global default, change {@link PIXI.settings.ROUND_PIXELS}
     *
     * @member {boolean}
     * @default false
     */
    roundPixels: boolean;
    /**
     * The multiply tint applied to the Mesh. This is a hex value. A value of
     * `0xFFFFFF` will remove any tint effect.
     *
     * @member {number}
     * @default 0xFFFFFF
     */
    tint: number;
    /**
     * The texture that the Mesh uses.
     *
     * @member {PIXI.Texture}
     */
    texture: PIXI.Texture;

    /**
     * Standard renderer draw.
     * @protected
     * @param {PIXI.Renderer} renderer - Instance to renderer.
     */
    protected _render(renderer: PIXI.Renderer): void;

    /**
     * Standard non-batching way of rendering.
     * @protected
     * @param {PIXI.Renderer} renderer - Instance to renderer.
     */
    protected _renderDefault(renderer: PIXI.Renderer): void;

    /**
     * Rendering by using the Batch system.
     * @protected
     * @param {PIXI.Renderer} renderer - Instance to renderer.
     */
    protected _renderToBatch(renderer: PIXI.Renderer): void;

    /**
     * Updates vertexData field based on transform and vertices
     */
    calculateVertices(): void;

    /**
     * Updates uv field based on from geometry uv's or batchUvs
     */
    calculateUvs(): void;

    /**
     * Updates the bounds of the mesh as a rectangle. The bounds calculation takes the worldTransform into account.
     * there must be a aVertexPosition attribute present in the geometry for bounds to be calculated correctly.
     *
     * @protected
     */
    protected _calculateBounds(): void;

    /**
     * Tests if a point is inside this mesh. Works only for PIXI.DRAW_MODES.TRIANGLES.
     *
     * @param {PIXI.Point} point the point to test
     * @return {boolean} the result of the test
     */
    containsPoint(point: PIXI.Point): boolean;

    /**
     * Destroys the Mesh object.
     *
     * @param {object|boolean} [options] - Options parameter. A boolean will act as if all
     *  options have been set to that value
     * @param {boolean} [options.children=false] - if set to true, all the children will have
     *  their destroy method called as well. 'options' will be passed on to those calls.
     */
    destroy(options?: { children?: boolean }): void;

    /**
     * The array of children of this container.
     *
     * @member {PIXI.DisplayObject[]} PIXI.Container#children
     * @readonly
     */
    readonly children: PIXI.DisplayObject[];
    /**
     * If set to true, the container will sort its children by zIndex value
     * when updateTransform() is called, or manually if sortChildren() is called.
     *
     * This actually changes the order of elements in the array, so should be treated
     * as a basic solution that is not performant compared to other solutions,
     * such as @link https://github.com/pixijs/pixi-display
     *
     * Also be aware of that this may not work nicely with the addChildAt() function,
     * as the zIndex sorting may cause the child to automatically sorted to another position.
     *
     * @see PIXI.settings.SORTABLE_CHILDREN
     *
     * @member {boolean} PIXI.Container#sortableChildren
     */
    sortableChildren: boolean;
    /**
     * Should children be sorted by zIndex at the next updateTransform call.
     * Will get automatically set to true if a new child is added, or if a child's zIndex changes.
     *
     * @member {boolean} PIXI.Container#sortDirty
     */
    sortDirty: boolean;

    /**
     * Overridable method that can be used by Container subclasses whenever the children array is modified
     *
     * @protected
     */
    protected onChildrenChange(): void;

    /**
     * Adds one or more children to the container.
     *
     * Multiple items can be added like so: `myContainer.addChild(thingOne, thingTwo, thingThree)`
     *
     * @param {...PIXI.DisplayObject} child - The DisplayObject(s) to add to the container
     * @return {PIXI.DisplayObject} The first child that was added.
     */
    addChild<TChildren extends PIXI.DisplayObject[]>(...child: TChildren): TChildren[0];

    /**
     * Adds a child to the container at a specified index. If the index is out of bounds an error will be thrown
     *
     * @param {PIXI.DisplayObject} child - The child to add
     * @param {number} index - The index to place the child in
     * @return {PIXI.DisplayObject} The child that was added.
     */
    addChildAt<T extends PIXI.DisplayObject>(child: T, index: number): T;

    /**
     * Swaps the position of 2 Display Objects within this container.
     *
     * @param {PIXI.DisplayObject} child - First display object to swap
     * @param {PIXI.DisplayObject} child2 - Second display object to swap
     */
    swapChildren(child: PIXI.DisplayObject, child2: PIXI.DisplayObject): void;

    /**
     * Returns the index position of a child DisplayObject instance
     *
     * @param {PIXI.DisplayObject} child - The DisplayObject instance to identify
     * @return {number} The index position of the child display object to identify
     */
    getChildIndex(child: PIXI.DisplayObject): number;

    /**
     * Changes the position of an existing child in the display object container
     *
     * @param {PIXI.DisplayObject} child - The child DisplayObject instance for which you want to change the index number
     * @param {number} index - The resulting index number for the child display object
     */
    setChildIndex(child: PIXI.DisplayObject, index: number): void;

    /**
     * Returns the child at the specified index
     *
     * @param {number} index - The index to get the child at
     * @return {PIXI.DisplayObject} The child at the given index, if any.
     */
    getChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes one or more children from the container.
     *
     * @param {...PIXI.DisplayObject} child - The DisplayObject(s) to remove
     * @return {PIXI.DisplayObject} The first child that was removed.
     */
    removeChild<TChildren extends PIXI.DisplayObject[]>(...child: TChildren): TChildren[0];

    /**
     * Removes a child from the specified index position.
     *
     * @param {number} index - The index to get the child from
     * @return {PIXI.DisplayObject} The child that was removed.
     */
    removeChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes all children from this container that are within the begin and end indexes.
     *
     * @param {number} [beginIndex=0] - The beginning position.
     * @param {number} [endIndex=this.children.length] - The ending position. Default value is size of the container.
     * @returns {PIXI.DisplayObject[]} List of removed children
     */
    removeChildren(beginIndex?: number, endIndex?: number): PIXI.DisplayObject[];

    /**
     * Sorts children by zIndex. Previous order is mantained for 2 children with the same zIndex.
     */
    sortChildren(): void;

    /**
     * Updates the transform on all children of this container for rendering
     */
    updateTransform(): void;

    /**
     * Recalculates the bounds of the container.
     *
     */
    calculateBounds(): void;

    /**
     * Renders the object using the WebGL renderer
     *
     * @param {PIXI.Renderer} renderer - The renderer
     */
    render(renderer: PIXI.Renderer): void;

    /**
     * Render the object using the WebGL renderer and advanced features.
     *
     * @protected
     * @param {PIXI.Renderer} renderer - The renderer
     */
    protected renderAdvanced(renderer: PIXI.Renderer): void;

    /**
     * Determines if the children to the displayObject can be clicked/touched
     * Setting this to false allows PixiJS to bypass a recursive `hitTest` function
     *
     * @member {boolean}
     * @memberof PIXI.Container#
     */
    interactiveChildren: boolean;

    /**
     * Returns the display object in the container.
     *
     * @method getChildByName
     * @memberof PIXI.Container#
     * @param {string} name - Instance name.
     * @return {PIXI.DisplayObject} The child with the specified name.
     */
    getChildByName(name: string): PIXI.DisplayObject;

    /**
     *  Flag for if the object is accessible. If true AccessibilityManager will overlay a
     *   shadow div with attributes set
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    accessible: boolean;
    /**
     * Sets the title attribute of the shadow div
     * If accessibleTitle AND accessibleHint has not been this will default to 'displayObject [tabIndex]'
     *
     * @member {?string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleTitle: string;
    /**
     * Sets the aria-label attribute of the shadow div
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleHint: string;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleActive: boolean;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleDiv: boolean;
    /**
     * Specify the type of div the accessible layer is. Screen readers treat the element differently
     * depending on this type. Defaults to button.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'button'
     */
    accessibleType: string;
    /**
     * Specify the pointer-events the accessible div will use
     * Defaults to auto.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'auto'
     */
    accessiblePointerEvents: string;
    /**
     * Setting to false will prevent any children inside this container to
     * be accessible. Defaults to true.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @default true
     */
    accessibleChildren: boolean;
    /**
     * World transform and local transform of this object.
     * This will become read-only later, please do not assign anything there unless you know what are you doing.
     *
     * @member {PIXI.Transform} PIXI.DisplayObject#transform
     */
    transform: PIXI.Transform;
    /**
     * The opacity of the object.
     *
     * @member {number} PIXI.DisplayObject#alpha
     */
    alpha: number;
    /**
     * The visibility of the object. If false the object will not be drawn, and
     * the updateTransform function will not be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds or call updateTransform manually.
     *
     * @member {boolean} PIXI.DisplayObject#visible
     */
    visible: boolean;
    /**
     * Can this object be rendered, if false the object will not be drawn but the updateTransform
     * methods will still be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds manually.
     *
     * @member {boolean} PIXI.DisplayObject#renderable
     */
    renderable: boolean;
    /**
     * The display object container that contains this display object.
     *
     * @member {PIXI.Container} PIXI.DisplayObject#parent
     * @readonly
     */
    readonly parent: PIXI.Container;
    /**
     * The multiplied alpha of the displayObject.
     *
     * @member {number} PIXI.DisplayObject#worldAlpha
     * @readonly
     */
    readonly worldAlpha: number;
    /**
     * Which index in the children array the display component was before the previous zIndex sort.
     * Used by containers to help sort objects with the same zIndex, by using previous array index as the decider.
     *
     * @member {number} PIXI.DisplayObject#_lastSortedIndex
     * @protected
     */
    protected _lastSortedIndex: number;
    /**
     * The zIndex of the displayObject.
     * A higher value will mean it will be rendered on top of other displayObjects within the same container.
     *
     * @member {number} PIXI.DisplayObject#_zIndex
     * @protected
     */
    protected _zIndex: number;
    /**
     * The area the filter is applied to. This is used as more of an optimization
     * rather than figuring out the dimensions of the displayObject each frame you can set this rectangle.
     *
     * Also works as an interaction mask.
     *
     * @member {?PIXI.Rectangle} PIXI.DisplayObject#filterArea
     */
    filterArea: PIXI.Rectangle;
    /**
     * Sets the filters for the displayObject.
     * * IMPORTANT: This is a WebGL only feature and will be ignored by the canvas renderer.
     * To remove filters simply set this property to `'null'`.
     *
     * @member {?PIXI.Filter[]} PIXI.DisplayObject#filters
     */
    filters: PIXI.Filter[];
    /**
     * The bounds object, this is used to calculate and store the bounds of the displayObject.
     *
     * @member {PIXI.Bounds} PIXI.DisplayObject#_bounds
     * @protected
     */
    protected _bounds: PIXI.Bounds;
    /**
     * The original, cached mask of the object.
     *
     * @member {PIXI.Graphics|PIXI.Sprite|null} PIXI.DisplayObject#_mask
     * @protected
     */
    protected _mask: PIXI.Graphics | PIXI.Sprite | null;
    /**
     * If the object has been destroyed via destroy(). If true, it should not be used.
     *
     * @member {boolean} PIXI.DisplayObject#_destroyed
     * @protected
     */
    protected _destroyed: boolean;
    /**
     * used to fast check if a sprite is.. a sprite!
     * @member {boolean} PIXI.DisplayObject#isSprite
     */
    isSprite: boolean;
    /**
     * Does any other displayObject use this object as a mask?
     * @member {boolean} PIXI.DisplayObject#isMask
     */
    isMask: boolean;
    /**
     * @protected
     * @member {PIXI.DisplayObject}
     */
    protected _tempDisplayObjectParent: PIXI.DisplayObject;

    /**
     * Recursively updates transform of all objects from the root to this one
     * internal function for toLocal()
     */
    _recursivePostUpdateTransform(): void;

    /**
     * Retrieves the bounds of the displayObject as a rectangle object.
     *
     * @param {boolean} [skipUpdate] - Setting to `true` will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @param {PIXI.Rectangle} [rect] - Optional rectangle to store the result of the bounds calculation.
     * @return {PIXI.Rectangle} The rectangular bounding area.
     */
    getBounds(skipUpdate?: boolean, rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Retrieves the local bounds of the displayObject as a rectangle object.
     *
     * @param {PIXI.Rectangle} [rect] - Optional rectangle to store the result of the bounds calculation.
     * @return {PIXI.Rectangle} The rectangular bounding area.
     */
    getLocalBounds(rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Calculates the global position of the display object.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform.
     * @return {PIXI.IPoint} A point object representing the position of this object.
     */
    toGlobal(position: PIXI.IPoint, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Calculates the local position of the display object relative to another point.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.DisplayObject} [from] - The DisplayObject to calculate the global position from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform
     * @return {PIXI.IPoint} A point object representing the position of this object
     */
    toLocal(position: PIXI.IPoint, from?: PIXI.DisplayObject, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Set the parent Container of this DisplayObject.
     *
     * @param {PIXI.Container} container - The Container to add this DisplayObject to.
     * @return {PIXI.Container} The Container that this DisplayObject was added to.
     */
    setParent(container: PIXI.Container): PIXI.Container;

    /**
     * Convenience function to set the position, scale, skew and pivot at once.
     *
     * @param {number} [x=0] - The X position
     * @param {number} [y=0] - The Y position
     * @param {number} [scaleX=1] - The X scale value
     * @param {number} [scaleY=1] - The Y scale value
     * @param {number} [rotation=0] - The rotation
     * @param {number} [skewX=0] - The X skew value
     * @param {number} [skewY=0] - The Y skew value
     * @param {number} [pivotX=0] - The X pivot value
     * @param {number} [pivotY=0] - The Y pivot value
     * @return {PIXI.DisplayObject} The DisplayObject instance
     */
    setTransform(
      x?: number,
      y?: number,
      scaleX?: number,
      scaleY?: number,
      rotation?: number,
      skewX?: number,
      skewY?: number,
      pivotX?: number,
      pivotY?: number,
    ): PIXI.DisplayObject;

    /**
     * The position of the displayObject on the x axis relative to the local coordinates of the parent.
     * An alias to position.x
     *
     * @member {number}
     */
    x: number;
    /**
     * The position of the displayObject on the y axis relative to the local coordinates of the parent.
     * An alias to position.y
     *
     * @member {number}
     */
    y: number;
    /**
     * Current transform of the object based on world (parent) factors.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly worldTransform: PIXI.Matrix;
    /**
     * Current transform of the object based on local factors: position, scale, other stuff.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly localTransform: PIXI.Matrix;
    /**
     * The coordinate of the object relative to the local coordinates of the parent.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    position: PIXI.IPoint;
    /**
     * The scale factor of the object.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    scale: PIXI.IPoint;
    /**
     * The pivot point of the displayObject that it rotates around.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    pivot: PIXI.IPoint;
    /**
     * The skew factor for the object in radians.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.ObservablePoint}
     */
    skew: PIXI.ObservablePoint;
    /**
     * The rotation of the object in radians.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    rotation: number;
    /**
     * The angle of the object in degrees.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    angle: number;
    /**
     * The zIndex of the displayObject.
     * If a container has the sortableChildren property set to true, children will be automatically
     * sorted by zIndex value; a higher value will mean it will be moved towards the end of the array,
     * and thus rendered on top of other displayObjects within the same container.
     *
     * @member {number}
     */
    zIndex: number;
    /**
     * Indicates if the object is globally visible.
     *
     * @member {boolean}
     * @readonly
     */
    readonly worldVisible: boolean;
    /**
     * Sets a mask for the displayObject. A mask is an object that limits the visibility of an
     * object to the shape of the mask applied to it. In PixiJS a regular mask must be a
     * {@link PIXI.Graphics} or a {@link PIXI.Sprite} object. This allows for much faster masking in canvas as it
     * utilities shape clipping. To remove a mask, set this property to `null`.
     *
     * For sprite mask both alpha and red channel are used. Black mask is the same as transparent mask.
     * @example
     * const graphics = new PIXI.Graphics();
     * graphics.beginFill(0xFF3300);
     * graphics.drawRect(50, 250, 100, 100);
     * graphics.endFill();
     *
     * const sprite = new PIXI.Sprite(texture);
     * sprite.mask = graphics;
     * @todo At the moment, PIXI.CanvasRenderer doesn't support PIXI.Sprite as mask.
     *
     * @member {PIXI.Container|PIXI.MaskData}
     */
    mask: PIXI.Container | PIXI.MaskData;

    /**
     * DisplayObject default updateTransform, does not update children of container.
     * Will crash if there's no parent element.
     *
     * @memberof PIXI.DisplayObject#
     * @function displayObjectUpdateTransform
     */
    displayObjectUpdateTransform(): void;

    /**
     * Enable interaction events for the DisplayObject. Touch, pointer and mouse
     * events will not be emitted unless `interactive` is set to `true`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.on('tap', (event) => {
     *    //handle event
     * });
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    interactive: boolean;
    /**
     * Interaction shape. Children will be hit first, then this shape will be checked.
     * Setting this will cause this shape to be checked in hit tests rather than the displayObject's bounds.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.hitArea = new PIXI.Rectangle(0, 0, 100, 100);
     * @member {PIXI.IHitArea}
     * @memberof PIXI.DisplayObject#
     */
    hitArea: PIXI.IHitArea;
    /**
     * If enabled, the mouse cursor use the pointer behavior when hovered over the displayObject if it is interactive
     * Setting this changes the 'cursor' property to `'pointer'`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.buttonMode = true;
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    buttonMode: boolean;
    /**
     * This defines what cursor mode is used when the mouse cursor
     * is hovered over the displayObject.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.cursor = 'wait';
     * @see https://developer.mozilla.org/en/docs/Web/CSS/cursor
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    cursor: string;
    /**
     * Set this to true if you want this display object to be cached as a bitmap.
     * This basically takes a snap shot of the display object as it is at that moment. It can
     * provide a performance benefit for complex static displayObjects.
     * To remove simply set this property to `false`
     *
     * IMPORTANT GOTCHA - Make sure that all your textures are preloaded BEFORE setting this property to true
     * as it will take a snapshot of what is currently there. If the textures have not loaded then they will not appear.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    cacheAsBitmap: boolean;
    /**
     * The instance name of the object.
     *
     * @memberof PIXI.DisplayObject#
     * @member {string} name
     */
    name: string;

    /**
     * Returns the global position of the displayObject. Does not depend on object scale, rotation and pivot.
     *
     * @method getGlobalPosition
     * @memberof PIXI.DisplayObject#
     * @param {PIXI.Point} [point=new PIXI.Point()] - The point to write the global value to.
     * @param {boolean} [skipUpdate=false] - Setting to true will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @return {PIXI.Point} The updated point.
     */
    getGlobalPosition(point?: PIXI.Point, skipUpdate?: boolean): PIXI.Point;
  }

  /**
   * The Simple Mesh class mimics Mesh in PixiJS v4, providing easy-to-use constructor arguments.
   * For more robust customization, use {@link PIXI.Mesh}.
   *
   * @class
   * @extends PIXI.Mesh
   * @memberof PIXI
   */
  class SimpleMesh extends PIXI.Mesh {
    constructor(
      texture?: PIXI.Texture,
      vertices?: Float32Array,
      uvs?: Float32Array,
      indices?: Uint16Array,
      drawMode?: number,
    );

    /**
     * upload vertices buffer each frame
     * @member {boolean} PIXI.SimpleMesh#autoUpdate
     */
    autoUpdate: boolean;
    /**
     * Collection of vertices data.
     * @member {Float32Array}
     */
    vertices: Float32Array;
    /**
     * Includes vertex positions, face indices, normals, colors, UVs, and
     * custom attributes within buffers, reducing the cost of passing all
     * this data to the GPU. Can be shared between multiple Mesh objects.
     * @member {PIXI.Geometry} PIXI.Mesh#geometry
     * @readonly
     */
    readonly geometry: PIXI.Geometry;
    /**
     * Represents the vertex and fragment shaders that processes the geometry and runs on the GPU.
     * Can be shared between multiple Mesh objects.
     * @member {PIXI.Shader|PIXI.MeshMaterial} PIXI.Mesh#shader
     */
    shader: PIXI.Shader | PIXI.MeshMaterial;
    /**
     * Represents the WebGL state the Mesh required to render, excludes shader and geometry. E.g.,
     * blend mode, culling, depth testing, direction of rendering triangles, backface, etc.
     * @member {PIXI.State} PIXI.Mesh#state
     */
    state: PIXI.State;
    /**
     * The way the Mesh should be drawn, can be any of the {@link PIXI.DRAW_MODES} constants.
     *
     * @member {number} PIXI.Mesh#drawMode
     * @see PIXI.DRAW_MODES
     */
    drawMode: number;
    /**
     * Typically the index of the IndexBuffer where to start drawing.
     * @member {number} PIXI.Mesh#start
     * @default 0
     */
    start: number;
    /**
     * How much of the geometry to draw, by default `0` renders everything.
     * @member {number} PIXI.Mesh#size
     * @default 0
     */
    size: number;
    /**
     * To change mesh uv's, change its uvBuffer data and increment its _updateID.
     * @member {PIXI.Buffer}
     * @readonly
     */
    readonly uvBuffer: PIXI.Buffer;
    /**
     * To change mesh vertices, change its uvBuffer data and increment its _updateID.
     * Incrementing _updateID is optional because most of Mesh objects do it anyway.
     * @member {PIXI.Buffer}
     * @readonly
     */
    readonly verticesBuffer: PIXI.Buffer;
    /**
     * Alias for {@link PIXI.Mesh#shader}.
     * @member {PIXI.Shader|PIXI.MeshMaterial}
     */
    material: PIXI.Shader | PIXI.MeshMaterial;
    /**
     * The blend mode to be applied to the Mesh. Apply a value of
     * `PIXI.BLEND_MODES.NORMAL` to reset the blend mode.
     *
     * @member {number}
     * @default PIXI.BLEND_MODES.NORMAL;
     * @see PIXI.BLEND_MODES
     */
    blendMode: number;
    /**
     * If true PixiJS will Math.floor() x/y values when rendering, stopping pixel interpolation.
     * Advantages can include sharper image quality (like text) and faster rendering on canvas.
     * The main disadvantage is movement of objects may appear less smooth.
     * To set the global default, change {@link PIXI.settings.ROUND_PIXELS}
     *
     * @member {boolean}
     * @default false
     */
    roundPixels: boolean;
    /**
     * The multiply tint applied to the Mesh. This is a hex value. A value of
     * `0xFFFFFF` will remove any tint effect.
     *
     * @member {number}
     * @default 0xFFFFFF
     */
    tint: number;
    /**
     * The texture that the Mesh uses.
     *
     * @member {PIXI.Texture}
     */
    texture: PIXI.Texture;

    /**
     * Standard renderer draw.
     * @protected
     * @param {PIXI.Renderer} renderer - Instance to renderer.
     */
    protected _render(renderer: PIXI.Renderer): void;

    /**
     * Standard non-batching way of rendering.
     * @protected
     * @param {PIXI.Renderer} renderer - Instance to renderer.
     */
    protected _renderDefault(renderer: PIXI.Renderer): void;

    /**
     * Rendering by using the Batch system.
     * @protected
     * @param {PIXI.Renderer} renderer - Instance to renderer.
     */
    protected _renderToBatch(renderer: PIXI.Renderer): void;

    /**
     * Updates vertexData field based on transform and vertices
     */
    calculateVertices(): void;

    /**
     * Updates uv field based on from geometry uv's or batchUvs
     */
    calculateUvs(): void;

    /**
     * Updates the bounds of the mesh as a rectangle. The bounds calculation takes the worldTransform into account.
     * there must be a aVertexPosition attribute present in the geometry for bounds to be calculated correctly.
     *
     * @protected
     */
    protected _calculateBounds(): void;

    /**
     * Tests if a point is inside this mesh. Works only for PIXI.DRAW_MODES.TRIANGLES.
     *
     * @param {PIXI.Point} point the point to test
     * @return {boolean} the result of the test
     */
    containsPoint(point: PIXI.Point): boolean;

    /**
     * Destroys the Mesh object.
     *
     * @param {object|boolean} [options] - Options parameter. A boolean will act as if all
     *  options have been set to that value
     * @param {boolean} [options.children=false] - if set to true, all the children will have
     *  their destroy method called as well. 'options' will be passed on to those calls.
     */
    destroy(options?: { children?: boolean }): void;

    /**
     * The array of children of this container.
     *
     * @member {PIXI.DisplayObject[]} PIXI.Container#children
     * @readonly
     */
    readonly children: PIXI.DisplayObject[];
    /**
     * If set to true, the container will sort its children by zIndex value
     * when updateTransform() is called, or manually if sortChildren() is called.
     *
     * This actually changes the order of elements in the array, so should be treated
     * as a basic solution that is not performant compared to other solutions,
     * such as @link https://github.com/pixijs/pixi-display
     *
     * Also be aware of that this may not work nicely with the addChildAt() function,
     * as the zIndex sorting may cause the child to automatically sorted to another position.
     *
     * @see PIXI.settings.SORTABLE_CHILDREN
     *
     * @member {boolean} PIXI.Container#sortableChildren
     */
    sortableChildren: boolean;
    /**
     * Should children be sorted by zIndex at the next updateTransform call.
     * Will get automatically set to true if a new child is added, or if a child's zIndex changes.
     *
     * @member {boolean} PIXI.Container#sortDirty
     */
    sortDirty: boolean;

    /**
     * Overridable method that can be used by Container subclasses whenever the children array is modified
     *
     * @protected
     */
    protected onChildrenChange(): void;

    /**
     * Adds one or more children to the container.
     *
     * Multiple items can be added like so: `myContainer.addChild(thingOne, thingTwo, thingThree)`
     *
     * @param {...PIXI.DisplayObject} child - The DisplayObject(s) to add to the container
     * @return {PIXI.DisplayObject} The first child that was added.
     */
    addChild<TChildren extends PIXI.DisplayObject[]>(...child: TChildren): TChildren[0];

    /**
     * Adds a child to the container at a specified index. If the index is out of bounds an error will be thrown
     *
     * @param {PIXI.DisplayObject} child - The child to add
     * @param {number} index - The index to place the child in
     * @return {PIXI.DisplayObject} The child that was added.
     */
    addChildAt<T extends PIXI.DisplayObject>(child: T, index: number): T;

    /**
     * Swaps the position of 2 Display Objects within this container.
     *
     * @param {PIXI.DisplayObject} child - First display object to swap
     * @param {PIXI.DisplayObject} child2 - Second display object to swap
     */
    swapChildren(child: PIXI.DisplayObject, child2: PIXI.DisplayObject): void;

    /**
     * Returns the index position of a child DisplayObject instance
     *
     * @param {PIXI.DisplayObject} child - The DisplayObject instance to identify
     * @return {number} The index position of the child display object to identify
     */
    getChildIndex(child: PIXI.DisplayObject): number;

    /**
     * Changes the position of an existing child in the display object container
     *
     * @param {PIXI.DisplayObject} child - The child DisplayObject instance for which you want to change the index number
     * @param {number} index - The resulting index number for the child display object
     */
    setChildIndex(child: PIXI.DisplayObject, index: number): void;

    /**
     * Returns the child at the specified index
     *
     * @param {number} index - The index to get the child at
     * @return {PIXI.DisplayObject} The child at the given index, if any.
     */
    getChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes one or more children from the container.
     *
     * @param {...PIXI.DisplayObject} child - The DisplayObject(s) to remove
     * @return {PIXI.DisplayObject} The first child that was removed.
     */
    removeChild<TChildren extends PIXI.DisplayObject[]>(...child: TChildren): TChildren[0];

    /**
     * Removes a child from the specified index position.
     *
     * @param {number} index - The index to get the child from
     * @return {PIXI.DisplayObject} The child that was removed.
     */
    removeChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes all children from this container that are within the begin and end indexes.
     *
     * @param {number} [beginIndex=0] - The beginning position.
     * @param {number} [endIndex=this.children.length] - The ending position. Default value is size of the container.
     * @returns {PIXI.DisplayObject[]} List of removed children
     */
    removeChildren(beginIndex?: number, endIndex?: number): PIXI.DisplayObject[];

    /**
     * Sorts children by zIndex. Previous order is mantained for 2 children with the same zIndex.
     */
    sortChildren(): void;

    /**
     * Updates the transform on all children of this container for rendering
     */
    updateTransform(): void;

    /**
     * Recalculates the bounds of the container.
     *
     */
    calculateBounds(): void;

    /**
     * Renders the object using the WebGL renderer
     *
     * @param {PIXI.Renderer} renderer - The renderer
     */
    render(renderer: PIXI.Renderer): void;

    /**
     * Render the object using the WebGL renderer and advanced features.
     *
     * @protected
     * @param {PIXI.Renderer} renderer - The renderer
     */
    protected renderAdvanced(renderer: PIXI.Renderer): void;

    /**
     * The width of the Container, setting this will actually modify the scale to achieve the value set
     *
     * @member {number}
     */
    width: number;
    /**
     * The height of the Container, setting this will actually modify the scale to achieve the value set
     *
     * @member {number}
     */
    height: number;
    /**
     * Determines if the children to the displayObject can be clicked/touched
     * Setting this to false allows PixiJS to bypass a recursive `hitTest` function
     *
     * @member {boolean}
     * @memberof PIXI.Container#
     */
    interactiveChildren: boolean;

    /**
     * Returns the display object in the container.
     *
     * @method getChildByName
     * @memberof PIXI.Container#
     * @param {string} name - Instance name.
     * @return {PIXI.DisplayObject} The child with the specified name.
     */
    getChildByName(name: string): PIXI.DisplayObject;

    /**
     *  Flag for if the object is accessible. If true AccessibilityManager will overlay a
     *   shadow div with attributes set
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    accessible: boolean;
    /**
     * Sets the title attribute of the shadow div
     * If accessibleTitle AND accessibleHint has not been this will default to 'displayObject [tabIndex]'
     *
     * @member {?string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleTitle: string;
    /**
     * Sets the aria-label attribute of the shadow div
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleHint: string;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleActive: boolean;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleDiv: boolean;
    /**
     * Specify the type of div the accessible layer is. Screen readers treat the element differently
     * depending on this type. Defaults to button.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'button'
     */
    accessibleType: string;
    /**
     * Specify the pointer-events the accessible div will use
     * Defaults to auto.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'auto'
     */
    accessiblePointerEvents: string;
    /**
     * Setting to false will prevent any children inside this container to
     * be accessible. Defaults to true.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @default true
     */
    accessibleChildren: boolean;
    /**
     * World transform and local transform of this object.
     * This will become read-only later, please do not assign anything there unless you know what are you doing.
     *
     * @member {PIXI.Transform} PIXI.DisplayObject#transform
     */
    transform: PIXI.Transform;
    /**
     * The opacity of the object.
     *
     * @member {number} PIXI.DisplayObject#alpha
     */
    alpha: number;
    /**
     * The visibility of the object. If false the object will not be drawn, and
     * the updateTransform function will not be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds or call updateTransform manually.
     *
     * @member {boolean} PIXI.DisplayObject#visible
     */
    visible: boolean;
    /**
     * Can this object be rendered, if false the object will not be drawn but the updateTransform
     * methods will still be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds manually.
     *
     * @member {boolean} PIXI.DisplayObject#renderable
     */
    renderable: boolean;
    /**
     * The display object container that contains this display object.
     *
     * @member {PIXI.Container} PIXI.DisplayObject#parent
     * @readonly
     */
    readonly parent: PIXI.Container;
    /**
     * The multiplied alpha of the displayObject.
     *
     * @member {number} PIXI.DisplayObject#worldAlpha
     * @readonly
     */
    readonly worldAlpha: number;
    /**
     * Which index in the children array the display component was before the previous zIndex sort.
     * Used by containers to help sort objects with the same zIndex, by using previous array index as the decider.
     *
     * @member {number} PIXI.DisplayObject#_lastSortedIndex
     * @protected
     */
    protected _lastSortedIndex: number;
    /**
     * The zIndex of the displayObject.
     * A higher value will mean it will be rendered on top of other displayObjects within the same container.
     *
     * @member {number} PIXI.DisplayObject#_zIndex
     * @protected
     */
    protected _zIndex: number;
    /**
     * The area the filter is applied to. This is used as more of an optimization
     * rather than figuring out the dimensions of the displayObject each frame you can set this rectangle.
     *
     * Also works as an interaction mask.
     *
     * @member {?PIXI.Rectangle} PIXI.DisplayObject#filterArea
     */
    filterArea: PIXI.Rectangle;
    /**
     * Sets the filters for the displayObject.
     * * IMPORTANT: This is a WebGL only feature and will be ignored by the canvas renderer.
     * To remove filters simply set this property to `'null'`.
     *
     * @member {?PIXI.Filter[]} PIXI.DisplayObject#filters
     */
    filters: PIXI.Filter[];
    /**
     * The bounds object, this is used to calculate and store the bounds of the displayObject.
     *
     * @member {PIXI.Bounds} PIXI.DisplayObject#_bounds
     * @protected
     */
    protected _bounds: PIXI.Bounds;
    /**
     * The original, cached mask of the object.
     *
     * @member {PIXI.Graphics|PIXI.Sprite|null} PIXI.DisplayObject#_mask
     * @protected
     */
    protected _mask: PIXI.Graphics | PIXI.Sprite | null;
    /**
     * If the object has been destroyed via destroy(). If true, it should not be used.
     *
     * @member {boolean} PIXI.DisplayObject#_destroyed
     * @protected
     */
    protected _destroyed: boolean;
    /**
     * used to fast check if a sprite is.. a sprite!
     * @member {boolean} PIXI.DisplayObject#isSprite
     */
    isSprite: boolean;
    /**
     * Does any other displayObject use this object as a mask?
     * @member {boolean} PIXI.DisplayObject#isMask
     */
    isMask: boolean;
    /**
     * @protected
     * @member {PIXI.DisplayObject}
     */
    protected _tempDisplayObjectParent: PIXI.DisplayObject;

    /**
     * Recursively updates transform of all objects from the root to this one
     * internal function for toLocal()
     */
    _recursivePostUpdateTransform(): void;

    /**
     * Retrieves the bounds of the displayObject as a rectangle object.
     *
     * @param {boolean} [skipUpdate] - Setting to `true` will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @param {PIXI.Rectangle} [rect] - Optional rectangle to store the result of the bounds calculation.
     * @return {PIXI.Rectangle} The rectangular bounding area.
     */
    getBounds(skipUpdate?: boolean, rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Retrieves the local bounds of the displayObject as a rectangle object.
     *
     * @param {PIXI.Rectangle} [rect] - Optional rectangle to store the result of the bounds calculation.
     * @return {PIXI.Rectangle} The rectangular bounding area.
     */
    getLocalBounds(rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Calculates the global position of the display object.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform.
     * @return {PIXI.IPoint} A point object representing the position of this object.
     */
    toGlobal(position: PIXI.IPoint, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Calculates the local position of the display object relative to another point.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.DisplayObject} [from] - The DisplayObject to calculate the global position from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform
     * @return {PIXI.IPoint} A point object representing the position of this object
     */
    toLocal(position: PIXI.IPoint, from?: PIXI.DisplayObject, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Set the parent Container of this DisplayObject.
     *
     * @param {PIXI.Container} container - The Container to add this DisplayObject to.
     * @return {PIXI.Container} The Container that this DisplayObject was added to.
     */
    setParent(container: PIXI.Container): PIXI.Container;

    /**
     * Convenience function to set the position, scale, skew and pivot at once.
     *
     * @param {number} [x=0] - The X position
     * @param {number} [y=0] - The Y position
     * @param {number} [scaleX=1] - The X scale value
     * @param {number} [scaleY=1] - The Y scale value
     * @param {number} [rotation=0] - The rotation
     * @param {number} [skewX=0] - The X skew value
     * @param {number} [skewY=0] - The Y skew value
     * @param {number} [pivotX=0] - The X pivot value
     * @param {number} [pivotY=0] - The Y pivot value
     * @return {PIXI.DisplayObject} The DisplayObject instance
     */
    setTransform(
      x?: number,
      y?: number,
      scaleX?: number,
      scaleY?: number,
      rotation?: number,
      skewX?: number,
      skewY?: number,
      pivotX?: number,
      pivotY?: number,
    ): PIXI.DisplayObject;

    /**
     * The position of the displayObject on the x axis relative to the local coordinates of the parent.
     * An alias to position.x
     *
     * @member {number}
     */
    x: number;
    /**
     * The position of the displayObject on the y axis relative to the local coordinates of the parent.
     * An alias to position.y
     *
     * @member {number}
     */
    y: number;
    /**
     * Current transform of the object based on world (parent) factors.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly worldTransform: PIXI.Matrix;
    /**
     * Current transform of the object based on local factors: position, scale, other stuff.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly localTransform: PIXI.Matrix;
    /**
     * The coordinate of the object relative to the local coordinates of the parent.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    position: PIXI.IPoint;
    /**
     * The scale factor of the object.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    scale: PIXI.IPoint;
    /**
     * The pivot point of the displayObject that it rotates around.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    pivot: PIXI.IPoint;
    /**
     * The skew factor for the object in radians.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.ObservablePoint}
     */
    skew: PIXI.ObservablePoint;
    /**
     * The rotation of the object in radians.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    rotation: number;
    /**
     * The angle of the object in degrees.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    angle: number;
    /**
     * The zIndex of the displayObject.
     * If a container has the sortableChildren property set to true, children will be automatically
     * sorted by zIndex value; a higher value will mean it will be moved towards the end of the array,
     * and thus rendered on top of other displayObjects within the same container.
     *
     * @member {number}
     */
    zIndex: number;
    /**
     * Indicates if the object is globally visible.
     *
     * @member {boolean}
     * @readonly
     */
    readonly worldVisible: boolean;
    /**
     * Sets a mask for the displayObject. A mask is an object that limits the visibility of an
     * object to the shape of the mask applied to it. In PixiJS a regular mask must be a
     * {@link PIXI.Graphics} or a {@link PIXI.Sprite} object. This allows for much faster masking in canvas as it
     * utilities shape clipping. To remove a mask, set this property to `null`.
     *
     * For sprite mask both alpha and red channel are used. Black mask is the same as transparent mask.
     * @example
     * const graphics = new PIXI.Graphics();
     * graphics.beginFill(0xFF3300);
     * graphics.drawRect(50, 250, 100, 100);
     * graphics.endFill();
     *
     * const sprite = new PIXI.Sprite(texture);
     * sprite.mask = graphics;
     * @todo At the moment, PIXI.CanvasRenderer doesn't support PIXI.Sprite as mask.
     *
     * @member {PIXI.Container|PIXI.MaskData}
     */
    mask: PIXI.Container | PIXI.MaskData;

    /**
     * DisplayObject default updateTransform, does not update children of container.
     * Will crash if there's no parent element.
     *
     * @memberof PIXI.DisplayObject#
     * @function displayObjectUpdateTransform
     */
    displayObjectUpdateTransform(): void;

    /**
     * Enable interaction events for the DisplayObject. Touch, pointer and mouse
     * events will not be emitted unless `interactive` is set to `true`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.on('tap', (event) => {
     *    //handle event
     * });
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    interactive: boolean;
    /**
     * Interaction shape. Children will be hit first, then this shape will be checked.
     * Setting this will cause this shape to be checked in hit tests rather than the displayObject's bounds.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.hitArea = new PIXI.Rectangle(0, 0, 100, 100);
     * @member {PIXI.IHitArea}
     * @memberof PIXI.DisplayObject#
     */
    hitArea: PIXI.IHitArea;
    /**
     * If enabled, the mouse cursor use the pointer behavior when hovered over the displayObject if it is interactive
     * Setting this changes the 'cursor' property to `'pointer'`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.buttonMode = true;
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    buttonMode: boolean;
    /**
     * This defines what cursor mode is used when the mouse cursor
     * is hovered over the displayObject.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.cursor = 'wait';
     * @see https://developer.mozilla.org/en/docs/Web/CSS/cursor
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    cursor: string;
    /**
     * Set this to true if you want this display object to be cached as a bitmap.
     * This basically takes a snap shot of the display object as it is at that moment. It can
     * provide a performance benefit for complex static displayObjects.
     * To remove simply set this property to `false`
     *
     * IMPORTANT GOTCHA - Make sure that all your textures are preloaded BEFORE setting this property to true
     * as it will take a snapshot of what is currently there. If the textures have not loaded then they will not appear.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    cacheAsBitmap: boolean;
    /**
     * The instance name of the object.
     *
     * @memberof PIXI.DisplayObject#
     * @member {string} name
     */
    name: string;

    /**
     * Returns the global position of the displayObject. Does not depend on object scale, rotation and pivot.
     *
     * @method getGlobalPosition
     * @memberof PIXI.DisplayObject#
     * @param {PIXI.Point} [point=new PIXI.Point()] - The point to write the global value to.
     * @param {boolean} [skipUpdate=false] - Setting to true will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @return {PIXI.Point} The updated point.
     */
    getGlobalPosition(point?: PIXI.Point, skipUpdate?: boolean): PIXI.Point;
  }

  /**
   * The SimplePlane allows you to draw a texture across several points and then manipulate these points
   *
   *```js
   * for (let i = 0; i < 20; i++) {
   *     points.push(new PIXI.Point(i * 50, 0));
   * };
   * let SimplePlane = new PIXI.SimplePlane(PIXI.Texture.from("snake.png"), points);
   *  ```
   *
   * @class
   * @extends PIXI.Mesh
   * @memberof PIXI
   *
   */
  class SimplePlane extends PIXI.Mesh {
    constructor(texture: PIXI.Texture, verticesX: number, verticesY: number);

    /**
     * Method used for overrides, to do something in case texture frame was changed.
     * Meshes based on plane can override it and change more details based on texture.
     */
    textureUpdated(): void;

    /**
     * Includes vertex positions, face indices, normals, colors, UVs, and
     * custom attributes within buffers, reducing the cost of passing all
     * this data to the GPU. Can be shared between multiple Mesh objects.
     * @member {PIXI.Geometry} PIXI.Mesh#geometry
     * @readonly
     */
    readonly geometry: PIXI.Geometry;
    /**
     * Represents the vertex and fragment shaders that processes the geometry and runs on the GPU.
     * Can be shared between multiple Mesh objects.
     * @member {PIXI.Shader|PIXI.MeshMaterial} PIXI.Mesh#shader
     */
    shader: PIXI.Shader | PIXI.MeshMaterial;
    /**
     * Represents the WebGL state the Mesh required to render, excludes shader and geometry. E.g.,
     * blend mode, culling, depth testing, direction of rendering triangles, backface, etc.
     * @member {PIXI.State} PIXI.Mesh#state
     */
    state: PIXI.State;
    /**
     * The way the Mesh should be drawn, can be any of the {@link PIXI.DRAW_MODES} constants.
     *
     * @member {number} PIXI.Mesh#drawMode
     * @see PIXI.DRAW_MODES
     */
    drawMode: number;
    /**
     * Typically the index of the IndexBuffer where to start drawing.
     * @member {number} PIXI.Mesh#start
     * @default 0
     */
    start: number;
    /**
     * How much of the geometry to draw, by default `0` renders everything.
     * @member {number} PIXI.Mesh#size
     * @default 0
     */
    size: number;
    /**
     * To change mesh uv's, change its uvBuffer data and increment its _updateID.
     * @member {PIXI.Buffer}
     * @readonly
     */
    readonly uvBuffer: PIXI.Buffer;
    /**
     * To change mesh vertices, change its uvBuffer data and increment its _updateID.
     * Incrementing _updateID is optional because most of Mesh objects do it anyway.
     * @member {PIXI.Buffer}
     * @readonly
     */
    readonly verticesBuffer: PIXI.Buffer;
    /**
     * Alias for {@link PIXI.Mesh#shader}.
     * @member {PIXI.Shader|PIXI.MeshMaterial}
     */
    material: PIXI.Shader | PIXI.MeshMaterial;
    /**
     * The blend mode to be applied to the Mesh. Apply a value of
     * `PIXI.BLEND_MODES.NORMAL` to reset the blend mode.
     *
     * @member {number}
     * @default PIXI.BLEND_MODES.NORMAL;
     * @see PIXI.BLEND_MODES
     */
    blendMode: number;
    /**
     * If true PixiJS will Math.floor() x/y values when rendering, stopping pixel interpolation.
     * Advantages can include sharper image quality (like text) and faster rendering on canvas.
     * The main disadvantage is movement of objects may appear less smooth.
     * To set the global default, change {@link PIXI.settings.ROUND_PIXELS}
     *
     * @member {boolean}
     * @default false
     */
    roundPixels: boolean;
    /**
     * The multiply tint applied to the Mesh. This is a hex value. A value of
     * `0xFFFFFF` will remove any tint effect.
     *
     * @member {number}
     * @default 0xFFFFFF
     */
    tint: number;
    /**
     * The texture that the Mesh uses.
     *
     * @member {PIXI.Texture}
     */
    texture: PIXI.Texture;

    /**
     * Standard renderer draw.
     * @protected
     * @param {PIXI.Renderer} renderer - Instance to renderer.
     */
    protected _render(renderer: PIXI.Renderer): void;

    /**
     * Standard non-batching way of rendering.
     * @protected
     * @param {PIXI.Renderer} renderer - Instance to renderer.
     */
    protected _renderDefault(renderer: PIXI.Renderer): void;

    /**
     * Rendering by using the Batch system.
     * @protected
     * @param {PIXI.Renderer} renderer - Instance to renderer.
     */
    protected _renderToBatch(renderer: PIXI.Renderer): void;

    /**
     * Updates vertexData field based on transform and vertices
     */
    calculateVertices(): void;

    /**
     * Updates uv field based on from geometry uv's or batchUvs
     */
    calculateUvs(): void;

    /**
     * Updates the bounds of the mesh as a rectangle. The bounds calculation takes the worldTransform into account.
     * there must be a aVertexPosition attribute present in the geometry for bounds to be calculated correctly.
     *
     * @protected
     */
    protected _calculateBounds(): void;

    /**
     * Tests if a point is inside this mesh. Works only for PIXI.DRAW_MODES.TRIANGLES.
     *
     * @param {PIXI.Point} point the point to test
     * @return {boolean} the result of the test
     */
    containsPoint(point: PIXI.Point): boolean;

    /**
     * Destroys the Mesh object.
     *
     * @param {object|boolean} [options] - Options parameter. A boolean will act as if all
     *  options have been set to that value
     * @param {boolean} [options.children=false] - if set to true, all the children will have
     *  their destroy method called as well. 'options' will be passed on to those calls.
     */
    destroy(options?: { children?: boolean }): void;

    /**
     * The array of children of this container.
     *
     * @member {PIXI.DisplayObject[]} PIXI.Container#children
     * @readonly
     */
    readonly children: PIXI.DisplayObject[];
    /**
     * If set to true, the container will sort its children by zIndex value
     * when updateTransform() is called, or manually if sortChildren() is called.
     *
     * This actually changes the order of elements in the array, so should be treated
     * as a basic solution that is not performant compared to other solutions,
     * such as @link https://github.com/pixijs/pixi-display
     *
     * Also be aware of that this may not work nicely with the addChildAt() function,
     * as the zIndex sorting may cause the child to automatically sorted to another position.
     *
     * @see PIXI.settings.SORTABLE_CHILDREN
     *
     * @member {boolean} PIXI.Container#sortableChildren
     */
    sortableChildren: boolean;
    /**
     * Should children be sorted by zIndex at the next updateTransform call.
     * Will get automatically set to true if a new child is added, or if a child's zIndex changes.
     *
     * @member {boolean} PIXI.Container#sortDirty
     */
    sortDirty: boolean;

    /**
     * Overridable method that can be used by Container subclasses whenever the children array is modified
     *
     * @protected
     */
    protected onChildrenChange(): void;

    /**
     * Adds one or more children to the container.
     *
     * Multiple items can be added like so: `myContainer.addChild(thingOne, thingTwo, thingThree)`
     *
     * @param {...PIXI.DisplayObject} child - The DisplayObject(s) to add to the container
     * @return {PIXI.DisplayObject} The first child that was added.
     */
    addChild<TChildren extends PIXI.DisplayObject[]>(...child: TChildren): TChildren[0];

    /**
     * Adds a child to the container at a specified index. If the index is out of bounds an error will be thrown
     *
     * @param {PIXI.DisplayObject} child - The child to add
     * @param {number} index - The index to place the child in
     * @return {PIXI.DisplayObject} The child that was added.
     */
    addChildAt<T extends PIXI.DisplayObject>(child: T, index: number): T;

    /**
     * Swaps the position of 2 Display Objects within this container.
     *
     * @param {PIXI.DisplayObject} child - First display object to swap
     * @param {PIXI.DisplayObject} child2 - Second display object to swap
     */
    swapChildren(child: PIXI.DisplayObject, child2: PIXI.DisplayObject): void;

    /**
     * Returns the index position of a child DisplayObject instance
     *
     * @param {PIXI.DisplayObject} child - The DisplayObject instance to identify
     * @return {number} The index position of the child display object to identify
     */
    getChildIndex(child: PIXI.DisplayObject): number;

    /**
     * Changes the position of an existing child in the display object container
     *
     * @param {PIXI.DisplayObject} child - The child DisplayObject instance for which you want to change the index number
     * @param {number} index - The resulting index number for the child display object
     */
    setChildIndex(child: PIXI.DisplayObject, index: number): void;

    /**
     * Returns the child at the specified index
     *
     * @param {number} index - The index to get the child at
     * @return {PIXI.DisplayObject} The child at the given index, if any.
     */
    getChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes one or more children from the container.
     *
     * @param {...PIXI.DisplayObject} child - The DisplayObject(s) to remove
     * @return {PIXI.DisplayObject} The first child that was removed.
     */
    removeChild<TChildren extends PIXI.DisplayObject[]>(...child: TChildren): TChildren[0];

    /**
     * Removes a child from the specified index position.
     *
     * @param {number} index - The index to get the child from
     * @return {PIXI.DisplayObject} The child that was removed.
     */
    removeChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes all children from this container that are within the begin and end indexes.
     *
     * @param {number} [beginIndex=0] - The beginning position.
     * @param {number} [endIndex=this.children.length] - The ending position. Default value is size of the container.
     * @returns {PIXI.DisplayObject[]} List of removed children
     */
    removeChildren(beginIndex?: number, endIndex?: number): PIXI.DisplayObject[];

    /**
     * Sorts children by zIndex. Previous order is mantained for 2 children with the same zIndex.
     */
    sortChildren(): void;

    /**
     * Updates the transform on all children of this container for rendering
     */
    updateTransform(): void;

    /**
     * Recalculates the bounds of the container.
     *
     */
    calculateBounds(): void;

    /**
     * Renders the object using the WebGL renderer
     *
     * @param {PIXI.Renderer} renderer - The renderer
     */
    render(renderer: PIXI.Renderer): void;

    /**
     * Render the object using the WebGL renderer and advanced features.
     *
     * @protected
     * @param {PIXI.Renderer} renderer - The renderer
     */
    protected renderAdvanced(renderer: PIXI.Renderer): void;

    /**
     * The width of the Container, setting this will actually modify the scale to achieve the value set
     *
     * @member {number}
     */
    width: number;
    /**
     * The height of the Container, setting this will actually modify the scale to achieve the value set
     *
     * @member {number}
     */
    height: number;
    /**
     * Determines if the children to the displayObject can be clicked/touched
     * Setting this to false allows PixiJS to bypass a recursive `hitTest` function
     *
     * @member {boolean}
     * @memberof PIXI.Container#
     */
    interactiveChildren: boolean;

    /**
     * Returns the display object in the container.
     *
     * @method getChildByName
     * @memberof PIXI.Container#
     * @param {string} name - Instance name.
     * @return {PIXI.DisplayObject} The child with the specified name.
     */
    getChildByName(name: string): PIXI.DisplayObject;

    /**
     *  Flag for if the object is accessible. If true AccessibilityManager will overlay a
     *   shadow div with attributes set
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    accessible: boolean;
    /**
     * Sets the title attribute of the shadow div
     * If accessibleTitle AND accessibleHint has not been this will default to 'displayObject [tabIndex]'
     *
     * @member {?string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleTitle: string;
    /**
     * Sets the aria-label attribute of the shadow div
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleHint: string;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleActive: boolean;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleDiv: boolean;
    /**
     * Specify the type of div the accessible layer is. Screen readers treat the element differently
     * depending on this type. Defaults to button.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'button'
     */
    accessibleType: string;
    /**
     * Specify the pointer-events the accessible div will use
     * Defaults to auto.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'auto'
     */
    accessiblePointerEvents: string;
    /**
     * Setting to false will prevent any children inside this container to
     * be accessible. Defaults to true.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @default true
     */
    accessibleChildren: boolean;
    /**
     * World transform and local transform of this object.
     * This will become read-only later, please do not assign anything there unless you know what are you doing.
     *
     * @member {PIXI.Transform} PIXI.DisplayObject#transform
     */
    transform: PIXI.Transform;
    /**
     * The opacity of the object.
     *
     * @member {number} PIXI.DisplayObject#alpha
     */
    alpha: number;
    /**
     * The visibility of the object. If false the object will not be drawn, and
     * the updateTransform function will not be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds or call updateTransform manually.
     *
     * @member {boolean} PIXI.DisplayObject#visible
     */
    visible: boolean;
    /**
     * Can this object be rendered, if false the object will not be drawn but the updateTransform
     * methods will still be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds manually.
     *
     * @member {boolean} PIXI.DisplayObject#renderable
     */
    renderable: boolean;
    /**
     * The display object container that contains this display object.
     *
     * @member {PIXI.Container} PIXI.DisplayObject#parent
     * @readonly
     */
    readonly parent: PIXI.Container;
    /**
     * The multiplied alpha of the displayObject.
     *
     * @member {number} PIXI.DisplayObject#worldAlpha
     * @readonly
     */
    readonly worldAlpha: number;
    /**
     * Which index in the children array the display component was before the previous zIndex sort.
     * Used by containers to help sort objects with the same zIndex, by using previous array index as the decider.
     *
     * @member {number} PIXI.DisplayObject#_lastSortedIndex
     * @protected
     */
    protected _lastSortedIndex: number;
    /**
     * The zIndex of the displayObject.
     * A higher value will mean it will be rendered on top of other displayObjects within the same container.
     *
     * @member {number} PIXI.DisplayObject#_zIndex
     * @protected
     */
    protected _zIndex: number;
    /**
     * The area the filter is applied to. This is used as more of an optimization
     * rather than figuring out the dimensions of the displayObject each frame you can set this rectangle.
     *
     * Also works as an interaction mask.
     *
     * @member {?PIXI.Rectangle} PIXI.DisplayObject#filterArea
     */
    filterArea: PIXI.Rectangle;
    /**
     * Sets the filters for the displayObject.
     * * IMPORTANT: This is a WebGL only feature and will be ignored by the canvas renderer.
     * To remove filters simply set this property to `'null'`.
     *
     * @member {?PIXI.Filter[]} PIXI.DisplayObject#filters
     */
    filters: PIXI.Filter[];
    /**
     * The bounds object, this is used to calculate and store the bounds of the displayObject.
     *
     * @member {PIXI.Bounds} PIXI.DisplayObject#_bounds
     * @protected
     */
    protected _bounds: PIXI.Bounds;
    /**
     * The original, cached mask of the object.
     *
     * @member {PIXI.Graphics|PIXI.Sprite|null} PIXI.DisplayObject#_mask
     * @protected
     */
    protected _mask: PIXI.Graphics | PIXI.Sprite | null;
    /**
     * If the object has been destroyed via destroy(). If true, it should not be used.
     *
     * @member {boolean} PIXI.DisplayObject#_destroyed
     * @protected
     */
    protected _destroyed: boolean;
    /**
     * used to fast check if a sprite is.. a sprite!
     * @member {boolean} PIXI.DisplayObject#isSprite
     */
    isSprite: boolean;
    /**
     * Does any other displayObject use this object as a mask?
     * @member {boolean} PIXI.DisplayObject#isMask
     */
    isMask: boolean;
    /**
     * @protected
     * @member {PIXI.DisplayObject}
     */
    protected _tempDisplayObjectParent: PIXI.DisplayObject;

    /**
     * Recursively updates transform of all objects from the root to this one
     * internal function for toLocal()
     */
    _recursivePostUpdateTransform(): void;

    /**
     * Retrieves the bounds of the displayObject as a rectangle object.
     *
     * @param {boolean} [skipUpdate] - Setting to `true` will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @param {PIXI.Rectangle} [rect] - Optional rectangle to store the result of the bounds calculation.
     * @return {PIXI.Rectangle} The rectangular bounding area.
     */
    getBounds(skipUpdate?: boolean, rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Retrieves the local bounds of the displayObject as a rectangle object.
     *
     * @param {PIXI.Rectangle} [rect] - Optional rectangle to store the result of the bounds calculation.
     * @return {PIXI.Rectangle} The rectangular bounding area.
     */
    getLocalBounds(rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Calculates the global position of the display object.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform.
     * @return {PIXI.IPoint} A point object representing the position of this object.
     */
    toGlobal(position: PIXI.IPoint, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Calculates the local position of the display object relative to another point.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.DisplayObject} [from] - The DisplayObject to calculate the global position from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform
     * @return {PIXI.IPoint} A point object representing the position of this object
     */
    toLocal(position: PIXI.IPoint, from?: PIXI.DisplayObject, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Set the parent Container of this DisplayObject.
     *
     * @param {PIXI.Container} container - The Container to add this DisplayObject to.
     * @return {PIXI.Container} The Container that this DisplayObject was added to.
     */
    setParent(container: PIXI.Container): PIXI.Container;

    /**
     * Convenience function to set the position, scale, skew and pivot at once.
     *
     * @param {number} [x=0] - The X position
     * @param {number} [y=0] - The Y position
     * @param {number} [scaleX=1] - The X scale value
     * @param {number} [scaleY=1] - The Y scale value
     * @param {number} [rotation=0] - The rotation
     * @param {number} [skewX=0] - The X skew value
     * @param {number} [skewY=0] - The Y skew value
     * @param {number} [pivotX=0] - The X pivot value
     * @param {number} [pivotY=0] - The Y pivot value
     * @return {PIXI.DisplayObject} The DisplayObject instance
     */
    setTransform(
      x?: number,
      y?: number,
      scaleX?: number,
      scaleY?: number,
      rotation?: number,
      skewX?: number,
      skewY?: number,
      pivotX?: number,
      pivotY?: number,
    ): PIXI.DisplayObject;

    /**
     * The position of the displayObject on the x axis relative to the local coordinates of the parent.
     * An alias to position.x
     *
     * @member {number}
     */
    x: number;
    /**
     * The position of the displayObject on the y axis relative to the local coordinates of the parent.
     * An alias to position.y
     *
     * @member {number}
     */
    y: number;
    /**
     * Current transform of the object based on world (parent) factors.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly worldTransform: PIXI.Matrix;
    /**
     * Current transform of the object based on local factors: position, scale, other stuff.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly localTransform: PIXI.Matrix;
    /**
     * The coordinate of the object relative to the local coordinates of the parent.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    position: PIXI.IPoint;
    /**
     * The scale factor of the object.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    scale: PIXI.IPoint;
    /**
     * The pivot point of the displayObject that it rotates around.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    pivot: PIXI.IPoint;
    /**
     * The skew factor for the object in radians.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.ObservablePoint}
     */
    skew: PIXI.ObservablePoint;
    /**
     * The rotation of the object in radians.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    rotation: number;
    /**
     * The angle of the object in degrees.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    angle: number;
    /**
     * The zIndex of the displayObject.
     * If a container has the sortableChildren property set to true, children will be automatically
     * sorted by zIndex value; a higher value will mean it will be moved towards the end of the array,
     * and thus rendered on top of other displayObjects within the same container.
     *
     * @member {number}
     */
    zIndex: number;
    /**
     * Indicates if the object is globally visible.
     *
     * @member {boolean}
     * @readonly
     */
    readonly worldVisible: boolean;
    /**
     * Sets a mask for the displayObject. A mask is an object that limits the visibility of an
     * object to the shape of the mask applied to it. In PixiJS a regular mask must be a
     * {@link PIXI.Graphics} or a {@link PIXI.Sprite} object. This allows for much faster masking in canvas as it
     * utilities shape clipping. To remove a mask, set this property to `null`.
     *
     * For sprite mask both alpha and red channel are used. Black mask is the same as transparent mask.
     * @example
     * const graphics = new PIXI.Graphics();
     * graphics.beginFill(0xFF3300);
     * graphics.drawRect(50, 250, 100, 100);
     * graphics.endFill();
     *
     * const sprite = new PIXI.Sprite(texture);
     * sprite.mask = graphics;
     * @todo At the moment, PIXI.CanvasRenderer doesn't support PIXI.Sprite as mask.
     *
     * @member {PIXI.Container|PIXI.MaskData}
     */
    mask: PIXI.Container | PIXI.MaskData;

    /**
     * DisplayObject default updateTransform, does not update children of container.
     * Will crash if there's no parent element.
     *
     * @memberof PIXI.DisplayObject#
     * @function displayObjectUpdateTransform
     */
    displayObjectUpdateTransform(): void;

    /**
     * Enable interaction events for the DisplayObject. Touch, pointer and mouse
     * events will not be emitted unless `interactive` is set to `true`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.on('tap', (event) => {
     *    //handle event
     * });
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    interactive: boolean;
    /**
     * Interaction shape. Children will be hit first, then this shape will be checked.
     * Setting this will cause this shape to be checked in hit tests rather than the displayObject's bounds.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.hitArea = new PIXI.Rectangle(0, 0, 100, 100);
     * @member {PIXI.IHitArea}
     * @memberof PIXI.DisplayObject#
     */
    hitArea: PIXI.IHitArea;
    /**
     * If enabled, the mouse cursor use the pointer behavior when hovered over the displayObject if it is interactive
     * Setting this changes the 'cursor' property to `'pointer'`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.buttonMode = true;
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    buttonMode: boolean;
    /**
     * This defines what cursor mode is used when the mouse cursor
     * is hovered over the displayObject.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.cursor = 'wait';
     * @see https://developer.mozilla.org/en/docs/Web/CSS/cursor
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    cursor: string;
    /**
     * Set this to true if you want this display object to be cached as a bitmap.
     * This basically takes a snap shot of the display object as it is at that moment. It can
     * provide a performance benefit for complex static displayObjects.
     * To remove simply set this property to `false`
     *
     * IMPORTANT GOTCHA - Make sure that all your textures are preloaded BEFORE setting this property to true
     * as it will take a snapshot of what is currently there. If the textures have not loaded then they will not appear.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    cacheAsBitmap: boolean;
    /**
     * The instance name of the object.
     *
     * @memberof PIXI.DisplayObject#
     * @member {string} name
     */
    name: string;

    /**
     * Returns the global position of the displayObject. Does not depend on object scale, rotation and pivot.
     *
     * @method getGlobalPosition
     * @memberof PIXI.DisplayObject#
     * @param {PIXI.Point} [point=new PIXI.Point()] - The point to write the global value to.
     * @param {boolean} [skipUpdate=false] - Setting to true will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @return {PIXI.Point} The updated point.
     */
    getGlobalPosition(point?: PIXI.Point, skipUpdate?: boolean): PIXI.Point;
  }

  /**
   * The rope allows you to draw a texture across several points and then manipulate these points
   *
   *```js
   * for (let i = 0; i < 20; i++) {
   *     points.push(new PIXI.Point(i * 50, 0));
   * };
   * let rope = new PIXI.SimpleRope(PIXI.Texture.from("snake.png"), points);
   *  ```
   *
   * @class
   * @extends PIXI.Mesh
   * @memberof PIXI
   *
   */
  class SimpleRope extends PIXI.Mesh {
    constructor(texture: PIXI.Texture, points: PIXI.Point[], textureScale?: number);

    /**
     * re-calculate vertices by rope points each frame
     *
     * @member {boolean} PIXI.SimpleRope#autoUpdate
     */
    autoUpdate: boolean;
    /**
     * Includes vertex positions, face indices, normals, colors, UVs, and
     * custom attributes within buffers, reducing the cost of passing all
     * this data to the GPU. Can be shared between multiple Mesh objects.
     * @member {PIXI.Geometry} PIXI.Mesh#geometry
     * @readonly
     */
    readonly geometry: PIXI.Geometry;
    /**
     * Represents the vertex and fragment shaders that processes the geometry and runs on the GPU.
     * Can be shared between multiple Mesh objects.
     * @member {PIXI.Shader|PIXI.MeshMaterial} PIXI.Mesh#shader
     */
    shader: PIXI.Shader | PIXI.MeshMaterial;
    /**
     * Represents the WebGL state the Mesh required to render, excludes shader and geometry. E.g.,
     * blend mode, culling, depth testing, direction of rendering triangles, backface, etc.
     * @member {PIXI.State} PIXI.Mesh#state
     */
    state: PIXI.State;
    /**
     * The way the Mesh should be drawn, can be any of the {@link PIXI.DRAW_MODES} constants.
     *
     * @member {number} PIXI.Mesh#drawMode
     * @see PIXI.DRAW_MODES
     */
    drawMode: number;
    /**
     * Typically the index of the IndexBuffer where to start drawing.
     * @member {number} PIXI.Mesh#start
     * @default 0
     */
    start: number;
    /**
     * How much of the geometry to draw, by default `0` renders everything.
     * @member {number} PIXI.Mesh#size
     * @default 0
     */
    size: number;
    /**
     * To change mesh uv's, change its uvBuffer data and increment its _updateID.
     * @member {PIXI.Buffer}
     * @readonly
     */
    readonly uvBuffer: PIXI.Buffer;
    /**
     * To change mesh vertices, change its uvBuffer data and increment its _updateID.
     * Incrementing _updateID is optional because most of Mesh objects do it anyway.
     * @member {PIXI.Buffer}
     * @readonly
     */
    readonly verticesBuffer: PIXI.Buffer;
    /**
     * Alias for {@link PIXI.Mesh#shader}.
     * @member {PIXI.Shader|PIXI.MeshMaterial}
     */
    material: PIXI.Shader | PIXI.MeshMaterial;
    /**
     * The blend mode to be applied to the Mesh. Apply a value of
     * `PIXI.BLEND_MODES.NORMAL` to reset the blend mode.
     *
     * @member {number}
     * @default PIXI.BLEND_MODES.NORMAL;
     * @see PIXI.BLEND_MODES
     */
    blendMode: number;
    /**
     * If true PixiJS will Math.floor() x/y values when rendering, stopping pixel interpolation.
     * Advantages can include sharper image quality (like text) and faster rendering on canvas.
     * The main disadvantage is movement of objects may appear less smooth.
     * To set the global default, change {@link PIXI.settings.ROUND_PIXELS}
     *
     * @member {boolean}
     * @default false
     */
    roundPixels: boolean;
    /**
     * The multiply tint applied to the Mesh. This is a hex value. A value of
     * `0xFFFFFF` will remove any tint effect.
     *
     * @member {number}
     * @default 0xFFFFFF
     */
    tint: number;
    /**
     * The texture that the Mesh uses.
     *
     * @member {PIXI.Texture}
     */
    texture: PIXI.Texture;

    /**
     * Standard renderer draw.
     * @protected
     * @param {PIXI.Renderer} renderer - Instance to renderer.
     */
    protected _render(renderer: PIXI.Renderer): void;

    /**
     * Standard non-batching way of rendering.
     * @protected
     * @param {PIXI.Renderer} renderer - Instance to renderer.
     */
    protected _renderDefault(renderer: PIXI.Renderer): void;

    /**
     * Rendering by using the Batch system.
     * @protected
     * @param {PIXI.Renderer} renderer - Instance to renderer.
     */
    protected _renderToBatch(renderer: PIXI.Renderer): void;

    /**
     * Updates vertexData field based on transform and vertices
     */
    calculateVertices(): void;

    /**
     * Updates uv field based on from geometry uv's or batchUvs
     */
    calculateUvs(): void;

    /**
     * Updates the bounds of the mesh as a rectangle. The bounds calculation takes the worldTransform into account.
     * there must be a aVertexPosition attribute present in the geometry for bounds to be calculated correctly.
     *
     * @protected
     */
    protected _calculateBounds(): void;

    /**
     * Tests if a point is inside this mesh. Works only for PIXI.DRAW_MODES.TRIANGLES.
     *
     * @param {PIXI.Point} point the point to test
     * @return {boolean} the result of the test
     */
    containsPoint(point: PIXI.Point): boolean;

    /**
     * Destroys the Mesh object.
     *
     * @param {object|boolean} [options] - Options parameter. A boolean will act as if all
     *  options have been set to that value
     * @param {boolean} [options.children=false] - if set to true, all the children will have
     *  their destroy method called as well. 'options' will be passed on to those calls.
     */
    destroy(options?: { children?: boolean }): void;

    /**
     * The array of children of this container.
     *
     * @member {PIXI.DisplayObject[]} PIXI.Container#children
     * @readonly
     */
    readonly children: PIXI.DisplayObject[];
    /**
     * If set to true, the container will sort its children by zIndex value
     * when updateTransform() is called, or manually if sortChildren() is called.
     *
     * This actually changes the order of elements in the array, so should be treated
     * as a basic solution that is not performant compared to other solutions,
     * such as @link https://github.com/pixijs/pixi-display
     *
     * Also be aware of that this may not work nicely with the addChildAt() function,
     * as the zIndex sorting may cause the child to automatically sorted to another position.
     *
     * @see PIXI.settings.SORTABLE_CHILDREN
     *
     * @member {boolean} PIXI.Container#sortableChildren
     */
    sortableChildren: boolean;
    /**
     * Should children be sorted by zIndex at the next updateTransform call.
     * Will get automatically set to true if a new child is added, or if a child's zIndex changes.
     *
     * @member {boolean} PIXI.Container#sortDirty
     */
    sortDirty: boolean;

    /**
     * Overridable method that can be used by Container subclasses whenever the children array is modified
     *
     * @protected
     */
    protected onChildrenChange(): void;

    /**
     * Adds one or more children to the container.
     *
     * Multiple items can be added like so: `myContainer.addChild(thingOne, thingTwo, thingThree)`
     *
     * @param {...PIXI.DisplayObject} child - The DisplayObject(s) to add to the container
     * @return {PIXI.DisplayObject} The first child that was added.
     */
    addChild<TChildren extends PIXI.DisplayObject[]>(...child: TChildren): TChildren[0];

    /**
     * Adds a child to the container at a specified index. If the index is out of bounds an error will be thrown
     *
     * @param {PIXI.DisplayObject} child - The child to add
     * @param {number} index - The index to place the child in
     * @return {PIXI.DisplayObject} The child that was added.
     */
    addChildAt<T extends PIXI.DisplayObject>(child: T, index: number): T;

    /**
     * Swaps the position of 2 Display Objects within this container.
     *
     * @param {PIXI.DisplayObject} child - First display object to swap
     * @param {PIXI.DisplayObject} child2 - Second display object to swap
     */
    swapChildren(child: PIXI.DisplayObject, child2: PIXI.DisplayObject): void;

    /**
     * Returns the index position of a child DisplayObject instance
     *
     * @param {PIXI.DisplayObject} child - The DisplayObject instance to identify
     * @return {number} The index position of the child display object to identify
     */
    getChildIndex(child: PIXI.DisplayObject): number;

    /**
     * Changes the position of an existing child in the display object container
     *
     * @param {PIXI.DisplayObject} child - The child DisplayObject instance for which you want to change the index number
     * @param {number} index - The resulting index number for the child display object
     */
    setChildIndex(child: PIXI.DisplayObject, index: number): void;

    /**
     * Returns the child at the specified index
     *
     * @param {number} index - The index to get the child at
     * @return {PIXI.DisplayObject} The child at the given index, if any.
     */
    getChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes one or more children from the container.
     *
     * @param {...PIXI.DisplayObject} child - The DisplayObject(s) to remove
     * @return {PIXI.DisplayObject} The first child that was removed.
     */
    removeChild<TChildren extends PIXI.DisplayObject[]>(...child: TChildren): TChildren[0];

    /**
     * Removes a child from the specified index position.
     *
     * @param {number} index - The index to get the child from
     * @return {PIXI.DisplayObject} The child that was removed.
     */
    removeChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes all children from this container that are within the begin and end indexes.
     *
     * @param {number} [beginIndex=0] - The beginning position.
     * @param {number} [endIndex=this.children.length] - The ending position. Default value is size of the container.
     * @returns {PIXI.DisplayObject[]} List of removed children
     */
    removeChildren(beginIndex?: number, endIndex?: number): PIXI.DisplayObject[];

    /**
     * Sorts children by zIndex. Previous order is mantained for 2 children with the same zIndex.
     */
    sortChildren(): void;

    /**
     * Updates the transform on all children of this container for rendering
     */
    updateTransform(): void;

    /**
     * Recalculates the bounds of the container.
     *
     */
    calculateBounds(): void;

    /**
     * Renders the object using the WebGL renderer
     *
     * @param {PIXI.Renderer} renderer - The renderer
     */
    render(renderer: PIXI.Renderer): void;

    /**
     * Render the object using the WebGL renderer and advanced features.
     *
     * @protected
     * @param {PIXI.Renderer} renderer - The renderer
     */
    protected renderAdvanced(renderer: PIXI.Renderer): void;

    /**
     * The width of the Container, setting this will actually modify the scale to achieve the value set
     *
     * @member {number}
     */
    width: number;
    /**
     * The height of the Container, setting this will actually modify the scale to achieve the value set
     *
     * @member {number}
     */
    height: number;
    /**
     * Determines if the children to the displayObject can be clicked/touched
     * Setting this to false allows PixiJS to bypass a recursive `hitTest` function
     *
     * @member {boolean}
     * @memberof PIXI.Container#
     */
    interactiveChildren: boolean;

    /**
     * Returns the display object in the container.
     *
     * @method getChildByName
     * @memberof PIXI.Container#
     * @param {string} name - Instance name.
     * @return {PIXI.DisplayObject} The child with the specified name.
     */
    getChildByName(name: string): PIXI.DisplayObject;

    /**
     *  Flag for if the object is accessible. If true AccessibilityManager will overlay a
     *   shadow div with attributes set
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    accessible: boolean;
    /**
     * Sets the title attribute of the shadow div
     * If accessibleTitle AND accessibleHint has not been this will default to 'displayObject [tabIndex]'
     *
     * @member {?string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleTitle: string;
    /**
     * Sets the aria-label attribute of the shadow div
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleHint: string;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleActive: boolean;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleDiv: boolean;
    /**
     * Specify the type of div the accessible layer is. Screen readers treat the element differently
     * depending on this type. Defaults to button.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'button'
     */
    accessibleType: string;
    /**
     * Specify the pointer-events the accessible div will use
     * Defaults to auto.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'auto'
     */
    accessiblePointerEvents: string;
    /**
     * Setting to false will prevent any children inside this container to
     * be accessible. Defaults to true.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @default true
     */
    accessibleChildren: boolean;
    /**
     * World transform and local transform of this object.
     * This will become read-only later, please do not assign anything there unless you know what are you doing.
     *
     * @member {PIXI.Transform} PIXI.DisplayObject#transform
     */
    transform: PIXI.Transform;
    /**
     * The opacity of the object.
     *
     * @member {number} PIXI.DisplayObject#alpha
     */
    alpha: number;
    /**
     * The visibility of the object. If false the object will not be drawn, and
     * the updateTransform function will not be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds or call updateTransform manually.
     *
     * @member {boolean} PIXI.DisplayObject#visible
     */
    visible: boolean;
    /**
     * Can this object be rendered, if false the object will not be drawn but the updateTransform
     * methods will still be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds manually.
     *
     * @member {boolean} PIXI.DisplayObject#renderable
     */
    renderable: boolean;
    /**
     * The display object container that contains this display object.
     *
     * @member {PIXI.Container} PIXI.DisplayObject#parent
     * @readonly
     */
    readonly parent: PIXI.Container;
    /**
     * The multiplied alpha of the displayObject.
     *
     * @member {number} PIXI.DisplayObject#worldAlpha
     * @readonly
     */
    readonly worldAlpha: number;
    /**
     * Which index in the children array the display component was before the previous zIndex sort.
     * Used by containers to help sort objects with the same zIndex, by using previous array index as the decider.
     *
     * @member {number} PIXI.DisplayObject#_lastSortedIndex
     * @protected
     */
    protected _lastSortedIndex: number;
    /**
     * The zIndex of the displayObject.
     * A higher value will mean it will be rendered on top of other displayObjects within the same container.
     *
     * @member {number} PIXI.DisplayObject#_zIndex
     * @protected
     */
    protected _zIndex: number;
    /**
     * The area the filter is applied to. This is used as more of an optimization
     * rather than figuring out the dimensions of the displayObject each frame you can set this rectangle.
     *
     * Also works as an interaction mask.
     *
     * @member {?PIXI.Rectangle} PIXI.DisplayObject#filterArea
     */
    filterArea: PIXI.Rectangle;
    /**
     * Sets the filters for the displayObject.
     * * IMPORTANT: This is a WebGL only feature and will be ignored by the canvas renderer.
     * To remove filters simply set this property to `'null'`.
     *
     * @member {?PIXI.Filter[]} PIXI.DisplayObject#filters
     */
    filters: PIXI.Filter[];
    /**
     * The bounds object, this is used to calculate and store the bounds of the displayObject.
     *
     * @member {PIXI.Bounds} PIXI.DisplayObject#_bounds
     * @protected
     */
    protected _bounds: PIXI.Bounds;
    /**
     * The original, cached mask of the object.
     *
     * @member {PIXI.Graphics|PIXI.Sprite|null} PIXI.DisplayObject#_mask
     * @protected
     */
    protected _mask: PIXI.Graphics | PIXI.Sprite | null;
    /**
     * If the object has been destroyed via destroy(). If true, it should not be used.
     *
     * @member {boolean} PIXI.DisplayObject#_destroyed
     * @protected
     */
    protected _destroyed: boolean;
    /**
     * used to fast check if a sprite is.. a sprite!
     * @member {boolean} PIXI.DisplayObject#isSprite
     */
    isSprite: boolean;
    /**
     * Does any other displayObject use this object as a mask?
     * @member {boolean} PIXI.DisplayObject#isMask
     */
    isMask: boolean;
    /**
     * @protected
     * @member {PIXI.DisplayObject}
     */
    protected _tempDisplayObjectParent: PIXI.DisplayObject;

    /**
     * Recursively updates transform of all objects from the root to this one
     * internal function for toLocal()
     */
    _recursivePostUpdateTransform(): void;

    /**
     * Retrieves the bounds of the displayObject as a rectangle object.
     *
     * @param {boolean} [skipUpdate] - Setting to `true` will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @param {PIXI.Rectangle} [rect] - Optional rectangle to store the result of the bounds calculation.
     * @return {PIXI.Rectangle} The rectangular bounding area.
     */
    getBounds(skipUpdate?: boolean, rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Retrieves the local bounds of the displayObject as a rectangle object.
     *
     * @param {PIXI.Rectangle} [rect] - Optional rectangle to store the result of the bounds calculation.
     * @return {PIXI.Rectangle} The rectangular bounding area.
     */
    getLocalBounds(rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Calculates the global position of the display object.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform.
     * @return {PIXI.IPoint} A point object representing the position of this object.
     */
    toGlobal(position: PIXI.IPoint, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Calculates the local position of the display object relative to another point.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.DisplayObject} [from] - The DisplayObject to calculate the global position from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform
     * @return {PIXI.IPoint} A point object representing the position of this object
     */
    toLocal(position: PIXI.IPoint, from?: PIXI.DisplayObject, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Set the parent Container of this DisplayObject.
     *
     * @param {PIXI.Container} container - The Container to add this DisplayObject to.
     * @return {PIXI.Container} The Container that this DisplayObject was added to.
     */
    setParent(container: PIXI.Container): PIXI.Container;

    /**
     * Convenience function to set the position, scale, skew and pivot at once.
     *
     * @param {number} [x=0] - The X position
     * @param {number} [y=0] - The Y position
     * @param {number} [scaleX=1] - The X scale value
     * @param {number} [scaleY=1] - The Y scale value
     * @param {number} [rotation=0] - The rotation
     * @param {number} [skewX=0] - The X skew value
     * @param {number} [skewY=0] - The Y skew value
     * @param {number} [pivotX=0] - The X pivot value
     * @param {number} [pivotY=0] - The Y pivot value
     * @return {PIXI.DisplayObject} The DisplayObject instance
     */
    setTransform(
      x?: number,
      y?: number,
      scaleX?: number,
      scaleY?: number,
      rotation?: number,
      skewX?: number,
      skewY?: number,
      pivotX?: number,
      pivotY?: number,
    ): PIXI.DisplayObject;

    /**
     * The position of the displayObject on the x axis relative to the local coordinates of the parent.
     * An alias to position.x
     *
     * @member {number}
     */
    x: number;
    /**
     * The position of the displayObject on the y axis relative to the local coordinates of the parent.
     * An alias to position.y
     *
     * @member {number}
     */
    y: number;
    /**
     * Current transform of the object based on world (parent) factors.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly worldTransform: PIXI.Matrix;
    /**
     * Current transform of the object based on local factors: position, scale, other stuff.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly localTransform: PIXI.Matrix;
    /**
     * The coordinate of the object relative to the local coordinates of the parent.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    position: PIXI.IPoint;
    /**
     * The scale factor of the object.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    scale: PIXI.IPoint;
    /**
     * The pivot point of the displayObject that it rotates around.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    pivot: PIXI.IPoint;
    /**
     * The skew factor for the object in radians.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.ObservablePoint}
     */
    skew: PIXI.ObservablePoint;
    /**
     * The rotation of the object in radians.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    rotation: number;
    /**
     * The angle of the object in degrees.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    angle: number;
    /**
     * The zIndex of the displayObject.
     * If a container has the sortableChildren property set to true, children will be automatically
     * sorted by zIndex value; a higher value will mean it will be moved towards the end of the array,
     * and thus rendered on top of other displayObjects within the same container.
     *
     * @member {number}
     */
    zIndex: number;
    /**
     * Indicates if the object is globally visible.
     *
     * @member {boolean}
     * @readonly
     */
    readonly worldVisible: boolean;
    /**
     * Sets a mask for the displayObject. A mask is an object that limits the visibility of an
     * object to the shape of the mask applied to it. In PixiJS a regular mask must be a
     * {@link PIXI.Graphics} or a {@link PIXI.Sprite} object. This allows for much faster masking in canvas as it
     * utilities shape clipping. To remove a mask, set this property to `null`.
     *
     * For sprite mask both alpha and red channel are used. Black mask is the same as transparent mask.
     * @example
     * const graphics = new PIXI.Graphics();
     * graphics.beginFill(0xFF3300);
     * graphics.drawRect(50, 250, 100, 100);
     * graphics.endFill();
     *
     * const sprite = new PIXI.Sprite(texture);
     * sprite.mask = graphics;
     * @todo At the moment, PIXI.CanvasRenderer doesn't support PIXI.Sprite as mask.
     *
     * @member {PIXI.Container|PIXI.MaskData}
     */
    mask: PIXI.Container | PIXI.MaskData;

    /**
     * DisplayObject default updateTransform, does not update children of container.
     * Will crash if there's no parent element.
     *
     * @memberof PIXI.DisplayObject#
     * @function displayObjectUpdateTransform
     */
    displayObjectUpdateTransform(): void;

    /**
     * Enable interaction events for the DisplayObject. Touch, pointer and mouse
     * events will not be emitted unless `interactive` is set to `true`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.on('tap', (event) => {
     *    //handle event
     * });
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    interactive: boolean;
    /**
     * Interaction shape. Children will be hit first, then this shape will be checked.
     * Setting this will cause this shape to be checked in hit tests rather than the displayObject's bounds.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.hitArea = new PIXI.Rectangle(0, 0, 100, 100);
     * @member {PIXI.IHitArea}
     * @memberof PIXI.DisplayObject#
     */
    hitArea: PIXI.IHitArea;
    /**
     * If enabled, the mouse cursor use the pointer behavior when hovered over the displayObject if it is interactive
     * Setting this changes the 'cursor' property to `'pointer'`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.buttonMode = true;
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    buttonMode: boolean;
    /**
     * This defines what cursor mode is used when the mouse cursor
     * is hovered over the displayObject.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.cursor = 'wait';
     * @see https://developer.mozilla.org/en/docs/Web/CSS/cursor
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    cursor: string;
    /**
     * Set this to true if you want this display object to be cached as a bitmap.
     * This basically takes a snap shot of the display object as it is at that moment. It can
     * provide a performance benefit for complex static displayObjects.
     * To remove simply set this property to `false`
     *
     * IMPORTANT GOTCHA - Make sure that all your textures are preloaded BEFORE setting this property to true
     * as it will take a snapshot of what is currently there. If the textures have not loaded then they will not appear.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    cacheAsBitmap: boolean;
    /**
     * The instance name of the object.
     *
     * @memberof PIXI.DisplayObject#
     * @member {string} name
     */
    name: string;

    /**
     * Returns the global position of the displayObject. Does not depend on object scale, rotation and pivot.
     *
     * @method getGlobalPosition
     * @memberof PIXI.DisplayObject#
     * @param {PIXI.Point} [point=new PIXI.Point()] - The point to write the global value to.
     * @param {boolean} [skipUpdate=false] - Setting to true will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @return {PIXI.Point} The updated point.
     */
    getGlobalPosition(point?: PIXI.Point, skipUpdate?: boolean): PIXI.Point;
  }

  /**
   * RopeGeometry allows you to draw a geometry across several points and then manipulate these points.
   *
   * ```js
   * for (let i = 0; i < 20; i++) {
   *     points.push(new PIXI.Point(i * 50, 0));
   * };
   * const rope = new PIXI.RopeGeometry(100, points);
   * ```
   *
   * @class
   * @extends PIXI.MeshGeometry
   * @memberof PIXI
   *
   */
  class RopeGeometry extends PIXI.MeshGeometry {
    constructor(width?: number, points?: PIXI.Point[], textureScale?: number);

    /**
     * An array of points that determine the rope
     * @member {PIXI.Point[]} PIXI.RopeGeometry#points
     */
    points: PIXI.Point[];
    /**
     * The width (i.e., thickness) of the rope.
     * @member {number} PIXI.RopeGeometry#width
     * @readOnly
     */
    readonly width: number;
    /**
     * Rope texture scale, if zero then the rope texture is stretched.
     * @member {number} PIXI.RopeGeometry#textureScale
     * @readOnly
     */
    readonly textureScale: number;

    /**
     * refreshes vertices of Rope mesh
     */
    updateVertices(): void;

    /**
     * A map of renderer IDs to webgl VAOs
     *
     * @protected
     * @type {object}
     */
    protected glVertexArrayObjects: any;
    /**
     * Number of instances in this geometry, pass it to `GeometrySystem.draw()`
     * @member {number} PIXI.Geometry#instanceCount
     * @default 1
     */
    instanceCount: number;
    /**
     * Count of existing (not destroyed) meshes that reference this geometry
     * @member {number} PIXI.Geometry#refCount
     */
    refCount: number;

    /**
     *
     * Adds an attribute to the geometry
     *
     * @param {String} id - the name of the attribute (matching up to a shader)
     * @param {PIXI.Buffer|number[]} [buffer] the buffer that holds the data of the attribute . You can also provide an Array and a buffer will be created from it.
     * @param {Number} [size=0] the size of the attribute. If you have 2 floats per vertex (eg position x and y) this would be 2
     * @param {Boolean} [normalized=false] should the data be normalized.
     * @param {Number} [type=PIXI.TYPES.FLOAT] what type of number is the attribute. Check {PIXI.TYPES} to see the ones available
     * @param {Number} [stride=0] How far apart (in floats) the start of each value is. (used for interleaving data)
     * @param {Number} [start=0] How far into the array to start reading values (used for interleaving data)
     *
     * @return {PIXI.Geometry} returns self, useful for chaining.
     */
    addAttribute(
      id: string,
      buffer?: PIXI.Buffer | number[],
      size?: number,
      normalized?: boolean,
      type?: number,
      stride?: number,
      start?: number,
    ): PIXI.Geometry;

    /**
     * returns the requested attribute
     *
     * @param {String} id  the name of the attribute required
     * @return {PIXI.Attribute} the attribute requested.
     */
    getAttribute(id: string): PIXI.Attribute;

    /**
     * returns the requested buffer
     *
     * @param {String} id  the name of the buffer required
     * @return {PIXI.Buffer} the buffer requested.
     */
    getBuffer(id: string): PIXI.Buffer;

    /**
     *
     * Adds an index buffer to the geometry
     * The index buffer contains integers, three for each triangle in the geometry, which reference the various attribute buffers (position, colour, UV coordinates, other UV coordinates, normal, …). There is only ONE index buffer.
     *
     * @param {PIXI.Buffer|number[]} [buffer] the buffer that holds the data of the index buffer. You can also provide an Array and a buffer will be created from it.
     * @return {PIXI.Geometry} returns self, useful for chaining.
     */
    addIndex(buffer?: PIXI.Buffer | number[]): PIXI.Geometry;

    /**
     * returns the index buffer
     *
     * @return {PIXI.Buffer} the index buffer.
     */
    getIndex(): PIXI.Buffer;

    /**
     * this function modifies the structure so that all current attributes become interleaved into a single buffer
     * This can be useful if your model remains static as it offers a little performance boost
     *
     * @return {PIXI.Geometry} returns self, useful for chaining.
     */
    interleave(): PIXI.Geometry;

    /**
     * disposes WebGL resources that are connected to this geometry
     */
    dispose(): void;

    /**
     * Destroys the geometry.
     */
    destroy(): void;

    /**
     * returns a clone of the geometry
     *
     * @returns {PIXI.Geometry} a new clone of this geometry
     */
    clone(): PIXI.Geometry;
  }

  /**
   * The ParticleContainer class is a really fast version of the Container built solely for speed,
   * so use when you need a lot of sprites or particles.
   *
   * The tradeoff of the ParticleContainer is that most advanced functionality will not work.
   * ParticleContainer implements the basic object transform (position, scale, rotation)
   * and some advanced functionality like tint (as of v4.5.6).
   *
   * Other more advanced functionality like masking, children, filters, etc will not work on sprites in this batch.
   *
   * It's extremely easy to use:
   * ```js
   * let container = new ParticleContainer();
   *
   * for (let i = 0; i < 100; ++i)
   * {
   *     let sprite = PIXI.Sprite.from("myImage.png");
   *     container.addChild(sprite);
   * }
   * ```
   *
   * And here you have a hundred sprites that will be rendered at the speed of light.
   *
   * @class
   * @extends PIXI.Container
   * @memberof PIXI
   */
  class ParticleContainer extends PIXI.Container {
    constructor(
      maxSize?: number,
      properties?: {
        vertices?: boolean;
        position?: boolean;
        rotation?: boolean;
        uvs?: boolean;
        tint?: boolean;
      },
      batchSize?: number,
      autoResize?: boolean,
    );

    /**
     * @member {boolean} PIXI.ParticleContainer#interactiveChildren
     *
     */
    interactiveChildren: boolean;
    /**
     * The blend mode to be applied to the sprite. Apply a value of `PIXI.BLEND_MODES.NORMAL`
     * to reset the blend mode.
     *
     * @member {number} PIXI.ParticleContainer#blendMode
     * @default PIXI.BLEND_MODES.NORMAL
     * @see PIXI.BLEND_MODES
     */
    blendMode: number;
    /**
     * If true, container allocates more batches in case there are more than `maxSize` particles.
     * @member {boolean} PIXI.ParticleContainer#autoResize
     * @default false
     */
    autoResize: boolean;
    /**
     * If true PixiJS will Math.floor() x/y values when rendering, stopping pixel interpolation.
     * Advantages can include sharper image quality (like text) and faster rendering on canvas.
     * The main disadvantage is movement of objects may appear less smooth.
     * Default to true here as performance is usually the priority for particles.
     *
     * @member {boolean} PIXI.ParticleContainer#roundPixels
     * @default true
     */
    roundPixels: boolean;
    /**
     * The texture used to render the children.
     *
     * @readonly
     * @member {PIXI.BaseTexture} PIXI.ParticleContainer#baseTexture
     */
    readonly baseTexture: PIXI.BaseTexture;

    /**
     * Sets the private properties array to dynamic / static based on the passed properties object
     *
     * @param {object} properties - The properties to be uploaded
     */
    setProperties(properties: any): void;

    /**
     * The tint applied to the container. This is a hex value.
     * A value of 0xFFFFFF will remove any tint effect.
     ** IMPORTANT: This is a WebGL only feature and will be ignored by the canvas renderer.
     * @member {number}
     * @default 0xFFFFFF
     */
    tint: number;

    /**
     * Destroys the container
     *
     * @param {object|boolean} [options] - Options parameter. A boolean will act as if all options
     *  have been set to that value
     * @param {boolean} [options.children=false] - if set to true, all the children will have their
     *  destroy method called as well. 'options' will be passed on to those calls.
     * @param {boolean} [options.texture=false] - Only used for child Sprites if options.children is set to true
     *  Should it destroy the texture of the child sprite
     * @param {boolean} [options.baseTexture=false] - Only used for child Sprites if options.children is set to true
     *  Should it destroy the base texture of the child sprite
     */
    destroy(options?: { children?: boolean; texture?: boolean; baseTexture?: boolean }): void;

    /**
     * The array of children of this container.
     *
     * @member {PIXI.DisplayObject[]} PIXI.Container#children
     * @readonly
     */
    readonly children: PIXI.DisplayObject[];
    /**
     * If set to true, the container will sort its children by zIndex value
     * when updateTransform() is called, or manually if sortChildren() is called.
     *
     * This actually changes the order of elements in the array, so should be treated
     * as a basic solution that is not performant compared to other solutions,
     * such as @link https://github.com/pixijs/pixi-display
     *
     * Also be aware of that this may not work nicely with the addChildAt() function,
     * as the zIndex sorting may cause the child to automatically sorted to another position.
     *
     * @see PIXI.settings.SORTABLE_CHILDREN
     *
     * @member {boolean} PIXI.Container#sortableChildren
     */
    sortableChildren: boolean;
    /**
     * Should children be sorted by zIndex at the next updateTransform call.
     * Will get automatically set to true if a new child is added, or if a child's zIndex changes.
     *
     * @member {boolean} PIXI.Container#sortDirty
     */
    sortDirty: boolean;

    /**
     * Adds one or more children to the container.
     *
     * Multiple items can be added like so: `myContainer.addChild(thingOne, thingTwo, thingThree)`
     *
     * @param {...PIXI.DisplayObject} child - The DisplayObject(s) to add to the container
     * @return {PIXI.DisplayObject} The first child that was added.
     */
    addChild<TChildren extends PIXI.DisplayObject[]>(...child: TChildren): TChildren[0];

    /**
     * Adds a child to the container at a specified index. If the index is out of bounds an error will be thrown
     *
     * @param {PIXI.DisplayObject} child - The child to add
     * @param {number} index - The index to place the child in
     * @return {PIXI.DisplayObject} The child that was added.
     */
    addChildAt<T extends PIXI.DisplayObject>(child: T, index: number): T;

    /**
     * Swaps the position of 2 Display Objects within this container.
     *
     * @param {PIXI.DisplayObject} child - First display object to swap
     * @param {PIXI.DisplayObject} child2 - Second display object to swap
     */
    swapChildren(child: PIXI.DisplayObject, child2: PIXI.DisplayObject): void;

    /**
     * Returns the index position of a child DisplayObject instance
     *
     * @param {PIXI.DisplayObject} child - The DisplayObject instance to identify
     * @return {number} The index position of the child display object to identify
     */
    getChildIndex(child: PIXI.DisplayObject): number;

    /**
     * Changes the position of an existing child in the display object container
     *
     * @param {PIXI.DisplayObject} child - The child DisplayObject instance for which you want to change the index number
     * @param {number} index - The resulting index number for the child display object
     */
    setChildIndex(child: PIXI.DisplayObject, index: number): void;

    /**
     * Returns the child at the specified index
     *
     * @param {number} index - The index to get the child at
     * @return {PIXI.DisplayObject} The child at the given index, if any.
     */
    getChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes one or more children from the container.
     *
     * @param {...PIXI.DisplayObject} child - The DisplayObject(s) to remove
     * @return {PIXI.DisplayObject} The first child that was removed.
     */
    removeChild<TChildren extends PIXI.DisplayObject[]>(...child: TChildren): TChildren[0];

    /**
     * Removes a child from the specified index position.
     *
     * @param {number} index - The index to get the child from
     * @return {PIXI.DisplayObject} The child that was removed.
     */
    removeChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes all children from this container that are within the begin and end indexes.
     *
     * @param {number} [beginIndex=0] - The beginning position.
     * @param {number} [endIndex=this.children.length] - The ending position. Default value is size of the container.
     * @returns {PIXI.DisplayObject[]} List of removed children
     */
    removeChildren(beginIndex?: number, endIndex?: number): PIXI.DisplayObject[];

    /**
     * Sorts children by zIndex. Previous order is mantained for 2 children with the same zIndex.
     */
    sortChildren(): void;

    /**
     * Recalculates the bounds of the container.
     *
     */
    calculateBounds(): void;

    /**
     * Recalculates the bounds of the object. Override this to
     * calculate the bounds of the specific object (not including children).
     *
     * @protected
     */
    protected _calculateBounds(): void;

    /**
     * Render the object using the WebGL renderer and advanced features.
     *
     * @protected
     * @param {PIXI.Renderer} renderer - The renderer
     */
    protected renderAdvanced(renderer: PIXI.Renderer): void;

    /**
     * To be overridden by the subclasses.
     *
     * @protected
     * @param {PIXI.Renderer} renderer - The renderer
     */
    protected _render(renderer: PIXI.Renderer): void;

    /**
     * The width of the Container, setting this will actually modify the scale to achieve the value set
     *
     * @member {number}
     */
    width: number;
    /**
     * The height of the Container, setting this will actually modify the scale to achieve the value set
     *
     * @member {number}
     */
    height: number;

    /**
     * Returns the display object in the container.
     *
     * @method getChildByName
     * @memberof PIXI.Container#
     * @param {string} name - Instance name.
     * @return {PIXI.DisplayObject} The child with the specified name.
     */
    getChildByName(name: string): PIXI.DisplayObject;

    /**
     *  Flag for if the object is accessible. If true AccessibilityManager will overlay a
     *   shadow div with attributes set
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    accessible: boolean;
    /**
     * Sets the title attribute of the shadow div
     * If accessibleTitle AND accessibleHint has not been this will default to 'displayObject [tabIndex]'
     *
     * @member {?string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleTitle: string;
    /**
     * Sets the aria-label attribute of the shadow div
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleHint: string;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleActive: boolean;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleDiv: boolean;
    /**
     * Specify the type of div the accessible layer is. Screen readers treat the element differently
     * depending on this type. Defaults to button.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'button'
     */
    accessibleType: string;
    /**
     * Specify the pointer-events the accessible div will use
     * Defaults to auto.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'auto'
     */
    accessiblePointerEvents: string;
    /**
     * Setting to false will prevent any children inside this container to
     * be accessible. Defaults to true.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @default true
     */
    accessibleChildren: boolean;
    /**
     * World transform and local transform of this object.
     * This will become read-only later, please do not assign anything there unless you know what are you doing.
     *
     * @member {PIXI.Transform} PIXI.DisplayObject#transform
     */
    transform: PIXI.Transform;
    /**
     * The opacity of the object.
     *
     * @member {number} PIXI.DisplayObject#alpha
     */
    alpha: number;
    /**
     * The visibility of the object. If false the object will not be drawn, and
     * the updateTransform function will not be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds or call updateTransform manually.
     *
     * @member {boolean} PIXI.DisplayObject#visible
     */
    visible: boolean;
    /**
     * Can this object be rendered, if false the object will not be drawn but the updateTransform
     * methods will still be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds manually.
     *
     * @member {boolean} PIXI.DisplayObject#renderable
     */
    renderable: boolean;
    /**
     * The display object container that contains this display object.
     *
     * @member {PIXI.Container} PIXI.DisplayObject#parent
     * @readonly
     */
    readonly parent: PIXI.Container;
    /**
     * The multiplied alpha of the displayObject.
     *
     * @member {number} PIXI.DisplayObject#worldAlpha
     * @readonly
     */
    readonly worldAlpha: number;
    /**
     * Which index in the children array the display component was before the previous zIndex sort.
     * Used by containers to help sort objects with the same zIndex, by using previous array index as the decider.
     *
     * @member {number} PIXI.DisplayObject#_lastSortedIndex
     * @protected
     */
    protected _lastSortedIndex: number;
    /**
     * The zIndex of the displayObject.
     * A higher value will mean it will be rendered on top of other displayObjects within the same container.
     *
     * @member {number} PIXI.DisplayObject#_zIndex
     * @protected
     */
    protected _zIndex: number;
    /**
     * The area the filter is applied to. This is used as more of an optimization
     * rather than figuring out the dimensions of the displayObject each frame you can set this rectangle.
     *
     * Also works as an interaction mask.
     *
     * @member {?PIXI.Rectangle} PIXI.DisplayObject#filterArea
     */
    filterArea: PIXI.Rectangle;
    /**
     * Sets the filters for the displayObject.
     * * IMPORTANT: This is a WebGL only feature and will be ignored by the canvas renderer.
     * To remove filters simply set this property to `'null'`.
     *
     * @member {?PIXI.Filter[]} PIXI.DisplayObject#filters
     */
    filters: PIXI.Filter[];
    /**
     * The bounds object, this is used to calculate and store the bounds of the displayObject.
     *
     * @member {PIXI.Bounds} PIXI.DisplayObject#_bounds
     * @protected
     */
    protected _bounds: PIXI.Bounds;
    /**
     * The original, cached mask of the object.
     *
     * @member {PIXI.Graphics|PIXI.Sprite|null} PIXI.DisplayObject#_mask
     * @protected
     */
    protected _mask: PIXI.Graphics | PIXI.Sprite | null;
    /**
     * If the object has been destroyed via destroy(). If true, it should not be used.
     *
     * @member {boolean} PIXI.DisplayObject#_destroyed
     * @protected
     */
    protected _destroyed: boolean;
    /**
     * used to fast check if a sprite is.. a sprite!
     * @member {boolean} PIXI.DisplayObject#isSprite
     */
    isSprite: boolean;
    /**
     * Does any other displayObject use this object as a mask?
     * @member {boolean} PIXI.DisplayObject#isMask
     */
    isMask: boolean;
    /**
     * @protected
     * @member {PIXI.DisplayObject}
     */
    protected _tempDisplayObjectParent: PIXI.DisplayObject;

    /**
     * Recursively updates transform of all objects from the root to this one
     * internal function for toLocal()
     */
    _recursivePostUpdateTransform(): void;

    /**
     * Retrieves the bounds of the displayObject as a rectangle object.
     *
     * @param {boolean} [skipUpdate] - Setting to `true` will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @param {PIXI.Rectangle} [rect] - Optional rectangle to store the result of the bounds calculation.
     * @return {PIXI.Rectangle} The rectangular bounding area.
     */
    getBounds(skipUpdate?: boolean, rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Retrieves the local bounds of the displayObject as a rectangle object.
     *
     * @param {PIXI.Rectangle} [rect] - Optional rectangle to store the result of the bounds calculation.
     * @return {PIXI.Rectangle} The rectangular bounding area.
     */
    getLocalBounds(rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Calculates the global position of the display object.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform.
     * @return {PIXI.IPoint} A point object representing the position of this object.
     */
    toGlobal(position: PIXI.IPoint, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Calculates the local position of the display object relative to another point.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.DisplayObject} [from] - The DisplayObject to calculate the global position from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform
     * @return {PIXI.IPoint} A point object representing the position of this object
     */
    toLocal(position: PIXI.IPoint, from?: PIXI.DisplayObject, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Set the parent Container of this DisplayObject.
     *
     * @param {PIXI.Container} container - The Container to add this DisplayObject to.
     * @return {PIXI.Container} The Container that this DisplayObject was added to.
     */
    setParent(container: PIXI.Container): PIXI.Container;

    /**
     * Convenience function to set the position, scale, skew and pivot at once.
     *
     * @param {number} [x=0] - The X position
     * @param {number} [y=0] - The Y position
     * @param {number} [scaleX=1] - The X scale value
     * @param {number} [scaleY=1] - The Y scale value
     * @param {number} [rotation=0] - The rotation
     * @param {number} [skewX=0] - The X skew value
     * @param {number} [skewY=0] - The Y skew value
     * @param {number} [pivotX=0] - The X pivot value
     * @param {number} [pivotY=0] - The Y pivot value
     * @return {PIXI.DisplayObject} The DisplayObject instance
     */
    setTransform(
      x?: number,
      y?: number,
      scaleX?: number,
      scaleY?: number,
      rotation?: number,
      skewX?: number,
      skewY?: number,
      pivotX?: number,
      pivotY?: number,
    ): PIXI.DisplayObject;

    /**
     * The position of the displayObject on the x axis relative to the local coordinates of the parent.
     * An alias to position.x
     *
     * @member {number}
     */
    x: number;
    /**
     * The position of the displayObject on the y axis relative to the local coordinates of the parent.
     * An alias to position.y
     *
     * @member {number}
     */
    y: number;
    /**
     * Current transform of the object based on world (parent) factors.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly worldTransform: PIXI.Matrix;
    /**
     * Current transform of the object based on local factors: position, scale, other stuff.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly localTransform: PIXI.Matrix;
    /**
     * The coordinate of the object relative to the local coordinates of the parent.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    position: PIXI.IPoint;
    /**
     * The scale factor of the object.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    scale: PIXI.IPoint;
    /**
     * The pivot point of the displayObject that it rotates around.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    pivot: PIXI.IPoint;
    /**
     * The skew factor for the object in radians.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.ObservablePoint}
     */
    skew: PIXI.ObservablePoint;
    /**
     * The rotation of the object in radians.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    rotation: number;
    /**
     * The angle of the object in degrees.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    angle: number;
    /**
     * The zIndex of the displayObject.
     * If a container has the sortableChildren property set to true, children will be automatically
     * sorted by zIndex value; a higher value will mean it will be moved towards the end of the array,
     * and thus rendered on top of other displayObjects within the same container.
     *
     * @member {number}
     */
    zIndex: number;
    /**
     * Indicates if the object is globally visible.
     *
     * @member {boolean}
     * @readonly
     */
    readonly worldVisible: boolean;
    /**
     * Sets a mask for the displayObject. A mask is an object that limits the visibility of an
     * object to the shape of the mask applied to it. In PixiJS a regular mask must be a
     * {@link PIXI.Graphics} or a {@link PIXI.Sprite} object. This allows for much faster masking in canvas as it
     * utilities shape clipping. To remove a mask, set this property to `null`.
     *
     * For sprite mask both alpha and red channel are used. Black mask is the same as transparent mask.
     * @example
     * const graphics = new PIXI.Graphics();
     * graphics.beginFill(0xFF3300);
     * graphics.drawRect(50, 250, 100, 100);
     * graphics.endFill();
     *
     * const sprite = new PIXI.Sprite(texture);
     * sprite.mask = graphics;
     * @todo At the moment, PIXI.CanvasRenderer doesn't support PIXI.Sprite as mask.
     *
     * @member {PIXI.Container|PIXI.MaskData}
     */
    mask: PIXI.Container | PIXI.MaskData;

    /**
     * DisplayObject default updateTransform, does not update children of container.
     * Will crash if there's no parent element.
     *
     * @memberof PIXI.DisplayObject#
     * @function displayObjectUpdateTransform
     */
    displayObjectUpdateTransform(): void;

    /**
     * Enable interaction events for the DisplayObject. Touch, pointer and mouse
     * events will not be emitted unless `interactive` is set to `true`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.on('tap', (event) => {
     *    //handle event
     * });
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    interactive: boolean;
    /**
     * Interaction shape. Children will be hit first, then this shape will be checked.
     * Setting this will cause this shape to be checked in hit tests rather than the displayObject's bounds.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.hitArea = new PIXI.Rectangle(0, 0, 100, 100);
     * @member {PIXI.IHitArea}
     * @memberof PIXI.DisplayObject#
     */
    hitArea: PIXI.IHitArea;
    /**
     * If enabled, the mouse cursor use the pointer behavior when hovered over the displayObject if it is interactive
     * Setting this changes the 'cursor' property to `'pointer'`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.buttonMode = true;
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    buttonMode: boolean;
    /**
     * This defines what cursor mode is used when the mouse cursor
     * is hovered over the displayObject.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.cursor = 'wait';
     * @see https://developer.mozilla.org/en/docs/Web/CSS/cursor
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    cursor: string;
    /**
     * Set this to true if you want this display object to be cached as a bitmap.
     * This basically takes a snap shot of the display object as it is at that moment. It can
     * provide a performance benefit for complex static displayObjects.
     * To remove simply set this property to `false`
     *
     * IMPORTANT GOTCHA - Make sure that all your textures are preloaded BEFORE setting this property to true
     * as it will take a snapshot of what is currently there. If the textures have not loaded then they will not appear.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    cacheAsBitmap: boolean;
    /**
     * The instance name of the object.
     *
     * @memberof PIXI.DisplayObject#
     * @member {string} name
     */
    name: string;

    /**
     * Returns the global position of the displayObject. Does not depend on object scale, rotation and pivot.
     *
     * @method getGlobalPosition
     * @memberof PIXI.DisplayObject#
     * @param {PIXI.Point} [point=new PIXI.Point()] - The point to write the global value to.
     * @param {boolean} [skipUpdate=false] - Setting to true will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @return {PIXI.Point} The updated point.
     */
    getGlobalPosition(point?: PIXI.Point, skipUpdate?: boolean): PIXI.Point;
  }

  /**
   * Renderer for Particles that is designer for speed over feature set.
   *
   * @class
   * @memberof PIXI
   */
  class ParticleRenderer {
    constructor(renderer: PIXI.Renderer);

    /**
     * The default shader that is used if a sprite doesn't have a more specific one.
     *
     * @member {PIXI.Shader} PIXI.ParticleRenderer#shader
     */
    shader: PIXI.Shader;
    /**
     * The WebGL state in which this renderer will work.
     *
     * @member {PIXI.State} PIXI.ParticleRenderer#state
     * @readonly
     */
    readonly state: PIXI.State;

    /**
     * Renders the particle container object.
     *
     * @param {PIXI.ParticleContainer} container - The container to render using this ParticleRenderer
     */
    render(container: PIXI.ParticleContainer): void;

    /**
     * Uploads the vertices.
     *
     * @param {PIXI.DisplayObject[]} children - the array of display objects to render
     * @param {number} startIndex - the index to start from in the children array
     * @param {number} amount - the amount of children that will have their vertices uploaded
     * @param {number[]} array - The vertices to upload.
     * @param {number} stride - Stride to use for iteration.
     * @param {number} offset - Offset to start at.
     */
    uploadVertices(
      children: PIXI.DisplayObject[],
      startIndex: number,
      amount: number,
      array: number[],
      stride: number,
      offset: number,
    ): void;

    /**
     * Uploads the position.
     *
     * @param {PIXI.DisplayObject[]} children - the array of display objects to render
     * @param {number} startIndex - the index to start from in the children array
     * @param {number} amount - the amount of children that will have their positions uploaded
     * @param {number[]} array - The vertices to upload.
     * @param {number} stride - Stride to use for iteration.
     * @param {number} offset - Offset to start at.
     */
    uploadPosition(
      children: PIXI.DisplayObject[],
      startIndex: number,
      amount: number,
      array: number[],
      stride: number,
      offset: number,
    ): void;

    /**
     * Uploads the rotiation.
     *
     * @param {PIXI.DisplayObject[]} children - the array of display objects to render
     * @param {number} startIndex - the index to start from in the children array
     * @param {number} amount - the amount of children that will have their rotation uploaded
     * @param {number[]} array - The vertices to upload.
     * @param {number} stride - Stride to use for iteration.
     * @param {number} offset - Offset to start at.
     */
    uploadRotation(
      children: PIXI.DisplayObject[],
      startIndex: number,
      amount: number,
      array: number[],
      stride: number,
      offset: number,
    ): void;

    /**
     * Uploads the Uvs
     *
     * @param {PIXI.DisplayObject[]} children - the array of display objects to render
     * @param {number} startIndex - the index to start from in the children array
     * @param {number} amount - the amount of children that will have their rotation uploaded
     * @param {number[]} array - The vertices to upload.
     * @param {number} stride - Stride to use for iteration.
     * @param {number} offset - Offset to start at.
     */
    uploadUvs(
      children: PIXI.DisplayObject[],
      startIndex: number,
      amount: number,
      array: number[],
      stride: number,
      offset: number,
    ): void;

    /**
     * Uploads the tint.
     *
     * @param {PIXI.DisplayObject[]} children - the array of display objects to render
     * @param {number} startIndex - the index to start from in the children array
     * @param {number} amount - the amount of children that will have their rotation uploaded
     * @param {number[]} array - The vertices to upload.
     * @param {number} stride - Stride to use for iteration.
     * @param {number} offset - Offset to start at.
     */
    uploadTint(
      children: PIXI.DisplayObject[],
      startIndex: number,
      amount: number,
      array: number[],
      stride: number,
      offset: number,
    ): void;

    /**
     * Destroys the ParticleRenderer.
     */
    destroy(): void;
  }

  /**
   * The prepare manager provides functionality to upload content to the GPU.
   *
   * BasePrepare handles basic queuing functionality and is extended by
   * {@link PIXI.Prepare} and {@link PIXI.CanvasPrepare}
   * to provide preparation capabilities specific to their respective renderers.
   *
   * @example
   * // Create a sprite
   * const sprite = PIXI.Sprite.from('something.png');
   *
   * // Load object into GPU
   * app.renderer.plugins.prepare.upload(sprite, () => {
   *
   *     //Texture(s) has been uploaded to GPU
   *     app.stage.addChild(sprite);
   *
   * })
   *
   * @abstract
   * @class
   * @memberof PIXI
   */
  class BasePrepare {
    constructor(renderer: PIXI.AbstractRenderer);

    /**
     * The limiter to be used to control how quickly items are prepared.
     * @type {PIXI.CountLimiter|PIXI.TimeLimiter}
     */
    limiter: PIXI.CountLimiter | PIXI.TimeLimiter;
    /**
     * Reference to the renderer.
     * @type {PIXI.AbstractRenderer}
     * @protected
     */
    protected renderer: PIXI.AbstractRenderer;
    /**
     * The only real difference between CanvasPrepare and Prepare is what they pass
     * to upload hooks. That different parameter is stored here.
     * @type {object}
     * @protected
     */
    protected uploadHookHelper: any;

    /**
     * Upload all the textures and graphics to the GPU.
     *
     * @param {Function|PIXI.DisplayObject|PIXI.Container|PIXI.BaseTexture|PIXI.Texture|PIXI.Graphics|PIXI.Text} item -
     *        Either the container or display object to search for items to upload, the items to upload themselves,
     *        or the callback function, if items have been added using `prepare.add`.
     * @param {Function} [done] - Optional callback when all queued uploads have completed
     */
    upload(
      item:
        | ((...params: any[]) => any)
        | PIXI.DisplayObject
        | PIXI.Container
        | PIXI.BaseTexture
        | PIXI.Texture
        | PIXI.Graphics
        | PIXI.Text,
      done?: (...params: any[]) => any,
    ): void;

    /**
     * Adds hooks for finding items.
     *
     * @param {Function} addHook - Function call that takes two parameters: `item:*, queue:Array`
     *          function must return `true` if it was able to add item to the queue.
     * @return {this} Instance of plugin for chaining.
     */
    registerFindHook(addHook: (...params: any[]) => any): this;

    /**
     * Adds hooks for uploading items.
     *
     * @param {Function} uploadHook - Function call that takes two parameters: `prepare:CanvasPrepare, item:*` and
     *          function must return `true` if it was able to handle upload of item.
     * @return {this} Instance of plugin for chaining.
     */
    registerUploadHook(uploadHook: (...params: any[]) => any): this;

    /**
     * Manually add an item to the uploading queue.
     *
     * @param {PIXI.DisplayObject|PIXI.Container|PIXI.BaseTexture|PIXI.Texture|PIXI.Graphics|PIXI.Text|*} item - Object to
     *        add to the queue
     * @return {this} Instance of plugin for chaining.
     */
    add(
      item: PIXI.DisplayObject | PIXI.Container | PIXI.BaseTexture | PIXI.Texture | PIXI.Graphics | PIXI.Text | any,
    ): this;

    /**
     * Destroys the plugin, don't use after this.
     *
     */
    destroy(): void;
  }

  /**
   * CountLimiter limits the number of items handled by a {@link PIXI.BasePrepare} to a specified
   * number of items per frame.
   *
   * @class
   * @memberof PIXI
   */
  class CountLimiter {
    constructor(maxItemsPerFrame: number);

    /**
     * Resets any counting properties to start fresh on a new frame.
     */
    beginFrame(): void;

    /**
     * Checks to see if another item can be uploaded. This should only be called once per item.
     * @return {boolean} If the item is allowed to be uploaded.
     */
    allowedToUpload(): boolean;
  }

  /**
   * The prepare plugin provides renderer-specific plugins for pre-rendering DisplayObjects. These plugins are useful for
   * asynchronously preparing and uploading to the GPU assets, textures, graphics waiting to be displayed.
   *
   * Do not instantiate this plugin directly. It is available from the `renderer.plugins` property.
   * See {@link PIXI.CanvasRenderer#plugins} or {@link PIXI.Renderer#plugins}.
   * @example
   * // Create a new application
   * const app = new PIXI.Application();
   * document.body.appendChild(app.view);
   *
   * // Don't start rendering right away
   * app.stop();
   *
   * // create a display object
   * const rect = new PIXI.Graphics()
   *     .beginFill(0x00ff00)
   *     .drawRect(40, 40, 200, 200);
   *
   * // Add to the stage
   * app.stage.addChild(rect);
   *
   * // Don't start rendering until the graphic is uploaded to the GPU
   * app.renderer.plugins.prepare.upload(app.stage, () => {
   *     app.start();
   * });
   *
   * @class
   * @extends PIXI.BasePrepare
   * @memberof PIXI
   */
  class Prepare extends PIXI.BasePrepare {
    constructor(renderer: PIXI.Renderer);

    /**
     * The limiter to be used to control how quickly items are prepared.
     * @type {PIXI.CountLimiter|PIXI.TimeLimiter}
     */
    limiter: PIXI.CountLimiter | PIXI.TimeLimiter;
    /**
     * Reference to the renderer.
     * @type {PIXI.AbstractRenderer}
     * @protected
     */
    protected renderer: PIXI.AbstractRenderer;
    /**
     * The only real difference between CanvasPrepare and Prepare is what they pass
     * to upload hooks. That different parameter is stored here.
     * @type {object}
     * @protected
     */
    protected uploadHookHelper: any;

    /**
     * Upload all the textures and graphics to the GPU.
     *
     * @param {Function|PIXI.DisplayObject|PIXI.Container|PIXI.BaseTexture|PIXI.Texture|PIXI.Graphics|PIXI.Text} item -
     *        Either the container or display object to search for items to upload, the items to upload themselves,
     *        or the callback function, if items have been added using `prepare.add`.
     * @param {Function} [done] - Optional callback when all queued uploads have completed
     */
    upload(
      item:
        | ((...params: any[]) => any)
        | PIXI.DisplayObject
        | PIXI.Container
        | PIXI.BaseTexture
        | PIXI.Texture
        | PIXI.Graphics
        | PIXI.Text,
      done?: (...params: any[]) => any,
    ): void;

    /**
     * Adds hooks for finding items.
     *
     * @param {Function} addHook - Function call that takes two parameters: `item:*, queue:Array`
     *          function must return `true` if it was able to add item to the queue.
     * @return {this} Instance of plugin for chaining.
     */
    registerFindHook(addHook: (...params: any[]) => any): this;

    /**
     * Adds hooks for uploading items.
     *
     * @param {Function} uploadHook - Function call that takes two parameters: `prepare:CanvasPrepare, item:*` and
     *          function must return `true` if it was able to handle upload of item.
     * @return {this} Instance of plugin for chaining.
     */
    registerUploadHook(uploadHook: (...params: any[]) => any): this;

    /**
     * Manually add an item to the uploading queue.
     *
     * @param {PIXI.DisplayObject|PIXI.Container|PIXI.BaseTexture|PIXI.Texture|PIXI.Graphics|PIXI.Text|*} item - Object to
     *        add to the queue
     * @return {this} Instance of plugin for chaining.
     */
    add(
      item: PIXI.DisplayObject | PIXI.Container | PIXI.BaseTexture | PIXI.Texture | PIXI.Graphics | PIXI.Text | any,
    ): this;

    /**
     * Destroys the plugin, don't use after this.
     *
     */
    destroy(): void;
  }

  /**
   * TimeLimiter limits the number of items handled by a {@link PIXI.BasePrepare} to a specified
   * number of milliseconds per frame.
   *
   * @class
   * @memberof PIXI
   */
  class TimeLimiter {
    constructor(maxMilliseconds: number);

    /**
     * Resets any counting properties to start fresh on a new frame.
     */
    beginFrame(): void;

    /**
     * Checks to see if another item can be uploaded. This should only be called once per item.
     * @return {boolean} If the item is allowed to be uploaded.
     */
    allowedToUpload(): boolean;
  }

  /**
   * A Runner is a highly performant and simple alternative to signals. Best used in situations
   * where events are dispatched to many objects at high frequency (say every frame!)
   *
   *
   * like a signal..
   * ```
   * import { Runner } from '@pixi/runner';
   *
   * const myObject = {
   *     loaded: new Runner('loaded')
   * }
   *
   * const listener = {
   *     loaded: function(){
   *         // thin
   *     }
   * }
   *
   * myObject.update.add(listener);
   *
   * myObject.loaded.emit();
   * ```
   *
   * Or for handling calling the same function on many items
   * ```
   * import { Runner } from '@pixi/runner';
   *
   * const myGame = {
   *     update: new Runner('update')
   * }
   *
   * const gameObject = {
   *     update: function(time){
   *         // update my gamey state
   *     }
   * }
   *
   * myGame.update.add(gameObject1);
   *
   * myGame.update.emit(time);
   * ```
   * @class
   * @memberof PIXI
   */
  class Runner {
    constructor(name: string);

    /**
     * Dispatch/Broadcast Runner to all listeners added to the queue.
     * @param {...any} params - optional parameters to pass to each listener
     * @return {PIXI.Runner}
     */
    emit(...params: any[]): PIXI.Runner;

    /**
     * Add a listener to the Runner
     *
     * Runners do not need to have scope or functions passed to them.
     * All that is required is to pass the listening object and ensure that it has contains a function that has the same name
     * as the name provided to the Runner when it was created.
     *
     * Eg A listener passed to this Runner will require a 'complete' function.
     *
     * ```
     * import { Runner } from '@pixi/runner';
     *
     * const complete = new Runner('complete');
     * ```
     *
     * The scope used will be the object itself.
     *
     * @param {any} item - The object that will be listening.
     * @return {PIXI.Runner}
     */
    add(item: any): PIXI.Runner;

    /**
     * Remove a single listener from the dispatch queue.
     * @param {any} item - The listenr that you would like to remove.
     * @return {PIXI.Runner}
     */
    remove(item: any): PIXI.Runner;

    /**
     * Check to see if the listener is already in the Runner
     * @param {any} item - The listener that you would like to check.
     */
    contains(item: any): void;

    /**
     * Remove all listeners from the Runner
     * @return {PIXI.Runner}
     */
    removeAll(): PIXI.Runner;

    /**
     * Remove all references, don't use after this.
     */
    destroy(): void;

    /**
     * `true` if there are no this Runner contains no listeners
     *
     * @member {boolean}
     * @readonly
     */
    readonly empty: boolean;
    /**
     * The name of the runner.
     *
     * @member {string}
     * @readonly
     */
    readonly name: string;

    /**
     * Alias for `emit`
     * @memberof PIXI.Runner#
     * @method dispatch
     * @see PIXI.Runner#emit
     */
    dispatch(): void;

    /**
     * Alias for `emit`
     * @memberof PIXI.Runner#
     * @method run
     * @see PIXI.Runner#emit
     */
    run(): void;
  }

  /**
   * User's customizable globals for overriding the default PIXI settings, such
   * as a renderer's default resolution, framerate, float precision, etc.
   * @example
   * // Use the native window resolution as the default resolution
   * // will support high-density displays when rendering
   * PIXI.settings.RESOLUTION = window.devicePixelRatio;
   *
   * // Disable interpolation when scaling, will make texture be pixelated
   * PIXI.settings.SCALE_MODE = PIXI.SCALE_MODES.NEAREST;
   * @namespace PIXI.settings
   */
  namespace settings {
    /**
     * The maximum support for using WebGL. If a device does not
     * support WebGL version, for instance WebGL 2, it will still
     * attempt to fallback support to WebGL 1. If you want to
     * explicitly remove feature support to target a more stable
     * baseline, prefer a lower environment.
     *
     * Due to {@link https://bugs.chromium.org/p/chromium/issues/detail?id=934823|bug in chromium}
     * we disable webgl2 by default for all non-apple mobile devices.
     *
     * @static
     * @name PREFER_ENV
     * @memberof PIXI.settings
     * @type {number}
     * @default PIXI.ENV.WEBGL2
     */
    var PREFER_ENV: number;
    /**
     * If set to `true`, Textures and BaseTexture objects stored
     * in the caches ({@link PIXI.utils.TextureCache TextureCache} and
     * {@link PIXI.utils.BaseTextureCache BaseTextureCache}) can *only* be
     * used when calling {@link PIXI.Texture.from Texture.from} or
     * {@link PIXI.BaseTexture.from BaseTexture.from}.
     * Otherwise, these `from` calls throw an exception. Using this property
     * can be useful if you want to enforce preloading all assets with
     * {@link PIXI.Loader Loader}.
     *
     * @static
     * @name STRICT_TEXTURE_CACHE
     * @memberof PIXI.settings
     * @type {boolean}
     * @default false
     */
    var STRICT_TEXTURE_CACHE: boolean;
    /**
     * Sets the default value for the container property 'sortableChildren'.
     * If set to true, the container will sort its children by zIndex value
     * when updateTransform() is called, or manually if sortChildren() is called.
     *
     * This actually changes the order of elements in the array, so should be treated
     * as a basic solution that is not performant compared to other solutions,
     * such as @link https://github.com/pixijs/pixi-display
     *
     * Also be aware of that this may not work nicely with the addChildAt() function,
     * as the zIndex sorting may cause the child to automatically sorted to another position.
     *
     * @static
     * @constant
     * @name SORTABLE_CHILDREN
     * @memberof PIXI.settings
     * @type {boolean}
     * @default false
     */
    var SORTABLE_CHILDREN: boolean;
    /**
     * Default number of uploads per frame using prepare plugin.
     *
     * @static
     * @memberof PIXI.settings
     * @name UPLOADS_PER_FRAME
     * @type {number}
     * @default 4
     */
    var UPLOADS_PER_FRAME: number;
    /**
     * If set to true WebGL will attempt make textures mimpaped by default.
     * Mipmapping will only succeed if the base texture uploaded has power of two dimensions.
     *
     * @static
     * @name MIPMAP_TEXTURES
     * @memberof PIXI.settings
     * @type {PIXI.MIPMAP_MODES}
     * @default PIXI.MIPMAP_MODES.POW2
     */
    var MIPMAP_TEXTURES: PIXI.MIPMAP_MODES;
    /**
     * Default anisotropic filtering level of textures.
     * Usually from 0 to 16
     *
     * @static
     * @name ANISOTROPIC_LEVEL
     * @memberof PIXI.settings
     * @type {number}
     * @default 0
     */
    var ANISOTROPIC_LEVEL: number;
    /**
     * Default resolution / device pixel ratio of the renderer.
     *
     * @static
     * @name RESOLUTION
     * @memberof PIXI.settings
     * @type {number}
     * @default 1
     */
    var RESOLUTION: number;
    /**
     * Default filter resolution.
     *
     * @static
     * @name FILTER_RESOLUTION
     * @memberof PIXI.settings
     * @type {number}
     * @default 1
     */
    var FILTER_RESOLUTION: number;
    /**
     * The maximum textures that this device supports.
     *
     * @static
     * @name SPRITE_MAX_TEXTURES
     * @memberof PIXI.settings
     * @type {number}
     * @default 32
     */
    var SPRITE_MAX_TEXTURES: number;
    /**
     * The default sprite batch size.
     *
     * The default aims to balance desktop and mobile devices.
     *
     * @static
     * @name SPRITE_BATCH_SIZE
     * @memberof PIXI.settings
     * @type {number}
     * @default 4096
     */
    var SPRITE_BATCH_SIZE: number;
    /**
     * The default render options if none are supplied to {@link PIXI.Renderer}
     * or {@link PIXI.CanvasRenderer}.
     *
     * @static
     * @name RENDER_OPTIONS
     * @memberof PIXI.settings
     * @type {object}
     * @property {HTMLCanvasElement} view=null
     * @property {number} resolution=1
     * @property {boolean} antialias=false
     * @property {boolean} forceFXAA=false
     * @property {boolean} autoDensity=false
     * @property {boolean} transparent=false
     * @property {number} backgroundColor=0x000000
     * @property {boolean} clearBeforeRender=true
     * @property {boolean} preserveDrawingBuffer=false
     * @property {number} width=800
     * @property {number} height=600
     * @property {boolean} legacy=false
     */
    var RENDER_OPTIONS: {
      view: HTMLCanvasElement;
      resolution: number;
      antialias: boolean;
      forceFXAA: boolean;
      autoDensity: boolean;
      transparent: boolean;
      backgroundColor: number;
      clearBeforeRender: boolean;
      preserveDrawingBuffer: boolean;
      width: number;
      height: number;
      legacy: boolean;
    };
    /**
     * Default Garbage Collection mode.
     *
     * @static
     * @name GC_MODE
     * @memberof PIXI.settings
     * @type {PIXI.GC_MODES}
     * @default PIXI.GC_MODES.AUTO
     */
    var GC_MODE: PIXI.GC_MODES;
    /**
     * Default Garbage Collection max idle.
     *
     * @static
     * @name GC_MAX_IDLE
     * @memberof PIXI.settings
     * @type {number}
     * @default 3600
     */
    var GC_MAX_IDLE: number;
    /**
     * Default Garbage Collection maximum check count.
     *
     * @static
     * @name GC_MAX_CHECK_COUNT
     * @memberof PIXI.settings
     * @type {number}
     * @default 600
     */
    var GC_MAX_CHECK_COUNT: number;
    /**
     * Default wrap modes that are supported by pixi.
     *
     * @static
     * @name WRAP_MODE
     * @memberof PIXI.settings
     * @type {PIXI.WRAP_MODES}
     * @default PIXI.WRAP_MODES.CLAMP
     */
    var WRAP_MODE: PIXI.WRAP_MODES;
    /**
     * Default scale mode for textures.
     *
     * @static
     * @name SCALE_MODE
     * @memberof PIXI.settings
     * @type {PIXI.SCALE_MODES}
     * @default PIXI.SCALE_MODES.LINEAR
     */
    var SCALE_MODE: PIXI.SCALE_MODES;
    /**
     * Default specify float precision in vertex shader.
     *
     * @static
     * @name PRECISION_VERTEX
     * @memberof PIXI.settings
     * @type {PIXI.PRECISION}
     * @default PIXI.PRECISION.HIGH
     */
    var PRECISION_VERTEX: PIXI.PRECISION;
    /**
     * Default specify float precision in fragment shader.
     * iOS is best set at highp due to https://github.com/pixijs/pixi.js/issues/3742
     *
     * @static
     * @name PRECISION_FRAGMENT
     * @memberof PIXI.settings
     * @type {PIXI.PRECISION}
     * @default PIXI.PRECISION.MEDIUM
     */
    var PRECISION_FRAGMENT: PIXI.PRECISION;
    /**
     * Can we upload the same buffer in a single frame?
     *
     * @static
     * @name CAN_UPLOAD_SAME_BUFFER
     * @memberof PIXI.settings
     * @type {boolean}
     */
    var CAN_UPLOAD_SAME_BUFFER: boolean;
    /**
     * Enables bitmap creation before image load. This feature is experimental.
     *
     * @static
     * @name CREATE_IMAGE_BITMAP
     * @memberof PIXI.settings
     * @type {boolean}
     * @default false
     */
    var CREATE_IMAGE_BITMAP: boolean;
    /**
     * If true PixiJS will Math.floor() x/y values when rendering, stopping pixel interpolation.
     * Advantages can include sharper image quality (like text) and faster rendering on canvas.
     * The main disadvantage is movement of objects may appear less smooth.
     *
     * @static
     * @constant
     * @memberof PIXI.settings
     * @type {boolean}
     * @default false
     */
    var ROUND_PIXELS: boolean;
    /**
     * Target frames per millisecond.
     *
     * @static
     * @name TARGET_FPMS
     * @memberof PIXI.settings
     * @type {number}
     * @default 0.06
     */
    var TARGET_FPMS: number;
    /**
     * The prefix that denotes a URL is for a retina asset.
     *
     * @static
     * @name RETINA_PREFIX
     * @memberof PIXI.settings
     * @type {RegExp}
     * @default /@([0-9\.]+)x/
     * @example `@2x`
     */
    var RETINA_PREFIX: RegExp;
    /**
     * Should the `failIfMajorPerformanceCaveat` flag be enabled as a context option used in the `isWebGLSupported` function.
     * For most scenarios this should be left as true, as otherwise the user may have a poor experience.
     * However, it can be useful to disable under certain scenarios, such as headless unit tests.
     *
     * @static
     * @name FAIL_IF_MAJOR_PERFORMANCE_CAVEAT
     * @memberof PIXI.settings
     * @type {boolean}
     * @default true
     */
    var FAIL_IF_MAJOR_PERFORMANCE_CAVEAT: boolean;
  }

  /**
   * The Sprite object is the base for all textured objects that are rendered to the screen
   *
   * A sprite can be created directly from an image like this:
   *
   * ```js
   * let sprite = PIXI.Sprite.from('assets/image.png');
   * ```
   *
   * The more efficient way to create sprites is using a {@link PIXI.Spritesheet},
   * as swapping base textures when rendering to the screen is inefficient.
   *
   * ```js
   * PIXI.Loader.shared.add("assets/spritesheet.json").load(setup);
   *
   * function setup() {
   *   let sheet = PIXI.Loader.shared.resources["assets/spritesheet.json"].spritesheet;
   *   let sprite = new PIXI.Sprite(sheet.textures["image.png"]);
   *   ...
   * }
   * ```
   *
   * @class
   * @extends PIXI.Container
   * @memberof PIXI
   */
  class Sprite extends PIXI.Container {
    constructor(texture?: PIXI.Texture);

    /**
     * The blend mode to be applied to the sprite. Apply a value of `PIXI.BLEND_MODES.NORMAL` to reset the blend mode.
     *
     * @member {number} PIXI.Sprite#blendMode
     * @default PIXI.BLEND_MODES.NORMAL
     * @see PIXI.BLEND_MODES
     */
    blendMode: number;
    /**
     * The shader that will be used to render the sprite. Set to null to remove a current shader.
     *
     * @member {PIXI.Filter|PIXI.Shader} PIXI.Sprite#shader
     */
    shader: PIXI.Filter | PIXI.Shader;
    /**
     * Cached tint value so we can tell when the tint is changed.
     * Value is used for 2d CanvasRenderer.
     *
     * @protected
     * @member {number} PIXI.Sprite#_cachedTint
     * @default 0xFFFFFF
     */
    protected _cachedTint: number;
    /**
     * Plugin that is responsible for rendering this element.
     * Allows to customize the rendering process without overriding '_render' & '_renderCanvas' methods.
     *
     * @member {string} PIXI.Sprite#pluginName
     * @default 'batch'
     */
    pluginName: string;
    /**
     * used to fast check if a sprite is.. a sprite!
     * @member {boolean} PIXI.Sprite#isSprite
     */
    isSprite: boolean;

    /**
     * calculates worldTransform * vertices, store it in vertexData
     */
    calculateVertices(): void;

    /**
     * calculates worldTransform * vertices for a non texture with a trim. store it in vertexTrimmedData
     * This is used to ensure that the true width and height of a trimmed texture is respected
     */
    calculateTrimmedVertices(): void;

    /**
     *
     * Renders the object using the WebGL renderer
     *
     * @protected
     * @param {PIXI.Renderer} renderer - The webgl renderer to use.
     */
    protected _render(renderer: PIXI.Renderer): void;

    /**
     * Updates the bounds of the sprite.
     *
     * @protected
     */
    protected _calculateBounds(): void;

    /**
     * Gets the local bounds of the sprite object.
     *
     * @param {PIXI.Rectangle} [rect] - The output rectangle.
     * @return {PIXI.Rectangle} The bounds.
     */
    getLocalBounds(rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Tests if a point is inside this sprite
     *
     * @param {PIXI.Point} point - the point to test
     * @return {boolean} the result of the test
     */
    containsPoint(point: PIXI.Point): boolean;

    /**
     * Destroys this sprite and optionally its texture and children
     *
     * @param {object|boolean} [options] - Options parameter. A boolean will act as if all options
     *  have been set to that value
     * @param {boolean} [options.children=false] - if set to true, all the children will have their destroy
     *      method called as well. 'options' will be passed on to those calls.
     * @param {boolean} [options.texture=false] - Should it destroy the current texture of the sprite as well
     * @param {boolean} [options.baseTexture=false] - Should it destroy the base texture of the sprite as well
     */
    destroy(options?: { children?: boolean; texture?: boolean; baseTexture?: boolean }): void;

    /**
     * Helper function that creates a new sprite based on the source you provide.
     * The source can be - frame id, image url, video url, canvas element, video element, base texture
     *
     * @static
     * @param {string|PIXI.Texture|HTMLCanvasElement|HTMLVideoElement} source Source to create texture from
     * @param {object} [options] See {@link PIXI.BaseTexture}'s constructor for options.
     * @return {PIXI.Sprite} The newly created sprite
     */
    static from(source: string | PIXI.Texture | HTMLCanvasElement | HTMLVideoElement, options?: any): PIXI.Sprite;

    /**
     * If true PixiJS will Math.floor() x/y values when rendering, stopping pixel interpolation.
     * Advantages can include sharper image quality (like text) and faster rendering on canvas.
     * The main disadvantage is movement of objects may appear less smooth.
     * To set the global default, change {@link PIXI.settings.ROUND_PIXELS}
     *
     * @member {boolean}
     * @default false
     */
    roundPixels: boolean;
    /**
     * The width of the sprite, setting this will actually modify the scale to achieve the value set
     *
     * @member {number}
     */
    width: number;
    /**
     * The height of the sprite, setting this will actually modify the scale to achieve the value set
     *
     * @member {number}
     */
    height: number;
    /**
     * The anchor sets the origin point of the text. The default value is taken from the {@link PIXI.Texture|Texture}
     * and passed to the constructor.
     *
     * The default is `(0,0)`, this means the text's origin is the top left.
     *
     * Setting the anchor to `(0.5,0.5)` means the text's origin is centered.
     *
     * Setting the anchor to `(1,1)` would mean the text's origin point will be the bottom right corner.
     *
     * If you pass only single parameter, it will set both x and y to the same value as shown in the example below.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.anchor.set(0.5); // This will set the origin to center. (0.5) is same as (0.5, 0.5).
     *
     * @member {PIXI.ObservablePoint}
     */
    anchor: PIXI.ObservablePoint;
    /**
     * The tint applied to the sprite. This is a hex value.
     * A value of 0xFFFFFF will remove any tint effect.
     *
     * @member {number}
     * @default 0xFFFFFF
     */
    tint: number;
    /**
     * The texture that the sprite is using
     *
     * @member {PIXI.Texture}
     */
    texture: PIXI.Texture;
    /**
     * The array of children of this container.
     *
     * @member {PIXI.DisplayObject[]} PIXI.Container#children
     * @readonly
     */
    readonly children: PIXI.DisplayObject[];
    /**
     * If set to true, the container will sort its children by zIndex value
     * when updateTransform() is called, or manually if sortChildren() is called.
     *
     * This actually changes the order of elements in the array, so should be treated
     * as a basic solution that is not performant compared to other solutions,
     * such as @link https://github.com/pixijs/pixi-display
     *
     * Also be aware of that this may not work nicely with the addChildAt() function,
     * as the zIndex sorting may cause the child to automatically sorted to another position.
     *
     * @see PIXI.settings.SORTABLE_CHILDREN
     *
     * @member {boolean} PIXI.Container#sortableChildren
     */
    sortableChildren: boolean;
    /**
     * Should children be sorted by zIndex at the next updateTransform call.
     * Will get automatically set to true if a new child is added, or if a child's zIndex changes.
     *
     * @member {boolean} PIXI.Container#sortDirty
     */
    sortDirty: boolean;

    /**
     * Overridable method that can be used by Container subclasses whenever the children array is modified
     *
     * @protected
     */
    protected onChildrenChange(): void;

    /**
     * Adds one or more children to the container.
     *
     * Multiple items can be added like so: `myContainer.addChild(thingOne, thingTwo, thingThree)`
     *
     * @param {...PIXI.DisplayObject} child - The DisplayObject(s) to add to the container
     * @return {PIXI.DisplayObject} The first child that was added.
     */
    addChild<TChildren extends PIXI.DisplayObject[]>(...child: TChildren): TChildren[0];

    /**
     * Adds a child to the container at a specified index. If the index is out of bounds an error will be thrown
     *
     * @param {PIXI.DisplayObject} child - The child to add
     * @param {number} index - The index to place the child in
     * @return {PIXI.DisplayObject} The child that was added.
     */
    addChildAt<T extends PIXI.DisplayObject>(child: T, index: number): T;

    /**
     * Swaps the position of 2 Display Objects within this container.
     *
     * @param {PIXI.DisplayObject} child - First display object to swap
     * @param {PIXI.DisplayObject} child2 - Second display object to swap
     */
    swapChildren(child: PIXI.DisplayObject, child2: PIXI.DisplayObject): void;

    /**
     * Returns the index position of a child DisplayObject instance
     *
     * @param {PIXI.DisplayObject} child - The DisplayObject instance to identify
     * @return {number} The index position of the child display object to identify
     */
    getChildIndex(child: PIXI.DisplayObject): number;

    /**
     * Changes the position of an existing child in the display object container
     *
     * @param {PIXI.DisplayObject} child - The child DisplayObject instance for which you want to change the index number
     * @param {number} index - The resulting index number for the child display object
     */
    setChildIndex(child: PIXI.DisplayObject, index: number): void;

    /**
     * Returns the child at the specified index
     *
     * @param {number} index - The index to get the child at
     * @return {PIXI.DisplayObject} The child at the given index, if any.
     */
    getChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes one or more children from the container.
     *
     * @param {...PIXI.DisplayObject} child - The DisplayObject(s) to remove
     * @return {PIXI.DisplayObject} The first child that was removed.
     */
    removeChild<TChildren extends PIXI.DisplayObject[]>(...child: TChildren): TChildren[0];

    /**
     * Removes a child from the specified index position.
     *
     * @param {number} index - The index to get the child from
     * @return {PIXI.DisplayObject} The child that was removed.
     */
    removeChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes all children from this container that are within the begin and end indexes.
     *
     * @param {number} [beginIndex=0] - The beginning position.
     * @param {number} [endIndex=this.children.length] - The ending position. Default value is size of the container.
     * @returns {PIXI.DisplayObject[]} List of removed children
     */
    removeChildren(beginIndex?: number, endIndex?: number): PIXI.DisplayObject[];

    /**
     * Sorts children by zIndex. Previous order is mantained for 2 children with the same zIndex.
     */
    sortChildren(): void;

    /**
     * Updates the transform on all children of this container for rendering
     */
    updateTransform(): void;

    /**
     * Recalculates the bounds of the container.
     *
     */
    calculateBounds(): void;

    /**
     * Renders the object using the WebGL renderer
     *
     * @param {PIXI.Renderer} renderer - The renderer
     */
    render(renderer: PIXI.Renderer): void;

    /**
     * Render the object using the WebGL renderer and advanced features.
     *
     * @protected
     * @param {PIXI.Renderer} renderer - The renderer
     */
    protected renderAdvanced(renderer: PIXI.Renderer): void;

    /**
     * Determines if the children to the displayObject can be clicked/touched
     * Setting this to false allows PixiJS to bypass a recursive `hitTest` function
     *
     * @member {boolean}
     * @memberof PIXI.Container#
     */
    interactiveChildren: boolean;

    /**
     * Returns the display object in the container.
     *
     * @method getChildByName
     * @memberof PIXI.Container#
     * @param {string} name - Instance name.
     * @return {PIXI.DisplayObject} The child with the specified name.
     */
    getChildByName(name: string): PIXI.DisplayObject;

    /**
     *  Flag for if the object is accessible. If true AccessibilityManager will overlay a
     *   shadow div with attributes set
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    accessible: boolean;
    /**
     * Sets the title attribute of the shadow div
     * If accessibleTitle AND accessibleHint has not been this will default to 'displayObject [tabIndex]'
     *
     * @member {?string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleTitle: string;
    /**
     * Sets the aria-label attribute of the shadow div
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleHint: string;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleActive: boolean;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleDiv: boolean;
    /**
     * Specify the type of div the accessible layer is. Screen readers treat the element differently
     * depending on this type. Defaults to button.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'button'
     */
    accessibleType: string;
    /**
     * Specify the pointer-events the accessible div will use
     * Defaults to auto.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'auto'
     */
    accessiblePointerEvents: string;
    /**
     * Setting to false will prevent any children inside this container to
     * be accessible. Defaults to true.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @default true
     */
    accessibleChildren: boolean;
    /**
     * World transform and local transform of this object.
     * This will become read-only later, please do not assign anything there unless you know what are you doing.
     *
     * @member {PIXI.Transform} PIXI.DisplayObject#transform
     */
    transform: PIXI.Transform;
    /**
     * The opacity of the object.
     *
     * @member {number} PIXI.DisplayObject#alpha
     */
    alpha: number;
    /**
     * The visibility of the object. If false the object will not be drawn, and
     * the updateTransform function will not be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds or call updateTransform manually.
     *
     * @member {boolean} PIXI.DisplayObject#visible
     */
    visible: boolean;
    /**
     * Can this object be rendered, if false the object will not be drawn but the updateTransform
     * methods will still be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds manually.
     *
     * @member {boolean} PIXI.DisplayObject#renderable
     */
    renderable: boolean;
    /**
     * The display object container that contains this display object.
     *
     * @member {PIXI.Container} PIXI.DisplayObject#parent
     * @readonly
     */
    readonly parent: PIXI.Container;
    /**
     * The multiplied alpha of the displayObject.
     *
     * @member {number} PIXI.DisplayObject#worldAlpha
     * @readonly
     */
    readonly worldAlpha: number;
    /**
     * Which index in the children array the display component was before the previous zIndex sort.
     * Used by containers to help sort objects with the same zIndex, by using previous array index as the decider.
     *
     * @member {number} PIXI.DisplayObject#_lastSortedIndex
     * @protected
     */
    protected _lastSortedIndex: number;
    /**
     * The zIndex of the displayObject.
     * A higher value will mean it will be rendered on top of other displayObjects within the same container.
     *
     * @member {number} PIXI.DisplayObject#_zIndex
     * @protected
     */
    protected _zIndex: number;
    /**
     * The area the filter is applied to. This is used as more of an optimization
     * rather than figuring out the dimensions of the displayObject each frame you can set this rectangle.
     *
     * Also works as an interaction mask.
     *
     * @member {?PIXI.Rectangle} PIXI.DisplayObject#filterArea
     */
    filterArea: PIXI.Rectangle;
    /**
     * Sets the filters for the displayObject.
     * * IMPORTANT: This is a WebGL only feature and will be ignored by the canvas renderer.
     * To remove filters simply set this property to `'null'`.
     *
     * @member {?PIXI.Filter[]} PIXI.DisplayObject#filters
     */
    filters: PIXI.Filter[];
    /**
     * The bounds object, this is used to calculate and store the bounds of the displayObject.
     *
     * @member {PIXI.Bounds} PIXI.DisplayObject#_bounds
     * @protected
     */
    protected _bounds: PIXI.Bounds;
    /**
     * The original, cached mask of the object.
     *
     * @member {PIXI.Graphics|PIXI.Sprite|null} PIXI.DisplayObject#_mask
     * @protected
     */
    protected _mask: PIXI.Graphics | PIXI.Sprite | null;
    /**
     * If the object has been destroyed via destroy(). If true, it should not be used.
     *
     * @member {boolean} PIXI.DisplayObject#_destroyed
     * @protected
     */
    protected _destroyed: boolean;
    /**
     * Does any other displayObject use this object as a mask?
     * @member {boolean} PIXI.DisplayObject#isMask
     */
    isMask: boolean;
    /**
     * @protected
     * @member {PIXI.DisplayObject}
     */
    protected _tempDisplayObjectParent: PIXI.DisplayObject;

    /**
     * Recursively updates transform of all objects from the root to this one
     * internal function for toLocal()
     */
    _recursivePostUpdateTransform(): void;

    /**
     * Retrieves the bounds of the displayObject as a rectangle object.
     *
     * @param {boolean} [skipUpdate] - Setting to `true` will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @param {PIXI.Rectangle} [rect] - Optional rectangle to store the result of the bounds calculation.
     * @return {PIXI.Rectangle} The rectangular bounding area.
     */
    getBounds(skipUpdate?: boolean, rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Calculates the global position of the display object.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform.
     * @return {PIXI.IPoint} A point object representing the position of this object.
     */
    toGlobal(position: PIXI.IPoint, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Calculates the local position of the display object relative to another point.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.DisplayObject} [from] - The DisplayObject to calculate the global position from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform
     * @return {PIXI.IPoint} A point object representing the position of this object
     */
    toLocal(position: PIXI.IPoint, from?: PIXI.DisplayObject, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Set the parent Container of this DisplayObject.
     *
     * @param {PIXI.Container} container - The Container to add this DisplayObject to.
     * @return {PIXI.Container} The Container that this DisplayObject was added to.
     */
    setParent(container: PIXI.Container): PIXI.Container;

    /**
     * Convenience function to set the position, scale, skew and pivot at once.
     *
     * @param {number} [x=0] - The X position
     * @param {number} [y=0] - The Y position
     * @param {number} [scaleX=1] - The X scale value
     * @param {number} [scaleY=1] - The Y scale value
     * @param {number} [rotation=0] - The rotation
     * @param {number} [skewX=0] - The X skew value
     * @param {number} [skewY=0] - The Y skew value
     * @param {number} [pivotX=0] - The X pivot value
     * @param {number} [pivotY=0] - The Y pivot value
     * @return {PIXI.DisplayObject} The DisplayObject instance
     */
    setTransform(
      x?: number,
      y?: number,
      scaleX?: number,
      scaleY?: number,
      rotation?: number,
      skewX?: number,
      skewY?: number,
      pivotX?: number,
      pivotY?: number,
    ): PIXI.DisplayObject;

    /**
     * The position of the displayObject on the x axis relative to the local coordinates of the parent.
     * An alias to position.x
     *
     * @member {number}
     */
    x: number;
    /**
     * The position of the displayObject on the y axis relative to the local coordinates of the parent.
     * An alias to position.y
     *
     * @member {number}
     */
    y: number;
    /**
     * Current transform of the object based on world (parent) factors.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly worldTransform: PIXI.Matrix;
    /**
     * Current transform of the object based on local factors: position, scale, other stuff.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly localTransform: PIXI.Matrix;
    /**
     * The coordinate of the object relative to the local coordinates of the parent.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    position: PIXI.IPoint;
    /**
     * The scale factor of the object.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    scale: PIXI.IPoint;
    /**
     * The pivot point of the displayObject that it rotates around.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    pivot: PIXI.IPoint;
    /**
     * The skew factor for the object in radians.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.ObservablePoint}
     */
    skew: PIXI.ObservablePoint;
    /**
     * The rotation of the object in radians.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    rotation: number;
    /**
     * The angle of the object in degrees.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    angle: number;
    /**
     * The zIndex of the displayObject.
     * If a container has the sortableChildren property set to true, children will be automatically
     * sorted by zIndex value; a higher value will mean it will be moved towards the end of the array,
     * and thus rendered on top of other displayObjects within the same container.
     *
     * @member {number}
     */
    zIndex: number;
    /**
     * Indicates if the object is globally visible.
     *
     * @member {boolean}
     * @readonly
     */
    readonly worldVisible: boolean;
    /**
     * Sets a mask for the displayObject. A mask is an object that limits the visibility of an
     * object to the shape of the mask applied to it. In PixiJS a regular mask must be a
     * {@link PIXI.Graphics} or a {@link PIXI.Sprite} object. This allows for much faster masking in canvas as it
     * utilities shape clipping. To remove a mask, set this property to `null`.
     *
     * For sprite mask both alpha and red channel are used. Black mask is the same as transparent mask.
     * @example
     * const graphics = new PIXI.Graphics();
     * graphics.beginFill(0xFF3300);
     * graphics.drawRect(50, 250, 100, 100);
     * graphics.endFill();
     *
     * const sprite = new PIXI.Sprite(texture);
     * sprite.mask = graphics;
     * @todo At the moment, PIXI.CanvasRenderer doesn't support PIXI.Sprite as mask.
     *
     * @member {PIXI.Container|PIXI.MaskData}
     */
    mask: PIXI.Container | PIXI.MaskData;

    /**
     * DisplayObject default updateTransform, does not update children of container.
     * Will crash if there's no parent element.
     *
     * @memberof PIXI.DisplayObject#
     * @function displayObjectUpdateTransform
     */
    displayObjectUpdateTransform(): void;

    /**
     * Enable interaction events for the DisplayObject. Touch, pointer and mouse
     * events will not be emitted unless `interactive` is set to `true`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.on('tap', (event) => {
     *    //handle event
     * });
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    interactive: boolean;
    /**
     * Interaction shape. Children will be hit first, then this shape will be checked.
     * Setting this will cause this shape to be checked in hit tests rather than the displayObject's bounds.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.hitArea = new PIXI.Rectangle(0, 0, 100, 100);
     * @member {PIXI.IHitArea}
     * @memberof PIXI.DisplayObject#
     */
    hitArea: PIXI.IHitArea;
    /**
     * If enabled, the mouse cursor use the pointer behavior when hovered over the displayObject if it is interactive
     * Setting this changes the 'cursor' property to `'pointer'`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.buttonMode = true;
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    buttonMode: boolean;
    /**
     * This defines what cursor mode is used when the mouse cursor
     * is hovered over the displayObject.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.cursor = 'wait';
     * @see https://developer.mozilla.org/en/docs/Web/CSS/cursor
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    cursor: string;
    /**
     * Set this to true if you want this display object to be cached as a bitmap.
     * This basically takes a snap shot of the display object as it is at that moment. It can
     * provide a performance benefit for complex static displayObjects.
     * To remove simply set this property to `false`
     *
     * IMPORTANT GOTCHA - Make sure that all your textures are preloaded BEFORE setting this property to true
     * as it will take a snapshot of what is currently there. If the textures have not loaded then they will not appear.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    cacheAsBitmap: boolean;
    /**
     * The instance name of the object.
     *
     * @memberof PIXI.DisplayObject#
     * @member {string} name
     */
    name: string;

    /**
     * Returns the global position of the displayObject. Does not depend on object scale, rotation and pivot.
     *
     * @method getGlobalPosition
     * @memberof PIXI.DisplayObject#
     * @param {PIXI.Point} [point=new PIXI.Point()] - The point to write the global value to.
     * @param {boolean} [skipUpdate=false] - Setting to true will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @return {PIXI.Point} The updated point.
     */
    getGlobalPosition(point?: PIXI.Point, skipUpdate?: boolean): PIXI.Point;
  }

  namespace AnimatedSprite {
    /**
     * @memberof PIXI.AnimatedSprite
     * @typedef {object} FrameObject
     * @type {object}
     * @property {PIXI.Texture} texture - The {@link PIXI.Texture} of the frame
     * @property {number} time - the duration of the frame in ms
     */
    type FrameObject = {
      texture: PIXI.Texture;
      time: number;
    };
  }

  /**
   * An AnimatedSprite is a simple way to display an animation depicted by a list of textures.
   *
   * ```js
   * let alienImages = ["image_sequence_01.png","image_sequence_02.png","image_sequence_03.png","image_sequence_04.png"];
   * let textureArray = [];
   *
   * for (let i=0; i < 4; i++)
   * {
   *      let texture = PIXI.Texture.from(alienImages[i]);
   *      textureArray.push(texture);
   * };
   *
   * let animatedSprite = new PIXI.AnimatedSprite(textureArray);
   * ```
   *
   * The more efficient and simpler way to create an animated sprite is using a {@link PIXI.Spritesheet}
   * containing the animation definitions:
   *
   * ```js
   * PIXI.Loader.shared.add("assets/spritesheet.json").load(setup);
   *
   * function setup() {
   *   let sheet = PIXI.Loader.shared.resources["assets/spritesheet.json"].spritesheet;
   *   animatedSprite = new PIXI.AnimatedSprite(sheet.animations["image_sequence"]);
   *   ...
   * }
   * ```
   *
   * @class
   * @extends PIXI.Sprite
   * @memberof PIXI
   */
  class AnimatedSprite extends PIXI.Sprite {
    constructor(textures: PIXI.Texture[] | PIXI.AnimatedSprite.FrameObject[], autoUpdate?: boolean);

    /**
     * The speed that the AnimatedSprite will play at. Higher is faster, lower is slower.
     *
     * @member {number} PIXI.AnimatedSprite#animationSpeed
     * @default 1
     */
    animationSpeed: number;
    /**
     * Whether or not the animate sprite repeats after playing.
     *
     * @member {boolean} PIXI.AnimatedSprite#loop
     * @default true
     */
    loop: boolean;
    /**
     * Update anchor to [Texture's defaultAnchor]{@link PIXI.Texture#defaultAnchor} when frame changes.
     *
     * Useful with [sprite sheet animations]{@link PIXI.Spritesheet#animations} created with tools.
     * Changing anchor for each frame allows to pin sprite origin to certain moving feature
     * of the frame (e.g. left foot).
     *
     * Note: Enabling this will override any previously set `anchor` on each frame change.
     *
     * @member {boolean} PIXI.AnimatedSprite#updateAnchor
     * @default false
     */
    updateAnchor: boolean;
    /**
     * Function to call when an AnimatedSprite finishes playing.
     *
     * @member {Function} PIXI.AnimatedSprite#onComplete
     */
    onComplete: (...params: any[]) => any;
    /**
     * Function to call when an AnimatedSprite changes which texture is being rendered.
     *
     * @member {Function} PIXI.AnimatedSprite#onFrameChange
     */
    onFrameChange: (...params: any[]) => any;
    /**
     * Function to call when `loop` is true, and an AnimatedSprite is played and loops around to start again.
     *
     * @member {Function} PIXI.AnimatedSprite#onLoop
     */
    onLoop: (...params: any[]) => any;

    /**
     * Stops the AnimatedSprite.
     *
     */
    stop(): void;

    /**
     * Plays the AnimatedSprite.
     *
     */
    play(): void;

    /**
     * Stops the AnimatedSprite and goes to a specific frame.
     *
     * @param {number} frameNumber - Frame index to stop at.
     */
    gotoAndStop(frameNumber: number): void;

    /**
     * Goes to a specific frame and begins playing the AnimatedSprite.
     *
     * @param {number} frameNumber - Frame index to start at.
     */
    gotoAndPlay(frameNumber: number): void;

    /**
     * Updates the object transform for rendering.
     *
     * @param {number} deltaTime - Time since last tick.
     */
    update(deltaTime: number): void;

    /**
     * Stops the AnimatedSprite and destroys it.
     *
     * @param {object|boolean} [options] - Options parameter. A boolean will act as if all options
     *  have been set to that value.
     * @param {boolean} [options.children=false] - If set to true, all the children will have their destroy
     *      method called as well. 'options' will be passed on to those calls.
     * @param {boolean} [options.texture=false] - Should it destroy the current texture of the sprite as well.
     * @param {boolean} [options.baseTexture=false] - Should it destroy the base texture of the sprite as well.
     */
    destroy(options?: { children?: boolean; texture?: boolean; baseTexture?: boolean }): void;

    /**
     * A short hand way of creating an AnimatedSprite from an array of frame ids.
     *
     * @static
     * @param {string[]} frames - The array of frames ids the AnimatedSprite will use as its texture frames.
     * @return {AnimatedSprite} The new animated sprite with the specified frames.
     */
    static fromFrames(frames: string[]): AnimatedSprite;

    /**
     * A short hand way of creating an AnimatedSprite from an array of image ids.
     *
     * @static
     * @param {string[]} images - The array of image urls the AnimatedSprite will use as its texture frames.
     * @return {AnimatedSprite} The new animate sprite with the specified images as frames.
     */
    static fromImages(images: string[]): AnimatedSprite;

    /**
     * The total number of frames in the AnimatedSprite. This is the same as number of textures
     * assigned to the AnimatedSprite.
     *
     * @readonly
     * @member {number}
     * @default 0
     */
    readonly totalFrames: number;
    /**
     * The array of textures used for this AnimatedSprite.
     *
     * @member {PIXI.Texture[]}
     */
    textures: PIXI.Texture[];
    /**
     * The AnimatedSprites current frame index.
     *
     * @member {number}
     * @readonly
     */
    readonly currentFrame: number;
    /**
     * Indicates if the AnimatedSprite is currently playing.
     *
     * @member {boolean}
     * @readonly
     */
    readonly playing: boolean;
    /**
     * Whether to use PIXI.Ticker.shared to auto update animation time
     *
     * @member {boolean}
     */
    autoUpdate: boolean;
    /**
     * The blend mode to be applied to the sprite. Apply a value of `PIXI.BLEND_MODES.NORMAL` to reset the blend mode.
     *
     * @member {number} PIXI.Sprite#blendMode
     * @default PIXI.BLEND_MODES.NORMAL
     * @see PIXI.BLEND_MODES
     */
    blendMode: number;
    /**
     * The shader that will be used to render the sprite. Set to null to remove a current shader.
     *
     * @member {PIXI.Filter|PIXI.Shader} PIXI.Sprite#shader
     */
    shader: PIXI.Filter | PIXI.Shader;
    /**
     * Cached tint value so we can tell when the tint is changed.
     * Value is used for 2d CanvasRenderer.
     *
     * @protected
     * @member {number} PIXI.Sprite#_cachedTint
     * @default 0xFFFFFF
     */
    protected _cachedTint: number;
    /**
     * Plugin that is responsible for rendering this element.
     * Allows to customize the rendering process without overriding '_render' & '_renderCanvas' methods.
     *
     * @member {string} PIXI.Sprite#pluginName
     * @default 'batch'
     */
    pluginName: string;
    /**
     * used to fast check if a sprite is.. a sprite!
     * @member {boolean} PIXI.Sprite#isSprite
     */
    isSprite: boolean;

    /**
     * calculates worldTransform * vertices, store it in vertexData
     */
    calculateVertices(): void;

    /**
     * calculates worldTransform * vertices for a non texture with a trim. store it in vertexTrimmedData
     * This is used to ensure that the true width and height of a trimmed texture is respected
     */
    calculateTrimmedVertices(): void;

    /**
     *
     * Renders the object using the WebGL renderer
     *
     * @protected
     * @param {PIXI.Renderer} renderer - The webgl renderer to use.
     */
    protected _render(renderer: PIXI.Renderer): void;

    /**
     * Updates the bounds of the sprite.
     *
     * @protected
     */
    protected _calculateBounds(): void;

    /**
     * Gets the local bounds of the sprite object.
     *
     * @param {PIXI.Rectangle} [rect] - The output rectangle.
     * @return {PIXI.Rectangle} The bounds.
     */
    getLocalBounds(rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Tests if a point is inside this sprite
     *
     * @param {PIXI.Point} point - the point to test
     * @return {boolean} the result of the test
     */
    containsPoint(point: PIXI.Point): boolean;

    /**
     * If true PixiJS will Math.floor() x/y values when rendering, stopping pixel interpolation.
     * Advantages can include sharper image quality (like text) and faster rendering on canvas.
     * The main disadvantage is movement of objects may appear less smooth.
     * To set the global default, change {@link PIXI.settings.ROUND_PIXELS}
     *
     * @member {boolean}
     * @default false
     */
    roundPixels: boolean;
    /**
     * The width of the sprite, setting this will actually modify the scale to achieve the value set
     *
     * @member {number}
     */
    width: number;
    /**
     * The height of the sprite, setting this will actually modify the scale to achieve the value set
     *
     * @member {number}
     */
    height: number;
    /**
     * The anchor sets the origin point of the text. The default value is taken from the {@link PIXI.Texture|Texture}
     * and passed to the constructor.
     *
     * The default is `(0,0)`, this means the text's origin is the top left.
     *
     * Setting the anchor to `(0.5,0.5)` means the text's origin is centered.
     *
     * Setting the anchor to `(1,1)` would mean the text's origin point will be the bottom right corner.
     *
     * If you pass only single parameter, it will set both x and y to the same value as shown in the example below.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.anchor.set(0.5); // This will set the origin to center. (0.5) is same as (0.5, 0.5).
     *
     * @member {PIXI.ObservablePoint}
     */
    anchor: PIXI.ObservablePoint;
    /**
     * The tint applied to the sprite. This is a hex value.
     * A value of 0xFFFFFF will remove any tint effect.
     *
     * @member {number}
     * @default 0xFFFFFF
     */
    tint: number;
    /**
     * The texture that the sprite is using
     *
     * @member {PIXI.Texture}
     */
    texture: PIXI.Texture;
    /**
     * The array of children of this container.
     *
     * @member {PIXI.DisplayObject[]} PIXI.Container#children
     * @readonly
     */
    readonly children: PIXI.DisplayObject[];
    /**
     * If set to true, the container will sort its children by zIndex value
     * when updateTransform() is called, or manually if sortChildren() is called.
     *
     * This actually changes the order of elements in the array, so should be treated
     * as a basic solution that is not performant compared to other solutions,
     * such as @link https://github.com/pixijs/pixi-display
     *
     * Also be aware of that this may not work nicely with the addChildAt() function,
     * as the zIndex sorting may cause the child to automatically sorted to another position.
     *
     * @see PIXI.settings.SORTABLE_CHILDREN
     *
     * @member {boolean} PIXI.Container#sortableChildren
     */
    sortableChildren: boolean;
    /**
     * Should children be sorted by zIndex at the next updateTransform call.
     * Will get automatically set to true if a new child is added, or if a child's zIndex changes.
     *
     * @member {boolean} PIXI.Container#sortDirty
     */
    sortDirty: boolean;

    /**
     * Overridable method that can be used by Container subclasses whenever the children array is modified
     *
     * @protected
     */
    protected onChildrenChange(): void;

    /**
     * Adds one or more children to the container.
     *
     * Multiple items can be added like so: `myContainer.addChild(thingOne, thingTwo, thingThree)`
     *
     * @param {...PIXI.DisplayObject} child - The DisplayObject(s) to add to the container
     * @return {PIXI.DisplayObject} The first child that was added.
     */
    addChild<TChildren extends PIXI.DisplayObject[]>(...child: TChildren): TChildren[0];

    /**
     * Adds a child to the container at a specified index. If the index is out of bounds an error will be thrown
     *
     * @param {PIXI.DisplayObject} child - The child to add
     * @param {number} index - The index to place the child in
     * @return {PIXI.DisplayObject} The child that was added.
     */
    addChildAt<T extends PIXI.DisplayObject>(child: T, index: number): T;

    /**
     * Swaps the position of 2 Display Objects within this container.
     *
     * @param {PIXI.DisplayObject} child - First display object to swap
     * @param {PIXI.DisplayObject} child2 - Second display object to swap
     */
    swapChildren(child: PIXI.DisplayObject, child2: PIXI.DisplayObject): void;

    /**
     * Returns the index position of a child DisplayObject instance
     *
     * @param {PIXI.DisplayObject} child - The DisplayObject instance to identify
     * @return {number} The index position of the child display object to identify
     */
    getChildIndex(child: PIXI.DisplayObject): number;

    /**
     * Changes the position of an existing child in the display object container
     *
     * @param {PIXI.DisplayObject} child - The child DisplayObject instance for which you want to change the index number
     * @param {number} index - The resulting index number for the child display object
     */
    setChildIndex(child: PIXI.DisplayObject, index: number): void;

    /**
     * Returns the child at the specified index
     *
     * @param {number} index - The index to get the child at
     * @return {PIXI.DisplayObject} The child at the given index, if any.
     */
    getChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes one or more children from the container.
     *
     * @param {...PIXI.DisplayObject} child - The DisplayObject(s) to remove
     * @return {PIXI.DisplayObject} The first child that was removed.
     */
    removeChild<TChildren extends PIXI.DisplayObject[]>(...child: TChildren): TChildren[0];

    /**
     * Removes a child from the specified index position.
     *
     * @param {number} index - The index to get the child from
     * @return {PIXI.DisplayObject} The child that was removed.
     */
    removeChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes all children from this container that are within the begin and end indexes.
     *
     * @param {number} [beginIndex=0] - The beginning position.
     * @param {number} [endIndex=this.children.length] - The ending position. Default value is size of the container.
     * @returns {PIXI.DisplayObject[]} List of removed children
     */
    removeChildren(beginIndex?: number, endIndex?: number): PIXI.DisplayObject[];

    /**
     * Sorts children by zIndex. Previous order is mantained for 2 children with the same zIndex.
     */
    sortChildren(): void;

    /**
     * Updates the transform on all children of this container for rendering
     */
    updateTransform(): void;

    /**
     * Recalculates the bounds of the container.
     *
     */
    calculateBounds(): void;

    /**
     * Renders the object using the WebGL renderer
     *
     * @param {PIXI.Renderer} renderer - The renderer
     */
    render(renderer: PIXI.Renderer): void;

    /**
     * Render the object using the WebGL renderer and advanced features.
     *
     * @protected
     * @param {PIXI.Renderer} renderer - The renderer
     */
    protected renderAdvanced(renderer: PIXI.Renderer): void;

    /**
     * Determines if the children to the displayObject can be clicked/touched
     * Setting this to false allows PixiJS to bypass a recursive `hitTest` function
     *
     * @member {boolean}
     * @memberof PIXI.Container#
     */
    interactiveChildren: boolean;

    /**
     * Returns the display object in the container.
     *
     * @method getChildByName
     * @memberof PIXI.Container#
     * @param {string} name - Instance name.
     * @return {PIXI.DisplayObject} The child with the specified name.
     */
    getChildByName(name: string): PIXI.DisplayObject;

    /**
     *  Flag for if the object is accessible. If true AccessibilityManager will overlay a
     *   shadow div with attributes set
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    accessible: boolean;
    /**
     * Sets the title attribute of the shadow div
     * If accessibleTitle AND accessibleHint has not been this will default to 'displayObject [tabIndex]'
     *
     * @member {?string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleTitle: string;
    /**
     * Sets the aria-label attribute of the shadow div
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleHint: string;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleActive: boolean;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleDiv: boolean;
    /**
     * Specify the type of div the accessible layer is. Screen readers treat the element differently
     * depending on this type. Defaults to button.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'button'
     */
    accessibleType: string;
    /**
     * Specify the pointer-events the accessible div will use
     * Defaults to auto.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'auto'
     */
    accessiblePointerEvents: string;
    /**
     * Setting to false will prevent any children inside this container to
     * be accessible. Defaults to true.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @default true
     */
    accessibleChildren: boolean;
    /**
     * World transform and local transform of this object.
     * This will become read-only later, please do not assign anything there unless you know what are you doing.
     *
     * @member {PIXI.Transform} PIXI.DisplayObject#transform
     */
    transform: PIXI.Transform;
    /**
     * The opacity of the object.
     *
     * @member {number} PIXI.DisplayObject#alpha
     */
    alpha: number;
    /**
     * The visibility of the object. If false the object will not be drawn, and
     * the updateTransform function will not be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds or call updateTransform manually.
     *
     * @member {boolean} PIXI.DisplayObject#visible
     */
    visible: boolean;
    /**
     * Can this object be rendered, if false the object will not be drawn but the updateTransform
     * methods will still be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds manually.
     *
     * @member {boolean} PIXI.DisplayObject#renderable
     */
    renderable: boolean;
    /**
     * The display object container that contains this display object.
     *
     * @member {PIXI.Container} PIXI.DisplayObject#parent
     * @readonly
     */
    readonly parent: PIXI.Container;
    /**
     * The multiplied alpha of the displayObject.
     *
     * @member {number} PIXI.DisplayObject#worldAlpha
     * @readonly
     */
    readonly worldAlpha: number;
    /**
     * Which index in the children array the display component was before the previous zIndex sort.
     * Used by containers to help sort objects with the same zIndex, by using previous array index as the decider.
     *
     * @member {number} PIXI.DisplayObject#_lastSortedIndex
     * @protected
     */
    protected _lastSortedIndex: number;
    /**
     * The zIndex of the displayObject.
     * A higher value will mean it will be rendered on top of other displayObjects within the same container.
     *
     * @member {number} PIXI.DisplayObject#_zIndex
     * @protected
     */
    protected _zIndex: number;
    /**
     * The area the filter is applied to. This is used as more of an optimization
     * rather than figuring out the dimensions of the displayObject each frame you can set this rectangle.
     *
     * Also works as an interaction mask.
     *
     * @member {?PIXI.Rectangle} PIXI.DisplayObject#filterArea
     */
    filterArea: PIXI.Rectangle;
    /**
     * Sets the filters for the displayObject.
     * * IMPORTANT: This is a WebGL only feature and will be ignored by the canvas renderer.
     * To remove filters simply set this property to `'null'`.
     *
     * @member {?PIXI.Filter[]} PIXI.DisplayObject#filters
     */
    filters: PIXI.Filter[];
    /**
     * The bounds object, this is used to calculate and store the bounds of the displayObject.
     *
     * @member {PIXI.Bounds} PIXI.DisplayObject#_bounds
     * @protected
     */
    protected _bounds: PIXI.Bounds;
    /**
     * The original, cached mask of the object.
     *
     * @member {PIXI.Graphics|PIXI.Sprite|null} PIXI.DisplayObject#_mask
     * @protected
     */
    protected _mask: PIXI.Graphics | PIXI.Sprite | null;
    /**
     * If the object has been destroyed via destroy(). If true, it should not be used.
     *
     * @member {boolean} PIXI.DisplayObject#_destroyed
     * @protected
     */
    protected _destroyed: boolean;
    /**
     * Does any other displayObject use this object as a mask?
     * @member {boolean} PIXI.DisplayObject#isMask
     */
    isMask: boolean;
    /**
     * @protected
     * @member {PIXI.DisplayObject}
     */
    protected _tempDisplayObjectParent: PIXI.DisplayObject;

    /**
     * Recursively updates transform of all objects from the root to this one
     * internal function for toLocal()
     */
    _recursivePostUpdateTransform(): void;

    /**
     * Retrieves the bounds of the displayObject as a rectangle object.
     *
     * @param {boolean} [skipUpdate] - Setting to `true` will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @param {PIXI.Rectangle} [rect] - Optional rectangle to store the result of the bounds calculation.
     * @return {PIXI.Rectangle} The rectangular bounding area.
     */
    getBounds(skipUpdate?: boolean, rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Calculates the global position of the display object.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform.
     * @return {PIXI.IPoint} A point object representing the position of this object.
     */
    toGlobal(position: PIXI.IPoint, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Calculates the local position of the display object relative to another point.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.DisplayObject} [from] - The DisplayObject to calculate the global position from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform
     * @return {PIXI.IPoint} A point object representing the position of this object
     */
    toLocal(position: PIXI.IPoint, from?: PIXI.DisplayObject, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Set the parent Container of this DisplayObject.
     *
     * @param {PIXI.Container} container - The Container to add this DisplayObject to.
     * @return {PIXI.Container} The Container that this DisplayObject was added to.
     */
    setParent(container: PIXI.Container): PIXI.Container;

    /**
     * Convenience function to set the position, scale, skew and pivot at once.
     *
     * @param {number} [x=0] - The X position
     * @param {number} [y=0] - The Y position
     * @param {number} [scaleX=1] - The X scale value
     * @param {number} [scaleY=1] - The Y scale value
     * @param {number} [rotation=0] - The rotation
     * @param {number} [skewX=0] - The X skew value
     * @param {number} [skewY=0] - The Y skew value
     * @param {number} [pivotX=0] - The X pivot value
     * @param {number} [pivotY=0] - The Y pivot value
     * @return {PIXI.DisplayObject} The DisplayObject instance
     */
    setTransform(
      x?: number,
      y?: number,
      scaleX?: number,
      scaleY?: number,
      rotation?: number,
      skewX?: number,
      skewY?: number,
      pivotX?: number,
      pivotY?: number,
    ): PIXI.DisplayObject;

    /**
     * The position of the displayObject on the x axis relative to the local coordinates of the parent.
     * An alias to position.x
     *
     * @member {number}
     */
    x: number;
    /**
     * The position of the displayObject on the y axis relative to the local coordinates of the parent.
     * An alias to position.y
     *
     * @member {number}
     */
    y: number;
    /**
     * Current transform of the object based on world (parent) factors.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly worldTransform: PIXI.Matrix;
    /**
     * Current transform of the object based on local factors: position, scale, other stuff.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly localTransform: PIXI.Matrix;
    /**
     * The coordinate of the object relative to the local coordinates of the parent.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    position: PIXI.IPoint;
    /**
     * The scale factor of the object.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    scale: PIXI.IPoint;
    /**
     * The pivot point of the displayObject that it rotates around.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    pivot: PIXI.IPoint;
    /**
     * The skew factor for the object in radians.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.ObservablePoint}
     */
    skew: PIXI.ObservablePoint;
    /**
     * The rotation of the object in radians.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    rotation: number;
    /**
     * The angle of the object in degrees.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    angle: number;
    /**
     * The zIndex of the displayObject.
     * If a container has the sortableChildren property set to true, children will be automatically
     * sorted by zIndex value; a higher value will mean it will be moved towards the end of the array,
     * and thus rendered on top of other displayObjects within the same container.
     *
     * @member {number}
     */
    zIndex: number;
    /**
     * Indicates if the object is globally visible.
     *
     * @member {boolean}
     * @readonly
     */
    readonly worldVisible: boolean;
    /**
     * Sets a mask for the displayObject. A mask is an object that limits the visibility of an
     * object to the shape of the mask applied to it. In PixiJS a regular mask must be a
     * {@link PIXI.Graphics} or a {@link PIXI.Sprite} object. This allows for much faster masking in canvas as it
     * utilities shape clipping. To remove a mask, set this property to `null`.
     *
     * For sprite mask both alpha and red channel are used. Black mask is the same as transparent mask.
     * @example
     * const graphics = new PIXI.Graphics();
     * graphics.beginFill(0xFF3300);
     * graphics.drawRect(50, 250, 100, 100);
     * graphics.endFill();
     *
     * const sprite = new PIXI.Sprite(texture);
     * sprite.mask = graphics;
     * @todo At the moment, PIXI.CanvasRenderer doesn't support PIXI.Sprite as mask.
     *
     * @member {PIXI.Container|PIXI.MaskData}
     */
    mask: PIXI.Container | PIXI.MaskData;

    /**
     * DisplayObject default updateTransform, does not update children of container.
     * Will crash if there's no parent element.
     *
     * @memberof PIXI.DisplayObject#
     * @function displayObjectUpdateTransform
     */
    displayObjectUpdateTransform(): void;

    /**
     * Enable interaction events for the DisplayObject. Touch, pointer and mouse
     * events will not be emitted unless `interactive` is set to `true`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.on('tap', (event) => {
     *    //handle event
     * });
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    interactive: boolean;
    /**
     * Interaction shape. Children will be hit first, then this shape will be checked.
     * Setting this will cause this shape to be checked in hit tests rather than the displayObject's bounds.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.hitArea = new PIXI.Rectangle(0, 0, 100, 100);
     * @member {PIXI.IHitArea}
     * @memberof PIXI.DisplayObject#
     */
    hitArea: PIXI.IHitArea;
    /**
     * If enabled, the mouse cursor use the pointer behavior when hovered over the displayObject if it is interactive
     * Setting this changes the 'cursor' property to `'pointer'`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.buttonMode = true;
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    buttonMode: boolean;
    /**
     * This defines what cursor mode is used when the mouse cursor
     * is hovered over the displayObject.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.cursor = 'wait';
     * @see https://developer.mozilla.org/en/docs/Web/CSS/cursor
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    cursor: string;
    /**
     * Set this to true if you want this display object to be cached as a bitmap.
     * This basically takes a snap shot of the display object as it is at that moment. It can
     * provide a performance benefit for complex static displayObjects.
     * To remove simply set this property to `false`
     *
     * IMPORTANT GOTCHA - Make sure that all your textures are preloaded BEFORE setting this property to true
     * as it will take a snapshot of what is currently there. If the textures have not loaded then they will not appear.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    cacheAsBitmap: boolean;
    /**
     * The instance name of the object.
     *
     * @memberof PIXI.DisplayObject#
     * @member {string} name
     */
    name: string;

    /**
     * Returns the global position of the displayObject. Does not depend on object scale, rotation and pivot.
     *
     * @method getGlobalPosition
     * @memberof PIXI.DisplayObject#
     * @param {PIXI.Point} [point=new PIXI.Point()] - The point to write the global value to.
     * @param {boolean} [skipUpdate=false] - Setting to true will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @return {PIXI.Point} The updated point.
     */
    getGlobalPosition(point?: PIXI.Point, skipUpdate?: boolean): PIXI.Point;
  }

  /**
   * A tiling sprite is a fast way of rendering a tiling image
   *
   * @class
   * @extends PIXI.Sprite
   * @memberof PIXI
   */
  class TilingSprite extends PIXI.Sprite {
    static from(
      source: number | string | PIXI.Texture | HTMLCanvasElement | HTMLVideoElement,
      options?: any,
    ): PIXI.Sprite;

    static fromFrame(): PIXI.Sprite;

    static fromImage(): PIXI.Sprite;

    constructor(texture: PIXI.Texture, width?: number, height?: number);

    /**
     * Tile transform
     *
     * @member {PIXI.Transform} PIXI.TilingSprite#tileTransform
     */
    tileTransform: PIXI.Transform;
    /**
     * matrix that is applied to UV to get the coords in Texture normalized space to coords in BaseTexture space
     *
     * @member {PIXI.TextureMatrix} PIXI.TilingSprite#uvMatrix
     */
    uvMatrix: PIXI.TextureMatrix;
    /**
     * Plugin that is responsible for rendering this element.
     * Allows to customize the rendering process without overriding '_render' method.
     *
     * @member {string} PIXI.TilingSprite#pluginName
     * @default 'tilingSprite'
     */
    pluginName: string;
    /**
     * Whether or not anchor affects uvs
     *
     * @member {boolean} PIXI.TilingSprite#uvRespectAnchor
     * @default false
     */
    uvRespectAnchor: boolean;
    /**
     * Changes frame clamping in corresponding textureTransform, shortcut
     * Change to -0.5 to add a pixel to the edge, recommended for transparent trimmed textures in atlas
     *
     * @default 0.5
     * @member {number}
     */
    clampMargin: number;
    /**
     * The scaling of the image that is being tiled
     *
     * @member {PIXI.ObservablePoint}
     */
    tileScale: PIXI.ObservablePoint;
    /**
     * The offset of the image that is being tiled
     *
     * @member {PIXI.ObservablePoint}
     */
    tilePosition: PIXI.ObservablePoint;

    /**
     * Renders the object using the WebGL renderer
     *
     * @protected
     * @param {PIXI.Renderer} renderer - The renderer
     */
    protected _render(renderer: PIXI.Renderer): void;

    /**
     * Updates the bounds of the tiling sprite.
     *
     * @protected
     */
    protected _calculateBounds(): void;

    /**
     * Gets the local bounds of the sprite object.
     *
     * @param {PIXI.Rectangle} rect - The output rectangle.
     * @return {PIXI.Rectangle} The bounds.
     */
    getLocalBounds(rect: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Checks if a point is inside this tiling sprite.
     *
     * @param {PIXI.Point} point - the point to check
     * @return {boolean} Whether or not the sprite contains the point.
     */
    containsPoint(point: PIXI.Point): boolean;

    /**
     * Destroys this sprite and optionally its texture and children
     *
     * @param {object|boolean} [options] - Options parameter. A boolean will act as if all options
     *  have been set to that value
     * @param {boolean} [options.children=false] - if set to true, all the children will have their destroy
     *      method called as well. 'options' will be passed on to those calls.
     * @param {boolean} [options.texture=false] - Should it destroy the current texture of the sprite as well
     * @param {boolean} [options.baseTexture=false] - Should it destroy the base texture of the sprite as well
     */
    destroy(options?: { children?: boolean; texture?: boolean; baseTexture?: boolean }): void;

    /**
     * Helper function that creates a new tiling sprite based on the source you provide.
     * The source can be - frame id, image url, video url, canvas element, video element, base texture
     *
     * @static
     * @param {string|PIXI.Texture|HTMLCanvasElement|HTMLVideoElement} source - Source to create texture from
     * @param {number} width - the width of the tiling sprite
     * @param {number} height - the height of the tiling sprite
     * @return {PIXI.TilingSprite} The newly created texture
     */
    static from(
      source: string | PIXI.Texture | HTMLCanvasElement | HTMLVideoElement,
      width: number,
      height: number,
    ): PIXI.TilingSprite;

    /**
     * Helper function that creates a tiling sprite that will use a texture from the TextureCache based on the frameId
     * The frame ids are created when a Texture packer file has been loaded
     *
     * @static
     * @param {string} frameId - The frame Id of the texture in the cache
     * @param {number} width - the width of the tiling sprite
     * @param {number} height - the height of the tiling sprite
     * @return {PIXI.TilingSprite} A new TilingSprite using a texture from the texture cache matching the frameId
     */
    static fromFrame(frameId: string, width: number, height: number): PIXI.TilingSprite;

    /**
     * Helper function that creates a sprite that will contain a texture based on an image url
     * If the image is not in the texture cache it will be loaded
     *
     * @static
     * @param {string} imageId - The image url of the texture
     * @param {number} width - the width of the tiling sprite
     * @param {number} height - the height of the tiling sprite
     * @param {Object} [options] - See {@link PIXI.BaseTexture}'s constructor for options.
     * @return {PIXI.TilingSprite} A new TilingSprite using a texture from the texture cache matching the image id
     */
    static fromImage(imageId: string, width: number, height: number, options?: any): PIXI.TilingSprite;

    /**
     * The width of the sprite, setting this will actually modify the scale to achieve the value set
     *
     * @member {number}
     */
    width: number;
    /**
     * The height of the TilingSprite, setting this will actually modify the scale to achieve the value set
     *
     * @member {number}
     */
    height: number;
    /**
     * The blend mode to be applied to the sprite. Apply a value of `PIXI.BLEND_MODES.NORMAL` to reset the blend mode.
     *
     * @member {number} PIXI.Sprite#blendMode
     * @default PIXI.BLEND_MODES.NORMAL
     * @see PIXI.BLEND_MODES
     */
    blendMode: number;
    /**
     * The shader that will be used to render the sprite. Set to null to remove a current shader.
     *
     * @member {PIXI.Filter|PIXI.Shader} PIXI.Sprite#shader
     */
    shader: PIXI.Filter | PIXI.Shader;
    /**
     * Cached tint value so we can tell when the tint is changed.
     * Value is used for 2d CanvasRenderer.
     *
     * @protected
     * @member {number} PIXI.Sprite#_cachedTint
     * @default 0xFFFFFF
     */
    protected _cachedTint: number;
    /**
     * used to fast check if a sprite is.. a sprite!
     * @member {boolean} PIXI.Sprite#isSprite
     */
    isSprite: boolean;

    /**
     * calculates worldTransform * vertices, store it in vertexData
     */
    calculateVertices(): void;

    /**
     * calculates worldTransform * vertices for a non texture with a trim. store it in vertexTrimmedData
     * This is used to ensure that the true width and height of a trimmed texture is respected
     */
    calculateTrimmedVertices(): void;

    /**
     * If true PixiJS will Math.floor() x/y values when rendering, stopping pixel interpolation.
     * Advantages can include sharper image quality (like text) and faster rendering on canvas.
     * The main disadvantage is movement of objects may appear less smooth.
     * To set the global default, change {@link PIXI.settings.ROUND_PIXELS}
     *
     * @member {boolean}
     * @default false
     */
    roundPixels: boolean;
    /**
     * The anchor sets the origin point of the text. The default value is taken from the {@link PIXI.Texture|Texture}
     * and passed to the constructor.
     *
     * The default is `(0,0)`, this means the text's origin is the top left.
     *
     * Setting the anchor to `(0.5,0.5)` means the text's origin is centered.
     *
     * Setting the anchor to `(1,1)` would mean the text's origin point will be the bottom right corner.
     *
     * If you pass only single parameter, it will set both x and y to the same value as shown in the example below.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.anchor.set(0.5); // This will set the origin to center. (0.5) is same as (0.5, 0.5).
     *
     * @member {PIXI.ObservablePoint}
     */
    anchor: PIXI.ObservablePoint;
    /**
     * The tint applied to the sprite. This is a hex value.
     * A value of 0xFFFFFF will remove any tint effect.
     *
     * @member {number}
     * @default 0xFFFFFF
     */
    tint: number;
    /**
     * The texture that the sprite is using
     *
     * @member {PIXI.Texture}
     */
    texture: PIXI.Texture;
    /**
     * The array of children of this container.
     *
     * @member {PIXI.DisplayObject[]} PIXI.Container#children
     * @readonly
     */
    readonly children: PIXI.DisplayObject[];
    /**
     * If set to true, the container will sort its children by zIndex value
     * when updateTransform() is called, or manually if sortChildren() is called.
     *
     * This actually changes the order of elements in the array, so should be treated
     * as a basic solution that is not performant compared to other solutions,
     * such as @link https://github.com/pixijs/pixi-display
     *
     * Also be aware of that this may not work nicely with the addChildAt() function,
     * as the zIndex sorting may cause the child to automatically sorted to another position.
     *
     * @see PIXI.settings.SORTABLE_CHILDREN
     *
     * @member {boolean} PIXI.Container#sortableChildren
     */
    sortableChildren: boolean;
    /**
     * Should children be sorted by zIndex at the next updateTransform call.
     * Will get automatically set to true if a new child is added, or if a child's zIndex changes.
     *
     * @member {boolean} PIXI.Container#sortDirty
     */
    sortDirty: boolean;

    /**
     * Overridable method that can be used by Container subclasses whenever the children array is modified
     *
     * @protected
     */
    protected onChildrenChange(): void;

    /**
     * Adds one or more children to the container.
     *
     * Multiple items can be added like so: `myContainer.addChild(thingOne, thingTwo, thingThree)`
     *
     * @param {...PIXI.DisplayObject} child - The DisplayObject(s) to add to the container
     * @return {PIXI.DisplayObject} The first child that was added.
     */
    addChild<TChildren extends PIXI.DisplayObject[]>(...child: TChildren): TChildren[0];

    /**
     * Adds a child to the container at a specified index. If the index is out of bounds an error will be thrown
     *
     * @param {PIXI.DisplayObject} child - The child to add
     * @param {number} index - The index to place the child in
     * @return {PIXI.DisplayObject} The child that was added.
     */
    addChildAt<T extends PIXI.DisplayObject>(child: T, index: number): T;

    /**
     * Swaps the position of 2 Display Objects within this container.
     *
     * @param {PIXI.DisplayObject} child - First display object to swap
     * @param {PIXI.DisplayObject} child2 - Second display object to swap
     */
    swapChildren(child: PIXI.DisplayObject, child2: PIXI.DisplayObject): void;

    /**
     * Returns the index position of a child DisplayObject instance
     *
     * @param {PIXI.DisplayObject} child - The DisplayObject instance to identify
     * @return {number} The index position of the child display object to identify
     */
    getChildIndex(child: PIXI.DisplayObject): number;

    /**
     * Changes the position of an existing child in the display object container
     *
     * @param {PIXI.DisplayObject} child - The child DisplayObject instance for which you want to change the index number
     * @param {number} index - The resulting index number for the child display object
     */
    setChildIndex(child: PIXI.DisplayObject, index: number): void;

    /**
     * Returns the child at the specified index
     *
     * @param {number} index - The index to get the child at
     * @return {PIXI.DisplayObject} The child at the given index, if any.
     */
    getChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes one or more children from the container.
     *
     * @param {...PIXI.DisplayObject} child - The DisplayObject(s) to remove
     * @return {PIXI.DisplayObject} The first child that was removed.
     */
    removeChild<TChildren extends PIXI.DisplayObject[]>(...child: TChildren): TChildren[0];

    /**
     * Removes a child from the specified index position.
     *
     * @param {number} index - The index to get the child from
     * @return {PIXI.DisplayObject} The child that was removed.
     */
    removeChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes all children from this container that are within the begin and end indexes.
     *
     * @param {number} [beginIndex=0] - The beginning position.
     * @param {number} [endIndex=this.children.length] - The ending position. Default value is size of the container.
     * @returns {PIXI.DisplayObject[]} List of removed children
     */
    removeChildren(beginIndex?: number, endIndex?: number): PIXI.DisplayObject[];

    /**
     * Sorts children by zIndex. Previous order is mantained for 2 children with the same zIndex.
     */
    sortChildren(): void;

    /**
     * Updates the transform on all children of this container for rendering
     */
    updateTransform(): void;

    /**
     * Recalculates the bounds of the container.
     *
     */
    calculateBounds(): void;

    /**
     * Renders the object using the WebGL renderer
     *
     * @param {PIXI.Renderer} renderer - The renderer
     */
    render(renderer: PIXI.Renderer): void;

    /**
     * Render the object using the WebGL renderer and advanced features.
     *
     * @protected
     * @param {PIXI.Renderer} renderer - The renderer
     */
    protected renderAdvanced(renderer: PIXI.Renderer): void;

    /**
     * Determines if the children to the displayObject can be clicked/touched
     * Setting this to false allows PixiJS to bypass a recursive `hitTest` function
     *
     * @member {boolean}
     * @memberof PIXI.Container#
     */
    interactiveChildren: boolean;

    /**
     * Returns the display object in the container.
     *
     * @method getChildByName
     * @memberof PIXI.Container#
     * @param {string} name - Instance name.
     * @return {PIXI.DisplayObject} The child with the specified name.
     */
    getChildByName(name: string): PIXI.DisplayObject;

    /**
     *  Flag for if the object is accessible. If true AccessibilityManager will overlay a
     *   shadow div with attributes set
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    accessible: boolean;
    /**
     * Sets the title attribute of the shadow div
     * If accessibleTitle AND accessibleHint has not been this will default to 'displayObject [tabIndex]'
     *
     * @member {?string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleTitle: string;
    /**
     * Sets the aria-label attribute of the shadow div
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleHint: string;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleActive: boolean;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleDiv: boolean;
    /**
     * Specify the type of div the accessible layer is. Screen readers treat the element differently
     * depending on this type. Defaults to button.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'button'
     */
    accessibleType: string;
    /**
     * Specify the pointer-events the accessible div will use
     * Defaults to auto.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'auto'
     */
    accessiblePointerEvents: string;
    /**
     * Setting to false will prevent any children inside this container to
     * be accessible. Defaults to true.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @default true
     */
    accessibleChildren: boolean;
    /**
     * World transform and local transform of this object.
     * This will become read-only later, please do not assign anything there unless you know what are you doing.
     *
     * @member {PIXI.Transform} PIXI.DisplayObject#transform
     */
    transform: PIXI.Transform;
    /**
     * The opacity of the object.
     *
     * @member {number} PIXI.DisplayObject#alpha
     */
    alpha: number;
    /**
     * The visibility of the object. If false the object will not be drawn, and
     * the updateTransform function will not be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds or call updateTransform manually.
     *
     * @member {boolean} PIXI.DisplayObject#visible
     */
    visible: boolean;
    /**
     * Can this object be rendered, if false the object will not be drawn but the updateTransform
     * methods will still be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds manually.
     *
     * @member {boolean} PIXI.DisplayObject#renderable
     */
    renderable: boolean;
    /**
     * The display object container that contains this display object.
     *
     * @member {PIXI.Container} PIXI.DisplayObject#parent
     * @readonly
     */
    readonly parent: PIXI.Container;
    /**
     * The multiplied alpha of the displayObject.
     *
     * @member {number} PIXI.DisplayObject#worldAlpha
     * @readonly
     */
    readonly worldAlpha: number;
    /**
     * Which index in the children array the display component was before the previous zIndex sort.
     * Used by containers to help sort objects with the same zIndex, by using previous array index as the decider.
     *
     * @member {number} PIXI.DisplayObject#_lastSortedIndex
     * @protected
     */
    protected _lastSortedIndex: number;
    /**
     * The zIndex of the displayObject.
     * A higher value will mean it will be rendered on top of other displayObjects within the same container.
     *
     * @member {number} PIXI.DisplayObject#_zIndex
     * @protected
     */
    protected _zIndex: number;
    /**
     * The area the filter is applied to. This is used as more of an optimization
     * rather than figuring out the dimensions of the displayObject each frame you can set this rectangle.
     *
     * Also works as an interaction mask.
     *
     * @member {?PIXI.Rectangle} PIXI.DisplayObject#filterArea
     */
    filterArea: PIXI.Rectangle;
    /**
     * Sets the filters for the displayObject.
     * * IMPORTANT: This is a WebGL only feature and will be ignored by the canvas renderer.
     * To remove filters simply set this property to `'null'`.
     *
     * @member {?PIXI.Filter[]} PIXI.DisplayObject#filters
     */
    filters: PIXI.Filter[];
    /**
     * The bounds object, this is used to calculate and store the bounds of the displayObject.
     *
     * @member {PIXI.Bounds} PIXI.DisplayObject#_bounds
     * @protected
     */
    protected _bounds: PIXI.Bounds;
    /**
     * The original, cached mask of the object.
     *
     * @member {PIXI.Graphics|PIXI.Sprite|null} PIXI.DisplayObject#_mask
     * @protected
     */
    protected _mask: PIXI.Graphics | PIXI.Sprite | null;
    /**
     * If the object has been destroyed via destroy(). If true, it should not be used.
     *
     * @member {boolean} PIXI.DisplayObject#_destroyed
     * @protected
     */
    protected _destroyed: boolean;
    /**
     * Does any other displayObject use this object as a mask?
     * @member {boolean} PIXI.DisplayObject#isMask
     */
    isMask: boolean;
    /**
     * @protected
     * @member {PIXI.DisplayObject}
     */
    protected _tempDisplayObjectParent: PIXI.DisplayObject;

    /**
     * Recursively updates transform of all objects from the root to this one
     * internal function for toLocal()
     */
    _recursivePostUpdateTransform(): void;

    /**
     * Retrieves the bounds of the displayObject as a rectangle object.
     *
     * @param {boolean} [skipUpdate] - Setting to `true` will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @param {PIXI.Rectangle} [rect] - Optional rectangle to store the result of the bounds calculation.
     * @return {PIXI.Rectangle} The rectangular bounding area.
     */
    getBounds(skipUpdate?: boolean, rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Calculates the global position of the display object.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform.
     * @return {PIXI.IPoint} A point object representing the position of this object.
     */
    toGlobal(position: PIXI.IPoint, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Calculates the local position of the display object relative to another point.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.DisplayObject} [from] - The DisplayObject to calculate the global position from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform
     * @return {PIXI.IPoint} A point object representing the position of this object
     */
    toLocal(position: PIXI.IPoint, from?: PIXI.DisplayObject, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Set the parent Container of this DisplayObject.
     *
     * @param {PIXI.Container} container - The Container to add this DisplayObject to.
     * @return {PIXI.Container} The Container that this DisplayObject was added to.
     */
    setParent(container: PIXI.Container): PIXI.Container;

    /**
     * Convenience function to set the position, scale, skew and pivot at once.
     *
     * @param {number} [x=0] - The X position
     * @param {number} [y=0] - The Y position
     * @param {number} [scaleX=1] - The X scale value
     * @param {number} [scaleY=1] - The Y scale value
     * @param {number} [rotation=0] - The rotation
     * @param {number} [skewX=0] - The X skew value
     * @param {number} [skewY=0] - The Y skew value
     * @param {number} [pivotX=0] - The X pivot value
     * @param {number} [pivotY=0] - The Y pivot value
     * @return {PIXI.DisplayObject} The DisplayObject instance
     */
    setTransform(
      x?: number,
      y?: number,
      scaleX?: number,
      scaleY?: number,
      rotation?: number,
      skewX?: number,
      skewY?: number,
      pivotX?: number,
      pivotY?: number,
    ): PIXI.DisplayObject;

    /**
     * The position of the displayObject on the x axis relative to the local coordinates of the parent.
     * An alias to position.x
     *
     * @member {number}
     */
    x: number;
    /**
     * The position of the displayObject on the y axis relative to the local coordinates of the parent.
     * An alias to position.y
     *
     * @member {number}
     */
    y: number;
    /**
     * Current transform of the object based on world (parent) factors.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly worldTransform: PIXI.Matrix;
    /**
     * Current transform of the object based on local factors: position, scale, other stuff.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly localTransform: PIXI.Matrix;
    /**
     * The coordinate of the object relative to the local coordinates of the parent.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    position: PIXI.IPoint;
    /**
     * The scale factor of the object.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    scale: PIXI.IPoint;
    /**
     * The pivot point of the displayObject that it rotates around.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    pivot: PIXI.IPoint;
    /**
     * The skew factor for the object in radians.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.ObservablePoint}
     */
    skew: PIXI.ObservablePoint;
    /**
     * The rotation of the object in radians.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    rotation: number;
    /**
     * The angle of the object in degrees.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    angle: number;
    /**
     * The zIndex of the displayObject.
     * If a container has the sortableChildren property set to true, children will be automatically
     * sorted by zIndex value; a higher value will mean it will be moved towards the end of the array,
     * and thus rendered on top of other displayObjects within the same container.
     *
     * @member {number}
     */
    zIndex: number;
    /**
     * Indicates if the object is globally visible.
     *
     * @member {boolean}
     * @readonly
     */
    readonly worldVisible: boolean;
    /**
     * Sets a mask for the displayObject. A mask is an object that limits the visibility of an
     * object to the shape of the mask applied to it. In PixiJS a regular mask must be a
     * {@link PIXI.Graphics} or a {@link PIXI.Sprite} object. This allows for much faster masking in canvas as it
     * utilities shape clipping. To remove a mask, set this property to `null`.
     *
     * For sprite mask both alpha and red channel are used. Black mask is the same as transparent mask.
     * @example
     * const graphics = new PIXI.Graphics();
     * graphics.beginFill(0xFF3300);
     * graphics.drawRect(50, 250, 100, 100);
     * graphics.endFill();
     *
     * const sprite = new PIXI.Sprite(texture);
     * sprite.mask = graphics;
     * @todo At the moment, PIXI.CanvasRenderer doesn't support PIXI.Sprite as mask.
     *
     * @member {PIXI.Container|PIXI.MaskData}
     */
    mask: PIXI.Container | PIXI.MaskData;

    /**
     * DisplayObject default updateTransform, does not update children of container.
     * Will crash if there's no parent element.
     *
     * @memberof PIXI.DisplayObject#
     * @function displayObjectUpdateTransform
     */
    displayObjectUpdateTransform(): void;

    /**
     * Enable interaction events for the DisplayObject. Touch, pointer and mouse
     * events will not be emitted unless `interactive` is set to `true`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.on('tap', (event) => {
     *    //handle event
     * });
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    interactive: boolean;
    /**
     * Interaction shape. Children will be hit first, then this shape will be checked.
     * Setting this will cause this shape to be checked in hit tests rather than the displayObject's bounds.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.hitArea = new PIXI.Rectangle(0, 0, 100, 100);
     * @member {PIXI.IHitArea}
     * @memberof PIXI.DisplayObject#
     */
    hitArea: PIXI.IHitArea;
    /**
     * If enabled, the mouse cursor use the pointer behavior when hovered over the displayObject if it is interactive
     * Setting this changes the 'cursor' property to `'pointer'`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.buttonMode = true;
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    buttonMode: boolean;
    /**
     * This defines what cursor mode is used when the mouse cursor
     * is hovered over the displayObject.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.cursor = 'wait';
     * @see https://developer.mozilla.org/en/docs/Web/CSS/cursor
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    cursor: string;
    /**
     * Set this to true if you want this display object to be cached as a bitmap.
     * This basically takes a snap shot of the display object as it is at that moment. It can
     * provide a performance benefit for complex static displayObjects.
     * To remove simply set this property to `false`
     *
     * IMPORTANT GOTCHA - Make sure that all your textures are preloaded BEFORE setting this property to true
     * as it will take a snapshot of what is currently there. If the textures have not loaded then they will not appear.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    cacheAsBitmap: boolean;
    /**
     * The instance name of the object.
     *
     * @memberof PIXI.DisplayObject#
     * @member {string} name
     */
    name: string;

    /**
     * Returns the global position of the displayObject. Does not depend on object scale, rotation and pivot.
     *
     * @method getGlobalPosition
     * @memberof PIXI.DisplayObject#
     * @param {PIXI.Point} [point=new PIXI.Point()] - The point to write the global value to.
     * @param {boolean} [skipUpdate=false] - Setting to true will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @return {PIXI.Point} The updated point.
     */
    getGlobalPosition(point?: PIXI.Point, skipUpdate?: boolean): PIXI.Point;
  }

  /**
   * WebGL renderer plugin for tiling sprites
   *
   * @class
   * @memberof PIXI
   * @extends PIXI.ObjectRenderer
   */
  class TilingSpriteRenderer extends PIXI.ObjectRenderer {
    constructor(renderer: PIXI.Renderer);

    /**
     * The WebGL state in which this renderer will work.
     *
     * @member {PIXI.State} PIXI.TilingSpriteRenderer#state
     * @readonly
     */
    readonly state: PIXI.State;

    /**
     *
     * @param {PIXI.TilingSprite} ts tilingSprite to be rendered
     */
    render(ts: PIXI.TilingSprite): void;

    /**
     * The renderer this manager works for.
     *
     * @member {PIXI.Renderer} PIXI.ObjectRenderer#renderer
     */
    renderer: PIXI.Renderer;

    /**
     * Stub method that should be used to empty the current
     * batch by rendering objects now.
     */
    flush(): void;

    /**
     * Generic destruction method that frees all resources. This
     * should be called by subclasses.
     */
    destroy(): void;

    /**
     * Stub method that initializes any state required before
     * rendering starts. It is different from the `prerender`
     * signal, which occurs every frame, in that it is called
     * whenever an object requests _this_ renderer specifically.
     */
    start(): void;

    /**
     * Stops the renderer. It should free up any state and
     * become dormant.
     */
    stop(): void;
  }

  /**
   * Utility class for maintaining reference to a collection
   * of Textures on a single Spritesheet.
   *
   * To access a sprite sheet from your code pass its JSON data file to Pixi's loader:
   *
   * ```js
   * PIXI.Loader.shared.add("images/spritesheet.json").load(setup);
   *
   * function setup() {
   *   let sheet = PIXI.Loader.shared.resources["images/spritesheet.json"].spritesheet;
   *   ...
   * }
   * ```
   * With the `sheet.textures` you can create Sprite objects,`sheet.animations` can be used to create an AnimatedSprite.
   *
   * Sprite sheets can be packed using tools like {@link https://codeandweb.com/texturepacker|TexturePacker},
   * {@link https://renderhjs.net/shoebox/|Shoebox} or {@link https://github.com/krzysztof-o/spritesheet.js|Spritesheet.js}.
   * Default anchor points (see {@link PIXI.Texture#defaultAnchor}) and grouping of animation sprites are currently only
   * supported by TexturePacker.
   *
   * @class
   * @memberof PIXI
   */
  class Spritesheet {
    constructor(baseTexture: PIXI.BaseTexture | PIXI.Texture, data: any, resolutionFilename?: string);

    /**
     * The maximum number of Textures to build per process.
     *
     * @type {number}
     * @default 1000
     */
    static BATCH_SIZE: number;
    /**
     * Reference to ths source texture.
     * @type {PIXI.BaseTexture}
     */
    baseTexture: PIXI.BaseTexture;
    /**
     * A map containing all textures of the sprite sheet.
     * Can be used to create a {@link PIXI.Sprite|Sprite}:
     * ```js
     * new PIXI.Sprite(sheet.textures["image.png"]);
     * ```
     * @member {Object} PIXI.Spritesheet#textures
     */
    textures: any;
    /**
     * A map containing the textures for each animation.
     * Can be used to create an {@link PIXI.AnimatedSprite|AnimatedSprite}:
     * ```js
     * new PIXI.AnimatedSprite(sheet.animations["anim_name"])
     * ```
     * @member {Object} PIXI.Spritesheet#animations
     */
    animations: any;
    /**
     * Reference to the original JSON data.
     * @type {Object}
     */
    data: any;
    /**
     * The resolution of the spritesheet.
     * @type {number}
     */
    resolution: number;

    /**
     * Parser spritesheet from loaded data. This is done asynchronously
     * to prevent creating too many Texture within a single process.
     *
     * @param {Function} callback - Callback when complete returns
     *        a map of the Textures for this spritesheet.
     */
    parse(callback: (...params: any[]) => any): void;

    /**
     * Destroy Spritesheet and don't use after this.
     *
     * @param {boolean} [destroyBase=false] Whether to destroy the base texture as well
     */
    destroy(destroyBase?: boolean): void;
  }

  interface SpritesheetLoader extends PIXI.ILoaderPlugin {}

  /**
   * {@link PIXI.Loader Loader} middleware for loading texture atlases that have been created with
   * TexturePacker or similar JSON-based spritesheet.
   *
   * This middleware automatically generates Texture resources.
   *
   * @class
   * @memberof PIXI
   * @implements PIXI.ILoaderPlugin
   */
  class SpritesheetLoader implements PIXI.ILoaderPlugin {
    /**
     * Called after a resource is loaded.
     * @see PIXI.Loader.loaderMiddleware
     * @param {PIXI.LoaderResource} resource
     * @param {function} next
     */
    static use(resource: PIXI.LoaderResource, next: (...params: any[]) => any): void;

    /**
     * Get the spritesheets root path
     * @param {PIXI.LoaderResource} resource - Resource to check path
     * @param {string} baseUrl - Base root url
     */
    static getResourcePath(resource: PIXI.LoaderResource, baseUrl: string): void;
  }

  /**
   * A Text Object will create a line or multiple lines of text.
   *
   * The text is created using the [Canvas API](https://developer.mozilla.org/en-US/docs/Web/API/Canvas_API).
   *
   * The primary advantage of this class over BitmapText is that you have great control over the style of the next,
   * which you can change at runtime.
   *
   * The primary disadvantages is that each piece of text has it's own texture, which can use more memory.
   * When text changes, this texture has to be re-generated and re-uploaded to the GPU, taking up time.
   *
   * To split a line you can use '\n' in your text string, or, on the `style` object,
   * change its `wordWrap` property to true and and give the `wordWrapWidth` property a value.
   *
   * A Text can be created directly from a string and a style object,
   * which can be generated [here](https://pixijs.io/pixi-text-style).
   *
   * ```js
   * let text = new PIXI.Text('This is a PixiJS text',{fontFamily : 'Arial', fontSize: 24, fill : 0xff1010, align : 'center'});
   * ```
   *
   * @class
   * @extends PIXI.Sprite
   * @memberof PIXI
   */
  class Text extends PIXI.Sprite {
    constructor(text: string, style?: any | PIXI.TextStyle, canvas?: HTMLCanvasElement);

    /**
     * The canvas element that everything is drawn to
     *
     * @member {HTMLCanvasElement} PIXI.Text#canvas
     */
    canvas: HTMLCanvasElement;
    /**
     * The canvas 2d context that everything is drawn with
     * @member {CanvasRenderingContext2D} PIXI.Text#context
     */
    context: CanvasRenderingContext2D;
    /**
     * The resolution / device pixel ratio of the canvas.
     * This is set to automatically match the renderer resolution by default, but can be overridden by setting manually.
     * @member {number} PIXI.Text#_resolution
     * @default 1
     */
    _resolution: number;

    /**
     * Gets the local bounds of the text object.
     *
     * @param {PIXI.Rectangle} rect - The output rectangle.
     * @return {PIXI.Rectangle} The bounds.
     */
    getLocalBounds(rect: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * calculates the bounds of the Text as a rectangle. The bounds calculation takes the worldTransform into account.
     * @protected
     */
    protected _calculateBounds(): void;

    /**
     * Destroys this text object.
     * Note* Unlike a Sprite, a Text object will automatically destroy its baseTexture and texture as
     * the majority of the time the texture will not be shared with any other Sprites.
     *
     * @param {object|boolean} [options] - Options parameter. A boolean will act as if all options
     *  have been set to that value
     * @param {boolean} [options.children=false] - if set to true, all the children will have their
     *  destroy method called as well. 'options' will be passed on to those calls.
     * @param {boolean} [options.texture=true] - Should it destroy the current texture of the sprite as well
     * @param {boolean} [options.baseTexture=true] - Should it destroy the base texture of the sprite as well
     */
    destroy(options?: { children?: boolean; texture?: boolean; baseTexture?: boolean }): void;

    /**
     * The width of the Text, setting this will actually modify the scale to achieve the value set
     *
     * @member {number}
     */
    width: number;
    /**
     * The height of the Text, setting this will actually modify the scale to achieve the value set
     *
     * @member {number}
     */
    height: number;
    /**
     * Set the style of the text. Set up an event listener to listen for changes on the style
     * object and mark the text as dirty.
     *
     * @member {object|PIXI.TextStyle}
     */
    style: any | PIXI.TextStyle;
    /**
     * Set the copy for the text object. To split a line you can use '\n'.
     *
     * @member {string}
     */
    text: string;
    /**
     * The resolution / device pixel ratio of the canvas.
     * This is set to automatically match the renderer resolution by default, but can be overridden by setting manually.
     * @member {number}
     * @default 1
     */
    resolution: number;
    /**
     * The blend mode to be applied to the sprite. Apply a value of `PIXI.BLEND_MODES.NORMAL` to reset the blend mode.
     *
     * @member {number} PIXI.Sprite#blendMode
     * @default PIXI.BLEND_MODES.NORMAL
     * @see PIXI.BLEND_MODES
     */
    blendMode: number;
    /**
     * The shader that will be used to render the sprite. Set to null to remove a current shader.
     *
     * @member {PIXI.Filter|PIXI.Shader} PIXI.Sprite#shader
     */
    shader: PIXI.Filter | PIXI.Shader;
    /**
     * Cached tint value so we can tell when the tint is changed.
     * Value is used for 2d CanvasRenderer.
     *
     * @protected
     * @member {number} PIXI.Sprite#_cachedTint
     * @default 0xFFFFFF
     */
    protected _cachedTint: number;
    /**
     * Plugin that is responsible for rendering this element.
     * Allows to customize the rendering process without overriding '_render' & '_renderCanvas' methods.
     *
     * @member {string} PIXI.Sprite#pluginName
     * @default 'batch'
     */
    pluginName: string;
    /**
     * used to fast check if a sprite is.. a sprite!
     * @member {boolean} PIXI.Sprite#isSprite
     */
    isSprite: boolean;

    /**
     * calculates worldTransform * vertices, store it in vertexData
     */
    calculateVertices(): void;

    /**
     * calculates worldTransform * vertices for a non texture with a trim. store it in vertexTrimmedData
     * This is used to ensure that the true width and height of a trimmed texture is respected
     */
    calculateTrimmedVertices(): void;

    /**
     * Tests if a point is inside this sprite
     *
     * @param {PIXI.Point} point - the point to test
     * @return {boolean} the result of the test
     */
    containsPoint(point: PIXI.Point): boolean;

    /**
     * If true PixiJS will Math.floor() x/y values when rendering, stopping pixel interpolation.
     * Advantages can include sharper image quality (like text) and faster rendering on canvas.
     * The main disadvantage is movement of objects may appear less smooth.
     * To set the global default, change {@link PIXI.settings.ROUND_PIXELS}
     *
     * @member {boolean}
     * @default false
     */
    roundPixels: boolean;
    /**
     * The anchor sets the origin point of the text. The default value is taken from the {@link PIXI.Texture|Texture}
     * and passed to the constructor.
     *
     * The default is `(0,0)`, this means the text's origin is the top left.
     *
     * Setting the anchor to `(0.5,0.5)` means the text's origin is centered.
     *
     * Setting the anchor to `(1,1)` would mean the text's origin point will be the bottom right corner.
     *
     * If you pass only single parameter, it will set both x and y to the same value as shown in the example below.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.anchor.set(0.5); // This will set the origin to center. (0.5) is same as (0.5, 0.5).
     *
     * @member {PIXI.ObservablePoint}
     */
    anchor: PIXI.ObservablePoint;
    /**
     * The tint applied to the sprite. This is a hex value.
     * A value of 0xFFFFFF will remove any tint effect.
     *
     * @member {number}
     * @default 0xFFFFFF
     */
    tint: number;
    /**
     * The texture that the sprite is using
     *
     * @member {PIXI.Texture}
     */
    texture: PIXI.Texture;
    /**
     * The array of children of this container.
     *
     * @member {PIXI.DisplayObject[]} PIXI.Container#children
     * @readonly
     */
    readonly children: PIXI.DisplayObject[];
    /**
     * If set to true, the container will sort its children by zIndex value
     * when updateTransform() is called, or manually if sortChildren() is called.
     *
     * This actually changes the order of elements in the array, so should be treated
     * as a basic solution that is not performant compared to other solutions,
     * such as @link https://github.com/pixijs/pixi-display
     *
     * Also be aware of that this may not work nicely with the addChildAt() function,
     * as the zIndex sorting may cause the child to automatically sorted to another position.
     *
     * @see PIXI.settings.SORTABLE_CHILDREN
     *
     * @member {boolean} PIXI.Container#sortableChildren
     */
    sortableChildren: boolean;
    /**
     * Should children be sorted by zIndex at the next updateTransform call.
     * Will get automatically set to true if a new child is added, or if a child's zIndex changes.
     *
     * @member {boolean} PIXI.Container#sortDirty
     */
    sortDirty: boolean;

    /**
     * Overridable method that can be used by Container subclasses whenever the children array is modified
     *
     * @protected
     */
    protected onChildrenChange(): void;

    /**
     * Adds one or more children to the container.
     *
     * Multiple items can be added like so: `myContainer.addChild(thingOne, thingTwo, thingThree)`
     *
     * @param {...PIXI.DisplayObject} child - The DisplayObject(s) to add to the container
     * @return {PIXI.DisplayObject} The first child that was added.
     */
    addChild<TChildren extends PIXI.DisplayObject[]>(...child: TChildren): TChildren[0];

    /**
     * Adds a child to the container at a specified index. If the index is out of bounds an error will be thrown
     *
     * @param {PIXI.DisplayObject} child - The child to add
     * @param {number} index - The index to place the child in
     * @return {PIXI.DisplayObject} The child that was added.
     */
    addChildAt<T extends PIXI.DisplayObject>(child: T, index: number): T;

    /**
     * Swaps the position of 2 Display Objects within this container.
     *
     * @param {PIXI.DisplayObject} child - First display object to swap
     * @param {PIXI.DisplayObject} child2 - Second display object to swap
     */
    swapChildren(child: PIXI.DisplayObject, child2: PIXI.DisplayObject): void;

    /**
     * Returns the index position of a child DisplayObject instance
     *
     * @param {PIXI.DisplayObject} child - The DisplayObject instance to identify
     * @return {number} The index position of the child display object to identify
     */
    getChildIndex(child: PIXI.DisplayObject): number;

    /**
     * Changes the position of an existing child in the display object container
     *
     * @param {PIXI.DisplayObject} child - The child DisplayObject instance for which you want to change the index number
     * @param {number} index - The resulting index number for the child display object
     */
    setChildIndex(child: PIXI.DisplayObject, index: number): void;

    /**
     * Returns the child at the specified index
     *
     * @param {number} index - The index to get the child at
     * @return {PIXI.DisplayObject} The child at the given index, if any.
     */
    getChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes one or more children from the container.
     *
     * @param {...PIXI.DisplayObject} child - The DisplayObject(s) to remove
     * @return {PIXI.DisplayObject} The first child that was removed.
     */
    removeChild<TChildren extends PIXI.DisplayObject[]>(...child: TChildren): TChildren[0];

    /**
     * Removes a child from the specified index position.
     *
     * @param {number} index - The index to get the child from
     * @return {PIXI.DisplayObject} The child that was removed.
     */
    removeChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes all children from this container that are within the begin and end indexes.
     *
     * @param {number} [beginIndex=0] - The beginning position.
     * @param {number} [endIndex=this.children.length] - The ending position. Default value is size of the container.
     * @returns {PIXI.DisplayObject[]} List of removed children
     */
    removeChildren(beginIndex?: number, endIndex?: number): PIXI.DisplayObject[];

    /**
     * Sorts children by zIndex. Previous order is mantained for 2 children with the same zIndex.
     */
    sortChildren(): void;

    /**
     * Updates the transform on all children of this container for rendering
     */
    updateTransform(): void;

    /**
     * Recalculates the bounds of the container.
     *
     */
    calculateBounds(): void;

    /**
     * Renders the object using the WebGL renderer
     *
     * @param {PIXI.Renderer} renderer - The renderer
     */
    render(renderer: PIXI.Renderer): void;

    /**
     * Render the object using the WebGL renderer and advanced features.
     *
     * @protected
     * @param {PIXI.Renderer} renderer - The renderer
     */
    protected renderAdvanced(renderer: PIXI.Renderer): void;

    /**
     * Determines if the children to the displayObject can be clicked/touched
     * Setting this to false allows PixiJS to bypass a recursive `hitTest` function
     *
     * @member {boolean}
     * @memberof PIXI.Container#
     */
    interactiveChildren: boolean;

    /**
     * Returns the display object in the container.
     *
     * @method getChildByName
     * @memberof PIXI.Container#
     * @param {string} name - Instance name.
     * @return {PIXI.DisplayObject} The child with the specified name.
     */
    getChildByName(name: string): PIXI.DisplayObject;

    /**
     *  Flag for if the object is accessible. If true AccessibilityManager will overlay a
     *   shadow div with attributes set
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    accessible: boolean;
    /**
     * Sets the title attribute of the shadow div
     * If accessibleTitle AND accessibleHint has not been this will default to 'displayObject [tabIndex]'
     *
     * @member {?string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleTitle: string;
    /**
     * Sets the aria-label attribute of the shadow div
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleHint: string;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleActive: boolean;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleDiv: boolean;
    /**
     * Specify the type of div the accessible layer is. Screen readers treat the element differently
     * depending on this type. Defaults to button.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'button'
     */
    accessibleType: string;
    /**
     * Specify the pointer-events the accessible div will use
     * Defaults to auto.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'auto'
     */
    accessiblePointerEvents: string;
    /**
     * Setting to false will prevent any children inside this container to
     * be accessible. Defaults to true.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @default true
     */
    accessibleChildren: boolean;
    /**
     * World transform and local transform of this object.
     * This will become read-only later, please do not assign anything there unless you know what are you doing.
     *
     * @member {PIXI.Transform} PIXI.DisplayObject#transform
     */
    transform: PIXI.Transform;
    /**
     * The opacity of the object.
     *
     * @member {number} PIXI.DisplayObject#alpha
     */
    alpha: number;
    /**
     * The visibility of the object. If false the object will not be drawn, and
     * the updateTransform function will not be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds or call updateTransform manually.
     *
     * @member {boolean} PIXI.DisplayObject#visible
     */
    visible: boolean;
    /**
     * Can this object be rendered, if false the object will not be drawn but the updateTransform
     * methods will still be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds manually.
     *
     * @member {boolean} PIXI.DisplayObject#renderable
     */
    renderable: boolean;
    /**
     * The display object container that contains this display object.
     *
     * @member {PIXI.Container} PIXI.DisplayObject#parent
     * @readonly
     */
    readonly parent: PIXI.Container;
    /**
     * The multiplied alpha of the displayObject.
     *
     * @member {number} PIXI.DisplayObject#worldAlpha
     * @readonly
     */
    readonly worldAlpha: number;
    /**
     * Which index in the children array the display component was before the previous zIndex sort.
     * Used by containers to help sort objects with the same zIndex, by using previous array index as the decider.
     *
     * @member {number} PIXI.DisplayObject#_lastSortedIndex
     * @protected
     */
    protected _lastSortedIndex: number;
    /**
     * The zIndex of the displayObject.
     * A higher value will mean it will be rendered on top of other displayObjects within the same container.
     *
     * @member {number} PIXI.DisplayObject#_zIndex
     * @protected
     */
    protected _zIndex: number;
    /**
     * The area the filter is applied to. This is used as more of an optimization
     * rather than figuring out the dimensions of the displayObject each frame you can set this rectangle.
     *
     * Also works as an interaction mask.
     *
     * @member {?PIXI.Rectangle} PIXI.DisplayObject#filterArea
     */
    filterArea: PIXI.Rectangle;
    /**
     * Sets the filters for the displayObject.
     * * IMPORTANT: This is a WebGL only feature and will be ignored by the canvas renderer.
     * To remove filters simply set this property to `'null'`.
     *
     * @member {?PIXI.Filter[]} PIXI.DisplayObject#filters
     */
    filters: PIXI.Filter[];
    /**
     * The bounds object, this is used to calculate and store the bounds of the displayObject.
     *
     * @member {PIXI.Bounds} PIXI.DisplayObject#_bounds
     * @protected
     */
    protected _bounds: PIXI.Bounds;
    /**
     * The original, cached mask of the object.
     *
     * @member {PIXI.Graphics|PIXI.Sprite|null} PIXI.DisplayObject#_mask
     * @protected
     */
    protected _mask: PIXI.Graphics | PIXI.Sprite | null;
    /**
     * If the object has been destroyed via destroy(). If true, it should not be used.
     *
     * @member {boolean} PIXI.DisplayObject#_destroyed
     * @protected
     */
    protected _destroyed: boolean;
    /**
     * Does any other displayObject use this object as a mask?
     * @member {boolean} PIXI.DisplayObject#isMask
     */
    isMask: boolean;
    /**
     * @protected
     * @member {PIXI.DisplayObject}
     */
    protected _tempDisplayObjectParent: PIXI.DisplayObject;

    /**
     * Recursively updates transform of all objects from the root to this one
     * internal function for toLocal()
     */
    _recursivePostUpdateTransform(): void;

    /**
     * Retrieves the bounds of the displayObject as a rectangle object.
     *
     * @param {boolean} [skipUpdate] - Setting to `true` will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @param {PIXI.Rectangle} [rect] - Optional rectangle to store the result of the bounds calculation.
     * @return {PIXI.Rectangle} The rectangular bounding area.
     */
    getBounds(skipUpdate?: boolean, rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Calculates the global position of the display object.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform.
     * @return {PIXI.IPoint} A point object representing the position of this object.
     */
    toGlobal(position: PIXI.IPoint, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Calculates the local position of the display object relative to another point.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.DisplayObject} [from] - The DisplayObject to calculate the global position from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform
     * @return {PIXI.IPoint} A point object representing the position of this object
     */
    toLocal(position: PIXI.IPoint, from?: PIXI.DisplayObject, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Set the parent Container of this DisplayObject.
     *
     * @param {PIXI.Container} container - The Container to add this DisplayObject to.
     * @return {PIXI.Container} The Container that this DisplayObject was added to.
     */
    setParent(container: PIXI.Container): PIXI.Container;

    /**
     * Convenience function to set the position, scale, skew and pivot at once.
     *
     * @param {number} [x=0] - The X position
     * @param {number} [y=0] - The Y position
     * @param {number} [scaleX=1] - The X scale value
     * @param {number} [scaleY=1] - The Y scale value
     * @param {number} [rotation=0] - The rotation
     * @param {number} [skewX=0] - The X skew value
     * @param {number} [skewY=0] - The Y skew value
     * @param {number} [pivotX=0] - The X pivot value
     * @param {number} [pivotY=0] - The Y pivot value
     * @return {PIXI.DisplayObject} The DisplayObject instance
     */
    setTransform(
      x?: number,
      y?: number,
      scaleX?: number,
      scaleY?: number,
      rotation?: number,
      skewX?: number,
      skewY?: number,
      pivotX?: number,
      pivotY?: number,
    ): PIXI.DisplayObject;

    /**
     * The position of the displayObject on the x axis relative to the local coordinates of the parent.
     * An alias to position.x
     *
     * @member {number}
     */
    x: number;
    /**
     * The position of the displayObject on the y axis relative to the local coordinates of the parent.
     * An alias to position.y
     *
     * @member {number}
     */
    y: number;
    /**
     * Current transform of the object based on world (parent) factors.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly worldTransform: PIXI.Matrix;
    /**
     * Current transform of the object based on local factors: position, scale, other stuff.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly localTransform: PIXI.Matrix;
    /**
     * The coordinate of the object relative to the local coordinates of the parent.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    position: PIXI.IPoint;
    /**
     * The scale factor of the object.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    scale: PIXI.IPoint;
    /**
     * The pivot point of the displayObject that it rotates around.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    pivot: PIXI.IPoint;
    /**
     * The skew factor for the object in radians.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.ObservablePoint}
     */
    skew: PIXI.ObservablePoint;
    /**
     * The rotation of the object in radians.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    rotation: number;
    /**
     * The angle of the object in degrees.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    angle: number;
    /**
     * The zIndex of the displayObject.
     * If a container has the sortableChildren property set to true, children will be automatically
     * sorted by zIndex value; a higher value will mean it will be moved towards the end of the array,
     * and thus rendered on top of other displayObjects within the same container.
     *
     * @member {number}
     */
    zIndex: number;
    /**
     * Indicates if the object is globally visible.
     *
     * @member {boolean}
     * @readonly
     */
    readonly worldVisible: boolean;
    /**
     * Sets a mask for the displayObject. A mask is an object that limits the visibility of an
     * object to the shape of the mask applied to it. In PixiJS a regular mask must be a
     * {@link PIXI.Graphics} or a {@link PIXI.Sprite} object. This allows for much faster masking in canvas as it
     * utilities shape clipping. To remove a mask, set this property to `null`.
     *
     * For sprite mask both alpha and red channel are used. Black mask is the same as transparent mask.
     * @example
     * const graphics = new PIXI.Graphics();
     * graphics.beginFill(0xFF3300);
     * graphics.drawRect(50, 250, 100, 100);
     * graphics.endFill();
     *
     * const sprite = new PIXI.Sprite(texture);
     * sprite.mask = graphics;
     * @todo At the moment, PIXI.CanvasRenderer doesn't support PIXI.Sprite as mask.
     *
     * @member {PIXI.Container|PIXI.MaskData}
     */
    mask: PIXI.Container | PIXI.MaskData;

    /**
     * DisplayObject default updateTransform, does not update children of container.
     * Will crash if there's no parent element.
     *
     * @memberof PIXI.DisplayObject#
     * @function displayObjectUpdateTransform
     */
    displayObjectUpdateTransform(): void;

    /**
     * Enable interaction events for the DisplayObject. Touch, pointer and mouse
     * events will not be emitted unless `interactive` is set to `true`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.on('tap', (event) => {
     *    //handle event
     * });
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    interactive: boolean;
    /**
     * Interaction shape. Children will be hit first, then this shape will be checked.
     * Setting this will cause this shape to be checked in hit tests rather than the displayObject's bounds.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.hitArea = new PIXI.Rectangle(0, 0, 100, 100);
     * @member {PIXI.IHitArea}
     * @memberof PIXI.DisplayObject#
     */
    hitArea: PIXI.IHitArea;
    /**
     * If enabled, the mouse cursor use the pointer behavior when hovered over the displayObject if it is interactive
     * Setting this changes the 'cursor' property to `'pointer'`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.buttonMode = true;
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    buttonMode: boolean;
    /**
     * This defines what cursor mode is used when the mouse cursor
     * is hovered over the displayObject.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.cursor = 'wait';
     * @see https://developer.mozilla.org/en/docs/Web/CSS/cursor
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    cursor: string;
    /**
     * Set this to true if you want this display object to be cached as a bitmap.
     * This basically takes a snap shot of the display object as it is at that moment. It can
     * provide a performance benefit for complex static displayObjects.
     * To remove simply set this property to `false`
     *
     * IMPORTANT GOTCHA - Make sure that all your textures are preloaded BEFORE setting this property to true
     * as it will take a snapshot of what is currently there. If the textures have not loaded then they will not appear.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    cacheAsBitmap: boolean;
    /**
     * The instance name of the object.
     *
     * @memberof PIXI.DisplayObject#
     * @member {string} name
     */
    name: string;

    /**
     * Returns the global position of the displayObject. Does not depend on object scale, rotation and pivot.
     *
     * @method getGlobalPosition
     * @memberof PIXI.DisplayObject#
     * @param {PIXI.Point} [point=new PIXI.Point()] - The point to write the global value to.
     * @param {boolean} [skipUpdate=false] - Setting to true will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @return {PIXI.Point} The updated point.
     */
    getGlobalPosition(point?: PIXI.Point, skipUpdate?: boolean): PIXI.Point;
  }

  /**
   * A number, or a string containing a number.
   *
   * @memberof PIXI
   * @typedef IFontMetrics
   * @property {number} ascent - Font ascent
   * @property {number} descent - Font descent
   * @property {number} fontSize - Font size
   */
  type IFontMetrics = {
    ascent: number;
    descent: number;
    fontSize: number;
  };

  /**
   * The TextMetrics object represents the measurement of a block of text with a specified style.
   *
   * ```js
   * let style = new PIXI.TextStyle({fontFamily : 'Arial', fontSize: 24, fill : 0xff1010, align : 'center'})
   * let textMetrics = PIXI.TextMetrics.measureText('Your text', style)
   * ```
   *
   * @class
   * @memberof PIXI
   */
  class TextMetrics {
    constructor(
      text: string,
      style: PIXI.TextStyle,
      width: number,
      height: number,
      lines: string[],
      lineWidths: number[],
      lineHeight: number,
      maxLineWidth: number,
      fontProperties: any,
    );

    /**
     * The text that was measured
     *
     * @member {string} PIXI.TextMetrics#text
     */
    text: string;
    /**
     * The style that was measured
     *
     * @member {PIXI.TextStyle} PIXI.TextMetrics#style
     */
    style: PIXI.TextStyle;
    /**
     * The measured width of the text
     *
     * @member {number} PIXI.TextMetrics#width
     */
    width: number;
    /**
     * The measured height of the text
     *
     * @member {number} PIXI.TextMetrics#height
     */
    height: number;
    /**
     * An array of lines of the text broken by new lines and wrapping is specified in style
     *
     * @member {string[]} PIXI.TextMetrics#lines
     */
    lines: string[];
    /**
     * An array of the line widths for each line matched to `lines`
     *
     * @member {number[]} PIXI.TextMetrics#lineWidths
     */
    lineWidths: number[];
    /**
     * The measured line height for this style
     *
     * @member {number} PIXI.TextMetrics#lineHeight
     */
    lineHeight: number;
    /**
     * The maximum line width for all measured lines
     *
     * @member {number} PIXI.TextMetrics#maxLineWidth
     */
    maxLineWidth: number;
    /**
     * The font properties object from TextMetrics.measureFont
     *
     * @member {PIXI.IFontMetrics} PIXI.TextMetrics#fontProperties
     */
    fontProperties: PIXI.IFontMetrics;

    /**
     * Measures the supplied string of text and returns a Rectangle.
     *
     * @param {string} text - the text to measure.
     * @param {PIXI.TextStyle} style - the text style to use for measuring
     * @param {boolean} [wordWrap] - optional override for if word-wrap should be applied to the text.
     * @param {HTMLCanvasElement} [canvas] - optional specification of the canvas to use for measuring.
     * @return {PIXI.TextMetrics} measured width and height of the text.
     */
    static measureText(
      text: string,
      style: PIXI.TextStyle,
      wordWrap?: boolean,
      canvas?: HTMLCanvasElement,
    ): PIXI.TextMetrics;

    /**
     * Overridable helper method used internally by TextMetrics, exposed to allow customizing the class's behavior.
     *
     * It allows one to customise which words should break
     * Examples are if the token is CJK or numbers.
     * It must return a boolean.
     *
     * @param  {string}  token       The token
     * @param  {boolean}  breakWords  The style attr break words
     * @return {boolean} whether to break word or not
     */
    static canBreakWords(token: string, breakWords: boolean): boolean;

    /**
     * Overridable helper method used internally by TextMetrics, exposed to allow customizing the class's behavior.
     *
     * It allows one to determine whether a pair of characters
     * should be broken by newlines
     * For example certain characters in CJK langs or numbers.
     * It must return a boolean.
     *
     * @param  {string}  char      The character
     * @param  {string}  nextChar  The next character
     * @param  {string}  token     The token/word the characters are from
     * @param  {number}  index     The index in the token of the char
     * @param  {boolean}  breakWords  The style attr break words
     * @return {boolean} whether to break word or not
     */
    static canBreakChars(char: string, nextChar: string, token: string, index: number, breakWords: boolean): boolean;

    /**
     * Overridable helper method used internally by TextMetrics, exposed to allow customizing the class's behavior.
     *
     * It is called when a token (usually a word) has to be split into separate pieces
     * in order to determine the point to break a word.
     * It must return an array of characters.
     *
     * @example
     * // Correctly splits emojis, eg "🤪🤪" will result in two element array, each with one emoji.
     * TextMetrics.wordWrapSplit = (token) => [...token];
     *
     * @param  {string}  token The token to split
     * @return {string[]} The characters of the token
     */
    static wordWrapSplit(token: string): string[];

    /**
     * Calculates the ascent, descent and fontSize of a given font-style
     *
     * @static
     * @param {string} font - String representing the style of the font
     * @return {PIXI.IFontMetrics} Font properties object
     */
    static measureFont(font: string): PIXI.IFontMetrics;

    /**
     * Clear font metrics in metrics cache.
     *
     * @static
     * @param {string} [font] - font name. If font name not set then clear cache for all fonts.
     */
    static clearMetrics(font?: string): void;

    /**
     * String used for calculate font metrics.
     * These characters are all tall to help calculate the height required for text.
     *
     * @static
     * @memberof PIXI.TextMetrics
     * @name METRICS_STRING
     * @type {string}
     * @default |ÉqÅ
     */
    static METRICS_STRING: string;
    /**
     * Baseline symbol for calculate font metrics.
     *
     * @static
     * @memberof PIXI.TextMetrics
     * @name BASELINE_SYMBOL
     * @type {string}
     * @default M
     */
    static BASELINE_SYMBOL: string;
    /**
     * Baseline multiplier for calculate font metrics.
     *
     * @static
     * @memberof PIXI.TextMetrics
     * @name BASELINE_MULTIPLIER
     * @type {number}
     * @default 1.4
     */
    static BASELINE_MULTIPLIER: number;
  }

  /**
   * A TextStyle Object contains information to decorate a Text objects.
   *
   * An instance can be shared between multiple Text objects; then changing the style will update all text objects using it.
   *
   * A tool can be used to generate a text style [here](https://pixijs.io/pixi-text-style).
   *
   * @class
   * @memberof PIXI
   */
  class TextStyle {
    constructor(style?: {
      align?: string;
      breakWords?: boolean;
      dropShadow?: boolean;
      dropShadowAlpha?: number;
      dropShadowAngle?: number;
      dropShadowBlur?: number;
      dropShadowColor?: string | number;
      dropShadowDistance?: number;
      fill?: string | string[] | number | number[] | CanvasGradient | CanvasPattern;
      fillGradientType?: number;
      fillGradientStops?: number[];
      fontFamily?: string | string[];
      fontSize?: number | string;
      fontStyle?: string;
      fontVariant?: string;
      fontWeight?: string;
      leading?: number;
      letterSpacing?: number;
      lineHeight?: number;
      lineJoin?: string;
      miterLimit?: number;
      padding?: number;
      stroke?: string | number;
      strokeThickness?: number;
      trim?: boolean;
      textBaseline?: string;
      whiteSpace?: string;
      wordWrap?: boolean;
      wordWrapWidth?: number;
    });

    /**
     * Creates a new TextStyle object with the same values as this one.
     * Note that the only the properties of the object are cloned.
     *
     * @return {PIXI.TextStyle} New cloned TextStyle object
     */
    clone(): PIXI.TextStyle;

    /**
     * Resets all properties to the defaults specified in TextStyle.prototype._default
     */
    reset(): void;

    /**
     * Alignment for multiline text ('left', 'center' or 'right'), does not affect single line text
     *
     * @member {string}
     */
    align: string;
    /**
     * Indicates if lines can be wrapped within words, it needs wordWrap to be set to true
     *
     * @member {boolean}
     */
    breakWords: boolean;
    /**
     * Set a drop shadow for the text
     *
     * @member {boolean}
     */
    dropShadow: boolean;
    /**
     * Set alpha for the drop shadow
     *
     * @member {number}
     */
    dropShadowAlpha: number;
    /**
     * Set a angle of the drop shadow
     *
     * @member {number}
     */
    dropShadowAngle: number;
    /**
     * Set a shadow blur radius
     *
     * @member {number}
     */
    dropShadowBlur: number;
    /**
     * A fill style to be used on the dropshadow e.g 'red', '#00FF00'
     *
     * @member {string|number}
     */
    dropShadowColor: string | number;
    /**
     * Set a distance of the drop shadow
     *
     * @member {number}
     */
    dropShadowDistance: number;
    /**
     * A canvas fillstyle that will be used on the text e.g 'red', '#00FF00'.
     * Can be an array to create a gradient eg ['#000000','#FFFFFF']
     * {@link https://developer.mozilla.org/en-US/docs/Web/API/CanvasRenderingContext2D/fillStyle|MDN}
     *
     * @member {string|string[]|number|number[]|CanvasGradient|CanvasPattern}
     */
    fill: string | string[] | number | number[] | CanvasGradient | CanvasPattern;
    /**
     * If fill is an array of colours to create a gradient, this can change the type/direction of the gradient.
     * See {@link PIXI.TEXT_GRADIENT}
     *
     * @member {number}
     */
    fillGradientType: number;
    /**
     * If fill is an array of colours to create a gradient, this array can set the stop points
     * (numbers between 0 and 1) for the color, overriding the default behaviour of evenly spacing them.
     *
     * @member {number[]}
     */
    fillGradientStops: number[];
    /**
     * The font family
     *
     * @member {string|string[]}
     */
    fontFamily: string | string[];
    /**
     * The font size
     * (as a number it converts to px, but as a string, equivalents are '26px','20pt','160%' or '1.6em')
     *
     * @member {number|string}
     */
    fontSize: number | string;
    /**
     * The font style
     * ('normal', 'italic' or 'oblique')
     *
     * @member {string}
     */
    fontStyle: string;
    /**
     * The font variant
     * ('normal' or 'small-caps')
     *
     * @member {string}
     */
    fontVariant: string;
    /**
     * The font weight
     * ('normal', 'bold', 'bolder', 'lighter' and '100', '200', '300', '400', '500', '600', '700', 800' or '900')
     *
     * @member {string}
     */
    fontWeight: string;
    /**
     * The amount of spacing between letters, default is 0
     *
     * @member {number}
     */
    letterSpacing: number;
    /**
     * The line height, a number that represents the vertical space that a letter uses
     *
     * @member {number}
     */
    lineHeight: number;
    /**
     * The space between lines
     *
     * @member {number}
     */
    leading: number;
    /**
     * The lineJoin property sets the type of corner created, it can resolve spiked text issues.
     * Default is 'miter' (creates a sharp corner).
     *
     * @member {string}
     */
    lineJoin: string;
    /**
     * The miter limit to use when using the 'miter' lineJoin mode
     * This can reduce or increase the spikiness of rendered text.
     *
     * @member {number}
     */
    miterLimit: number;
    /**
     * Occasionally some fonts are cropped. Adding some padding will prevent this from happening
     * by adding padding to all sides of the text.
     *
     * @member {number}
     */
    padding: number;
    /**
     * A canvas fillstyle that will be used on the text stroke
     * e.g 'blue', '#FCFF00'
     *
     * @member {string|number}
     */
    stroke: string | number;
    /**
     * A number that represents the thickness of the stroke.
     * Default is 0 (no stroke)
     *
     * @member {number}
     */
    strokeThickness: number;
    /**
     * The baseline of the text that is rendered.
     *
     * @member {string}
     */
    textBaseline: string;
    /**
     * Trim transparent borders
     *
     * @member {boolean}
     */
    trim: boolean;
    /**
     * How newlines and spaces should be handled.
     * Default is 'pre' (preserve, preserve).
     *
     *  value       | New lines     |   Spaces
     *  ---         | ---           |   ---
     * 'normal'     | Collapse      |   Collapse
     * 'pre'        | Preserve      |   Preserve
     * 'pre-line'   | Preserve      |   Collapse
     *
     * @member {string}
     */
    whiteSpace: string;
    /**
     * Indicates if word wrap should be used
     *
     * @member {boolean}
     */
    wordWrap: boolean;
    /**
     * The width at which text will wrap, it needs wordWrap to be set to true
     *
     * @member {number}
     */
    wordWrapWidth: number;

    /**
     * Generates a font style string to use for `TextMetrics.measureFont()`.
     *
     * @return {string} Font style string, for passing to `TextMetrics.measureFont()`
     */
    toFontString(): string;
  }

  /**
   * Constants that define the type of gradient on text.
   *
   * @static
   * @constant
   * @name TEXT_GRADIENT
   * @memberof PIXI
   * @type {object}
   * @property {number} LINEAR_VERTICAL Vertical gradient
   * @property {number} LINEAR_HORIZONTAL Linear gradient
   */
  var TEXT_GRADIENT: {
    LINEAR_VERTICAL: number;
    LINEAR_HORIZONTAL: number;
  };

  interface BitmapFontLoader extends PIXI.ILoaderPlugin {}

  /**
   * {@link PIXI.Loader Loader} middleware for loading
   * bitmap-based fonts suitable for using with {@link PIXI.BitmapText}.
   * @class
   * @memberof PIXI
   * @implements PIXI.ILoaderPlugin
   */
  class BitmapFontLoader implements PIXI.ILoaderPlugin {
    /**
     * Register a BitmapText font from loader resource.
     *
     * @param {PIXI.LoaderResource} resource - Loader resource.
     * @param {PIXI.Texture} texture - Reference to texture.
     */
    static parse(resource: PIXI.LoaderResource, texture: PIXI.Texture): void;

    /**
     * Called when the plugin is installed.
     *
     * @see PIXI.Loader.registerPlugin
     */
    static add(): void;

    /**
     * Called after a resource is loaded.
     * @see PIXI.Loader.loaderMiddleware
     * @param {PIXI.LoaderResource} resource
     * @param {function} next
     */
    static use(resource: PIXI.LoaderResource, next: (...params: any[]) => any): void;
  }

  /**
   * A BitmapText object will create a line or multiple lines of text using bitmap font.
   *
   * The primary advantage of this class over Text is that all of your textures are pre-generated and loading,
   * meaning that rendering is fast, and changing text has no performance implications.
   *
   * The primary disadvantage is that you need to preload the bitmap font assets, and thus the styling is set in stone.
   * Supporting character sets other than latin, such as CJK languages, may be impractical due to the number of characters.
   *
   * To split a line you can use '\n', '\r' or '\r\n' in your string.
   *
   * You can generate the fnt files using
   * http://www.angelcode.com/products/bmfont/ for Windows or
   * http://www.bmglyph.com/ for Mac.
   *
   * A BitmapText can only be created when the font is loaded.
   *
   * ```js
   * // in this case the font is in a file called 'desyrel.fnt'
   * let bitmapText = new PIXI.BitmapText("text using a fancy font!", {font: "35px Desyrel", align: "right"});
   * ```
   *
   * @class
   * @extends PIXI.Container
   * @memberof PIXI
   */
  class BitmapText extends PIXI.Container {
    constructor(
      text: string,
      style: {
        font: {
          name?: string;
          size?: number;
        };
        align?: string;
        tint?: number;
      },
    );

    /**
     * The dirty state of this object.
     *
     * @member {boolean} PIXI.BitmapText#dirty
     */
    dirty: boolean;
    /**
     * If true PixiJS will Math.floor() x/y values when rendering, stopping pixel interpolation.
     * Advantages can include sharper image quality (like text) and faster rendering on canvas.
     * The main disadvantage is movement of objects may appear less smooth.
     * To set the global default, change {@link PIXI.settings.ROUND_PIXELS}
     *
     * @member {boolean} PIXI.BitmapText#roundPixels
     * @default false
     */
    roundPixels: boolean;

    /**
     * Validates text before calling parent's getLocalBounds
     *
     * @return {PIXI.Rectangle} The rectangular bounding area
     */
    getLocalBounds(): PIXI.Rectangle;

    /**
     * The tint of the BitmapText object.
     *
     * @member {number}
     */
    tint: number;
    /**
     * The alignment of the BitmapText object.
     *
     * @member {string}
     * @default 'left'
     */
    align: string;
    /**
     * The anchor sets the origin point of the text.
     *
     * The default is `(0,0)`, this means the text's origin is the top left.
     *
     * Setting the anchor to `(0.5,0.5)` means the text's origin is centered.
     *
     * Setting the anchor to `(1,1)` would mean the text's origin point will be the bottom right corner.
     *
     * @member {PIXI.Point | number}
     */
    anchor: PIXI.Point | number;
    /**
     * The font descriptor of the BitmapText object.
     *
     * @member {object}
     */
    font: any;
    /**
     * The text of the BitmapText object.
     *
     * @member {string}
     */
    text: string;
    /**
     * The max width of this bitmap text in pixels. If the text provided is longer than the
     * value provided, line breaks will be automatically inserted in the last whitespace.
     * Disable by setting the value to 0.
     *
     * @member {number}
     */
    maxWidth: number;
    /**
     * The max line height. This is useful when trying to use the total height of the Text,
     * i.e. when trying to vertically align.
     *
     * @member {number}
     * @readonly
     */
    readonly maxLineHeight: number;
    /**
     * The width of the overall text, different from fontSize,
     * which is defined in the style object.
     *
     * @member {number}
     * @readonly
     */
    readonly textWidth: number;
    /**
     * Additional space between characters.
     *
     * @member {number}
     */
    letterSpacing: number;
    /**
     * The height of the overall text, different from fontSize,
     * which is defined in the style object.
     *
     * @member {number}
     * @readonly
     */
    readonly textHeight: number;

    /**
     * Register a bitmap font with data and a texture.
     *
     * @static
     * @param {XMLDocument} xml - The XML document data.
     * @param {Object.<string, PIXI.Texture>|PIXI.Texture|PIXI.Texture[]} textures - List of textures for each page.
     *  If providing an object, the key is the `<page>` element's `file` attribute in the FNT file.
     * @return {Object} Result font object with font, size, lineHeight and char fields.
     */
    static registerFont(
      xml: XMLDocument,
      textures:
        | {
            [key: string]: PIXI.Texture;
          }
        | PIXI.Texture
        | PIXI.Texture[],
    ): any;

    /**
     * The array of children of this container.
     *
     * @member {PIXI.DisplayObject[]} PIXI.Container#children
     * @readonly
     */
    readonly children: PIXI.DisplayObject[];
    /**
     * If set to true, the container will sort its children by zIndex value
     * when updateTransform() is called, or manually if sortChildren() is called.
     *
     * This actually changes the order of elements in the array, so should be treated
     * as a basic solution that is not performant compared to other solutions,
     * such as @link https://github.com/pixijs/pixi-display
     *
     * Also be aware of that this may not work nicely with the addChildAt() function,
     * as the zIndex sorting may cause the child to automatically sorted to another position.
     *
     * @see PIXI.settings.SORTABLE_CHILDREN
     *
     * @member {boolean} PIXI.Container#sortableChildren
     */
    sortableChildren: boolean;
    /**
     * Should children be sorted by zIndex at the next updateTransform call.
     * Will get automatically set to true if a new child is added, or if a child's zIndex changes.
     *
     * @member {boolean} PIXI.Container#sortDirty
     */
    sortDirty: boolean;

    /**
     * Overridable method that can be used by Container subclasses whenever the children array is modified
     *
     * @protected
     */
    protected onChildrenChange(): void;

    /**
     * Adds one or more children to the container.
     *
     * Multiple items can be added like so: `myContainer.addChild(thingOne, thingTwo, thingThree)`
     *
     * @param {...PIXI.DisplayObject} child - The DisplayObject(s) to add to the container
     * @return {PIXI.DisplayObject} The first child that was added.
     */
    addChild<TChildren extends PIXI.DisplayObject[]>(...child: TChildren): TChildren[0];

    /**
     * Adds a child to the container at a specified index. If the index is out of bounds an error will be thrown
     *
     * @param {PIXI.DisplayObject} child - The child to add
     * @param {number} index - The index to place the child in
     * @return {PIXI.DisplayObject} The child that was added.
     */
    addChildAt<T extends PIXI.DisplayObject>(child: T, index: number): T;

    /**
     * Swaps the position of 2 Display Objects within this container.
     *
     * @param {PIXI.DisplayObject} child - First display object to swap
     * @param {PIXI.DisplayObject} child2 - Second display object to swap
     */
    swapChildren(child: PIXI.DisplayObject, child2: PIXI.DisplayObject): void;

    /**
     * Returns the index position of a child DisplayObject instance
     *
     * @param {PIXI.DisplayObject} child - The DisplayObject instance to identify
     * @return {number} The index position of the child display object to identify
     */
    getChildIndex(child: PIXI.DisplayObject): number;

    /**
     * Changes the position of an existing child in the display object container
     *
     * @param {PIXI.DisplayObject} child - The child DisplayObject instance for which you want to change the index number
     * @param {number} index - The resulting index number for the child display object
     */
    setChildIndex(child: PIXI.DisplayObject, index: number): void;

    /**
     * Returns the child at the specified index
     *
     * @param {number} index - The index to get the child at
     * @return {PIXI.DisplayObject} The child at the given index, if any.
     */
    getChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes one or more children from the container.
     *
     * @param {...PIXI.DisplayObject} child - The DisplayObject(s) to remove
     * @return {PIXI.DisplayObject} The first child that was removed.
     */
    removeChild<TChildren extends PIXI.DisplayObject[]>(...child: TChildren): TChildren[0];

    /**
     * Removes a child from the specified index position.
     *
     * @param {number} index - The index to get the child from
     * @return {PIXI.DisplayObject} The child that was removed.
     */
    removeChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes all children from this container that are within the begin and end indexes.
     *
     * @param {number} [beginIndex=0] - The beginning position.
     * @param {number} [endIndex=this.children.length] - The ending position. Default value is size of the container.
     * @returns {PIXI.DisplayObject[]} List of removed children
     */
    removeChildren(beginIndex?: number, endIndex?: number): PIXI.DisplayObject[];

    /**
     * Sorts children by zIndex. Previous order is mantained for 2 children with the same zIndex.
     */
    sortChildren(): void;

    /**
     * Recalculates the bounds of the container.
     *
     */
    calculateBounds(): void;

    /**
     * Recalculates the bounds of the object. Override this to
     * calculate the bounds of the specific object (not including children).
     *
     * @protected
     */
    protected _calculateBounds(): void;

    /**
     * Renders the object using the WebGL renderer
     *
     * @param {PIXI.Renderer} renderer - The renderer
     */
    render(renderer: PIXI.Renderer): void;

    /**
     * Render the object using the WebGL renderer and advanced features.
     *
     * @protected
     * @param {PIXI.Renderer} renderer - The renderer
     */
    protected renderAdvanced(renderer: PIXI.Renderer): void;

    /**
     * To be overridden by the subclasses.
     *
     * @protected
     * @param {PIXI.Renderer} renderer - The renderer
     */
    protected _render(renderer: PIXI.Renderer): void;

    /**
     * Removes all internal references and listeners as well as removes children from the display list.
     * Do not use a Container after calling `destroy`.
     *
     * @param {object|boolean} [options] - Options parameter. A boolean will act as if all options
     *  have been set to that value
     * @param {boolean} [options.children=false] - if set to true, all the children will have their destroy
     *  method called as well. 'options' will be passed on to those calls.
     * @param {boolean} [options.texture=false] - Only used for child Sprites if options.children is set to true
     *  Should it destroy the texture of the child sprite
     * @param {boolean} [options.baseTexture=false] - Only used for child Sprites if options.children is set to true
     *  Should it destroy the base texture of the child sprite
     */
    destroy(options?: { children?: boolean; texture?: boolean; baseTexture?: boolean }): void;

    /**
     * The width of the Container, setting this will actually modify the scale to achieve the value set
     *
     * @member {number}
     */
    width: number;
    /**
     * The height of the Container, setting this will actually modify the scale to achieve the value set
     *
     * @member {number}
     */
    height: number;
    /**
     * Determines if the children to the displayObject can be clicked/touched
     * Setting this to false allows PixiJS to bypass a recursive `hitTest` function
     *
     * @member {boolean}
     * @memberof PIXI.Container#
     */
    interactiveChildren: boolean;

    /**
     * Returns the display object in the container.
     *
     * @method getChildByName
     * @memberof PIXI.Container#
     * @param {string} name - Instance name.
     * @return {PIXI.DisplayObject} The child with the specified name.
     */
    getChildByName(name: string): PIXI.DisplayObject;

    /**
     *  Flag for if the object is accessible. If true AccessibilityManager will overlay a
     *   shadow div with attributes set
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    accessible: boolean;
    /**
     * Sets the title attribute of the shadow div
     * If accessibleTitle AND accessibleHint has not been this will default to 'displayObject [tabIndex]'
     *
     * @member {?string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleTitle: string;
    /**
     * Sets the aria-label attribute of the shadow div
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    accessibleHint: string;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleActive: boolean;
    /**
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @todo Needs docs.
     */
    _accessibleDiv: boolean;
    /**
     * Specify the type of div the accessible layer is. Screen readers treat the element differently
     * depending on this type. Defaults to button.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'button'
     */
    accessibleType: string;
    /**
     * Specify the pointer-events the accessible div will use
     * Defaults to auto.
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     * @default 'auto'
     */
    accessiblePointerEvents: string;
    /**
     * Setting to false will prevent any children inside this container to
     * be accessible. Defaults to true.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     * @default true
     */
    accessibleChildren: boolean;
    /**
     * World transform and local transform of this object.
     * This will become read-only later, please do not assign anything there unless you know what are you doing.
     *
     * @member {PIXI.Transform} PIXI.DisplayObject#transform
     */
    transform: PIXI.Transform;
    /**
     * The opacity of the object.
     *
     * @member {number} PIXI.DisplayObject#alpha
     */
    alpha: number;
    /**
     * The visibility of the object. If false the object will not be drawn, and
     * the updateTransform function will not be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds or call updateTransform manually.
     *
     * @member {boolean} PIXI.DisplayObject#visible
     */
    visible: boolean;
    /**
     * Can this object be rendered, if false the object will not be drawn but the updateTransform
     * methods will still be called.
     *
     * Only affects recursive calls from parent. You can ask for bounds manually.
     *
     * @member {boolean} PIXI.DisplayObject#renderable
     */
    renderable: boolean;
    /**
     * The display object container that contains this display object.
     *
     * @member {PIXI.Container} PIXI.DisplayObject#parent
     * @readonly
     */
    readonly parent: PIXI.Container;
    /**
     * The multiplied alpha of the displayObject.
     *
     * @member {number} PIXI.DisplayObject#worldAlpha
     * @readonly
     */
    readonly worldAlpha: number;
    /**
     * Which index in the children array the display component was before the previous zIndex sort.
     * Used by containers to help sort objects with the same zIndex, by using previous array index as the decider.
     *
     * @member {number} PIXI.DisplayObject#_lastSortedIndex
     * @protected
     */
    protected _lastSortedIndex: number;
    /**
     * The zIndex of the displayObject.
     * A higher value will mean it will be rendered on top of other displayObjects within the same container.
     *
     * @member {number} PIXI.DisplayObject#_zIndex
     * @protected
     */
    protected _zIndex: number;
    /**
     * The area the filter is applied to. This is used as more of an optimization
     * rather than figuring out the dimensions of the displayObject each frame you can set this rectangle.
     *
     * Also works as an interaction mask.
     *
     * @member {?PIXI.Rectangle} PIXI.DisplayObject#filterArea
     */
    filterArea: PIXI.Rectangle;
    /**
     * Sets the filters for the displayObject.
     * * IMPORTANT: This is a WebGL only feature and will be ignored by the canvas renderer.
     * To remove filters simply set this property to `'null'`.
     *
     * @member {?PIXI.Filter[]} PIXI.DisplayObject#filters
     */
    filters: PIXI.Filter[];
    /**
     * The bounds object, this is used to calculate and store the bounds of the displayObject.
     *
     * @member {PIXI.Bounds} PIXI.DisplayObject#_bounds
     * @protected
     */
    protected _bounds: PIXI.Bounds;
    /**
     * The original, cached mask of the object.
     *
     * @member {PIXI.Graphics|PIXI.Sprite|null} PIXI.DisplayObject#_mask
     * @protected
     */
    protected _mask: PIXI.Graphics | PIXI.Sprite | null;
    /**
     * If the object has been destroyed via destroy(). If true, it should not be used.
     *
     * @member {boolean} PIXI.DisplayObject#_destroyed
     * @protected
     */
    protected _destroyed: boolean;
    /**
     * used to fast check if a sprite is.. a sprite!
     * @member {boolean} PIXI.DisplayObject#isSprite
     */
    isSprite: boolean;
    /**
     * Does any other displayObject use this object as a mask?
     * @member {boolean} PIXI.DisplayObject#isMask
     */
    isMask: boolean;
    /**
     * @protected
     * @member {PIXI.DisplayObject}
     */
    protected _tempDisplayObjectParent: PIXI.DisplayObject;

    /**
     * Recursively updates transform of all objects from the root to this one
     * internal function for toLocal()
     */
    _recursivePostUpdateTransform(): void;

    /**
     * Retrieves the bounds of the displayObject as a rectangle object.
     *
     * @param {boolean} [skipUpdate] - Setting to `true` will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @param {PIXI.Rectangle} [rect] - Optional rectangle to store the result of the bounds calculation.
     * @return {PIXI.Rectangle} The rectangular bounding area.
     */
    getBounds(skipUpdate?: boolean, rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Calculates the global position of the display object.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform.
     * @return {PIXI.IPoint} A point object representing the position of this object.
     */
    toGlobal(position: PIXI.IPoint, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Calculates the local position of the display object relative to another point.
     *
     * @param {PIXI.IPoint} position - The world origin to calculate from.
     * @param {PIXI.DisplayObject} [from] - The DisplayObject to calculate the global position from.
     * @param {PIXI.IPoint} [point] - A Point object in which to store the value, optional
     *  (otherwise will create a new Point).
     * @param {boolean} [skipUpdate=false] - Should we skip the update transform
     * @return {PIXI.IPoint} A point object representing the position of this object
     */
    toLocal(position: PIXI.IPoint, from?: PIXI.DisplayObject, point?: PIXI.IPoint, skipUpdate?: boolean): PIXI.IPoint;

    /**
     * Set the parent Container of this DisplayObject.
     *
     * @param {PIXI.Container} container - The Container to add this DisplayObject to.
     * @return {PIXI.Container} The Container that this DisplayObject was added to.
     */
    setParent(container: PIXI.Container): PIXI.Container;

    /**
     * Convenience function to set the position, scale, skew and pivot at once.
     *
     * @param {number} [x=0] - The X position
     * @param {number} [y=0] - The Y position
     * @param {number} [scaleX=1] - The X scale value
     * @param {number} [scaleY=1] - The Y scale value
     * @param {number} [rotation=0] - The rotation
     * @param {number} [skewX=0] - The X skew value
     * @param {number} [skewY=0] - The Y skew value
     * @param {number} [pivotX=0] - The X pivot value
     * @param {number} [pivotY=0] - The Y pivot value
     * @return {PIXI.DisplayObject} The DisplayObject instance
     */
    setTransform(
      x?: number,
      y?: number,
      scaleX?: number,
      scaleY?: number,
      rotation?: number,
      skewX?: number,
      skewY?: number,
      pivotX?: number,
      pivotY?: number,
    ): PIXI.DisplayObject;

    /**
     * The position of the displayObject on the x axis relative to the local coordinates of the parent.
     * An alias to position.x
     *
     * @member {number}
     */
    x: number;
    /**
     * The position of the displayObject on the y axis relative to the local coordinates of the parent.
     * An alias to position.y
     *
     * @member {number}
     */
    y: number;
    /**
     * Current transform of the object based on world (parent) factors.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly worldTransform: PIXI.Matrix;
    /**
     * Current transform of the object based on local factors: position, scale, other stuff.
     *
     * @member {PIXI.Matrix}
     * @readonly
     */
    readonly localTransform: PIXI.Matrix;
    /**
     * The coordinate of the object relative to the local coordinates of the parent.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    position: PIXI.IPoint;
    /**
     * The scale factor of the object.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    scale: PIXI.IPoint;
    /**
     * The pivot point of the displayObject that it rotates around.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.IPoint}
     */
    pivot: PIXI.IPoint;
    /**
     * The skew factor for the object in radians.
     * Assignment by value since pixi-v4.
     *
     * @member {PIXI.ObservablePoint}
     */
    skew: PIXI.ObservablePoint;
    /**
     * The rotation of the object in radians.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    rotation: number;
    /**
     * The angle of the object in degrees.
     * 'rotation' and 'angle' have the same effect on a display object; rotation is in radians, angle is in degrees.
     *
     * @member {number}
     */
    angle: number;
    /**
     * The zIndex of the displayObject.
     * If a container has the sortableChildren property set to true, children will be automatically
     * sorted by zIndex value; a higher value will mean it will be moved towards the end of the array,
     * and thus rendered on top of other displayObjects within the same container.
     *
     * @member {number}
     */
    zIndex: number;
    /**
     * Indicates if the object is globally visible.
     *
     * @member {boolean}
     * @readonly
     */
    readonly worldVisible: boolean;
    /**
     * Sets a mask for the displayObject. A mask is an object that limits the visibility of an
     * object to the shape of the mask applied to it. In PixiJS a regular mask must be a
     * {@link PIXI.Graphics} or a {@link PIXI.Sprite} object. This allows for much faster masking in canvas as it
     * utilities shape clipping. To remove a mask, set this property to `null`.
     *
     * For sprite mask both alpha and red channel are used. Black mask is the same as transparent mask.
     * @example
     * const graphics = new PIXI.Graphics();
     * graphics.beginFill(0xFF3300);
     * graphics.drawRect(50, 250, 100, 100);
     * graphics.endFill();
     *
     * const sprite = new PIXI.Sprite(texture);
     * sprite.mask = graphics;
     * @todo At the moment, PIXI.CanvasRenderer doesn't support PIXI.Sprite as mask.
     *
     * @member {PIXI.Container|PIXI.MaskData}
     */
    mask: PIXI.Container | PIXI.MaskData;

    /**
     * DisplayObject default updateTransform, does not update children of container.
     * Will crash if there's no parent element.
     *
     * @memberof PIXI.DisplayObject#
     * @function displayObjectUpdateTransform
     */
    displayObjectUpdateTransform(): void;

    /**
     * Enable interaction events for the DisplayObject. Touch, pointer and mouse
     * events will not be emitted unless `interactive` is set to `true`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.on('tap', (event) => {
     *    //handle event
     * });
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    interactive: boolean;
    /**
     * Interaction shape. Children will be hit first, then this shape will be checked.
     * Setting this will cause this shape to be checked in hit tests rather than the displayObject's bounds.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.hitArea = new PIXI.Rectangle(0, 0, 100, 100);
     * @member {PIXI.IHitArea}
     * @memberof PIXI.DisplayObject#
     */
    hitArea: PIXI.IHitArea;
    /**
     * If enabled, the mouse cursor use the pointer behavior when hovered over the displayObject if it is interactive
     * Setting this changes the 'cursor' property to `'pointer'`.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.buttonMode = true;
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    buttonMode: boolean;
    /**
     * This defines what cursor mode is used when the mouse cursor
     * is hovered over the displayObject.
     *
     * @example
     * const sprite = new PIXI.Sprite(texture);
     * sprite.interactive = true;
     * sprite.cursor = 'wait';
     * @see https://developer.mozilla.org/en/docs/Web/CSS/cursor
     *
     * @member {string}
     * @memberof PIXI.DisplayObject#
     */
    cursor: string;
    /**
     * Set this to true if you want this display object to be cached as a bitmap.
     * This basically takes a snap shot of the display object as it is at that moment. It can
     * provide a performance benefit for complex static displayObjects.
     * To remove simply set this property to `false`
     *
     * IMPORTANT GOTCHA - Make sure that all your textures are preloaded BEFORE setting this property to true
     * as it will take a snapshot of what is currently there. If the textures have not loaded then they will not appear.
     *
     * @member {boolean}
     * @memberof PIXI.DisplayObject#
     */
    cacheAsBitmap: boolean;
    /**
     * The instance name of the object.
     *
     * @memberof PIXI.DisplayObject#
     * @member {string} name
     */
    name: string;

    /**
     * Returns the global position of the displayObject. Does not depend on object scale, rotation and pivot.
     *
     * @method getGlobalPosition
     * @memberof PIXI.DisplayObject#
     * @param {PIXI.Point} [point=new PIXI.Point()] - The point to write the global value to.
     * @param {boolean} [skipUpdate=false] - Setting to true will stop the transforms of the scene graph from
     *  being updated. This means the calculation returned MAY be out of date BUT will give you a
     *  nice performance boost.
     * @return {PIXI.Point} The updated point.
     */
    getGlobalPosition(point?: PIXI.Point, skipUpdate?: boolean): PIXI.Point;
  }

  /**
   * A Ticker class that runs an update loop that other objects listen to.
   *
   * This class is composed around listeners meant for execution on the next requested animation frame.
   * Animation frames are requested only when necessary, e.g. When the ticker is started and the emitter has listeners.
   *
   * @class
   * @memberof PIXI
   */
  class Ticker {
    constructor();

    /**
     * Whether or not this ticker should invoke the method
     * {@link PIXI.Ticker#start} automatically
     * when a listener is added.
     *
     * @member {boolean} PIXI.Ticker#autoStart
     * @default false
     */
    autoStart: boolean;
    /**
     * Scalar time value from last frame to this frame.
     * This value is capped by setting {@link PIXI.Ticker#minFPS}
     * and is scaled with {@link PIXI.Ticker#speed}.
     * **Note:** The cap may be exceeded by scaling.
     *
     * @member {number} PIXI.Ticker#deltaTime
     * @default 1
     */
    deltaTime: number;
    /**
     * Scaler time elapsed in milliseconds from last frame to this frame.
     * This value is capped by setting {@link PIXI.Ticker#minFPS}
     * and is scaled with {@link PIXI.Ticker#speed}.
     * **Note:** The cap may be exceeded by scaling.
     * If the platform supports DOMHighResTimeStamp,
     * this value will have a precision of 1 µs.
     * Defaults to target frame time
     *
     * @member {number} PIXI.Ticker#deltaMS
     * @default 16.66
     */
    deltaMS: number;
    /**
     * Time elapsed in milliseconds from last frame to this frame.
     * Opposed to what the scalar {@link PIXI.Ticker#deltaTime}
     * is based, this value is neither capped nor scaled.
     * If the platform supports DOMHighResTimeStamp,
     * this value will have a precision of 1 µs.
     * Defaults to target frame time
     *
     * @member {number} PIXI.Ticker#elapsedMS
     * @default 16.66
     */
    elapsedMS: number;
    /**
     * The last time {@link PIXI.Ticker#update} was invoked.
     * This value is also reset internally outside of invoking
     * update, but only when a new animation frame is requested.
     * If the platform supports DOMHighResTimeStamp,
     * this value will have a precision of 1 µs.
     *
     * @member {number} PIXI.Ticker#lastTime
     * @default -1
     */
    lastTime: number;
    /**
     * Factor of current {@link PIXI.Ticker#deltaTime}.
     * @example
     * // Scales ticker.deltaTime to what would be
     * // the equivalent of approximately 120 FPS
     * ticker.speed = 2;
     *
     * @member {number} PIXI.Ticker#speed
     * @default 1
     */
    speed: number;
    /**
     * Whether or not this ticker has been started.
     * `true` if {@link PIXI.Ticker#start} has been called.
     * `false` if {@link PIXI.Ticker#stop} has been called.
     * While `false`, this value may change to `true` in the
     * event of {@link PIXI.Ticker#autoStart} being `true`
     * and a listener is added.
     *
     * @member {boolean} PIXI.Ticker#started
     * @default false
     */
    started: boolean;

    /**
     * Register a handler for tick events. Calls continuously unless
     * it is removed or the ticker is stopped.
     *
     * @param {Function} fn - The listener function to be added for updates
     * @param {*} [context] - The listener context
     * @param {number} [priority=PIXI.UPDATE_PRIORITY.NORMAL] - The priority for emitting
     * @returns {PIXI.Ticker} This instance of a ticker
     */
    add(fn: (...params: any[]) => any, context?: any, priority?: number): PIXI.Ticker;

    /**
     * Add a handler for the tick event which is only execute once.
     *
     * @param {Function} fn - The listener function to be added for one update
     * @param {*} [context] - The listener context
     * @param {number} [priority=PIXI.UPDATE_PRIORITY.NORMAL] - The priority for emitting
     * @returns {PIXI.Ticker} This instance of a ticker
     */
    addOnce(fn: (...params: any[]) => any, context?: any, priority?: number): PIXI.Ticker;

    /**
     * Removes any handlers matching the function and context parameters.
     * If no handlers are left after removing, then it cancels the animation frame.
     *
     * @param {Function} fn - The listener function to be removed
     * @param {*} [context] - The listener context to be removed
     * @returns {PIXI.Ticker} This instance of a ticker
     */
    remove(fn: (...params: any[]) => any, context?: any): PIXI.Ticker;

    /**
     * Counts the number of listeners on this ticker.
     *
     * @returns {number} The number of listeners on this ticker
     */
    count: any;

    /**
     * Starts the ticker. If the ticker has listeners
     * a new animation frame is requested at this point.
     */
    start(): void;

    /**
     * Stops the ticker. If the ticker has requested
     * an animation frame it is canceled at this point.
     */
    stop(): void;

    /**
     * Destroy the ticker and don't use after this. Calling
     * this method removes all references to internal events.
     */
    destroy(): void;

    /**
     * Triggers an update. An update entails setting the
     * current {@link PIXI.Ticker#elapsedMS},
     * the current {@link PIXI.Ticker#deltaTime},
     * invoking all listeners with current deltaTime,
     * and then finally setting {@link PIXI.Ticker#lastTime}
     * with the value of currentTime that was provided.
     * This method will be called automatically by animation
     * frame callbacks if the ticker instance has been started
     * and listeners are added.
     *
     * @param {number} [currentTime=performance.now()] - the current time of execution
     */
    update(currentTime?: number): void;

    /**
     * The frames per second at which this ticker is running.
     * The default is approximately 60 in most modern browsers.
     * **Note:** This does not factor in the value of
     * {@link PIXI.Ticker#speed}, which is specific
     * to scaling {@link PIXI.Ticker#deltaTime}.
     *
     * @member {number}
     * @readonly
     */
    readonly FPS: number;
    /**
     * Manages the maximum amount of milliseconds allowed to
     * elapse between invoking {@link PIXI.Ticker#update}.
     * This value is used to cap {@link PIXI.Ticker#deltaTime},
     * but does not effect the measured value of {@link PIXI.Ticker#FPS}.
     * When setting this property it is clamped to a value between
     * `0` and `PIXI.settings.TARGET_FPMS * 1000`.
     *
     * @member {number}
     * @default 10
     */
    minFPS: number;
    /**
     * Manages the minimum amount of milliseconds required to
     * elapse between invoking {@link PIXI.Ticker#update}.
     * This will effect the measured value of {@link PIXI.Ticker#FPS}.
     * If it is set to `0`, then there is no limit; PixiJS will render as many frames as it can.
     * Otherwise it will be at least `minFPS`
     *
     * @member {number}
     * @default 0
     */
    maxFPS: number;
    /**
     * The shared ticker instance used by {@link PIXI.AnimatedSprite} and by
     * {@link PIXI.VideoResource} to update animation frames / video textures.
     *
     * It may also be used by {@link PIXI.Application} if created with the `sharedTicker` option property set to true.
     *
     * The property {@link PIXI.Ticker#autoStart} is set to `true` for this instance.
     * Please follow the examples for usage, including how to opt-out of auto-starting the shared ticker.
     *
     * @example
     * let ticker = PIXI.Ticker.shared;
     * // Set this to prevent starting this ticker when listeners are added.
     * // By default this is true only for the PIXI.Ticker.shared instance.
     * ticker.autoStart = false;
     * // FYI, call this to ensure the ticker is stopped. It should be stopped
     * // if you have not attempted to render anything yet.
     * ticker.stop();
     * // Call this when you are ready for a running shared ticker.
     * ticker.start();
     *
     * @example
     * // You may use the shared ticker to render...
     * let renderer = PIXI.autoDetectRenderer();
     * let stage = new PIXI.Container();
     * document.body.appendChild(renderer.view);
     * ticker.add(function (time) {
     *     renderer.render(stage);
     * });
     *
     * @example
     * // Or you can just update it manually.
     * ticker.autoStart = false;
     * ticker.stop();
     * function animate(time) {
     *     ticker.update(time);
     *     renderer.render(stage);
     *     requestAnimationFrame(animate);
     * }
     * animate(performance.now());
     *
     * @member {PIXI.Ticker}
     * @static
     */
    static shared: PIXI.Ticker;
    /**
     * The system ticker instance used by {@link PIXI.interaction.InteractionManager} and by
     * {@link PIXI.BasePrepare} for core timing functionality that shouldn't usually need to be paused,
     * unlike the `shared` ticker which drives visual animations and rendering which may want to be paused.
     *
     * The property {@link PIXI.Ticker#autoStart} is set to `true` for this instance.
     *
     * @member {PIXI.Ticker}
     * @static
     */
    static system: PIXI.Ticker;
  }

  /**
   * Middleware for for Application Ticker.
   *
   * @example
   * import {TickerPlugin} from '@pixi/ticker';
   * import {Application} from '@pixi/app';
   * Application.registerPlugin(TickerPlugin);
   *
   * @class
   * @memberof PIXI
   */
  class TickerPlugin {}

  /**
   * Represents the update priorities used by internal PIXI classes when registered with
   * the {@link PIXI.Ticker} object. Higher priority items are updated first and lower
   * priority items, such as render, should go later.
   *
   * @static
   * @constant
   * @name UPDATE_PRIORITY
   * @memberof PIXI
   * @enum {number}
   * @property {number} INTERACTION=50 Highest priority, used for {@link PIXI.interaction.InteractionManager}
   * @property {number} HIGH=25 High priority updating, {@link PIXI.VideoBaseTexture} and {@link PIXI.AnimatedSprite}
   * @property {number} NORMAL=0 Default priority for ticker events, see {@link PIXI.Ticker#add}.
   * @property {number} LOW=-25 Low priority used for {@link PIXI.Application} rendering.
   * @property {number} UTILITY=-50 Lowest priority used for {@link PIXI.prepare.BasePrepare} utility.
   */
  enum UPDATE_PRIORITY {
    INTERACTION,
    HIGH,
    NORMAL,
    LOW,
    UTILITY,
  }

  /**
   * Regexp for data URI.
   * Based on: {@link https://github.com/ragingwind/data-uri-regex}
   *
   * @static
   * @constant {RegExp|string} DATA_URI
   * @memberof PIXI
   * @example data:image/png;base64
   */
  var DATA_URI: RegExp | string;
  /**
   * Generalized convenience utilities for PIXI.
   * @example
   * // Extend PIXI's internal Event Emitter.
   * class MyEmitter extends PIXI.utils.EventEmitter {
   *   constructor() {
   *      super();
   *      console.log("Emitter created!");
   *   }
   * }
   *
   * // Get info on current device
   * console.log(PIXI.utils.isMobile);
   *
   * // Convert hex color to string
   * console.log(PIXI.utils.hex2string(0xff00ff)); // returns: "#ff00ff"
   * @namespace PIXI.utils
   */
  namespace utils {
    /**
     * Skips the hello message of renderers that are created after this is run.
     *
     * @function skipHello
     * @memberof PIXI.utils
     */
    function skipHello(): void;

    /**
     * Logs out the version and renderer information for this running instance of PIXI.
     * If you don't want to see this message you can run `PIXI.utils.skipHello()` before
     * creating your renderer. Keep in mind that doing that will forever make you a jerk face.
     *
     * @static
     * @function sayHello
     * @memberof PIXI.utils
     * @param {string} type - The string renderer type to log.
     */
    function sayHello(type: string): void;

    /**
     * Helper for checking for WebGL support.
     *
     * @memberof PIXI.utils
     * @function isWebGLSupported
     * @return {boolean} Is WebGL supported.
     */
    function isWebGLSupported(): boolean;

    /**
     * Converts a hexadecimal color number to an [R, G, B] array of normalized floats (numbers from 0.0 to 1.0).
     *
     * @example
     * PIXI.utils.hex2rgb(0xffffff); // returns [1, 1, 1]
     * @memberof PIXI.utils
     * @function hex2rgb
     * @param {number} hex - The hexadecimal number to convert
     * @param  {number[]} [out=[]] If supplied, this array will be used rather than returning a new one
     * @return {number[]} An array representing the [R, G, B] of the color where all values are floats.
     */
    function hex2rgb(hex: number, out?: number[]): number[];

    /**
     * Converts a hexadecimal color number to a string.
     *
     * @example
     * PIXI.utils.hex2string(0xffffff); // returns "#ffffff"
     * @memberof PIXI.utils
     * @function hex2string
     * @param {number} hex - Number in hex (e.g., `0xffffff`)
     * @return {string} The string color (e.g., `"#ffffff"`).
     */
    function hex2string(hex: number): string;

    /**
     * Converts a hexadecimal string to a hexadecimal color number.
     *
     * @example
     * PIXI.utils.string2hex("#ffffff"); // returns 0xffffff
     * @memberof PIXI.utils
     * @function string2hex
     * @param {string} The string color (e.g., `"#ffffff"`)
     * @return {number} Number in hexadecimal.
     */
    function string2hex(The: string): number;

    /**
     * Converts a color as an [R, G, B] array of normalized floats to a hexadecimal number.
     *
     * @example
     * PIXI.utils.rgb2hex([1, 1, 1]); // returns 0xffffff
     * @memberof PIXI.utils
     * @function rgb2hex
     * @param {number[]} rgb - Array of numbers where all values are normalized floats from 0.0 to 1.0.
     * @return {number} Number in hexadecimal.
     */
    function rgb2hex(rgb: number[]): number;

    /**
     * maps premultiply flag and blendMode to adjusted blendMode
     * @memberof PIXI.utils
     * @const premultiplyBlendMode
     * @type {Array<number[]>}
     */
    var premultiplyBlendMode: number[][];

    /**
     * changes blendMode according to texture format
     *
     * @memberof PIXI.utils
     * @function correctBlendMode
     * @param {number} blendMode supposed blend mode
     * @param {boolean} premultiplied  whether source is premultiplied
     * @returns {number} true blend mode for this texture
     */
    function correctBlendMode(blendMode: number, premultiplied: boolean): number;

    /**
     * combines rgb and alpha to out array
     *
     * @memberof PIXI.utils
     * @function premultiplyRgba
     * @param {Float32Array|number[]} rgb input rgb
     * @param {number} alpha alpha param
     * @param {Float32Array} [out] output
     * @param {boolean} [premultiply=true] do premultiply it
     * @returns {Float32Array} vec4 rgba
     */
    function premultiplyRgba(
      rgb: Float32Array | number[],
      alpha: number,
      out?: Float32Array,
      premultiply?: boolean,
    ): Float32Array;

    /**
     * premultiplies tint
     *
     * @memberof PIXI.utils
     * @function premultiplyTint
     * @param {number} tint integer RGB
     * @param {number} alpha floating point alpha (0.0-1.0)
     * @returns {number} tint multiplied by alpha
     */
    function premultiplyTint(tint: number, alpha: number): number;

    /**
     * converts integer tint and float alpha to vec4 form, premultiplies by default
     *
     * @memberof PIXI.utils
     * @function premultiplyTintToRgba
     * @param {number} tint input tint
     * @param {number} alpha alpha param
     * @param {Float32Array} [out] output
     * @param {boolean} [premultiply=true] do premultiply it
     * @returns {Float32Array} vec4 rgba
     */
    function premultiplyTintToRgba(
      tint: number,
      alpha: number,
      out?: Float32Array,
      premultiply?: boolean,
    ): Float32Array;

    /**
     * Generic Mask Stack data structure
     *
     * @memberof PIXI.utils
     * @function createIndicesForQuads
     * @param {number} size - Number of quads
     * @param {Uint16Array|Uint32Array} [outBuffer] - Buffer for output, length has to be `6 * size`
     * @return {Uint16Array|Uint32Array} - Resulting index buffer
     */
    function createIndicesForQuads(size: number, outBuffer?: Uint16Array | Uint32Array): Uint16Array | Uint32Array;

    /**
     * Rounds to next power of two.
     *
     * @function nextPow2
     * @memberof PIXI.utils
     * @param {number} v input value
     * @return {number}
     */
    function nextPow2(v: number): number;

    /**
     * Checks if a number is a power of two.
     *
     * @function isPow2
     * @memberof PIXI.utils
     * @param {number} v input value
     * @return {boolean} `true` if value is power of two
     */
    function isPow2(v: number): boolean;

    /**
     * Computes ceil of log base 2
     *
     * @function log2
     * @memberof PIXI.utils
     * @param {number} v input value
     * @return {number} logarithm base 2
     */
    function log2(v: number): number;

    /**
     * Remove items from a javascript array without generating garbage
     *
     * @function removeItems
     * @memberof PIXI.utils
     * @param {Array<any>} arr Array to remove elements from
     * @param {number} startIdx starting index
     * @param {number} removeCount how many to remove
     */
    function removeItems(arr: any[], startIdx: number, removeCount: number): void;

    /**
     * Returns sign of number
     *
     * @memberof PIXI.utils
     * @function sign
     * @param {number} n - the number to check the sign of
     * @returns {number} 0 if `n` is 0, -1 if `n` is negative, 1 if `n` is positive
     */
    function sign(n: number): number;

    /**
     * Gets the next unique identifier
     *
     * @memberof PIXI.utils
     * @function uid
     * @return {number} The next unique identifier to use.
     */
    function uid(): number;

    /**
     * A simple JS library that detects mobile devices.
     *
     * @see {@link https://github.com/kaimallea/isMobile}
     *
     * @memberof PIXI.utils
     * @name isMobile
     * @type {Object}
     * @property {boolean} any - `true` if current platform is tablet or phone device
     * @property {boolean} tablet - `true` if current platform large-screen tablet device
     * @property {boolean} phone - `true` if current platform small-screen phone device
     * @property {object} apple
     * @property {boolean} apple.device - `true` if any Apple device
     * @property {boolean} apple.tablet - `true` if any Apple iPad
     * @property {boolean} apple.phone - `true` if any Apple iPhone
     * @property {boolean} apple.ipod - `true` if any iPod
     * @property {object} android
     * @property {boolean} android.device - `true` if any Android device
     * @property {boolean} android.tablet - `true` if any Android tablet
     * @property {boolean} android.phone - `true` if any Android phone
     * @property {object} amazon
     * @property {boolean} amazon.device - `true` if any Silk device
     * @property {boolean} amazon.tablet - `true` if any Silk tablet
     * @property {boolean} amazon.phone - `true` if any Silk phone
     * @property {object} windows
     * @property {boolean} windows.device - `true` if any Windows device
     * @property {boolean} windows.tablet - `true` if any Windows tablet
     * @property {boolean} windows.phone - `true` if any Windows phone
     */
    var isMobile: {
      any: boolean;
      tablet: boolean;
      phone: boolean;
      apple: {
        device: boolean;
        tablet: boolean;
        phone: boolean;
        ipod: boolean;
      };
      android: {
        device: boolean;
        tablet: boolean;
        phone: boolean;
      };
      amazon: {
        device: boolean;
        tablet: boolean;
        phone: boolean;
      };
      windows: {
        device: boolean;
        tablet: boolean;
        phone: boolean;
      };
    };

    /**
     * A high performance event emitter
     *
     * @see {@link https://github.com/primus/eventemitter3}
     *
     * @memberof PIXI.utils
     * @class EventEmitter
     * @type {EventEmitter}
     */
    class EventEmitter {}

    /**
     * A polygon triangulation library
     *
     * @see {@link https://github.com/mapbox/earcut}
     *
     * @memberof PIXI.utils
     * @function earcut
     * @param {number[]} vertices - A flat array of vertex coordinates
     * @param {number[]} [holes] - An array of hole indices
     * @param {number} [dimensions=2] The number of coordinates per vertex in the input array
     * @return {number[]} Triangulated polygon
     */
    function earcut(vertices: number[], holes?: number[], dimensions?: number): number[];

    /**
     * Node.js compatible URL utilities.
     *
     * @see https://www.npmjs.com/package/url
     *
     * @memberof PIXI.utils
     * @name url
     * @type {object}
     */
    var url: any;

    /**
     * Helper for warning developers about deprecated features & settings.
     * A stack track for warnings is given; useful for tracking-down where
     * deprecated methods/properties/classes are being used within the code.
     *
     * @memberof PIXI.utils
     * @function deprecation
     * @param {string} version - The version where the feature became deprecated
     * @param {string} message - Message should include what is deprecated, where, and the new solution
     * @param {number} [ignoreDepth=3] - The number of steps to ignore at the top of the error stack
     *        this is mostly to ignore internal deprecation calls.
     */
    function deprecation(version: string, message: string, ignoreDepth?: number): void;

    /**
     * Creates a Canvas element of the given size to be used as a target for rendering to.
     *
     * @class
     * @memberof PIXI.utils
     */
    class CanvasRenderTarget {
      constructor(width: number, height: number, resolution?: number);

      /**
       * The Canvas object that belongs to this CanvasRenderTarget.
       *
       * @member {HTMLCanvasElement} PIXI.utils.CanvasRenderTarget#canvas
       */
      canvas: HTMLCanvasElement;
      /**
       * A CanvasRenderingContext2D object representing a two-dimensional rendering context.
       *
       * @member {CanvasRenderingContext2D} PIXI.utils.CanvasRenderTarget#context
       */
      context: CanvasRenderingContext2D;

      /**
       * Resizes the canvas to the specified width and height.
       *
       * @param {number} width - the new width of the canvas
       * @param {number} height - the new height of the canvas
       */
      resize(width: number, height: number): void;

      /**
       * Destroys this canvas.
       *
       */
      destroy(): void;

      /**
       * The width of the canvas buffer in pixels.
       *
       * @member {number}
       */
      width: number;
      /**
       * The height of the canvas buffer in pixels.
       *
       * @member {number}
       */
      height: number;
    }

    /**
     * @todo Describe property usage
     *
     * @static
     * @name ProgramCache
     * @memberof PIXI.utils
     * @type {Object}
     */
    var ProgramCache: any;
    /**
     * @todo Describe property usage
     *
     * @static
     * @name TextureCache
     * @memberof PIXI.utils
     * @type {Object}
     */
    var TextureCache: any;
    /**
     * @todo Describe property usage
     *
     * @static
     * @name BaseTextureCache
     * @memberof PIXI.utils
     * @type {Object}
     */
    var BaseTextureCache: any;

    /**
     * Destroys all texture in the cache
     *
     * @memberof PIXI.utils
     * @function destroyTextureCache
     */
    function destroyTextureCache(): void;

    /**
     * Removes all textures from cache, but does not destroy them
     *
     * @memberof PIXI.utils
     * @function clearTextureCache
     */
    function clearTextureCache(): void;

    /**
     * Trim transparent borders from a canvas
     *
     * @memberof PIXI.utils
     * @function trimCanvas
     * @param {HTMLCanvasElement} canvas - the canvas to trim
     * @returns {object} Trim data
     */
    function trimCanvas(canvas: HTMLCanvasElement): any;

    /**
     * @memberof PIXI.utils
     * @interface DecomposedDataUri
     */
    interface DecomposedDataUri {
      /**
       * type, eg. `image`
       * @memberof PIXI.utils.DecomposedDataUri#
       * @member {string} mediaType
       */
      mediaType: string;
      /**
       * Sub type, eg. `png`
       * @memberof PIXI.utils.DecomposedDataUri#
       * @member {string} subType
       */
      subType: string;
      /**
       * @memberof PIXI.utils.DecomposedDataUri#
       * @member {string} charset
       */
      charset: string;
      /**
       * Data encoding, eg. `base64`
       * @memberof PIXI.utils.DecomposedDataUri#
       * @member {string} encoding
       */
      encoding: string;
      /**
       * The actual data
       * @memberof PIXI.utils.DecomposedDataUri#
       * @member {string} data
       */
      data: string;
    }

    /**
     * Split a data URI into components. Returns undefined if
     * parameter `dataUri` is not a valid data URI.
     *
     * @memberof PIXI.utils
     * @function decomposeDataUri
     * @param {string} dataUri - the data URI to check
     * @return {PIXI.utils.DecomposedDataUri|undefined} The decomposed data uri or undefined
     */
    function decomposeDataUri(dataUri: string): PIXI.utils.DecomposedDataUri | undefined;

    /**
     * get the resolution / device pixel ratio of an asset by looking for the prefix
     * used by spritesheets and image urls
     *
     * @memberof PIXI.utils
     * @function getResolutionOfUrl
     * @param {string} url - the image path
     * @param {number} [defaultValue=1] - the defaultValue if no filename prefix is set.
     * @return {number} resolution / device pixel ratio of an asset
     */
    function getResolutionOfUrl(url: string, defaultValue?: number): number;
  }
}

/**
 * @interface SharedArrayBuffer
 */
declare interface SharedArrayBuffer {}

/**
 * @interface OffscreenCanvas
 */
declare interface OffscreenCanvas {}

declare namespace PIXI {
  namespace utils {
    // https://github.com/primus/eventemitter3
    export interface EventEmitter {
      /**
       * Return an array listing the events for which the emitter has registered listeners.
       *
       * @returns {(string | symbol)[]}
       */
      eventNames(): Array<string | symbol>;

      /**
       * Return the listeners registered for a given event.
       *
       * @param {(string | symbol)} event The event name.
       * @returns {Function[]}
       */
      //tslint:disable-next-line:ban-types forbidden-types
      listeners(event: string | symbol): Array<Function>;

      /**
       * Return the number of listeners listening to a given event.
       *
       * @param {(string | symbol)} event The event name.
       * @returns {number}
       */
      listenerCount(event: string | symbol): number;

      /**
       * Calls each of the listeners registered for a given event.
       *
       * @param {(string | symbol)} event The event name.
       * @param {...*} args Arguments that are passed to registered listeners
       * @returns {boolean} `true` if the event had listeners, else `false`.
       */
      emit(event: string | symbol, ...args: any[]): boolean;

      /**
       * Add a listener for a given event.
       *
       * @param {(string | symbol)} event The event name.
       * @param {Function} fn The listener function.
       * @param {*} [context=this] The context to invoke the listener with.
       * @returns {EventEmitter} `this`.
       */
      //tslint:disable-next-line:ban-types forbidden-types
      on(event: string | symbol, fn: Function, context?: any): this;

      /**
       * Add a one-time listener for a given event.
       *
       * @param {(string | symbol)} event The event name.
       * @param {Function} fn The listener function.
       * @param {*} [context=this] The context to invoke the listener with.
       * @returns {EventEmitter} `this`.
       */
      //tslint:disable-next-line:ban-types forbidden-types
      once(event: string | symbol, fn: Function, context?: any): this;

      /**
       * Remove the listeners of a given event.
       *
       * @param {(string | symbol)} event The event name.
       * @param {Function} fn Only remove the listeners that match this function.
       * @param {*} context Only remove the listeners that have this context.
       * @param {boolean} once Only remove one-time listeners.
       * @returns {EventEmitter} `this`.
       */
      //tslint:disable-next-line:ban-types forbidden-types
      removeListener(event: string | symbol, fn?: Function, context?: any, once?: boolean): this;

      /**
       * Remove all listeners, or those of the specified event.
       *
       * @param {(string | symbol)} event The event name.
       * @returns {EventEmitter} `this`.
       */
      removeAllListeners(event?: string | symbol): this;

      /**
       * Alias method for `removeListener`
       */
      //tslint:disable-next-line:ban-types forbidden-types
      off(event: string | symbol, fn?: Function, context?: any, once?: boolean): this;

      /**
       * Alias method for `on`
       */
      //tslint:disable-next-line:ban-types forbidden-types
      addListener(event: string | symbol, fn: Function, context?: any): this;
    }
  }

  namespace interaction {
    type InteractionPointerEvents =
      | 'pointerdown'
      | 'pointercancel'
      | 'pointerup'
      | 'pointertap'
      | 'pointerupoutside'
      | 'pointermove'
      | 'pointerover'
      | 'pointerout';
    type InteractionTouchEvents = 'touchstart' | 'touchcancel' | 'touchend' | 'touchendoutside' | 'touchmove' | 'tap';
    type InteractionMouseEvents =
      | 'rightdown'
      | 'mousedown'
      | 'rightup'
      | 'mouseup'
      | 'rightclick'
      | 'click'
      | 'rightupoutside'
      | 'mouseupoutside'
      | 'mousemove'
      | 'mouseover'
      | 'mouseout';
    type InteractionPixiEvents = 'added' | 'removed';
    type InteractionEventTypes =
      | InteractionPointerEvents
      | InteractionTouchEvents
      | InteractionMouseEvents
      | InteractionPixiEvents;
  }

  export interface DisplayObject {
    on(
      event: interaction.InteractionEventTypes,
      fn: (event: interaction.InteractionEvent) => void,
      context?: any,
    ): this;

    //tslint:disable-next-line:ban-types forbidden-types
    on(event: string | symbol, fn: Function, context?: any): this;

    once(
      event: interaction.InteractionEventTypes,
      fn: (event: interaction.InteractionEvent) => void,
      context?: any,
    ): this;

    //tslint:disable-next-line:ban-types forbidden-types
    once(event: string | symbol, fn: Function, context?: any): this;

    removeListener(
      event: interaction.InteractionEventTypes,
      fn?: (event: interaction.InteractionEvent) => void,
      context?: any,
    ): this;

    //tslint:disable-next-line:ban-types forbidden-types
    removeListener(event: string | symbol, fn?: Function, context?: any): this;

    removeAllListeners(event?: interaction.InteractionEventTypes): this;

    removeAllListeners(event?: string | symbol): this;

    off(
      event: interaction.InteractionEventTypes,
      fn?: (event: interaction.InteractionEvent) => void,
      context?: any,
    ): this;

    //tslint:disable-next-line:ban-types forbidden-types
    off(event: string | symbol, fn?: Function, context?: any): this;

    addListener(
      event: interaction.InteractionEventTypes,
      fn: (event: interaction.InteractionEvent) => void,
      context?: any,
    ): this;

    //tslint:disable-next-line:ban-types forbidden-types
    addListener(event: string | symbol, fn: Function, context?: any): this;
  }

  export interface Container {
    once(event: 'added' | 'removed', fn: (displayObject: DisplayObject) => void, context?: any): this;

    //tslint:disable-next-line:ban-types forbidden-types
    once(event: string, fn: Function, context?: any): this;

    on(event: 'added' | 'removed', fn: (displayObject: DisplayObject) => void, context?: any): this;

    //tslint:disable-next-line:ban-types forbidden-types
    on(event: string, fn: Function, context?: any): this;

    //tslint:disable-next-line:ban-types forbidden-types
    off(event: 'added' | 'removed' | string, fn?: Function, context?: any): this;
  }
}

declare namespace PIXI {
  export interface Loader extends utils.EventEmitter {
    baseUrl: string;
    progress: number;
    loading: boolean;
    defaultQueryString: string;
    resources: IResourceDictionary;
    concurrency: number;

    add(...params: any[]): this;

    //tslint:disable-next-line:ban-types forbidden-types
    add(name: string, url: string, options?: ILoaderOptions, cb?: Function): this;

    //tslint:disable-next-line:ban-types forbidden-types
    add(obj: string | any | any[], options?: ILoaderOptions, cb?: Function): this;

    //tslint:disable-next-line:ban-types forbidden-types
    pre(fn: Function): this;

    //tslint:disable-next-line:ban-types forbidden-types
    use(fn: Function): this;

    reset(): this;

    //tslint:disable-next-line:ban-types forbidden-types
    load(cb?: (loader: Loader, resources: Partial<Record<string, LoaderResource>>) => void): this;

    destroy(): void;

    // depreciation

    on(event: 'complete', fn: (loader: Loader, object: any) => void, context?: any): this;

    on(event: 'error', fn: (error: Error, loader: Loader, resource: LoaderResource) => void, context?: any): this;

    on(event: 'load' | 'progress', fn: (loader: Loader, resource: LoaderResource) => void, context?: any): this;

    on(event: 'start', fn: (loader: Loader) => void, context?: any): this;

    once(event: 'complete', fn: (loader: Loader, object: any) => void, context?: any): this;

    once(event: 'error', fn: (error: Error, loader: Loader, resource: LoaderResource) => void, context?: any): this;

    once(event: 'load' | 'progress', fn: (loader: Loader, resource: LoaderResource) => void, context?: any): this;

    once(event: 'start', fn: (loader: Loader) => void, context?: any): this;

    //tslint:disable-next-line:ban-types forbidden-types
    off(event: 'complete' | 'error' | 'load' | 'progress' | 'start' | string, fn?: Function, context?: any): this;
  }

  export interface IResourceDictionary {
    [index: string]: LoaderResource;
  }

  export interface ITextureDictionary {
    [index: string]: Texture;
  }

  export interface ILoaderOptions {
    crossOrigin?: boolean | string;
    loadType?: number;
    xhrType?: string;
    metadata?: {
      loadElement?: HTMLImageElement | HTMLAudioElement | HTMLVideoElement;
      skipSource?: boolean;
      mimeType?: string | string[];
    };
  }

  export interface LoaderResource {
    name: string;
    url: string;
    extension: string;
    data: any;
    crossOrigin: boolean | string;
    loadType: number;
    xhrType: string;
    metadata: any;
    error: Error;
    xhr: XMLHttpRequest | null;
    children: LoaderResource[];
    type: number;
    progressChunk: number;
    isDataUrl: boolean;
    isComplete: boolean;
    isLoading: boolean;

    complete(): void;

    abort(message?: string): void;

    //tslint:disable-next-line:ban-types forbidden-types
    load(cb?: Function): void;

    texture: Texture;
    spineAtlas: any;
    spineData: any;
    spritesheet?: Spritesheet;
    textures?: ITextureDictionary;
  }

  namespace LoaderResource {
    function setExtensionLoadType(extname: string, loadType: number): void;

    function setExtensionXhrType(extname: string, xhrType: string): void;

    export enum STATUS_FLAGS {
      NONE = 0,
      DATA_URL = 1 << 0,
      COMPLETE = 1 << 1,
      LOADING = 1 << 2,
    }

    export enum TYPE {
      UNKNOWN = 0,
      JSON = 1,
      XML = 2,
      IMAGE = 3,
      AUDIO = 4,
      VIDEO = 5,
      TEXT = 6,
    }

    export enum LOAD_TYPE {
      /** Uses XMLHttpRequest to load the resource. */
      XHR = 1,
      /** Uses an `Image` object to load the resource. */
      IMAGE = 2,
      /** Uses an `Audio` object to load the resource. */
      AUDIO = 3,
      /** Uses a `Video` object to load the resource. */
      VIDEO = 4,
    }

    export enum XHR_RESPONSE_TYPE {
      /** string */
      DEFAULT = 'text',
      /** ArrayBuffer */
      BUFFER = 'arraybuffer',
      /** Blob */
      BLOB = 'blob',
      /** Document */
      DOCUMENT = 'document',
      /** Object */
      JSON = 'json',
      /** String */
      TEXT = 'text',
    }

    let EMPTY_GIF: string;
  }
}

declare module 'pixi.js' {
  export = PIXI;
} /* tslint:disable */ // Generated by Haxe TypeScript Declaration Generator :)

/**
 * Allows you to map pixels to the screen or draw rectangles.
 */
declare class Bitmap {
  /**
   * The basic object that represents an image.
   *
   * @class Bitmap
   * @constructor
   * @param {Int} width The width of the bitmap
   * @param {Int} height The height of the bitmap
   */
  constructor(width?: number, height?: number);

  /**
   * The face name of the font.
   *
   * @property fontFace
   * @type String
   */
  fontFace: string;
  /**
   * The size of the font in pixels.
   *
   * @property fontSize
   * @type Number
   */
  fontSize: number;
  /**
   * Whether the font is italic.
   *
   * @property fontItalic
   * @type Boolean
   */
  fontItalic: number;
  /**
   * The color of the text in CSS format.
   *
   * @property textColor
   * @type String
   */
  textColor: string;
  /**
   * The color of the outline of the text in CSS format.
   *
   * @property outlineColor
   * @type String
   */
  outlineColor: string;
  /**
   * The width of the outline of the text.
   *
   * @property outlineWidth
   * @type Number
   */
  outlineWidth: number;
  /**
   * [read-only] The url of the image file.
   *
   * @property url
   * @type String
   */
  url: string;
  /**
   * [read-only] The base texture that holds the image.
   *
   * @property baseTexture
   * @type PIXI.BaseTexture
   */
  baseTexture: PIXI.BaseTexture;
  /**
   * [read-only] The bitmap canvas.
   *
   * @property canvas
   * @type HTMLCanvasElement
   */
  canvas: HTMLCanvasElement;
  /**
   * [read-only] The 2d context of the bitmap canvas.
   *
   * @property context
   * @type CanvasRenderingContext2D
   */
  context: CanvasRenderingContext2D;
  /**
   * [read-only] The width of the bitmap.
   *
   * @property width
   * @type Number
   */
  width: number;
  /**
   * [read-only] The height of the bitmap.
   *
   * @property height
   * @type Number
   */
  height: number;
  /**
   * [read-only] The rectangle of the bitmap.
   *
   * @property rect
   * @type Rectangle
   */
  rect: Rectangle;
  /**
   * Whether the smooth scaling is applied.
   *
   * @property smooth
   * @type Boolean
   */
  smooth: boolean;
  /**
   * The opacity of the drawing object in the range (0, 255).
   *
   * @property paintOpacity
   * @type Number
   */
  paintOpacity: number;
  /**
   * Cache entry, for images. In all cases _url is the same as cacheEntry.key
   * @type CacheEntry
   */
  cacheEntry: CacheEntry;

  /**
   * Checks whether the bitmap is ready to render.
   *
   * @method isReady
   * @return {Boolean} True if the bitmap is ready to render
   */
  isReady(): boolean;

  /**
   * Checks whether a loading error has occurred.
   *
   * @method isError
   * @return {Boolean} True if a loading error has occurred
   */
  isError(): boolean;

  /**
   * Resizes the bitmap.
   *
   * @method resize
   * @param {Number} width The new width of the bitmap
   * @param {Number} height The new height of the bitmap
   */
  resize(width: number, height: number): void;

  /**
   * Performs a block transfer.
   *
   * @method blt
   * @param {Bitmap} source The bitmap to draw
   * @param {Number} sx The x coordinate in the source
   * @param {Number} sy The y coordinate in the source
   * @param {Number} sw The width of the source image
   * @param {Number} sh The height of the source image
   * @param {Number} dx The x coordinate in the destination
   * @param {Number} dy The y coordinate in the destination
   * @param {Number} [dw=sw] The width to draw the image in the destination
   * @param {Number} [dh=sh] The height to draw the image in the destination
   */
  blt(
    source: Bitmap,
    sx: number,
    sy: number,
    sw: number,
    sh: number,
    dx: number,
    dy: number,
    dw?: number,
    dh?: number,
  ): void;

  /**
   * Returns pixel color at the specified point.
   *
   * @method getPixel
   * @param {Number} x The x coordinate of the pixel in the bitmap
   * @param {Number} y The y coordinate of the pixel in the bitmap
   * @return {String} The pixel color (hex format)
   */
  getPixel(x: number, y: number): string;

  /**
   * Returns alpha pixel value at the specified point.
   *
   * @method getAlphaPixel
   * @param {Number} x The x coordinate of the pixel in the bitmap
   * @param {Number} y The y coordinate of the pixel in the bitmap
   * @return {String} The alpha value
   */
  getAlphaPixel(x: number, y: number): string;

  /**
   * Clears the specified rectangle.
   *
   * @method clearRect
   * @param {Number} x The x coordinate for the upper-left corner
   * @param {Number} y The y coordinate for the upper-left corner
   * @param {Number} width The width of the rectangle to clear
   * @param {Number} height The height of the rectangle to clear
   */
  clearRect(x: number, y: number, width: number, height: number): void;

  /**
   * Clears the entire bitmap.
   *
   * @method clear
   */
  clear(): void;

  /**
   * Fills the specified rectangle.
   *
   * @method fillRect
   * @param {Number} x The x coordinate for the upper-left corner
   * @param {Number} y The y coordinate for the upper-left corner
   * @param {Number} width The width of the rectangle to clear
   * @param {Number} height The height of the rectangle to clear
   * @param {String} color The color of the rectangle in CSS format
   */
  fillRect(x: number, y: number, width: number, height: number, color: string): void;

  /**
   * Fills the entire bitmap.
   *
   * @method fillAll
   * @param {String} color The color of the rectangle in CSS format
   */
  fillAll(color: string): void;

  /**
   * Draws the rectangle with a gradation.
   *
   * @method gradientFillRect
   * @param {Number} x The x coordinate for the upper-left corner
   * @param {Number} y The y coordinate for the upper-left corner
   * @param {Number} width The width of the rectangle to clear
   * @param {Number} height The height of the rectangle to clear
   * @param {String} color1 The start color of the gradation
   * @param {String} color2 The end color of the gradation
   * @param {Boolean} vertical Whether it draws a vertical gradient
   */
  gradientFillRect(
    x: number,
    y: number,
    width: number,
    height: number,
    color1: string,
    color2: string,
    vertical?: boolean,
  ): void;

  /**
   * Draw the filled circle.
   *
   * @method drawCircle
   * @param {Number} x The x coordinate of the center of the circle
   * @param {Number} y The y coordinate of the center of the circle
   * @param {Number} radius The radius of the circle
   * @param {String} color The color of the circle in CSS format
   */
  drawCircle(x: number, y: number, radius: number, color: string): void;

  /**
   * Draws the outline text to the bitmap.
   *
   * @method drawText
   * @param {String} text The text that will be drawn
   * @param {Number} x The x coordinate for the left of the text
   * @param {Number} y The y coordinate for the top of the text
   * @param {Number} maxWidth The maximum allowed width of the text
   * @param {Number} lineHeight The height of the text line
   * @param {String} align The alignment of the text
   */
  drawText(text: string, x: number, y: number, maxWidth: number, lineHeight: number, align?: string): void;

  /**
   * Returns the width of the specified text.
   *
   * @method measureTextWidth
   * @param {String} text The text to be measured
   * @return {Number} The width of the text in pixels
   */
  measureTextWidth(text: string): number;

  /**
   * Changes the color tone of the entire bitmap.
   *
   * @method adjustTone
   * @param {Number} r The red strength in the range (-255, 255)
   * @param {Number} g The green strength in the range (-255, 255)
   * @param {Number} b The blue strength in the range (-255, 255)
   */
  adjustTone(r: number, g: number, b: number): void;

  /**
   * Rotates the hue of the entire bitmap.
   *
   * @method rotateHue
   * @param {Number} offset The hue offset in 360 degrees
   */
  rotateHue(offset: number): void;

  /**
   * Applies a blur effect to the bitmap.
   *
   * @method blur
   */
  blur(): void;

  /**
   * Add a callback function that will be called when the bitmap is loaded.
   *
   * @method addLoadListener
   * @param {Function} listner The callback function
   */
  addLoadListener(listner: (bitmap: Bitmap) => void): void;

  /**
   * touch the resource
   * @method touch
   */
  touch(): void;

  /**
   * Performs a block transfer, using assumption that original image was not modified (no hue)
   *
   * @method blt
   * @param {Bitmap} source The bitmap to draw
   * @param {Int} sx The x coordinate in the source
   * @param {Int} sy The y coordinate in the source
   * @param {Int} sw The width of the source image
   * @param {Int} sh The height of the source image
   * @param {Int} dx The x coordinate in the destination
   * @param {Int} dy The y coordinate in the destination
   * @param {Int} [dw=sw] The width to draw the image in the destination
   * @param {Int} [dh=sh] The height to draw the image in the destination
   */
  bltImage(
    source: Bitmap,
    sx: number,
    sy: number,
    sw: number,
    sh: number,
    dx: number,
    dy: number,
    dw?: number,
    dh?: number,
  ): void;

  /**
   * Loads a image file and returns a new bitmap object.
   *
   * @static
   * @method load
   * @param {String} url The image url of the texture
   * @return Bitmap
   */
  static load(url: string): Bitmap;

  /**
   * Takes a snapshot of the game screen and returns a new bitmap object.
   *
   * @static
   * @method snap
   * @param {Stage} stage The stage object
   * @return Bitmap
   */
  static snap(stage: Stage): Bitmap;
}

declare class CacheEntry {
  /**
   * The resource class. Allows to be collected as a garbage if not use for some time or ticks
   *
   * @class CacheEntry
   * @constructor
   * @param {ResourceManager} resource manager
   * @param {string} key, url of the resource
   * @param {string} item - Bitmap, HTML5Audio, WebAudio - whatever you want to store in the cache
   */
  constructor(cache: CacheMap, key: string, item: string);

  /**
   * frees the resource
   */
  free(byTTL?: boolean): void;

  /**
   * Allocates the resource
   * @returns {CacheEntry}
   */
  allocate(): CacheEntry;

  /**
   * Sets the time to live
   * @param {number} ticks TTL in ticks, 0 if not set
   * @param {number} time TTL in seconds, 0 if not set
   * @returns {CacheEntry}
   */
  setTimeToLive(ticks?: number, seconds?: number): CacheEntry;

  isStillAlive(): boolean;

  /**
   * makes sure that resource wont freed by Time To Live
   * if resource was already freed by TTL, put it in cache again
   */
  touch(): void;
}

declare class CacheMap {
  /**
   * Cache for images, audio, or any other kind of resource
   * @param manager
   * @constructor
   */
  constructor(manager: ImageManager);

  /**
   * checks ttl of all elements and removes dead ones
   */
  checkTTL(): void;

  /**
   * cache item
   * @param key url of cache element
   * @returns {*|null}
   */
  getItem(key: string): any;

  clear(): void;

  setItem(key: string, item: any): CacheEntry;

  update(ticks: number, delta: number): void;
}

/**
 * The static class that carries out graphics processing.
 *
 * @class Graphics
 */
declare class Graphics {
  protected constructor();

  /**
   * The total frame count of the game screen.
   *
   * @static
   * @property frameCount
   * @type Number
   */
  static frameCount: number;
  /**
   * The alias of PIXI.blendModes.NORMAL.
   *
   * @static
   * @property BLEND_NORMAL
   * @type Number
   * @final
   */
  static BLEND_NORMAL: number;
  /**
   * The alias of PIXI.blendModes.ADD.
   *
   * @static
   * @property BLEND_ADD
   * @type Number
   * @final
   */
  static BLEND_ADD: number;
  /**
   * The alias of PIXI.blendModes.MULTIPLY.
   *
   * @static
   * @property BLEND_MULTIPLY
   * @type Number
   * @final
   */
  static BLEND_MULTIPLY: number;
  /**
   * The alias of PIXI.blendModes.SCREEN.
   *
   * @static
   * @property BLEND_SCREEN
   * @type Number
   * @final
   */
  static BLEND_SCREEN: number;
  /**
   * The width of the game screen.
   *
   * @static
   * @property width
   * @type Number
   */
  static width: number;
  /**
   * The height of the game screen.
   *
   * @static
   * @property height
   * @type Number
   */
  static height: number;
  /**
   * The width of the window display area.
   *
   * @static
   * @property boxWidth
   * @type Number
   */
  static boxWidth: number;
  /**
   * The height of the window display area.
   *
   * @static
   * @property boxHeight
   * @type Number
   */
  static boxHeight: number;
  /**
   * The zoom scale of the game screen.
   *
   * @static
   * @property scale
   * @type Number
   */
  static scale: number;

  /**
   * Initializes the graphics system.
   *
   * @static
   * @method initialize
   * @param {Number} width The width of the game screen
   * @param {Number} height The height of the game screen
   * @param {String} type The type of the renderer.
   *                 'canvas', 'webgl', or 'auto'.
   */
  static initialize(width?: number, height?: number, type?: string): void;

  /**
   * Marks the beginning of each frame for FPSMeter.
   *
   * @static
   * @method tickStart
   */
  static tickStart(): void;

  /**
   * Marks the end of each frame for FPSMeter.
   *
   * @static
   * @method tickEnd
   */
  static tickEnd(): void;

  /**
   * Renders the stage to the game screen.
   *
   * @static
   * @method render
   * @param {Stage} stage The stage object to be rendered
   */
  static render(stage?: Stage): void;

  /**
   * Checks whether the renderer type is WebGL.
   *
   * @static
   * @method isWebGL
   * @return {Boolean} True if the renderer type is WebGL
   */
  static isWebGL(): boolean;

  /**
   * Checks whether the current browser supports WebGL.
   *
   * @static
   * @method hasWebGL
   * @return {Boolean} True if the current browser supports WebGL.
   */
  static hasWebGL(): boolean;

  /**
   * Checks whether the canvas blend mode 'difference' is supported.
   *
   * @static
   * @method canUseDifferenceBlend
   * @return {Boolean} True if the canvas blend mode 'difference' is supported
   */
  static canUseDifferenceBlend(): boolean;

  /**
   * Checks whether the canvas blend mode 'saturation' is supported.
   *
   * @static
   * @method canUseSaturationBlend
   * @return {Boolean} True if the canvas blend mode 'saturation' is supported
   */
  static canUseSaturationBlend(): boolean;

  /**
   * Sets the source of the "Now Loading" image.
   *
   * @static
   * @method setLoadingImage
   * @param {String} Path of image
   */
  static setLoadingImage(src: string): void;

  /**
   * Initializes the counter for displaying the "Now Loading" image.
   *
   * @static
   * @method startLoading
   */
  static startLoading(): void;

  /**
   * Increments the loading counter and displays the "Now Loading" image if necessary.
   *
   * @static
   * @method updateLoading
   */
  static updateLoading(): void;

  /**
   * Erases the "Now Loading" image.
   *
   * @static
   * @method endLoading
   */
  static endLoading(): void;

  /**
   * Displays the error text to the screen.
   *
   * @static
   * @method printError
   * @param {String} name The name of the error
   * @param {String} message The message of the error
   */
  static printError(name: string, message: string): void;

  /**
   * Shows the FPSMeter element.
   *
   * @static
   * @method showFps
   */
  static showFps(): void;

  /**
   * Hides the FPSMeter element.
   *
   * @static
   * @method hideFps
   */
  static hideFps(): void;

  /**
   * Loads a font file.
   *
   * @static
   * @method loadFont
   * @param {String} name The face name of the font
   * @param {String} url The url of the font file
   */
  static loadFont(name: string, url: string): void;

  /**
   * Checks whether the font file is loaded.
   *
   * @static
   * @method isFontLoaded
   * @param {String} name The face name of the font
   * @return {Boolean} True if the font file is loaded
   */
  static isFontLoaded(name: string): boolean;

  /**
   * Starts playback of a video.
   *
   * @static
   * @method playVideo
   * @param {String} src
   */
  static playVideo(src: string): void;

  /**
   * Checks whether the video is playing.
   *
   * @static
   * @method isVideoPlaying
   * @return {Boolean} True if the video is playing
   */
  static isVideoPlaying(): boolean;

  /**
   * Checks whether the browser can play the specified video type.
   *
   * @static
   * @method canPlayVideoType
   * @param {String} type The video type to test support for
   * @return {Boolean} True if the browser can play the specified video type
   */
  static canPlayVideoType(type: string): boolean;

  /**
   * Converts an x coordinate on the page to the corresponding
   * x coordinate on the canvas area.
   *
   * @static
   * @method pageToCanvasX
   * @param {Number} x The x coordinate on the page to be converted
   * @return {Number} The x coordinate on the canvas area
   */
  static pageToCanvasX(x: number): number;

  /**
   * Converts a y coordinate on the page to the corresponding
   * y coordinate on the canvas area.
   *
   * @static
   * @method pageToCanvasY
   * @param {Number} y The y coordinate on the page to be converted
   * @return {Number} The y coordinate on the canvas area
   */
  static pageToCanvasY(y: number): number;

  /**
   * Checks whether the specified point is inside the game canvas area.
   *
   * @static
   * @method isInsideCanvas
   * @param {Number} x The x coordinate on the canvas area
   * @param {Number} y The y coordinate on the canvas area
   * @return {Boolean} True if the specified point is inside the game canvas area
   */
  static isInsideCanvas(x: number, y: number): boolean;

  /**
   * Calls pixi.js garbage collector
   */
  static callGC(): void;

  static _renderer: PIXI.AbstractRenderer;
}

declare class HTML5Audio {
  protected constructor();

  _initialized: boolean;
  _unlocked: boolean;
  _audioElement: HTMLAudioElement;
  _gainTweenInterval: number;
  _tweenGain: number;
  _tweenTargetGain: number;
  _tweenGainStep: number;
  _staticSePath: boolean;
  _volume: number;
  _loadListeners: (() => void)[];
  _hasError: boolean;
  __autoPlay: boolean;
  _isLoading: boolean;
  _buffered: boolean;
  /**
   * [read-only] The url of the audio file.
   *
   * @property url
   * @type String
   */
  url: string;
  /**
   * The volume of the audio.
   *
   * @property volume
   * @type Number
   */
  volume: number;

  /**
   * Sets up the Html5 Audio.
   *
   * @static
   * @method setup
   * @param {String} url The url of the audio file
   */
  setup(url: string): void;

  /**
   * Initializes the audio system.
   *
   * @static
   * @method initialize
   * @return {Boolean} True if the audio system is available
   */
  initialize(): void;

  /**
   * Clears the audio data.
   *
   * @static
   * @method clear
   */
  clear(): void;

  /**
   * Set the URL of static se.
   *
   * @static
   * @param {String} url
   */
  setStaticSe(url: string): void;

  /**
   * Checks whether the audio data is ready to play.
   *
   * @static
   * @method isReady
   * @return {Boolean} True if the audio data is ready to play
   */
  isReady(): boolean;

  /**
   * Checks whether a loading error has occurred.
   *
   * @static
   * @method isError
   * @return {Boolean} True if a loading error has occurred
   */
  isError(): boolean;

  /**
   * Checks whether the audio is playing.
   *
   * @static
   * @method isPlaying
   * @return {Boolean} True if the audio is playing
   */
  isPlaying(): boolean;

  /**
   * Plays the audio.
   *
   * @static
   * @method play
   * @param {Boolean} loop Whether the audio data play in a loop
   * @param {Number} offset The start position to play in seconds
   */
  play(loop: boolean, offset: number): void;

  /**
   * Stops the audio.
   *
   * @static
   * @method stop
   */
  stop(): void;

  /**
   * Performs the audio fade-in.
   *
   * @static
   * @method fadeIn
   * @param {Number} duration Fade-in time in seconds
   */
  fadeIn(duration: number): void;

  /**
   * Performs the audio fade-out.
   *
   * @static
   * @method fadeOut
   * @param {Number} duration Fade-out time in seconds
   */
  fadeOut(duration: number): void;

  /**
   * Gets the seek position of the audio.
   *
   * @static
   * @method seek
   */
  seek(): void;
}

declare class Input {
  protected constructor();

  /**
   * The wait time of the key repeat in frames.
   *
   * @static
   * @property keyRepeatWait
   * @type Number
   */
  static keyRepeatWait: number;
  /**
   * The interval of the key repeat in frames.
   *
   * @static
   * @property keyRepeatInterval
   * @type Number
   */
  static keyRepeatInterval: number;
  /**
   * A hash table to convert from a virtual key code to a mapped key name.
   *
   * @static
   * @property keyMapper
   * @type Object
   * ```
   * {
   * [key:number]:string
   * }
   * ```
   */
  static keyMapper: Object;
  /**
   * A hash table to convert from a gamepad button to a mapped key name.
   *
   * @static
   * @property gamepadMapper
   * @type Object
   * ```
   * {
   * [key:number]:String
   * }
   * ```
   */
  static gamepadMapper: Object;
  /**
   * [read-only] The four direction value as a number of the numpad, or 0 for neutral.
   *
   * @static
   * @property dir4
   * @type Number
   */
  static dir4: number;
  /**
   * [read-only] The eight direction value as a number of the numpad, or 0 for neutral.
   *
   * @static
   * @property dir8
   * @type Number
   */
  static dir8: number;
  /**
   * [read-only] The time of the last input in milliseconds.
   *
   * @static
   * @property date
   * @type Number
   */
  static date: number;

  /**
   * Initializes the input system.
   *
   * @static
   * @method initialize
   */
  static initialize(): void;

  /**
   * Clears all the input data.
   *
   * @static
   * @method clear
   */
  static clear(): void;

  /**
   * Updates the input data.
   *
   * @static
   * @method update
   */
  static update(): void;

  /**
   * Checks whether a key is currently pressed down.
   *
   * @static
   * @method isPressed
   * @param {String} keyName The mapped name of the key
   * @return {Boolean} True if the key is pressed
   */
  static isPressed(keyName: string): boolean;

  /**
   * Checks whether a key is just pressed.
   *
   * @static
   * @method isTriggered
   * @param {String} keyName The mapped name of the key
   * @return {Boolean} True if the key is triggered
   */
  static isTriggered(keyName: string): boolean;

  /**
   * Checks whether a key is just pressed or a key repeat occurred.
   *
   * @static
   * @method isRepeated
   * @param {String} keyName The mapped name of the key
   * @return {Boolean} True if the key is repeated
   */
  static nisRepeated(keyName: string): boolean;

  /**
   * Checks whether a key is kept depressed.
   *
   * @static
   * @method isLongPressed
   * @param {String} keyName The mapped name of the key
   * @return {Boolean} True if the key is long-pressed
   */
  static isLongPressed(keyName: string): boolean;
}

/**
 * The static class that handles JSON with object information.
 *
 * @class JsonEx
 */
declare class JsonEx {
  protected constructor();

  /**
   * The maximum depth of objects.
   *
   * @static
   * @property maxDepth
   * @type Number
   * @default 100
   */
  maxDepth: number;

  /**
   * Makes a deep copy of the specified object.
   *
   * @static
   * @method makeDeepCopy
   * @param {Object} object The object to be copied
   * @return {Object} The copied object
   */
  static makeDeepCopy(object: any): any;

  /**
   * Converts an object to a JSON string with object information.
   *
   * @static
   * @method stringify
   * @param {Object} object The object to be converted
   * @return {String} The JSON string
   */
  static stringify(object: any): string;

  /**
   * Parses a JSON string and reconstructs the corresponding object.
   *
   * @static
   * @method parse
   * @param {String} json The JSON string
   * @return {Object} The reconstructed object
   */
  static parse(json: string): any;
}

declare namespace PIXI {
  declare class Point {
    /**
     * The Point object represents a location in a two-dimensional coordinate system, where x represents
     * the horizontal axis and y represents the vertical axis.
     *
     * @class
     * @namespace PIXI
     * @param [x=0] {number} position of the point on the x axis
     * @param [y=0] {number} position of the point on the y axis
     */
    constructor(x?: number, y?: number);

    /**
     * Copies x and y from the given point
     *
     * @param {Point} The point to copy from
     * @return {Point} Returns itself.
     */
    copyFrom(p: PIXI.Point): PIXI.Point;

    /**
     * Copies x and y into the given point
     *
     * @param {Point} p - The point to copy.
     * @return {Point} Given point with values updated
     */
    copyTo(p: PIXI.Point): PIXI.Point;

    /**
     * Returns true if the given point is equal to this point
     *
     * @method copy
     * @param {Point} the point to compare
     * @return {Bool} the result of the comparison
     */
    equals(p: PIXI.Point): boolean;

    /**
     * Sets the point to a new x and y position.
     * If y is ommited, both x and y will be set to x.
     *
     * @method set
     * @param [x=0] {Float} position of the point on the x axis
     * @param [y=0] {Float} position of the point on the y axis
     */
    set(x?: number, y?: number): void;

    /**
     * @property x
     * @type Float
     * @default 0
     */
    x: number;
    /**
     * @property y
     * @type Float
     * @default 0
     */
    y: number;
  }
}

declare class Point extends PIXI.Point {
  /**
   * The point class.
   *
   * @class Point
   * @constructor
   * @param {Number} x The x coordinate
   * @param {Number} y The y coordinate
   */
  constructor(x?: number, y?: number);

  /**
   * The x coordinate.
   *
   * @property x
   * @type Number
   */
  x: number;
  /**
   * The y coordinate.
   *
   * @property y
   * @type Number
   */
  y: number;
}

declare namespace pixi.core.math.shapes {
  declare class Shape {
    protected constructor();

    /**
     * Defines the type of the shape to avoid instanceof (Std.is) checks
     */
    type: any;

    /**
     * Checks whether the x and y coordinates passed to this function are contained within this Shape
     *
     * @method contains
     * @param x {Float} The X coordinate of the point to test
     * @param y {Float} The Y coordinate of the point to test
     * @return {Bool} Whether the x/y coords are within this Rectangle
     */
    contains(x: number, y: number): boolean;
  }
}

declare namespace PIXI {
  declare class Rectangle extends pixi.core.math.shapes.Shape {
    /**
     * the Rectangle object is an area defined by its position, as indicated by its top-left corner point (x, y) and by its width and its height.
     *
     * @class Rectangle
     * @constructor
     * @param x {Float} The X coord of the upper-left corner of the rectangle
     * @param y {Float} The Y coord of the upper-left corner of the rectangle
     * @param width {Float} The overall width of this rectangle
     * @param height {Float} The overall height of this rectangle
     */
    constructor(x?: number, y?: number, width?: number, height?: number);

    /**
     * Creates a clone of this Rectangle instance
     *
     * @method clone
     * @return {Rectangle} a copy of the rectangle
     */
    clone(): PIXI.Rectangle;

    /**
     * Copies another Rectangle to this one.
     *
     * @param  rectangle {Rectangle} The rectangle to copy
     * @return {Rectangle} Returns itself.
     */
    copy(rectangle: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Checks whether the x and y coordinates given are contained within the rectangle.
     *
     * @param  x {Float} x coordinate of the point to test.
     * @param  y {Float} y coordinate of the point to test.
     * @return {Bool} whether the x and y coordinates are within this Rectangle.
     */
    contains(x: number, y: number): boolean;

    /**
     * Pads this rectangle, making it grow in all directions.
     *
     * @param  padX {Float} The horizontal padding amount.
     * @param  padY {Float} The vertical padding amount.
     */
    pad(padX: number, padY: number): void;

    /**
     * Fits this rectangle around the passed one.
     *
     * @param  rectangle {Rectangle} The rectangle to fit.
     */
    fit(rectangle: PIXI.Rectangle): void;

    /**
     * Enlarges this rectangle to fit around the passed one.
     *
     * @param  rectangle {Rectangle} The rectangle to include.
     */
    enlarge(rectangle: PIXI.Rectangle): void;

    /**
     * @property x
     * @type Float
     * @default 0
     */
    x: number;
    /**
     * @property y
     * @type Float
     * @default 0
     */
    y: number;
    /**
     * @property width
     * @type Float
     * @default 0
     */
    width: number;
    /**
     * @property height
     * @type Float
     * @default 0
     */
    height: number;
    /**
     * @property left
     * @type Float
     * @default 0
     */
    left: number;
    /**
     * @property right
     * @type Float
     * @default 0
     */
    right: number;
    /**
     * @property top
     * @type Float
     * @default 0
     */
    top: number;
    /**
     * @property bottom
     * @type Float
     * @default 0
     */
    bottom: number;
  }
}

declare class Rectangle extends PIXI.Rectangle {
  /**
   * The rectangle class.
   *
   * @class Rectangle
   * @constructor
   * @param {Number} x The x coordinate for the upper-left corner
   * @param {Number} y The y coordinate for the upper-left corner
   * @param {Number} width The width of the rectangle
   * @param {Number} height The height of the rectangle
   */
  constructor(x?: number, y?: number, width?: number, height?: number);

  /**
   * @static
   * @property emptyRectangle
   * @type Rectangle
   */
  static emptyRectangle: Rectangle;
}

declare namespace PIXI.utils {
  declare class EventEmitter {
    /**
     * Minimal EventEmitter interface that is molded against the Node.js
     * EventEmitter interface.
     *
     * @constructor
     */
    constructor();

    /**
     * Return a list of assigned event listeners.
     *
     * @param {String} eventName
     * @return {Array}
     */
    listeners(event: string): any[];

    /**
     * Emit an event to all registered event listeners.
     *
     * @alias dispatchEvent
     * @param eventName {String} The name of the event.
     * @return {Bool} Indication if we've emitted an event.
     */
    emit(event: string, a1?: any, a2?: any, a3?: any, a4?: any, a5?: any): boolean;

    /**
     * Register a new EventListener for the given event.
     *
     * @param {String} event Name of the event.
     * @param {function} fn Callback function.
     * @param {Mixed} context The context of the function.
     */
    on(event: string, fn: (arg0: PIXI.interaction.InteractionEvent) => void, context?: any): void;

    /**
     * Add an EventListener that's only called once.
     *
     * @param {String} event Name of the event.
     * @param {function} fn Callback function.
     * @param {Mixed} context The context of the function.
     */
    once(event: string, fn: (arg0: PIXI.interaction.InteractionEvent) => void, context?: any): void;

    /**
     * Register a new EventListener for the given event.
     *
     * @param {String} event Name of the event.
     * @param {function} fn Callback function.
     * @param {Mixed} context The context of the function.
     */
    addListener(event: string, fn: (arg0: PIXI.interaction.InteractionEvent) => void, context?: any): void;

    /**
     * Remove event listeners.
     *
     * @param {String} event The event we want to remove.
     * @param {function} fn The listener that we need to find.
     * @param {Bool} once Only remove once listeners.
     */
    off(event: string, fn: (arg0: PIXI.interaction.InteractionEvent) => void, once?: boolean): void;

    /**
     * Remove event listeners.
     *
     * @param {String} event The event we want to remove.
     * @param {function} fn The listener that we need to find.
     * @param {Bool} once Only remove once listeners.
     */
    removeListener(event: string, fn: (arg0: PIXI.interaction.InteractionEvent) => void, once?: boolean): void;

    /**
     * Remove all listeners or only the listeners for the specified event.
     *
     * @param {String} event The event want to remove all listeners for.
     */
    removeAllListeners(event?: string): void;
  }
}

declare namespace PIXI.interaction {
  declare class interactiveTarget extends PIXI.utils.EventEmitter {
    protected constructor();

    /**
     * Fired when a pointer device button (usually a mouse button) is pressed on the display
     * object.
     *
     * @event mousedown
     * @memberof InteractiveTarget#
     */
    mousedown(event: PIXI.interaction.InteractionEvent): void;

    /**
     * Fired when a pointer device secondary button (usually a mouse right-button) is pressed
     * on the display object.
     *
     * @event rightdown
     * @memberof InteractiveTarget#
     */
    rightdown(event: PIXI.interaction.InteractionEvent): void;

    /**
     * Fired when a pointer device button (usually a mouse button) is released over the display
     * object.
     *
     * @event mouseup
     * @memberof InteractiveTarget#
     */
    mouseup(event: PIXI.interaction.InteractionEvent): void;

    /**
     * Fired when a pointer device secondary button (usually a mouse right-button) is released
     * over the display object.
     *
     * @event rightup
     * @memberof InteractiveTarget#
     */
    rightup(event: PIXI.interaction.InteractionEvent): void;

    /**
     * Fired when a pointer device button (usually a mouse button) is pressed and released on
     * the display object.
     *
     * @event click
     * @memberof InteractiveTarget#
     */
    click(event: PIXI.interaction.InteractionEvent): void;

    /**
     * Fired when a pointer device secondary button (usually a mouse right-button) is pressed
     * and released on the display object.
     *
     * @event rightclick
     * @memberof InteractiveTarget#
     */
    rightclick(event: PIXI.interaction.InteractionEvent): void;

    /**
     * Fired when a pointer device button (usually a mouse button) is released outside the
     * display object that initially registered a
     * [mousedown]{@link PIXI.interaction.InteractionManager#event:mousedown}.
     *
     * @event mouseupoutside
     * @memberof InteractiveTarget#
     */
    mouseupoutside(event: PIXI.interaction.InteractionEvent): void;

    /**
     * Fired when a pointer device secondary button (usually a mouse right-button) is released
     * outside the display object that initially registered a
     * [rightdown]{@link PIXI.interaction.InteractionManager#event:rightdown}.
     *
     * @event rightupoutside
     * @memberof InteractiveTarget#
     */
    rightupoutside(event: PIXI.interaction.InteractionEvent): void;

    /**
     * Fired when a pointer device (usually a mouse) is moved while over the display object
     *
     * @event mousemove
     * @memberof InteractiveTarget#
     */
    mousemove(event: PIXI.interaction.InteractionEvent): void;

    /**
     * Fired when a pointer device (usually a mouse) is moved onto the display object
     *
     * @event mouseover
     * @memberof InteractiveTarget#
     */
    mouseover(event: PIXI.interaction.InteractionEvent): void;

    /**
     * Fired when a pointer device (usually a mouse) is moved off the display object
     *
     * @event mouseout
     * @memberof InteractiveTarget#
     */
    mouseout(event: PIXI.interaction.InteractionEvent): void;

    /**
     * Fired when a pointer device button is pressed on the display object.
     *
     * @event pointerdown
     * @memberof InteractiveTarget#
     */
    pointerdown(event: PIXI.interaction.InteractionEvent): void;

    /**
     * Fired when a pointer device button is released over the display object.
     *
     * @event pointerup
     * @memberof InteractiveTarget#
     */
    pointerup(event: PIXI.interaction.InteractionEvent): void;

    /**
     * Fired when a pointer device button is pressed and released on the display object.
     *
     * @event pointertap
     * @memberof InteractiveTarget#
     */
    pointertap(event: PIXI.interaction.InteractionEvent): void;

    /**
     * Fired when a pointer device button is released outside the display object that initially
     * registered a [pointerdown]{@link PIXI.interaction.InteractionManager#event:pointerdown}.
     *
     * @event pointerupoutside
     * @memberof InteractiveTarget#
     */
    pointerupoutside(event: PIXI.interaction.InteractionEvent): void;

    /**
     * Fired when a pointer device is moved while over the display object
     *
     * @event pointermove
     * @memberof InteractiveTarget#
     */
    pointermove(event: PIXI.interaction.InteractionEvent): void;

    /**
     * Fired when a pointer device is moved onto the display object
     *
     * @event pointerover
     * @memberof InteractiveTarget#
     */
    pointerover(event: PIXI.interaction.InteractionEvent): void;

    /**
     * Fired when a pointer device is moved off the display object
     *
     * @event pointerout
     * @memberof InteractiveTarget#
     */
    pointerout(event: PIXI.interaction.InteractionEvent): void;

    /**
     * Fired when a touch point is placed on the display object.
     *
     * @event touchstart
     * @memberof InteractiveTarget#
     */
    touchstart(event: PIXI.interaction.InteractionEvent): void;

    /**
     * Fired when a touch point is removed from the display object.
     *
     * @event touchend
     * @memberof InteractiveTarget#
     */
    touchend(event: PIXI.interaction.InteractionEvent): void;

    /**
     * Fired when a touch point is placed and removed from the display object.
     *
     * @event tap
     * @memberof InteractiveTarget#
     */
    tap(event: PIXI.interaction.InteractionEvent): void;

    /**
     * Fired when a touch point is removed outside of the display object that initially
     * registered a [touchstart]{@link PIXI.interaction.InteractionManager#event:touchstart}.
     *
     * @event touchendoutside
     * @memberof InteractiveTarget#
     */
    touchendoutside(event: PIXI.interaction.InteractionEvent): void;

    /**
     * Fired when a touch point is moved along the display object.
     *
     * @event touchmove
     * @memberof InteractiveTarget#
     */
    touchmove(event: PIXI.interaction.InteractionEvent): void;

    /**
     * Indicates if the displayObject is interactive or not.
     *
     * @member {Bool}
     * @default false
     * @memberof InteractiveTarget#
     */
    interactive: boolean;
    /**
     * Indicates if the children of displayObject are interactive or not.
     *
     * @member {Bool}
     * @default true
     * @memberof InteractiveTarget#
     */
    interactiveChildren: boolean;
    /**
     * Interaction shape. Children will be hit first, then this shape will be checked.
     *
     * @member {Shape}
     * @memberof InteractiveTarget#
     * @default null
     */
    hitArea: pixi.core.math.shapes.Shape;
    /**
     * Indicates if the displayObject uses button mode or normal mode.
     *
     * @member {Bool}
     * @default false
     * @memberof InteractiveTarget#
     */
    buttonMode: boolean;
    /**
     * Default cursor.
     *
     * @member {String}
     * @default pointer
     * @memberof InteractiveTarget#
     */
    defaultCursor: string;
    /**
     * Internal check to detect if the mouse cursor is hovered over the displayObject
     *
     * @member {Bool}
     * @private
     */
    _over: boolean;
    /**
     * Internal check to detect if the left mouse button is pressed on the displayObject
     *
     * @member {Bool}
     * @private
     */
    _isLeftDown: boolean;
    /**
     * Internal check to detect if the right mouse button is pressed on the displayObject
     *
     * @member {Bool}
     * @private
     */
    _isRightDown: boolean;
    /**
     * Internal check to detect if the pointer cursor is hovered over the displayObject
     *
     * @member {Bool}
     * @private
     */
    _pointerOver: boolean;
    /**
     * Internal check to detect if the pointer is down on the displayObject
     *
     * @member {Bool}
     * @private
     */
    _pointerDown: boolean;
    /**
     * Internal check to detect if a user has touched the displayObject
     *
     * @member {Bool}
     * @private
     */
    _touchDown: boolean;
  }
}

declare namespace pixi.core.display {
  export type DestroyOptions = {
    /**
     * {boolean} [options.baseTexture=false] - Only used for child Sprites if options.children is set to true
     * Should it destroy the base texture of the child sprite
     */
    baseTexture: boolean;
    /**
     * {boolean} [options.children=false] - if set to true, all the children will have their destroy
     *  method called as well. 'options' will be passed on to those calls.
     */
    children: boolean;
    /**
     * {boolean} [options.texture=false] - Only used for child Sprites if options.children is set to true
     * Should it destroy the texture of the child sprite
     */
    texture: boolean;
  };
}

declare namespace PIXI {
  declare class DisplayObject extends PIXI.interaction.interactiveTarget {
    /**
     * The base class for all objects that are rendered on the screen.
     * This is an abstract class and should not be used on its own rather it should be extended.
     *
     * @class
     * @namespace PIXI
     */
    constructor();

    /**
     * Retrieves the bounds of the displayObject as a rectangle object
     *
     * @param skipUpdate {Matrix} setting to true will stop the transforms of the scene graph from being updated.
     * This means the calculation returned MAY be out of date BUT will give you a nice performance boost
     * @param rect {Rectangle} Optional rectangle to store the result of the bounds calculation
     * @return {Rectangle} the rectangular bounding area
     */
    getBounds(skipUpdate?: boolean, rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Retrieves the local bounds of the displayObject as a rectangle object
     *
     * @param rect {Rectangle} Optional rectangle to store the result of the bounds calculation
     * @return {Rectangle} the rectangular bounding area
     */
    getLocalBounds(rect?: PIXI.Rectangle): PIXI.Rectangle;

    /**
     * Calculates the global position of the display object
     *
     * @param position {Point} The world origin to calculate from
     * @param point {Point} A Point in which to store the value, optional (otherwise a new Point is created)
     * @param skipUpdate {Bool} Should we skip the update transform
     * @return {Point} A point object representing the position of this object
     */
    toGlobal(position: PIXI.Point, point?: PIXI.Point, skipUpdate?: boolean): PIXI.Point;

    /**
     * Calculates the local position of the display object relative to another point
     *
     * @param position {Point} The world origin to calculate from
     * @param [from] {DisplayObject} The DisplayObject to calculate the global position from
     * @param [point] {Point} A Point object in which to store the value, optional (otherwise will create a new Point)
     * @return {Point} A point object representing the position of this object
     */
    toLocal(position: PIXI.Point, frm?: PIXI.DisplayObject, point?: PIXI.Point): PIXI.Point;

    updateTransform(): void;

    displayObjectUpdateTransform(): void;

    /**
     * Set the parent Container of this DisplayObject
     *
     * @param container {Container} The Container to add this DisplayObject to
     * @return {Container} The Container that this DisplayObject was added to
     */
    setParent(container: PIXI.Container): PIXI.Container;

    /**
     * Convenience function to set the postion, scale, skew and pivot at once.
     *
     * @param [x=0] {Float} The X position
     * @param [y=0] {Float} The Y position
     * @param [scaleX=1] {Float} The X scale value
     * @param [scaleY=1] {Float} The Y scale value
     * @param [skewX=0] {Float} The X skew value
     * @param [skewY=0] {Float} The Y skew value
     * @param [pivotX=0] {Float} The X pivot value
     * @param [pivotY=0] {Float} The Y pivot value
     * @return {DisplayObject}
     */
    setTransform(
      x?: number,
      y?: number,
      scaleX?: number,
      scaleY?: number,
      rotation?: number,
      skewX?: number,
      skewY?: number,
      pivotX?: number,
      pivotY?: number,
    ): PIXI.DisplayObject;

    /**
     * Base destroy method for generic display objects
     * Removes all internal references and listeners as well as removes children from the display list.
     *
     * @param {object|boolean} [options] - Options parameter. A boolean will act as if all options
     *  have been set to that value
     * @param {boolean} [options.children=false] - if set to true, all the children will have their destroy
     *  method called as well. 'options' will be passed on to those calls.
     * @param {boolean} [options.texture=false] - Only used for child Sprites if options.children is set to true
     *  Should it destroy the texture of the child sprite
     * @param {boolean} [options.baseTexture=false] - Only used for child Sprites if options.children is set to true
     *  Should it destroy the base texture of the child sprite
     */
    destroy(options?: boolean | pixi.core.display.DestroyOptions): void;

    /**
     * The instance name of the object.
     *
     * @member {String}
     */
    name: string;
    /**
     * Set this to true if you want this display object to be cached as a bitmap.
     * This basically takes a snap shot of the display object as it is at that moment. It can provide a performance benefit for complex static displayObjects.
     * To remove simply set this property to 'null'
     *
     * @member {Bool}
     * @memberof DisplayObject#
     */
    cacheAsBitmap: boolean;
    /**
     * The opacity of the object.
     *
     * @member {Float}
     */
    alpha: number;
    /**
     * World transform and local transform of this object. This will become read-only later,
     * please do not assign anything there unless you know what are you doing.
     *
     * @member {Transform}
     */
    transform: PIXI.Transform;
    /**
     * The visibility of the object. If false the object will not be drawn, and
     * the updateTransform function will not be called.
     *
     * @member {Bool}
     */
    visible: boolean;
    /**
     * Can this object be rendered, if false the object will not be drawn but the updateTransform
     * methods will still be called.
     *
     * @member {Bool}
     */
    renderable: boolean;
    /**
     * The display object container that contains this display object.
     *
     * @member {Container}
     * @readOnly
     */
    parent: PIXI.Container;
    /**
     * The multiplied alpha of the displayObject
     *
     * @member {Float}
     * @readOnly
     */
    worldAlpha: number;
    /**
     * The area the filter is applied to. This is used as more of an optimisation
     * rather than figuring out the dimensions of the displayObject each frame you can set this rectangle
     *
     * Also works as an interaction mask
     *
     * @member {Rectangle}
     */
    filterArea: PIXI.Rectangle;
    /**
     * The position of the displayObject on the x axis relative to the local coordinates of the parent.
     *
     * @member {Float}
     * @memberof DisplayObject#
     */
    x: number;
    /**
     * The position of the displayObject on the y axis relative to the local coordinates of the parent.
     *
     * @member {Float}
     * @memberof DisplayObject#
     */
    y: number;
    /**
     * Current transform of the object based on world (parent) factors
     *
     * @member {Matrix}
     * @readOnly
     */
    worldTransform: PIXI.Matrix;
    /**
     * Current transform of the object based on local factors: position, scale, other stuff
     *
     * @member {PIXI.Matrix}
     * @readOnly
     */
    localTransform: PIXI.Matrix;
    /**
     * The coordinate of the object relative to the local coordinates of the parent.
     *
     * @member {Point}
     */
    position: PIXI.Point;
    /**
     * The scale factor of the object.
     *
     * @member {Point}
     */
    scale: PIXI.Point;
    /**
     * The pivot point of the displayObject that it rotates around
     *
     * @member {Point}
     */
    pivot: PIXI.Point;
    /**
     * The skew factor for the object in radians.
     *
     * @member {Point}
     */
    skew: PIXI.Point;
    /**
     * The rotation of the object in radians.
     *
     * @member {Float}
     */
    rotation: number;
    /**
     * Indicates if the displayObject is globally visible.
     *
     * @member {Bool}
     * @memberof DisplayObject#
     * @readonly
     */
    worldVisible: boolean;
    /**
     * Sets a mask for the displayObject. A mask is an object that limits the visibility of an object to the shape of the mask applied to it.
     * In PIXI a regular mask must be a PIXI.Graphics or a PIXI.Sprite object. This allows for much faster masking in canvas as it utilises shape clipping.
     * To remove a mask, set this property to null.
     *
     * For the moment, PIXI.CanvasRenderer doesn't support PIXI.Sprite as mask.
     *
     * @member {Graphics|Sprite}
     * @memberof DisplayObject#
     */
    mask: PIXI.Sprite | PIXI.Graphics;
    /**
     * Sets the filters for the displayObject.
     * * IMPORTANT: This is a webGL only feature and will be ignored by the canvas renderer.
     * To remove filters simply set this property to 'null'
     *
     * @member {Array<AbstractFilter>}
     * @memberof DisplayObject#
     */
    filters: PIXI.Filter[];
    /**
     *  Flag for if the object is accessible. If true AccessibilityManager will overlay a
     *   shadow div with attributes set
     *
     * @member {Bool}
     */
    accessible: boolean;
    /**
     * Sets the title attribute of the shadow div
     * If accessibleTitle AND accessibleHint has not been this will default to 'displayObject [tabIndex]'
     *
     * @member {String}
     */
    accessibleTitle: string;
    /**
     * Sets the aria-label attribute of the shadow div
     *
     * @member {String}
     */
    accessibleHint: string;
    tabIndex: number;
  }
}

declare namespace PIXI {
  declare class Container extends PIXI.DisplayObject {
    /**
     * A Container represents a collection of display objects.
     * It is the base class of all display objects that act as a container for other objects.
     *
     * @class
     * @extends DisplayObject
     * @namespace PIXI
     */
    constructor();

    /**
     * The array of children of this container.
     *
     * @member {DisplayObject[]}
     * @readonly
     */
    children: PIXI.DisplayObject[];
    /**
     * The width of the Container, setting this will actually modify the scale to achieve the value set
     *
     * @member {Float}
     * @memberof Container#
     */
    width: number;
    /**
     * The height of the Container, setting this will actually modify the scale to achieve the value set
     *
     * @member {Float}
     * @memberof Container#
     */
    height: number;

    /**
     * Adds a child to the container.
     *
     * You can also add multple items like so: myContainer.addChild(thinkOne, thingTwo, thingThree)
     *
     * @param child {DisplayObject} The DisplayObject to add to the container
     * @return {DisplayObject} The child that was added.
     */
    addChild<T>(child: T): T;

    /**
     * Adds a child to the container at a specified index.
     * If the index is out of bounds an error will be thrown.
     *
     * @param child {DisplayObject} The child to add
     * @param index {Int} The index to place the child in
     * @return {DisplayObject} The child that was added.
     */
    addChildAt<T>(child: T, index: number): T;

    /**
     * Swaps the position of 2 Display Objects within this container.
     *
     * @param child1 {DisplayObject}
     * @param child2 {DisplayObject}
     */
    swapChildren(child1: PIXI.DisplayObject, child2: PIXI.DisplayObject): void;

    /**
     * Returns the index position of a child DisplayObject instance
     *
     * @param child {DisplayObject} The DisplayObject instance to identify
     * @return {Int} The index position of the child display object to identify
     */
    getChildIndex(child: PIXI.DisplayObject): number;

    /**
     * Changes the position of an existing child in the display object container
     *
     * @param child {DisplayObject} The child DisplayObject instance for which you want to change the index number
     * @param index {Int} The resulting index number for the child display object
     */
    setChildIndex(child: PIXI.DisplayObject, index: number): void;

    /**
     * Returns the child at the specified index
     *
     * @param index {Int} The index to get the child from
     * @return {DisplayObject} The child at the given index, if any.
     */
    getChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes a child from the container.
     *
     * @param child {DisplayObject} The DisplayObject to remove
     * @return {DisplayObject} The child that was removed.
     */
    removeChild(child: PIXI.DisplayObject): PIXI.DisplayObject;

    /**
     * Removes a child from the specified index position.
     *
     * @param index {Int} The index to get the child from
     * @return {DisplayObject} The child that was removed.
     */
    removeChildAt(index: number): PIXI.DisplayObject;

    /**
     * Removes all children from this container that are within the begin and end indexes.
     *
     * @param beginIndex {Int} The beginning position. Default value is 0.
     * @param endIndex {Int} The ending position. Default value is size of the container.
     * @return {Array<DisplayObject>} The children that are removed.
     */
    removeChildren(beginIndex?: number, endIndex?: number): PIXI.DisplayObject[];

    /**
     * Returns the display object in the container
     *
     * @param name {string} instance name
     * @return {DisplayObject}
     */
    getChildByName(name: string): PIXI.DisplayObject;
  }
}

declare namespace PIXI {
  declare class Sprite extends PIXI.Container {
    /**
     * The Sprite object is the base for all textured objects that are rendered to the screen
     *
     * A sprite can be created directly from an image like this:
     *
     * ```haxe
     *  var sprite = new Sprite.fromImage('assets/image.png');
     * ```
     *
     * @class Sprite
     * @extends Container
     * @namespace PIXI
     * @param texture {Texture} The texture for this sprite
     */
    constructor(texture?: PIXI.Texture);

    /**
     * The anchor sets the origin point of the texture.
     * The default is 0,0 this means the texture's origin is the top left
     * Setting than anchor to 0.5,0.5 means the textures origin is centered
     * Setting the anchor to 1,1 would mean the textures origin points will be the bottom right corner
     *
     * @member {ObservablePoint}
     */
    anchor: PIXI.ObservablePoint;
    /**
     * The texture that the sprite is using
     *
     * @member {Texture}
     */
    texture: PIXI.Texture;
    /**
     * The tint applied to the sprite. This is a hex value. A value of 0xFFFFFF will remove any tint effect.
     *
     * @member {Int}
     * @default 0xFFFFFF
     */
    tint: number;
    /**
     * The blend mode to be applied to the sprite. Set to CONST.blendModes.NORMAL to remove any blend mode.
     *
     * @member {Int}
     * @default CONST.blendModes.NORMAL;
     */
    blendMode: any;
    /**
     * The shader that will be used to render the sprite. Set to null to remove a current shader.
     *
     * @member {Shader|Filter}
     */
    shader: PIXI.Shader | PIXI.Filter;
    /**
     * Plugin that is responsible for rendering this element.
     * Allows to customize the rendering process without overriding '_render' & '_renderCanvas' methods.
     *
     * @member {String}
     * @default 'sprite'
     */
    pluginName: string;

    /**
     * Tests if a point is inside this sprite
     *
     * @param {Point} point - the point to test
     * @return {Bool} the result of the test
     */
    containsPoint(point: PIXI.Point): boolean;

    /**
     * Helper function that creates a new sprite based on the source you provide.
     * The source can be - frame id, image url, video url, canvas element, video element, base texture
     *
     * @static
     * @param {Int|String|BaseTexture|HTMLCanvasElement|HTMLVideoElement} source Source to create texture from
     * @return {Sprite} The newly created sprite
     */
    static from(source: any): PIXI.Sprite;

    /**
     * Helper function that creates a sprite that will contain a texture from the TextureCache based on the frameId
     * The frame ids are created when a Texture packer file has been loaded
     *
     * @static
     * @param frameId {String} The frame Id of the texture in the cache
     * @return {Sprite} A new Sprite using a texture from the texture cache matching the frameId
     */
    static fromFrame(frameId: string): PIXI.Sprite;

    /**
     * Helper function that creates a sprite that will contain a texture based on an image url
     * If the image is not in the texture cache it will be loaded
     *
     * @static
     * @param imageId {String} The image url of the texture
     * @param [crossorigin=(auto)] {Bool} Should use anonymouse CORS? Defaults to true if the URL is not a data-URI.
     * @param [scaleMode=scaleModes.DEFAULT] {ScaleModes} See {@link scaleModes} for possible values
     * @return {Sprite} A new Sprite using a texture from the texture cache matching the image id
     */
    static fromImage(imageId: string, crossorigin?: boolean, scaleMode?: any): PIXI.Sprite;
  }
}

declare class ScreenSprite extends PIXI.Sprite {
  /**
   * The sprite which covers the entire game screen.
   *
   * @class ScreenSprite
   * @constructor
   */
  constructor();

  /**
   * The opacity of the sprite (0 to 255).
   *
   * @property opacity
   * @type Number
   */
  opacity: number;

  /**
   * Sets black to the color of the screen sprite.
   *
   * @method setBlack
   */
  setBlack(): void;

  /**
   * Sets white to the color of the screen sprite.
   *
   * @method setWhite
   */
  setWhite(): void;

  /**
   * Sets the color of the screen sprite by values.
   *
   * @method setColor
   * @param {Number} r The red value in the range (0, 255)
   * @param {Number} g The green value in the range (0, 255)
   * @param {Number} b The blue value in the range (0, 255)
   */
  setColor(r?: number, g?: number, b?: number): void;
}

declare class Tilemap extends PIXI.Container {
  /**
   * [read-only] The array of children of the sprite.
   *
   * @property children
   * @type Array<PIXI.DisplayObject>
   */
  constructor();

  /**
   * The bitmaps used as a tileset.
   *
   * @property bitmaps
   * @type Array
   */
  bitmaps: Bitmap[];
  /**
   * The origin point of the tilemap for scrolling.
   *
   * @property origin
   * @type Point
   */
  origin: Point;
  /**
   * The tileset flags.
   *
   * @property flags
   * @type Array
   */
  flags: number[];
  /**
   * The animation count for autotiles.
   *
   * @property animationCount
   * @type Number
   */
  animationCount: number;
  /**
   * Whether the tilemap loops horizontal.
   *
   * @property horizontalWrap
   * @type Boolean
   */
  horizontalWrap: boolean;
  /**
   * Whether the tilemap loops vertical.
   *
   * @property verticalWrap
   * @type Boolean
   */
  verticalWrap: boolean;
  /**
   * The width of the screen in pixels.
   *
   * @property width
   * @type Number
   */
  tileWidth: number;
  /**
   * The height of a tile in pixels.
   *
   * @property tileHeight
   * @type Number
   */
  tileHeight: number;

  /**
   * Sets the tilemap data.
   *
   * @method setData
   * @param {Number} width The width of the map in number of tiles
   * @param {Number} height The height of the map in number of tiles
   * @param {Array} data The one dimensional array for the map data
   */
  setData(width: number, height: number, data: number[]): void;

  /**
   * Checks whether the tileset is ready to render.
   *
   * @method isReady
   * @type Boolean
   * @return {Boolean} True if the tilemap is ready
   */
  isReady(): boolean;

  /**
   * Updates the tilemap for each frame.
   *
   * @method update
   */
  update(): void;

  /**
   * @method updateTransform
   * @private
   */
  updateTransform(): void;

  /**
   * Forces to repaint the entire static
   *
   * @method refresh
   */
  refresh(): void;

  /**
   * Adds a child to the container.
   *
   * @method addChild
   * @param {PIXI.DisplayObject} child The child to add
   * @return {PIXI.DisplayObject} The child that was added
   */
  addChild(child: PIXI.DisplayObject): PIXI.DisplayObject;

  /**
   * Adds a child to the container at a specified index.
   *
   * @method addChildAt
   * @param {PIXI.DisplayObject} child The child to add
   * @param {Number} index The index to place the child in
   * @return {PIXI.DisplayObject} The child that was added
   */
  addChildAt(child: PIXI.DisplayObject, index: number): PIXI.DisplayObject;

  /**
   * Removes a child from the container.
   *
   * @method removeChild
   * @param {PIXI.DisplayObject} child The child to remove
   * @return {PIXI.DisplayObject} The child that was removed
   */
  removeChild(child: PIXI.DisplayObject): PIXI.DisplayObject;

  /**
   * Removes a child from the specified index position.
   *
   * @method removeChildAt
   * @param {Number} index The index to get the child from
   * @return {PIXI.DisplayObject} The child that was removed
   */
  removeChildAt(index: number): PIXI.DisplayObject;

  /**
   * Forces to refresh the tileset
   *
   * @method refresh
   */
  refreshTileset(): void;

  static TILE_ID_A1: number;
  static TILE_ID_A2: number;
  static TILE_ID_A3: number;
  static TILE_ID_A4: number;
  static TILE_ID_A5: number;
  static TILE_ID_B: number;
  static TILE_ID_C: number;
  static TILE_ID_D: number;
  static TILE_ID_E: number;
  static TILE_ID_MAX: number;
  static FLOOR_AUTOTILE_TABLE: number[][][];
  static WALL_AUTOTILE_TABLE: number[][][];
  static WATERFALL_AUTOTILE_TABLE: number[][][];

  static isVisibleTile(tileId: number): boolean;

  static isAutotile(tileId: number): boolean;

  static getAutotileKind(tileId: number): number;

  static getAutotileShape(tileId: number): number;

  static makeAutotileId(kind: number, shape: number): number;

  static isSameKindTile(tileID1: number, tileID2: number): boolean;

  static isTileA1(tileId: number): boolean;

  static isTileA2(tileId: number): boolean;

  static isTileA3(tileId: number): boolean;

  static isTileA4(tileId: number): boolean;

  static isTileA5(tileId: number): boolean;

  static isWaterTile(tileId: number): boolean;

  static isWaterfallTile(tileId: number): boolean;

  static isGroundTile(tileId: number): boolean;

  static isShadowingTile(tileId: number): boolean;

  static isRoofTile(tileId: number): boolean;

  static isWallTopTile(tileId: number): boolean;

  static isWallSideTile(tileId: number): boolean;

  static isWallTile(tileId: number): boolean;

  static isFloorTypeAutotile(tileId: number): boolean;

  static isWallTypeAutotile(tileId: number): boolean;

  static isWaterfallTypeAutotile(tileId: number): boolean;
}

declare class ShaderTilemap extends Tilemap {
  protected constructor();

  /**
   * PIXI render method
   *
   * @method renderWebGL
   * @param {PIXI.WebGLRenderer} pixi renderer
   */
  renderWebGL(renderer: PIXI.Renderer): void;

  /**
   * PIXI render method
   *
   * @method renderCanvas
   * @param {PIXI.CanvasRenderer} pixi renderer
   */
  renderCanvas(renderer: PIXI.CanvasRenderer): void;

  /**
   * Forces to repaint the entire tilemap AND update bitmaps list if needed
   *
   * @method refresh
   */
  refresh(): void;

  /**
   * Call after you update tileset
   *
   * @method refreshTileset
   */
  refreshTileset(): void;
}

declare class Sprite extends PIXI.Sprite {
  /**
   * The basic object that is rendered to the game screen.
   *
   * @class Sprite
   * @constructor
   * @param {Bitmap} bitmap The image for the sprite
   */
  constructor(bitmap?: Bitmap);

  /**
   * The image for the sprite.
   *
   * @property bitmap
   * @type Bitmap
   */
  bitmap: Bitmap;
  /**
   * The width of the sprite without the scale.
   *
   * @property width
   * @type Number
   */
  opacity: number;
  /**
   * The visibility of the sprite.
   *
   * @property visible
   * @type Boolean
   */
  visible: boolean;
  /**
   * The x coordinate of the sprite.
   *
   * @property x
   * @type Number
   */
  x: number;
  /**
   * The y coordinate of the sprite.
   *
   * @property y
   * @type Number
   */
  y: number;
  /**
   * The origin point of the sprite. (0,0) to (1,1).
   *
   * @property anchor
   * @type Point
   */
  point: PIXI.Point;
  /**
   * The scale factor of the sprite.
   *
   * @property scale
   * @type Point
   */
  scale: PIXI.Point;
  /**
   * The rotation of the sprite in radians.
   *
   * @property rotation
   * @type Number
   */
  rotation: number;
  /**
   * The blend mode to be applied to the sprite.
   *
   * @property blendMode
   * @type Number
   */
  blendMode: number;
  /**
   * Sets the filters for the sprite.
   *
   * @property filters
   * @type Array<PIXI.Filter>
   */
  filters: PIXI.Filter[];
  /**
   * [read-only] The array of children of the sprite.
   *
   * @property children
   * @type Array<PIXI.DisplayObject>
   */
  spriteId: number;
  opaque: boolean;
  voidFilter: PIXI.Filter;

  initialize(bitmap?: Bitmap): void;

  /**
   * Updates the sprite for each frame.
   *
   * @method update
   */
  update(): void;

  /**
   * @method updateTransform
   * @private
   */
  updateTransform(): void;

  /**
   * Sets the x and y at once.
   *
   * @method move
   * @param {Number} x The x coordinate of the sprite
   * @param {Number} y The y coordinate of the sprite
   */
  move(x: number, y: number): void;

  /**
   * Sets the rectagle of the bitmap that the sprite displays.
   *
   * @method setFrame
   * @param {Number} x The x coordinate of the frame
   * @param {Number} y The y coordinate of the frame
   * @param {Number} width The width of the frame
   * @param {Number} height The height of the frame
   */
  setFrame(x: number, y: number, width: number, height: number): void;

  /**
   * Gets the blend color for the sprite.
   *
   * @method getBlendColor
   * @return {Array} The blend color [r, g, b, a]
   */
  getBlendColor(): number[];

  /**
   * Sets the blend color for the sprite.
   *
   * @method setBlendColor
   * @param {Array} color The blend color [r, g, b, a]
   */
  setBlendColor(color: number[]): void;

  /**
   * Gets the color tone for the sprite.
   *
   * @method getColorTone
   * @return {Array} The color tone [r, g, b, gray]
   */
  getColorTone(): number[];

  /**
   * Sets the color tone for the sprite.
   *
   * @method setColorTone
   * @param {Array} tone The color tone [r, g, b, gray]
   */
  setColorTone(tone: number[]): void;

  /**
   * Adds a child to the container.
   *
   * @method addChild
   * @param {PIXI.DisplayObject} child The child to add
   * @return {PIXI.DisplayObject} The child that was added
   */
  addChild(child: PIXI.DisplayObject): PIXI.DisplayObject;

  /**
   * Adds a child to the container at a specified index.
   *
   * @method addChildAt
   * @param {PIXI.DisplayObject} child The child to add
   * @param {Number} index The index to place the child in
   * @return {PIXI.DisplayObject} The child that was added
   */
  addChildAt(child: PIXI.DisplayObject, index: number): PIXI.DisplayObject;

  /**
   * Removes a child from the container.
   *
   * @method removeChild
   * @param {PIXI.DisplayObject} child The child to remove
   * @return {PIXI.DisplayObject} The child that was removed
   */
  removeChild(child: PIXI.DisplayObject): PIXI.DisplayObject;

  /**
   * Removes a child from the specified index position.
   *
   * @method removeChildAt
   * @param {Number} index The index to get the child from
   * @return {PIXI.DisplayObject} The child that was removed
   */
  removeChildAt(index: number): PIXI.DisplayObject;

  /**
   * Number of the created objects.
   *
   * @static
   * @property _counter
   * @type Number
   */
  static _counter: number;
}

declare class Stage extends PIXI.Container {
  /**
   * The root object of the display tree.
   *
   * @class Stage
   * @constructor
   */
  constructor();

  /**
   * Adds a child to the container.
   *
   * @method addChild
   * @param {PIXI.DisplayObject} child The child to add
   * @return {PIXI.DisplayObject} The child that was added
   */
  addChild(child: PIXI.DisplayObject): PIXI.DisplayObject;

  /**
   * Adds a child to the container at a specified index.
   *
   * @method addChildAt
   * @param {PIXI.DisplayObject} child The child to add
   * @param {Number} index The index to place the child in
   * @return {PIXI.DisplayObject} The child that was added
   */
  addChildAt(child: PIXI.DisplayObject, index: number): PIXI.DisplayObject;

  /**
   * Removes a child from the container.
   *
   * @method removeChild
   * @param {PIXI.DisplayObject} child The child to remove
   * @return {PIXI.DisplayObject} The child that was removed
   */
  removeChild(child: PIXI.DisplayObject): PIXI.DisplayObject;

  /**
   * Removes a child from the specified index position.
   *
   * @method removeChildAt
   * @param {Number} index The index to get the child from
   * @return {PIXI.DisplayObject} The child that was removed
   */
  removeChildAt(index: number): PIXI.DisplayObject;
}

declare namespace PIXI {
  declare class TilingSprite extends PIXI.Sprite {
    /**
     * A tiling sprite is a fast way of rendering a tiling image
     *
     * @class
     * @extends Sprite
     * @memberof PIXI.extras
     * @param texture {Texture} the texture of the tiling sprite
     * @param width {Float} [width=100] the width of the tiling sprite
     * @param height {Float} [height=100] the height of the tiling sprite
     */
    constructor(texture: PIXI.Texture, width?: number, height?: number);

    /**
     * Tile transform
     *
     * @member {Transform}
     */
    tileTransform: PIXI.Transform;
    /**
     * Whether or not anchor affects uvs
     *
     * @member {Bool}
     * @default false
     */
    uvRespectAnchor: boolean;
    /**
     * The scaling of the image that is being tiled
     *
     * @member {ObservablePoint}
     */
    tileScale: PIXI.ObservablePoint;
    /**
     * The offset position of the image that is being tiled
     *
     * @member {ObservablePoint}
     */
    tilePosition: PIXI.ObservablePoint;
    /**
     * Changes frame clamping in corresponding textureTransform, shortcut
     * Change to -0.5 to add a pixel to the edge, recommended for transparent trimmed textures in atlas
     * @default 0.5
     */
    clampMargin: number;

    /**
     * Helper function that creates a new sprite based on the source you provide.
     * The source can be - frame id, image url, video url, canvas element, video element, base texture
     *
     * @static
     * @param {Int|String|BaseTexture|HTMLCanvasElement|HTMLVideoElement} source Source to create texture from
     * @param {Float} width - the width of the tiling sprite
     * @param {Float} height - the height of the tiling sprite
     * @return {TilingSprite} The newly created sprite
     */
    static from(source: any, width: number, height: number): PIXI.TilingSprite;

    /**
     * Helper function that creates a tiling sprite that will use a texture from the TextureCache based on the frameId
     * The frame ids are created when a Texture packer file has been loaded
     *
     * @static
     * @param {String} frameId - The frame Id of the texture in the cache
     * @param {Float} width - the width of the tiling sprite
     * @param {Float} height - the height of the tiling sprite
     * @return {TilingSprite} A new TilingSprite using a texture from the texture cache matching the frameId
     */
    static fromFrame(frameId: string, width: number, height: number): PIXI.TilingSprite;

    /**
     * Helper function that creates a sprite that will contain a texture based on an image url
     * If the image is not in the texture cache it will be loaded
     *
     * @static
     * @param {String} imageId - The image url of the texture
     * @param {Float} width - the width of the tiling sprite
     * @param {Float} height - the height of the tiling sprite
     * @param {Bool} [crossorigin] - if you want to specify the cross-origin parameter
     * @param {ScaleModes} [scaleMode=PIXI.settings.SCALE_MODE] - if you want to specify the scale mode,
     *  see {@link PIXI.SCALE_MODES} for possible values
     * @return {TilingSprite} A new TilingSprite using a texture from the texture cache matching the image id
     */
    static fromImage(
      imageId: string,
      width: number,
      height: number,
      crossorigin?: boolean,
      scaleMode?: any,
    ): PIXI.TilingSprite;
  }
}

declare class TilingSprite extends PIXI.TilingSprite {
  /**
   * The sprite object for a tiling image.
   *
   * @class TilingSprite
   * @constructor
   * @param {Bitmap} bitmap The image for the tiling sprite
   */
  constructor(bitmap: Bitmap);

  /**
   * The origin point of the tiling sprite for scrolling.
   *
   * @property origin
   * @type Point
   */
  origin: Point;
  /**
   * The image for the tiling sprite.
   *
   * @property bitmap
   * @type Bitmap
   */
  bitmap: Bitmap;
  /**
   * The opacity of the tiling sprite (0 to 255).
   *
   * @property opacity
   * @type Number
   */
  opacity: number;
  /**
   * The visibility of the tiling sprite.
   *
   * @property visible
   * @type Boolean
   */
  visibility: boolean;
  /**
   * The x coordinate of the tiling sprite.
   *
   * @property x
   * @type Number
   */
  x: number;
  /**
   * The y coordinate of the tiling sprite.
   *
   * @property y
   * @type Number
   */
  y: number;
  spriteId: number;

  /**
   * Updates the tiling sprite for each frame.
   *
   * @method update
   */
  update(): void;

  /**
   * @method updateTransform
   * @private
   */
  updateTransform(): void;

  /**
   * Sets the x, y, width, and height all at once.
   *
   * @method move
   * @param {Number} x The x coordinate of the tiling sprite
   * @param {Number} y The y coordinate of the tiling sprite
   * @param {Number} width The width of the tiling sprite
   * @param {Number} height The height of the tiling sprite
   */
  move(x?: number, y?: number, width?: number, height?: number): void;

  /**
   * Specifies the region of the image that the tiling sprite will use.
   *
   * @method setFrame
   * @param {Number} x The x coordinate of the frame
   * @param {Number} y The y coordinate of the frame
   * @param {Number} width The width of the frame
   * @param {Number} height The height of the frame
   */
  setFrame(x: number, y: number, width: number, height: number): void;

  updateTransformTS(): void;

  /**
   * @method generateTilingTexture
   * @param {Boolean} arg
   */
  generateTilingTexture(arg: boolean): void;
}

declare namespace PIXI {
  declare class Filter {
    /**
     * @class
     * @memberof PIXI
     * @param [vertexSrc] {String} The source of the vertex shader.
     * @param [fragmentSrc] {String} The source of the fragment shader.
     * @param [uniforms] {Dynamic} Custom uniforms to use to augment the built-in ones.
     */
    constructor(vertexSrc?: string, fragmentSrc?: string, uniforms?: any);

    /**
     * Legacy filters use position and uvs from attributes
     */
    readonly legacy: boolean;
    /**
     * Sets the blendmode of the filter. Default: PIXI.BLEND_MODES.NORMAL
     */
    blendMode: any;
    /**
     * Program that the shader uses
     */
    program: PIXI.Program;
    /**
     * The WebGL state the filter requires to render
     */
    state: PIXI.State;
    /**
     * The vertex shader.
     *
     * @member {String}
     */
    vertexSrc: string;
    /**
     * The fragment shader.
     *
     * @member {String}
     */
    fragmentSrc: string;
    /**
     * An object containing the current values of custom uniforms.
     * @example <caption>Updating the value of a custom uniform</caption>
     * filter.uniforms.time = performance.now();
     *
     * @member {Dynamic}
     */
    uniforms: any;
    /**
     * The padding of the filter. Some filters require extra space to breath such as a blur.
     * Increasing this will add extra width and height to the bounds of the object that the
     * filter is applied to.
     *
     * @member {Float}
     */
    padding: number;
    /**
     * The resolution of the filter. Setting this to be lower will lower the quality but
     * increase the performance of the filter.
     *
     * @member {Float}
     */
    resolution: number;
    /**
     * If enabled is true the filter is applied, if false it will not.
     *
     * @member {Bool}
     */
    enabled: boolean;
    /**
     * If enabled, PixiJS will fit the filter area into boundaries for better performance. Switch it off if it does not work for specific shader.
     * Workaround for http://jsfiddle.net/xbmhh207/1/
     * @default true
     * @member {Bool}
     */
    autoFit: boolean;

    apply(
      filterManager: PIXI.systems.FilterSystem,
      input: PIXI.RenderTexture,
      output: PIXI.RenderTexture,
      clear?: boolean,
      currentState?: pixi.core.renderers.webgl.filters.CurrentState,
    ): void;

    /**
     * The default vertex shader source
     *
     * @static
     * @constant
     */
    static defaultVertexSrc: any;
    /**
     * The default fragment shader source
     *
     * @static
     * @constant
     */
    static defaultFragmentSrc: any;
  }
}

declare namespace PIXI.filters {
  declare class ColorMatrixFilter extends PIXI.Filter {
    /**
     * The ColorMatrixFilter class lets you apply a 5x5 matrix transformation on the RGBA
     * color and alpha values of every pixel on your displayObject to produce a result
     * with a new set of RGBA color and alpha values. It's pretty powerful!
     *
     * ```js
     *  var colorMatrix = new ColorMatrixFilter();
     *  container.filters = [colorMatrix];
     *  colorMatrix.contrast(2);
     * ```
     * @author Clément Chenebault <clement@goodboydigital.com>
     * @class
     * @extends AbstractFilter
     * @memberof PIXI.filters
     */
    constructor();

    /**
     * Sets the matrix of the color matrix filter
     *
     * @member {number[]}
     * @memberof ColorMatrixFilter#
     * @default [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1]
     */
    matrix: number[];

    /**
     * Adjusts brightness
     *
     * Multiply the current matrix
     * @param b {Float} value of the brigthness (0 is black)
     * @param multiply {Bool} refer to ._loadMatrix() method
     */
    brightness(b: number, multiply: boolean): void;

    /**
     * Set the matrices in grey scales
     *
     * Multiply the current matrix
     * @param scale {Float} value of the grey (0 is black)
     * @param multiply {Bool} refer to ._loadMatrix() method
     */
    greyscale(scale: number, multiply: boolean): void;

    /**
     * Set the black and white matrice
     * Multiply the current matrix
     *
     * @param multiply {Bool} refer to ._loadMatrix() method
     */
    blackAndWhite(multiply: boolean): void;

    /**
     * Set the hue propertie of the color
     *
     * Multiply the current matrix
     * @param rotation {Float} in degrees
     * @param multiply {Bool} refer to ._loadMatrix() method
     */
    hue(rotation: number, multiply: boolean): void;

    /**
     * Set the contrast matrix, increase the separation between dark and bright
     * Increase contrast : shadows darker and highlights brighter
     * Decrease contrast : bring the shadows up and the highlights down
     *
     * @param amount {Float} value of the contrast
     * @param multiply {Bool} refer to ._loadMatrix() method
     */
    contrast(amount: number, multiply: boolean): void;

    /**
     * Set the saturation matrix, increase the separation between colors
     * Increase saturation : increase contrast, brightness, and sharpness
     * @param amount {Float}
     * @param multiply {Bool} refer to ._loadMatrix() method
     */
    saturation(amount: number, multiply: boolean): void;

    /**
     * Desaturate image (remove color)
     *
     * Call the saturate function
     *
     * @param multiply {Bool} refer to ._loadMatrix() method
     */
    desaturate(multiply: boolean): void;

    /**
     * Negative image (inverse of classic rgb matrix)
     *
     * @param multiply {Bool} refer to ._loadMatrix() method
     */
    negative(multiply: boolean): void;

    /**
     * Sepia image
     *
     * @param multiply {Bool} refer to ._loadMatrix() method
     */
    sepia(multiply: boolean): void;

    /**
     * Color motion picture process invented in 1916 (thanks Dominic Szablewski)
     *
     * @param multiply {Bool} refer to ._loadMatrix() method
     */
    technicolor(multiply: boolean): void;

    /**
     * Polaroid filter
     *
     * @param multiply {boolean} refer to ._loadMatrix() method
     */
    polaroid(multiply: boolean): void;

    /**
     * Filter who transforms : Red -> Blue and Blue -> Red
     *
     * @param multiply {Bool} refer to ._loadMatrix() method
     */
    toBGR(multiply: boolean): void;

    /**
     * Color reversal film introduced by Eastman Kodak in 1935. (thanks Dominic Szablewski)
     *
     * @param multiply {Bool} refer to ._loadMatrix() method
     */
    kodachrome(multiply: boolean): void;

    /**
     * Brown delicious browni filter (thanks Dominic Szablewski)
     *
     * @param multiply {Bool} refer to ._loadMatrix() method
     */
    browni(multiply: boolean): void;

    vintage(multiply: boolean): void;

    colorTone(desaturation: number, toned: number, lightColor: number, darkColor: number, multiply: boolean): void;

    night(intensity: number, multiply: boolean): void;

    predator(amount: number, multiply: boolean): void;

    lsd(multiply: boolean): void;

    reset(): void;
  }
}

declare class ToneFilter extends PIXI.filters.ColorMatrixFilter {
  /**
   * The color matrix filter for WebGL.
   *
   * @class ToneFilter
   * @extends PIXI.Filter
   * @constructor
   */
  constructor();

  /**
   * Changes the hue.
   *
   * @method adjustHue
   * @param {Number} value The hue value in the range (-360, 360)
   */
  adjustHue(value?: number): void;

  /**
   * Changes the saturation.
   *
   * @method adjustSaturation
   * @param {Number} value The saturation value in the range (-255, 255)
   */
  adjustSaturation(value?: number): void;

  /**
   * Changes the tone.
   *
   * @method adjustTone
   * @param {Number} r The red strength in the range (-255, 255)
   * @param {Number} g The green strength in the range (-255, 255)
   * @param {Number} b The blue strength in the range (-255, 255)
   */
  adjustTone(r?: number, g?: number, b?: number): void;
}

declare class ToneSprite extends PIXI.Container {
  /**
   * The sprite which changes the screen color in 2D canvas mode.
   *
   * @class ToneSprite
   * @constructor
   */
  constructor();

  /**
   * Clears the tone.
   *
   * @method reset
   */
  clear(): void;

  /**
   * Sets the tone.
   *
   * @method setTone
   * @param {Number} r The red strength in the range (-255, 255)
   * @param {Number} g The green strength in the range (-255, 255)
   * @param {Number} b The blue strength in the range (-255, 255)
   * @param {Number} gray The grayscale level in the range (0, 255)
   */
  setTone(r: number, g: number, b: number, gray: number): void;
}

declare namespace rm.types {
  export const enum Frames {
    base = -1,
  }
}

declare class TouchInput {
  static _onTouchStart: any;

  protected constructor();

  static _mousePressed: boolean;
  static _screenPressed: boolean;
  static _pressedTime: number;
  static _date: number;
  static _x: number;
  static _y: number;
  /**
   * The wait time of the pseudo key repeat in frames.
   *
   * @static
   * @property keyRepeatWait
   * @type Number
   */
  static keyRepeatWait: rm.types.Frames;
  /**
   * The interval of the pseudo key repeat in frames.
   *
   * @static
   * @property keyRepeatInterval
   * @type Number
   */
  static keyRepeatInterval: rm.types.Frames;
  /**
   * [read-only] The horizontal scroll amount.
   *
   * @static
   * @property wheelX
   * @type Number
   */
  static wheelX: number;
  /**
   * [read-only] The vertical scroll amount.
   *
   * @static
   * @property wheelY
   * @type Number
   */
  static wheelY: number;
  /**
   * [read-only] The x coordinate on the canvas area of the latest touch event.
   *
   * @static
   * @property x
   * @type Number
   */
  static x: number;
  /**
   * [read-only] The y coordinate on the canvas area of the latest touch event.
   *
   * @static
   * @property y
   * @type Number
   */
  static y: number;
  /**
   * [read-only] The time of the last input in milliseconds.
   *
   * @static
   * @property date
   * @type Number
   */
  static date: number;

  /**
   * Initializes the touch system.
   *
   * @static
   * @method initialize
   */
  static initialize(): void;

  /**
   * Clears all the touch data.
   *
   * @static
   * @method clear
   */
  static clear(): void;

  /**
   * Updates the touch data.
   *
   * @static
   * @method update
   */
  static update(): void;

  /**
   * Checks whether the mouse button or touchscreen is currently pressed down.
   *
   * @static
   * @method isPressed
   * @return {Boolean} True if the mouse button or touchscreen is pressed
   */
  static isPressed(): boolean;

  /**
   * Checks whether the left mouse button or touchscreen is just pressed.
   *
   * @static
   * @method isTriggered
   * @return {Boolean} True if the mouse button or touchscreen is triggered
   */
  static isTriggered(): boolean;

  /**
   * Checks whether the left mouse button or touchscreen is just pressed
   * or a pseudo key repeat occurred.
   *
   * @static
   * @method isRepeated
   * @return {Boolean} True if the mouse button or touchscreen is repeated
   */
  static isRepeated(): boolean;

  /**
   * Checks whether the left mouse button or touchscreen is kept depressed.
   *
   * @static
   * @method isLongPressed
   * @return {Boolean} True if the left mouse button or touchscreen is long-pressed
   */
  static isLongPressed(): boolean;

  /**
   * Checks whether the right mouse button is just pressed.
   *
   * @static
   * @method isCancelled
   * @return {Boolean} True if the right mouse button is just pressed
   */
  static isCancelled(): boolean;

  /**
   * Checks whether the mouse or a finger on the touchscreen is moved.
   *
   * @static
   * @method isMoved
   * @return {Boolean} True if the mouse or a finger on the touchscreen is moved
   */
  static isMoved(): boolean;

  /**
   * Checks whether the left mouse button or touchscreen is released.
   *
   * @static
   * @method isReleased
   * @return {Boolean} True if the mouse button or touchscreen is released
   */
  static isReleased(): boolean;
}

declare class Utils {
  protected constructor();

  /**
   * The name of the RPG Maker. 'MV' in the current version.
   */
  static RPGMAKER_NAME: string;
  /**
   * The version of the RPG Maker.
   *
   * @static
   * @property RPGMAKER_VERSION
   * @type String
   * @final
   */
  static RPGMAKER_VERSION: string;

  /**
   * Checks whether the option is in the query string.
   *
   * @param {String} name The option name
   * @return {Boolean} True if the option is in the query string
   */
  static isOptionValid(name: string): boolean;

  /**
   * Checks whether the platform is NW.js.
   *
   * @return {Boolean} True if the platform is NW.js
   */
  static isNwjs(): boolean;

  /**
   * Checks whether the platform is a mobile device.
   *
   * @static
   * @method isMobileDevice
   * @return {Boolean} True if the platform is a mobile device
   */
  static isMobileDevice(): boolean;

  /**
   * Checks whether the browser is Mobile Safari.
   *
   * @static
   * @method isMobileSafari
   * @return {Boolean} True if the browser is Mobile Safari
   */
  static isMobileSafari(): boolean;

  /**
   * Checks whether the browser is Android Chrome.
   *
   * @return {Boolean} True if the browser is Android Chrome
   */
  static isAndroidChrome(): boolean;

  /**
   * Checks whether the browser can read files in the game folder.
   *
   * @return {Boolean} True if the browser can read files in the game folder
   */
  static canReadGameFiles(): boolean;

  /**
   * Makes a CSS color string from RGB values.
   *
   * @static
   * @method rgbToCssColor
   * @param {Int} r The red value in the range (0, 255)
   * @param {Int} g The green value in the range (0, 255)
   * @param {Int} b The blue value in the range (0, 255)
   * @return {String} CSS color string
   */
  static rgbToCssColor(r: number, g: number, b: number): string;

  static isSupportPassiveEvent(): boolean;

  static generateRuntimeId(): number;

  /**
   * Encodes a URI component without escaping slash characters.
   *
   * @param {string} str - The input string.
   * @returns {string} Encoded string.
   */
  static encodeURI(str: string): string;
}

declare class Weather extends PIXI.Container {
  /**
   * The weather effect which displays rain, storm, or snow.
   *
   * @class Weather
   * @constructor
   */
  constructor();

  /**
   * The type of the weather in ['none', 'rain', 'storm', 'snow'].
   *
   * @property type
   * @type String
   */
  type: string;
  /**
   * The power of the weather in the range (0, 9).
   *
   * @property power
   * @type Number
   */
  power: number;
  /**
   * The origin point of the weather for scrolling.
   *
   * @property origin
   * @type Point
   */
  origin: Point;

  /**
   * Updates the weather for each frame.
   *
   * @method update
   */
  update(): void;
}

declare class WindowLayer extends PIXI.Container {
  /**
   * The layer which contains game windows.
   *
   * @class WindowLayer
   * @constructor
   */
  constructor();

  /**
   * The width of the window layer in pixels.
   *
   * @property width
   * @type Number
   */
  voidFilter: PIXI.Filter;

  /**
   * Sets the x, y, width, and height all at once.
   *
   * @method move
   * @param {Number} x The x coordinate of the window layer
   * @param {Number} y The y coordinate of the window layer
   * @param {Number} width The width of the window layer
   * @param {Number} height The height of the window layer
   */
  move(x: number, y: number, width: number, height: number): void;

  /**
   * Updates the window layer for each frame.
   *
   * @method update
   */
  update(): void;

  /**
   * Adds a child to the container.
   *
   * @method addChild
   * @param {PIXI.DisplayObject} child The child to add
   * @return {PIXI.DisplayObject} The child that was added
   */
  addChild(child: PIXI.DisplayObject): PIXI.DisplayObject;

  /**
   * Adds a child to the container at a specified index.
   *
   * @method addChildAt
   * @param {PIXI.DisplayObject} child The child to add
   * @param {Number} index The index to place the child in
   * @return {PIXI.DisplayObject} The child that was added
   */
  addChildAt(child: PIXI.DisplayObject, index: number): PIXI.DisplayObject;

  /**
   * Removes a child from the container.
   *
   * @method removeChild
   * @param {PIXI.DisplayObject} child The child to remove
   * @return {PIXI.DisplayObject} The child that was removed
   */
  removeChild(child: PIXI.DisplayObject): PIXI.DisplayObject;

  /**
   * Removes a child from the specified index position.
   *
   * @method removeChildAt
   * @param {Number} index The index to get the child from
   * @return {PIXI.DisplayObject} The child that was removed
   */
  removeChildAt(index: number): PIXI.DisplayObject;

  /**
   * @method _renderCanvas
   * @param {PIXI.CanvasRenderer} renderer
   * @private
   */
  _renderCanvas(renderer: PIXI.CanvasRenderer): void;

  /**
   * @method _renderWebGL
   * @param {PIXI.WebGLRenderer} renderer
   * @private
   */
  _renderWebGL(renderer: PIXI.Renderer): void;
}

declare class Window extends PIXI.Container {
  /**
   * The visibility of the sprite.
   *
   * @property visible
   * @type Boolean
   */
  constructor();

  /**
   * The origin point of the window for scrolling.
   *
   * @property origin
   * @type Point
   */
  origin: PIXI.Point;
  /**
   * The active state for the window.
   *
   * @property active
   * @type Boolean
   */
  active: boolean;
  /**
   * The visibility of the down scroll arrow.
   *
   * @property downArrowVisible
   * @type Boolean
   */
  downArrowVisible: boolean;
  /**
   * The visibility of the up scroll arrow.
   *
   * @property upArrowVisible
   * @type Boolean
   */
  upArrowVisible: boolean;
  /**
   * The visibility of the pause sign.
   *
   * @property pause
   * @type Boolean
   */
  pause: boolean;
  /**
   * The image used as a window skin.
   *
   * @property windowskin
   * @type Bitmap
   */
  windowskin: Bitmap;
  /**
   * The bitmap used for the window contents.
   *
   * @property contents
   * @type Bitmap
   */
  contents: Bitmap;
  /**
   * The width of the window in pixels.
   *
   * @property width
   * @type Number
   */
  padding: number;
  /**
   * The size of the margin for the window background.
   *
   * @property margin
   * @type Number
   */
  margin: number;
  /**
   * The opacity of the window without contents (0 to 255).
   *
   * @property opacity
   * @type Number
   */
  opacity: number;
  /**
   * The opacity of the window background (0 to 255).
   *
   * @property backOpacity
   * @type Number
   */
  backOpacity: number;
  /**
   * The opacity of the window contents (0 to 255).
   *
   * @property contentsOpacity
   * @type Number
   */
  contentsOpacity: number;
  /**
   * The openness of the window (0 to 255).
   *
   * @property openness
   * @type Number
   */
  openness: number;
  /**
   * The width of the content area in pixels.
   */
  innerWidth: number;
  /**
   * The height of the content area in pixels.
   */
  innerHeight: number;
  /**
   * The rectangle of the content area
   */
  innerRect: Rectangle;

  /**
   * Updates the window for each frame.
   *
   * @method update
   */
  update(): void;

  /**
   * Sets the x, y, width, and height all at once.
   *
   * @method move
   * @param {Number} x The x coordinate of the window
   * @param {Number} y The y coordinate of the window
   * @param {Number} width The width of the window
   * @param {Number} height The height of the window
   */
  move(x?: number, y?: number, width?: number, height?: number): void;

  /**
   * Returns true if the window is completely open (openness == 255).
   *
   * @method isOpen
   * @return {Boolean}
   */
  isOpen(): boolean;

  /**
   * Returns true if the window is completely closed (openness == 0).
   *
   * @method isClosed
   * @return {Boolean}
   */
  isClosed(): boolean;

  /**
   * Sets the position of the command cursor.
   *
   * @method setCursorRect
   * @param {Number} x The x coordinate of the cursor
   * @param {Number} y The y coordinate of the cursor
   * @param {Number} width The width of the cursor
   * @param {Number} height The height of the cursor
   */
  setCursorRect(x?: number, y?: number, width?: number, height?: number): void;

  /**
   * Changes the color of the background.
   *
   * @method setTone
   * @param {Number} r The red value in the range (-255, 255)
   * @param {Number} g The green value in the range (-255, 255)
   * @param {Number} b The blue value in the range (-255, 255)
   */
  setTone(r: number, g: number, b: number): void;

  /**
   * Adds a child between the background and contents.
   *
   * @method addChildToBack
   * @param {PIXI.DisplayObject} child The child to add
   * @return {PIXI.DisplayObject} The child that was added
   */
  addChildToBack(child: PIXI.DisplayObject): PIXI.DisplayObject;

  /**
   * Adds a child to the container.
   *
   * @method addChild
   * @param {PIXI.DisplayObject} child The child to add
   * @return {PIXI.DisplayObject} The child that was added
   */
  addChild(child: PIXI.DisplayObject): PIXI.DisplayObject;

  /**
   * Adds a child to the container at a specified index.
   *
   * @method addChildAt
   * @param {PIXI.DisplayObject} child The child to add
   * @param {Number} index The index to place the child in
   * @return {PIXI.DisplayObject} The child that was added
   */
  addChildAt(child: PIXI.DisplayObject, index: number): PIXI.DisplayObject;

  /**
   * Removes a child from the container.
   *
   * @method removeChild
   * @param {PIXI.DisplayObject} child The child to remove
   * @return {PIXI.DisplayObject} The child that was removed
   */
  removeChild(child: PIXI.DisplayObject): PIXI.DisplayObject;

  /**
   * Removes a child from the specified index position.
   *
   * @method removeChildAt
   * @param {Number} index The index to get the child from
   * @return {PIXI.DisplayObject} The child that was removed
   */
  removeChildAt(index: number): PIXI.DisplayObject;

  /**
   * Move the cursor position by the given amount
   * @param x
   * @param y
   */
  moveCursorBy(x: number, y: number): void;

  /**
   * Moves the inner children by the given amount.
   * @param x
   * @param y
   */
  moveInnerChildrenBy(x: number, y: number): void;

  /**
   * Adds a child to the client area.
   * @param child
   * @return DisplayObject
   */
  addInnerChild(child: PIXI.DisplayObject): PIXI.DisplayObject;

  /**
   * Draws the window shape into PIXI.Graphics object. Used by WindowLayer.
   * @param graphics
   */
  drawShape(graphics: PIXI.Graphics): void;

  /**
   * Destroys the window
   */
  destroy(): void;

  /**
   * @method updateTransform
   * @private
   */
  updateTransform(): void;
}

declare namespace rm.types {
  export type AudioParameters = {
    name: string;
    pan: number;
    pitch: number;
    pos: number;
    volume: number;
  };
}

/**
 * Static class that handles BGM, BGS, ME, and SE.
 */
declare class AudioManager {
  protected constructor();

  static _bgmVolume: number;
  static _bgsVolume: number;
  static _meVolume: number;
  static _seVolume: number;
  static _currentBgm: rm.types.AudioParameters;
  static _currentBgs: rm.types.AudioParameters;
  static _bgmBuffer: HTML5Audio;
  static _bgsBuffer: HTML5Audio;
  static _meBuffer: HTML5Audio;
  static _seBuffers: HTML5Audio[];
  static _staticBuffers: HTML5Audio[];
  static _replayFadeTime: number;
  static _path: string;
  static _blobUrl: string;
  static bgmVolume: number;
  static bgsVolume: number;
  static meVolume: number;
  static seVolume: number;

  static playBgm(bgm: rm.types.AudioParameters, pos?: number): void;

  static replayBgm(bgm: rm.types.AudioParameters): void;

  static isCurrentBgm(bgm: rm.types.AudioParameters): boolean;

  static updateBgmParameters(bgm: rm.types.AudioParameters): void;

  static pdateCurrentBgm(bgm: rm.types.AudioParameters, pos: number): void;

  static stopBgm(): void;

  static fadeOutBgm(duration: number): void;

  static fadeInBgm(duration: number): void;

  static playBgs(bgs: rm.types.AudioParameters, pos?: number): void;

  static replayBgs(bgs: rm.types.AudioParameters): void;

  static isCurrentBgs(bgs: rm.types.AudioParameters): boolean;

  static updateBgsParameters(bgs: rm.types.AudioParameters): void;

  static updateCurrentBgs(bgs: rm.types.AudioParameters, pos: number): void;

  static stopBgs(): void;

  static fadeOutBgs(duration: number): void;

  static fadeInBgs(duration: number): void;

  static playMe(me: rm.types.AudioParameters): void;

  static updateMeParameters(me: rm.types.AudioParameters): void;

  static fadeOutMe(duration: number): void;

  static stopMe(): void;

  static playSe(se: rm.types.AudioParameters): void;

  static updateSeParameters(buffer: rm.types.AudioParameters, se: rm.types.AudioParameters): void;

  static stopSe(): void;

  static playStaticSe(se: rm.types.AudioParameters): void;

  static loadStaticSe(se: rm.types.AudioParameters): void;

  static isStaticSe(se: rm.types.AudioParameters): boolean;

  static stopAll(): void;

  static saveBgm(): rm.types.AudioParameters;

  static saveBgs(): rm.types.AudioParameters;

  static makeEmptyAudioObject(): rm.types.AudioParameters;

  static createBuffer(): HTML5Audio;

  static updateBufferParameters(
    buffer: rm.types.AudioParameters,
    configVolume: number,
    audio: rm.types.AudioParameters,
  ): void;

  static audioFileExt(): string;

  static shouldUseHtml5Audio(): string;

  static checkErrors(): void;

  static checkWebAudioError(webAudio?: HTML5Audio): void;

  static playEncryptedBgm(bgm: rm.types.AudioParameters, pos?: number): void;

  static createDecryptBuffer(url: string, bgm: rm.types.AudioParameters, pos?: number): void;
}

declare namespace rm.types {
  /**
   * A superclass of actor, class, skill, item, weapon, armor, enemy, and state.
   *
   * Some items are unnecessary depending on the type of data, but they are included for convenience sake.
   */
  export type BaseItem = {
    /**
     * The description text.
     */
    description: string;
    /**
     * The icon number.
     */
    iconIndex: number;
    /**
     * The item ID.
     */
    id: number;
    meta: Object;
    /**
     * The item name.
     */
    name: string;
    note: string;
  };
}

declare namespace rm.types {
  export type BattleRewards = {
    exp: number;
    gold: number;
    items: rm.types.BaseItem[];
  };
}

declare namespace rm.types {
  export const enum BattleResult {
    base = -1,
  }
}

/**
 * Manages the game battles to process all of the
 * events that happen within.
 */
declare class BattleManager {
  protected constructor();

  _phase: string;
  _canEscape: boolean;
  _canLose: boolean;
  _battleTest: boolean;
  _eventCallback: () => void;
  _preemptive: boolean;
  _surprise: boolean;
  _actorIndex: number;
  _actionForceBattler: Game_Battler;
  _mapBgm: rm.types.AudioParameters;
  _mapBgs: rm.types.AudioParameters;
  _actionBattlers: Game_Battler[];
  _subject: Game_Battler;
  _action: Game_Action;
  _targets: Game_Battler[];
  _logWindow: Window_BattleLog;
  _statusWindow: Window_BattleStatus;
  _spriteset: Spriteset_Battle;
  _escapeRatio: number;
  _escaped: boolean;
  _rewards: rm.types.BattleRewards;

  static setup(troopId: number, canEscape: boolean, canLose: boolean): void;

  static initMembers(): void;

  static isBattleTest(): boolean;

  static setBattleTest(battleTest: boolean): void;

  static setEventCallback(callback: () => void): void;

  static setLogWindow(logWindow: Window_BattleLog): void;

  static setStatusWindow(statusWindow: Window_BattleStatus): void;

  static setSpriteset(spriteset: Spriteset_Battle): void;

  static onEncounter(): void;

  static ratePreemptive(): number;

  static rateSurprise(): number;

  static saveBgmAndBgs(): void;

  static playBattleBgm(): void;

  static playVictoryMe(): void;

  static playDefeatMe(): void;

  static replayBgmAndBgs(): void;

  static makeEscapeRatio(): void;

  static update(): void;

  static updateEvent(): boolean;

  static updateEventMain(): boolean;

  static isBusy(): boolean;

  static isInputting(): boolean;

  static isInTurn(): boolean;

  static isTurnEnd(): boolean;

  static isAborting(): boolean;

  static isBattleEnd(): boolean;

  static canEscape(): boolean;

  static canLose(): boolean;

  static isEscaped(): boolean;

  static actor(): Game_Actor;

  static clearActor(): void;

  static changeActor(newActorIndex: number, lastActorActionState: string): void;

  static startBattle(): void;

  static displayStartMessages(): void;

  static startInput(): void;

  static inputtingAction(): Game_Action;

  static selectNextCommand(): void;

  static selectPreviousCommand(): void;

  static refreshStatus(): void;

  static startTurn(): void;

  static updateTurn(): void;

  static processTurn(): void;

  static endTurn(): void;

  static updateTurnEnd(): void;

  static getNextSubject(): Game_Battler;

  static allBattleMembers(): Game_Battler[];

  static makeActionOrders(): void;

  static startAction(): void;

  static updateAction(): void;

  static endAction(): void;

  static invokeAction(): void;

  static invokeNormalAction(): void;

  static invokeCounterAttack(): void;

  static invokeMagicReflection(): void;

  static applySubstitute(target: Game_Battler): Game_Battler;

  static checkSubstitute(target: Game_Battler): boolean;

  static isActionForced(): boolean;

  static forceAction(battler: Game_Battler): void;

  static processForcedAction(): void;

  static abort(): void;

  static checkBattleEnd(): boolean;

  static checkAbort(): boolean;

  static processVictory(): void;

  static processEscape(): boolean;

  static processAbort(): void;

  static processDefeat(): void;

  static endBattle(result: rm.types.BattleResult): void;

  static updateBattleEnd(): void;

  static makeRewards(): void;

  static displayVictoryMessage(): void;

  static displayDefeatMessage(): void;

  static displayEscapeSuccessMessage(): void;

  static displayEscapeFailureMessage(): void;

  static displayRewards(): void;

  static displayExp(): void;

  static displayGold(): void;

  static displayDropItems(): void;

  static gainRewards(): void;

  static gainExp(): void;

  static gainGold(): void;

  static gainDropItems(): void;
}

declare namespace rm.types {}

declare namespace rm.types {
  export const enum DamageColorType {
    HP_DAMAGE = 0,
    HP_RECOVER = 1,
    MP_DAMAGE = 2,
    MP_RECOVER = 3,
    DEFAULT = 4,
  }
}

declare class ColorManager {
  protected constructor();

  _windowSkin: Bitmap;

  /**
   * Loads the window skin into ColorManager.
   */
  static loadWindowSkin(): void;

  /**
   * Loads the text color using the window skin color Index.
   * @param windowSkinColorIndex
   * @returns Color
   */
  static textColor(windowSkinColorIndex: number): rm.types.Color;

  /**
   * Uses the normal color, index 0
   * of the window skin.
   */
  static normalColor(): rm.types.Color;

  /**
   * Uses the system color; index 16 on the window skin.
   * @return Color
   */
  static systemColor(): rm.types.Color;

  /**
   * Uses the crisis color; index 17 on the window skin.
   * @return Color
   */
  static crisisColor(): rm.types.Color;

  /**
   * Uses the death color; index 18 on the window skin.
   * @return Color
   */
  static deathColor(): rm.types.Color;

  /**
   * Uses the  gauge back color; index 19 on the window skin.
   * @return Color
   */
  static gaugeBackColor(): rm.types.Color;

  /**
   * Uses the hp gauge color 1. Index 20 on the window skin.
   * @return Color
   */
  static hpGaugeColor1(): rm.types.Color;

  /**
   * Uses the hp guage color 2. Index 21 on the window skin.
   * @return Color
   */
  static hpGaugeColor2(): rm.types.Color;

  /**
   * Uses the mp gauge color 1. Index 22 on the window skin.
   * @return Color
   */
  static mpGaugeColor1(): rm.types.Color;

  /**
   * Uses the mp gauge color 2. Index 23 on the window skin.
   * @return Color
   */
  static mpGaugeColor2(): rm.types.Color;

  /**
   * Uses the mp cost color. Index 23 on the window skin.
   * @return Color
   */
  static mpCostColor(): rm.types.Color;

  /**
   * Uses the power up color. Index 24 on the window skin.
   * @return Color
   */
  static powerUpColor(): rm.types.Color;

  /**
   * Uses the power down color. Index 25 on the window skin.
   * @return Color
   */
  static powerDownColor(): rm.types.Color;

  /**
   * Uses the ct gauge color 1. Index 26 on the window skin.
   * @return Color
   */
  static ctGaugeColor1(): rm.types.Color;

  /**
   * Uses the ct gauge color 2. Index 27 on the window skin.
   * @return Color
   */
  static ctGaugeColor2(): rm.types.Color;

  /**
   * Uses the tp gauge color 1. Index 28 on the window skin.
   * @return Color
   */
  static tpGaugeColor1(): rm.types.Color;

  /**
   * Uses the tp gauge color 2. Index 29 on the window skin.
   * @return Color
   */
  static tpGaugeColor2(): rm.types.Color;

  /**
   * Uses the tp cost color. Index 29 on the window skin.
   * @return Color
   */
  static tpCostColor(): rm.types.Color;

  /**
   * Uses the pending color.
   * @return Color
   */
  static pendingColor(): rm.types.Color;

  /**
   * Given the actor status the color is different.
   * Alive -> Normal Color
   * Dying -> Crisis Color
   * Dead -> Death Color
   * If it's not an actor, it returns the normal color.
   * @param actor
   * @return Color
   */
  static hpColor(actor: Game_Actor): rm.types.Color;

  /**
   * Based on actor; currently returns normal color.
   * @return Color
   */
  static mpColor(): rm.types.Color;

  /**
   * Based on actor; currently returns normal color.
   * @return Color
   */
  static tpColor(): rm.types.Color;

  /**
   * This returns the text color based on change.
   * change > 0 -> Power Up Color
   * change < 0 -> Power Down Color
   * else -> Normal color.
   * @param change
   * @return Color
   */
  static paramChangeTextColor(change: number): rm.types.Color;

  /**
   * Returns the color based on Color Type.
   * 0 -> HP Damage Color
   * 1 -> HP Recover Color
   * 2 -> MP Damage Color
   * 3 -> MP Recover Color
   * Number -> Default Color
   * @param colorType
   * @return Color
   */
  static damageColor(colorType: rm.types.DamageColorType): rm.types.Color;

  /**
   * Returns the color "rgba(0, 0, 0, 0.6)"
   * @return Color
   */
  static outlineColor(): rm.types.Color;

  /**
   * Returns the color "rgba(0, 0, 0, 0.6)"
   * @return Color
   */
  static dimColor1(): rm.types.Color;

  /**
   * Returns the color "rgba(0, 0, 0, 0)"
   * @return Color
   */
  static dimColor2(): rm.types.Color;

  /**
   * Returns the color "rgba(32, 32, 32, 0.5)"
   * @return Color
   */
  static itemBackColor1(): rm.types.Color;

  /**
   * Returns the color "rgba(0, 0, 0, 0.5)"
   * @return Color
   */
  static itemBackColor2(): rm.types.Color;
}

declare namespace rm.types {
  export type ConfigData = {
    alwaysDash: boolean;
    bgmVolume: number;
    bgsVolume: number;
    commandRemember: boolean;
    meVolume: number;
    seVolume: number;
  };
}

/**
 * Manages the configuration of RPGMakerMZ Config Data.
 */
declare class ConfigManager {
  protected constructor();

  static load(): void;

  static save(): void;

  static makeData(): rm.types.ConfigData;

  static applyData(config: rm.types.ConfigData): void;

  static readFlag(config: rm.types.ConfigData, name: string): boolean;

  static readVolume(config: rm.types.ConfigData, name: string): number;
}

/**
 * Manages the game data: saving, loading, meta data,
 * and map information.
 */
declare class DataManager {
  protected constructor();

  static _globalId: string;
  static _lastAccessId: number;
  static _errorUrl: any;
  static _databaseFiles: JSON[];

  static loadDatabase(): void;

  static loadDataFile(name: string, src: string): void;

  static isDatabaseLoaded(): boolean;

  static loadMapData(mapId: number): void;

  static makeEmptyMap(): void;

  static isMapLoaded(): boolean;

  static onLoad(object: any): void;

  /**
   * Extracts Meta Data. A Json Like Object
   * @param data
   */
  static extractMetaData(data: any): void;

  static checkError(): void;

  static isBattleTest(): boolean;

  static isEventTest(): boolean;

  /**
   * Returns a Bool value given an item.
   * Item being a database object representing a skill.
   * @static
   * @param {object} item
   * @returns {Bool}
   * @memberof DataManager
   */
  static isSkill(item: rm.types.BaseItem): boolean;

  /**
   * Returns a Bool value given an item.
   * Item must be a database item to be true.
   * @static
   * @param {object} item
   * @returns {Bool}
   * @memberof DataManager
   */
  static isItem(item: rm.types.BaseItem): boolean;

  /**
   * Returns a Bool value given an item.
   * Item must be a database weapon to be true.
   * @static
   * @param {object} item
   * @returns {Bool}
   * @memberof DataManager
   */
  static isWeapon(item: rm.types.BaseItem): boolean;

  /**
   * Returns a Bool value given an item.
   * Item must be a database armor to be true.
   * @static
   * @param {object} item
   * @returns {Bool}
   * @memberof DataManager
   */
  static isArmor(item: rm.types.BaseItem): boolean;

  static createGameObjects(): void;

  static setupNewGame(): void;

  static setupBattleTest(): void;

  static setupEventTest(): void;

  static loadGlobalInfo(): void;

  /**
   * JSON like object
   * @param info
   */
  static saveGlobalInfo(info: any): void;

  /**
   * Returns a Bool value given a savefileId.
   * If the savefileId exists; the value will be true.
   * @static
   * @param {number} savefileId
   * @returns {Bool}
   * @memberof DataManager
   */
  static isThisGameFile(savefileId: number): boolean;

  static isAnySavefileExists(): boolean;

  static latestSavefileId(): number;

  static loadAllSavefileImages(): void;

  /**
   * JSON Like Object for save file images
   */
  static loadSavefileImages(info: any): void;

  /**
   * Returns the maximum number of save files
   * allocated in the game.
   * @static
   * @returns {number}
   * @memberof DataManager
   */
  static maxSavefiles(): number;

  static isMapObject(object: any): boolean;

  /**
   * Saves the RPGMakerMV game given a savefileId.
   * Returns true if successful.
   * @static
   * @param {number} savefileId
   * @returns {Bool}
   * @memberof DataManager
   */
  static saveGame(savefileId: number): Promise<any>;

  static loadGame(savefileId: number): Promise<any>;

  static makeSavename(savefileId: number): string;

  /**
   * Returns the last accessed save fileId upon
   * saving or loading the game.
   * @static
   * @returns {number}
   * @memberof DataManager
   */
  static lastAccessedSavefileId(): number;

  static saveGameWithoutRescue(savefileId: number): boolean;

  static loadGameWithoutRescue(savefileId: number): boolean;

  static selectSavefileForNewGame(): void;

  /**
   * Creates a new save file on the global
   * save file information containing important
   * data such as play time, characters, timestamp.
   * @static
   * @memberof DataManager
   */
  static makeSavefileInfo(): void;

  /**
   * Creates the save file contents when saving your game.
   * Good for aliasing when you want to add more data
   * to the game's save contents.
   * @static
   * @returns {object}
   * @memberof DataManager
   */
  static makeSaveContents(): any;

  /**
   * Extracts the save file contents into the game's
   * variables. An excellent method to alias when you
   * wish to extend the game's save contents for a plugin.
   * @static
   * @param {object} contents
   * @memberof DataManager
   */
  static extractSaveContents(contents: any): void;
}

declare class EffectManager {
  protected constructor();

  static _cache: { [key: string]: any };
  static _errorUrls: any[];

  /**
   * Load the Effect from a file.
   * @param fileName
   * @return Effect
   */
  static load(fileName: string): any;

  /**
   * Starts loading  the Effect from URL
   * @param url
   * @return Effect
   */
  static startLoading(url: string): any;

  /**
   * Clears the cache and releases the Effect from
   * Graphics.
   */
  static clear(): void;

  /**
   * Not Implemented
   */
  static onLoad(): void;

  /**
   * Makes a url to an Effekseer file.
   * @param fileName
   * @return String
   */
  static makeUrl(fileName: string): string;

  /**
   * Checks the error in the errorUrls.
   */
  static checkErrors(): void;

  /**
   * Throws the loading error.
   * @param url
   */
  static throwLoadError(url: string): void;

  /**
   * Returns true if the EffectManager is ready.
   * @return Bool
   */
  static isReady(): boolean;
}

/**
 * Static class that manages images. Loading the images
 * and create bitmap objects.
 */
declare class ImageManager {
  protected constructor();

  /**
   *  returns 32
   */
  static iconWidth: number;
  /**
   * returns 32
   */
  static iconHeight: number;
  /**
   * 144
   */
  static faceWidth: number;
  /**
   * returns 144
   */
  static faceHeight: number;
  static _cache: { [key: string]: any };
  static _system: { [key: string]: any };
  static _emptyBitmap: Bitmap;

  /**
   * Loads the bitmap from url and returns it.
   * @param url
   * @return Bitmap
   */
  static loadBitmapFromUrl(url: string): Bitmap;

  /**
   * Throws a load error and retries loading the bitmap.
   * @param bitmap
   */
  static throwLoadError(bitmap: Bitmap): void;

  static loadFace(filename: string, hue?: number): Bitmap;

  /**
   * Loads a Bitmap object from the 'img/parallaxes/' folder
   * and returns it.
   * @param {string} filename
   * @param {number} [hue]
   * @returns {Bitmap}
   * @memberof ImageManagerStatic
   */
  static loadParallax(filename: string, hue?: number): Bitmap;

  /**
   * Loads a Bitmap object from the 'img/pictures/' folder
   * and returns it.
   * @param {string} filename
   * @param {number} [hue]
   * @returns {Bitmap}
   * @memberof ImageManagerStatic
   */
  static loadPicture(filename: string, hue?: number): Bitmap;

  /**
   * Loads a Bitmap object from the 'img/animations/' folder
   * and returns it.
   * @param {string} filename
   * @param {number} [hue]
   * @returns {Bitmap}
   * @memberof ImageManagerStatic
   */
  static loadAnimation(filename: string, hue?: number): Bitmap;

  /**
   * Loads a Bitmap object from the 'img/battlebacks1/' folder
   *  and returns it.
   * @param {string} filename
   * @param {number} [hue]
   * @returns {Bitmap}
   * @memberof ImageManagerStatic
   */
  static loadBattleback1(filename: string, hue?: number): Bitmap;

  /**
   * Loads a Bitmap object from the 'img/battlebacks2/' folder
   * and returns it.
   * @param {string} filename
   * @param {number} [hue]
   * @returns {Bitmap}
   * @memberof ImageManagerStatic
   */
  static loadBattleback2(filename: string, hue?: number): Bitmap;

  /**
   * Loads a Bitmap object from the 'img/enemies/' folder
   * and returns it.
   * @param {string} filename
   * @param {number} [hue]
   * @returns {Bitmap}
   * @memberof ImageManagerStatic
   */
  static loadEnemy(filename: string, hue?: number): Bitmap;

  /**
   * Loads a Bitmap object from the 'img/characters/' folder
   * and returns it.
   * @param {string} filename
   * @param {number} [hue]
   * @returns {Bitmap}
   * @memberof ImageManagerStatic
   */
  static loadCharacter(filename: string, hue?: number): Bitmap;

  static loadSvActor(filename: string, hue?: number): Bitmap;

  /**
   * Loads a Bitmap object from the 'img/sv_enemies/' folder
   * and returns it.
   * @param {string} filename
   * @param {number} [hue]
   * @returns {Bitmap}
   * @memberof ImageManagerStatic
   */
  static loadSvEnemy(filename: string, hue?: number): Bitmap;

  /**
   * Loads a Bitmap object from 'img/system/' folder
   * and returns it.
   * @param {string} filename
   * @param {number} [hue]
   * @returns {Bitmap}
   * @memberof ImageManagerStatic
   */
  static loadSystem(filename: string, hue?: number): Bitmap;

  /**
   * Loads a Bitmap object from the 'img/tilesets/' folder
   * and returns it.
   * @param {string} filename
   * @param {number} [hue]
   * @returns {Bitmap}
   * @memberof ImageManagerStatic
   */
  static loadTileset(filename: string, hue?: number): Bitmap;

  /**
   * Loads a Bitmap object from the 'img/titles1/' folder
   * and returns it.
   * @param {string} filename
   * @param {number} [hue]
   * @returns {Bitmap}
   * @memberof ImageManagerStatic
   */
  static loadTitle1(filename: string, hue?: number): Bitmap;

  /**
   * Loads a Bitmap object from the 'img/titles2/' folder
   * and returns it.
   * @param {string} filename
   * @param {number} [hue]
   * @returns {Bitmap}
   * @memberof ImageManagerStatic
   */
  static loadTitle2(filename: string, hue?: number): Bitmap;

  /**
   * Loads a Bitmap object from any folder and returns it.
   *
   * @param {string} folder
   * @param {string} filename
   * @param {number} hue
   * @param {boolean} smooth
   * @returns {Bitmap}
   * @memberof ImageManagerStatic
   */
  static loadBitmap(folder: string, filename: string, hue: number, smooth: boolean): Bitmap;

  /**
   * Loads an empty Bitmap object and returns it.
   *
   * @param {string} path
   * @param {number} hue
   * @returns {Bitmap}
   * @memberof ImageManagerStatic
   */
  static loadEmptyBitmap(path: string, hue: number): Bitmap;

  /**
   * Loads a Bitmap object given a path
   * and returns it.
   * @param {string} path
   * @param {number} hue
   * @returns {Bitmap}
   * @memberof ImageManagerStatic
   */
  static loadNormalBitmap(path: string, hue: number): Bitmap;

  /**
   * Clears the image cache in RPGMakerMV.
   *
   * @memberof ImageManagerStatic
   */
  static clear(): void;

  /**
   * Returns true if the image cache
   * is ready.
   * @returns {boolean}
   * @memberof ImageManagerStatic
   */
  static isReady(): boolean;

  /**
   * Returns true if the given filename
   * is an object character. Must contain
   * a  '!' in the file name to be an
   * object character.
   * @param {string} filename
   * @returns {boolean}
   * @memberof ImageManagerStatic
   */
  static isObjectCharacter(filename: string): boolean;

  /**
   * Returns true if the given filename is
   * a large character. Must contain a '$'
   * in the file name to be a large character.
   * @param {string} filename
   * @returns {boolean}
   * @memberof ImageManagerStatic
   */
  static isBigCharacter(filename: string): boolean;

  static isZeroParallax(filename: string): boolean;
}

declare namespace rm.types {
  /**
   * Plugin Settings for RPGMakerMV/MZ
   */
  export type PluginSettings = {
    /**
     * Plugin Description
     */
    description: string;
    /**
     * Plugin Name
     */
    name: string;
    /**
     * Plugin Parameters in a map/dictionary like syntax.
     * Example:
     * ```js
     * parameters["TextSpeed"]
     * ```
     */
    parameters: { [key: string]: any };
    /**
     * Plugin Status On/Off
     */
    status: string;
  };
}

/**
 * Static class that manages plugins
 */
declare class PluginManager {
  protected constructor();

  static _path: string;
  static _scripts: string[];
  static _errorUrls: string[];

  static setup(plugins: rm.types.PluginSettings[]): void;

  static checkErrors(): void;

  /**
   * Returns all of the plugin parameters for an RPGMakerMZ
   * plugin in a JSON Like String Format.
   * Should typedef this for ease of parsing.
   * @param name
   * @return Any
   */
  static parameters(name: string): any;

  /**
   * Uses internally by the Plugin Manager to load
   * the plugin scripts.
   * @param name
   */
  static loadScript(name: string): void;

  static onError(e: Event): void;

  /**
   * Regsters a new plugin command in MZ using a lambda expression
   * The Variable of the lambda should be transformed into the appropriate
   * typed def struct for your use case for your command.
   * Example Syntax For Commented Parameters
   *
   *```js
   * //Should be in comment
   * @command PluginCommandFunctionName
   * @text My Plugin Command
   * @desc Plugin Command Description Text
   * @arg Actors
   * @text Which Actors?
   * @type actor[]
   * @desc Select which Actor ID(s) to affect.
   * @default ["1"]
   * @arg State
   * @text State
   * @type state
   * @desc Which State to apply?
   * @default 1
   * PluginManager.registerCommand("TestPlugin", "PluginCommandFunctionName", args => {
   * // Get Arguments
   * const actorIds = JSON.parse(args.Actors);
   * const stateId = args.State;
   * // Use the arguments
   * for (const actorId of actorIds) {
   * const actor = $gameParty.members().find(member => member.actorId() === Number(actorId));
   * if (actor) {
   * actor.addState(stateId);
   * }
   * }
   * });
   * ```
   * @param pluginName
   * @param commandName
   * @param lambda
   */
  static registerCommand(pluginName: string, commandName: string, lambda: (arg0: any) => void): void;
}

/**
 * Manages the scenes of the game.
 */
declare class SceneManager {
  protected constructor();

  static _scene: Scene_Base;
  static _nextScene: Scene_Base;
  static _stack: any[];
  static _screenWidth: number;
  static _screenHeight: number;
  static _backgroundBitmap: Bitmap;
  static _boxWidth: number;
  static _boxHeight: number;
  static _deltaTime: number;
  static _currentTime: number;
  static _accumulator: number;
  /**
   * Convenience variable for getting the current scene.
   */
  static _scene: any;

  static preferableRendererType(): string;

  static shouldUseCanvasRenderer(): boolean;

  static initialize(): void;

  static initGraphics(): void;

  static initAudio(): void;

  static initInput(): void;

  static initNwjs(): void;

  static update(): void;

  static terminate(): void;

  static tickStart(): void;

  static tickEnd(): void;

  static changeScene(): void;

  static updateScene(): void;

  static renderScene(): void;

  static onSceneCreate(): void;

  static onSceneStart(): void;

  static onSceneLoading(): void;

  static onError(e: Event): void;

  static onKeyDown(event: KeyboardEvent): void;

  static catchException(e: Event): void;

  static isSceneChanging(): boolean;

  static isCurrentSceneBusy(): boolean;

  static isCurrentSceneStarted(): boolean;

  static isNextScene(sceneClass: any): boolean;

  static isPreviousScene(sceneClass: any): boolean;

  /**
   * Goes to the scene passed in.
   * @param sceneClass
   */
  static goto(sceneClass: any): void;

  /**
   * Pushes the scene passed in to the scene stack.
   * Immediately goes to the scene passed in.
   * @param sceneClass
   */
  static push(sceneClass: any): void;

  /**
   * Pops the current scene from the stack.
   */
  static pop(): void;

  /**
   * Exits the current scene.
   */
  static exit(): void;

  static clearStack(): void;

  /**
   * Stops the current scene.
   */
  static stop(): void;

  static prepareNextScene(): void;

  static snap(): Bitmap;

  static snapForBackground(): void;

  static backgroundBitmap(): Bitmap;

  static updateManagers(ticks: number, delta: number): void;

  static showDevTools(): void;
}

/**
 * Static class that plays sound effects defined in the database.
 */
declare class SoundManager {
  protected constructor();

  static preloadImportantSounds(): void;

  static loadSystemSound(databaseNumber: number): void;

  static playSystemSound(databaseNumber: number): void;

  static playCursor(): void;

  static playOk(): void;

  static playCancel(): void;

  static playBuzzer(): void;

  static playEquip(): void;

  static playSave(): void;

  static playLoad(): void;

  static playBattleStart(): void;

  static playEscape(): void;

  static playEnemyAttack(): void;

  static playEnemyDamage(): void;

  static playEnemyCollapse(): void;

  static playBossCollapse1(): void;

  static playBossCollapse2(): void;

  static playActorDamage(): void;

  static playActorCollapse(): void;

  static playRecovery(): void;

  static playMiss(): void;

  static playEvasion(): void;

  static playMagicEvasion(): void;

  static playReflection(): void;

  static playShop(): void;

  static playUseItem(): void;

  static playUseSkill(): void;
}

/**
 * The static class that manages storage of save game data.
 */
declare class StorageManager {
  protected constructor();

  static saveObject(saveName: string, object: any): Promise<any>;

  static loadObject(object: any): Promise<any>;

  static objectToJson(object: any): Promise<any>;

  static jsonToObject(json: JSON): Promise<any>;

  static jsonToZip(json: JSON): Promise<any>;

  static zipToJson(zip: any): Promise<any>;

  static saveZip(saveName: string, zip: any): Promise<any>;

  static loadZip(saveName: string, zip: any): Promise<any>;

  static saveToForage(saveName: string): Promise<any>;

  static loadFromForage(saveName: string): Promise<any>;

  static forageExists(saveName: string): boolean;

  static removeForage(saveName: string): Promise<any>;

  static updateForageKeys(): Promise<any>;

  static forageKeysUpdated(): boolean;

  static fsMkdir(path: string): void;

  static fsRename(oldPath: string, newPath: string): void;

  static fsUnlink(path: string): void;

  static fsReadFile(path: string): void;

  static fsWriteFile(path: string, data: any): void;

  static fileDirectoryPath(): string;

  static filePath(): string;

  static forageKey(): string;

  static forageTestKey(): string;

  static save(savefileId: number, json: string): void;

  static load(savefileId: number): string;

  static exists(savefileId: number): boolean;

  static remove(savefileId: number): void;

  static backup(savefileId: number): void;

  static backupExists(savefileId: number): boolean;

  static cleanBackup(savefileId: number): boolean;

  static restoreBackup(savefileId: number): void;

  static isLocalMode(): boolean;

  static saveToLocalFile(savefileId: number, json: string): void;

  static loadFromLocalFile(savefileId: number): string;

  static loadFromLocalBackupFile(savefileId: number): string;

  static localFileBackupExists(savefileId: number): boolean;

  static localFileExists(savefileId: number): boolean;

  static removeLocalFile(savefileId: number): void;

  static saveToWebStorage(savefileId: number, json: string): void;

  static loadFromWebStorage(savefileId: number): string;

  static loadFromWebStorageBackup(savefileId: number): string;

  static webStorageBackupExists(savefileId: number): boolean;

  static webStorageExists(savefileId: number): boolean;

  static removeWebStorage(savefileId: number): void;

  static localFileDirectoryPath(): string;

  static localFilePath(savefileId: number): string;

  static webStorageKey(savefileId: number): string;
}

/**
 * The static class that handles terms and messages.
 * This is linked to the terms and information
 * contained within the editor.
 */
declare class TextManager {
  protected constructor();

  static currencyUnit: string;
  static level: string;
  static levelA: string;
  static hp: string;
  static hpA: string;
  static mp: string;
  static mpA: string;
  static tp: string;
  static tpA: string;
  static exp: string;
  static expA: string;
  static fight: string;
  static escape: string;
  static attack: string;
  static guard: string;
  static item: string;
  static skill: string;
  static equip: string;
  static status: string;
  static formation: string;
  static save: string;
  static gameEnd: string;
  static options: string;
  static weapon: string;
  static armor: string;
  static keyItem: string;
  static equip2: string;
  static optimize: string;
  static clear: string;
  static newGame: string;
  static continue_: string;
  static toTitle: string;
  static cancel: string;
  static buy: string;
  static sell: string;
  static alwaysDash: string;
  static commandRemember: string;
  static bgmVolume: string;
  static bgsVolume: string;
  static meVolume: string;
  static seVolume: string;
  static possession: string;
  static expTotal: string;
  static expNext: string;
  static saveMessage: string;
  static loadMessage: string;
  static file: string;
  static partyName: string;
  static emerge: string;
  static preemptive: string;
  static surprise: string;
  static escapeStart: string;
  static escapeFailure: string;
  static victory: string;
  static defeat: string;
  static obtainExp: string;
  static obtainGold: string;
  static obtainItem: string;
  static levelUp: string;
  static obtainSkill: string;
  static useItem: string;
  static criticalToEnemy: string;
  static criticalToActor: string;
  static actorDamage: string;
  static actorRecovery: string;
  static actorGain: string;
  static actorLoss: string;
  static actorDrain: string;
  static actorNoDamage: string;
  static actorNoHit: string;
  static enemyDamage: string;
  static enemyRecovery: string;
  static enemyGain: string;
  static enemyLoss: string;
  static enemyDrain: string;
  static enemyNoDamage: string;
  static enemyNoHit: string;
  static evasion: string;
  static magicEvasion: string;
  static magicReflection: string;
  static counterAttack: string;
  static substitute: string;
  static buffAdd: string;
  static debuffAdd: string;
  static buffRemove: string;
  static actionFailure: string;

  static basic(basicId: number): string;

  static param(paramId: number): string;

  static command(commandId: number): string;

  static message(messageId: string): string;

  static getter(method: string, param: number): string;
}

declare namespace rm.types {
  /**
   * The data class for enemy [Actions].
   */
  export type EnemyAction = {
    /**
     * The first parameter of the condition.
     */
    conditionParam1: number;
    /**
     * The second parameter of the condition.
     */
    conditionParam2: number;
    /**
     * The type of condition.
     *
     * 0: Always
     * 1: Turn No.
     * 2: HP
     * 3: MP
     * 4: State
     * 5: Party Level
     * 6: Switch
     */
    conditionType: number;
    /**
     * The action's priority rating (1..10).
     */
    rating: number;
    /**
     * The ID of skills to be employed as actions.
     */
    skillId: number;
  };
}

declare namespace rm.types {
  /**
   * The data class for damage.
   */
  export type Damage = {
    /**
     * Critical hit (true/false).
     */
    critical: boolean;
    /**
     * The element ID.
     */
    elementId: number;
    /**
     * The formula.
     */
    formula: string;
    /**
     * The type of damage.
     *
     * 0: None
     * 1: HP damage
     * 2: MP damage
     * 3: HP recovery
     * 4: MP recovery
     * 5: HP drain
     * 6: MP drain
     */
    type: number;
    /**
     * The degree of variability.
     */
    variance: number;
  };
}

declare namespace rm.types {
  /**
   * The data class for use effects.
   */
  export type Effect = {
    /**
     * The use effect code.
     */
    code: number;
    /**
     * The ID of data (state, parameter, and so on) according to the type of use effect.
     */
    dataId: number;
    /**
     * Value 1 set according to the type of use effect.
     */
    value1: number;
    /**
     * Value 2 set according to the type of use effect.
     */
    value2: number;
  };
}

declare namespace rm.types {
  export type UsableItem = {
    /**
     * The animation ID.
     */
    animationId: number;
    /**
     * Damage (RPG.Damage).
     */
    damage: rm.types.Damage;
    /**
     * The description text.
     */
    description: string;
    /**
     * A list of use effects. An RPG.Effect array.
     */
    effects: rm.types.Effect[];
    /**
     * The type of hit.
     *
     * 0: Certain hit
     * 1: Physical attack
     * 2: Magical attack
     */
    hitType: number;
    /**
     * The icon number.
     */
    iconIndex: number;
    /**
     * The item ID.
     */
    id: number;
    meta: Object;
    /**
     * The item name.
     */
    name: string;
    note: string;
    /**
     * When the item/skill may be used.
     *
     * 0: Always
     * 1: Only in battle
     * 2: Only from the menu
     * 3: Never
     */
    occasion: number;
    /**
     * The number of repeats.
     */
    repeats: number;
    /**
     * The scope of effects.
     *
     * 0: None
     * 1: One Enemy
     * 2: All Enemies
     * 3: One Random Enemy
     * 4: Two Random Enemies
     * 5: Three Random Enemies
     * 6: Four Random Enemies
     * 7: One Ally
     * 8: All Allies
     * 9: One Ally (Dead)
     * 10: All Allies (Dead)
     * 11: The User
     */
    scope: number;
    /**
     * The speed correction.
     */
    speed: number;
    /**
     * The success rate.
     */
    successRate: number;
    /**
     * The number of TP gained.
     */
    tpGain: number;
  };
}

/**
 * Game_Action
 * The game object class representing a battle action.
 * @class Game_Action
 */
declare class Game_Action {
  constructor(subject: Game_Battler, forcing: boolean);

  _subjectActorId: number;
  _subjectEnemyIndex: number;
  _targetIndex: number;
  _forcing: boolean;
  _item: Game_Item;

  clear(): void;

  setSubject(subject: Game_Battler): void;

  subject(): Game_Battler;

  friendsUnit(): Game_Unit;

  opponentsUnit(): Game_Unit;

  setEnemyAction(action: rm.types.EnemyAction): void;

  setAttack(): void;

  setGuard(): void;

  setSkill(skillId: number): void;

  setItem(itemId: number): void;

  setItemObject(object: rm.types.UsableItem): void;

  setTarget(targetIndex: number): void;

  item(): rm.types.UsableItem;

  isSkill(): boolean;

  isItem(): boolean;

  numRepeats(): number;

  checkItemScope(list: number[]): boolean;

  isForOpponent(): boolean;

  isForFriend(): boolean;

  isForDeadFriend(): boolean;

  isForUser(): boolean;

  isForOne(): boolean;

  isForRandom(): boolean;

  isForAll(): boolean;

  needsSelection(): number;

  numTargets(): number;

  checkDamageType(list: number[]): boolean;

  isHpEffect(): boolean;

  isMpEffect(): boolean;

  isDamage(): boolean;

  isRecover(): boolean;

  isDrain(): boolean;

  isHpRecover(): boolean;

  isMpRecover(): boolean;

  isCertainHit(): boolean;

  isPhysical(): boolean;

  isMagical(): boolean;

  isAttack(): boolean;

  isGuard(): boolean;

  isMagicSkill(): boolean;

  decideRandomTarget(): void;

  setConfusion(): void;

  prepare(): void;

  isValid(): boolean;

  speed(): number;

  makeTargets(): Game_Battler[];

  repeatTargets(targets: Game_Battler[]): Game_Battler[];

  confusionTarget(): Game_Battler;

  targetsForOpponents(): Game_Battler[];

  targetsForFriends(): Game_Battler[];

  evaluate(): number;

  itemTargetCandidates(): Game_Battler[];

  evaluateWithTarget(target: Game_Battler): number;

  testApply(target: Game_Battler): boolean;

  hasItemAnyValidEffects(target: Game_Battler): boolean;

  testItemEffect(target: Game_Battler, effect: rm.types.Effect): boolean;

  itemCnt(target: Game_Battler): number;

  itemMrf(target: Game_Battler): number;

  itemHit(target: Game_Battler): number;

  itemEva(target: Game_Battler): number;

  itemCri(target: Game_Battler): number;

  apply(target: Game_Battler): void;

  makeDamageValue(target: Game_Battler, critical: boolean): number;

  evalDamageFormula(target: Game_Battler): number;

  calcElementRate(target: Game_Battler): number;

  elementsMaxRate(target: Game_Battler, elements: number[]): number;

  applyCritical(damage: number): number;

  applyVariance(damage: number, variance: number): number;

  applyGuard(damage: number, target: Game_Battler): number;

  executeDamage(target: Game_Battler, value: number): void;

  executeHpDamage(target: Game_Battler, value: number): void;

  executeMpDamage(target: Game_Battler, value: number): void;

  gainDrainedHp(value: number): void;

  gainDrainedMp(value: number): void;

  applyItemEffect(target: Game_Battler, effect: rm.types.Effect): void;

  itemEffectRecoverHp(target: Game_Battler, effect: rm.types.Effect): void;

  itemEffectRecoverMp(target: Game_Battler, effect: rm.types.Effect): void;

  itemEffectGainTp(target: Game_Battler, effect: rm.types.Effect): void;

  itemEffectAddState(target: Game_Battler, effect: rm.types.Effect): void;

  itemEffectAddAttackState(target: Game_Battler, effect: rm.types.Effect): void;

  itemEffectAddNormalState(target: Game_Battler, effect: rm.types.Effect): void;

  itemEffectRemoveState(target: Game_Battler, effect: rm.types.Effect): void;

  itemEffectAddBuff(target: Game_Battler, effect: rm.types.Effect): void;

  itemEffectAddDebuff(target: Game_Battler, effect: rm.types.Effect): void;

  itemEffectRemoveBuff(target: Game_Battler, effect: rm.types.Effect): void;

  itemEffectRemoveDebuff(target: Game_Battler, effect: rm.types.Effect): void;

  itemEffectSpecial(target: Game_Battler, effect: rm.types.Effect): void;

  itemEffectGrow(target: Game_Battler, effect: rm.types.Effect): void;

  itemEffectLearnSkill(target: Game_Battler, effect: rm.types.Effect): void;

  itemEffectCommonEvent(target: Game_Battler, effect: rm.types.Effect): void;

  makeSuccess(target: Game_Battler): void;

  applyItemUserEffect(target: Game_Battler): void;

  lukEffectRate(target: Game_Battler): number;

  applyGlobal(): void;

  static EFFECT_RECOVER_HP: number;
  static EFFECT_RECOVER_MP: number;
  static EFFECT_GAIN_TP: number;
  static EFFECT_ADD_STATE: number;
  static EFFECT_REMOVE_STATE: number;
  static EFFECT_ADD_BUFF: number;
  static EFFECT_ADD_DEBUFF: number;
  static EFFECT_REMOVE_BUFF: number;
  static EFFECT_REMOVE_DEBUFF: number;
  static EFFECT_SPECIAL: number;
  static EFFECT_GROW: number;
  static EFFECT_LEARN_SKILL: number;
  static EFFECT_COMMON_EVENT: number;
  static SPECIAL_EFFECT_ESCAPE: number;
  static HITTYPE_CERTAIN: number;
  static HITTYPE_PHYSICAL: number;
  static HITTYPE_MAGICAL: number;
}

declare namespace haxe.display {}

declare namespace haxe.display {}

declare namespace rm.types {
  export type Trait = {
    /**
     * The trait code.
     */
    code: number;
    /**
     * The ID of the data (such as elements or states) according to the type of the trait.
     */
    dataId: number;
    /**
     * The map tree expansion flag, which is used internally.
     */
    expanded: boolean;
    /**
     * The x-axis scroll position, which is used internally.
     */
    scrollX: number;
    /**
     * The y-axis scroll position, which is used internally.
     */
    scrollY: number;
    /**
     * The value set according to the type of the trait.
     */
    value: number;
  };
}

declare namespace rm.types {
  /**
   * The data class for state.
   */
  export type State = {
    /**
     * The timing of automatic state removal.
     *
     * 0: None
     * 1: At end of action
     * 2: At end of turn
     */
    autoRemovalTiming: number;
    /**
     * Chance of state being removed by damage (%).
     */
    chanceByDamage: number;
    description: string;
    doc: string;
    /**
     * The icon number.
     */
    iconIndex: number;
    /**
     * The ID.
     */
    id: number;
    internal: boolean;
    links: string[];
    /**
     * The maximum turns of the duration.
     */
    maxTurns: number;
    /**
     * The message when an actor fell in the state.
     */
    message1: string;
    /**
     * The message when an enemy fell in the state.
     */
    message2: string;
    /**
     * The message when the state remains.
     */
    message3: string;
    /**
     * The message when the state is removed.
     */
    message4: number;
    /**
     * The minimum turns of the duration.
     */
    minTurns: number;
    /**
     * The side-view motion.
     */
    motion: number;
    /**
     * The name.
     */
    name: string;
    /**
     * The side-view overlay.
     */
    overlay: number;
    parameters: string[];
    platforms: haxe.display.Platform[];
    /**
     * The state priority (0..100).
     */
    priority: number;
    releaseByDamage: boolean;
    /**
     * Removes state at end of battle (true/false).
     */
    removeAtBattleEnd: boolean;
    /**
     * Removes state by damage (true/false).
     */
    removeByDamage: boolean;
    /**
     * Removes state by action restriction (true/false).
     */
    removeByRestriction: boolean;
    /**
     * Removes state by walking (true/false).
     */
    removeByWalking: boolean;
    /**
     * Action restrictions.
     *
     * 0: None
     * 1: Attack enemy
     * 2: Attack enemy or ally
     * 3: Attack ally
     * 4: Cannot act
     */
    restriction: number;
    /**
     * Number of steps until state is removed.
     */
    stepToRemove: number;
    targets: haxe.display.MetadataTarget[];
    /**
     * The array of Trait data.
     */
    traits: rm.types.Trait[];
  };
}

/**
 * -----------------------------------------------------------------------------
 * Game_ActionResult
 *
 * The game object declare class for a result of a battle action. For convinience, all
 * member variables in this declare class are public.
 * @class Game_ActionResult
 */
declare class Game_ActionResult {
  constructor();

  used: boolean;
  missed: boolean;
  evaded: boolean;
  physical: boolean;
  drain: boolean;
  critical: boolean;
  success: boolean;
  hpAffected: boolean;
  hpDamage: number;
  mpDamage: number;
  tpDamage: number;
  addedStates: number[];
  removedStates: number[];
  addedBuffs: number[];
  addedDebuffs: number[];
  removedBuffs: number[];

  initialize(): void;

  /**
   * Clears the game action result.
   *
   * @memberof Game_ActionResult
   */
  clear(): void;

  /**
   * Returns the added states from the action result.
   *
   * @returns {Array<RPG.State>}
   * @memberof Game_ActionResult
   */
  addedStateObjects(): rm.types.State[];

  /**
   * Returns the removes states from the result.
   *
   * @returns {Array<RPG.State>}
   * @memberof Game_ActionResult
   */
  removedStateObjects(): rm.types.State[];

  isStatusAffected(): boolean;

  /**
   * Returns true if the action result is a hit.
   *
   * @returns {Bool}
   * @memberof Game_ActionResult
   */
  isHit(): boolean;

  isStateAdded(stateId: number): boolean;

  pushAddedState(stateId: number): void;

  isStateRemoved(stateId: number): boolean;

  pushRemovedState(stateId: number): void;

  /**
   * Returns true if the a buff is added to the specified param
   * from the action result.
   * @param {number} paramId
   * @returns {Bool}
   * @memberof Game_ActionResult
   */
  isBuffAdded(paramId: number): boolean;

  pushAddedBuff(paramId: number): void;

  isDebuffAdded(paramId: number): boolean;

  pushAddedDebuff(paramId: number): void;

  isBuffRemoved(paramId: number): boolean;

  pushRemovedBuff(paramId: number): void;
}

declare namespace rm.types {
  /**
   * The data class for skills.
   */
  export type Skill = {
    /**
     * The animation ID.
     */
    animationId: number;
    /**
     * Damage (RPG.Damage).
     */
    damage: rm.types.Damage;
    /**
     * The description text.
     */
    description: string;
    /**
     * A list of use effects. An RPG.Effect array.
     */
    effects: rm.types.Effect[];
    /**
     * The type of hit.
     *
     * 0: Certain hit
     * 1: Physical attack
     * 2: Magical attack
     */
    hitType: number;
    /**
     * The icon number.
     */
    iconIndex: number;
    /**
     * The item ID.
     */
    id: number;
    /**
     * The use message.
     */
    message1: string;
    /**
     * The use message.
     */
    message2: string;
    meta: Object;
    /**
     * Number of MP consumed.
     */
    mpCost: number;
    /**
     * The item name.
     */
    name: string;
    note: string;
    /**
     * When the item/skill may be used.
     *
     * 0: Always
     * 1: Only in battle
     * 2: Only from the menu
     * 3: Never
     */
    occasion: number;
    /**
     * The number of repeats.
     */
    repeats: number;
    /**
     * Weapon type required.
     */
    requiredWtypeId1: number;
    /**
     * Weapon type required.
     */
    requiredWtypeId2: number;
    /**
     * The scope of effects.
     *
     * 0: None
     * 1: One Enemy
     * 2: All Enemies
     * 3: One Random Enemy
     * 4: Two Random Enemies
     * 5: Three Random Enemies
     * 6: Four Random Enemies
     * 7: One Ally
     * 8: All Allies
     * 9: One Ally (Dead)
     * 10: All Allies (Dead)
     * 11: The User
     */
    scope: number;
    /**
     * The speed correction.
     */
    speed: number;
    /**
     * Skill type ID.
     */
    stypeId: number;
    /**
     * The success rate.
     */
    successRate: number;
    /**
     * Number of TP consumed
     */
    tpCost: number;
    /**
     * The number of TP gained.
     */
    tpGain: number;
  };
}

declare namespace rm.types {
  /**
   * The data class for items.
   */
  export type Item = {
    /**
     * The animation ID.
     */
    animationId: number;
    /**
     * The truth value indicating whether the item disappears when used.
     */
    consumable: boolean;
    /**
     * Damage (RPG.Damage).
     */
    damage: rm.types.Damage;
    /**
     * The description text.
     */
    description: string;
    /**
     * A list of use effects. An RPG.Effect array.
     */
    effects: rm.types.Effect[];
    /**
     * The type of hit.
     *
     * 0: Certain hit
     * 1: Physical attack
     * 2: Magical attack
     */
    hitType: number;
    /**
     * The icon number.
     */
    iconIndex: number;
    /**
     * The item ID.
     */
    id: number;
    /**
     * The item type ID.
     *
     * 1: Regular item
     * 2: Key item
     */
    itypeId: number;
    meta: Object;
    /**
     * The item name.
     */
    name: string;
    note: string;
    /**
     * When the item/skill may be used.
     *
     * 0: Always
     * 1: Only in battle
     * 2: Only from the menu
     * 3: Never
     */
    occasion: number;
    /**
     * The item's price.
     */
    price: number;
    /**
     * The number of repeats.
     */
    repeats: number;
    /**
     * The scope of effects.
     *
     * 0: None
     * 1: One Enemy
     * 2: All Enemies
     * 3: One Random Enemy
     * 4: Two Random Enemies
     * 5: Three Random Enemies
     * 6: Four Random Enemies
     * 7: One Ally
     * 8: All Allies
     * 9: One Ally (Dead)
     * 10: All Allies (Dead)
     * 11: The User
     */
    scope: number;
    /**
     * The speed correction.
     */
    speed: number;
    /**
     * The success rate.
     */
    successRate: number;
    /**
     * The number of TP gained.
     */
    tpGain: number;
  };
}

declare namespace rm.types {
  /**
   * A superclass of weapons and armor.
   */
  export type EquipItem = {
    /**
     * The description text.
     */
    description: string;
    /**
     * The type of weapon or armor.
     *
     * 0: Weapon
     * 1: Shield
     * 2: Head
     * 3: Body
     * 4: Accessory
     */
    etypeId: number;
    /**
     * The icon number.
     */
    iconIndex: number;
    /**
     * The item ID.
     */
    id: number;
    meta: Object;
    /**
     * The item name.
     */
    name: string;
    note: string;
    /**
     * The amount of parameter change. An array of integers using the following IDs as subscripts:
     *
     * 0: Maximum hit points
     * 1: Maximum magic points
     * 2: Attack power
     * 3: Defense power
     * 4: Magic attack power
     * 5: Magic defense power
     * 6: Agility
     * 7: Luck
     */
    params: number[];
    /**
     * The price of the weapon or armor.
     */
    price: number;
    /**
     * The array of Trait data.
     */
    traits: rm.types.Trait[];
  };
}

/**
 * -----------------------------------------------------------------------------
 * Game_BattlerBase
 *
 * The superdeclare class of Game_Battler. It mainly contains parameters calculation.
 * @class Game_BattlerBase
 */
declare class Game_BattlerBase {
  constructor();

  initialize(): void;

  _hp: number;
  _mp: number;
  _tp: number;
  _hidden: boolean;
  _paramPlus: number[];
  _states: number[];
  /**
   * [stateId: Int]:Int
   * }
   * Access using number/integer
   */
  _stateTurns: { [key: string]: any };
  _buffs: number[];
  _buffTurns: number[];
  /**
   * [read-only] Hit Points
   */
  readonly hp: number;
  /**
   * [read-only] Magic Points
   */
  readonly mp: number;
  /**
   * [read-only] Tactical Points
   */
  readonly tp: number;
  /**
   * [read-only] Maximum Hit Points - param 0
   */
  readonly mhp: number;
  /**
   * [read-only] Maximum Magic Points - param 1
   */
  readonly mmp: number;
  /**
   * [read-only] ATtacK power - param 2
   */
  readonly atk: number;
  /**
   * [read-only] DEFense power - param 3
   */
  readonly def: number;
  /**
   * [read-only] Magic Attack power - param 4
   */
  readonly mat: number;
  /**
   * [read-only] Magic Defense power - param 5
   */
  readonly mdf: number;
  /**
   * [read-only] Agility - param 6
   */
  readonly agi: number;
  /**
   * [read-only] LucK - param 7
   */
  readonly luk: number;
  /**
   * [read-only] HIT rate -xparam 0
   */
  readonly hit: number;
  /**
   * [read-only] EVAsion rate
   */
  readonly eva: number;
  /**
   * [read-only] CRItical rate
   */
  readonly cri: number;
  /**
   * [read-only] Critical EVasion rate
   */
  readonly cev: number;
  /**
   * [read-only] Magic EVasion rate
   */
  readonly mev: number;
  /**
   * [read-only] Magic ReFlection rate
   */
  readonly mrf: number;
  /**
   * [read-only] CouNTer attack rate
   */
  readonly cnt: number;
  /**
   * [read-only] Hp ReGeneration rate
   */
  readonly hrg: number;
  /**
   * [read-only] Mp ReGeneration rate
   */
  readonly mrg: number;
  /**
   * [read-only] Tp ReGeneration rate
   */
  readonly trg: number;
  /**
   * [read-only] TarGet Rate
   */
  readonly tgr: number;
  /**
   * [read-only] Ggweqrtg*xzuaRD effect rate
   */
  readonly grd: number;
  /**
   * [read-only] RECovery effect rate
   */
  readonly rec: number;
  /**
   * [read-only] PHArmacology
   */
  readonly pha: number;
  /**
   * [read-only] Mp Cost Rate
   */
  readonly mcr: number;
  /**
   * [read-only] Tp Charge Rate
   */
  readonly tcr: number;
  /**
   * [read-only] Physical Damage Rate
   */
  readonly pdr: number;
  /**
   * [read-only] Magical Damage Rate
   */
  readonly mdr: number;
  /**
   * [read-only] Floor Damage Rate
   */
  readonly fdr: number;
  /**
   * [read-only] EXperience Rate
   */
  readonly exr: number;

  initMembers(): void;

  /**
   * Clears any modifications to
   * the base parameters.
   */
  clearParamPlus(): void;

  /**
   * Clears states applied to the actors.
   */
  clearStates(): void;

  /**
   * Erases the current state from the game battler given the
   * stateId in the editor database.
   * @param {number} stateId
   * @memberof Game_BattlerBase
   */
  eraseState(stateId: number): void;

  /**
   * Returns true if the battler is affected by the specified state given
   * the state id.
   * @param {number} stateId
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  isStateAffected(stateId: number): boolean;

  isDeathStateAffected(): boolean;

  /**
   * Returns the death state id.
   *
   * @returns {number}
   * @memberof Game_BattlerBase
   */
  deathStateId(): number;

  /**
   * Resets the state count of the specified state, given the state id.
   *
   * @param {number} stateId
   * @memberof Game_BattlerBase
   */
  resetStateCounts(stateId: number): void;

  /**
   * Returns true if the state, given the state id is expired.
   *
   * @param {number} stateId
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  isStateExpired(stateId: number): boolean;

  updateStateTurns(): void;

  /**
   * Clears buffs from the battler.
   *
   * @memberof Game_BattlerBase
   */
  clearBuffs(): void;

  eraseBuff(paramId: number): void;

  /**
   * Returns the length of the buff.
   *
   * @returns {number}
   * @memberof Game_BattlerBase
   */
  buffLength(): number;

  /**
   * Buffs the current parameter id.
   *
   * @param {number} paramId
   * @returns {number}
   * @memberof Game_BattlerBase
   */
  buff(paramId: number): number;

  isBuffAffected(paramId: number): boolean;

  isDebuffAffected(paramId: number): boolean;

  isBuffOrDebuffAffected(paramId: number): boolean;

  isMaxBuffAffected(paramId: number): boolean;

  isMaxDebuffAffected(paramId: number): boolean;

  increaseBuff(paramId: number): void;

  decreaseBuff(paramId: number): void;

  overwriteBuffTurns(paramId: number, turns: number): void;

  isBuffExpired(paramId: number): boolean;

  /**
   * Updates the buff turns on battler.
   *
   * @memberof Game_BattlerBase
   */
  updateBuffTurns(): void;

  /**
   * Kills the battler.
   *
   * @memberof Game_BattlerBase
   */
  die(): void;

  /**
   * Revives the battler.
   *
   * @memberof Game_BattlerBase
   */
  revive(): void;

  /**
   * Returns the states applied to the battler.
   *
   * @returns {Array<RPG.State>}
   * @memberof Game_BattlerBase
   */
  states(): rm.types.State[];

  /**
   * Returns the array of state icons attached to the battler;
   * this is determined by the active states on the battler.
   * @returns {Array<number>}
   * @memberof Game_BattlerBase
   */
  stateIcons(): number[];

  /**
   * Returns the array of buff icons attached to the battler;
   * this is determined by the active buffs on the battler.
   * @returns {Array<number>}
   * @memberof Game_BattlerBase
   */
  buffIcons(): number[];

  buffIconIndex(buffLevel: number, paramId: number): number;

  /**
   * Returns all of the icons attached to the battler.
   *
   * @returns {Array<number>}
   * @memberof Game_BattlerBase
   */
  allIcons(): number[];

  /**
   * Returns the trait object to the user
   * @return Array<any>
   */
  traitObjects(): any[];

  /**
   * Returns all the traits of the battler.
   *
   * @returns {Array<RPG.Trait>}
   * @memberof Game_BattlerBase
   */
  allTraits(): rm.types.Trait[];

  traits(code: number): rm.types.Trait[];

  traitsWithId(code: number, traitId: number): rm.types.Trait[];

  traitsPi(code: number, traitId: number): number;

  traitsSum(code: number, traitId: number): number;

  traitsSumAll(code: number): number;

  traitsSet(code: number): number[];

  /**
   * Returns the base parameters of the battler; this is determined by their
   * current level and the paramId given.
   * @param {number} paramId
   * @returns {number}
   * @memberof Game_BattlerBase
   */
  paramBase(paramId: number): number;

  paramPlus(paramId: number): number;

  paramMin(paramId: number): number;

  paramMax(paramId: number): number;

  paramRate(paramId: number): number;

  paramBuffRate(paramId: number): number;

  /**
   * Returns a standard parameter, given a paramId;
   * standard parameters include: HP, MP, Atk, M.Atk, Def, M.Def, Luck, Agility.
   * @param {number} paramId
   * @returns {number}
   * @memberof Game_BattlerBase
   */
  param(paramId: number): number;

  /**
   * Returns the Extra parameter, given a  x parameter id.
   * These include
   * Hit rate
   * Evasion rate
   * Critical rate
   * Magic evasion rate
   * Magic reflection rate
   * Counter attack rate
   * Hp regeneration rate
   * Mp regeneration rate
   * Tp regeneration rate
   * @param xparamId
   * @return Int
   */
  xparam(xparamId: number): number;

  sparam(sparamId: number): number;

  elementRate(elementId: number): number;

  debuffRate(paramId: number): number;

  stateRate(stateId: number): number;

  stateResistSet(): number[];

  isStateResist(stateId: number): boolean;

  /**
   * Returns the attack elements of the battler
   * as a list of numbers.
   * @returns {Array<number>}
   * @memberof Game_BattlerBase
   */
  attackElements(): number[];

  /**
   * Returns the attack states of the battler as a
   * list of numbers.
   * @returns {Array<number>}
   * @memberof Game_BattlerBase
   */
  attackStates(): number[];

  attackStatesRate(stateId: number): void;

  /**
   * Returns the attack speed of the battler.
   *
   * @returns {number}
   * @memberof Game_BattlerBase
   */
  attackSpeed(): number;

  /**
   * Returns the number of attacks available to the battler.
   *
   * @returns {number}
   * @memberof Game_BattlerBase
   */
  attackTimesAdd(): number;

  /**
   * Returns an array of integers
   * representing skill type ids.
   * @return Array<Int>
   */
  addedSkillTypes(): number[];

  isSkillTypeSealed(skilltypeId: number): boolean;

  addedSkills(): number[];

  isSkillSealed(skillId: number): boolean;

  isEquipWtypeOk(weaponTypeId: number): boolean;

  isEquipAtypeOk(armorTypeId: number): boolean;

  isEquipTypeLocked(equipmentTypeId: number): boolean;

  isEquipTypeSealed(equipmentTypeId: number): boolean;

  /**
   * Returns the battler slot type of a trait.
   *
   * @returns {number}
   * @memberof Game_BattlerBase
   */
  slotType(): number;

  /**
   * Returns true if the battler dual wields.
   *
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  isDualWield(): boolean;

  actionPlusSet(): number[];

  /**
   * Takes a special flag
   * @param flagId
   * @return Bool
   */
  specialFlag(flagId: number): boolean;

  /**
   * Returns the collapse type of the battler.
   * This is represented as an Int.
   * @returns {Int}
   * @memberof Game_BattlerBase
   */
  collapseType(): number;

  partyAbility(abilityId: number): boolean;

  /**
   * Returns true if the battler is set to battle automatically.
   *
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  isAutoBattle(): boolean;

  /**
   * Returns true if the battler is guarding.
   *
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  isGuard(): boolean;

  isSubstitute(): boolean;

  /**
   * Returns true if tp is preserved between battles.
   *
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  isPreserveTp(): boolean;

  addParam(paramId: number, value: number): void;

  /**
   * Sets the battler hp.
   *
   * @param {number} hp
   * @memberof Game_BattlerBase
   */
  setHp(hp: number): void;

  /**
   * Sets the battler mp.
   *
   * @param {number} mp
   * @memberof Game_BattlerBase
   */
  setMp(mp: number): void;

  /**
   * Sets the battler tp.
   *
   * @param {number} tp
   * @memberof Game_BattlerBase
   */
  setTp(tp: number): void;

  /**
   * Returns the maximum tp of the battler.
   *
   * @returns {number}
   * @memberof Game_BattlerBase
   */
  maxTp(): number;

  /**
   * Refreshes the battler.
   *
   * @memberof Game_BattlerBase
   */
  refresh(): void;

  /**
   * Recovers the battler from all states and restores the
   * battler to maximum hp and mp.
   * @memberof Game_BattlerBase
   */
  recoverAll(): void;

  /**
   * Returns the percentage of the battler's hp left as a float.
   *
   * @returns {number}
   * @memberof Game_BattlerBase
   */
  hpRate(): number;

  /**
   * Returns the percentage of the battler's mp left as a float.
   *
   * @returns {number}
   * @memberof Game_BattlerBase
   */
  mpRate(): number;

  /**
   * Returns the percentage of the battler's tp left as a float.
   *
   * @returns {number}
   * @memberof Game_BattlerBase
   */
  tpRate(): number;

  /**
   * Hides the game battler.
   *
   * @memberof Game_BattlerBase
   */
  hide(): void;

  /**
   * Shows the game battler.
   *
   * @memberof Game_BattlerBase
   */
  appear(): void;

  /**
   * Returns true if the game battler is hidden.
   *
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  isHidden(): boolean;

  /**
   * Returns true if the game battler is not hidden.
   *
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  isAppeared(): boolean;

  /**
   * Returns true if the battler is dead.
   *
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  isDead(): boolean;

  /**
   * Returns true if the battler is alive.
   *
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  isAlive(): boolean;

  /**
   * Returns true if the battler is dying.
   *
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  isDying(): boolean;

  /**
   * Returns true if the game battler is restricted.
   *
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  isRestricted(): boolean;

  /**
   * Returns true if the battler can input actions.
   *
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  canInput(): boolean;

  canMove(): boolean;

  /**
   * Returns true if the battler is confused.
   *
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  isConfused(): boolean;

  /**
   * Returns the confusion level of the battler.
   *
   * @returns {number}
   * @memberof Game_BattlerBase
   */
  confusionLevel(): number;

  /**
   * Returns true if the battler is an actor.
   *
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  isActor(): boolean;

  /**
   * Returns true if the battler is an enemy.
   *
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  isEnemy(): boolean;

  /**
   * Sorts the states attached to the battler.
   *
   * @memberof Game_BattlerBase
   */
  sortStates(): void;

  /**
   * Returns the number of the restriction.
   *
   * @returns {number}
   * @memberof Game_BattlerBase
   */
  restriction(): number;

  /**
   * Adds a new state given a state id to the battler.
   *
   * @param {number} stateId
   * @memberof Game_BattlerBase
   */
  addNewState(stateId: number): void;

  /**
   * Handler for when the battler is restricted.
   *
   * @memberof Game_BattlerBase
   */
  onRestrict(): void;

  mostImportantStateText(): string;

  stateMotionIndex(): number;

  stateOverlayIndex(): number;

  /**
   * Returns true if the skill is a weapon type
   * oriented skill.
   * @param {RPG.Skill} skill
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  isSkillWtypeOk(skill: rm.types.Skill): boolean;

  /**
   * Returns the mp cost of the skill.
   *
   * @param {RPG.Skill} skill
   * @returns {number}
   * @memberof Game_BattlerBase
   */
  skillMpCost(skill: rm.types.Skill): number;

  /**
   * Returns the tp cost of the skill.
   *
   * @param {RPG.Skill} skill
   * @returns {number}
   * @memberof Game_BattlerBase
   */
  skillTpCost(skill: rm.types.Skill): number;

  /**
   * Returns true if the battler can pay the cost
   * of the specified skill.
   * @param {RPG.Skill} skill
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  canPaySkillCost(skill: rm.types.Skill): boolean;

  /**
   * Pays the cost of the skill when activating the skill.
   *
   * @param {RPG.Skill} skill
   * @memberof Game_BattlerBase
   */
  paySkillCost(skill: rm.types.Skill): void;

  /**
   * Returns true if the item occasion is okay.
   *
   * @param {RPG.UsableItem} item
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  isOccasionOk(item: rm.types.UsableItem): boolean;

  meetsUsableItemConditions(item: rm.types.UsableItem): boolean;

  /**
   * Returns true if the battler meets the skill conditions.
   *
   * @param {RPG.Skill} skill
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  meetsSkillConditions(skill: rm.types.Skill): boolean;

  /**
   * Returns true if the battler meets the item conditions.
   *
   * @param {RPG.Item} item
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  meetsItemConditions(item: rm.types.Item): boolean;

  /**
   * Returns true if the battler can use the item.
   *
   * @param {RPG.UsableItem} item
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  canUse(item: rm.types.UsableItem): boolean;

  /**
   * Returns true if the battler can equip the item.
   *
   * @param {RPG.EquipItem} item
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  canEquip(item: rm.types.EquipItem): boolean;

  /**
   * Returns true if the battler can equip a weapon.
   *
   * @param {RPG.EquipItem} item
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  canEquipWeapon(item: rm.types.EquipItem): boolean;

  /**
   * Returns true if the battler can equip armor.
   *
   * @param {RPG.EquipItem} item
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  canEquipArmor(item: rm.types.EquipItem): boolean;

  /**
   * Returns the attack skill id in the database.
   *
   * @returns {number}
   * @memberof Game_BattlerBase
   */
  attackSkillid(): number;

  /**
   * Returns the guard skill id in the database.
   *
   * @returns {number}
   * @memberof Game_BattlerBase
   */
  guardSkillId(): number;

  /**
   * Returns true if the battler can attack.
   *
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  canAttack(): boolean;

  /**
   * Returns true if the battler can guard.
   *
   * @returns {Bool}
   * @memberof Game_BattlerBase
   */
  canGuard(): boolean;

  /**
   * Trait element rate; default to 11.
   */
  static TRAIT_ELEMENT_RATE: number;
  /**
   * Debuff Rate; default to 12.
   */
  static TRAIT_DEBUFF_RATE: number;
  /**
   * Trait state rate; default to 13.
   */
  static TRAIT_STATE_RATE: number;
  /**
   * Trait state resist; default to 14.
   */
  static TRAIT_STATE_RESIST: number;
  /**
   * Trait param; default to 21.
   */
  static TRAIT_PARAM: number;
  /**
   * Trait x param; default to 22.
   */
  static TRAIT_XPARAM: number;
  /**
   * Trait s param; default to 23.
   */
  static TRAIT_SPARAM: number;
  /**
   * Trait attack element; default to 31.
   */
  static TRAIT_ATTACK_ELEMENT: number;
  /**
   * Trait attack state; default to 32.
   */
  static TRAIT_ATTACK_STATE: number;
  /**
   * Trait attack speed; default to 33.
   */
  static TRAIT_ATTACK_SPEED: number;
  /**
   * Trait attack times; default to 34.
   */
  static TRAIT_ATTACK_TIMES: number;
  /**
   * Trait Stype(Skill type ) add; default to 41.
   */
  static TRAIT_STYPE_ADD: number;
  /**
   * Trait Stype(Skill type) seal; default to 42.
   */
  static TRAIT_STYPE_SEAL: number;
  /**
   * Trait Skill add; default to 43.
   */
  static TRAIT_SKILL_ADD: number;
  /**
   * Trait skill  seal; default to 44.
   */
  static TRAIT_SKILL_SEAL: number;
  /**
   * Trait WType(equip weapon type); default to 51.
   */
  static TRAIT_EQUIP_WTYPE: number;
  /**
   * Trait AType(Equip armor type); default to 52.
   */
  static TRAIT_EQUIP_ATYPE: number;
  /**
   * Trait equipment lock; default to 53.
   */
  static TRAIT_EQUIP_LOCK: number;
  /**
   * Trait equipment seal; default to 54.
   */
  static TRAIT_EQUIP_SEAL: number;
  /**
   * Trait slot type;default to 55.
   */
  static TRAIT_SLOT_TYPE: number;
  /**
   * Trait action plus; default to 61.
   */
  static TRAIT_ACTION_PLUS: number;
  /**
   * Trait special flag; default to 62.
   */
  static TRAIT_SPECIAL_FLAG: number;
  /**
   * Trait collapse type; default to 63.
   */
  static TRAIT_COLLAPSE_TYPE: number;
  /**
   * Trait party ability; default to 64.
   */
  static TRAIT_PARTY_ABILITY: number;
  /**
   * Flag ID auto battle; default to 0;
   */
  static FLAG_ID_AUTO_BATTLE: number;
  /**
   * Flag ID guard; default to 1.
   */
  static FLAG_ID_GUARD: number;
  /**
   * Flag Id substitude; default to 2.
   */
  static FLAG_ID_SUBSTITUTE: number;
  /**
   * Flag id preserve type; default to 3.
   */
  static FLAG_ID_PRESERVE_TP: number;
  /**
   * Icon buff start, default to 32.
   */
  static ICON_BUFF_START: number;
  /**
   * Icon debuff start; default to 48.
   */
  static ICON_DEBUFF_START: number;
}

declare namespace rm.types {
  export type BattlerAnimation = {
    animationId: number;
    delay: number;
    mirror: boolean;
  };
}

declare namespace rm.types {
  export const enum MotionType {
    WALK = 'walk',
    WAIT = 'wait',
    CHANT = 'chant',
    GUARD = 'guard',
    DAMAGE = 'damage',
    EVADE = 'evade',
    THRUST = 'thrust',
    MISSLE = 'missle',
    SKILL = 'SKILL',
    SPELL = 'spell',
    ITEM = 'item',
    ESCAPE = 'escape',
    VICTORY = 'victory',
    DYING = 'dying',
    ABNORMAL = 'abnormal',
    SLEEP = 'sleep',
    DEAD = 'dead',
  }
}

declare namespace rm.types {
  export const enum WeaponImageId {
    base = -1,
  }
}

declare namespace rm.types {
  export const enum AnimationId {
    base = -1,
  }
}

declare namespace rm.types {
  export const enum ActionState {
    base = '',
  }
}

declare class Game_Battler extends Game_BattlerBase {
  protected constructor();

  _actions: Game_Action[];
  _speed: number;
  _result: Game_ActionResult;
  _actionState: string;
  _lastTargetIndex: number;
  _animations: rm.types.BattlerAnimation[];
  _damagePopup: boolean;
  _effectType: string;
  _motionType: rm.types.MotionType;
  _weaponImageId: number;
  _motionRefresh: boolean;
  _selected: boolean;

  /**
   * Returns the name of the battler.
   *
   * @returns {String}
   * @memberof Game_Battler
   */
  name(): string;

  /**
   * Returns the battler name of the battler;
   * the battler name is associated with the file used as the battler graphic.
   * @returns {String}
   * @memberof Game_Battler
   */
  battlerName(): string;

  /**
   * Returns the index of the battler.
   *
   * @returns {number}
   * @memberof Game_Battler
   */
  index(): number;

  /**
   * Returns the unit of the battler; this is either the
   * game party or game troop.
   * @returns {Game_Unit}
   * @memberof Game_Battler
   */
  friendsUnit(): Game_Unit;

  /**
   * Returns the opponents unit; this is either
   * game party or game troop.
   * @returns {Game_Unit}
   * @memberof Game_Battler
   */
  opponentsUnit(): Game_Unit;

  /**
   * Clears animations from the battler.
   *
   * @memberof Game_Battler
   */
  clearAnimations(): void;

  /**
   * Clear damage pop up from the battler.
   *
   * @memberof Game_Battler
   */
  clearDamagePopup(): void;

  /**
   * Clear weapon animation from the battler.
   *
   * @memberof Game_Battler
   */
  clearWeaponAnimation(): void;

  /**
   * Clears effect from the battler.
   *
   * @memberof Game_Battler
   */
  clearEffect(): void;

  /**
   * Clears motion from the battler.
   *
   * @memberof Game_Battler
   */
  clearMotion(): void;

  requestEffect(effectType: string): void;

  /**
   * Request the specified motion on the game battler.
   *
   * @param {String} motionType
   * @memberof Game_Battler
   */
  requestMotion(motionType: string): void;

  requestMotionRefresh(): void;

  select(): void;

  deselect(): void;

  isAnimationRequested(): boolean;

  isDamagePopupRequested(): boolean;

  isEffectRequested(): boolean;

  /**
   * Returns true if a motion is requested.
   *
   * @returns {Bool}
   * @memberof Game_Battler
   */
  isMotionRequested(): boolean;

  /**
   * Returns true if a weapon animation is requested.
   *
   * @returns {Bool}
   * @memberof Game_Battler
   */
  isWeaponAnimationRequested(): boolean;

  isMotionRefreshRequested(): boolean;

  isSelected(): boolean;

  /**
   * Returns the effect type of the battler.
   *
   * @returns {String}
   * @memberof Game_Battler
   */
  effectType(): string;

  /**
   * Returns the motion type of the battler.
   *
   * @returns {String}
   * @memberof Game_Battler
   */
  motionType(): string;

  /**
   * Returns the weapon image id.
   *
   * @returns {number}
   * @memberof Game_Battler
   */
  weaponImageId(): rm.types.WeaponImageId;

  /**
   * Shifts the battler animation.
   *
   * @returns {BattlerAnimation}
   * @memberof Game_Battler
   */
  shiftAnimation(): rm.types.BattlerAnimation;

  /**
   * Starts the specified animation, given the animation id on the
   * battler.
   * @param {number} animationId
   * @param {Bool} mirror
   * @param {number} delay
   * @memberof Game_Battler
   */
  startAnimation(animationId: rm.types.AnimationId, mirror: boolean, delay: number): void;

  /**
   * Starts a damage pop up on the battler.
   *
   * @memberof Game_Battler
   */
  startDamagePopup(): void;

  /**
   * Starts the weapon animation on te battler given a weapon id.
   *
   * @param {number} weaponImageId
   * @memberof Game_Battler
   */
  startWeaponAnimation(weaponImageId: rm.types.WeaponImageId): void;

  action(index: number): Game_Action;

  /**
   * Sets the action at the specified index for the battler.
   *
   * @param {number} index
   * @param {Game_Action} action
   * @memberof Game_Battler
   */
  setAction(index: number, action: Game_Action): void;

  /**
   * Returns the number of battler actions.
   *
   * @returns {number}
   * @memberof Game_Battler
   */
  numActions(): number;

  /**
   * Clears the battler actions.
   *
   * @memberof Game_Battler
   */
  clearActions(): void;

  /**
   * Returns the battler action result.
   *
   * @returns {Game_ActionResult}
   * @memberof Game_Battler
   */
  result(): Game_ActionResult;

  /**
   * Clears the battler action result.
   *
   * @memberof Game_Battler
   */
  clearResult(): void;

  /**
   * Refreshes the battler.
   *
   * @memberof Game_Battler
   */
  refresh(): void;

  /**
   * Adds a state to the battler given the specified
   * state id.
   * @param {number} stateId
   * @memberof Game_Battler
   */
  addState(stateId: number): void;

  /**
   * Returns true if the specified state given the state id
   * is addable.
   * @param {number} stateId
   * @returns {Bool}
   * @memberof Game_Battler
   */
  isStateAddable(stateId: number): boolean;

  /**
   * Returns true if the specified state given the state id
   * restricts.
   *
   * @param {number} stateId
   * @returns {Bool}
   * @memberof Game_Battler
   */
  isStateRestrict(stateId: number): boolean;

  /**
   * Handler for when theb attler is restricted.
   *
   * @memberof Game_Battler
   */
  onRestrict(): void;

  /**
   * Removes the specified state given the state id.
   *
   * @param {number} stateId
   * @memberof Game_Battler
   */
  removeState(stateId: number): void;

  /**
   * Has the battler escape from battle; plays a sound on escaping.
   *
   * @memberof Game_Battler
   */
  escape(): void;

  /**
   * Adds a buff to the battler for the specified number of turns
   * on the selected parameter.
   * @param {number} paramId
   * @param {number} turns
   * @memberof Game_Battler
   */
  addBuff(paramId: number, turns: number): void;

  /**
   * Adds a debuff to the battler for the specified number of turns
   * on the selected parameter.
   * @param {number} paramId
   * @param {number} turns
   * @memberof Game_Battler
   */
  addDebuff(paramId: number, turns: number): void;

  removeBuff(paramId: number): void;

  removeBattleStates(): void;

  /**
   * Removes all buffs from the battler.
   *
   * @memberof Game_Battler
   */
  removeAllBuffs(): void;

  removeStatesAuto(timing: number): void;

  removeBuffsAuto(): void;

  removeStatesByDamage(): void;

  /**
   * Creates the number of times for
   * an action.
   * @returns {number}
   * @memberof Game_Battler
   */
  makeActionTimes(): number;

  /**
   * Creates the actions for the battler.
   *
   * @memberof Game_Battler
   */
  makeActions(): void;

  /**
   * Returns the speed of the battler.
   *
   * @returns {number}
   * @memberof Game_Battler
   */
  speed(): number;

  /**
   * Calculates the speed of the battler.
   *
   * @memberof Game_Battler
   */
  makeSpeed(): void;

  /**
   * Returns the current action of the battler.
   *
   * @returns {Game_Action}
   * @memberof Game_Battler
   */
  currentAction(): Game_Action;

  /**
   * Removes the current battler action.
   *
   * @memberof Game_Battler
   */
  removeCurrentAction(): void;

  /**
   * Sets the last target based on the target passed in.
   * @param target
   */
  setLastTarget(target: Game_Battler): void;

  forceAction(skillId: number, targetIndex: number): void;

  /**
   * Has theb attler use the given item.
   *
   * @param {RPG.UsableItem} item
   * @memberof Game_Battler
   */
  useItem(item: rm.types.UsableItem): void;

  /**
   * Has the battler consume the given item.
   *
   * @param {RPG.UsableItem} item
   * @memberof Game_Battler
   */
  consumeItem(item: rm.types.UsableItem): void;

  /**
   * Adds the specified amount of hp to the battler.
   *
   * @param {number} value
   * @memberof Game_Battler
   */
  gainHp(value: number): void;

  /**
   * Adds the specified amount of mp to the battler.
   *
   * @param {number} value
   * @memberof Game_Battler
   */
  gainMp(value: number): void;

  /**
   * Adds the specified amount of tp to the battler.
   *
   * @param {number} value
   * @memberof Game_Battler
   */
  gainTp(value: number): void;

  /**
   * Adds a specified amount of tp to the battler silently.
   *
   * @param {number} value
   * @memberof Game_Battler
   */
  gainSilentTp(value: number): void;

  /**
   * Initializes the battler's tp; tp is random.
   *
   * @memberof Game_Battler
   */
  initTp(): void;

  /**
   * Clears the battler's tp.
   *
   * @memberof Game_Battler
   */
  clearTp(): void;

  chargeTpByDamage(damageRate: number): void;

  /**
   * Has the battler regenerate hp based on their hp regen.
   *
   * @memberof Game_Battler
   */
  regenerateHp(): void;

  maxSlipDamage(): number;

  /**
   * Has the battler regenerate mp based on their mp regen.
   *
   * @memberof Game_Battler
   */
  regenerateMp(): void;

  /**
   * Has the battler regenerate tp based on their tp regen.
   *
   * @memberof Game_Battler
   */
  regenerateTp(): void;

  /**
   * Has the battler regenerate all resources based on
   * their respective regeneration stats.
   * @memberof Game_Battler
   */
  regenerateAll(): void;

  /**
   * Handler for when battle has started.
   *
   * @memberof Game_Battler
   */
  onBattleStart(): void;

  /**
   * Handler for when all actions end
   */
  onAllActionsEnd(): void;

  /**
   * Handler for when turn ends
   */
  onTurnEnd(): void;

  /**
   * Handler for when battle ends
   */
  onBattleEnd(): void;

  /**
   * Handler for when damage is done
   * @param value
   */
  onDamage(value: number): void;

  /**
   * Sets the action state
   * @param actionState
   */
  setActionState(actionState: rm.types.ActionState): void;

  isUndecided(): boolean;

  /**
   * Returns true if the battler is inputting commands in battle.
   *
   * @returns {Bool}
   * @memberof Game_Battler
   */
  isInputting(): boolean;

  /**
   * Returns true if the battler is waiting in battle.
   *
   * @returns {Bool}
   * @memberof Game_Battler
   */
  isWaiting(): boolean;

  /**
   * Returns true if the battler is performing an action in battle.
   *
   * @returns {Bool}
   * @memberof Game_Battler
   */
  isActing(): boolean;

  /**
   * Returns true if the battler is chanting in combat.
   *
   * @returns {Bool}
   * @memberof Game_Battler
   */
  isChanting(): boolean;

  /**
   * Returns true if the battler is waiting to guard.
   *
   * @returns {Bool}
   * @memberof Game_Battler
   */
  isGuardWaiting(): boolean;

  /**
   * Perform action start motion, given the specified game action.
   *
   * @param {Game_Action} action
   * @memberof Game_Battler
   */
  performActionStart(action: Game_Action): void;

  /**
   * Perform given action motion.
   *
   * @param {Game_Action} action
   * @memberof Game_Battler
   */
  performAction(action: Game_Action): void;

  /**
   * Perform action end motion.
   *
   * @memberof Game_Battler
   */
  performActionEnd(): void;

  /**
   * Perform damage motion.
   *
   * @memberof Game_Battler
   */
  performDamage(): void;

  /**
   * Perform miss motion.
   *
   * @memberof Game_Battler
   */
  performMiss(): void;

  /**
   * Perform recovery motion.
   *
   * @memberof Game_Battler
   */
  performRecovery(): void;

  /**
   * Perform evasion motion.
   *
   * @memberof Game_Battler
   */
  performEvasion(): void;

  /**
   * Perform magic evasion motion.
   *
   * @memberof Game_Battler
   */
  performMagicEvasion(): void;

  /**
   * Perform counter motion.
   *
   * @memberof Game_Battler
   */
  performCounter(): void;

  /**
   * Performs the reflect motion.
   *
   * @memberof Game_Battler
   */
  performReflection(): void;

  /**
   * Perform substitute motion with the specified game battler.
   *
   * @param {Game_Battler} target
   * @memberof Game_Battler
   */
  performSubstitute(target: Game_Battler): void;

  /**
   * Performs the collapse motion.
   *
   * @memberof Game_Battler
   */
  performCollapse(): void;
}

declare namespace rm.types {
  /**
   * The data class for actors.
   */
  export type Actor = {
    /**
     * The file name of the actor's battler graphic.
     */
    battlerName: string;
    /**
     * The index (0..7) of the actor's walking graphic.
     */
    characterIndex: number;
    /**
     * The file name of the actor's walking graphic.
     */
    characterName: string;
    /**
     * The actor's class ID.
     */
    classId: number;
    doc: string;
    /**
     * The actor's initial equipment. An array of weapon IDs or armor IDs with the following subscripts:
     */
    equips: number[];
    /**
     * The index (0..7) of the actor's face graphic.
     */
    faceIndex: number;
    /**
     * The file name of the actor's face graphic.
     */
    faceName: string;
    /**
     * The ID.
     */
    id: string;
    /**
     * The actor's initial level.
     */
    initialLevel: number;
    internal: boolean;
    links: string[];
    /**
     * The actor's max level
     */
    maxLevel: number;
    /**
     * The name.
     */
    name: string;
    /**
     * The actor's nickname.
     */
    nickname: string;
    parameters: string[];
    platforms: haxe.display.Platform[];
    /**
     * The profile.
     */
    profile: string;
    targets: haxe.display.MetadataTarget[];
    /**
     * The array of Trait data.
     */
    traits: rm.types.Trait[];
  };
}

declare namespace rm.types {
  /**
   * The data class for weapons.
   */
  export type Weapon = {
    /**
     * The animation ID when using the weapon.
     */
    animationId: number;
    /**
     * The description text.
     */
    description: string;
    /**
     * The type of weapon or armor.
     *
     * 0: Weapon
     * 1: Shield
     * 2: Head
     * 3: Body
     * 4: Accessory
     */
    etypeId: number;
    /**
     * The icon number.
     */
    iconIndex: number;
    /**
     * The item ID.
     */
    id: number;
    meta: Object;
    /**
     * The item name.
     */
    name: string;
    note: string;
    /**
     * The amount of parameter change. An array of integers using the following IDs as subscripts:
     *
     * 0: Maximum hit points
     * 1: Maximum magic points
     * 2: Attack power
     * 3: Defense power
     * 4: Magic attack power
     * 5: Magic defense power
     * 6: Agility
     * 7: Luck
     */
    params: number[];
    /**
     * The price of the weapon or armor.
     */
    price: number;
    /**
     * The array of Trait data.
     */
    traits: rm.types.Trait[];
    /**
     * The weapon type ID.
     */
    wtypeId: number;
  };
}

declare namespace rm.types {
  /**
   * The data class for armor.
   */
  export type Armor = {
    /**
     * The armor type ID.
     */
    atypeId: number;
    /**
     * The description text.
     */
    description: string;
    /**
     * The type of weapon or armor.
     *
     * 0: Weapon
     * 1: Shield
     * 2: Head
     * 3: Body
     * 4: Accessory
     */
    etypeId: number;
    /**
     * The icon number.
     */
    iconIndex: number;
    /**
     * The item ID.
     */
    id: number;
    meta: Object;
    /**
     * The item name.
     */
    name: string;
    note: string;
    /**
     * The amount of parameter change. An array of integers using the following IDs as subscripts:
     *
     * 0: Maximum hit points
     * 1: Maximum magic points
     * 2: Attack power
     * 3: Defense power
     * 4: Magic attack power
     * 5: Magic defense power
     * 6: Agility
     * 7: Luck
     */
    params: number[];
    /**
     * The price of the weapon or armor.
     */
    price: number;
    /**
     * The array of Trait data.
     */
    traits: rm.types.Trait[];
  };
}

declare namespace rm.types {
  /**
   * The data class for a class's [Skills to Learn].
   */
  export type ClassLearning = {
    doc: string;
    internal: boolean;
    /**
     * The data class for a class's [Skills to Learn].
     */
    level: number;
    links: string[];
    name: string;
    parameters: string[];
    platforms: haxe.display.Platform[];
    /**
     * The ID of the skill to learn.
     */
    skillId: number;
    targets: haxe.display.MetadataTarget[];
  };
}

declare namespace rm.types {
  /**
   * The data class for class.
   */
  export type RPGClass = {
    doc: string;
    /**
     * An array of values that decides the experience curve. The subscripts are as follows:
     *
     * 0: Base value
     * 1: Extra value
     * 2: Acceleration A
     * 3: Acceleration B
     */
    expParams: number[];
    /**
     * The ID.
     */
    id: number;
    internal: boolean;
    /**
     * The skills to learn. An array of RPG.Class.Learning.
     */
    learnings: rm.types.ClassLearning[];
    links: string[];
    /**
     * The name.
     */
    name: string;
    parameters: string[];
    /**
     * The parameter development curve. A 2-dimensional array containing ordinary parameters according to level (Table).
     *
     * The format is params[param_id, level], and param_id is assigned as follows:
     *
     * 0: Maximum hit points
     * 1: Maximum magic points
     * 2: Attack power
     * 3: Defense power
     * 4: Magic attack power
     * 5: Magic defense power
     * 6: Agility
     * 7: Luck
     */
    params: number[][];
    platforms: haxe.display.Platform[];
    targets: haxe.display.MetadataTarget[];
    /**
     * The array of Trait data.
     */
    traits: rm.types.Trait[];
  };
}

declare class Game_Actor extends Game_Battler {
  constructor(actorId: number);

  _actorId: number;
  _name: string;
  _nickname: string;
  _profile: string;
  _classId: number;
  _level: number;
  _characterName: string;
  _characterIndex: number;
  _faceName: string;
  _faceIndex: number;
  _battlerName: string;
  _exp: Object;
  /**
   * Skill Ids
   */
  _skills: number[];
  _equips: Game_Item[];
  _actionInputIndex: number;
  _lastMenuSkill: Game_Item;
  _lastBattleSkill: Game_Item;
  _lastCommandSymbol: string;
  _stateSteps: Object;
  /**
   * [read-only]
   */
  level: number;

  setup(actorId: number): void;

  /**
   * Returns the id of the actor.
   *
   * @returns {Int}
   * @memberof Game_Actor
   */
  actorId(): number;

  /**
   * Returns the database information of the actor.
   *
   * @returns {Actor}
   * @memberof Game_Actor
   */
  actor(): rm.types.Actor;

  /**
   * Returns the actor's name.
   * @return String
   */
  name(): string;

  /**
   * Sets the actor name.
   *
   * @param {String} name
   * @memberof Game_Actor
   */
  setName(name: string): void;

  /**
   * Returns the nickname of the actor.
   *
   * @returns {String}
   * @memberof Game_Actor
   */
  nickname(): string;

  /**
   * Sets the nickname of the actor.
   *
   * @param {String} nickname
   * @memberof Game_Actor
   */
  setNickname(nickname: string): void;

  /**
   * Returns the actor profile.
   *
   * @returns {String}
   * @memberof Game_Actor
   */
  profile(): string;

  /**
   * Sets the actor profile.
   *
   * @param {String} profile
   * @memberof Game_Actor
   */
  setProfile(profile: string): void;

  /**
   * Returns the face name of the actor; this is
   * the image of faces for the actor.
   * @returns {String}
   * @memberof Game_Actor
   */
  faceName(): string;

  /**
   * Returns the face index of the actor.
   *
   * @returns {Int}
   * @memberof Game_Actor
   */
  faceIndex(): number;

  /**
   * Clears all states from the actor.
   *
   * @memberof Game_Actor
   */
  clearStates(): void;

  /**
   * Erase the specified state from the actor.
   *
   * @param {Int} stateId
   * @memberof Game_Actor
   */
  eraseState(stateId: number): void;

  /**
   * Reset state count of the specified state.
   *
   * @param {Int} stateId
   * @memberof Game_Actor
   */
  resetStateCounts(stateId: number): void;

  /**
   * Initialize images of the actor.
   *
   * @memberof Game_Actor
   */
  initImages(): void;

  /**
   * Returns the exp required to level.
   *
   * @param {Int} level
   * @returns {Int}
   * @memberof Game_Actor
   */
  expForLevel(level: number): number;

  /**
   * Initialize exp of the actor.
   *
   * @memberof Game_Actor
   */
  initExp(): void;

  /**
   * Returns the current experience points of the actor.
   *
   * @returns {Int}
   * @memberof Game_Actor
   */
  currentExp(): number;

  /**
   * Returns the current level's experience for the actor.
   *
   * @returns {Int}
   * @memberof Game_Actor
   */
  currentLevelExp(): number;

  /**
   * Returns the experience points for the next level of the actor.
   *
   * @returns {Int}
   * @memberof Game_Actor
   */
  nextLevelExp(): number;

  /**
   * Returns the next required experience points for the actor to level up.
   *
   * @returns {Int}
   * @memberof Game_Actor
   */
  nextRequiredExp(): number;

  /**
   * Returns the maximum level of the actor.
   *
   * @memberof Game_Actor
   */
  maxLevel(): void;

  /**
   * Returns true if the actor is max level.
   *
   * @returns {Bool}
   * @memberof Game_Actor
   */
  isMaxLevel(): boolean;

  /**
   * Initialize actor skills.
   *
   * @memberof Game_Actor
   */
  initSkills(): void;

  /**
   * Initialize actor equipment in the given slots.
   *
   * @param {Array<Int>} equips
   * @memberof Game_Actor
   */
  initEquips(equips: number[]): void;

  /**
   * Returns the equip slots of the actor.
   *
   * @returns {Array<Int>}
   * @memberof Game_Actor
   */
  equipSlots(): number[];

  /**
   * Returns the equipment of the actor.
   *
   * @returns {Array<EquipItem>}
   * @memberof Game_Actor
   */
  equips(): rm.types.EquipItem[];

  /**
   * Returns the weapon of the actor.
   *
   * @returns {Array<Weapon>}
   * @memberof Game_Actor
   */
  weapons(): rm.types.Weapon[];

  /**
   * Returns the armor of the actor.
   *
   * @returns {Array<Armor>}
   * @memberof Game_Actor
   */
  armors(): rm.types.Armor[];

  /**
   * Returns true if the actor has a weapon.
   *
   * @param {Weapon} weapon
   * @returns {Bool}
   * @memberof Game_Actor
   */
  hasWeapon(weapon: rm.types.Weapon): boolean;

  /**
   * Returns true if the actor has armor.
   *
   * @param {Armor} armor
   * @returns {Bool}
   * @memberof Game_Actor
   */
  hasArmor(armor: rm.types.Armor): boolean;

  /**
   * Returns true if the equip change is okay in the given slot.
   *
   * @param {Int} slotId
   * @returns {Bool}
   * @memberof Game_Actor
   */
  isEquipChangeOk(slotId: number): boolean;

  /**
   * Changes the actor equipment in the given slot with the
   * given equip item. Places the original item into the party
   * inventory.
   * @param {Int} slotId
   * @param {EquipItem} item
   * @memberof Game_Actor
   */
  changeEquip(slotId: number, item: rm.types.EquipItem): void;

  /**
   * Forces the actor to change equipment in the given slot
   * with the given equip item without placing the item back into
   * the party inventory.
   * @param {Int} slotId
   * @param {EquipItem} item
   * @memberof Game_Actor
   */
  forceChangeEquip(slotId: number, item: rm.types.EquipItem): void;

  /**
   * Trades the new item with the old item in the party inventory.
   *
   * @param {EquipItem} newItem
   * @param {EquipItem} oldItem
   * @returns {Bool}
   * @memberof Game_Actor
   */
  tradeItemWithParty(newItem: rm.types.EquipItem, oldItem: rm.types.EquipItem): boolean;

  /**
   * Changes the actor equip with an item based on the equip id.
   *
   * @param {Int} etypeId
   * @param {Int} itemId
   * @memberof Game_Actor
   */
  changeEquipById(etypeId: number, itemId: number): void;

  /**
   * Returns true if the actor is equipped with the specific item.
   *
   * @param {EquipItem} item
   * @returns {Bool}
   * @memberof Game_Actor
   */
  isEquipped(item: rm.types.EquipItem): boolean;

  /**
   * Discards the given equip item from the actor; item
   * is not return to the party inventory.
   * @param {EquipItem} item
   * @memberof Game_Actor
   */
  discardEquip(item: rm.types.EquipItem): void;

  /**
   * Returns items the actor can't normally equip to the party inventory.
   *
   * @param {Bool} forcing
   * @memberof Game_Actor
   */
  releaseUnequippableItems(forcing: boolean): void;

  /**
   * Clears the actor's equipment; items are returned to the inventory.
   *
   * @memberof Game_Actor
   */
  clearEquipments(): void;

  /**
   * Optimize the actor's equipment.
   *
   * @memberof Game_Actor
   */
  optimizeEquipments(): void;

  /**
   * Equips the best item in the given slot.
   *
   * @param {Int} slotId
   * @memberof Game_Actor
   */
  bestEquipItem(slotId: number): void;

  /**
   * Calculates the equip item performance and returns the sum/difference.
   *
   * @param {EquipItem} item
   * @returns {Int}
   * @memberof Game_Actor
   */
  calcEquipItemPerformance(item: rm.types.EquipItem): number;

  isSkillWtypeOk(skill: rm.types.Skill): boolean;

  isWtypeEquipped(wtypeId: number): boolean;

  /**
   * Refreshes the actor.
   *
   * @memberof Game_Actor
   */
  refresh(): void;

  friendsUnit(): Game_Party;

  opponentsUnit(): Game_Temp;

  /**
   * Returns true if the actor is a member in battle.
   *
   * @returns {Bool}
   * @memberof Game_Actor
   */
  isBattleMember(): boolean;

  isFormationChangeOk(): boolean;

  /**
   * Returns the current class of the actor from the database.
   *
   * @returns {Class}
   * @memberof Game_Actor
   */
  currentClass(): rm.types.RPGClass;

  /**
   * Returns true if the actor is the specified class from the database.
   *
   * @param {Class} gameClass
   * @returns {Bool}
   * @memberof Game_Actor
   */
  isClass(gameClass: rm.types.RPGClass): boolean;

  /**
   * Returns the actor's skills; even if the skills are not usable.
   *
   * @returns {Array<Skill>}
   * @memberof Game_Actor
   */
  skills(): rm.types.Skill[];

  /**
   * Returns the usable skills of the actor.
   *
   * @returns {Array<Skill>}
   * @memberof Game_Actor
   */
  usableSkills(): rm.types.Skill[];

  /**
   * Returns the attack element ids.
   *
   * @returns {Array<Int>}
   * @memberof Game_Actor
   */
  attackElements(): number[];

  /**
   * Returns true if the actor has no weapon.
   *
   * @returns {Bool}
   * @memberof Game_Actor
   */
  hasNoWeapons(): boolean;

  /**
   * Returns the element id of barehanded attacks.
   * By default this is 1.
   *
   * @returns {Int}
   * @memberof Game_Actor
   */
  bareHandsElementId(): number;

  /**
   * Returns the base value of the parameter.
   * @param paramId
   * @return Int
   */
  paramBase(paramId: number): number;

  /**
   * Returns the first attack animation id.
   *
   * @returns {Int}
   * @memberof Game_Actor
   */
  attackAnimationId1(): number;

  /**
   * Returns the second attack animation id.
   *
   * @returns {Int}
   * @memberof Game_Actor
   */
  attackAnimationId2(): number;

  /**
   * Returns the animation id for a barehanded attack.
   *
   * @returns {Int}
   * @memberof Game_Actor
   */
  bareHandsAnimationId(): number;

  /**
   * Change the actor experience points; leveling up the actor
   * if it's above the required exp for the current level.
   * If show is set to true, actor level up with be displayed.
   * @param {Int} exp
   * @param {Bool} show
   * @memberof Game_Actor
   */
  changeExp(exp: number, show: boolean): void;

  /**
   * Level up the actor.
   *
   * @memberof Game_Actor
   */
  levelUp(): void;

  /**
   * Level down the actor.
   *
   * @memberof Game_Actor
   */
  levelDown(): void;

  findNewSkills(lastSkills: rm.types.Skill[]): rm.types.Skill[];

  /**
   * Displays the actor level up in a message window, with the learned skills.
   *
   * @param {Array<Skill>} newSkills
   * @memberof Game_Actor
   */
  displayLevelUp(newSkills: rm.types.Skill[]): void;

  /**
   * Gives the specified exp to the actor.
   *
   * @param {Int} exp
   * @memberof Game_Actor
   */
  gainExp(exp: number): void;

  /**
   * Returns the final exp rate of the actor based on if the actor
   * is a reserved party member or an active battle member.
   * @returns {Int}
   * @memberof Game_Actor
   */
  finalExpRate(): number;

  /**
   * Returns the exp rate of actors not in battle; this is set in the database.
   *
   * @returns {Int}
   * @memberof Game_Actor
   */
  benchMembersExpRate(): number;

  /**
   * Returns true if the actor should display level up in a message window.
   *
   * @returns {Bool}
   * @memberof Game_Actor
   */
  shouldDisplayLevelUp(): boolean;

  /**
   * Changes the actor level; if show is set to true,
   * the actor level will be displayed.
   * @param {Int} level
   * @param {Bool} show
   * @memberof Game_Actor
   */
  changeLevel(level: number, show: boolean): void;

  /**
   * Actor learns the specified skill given the skill id.
   *
   * @param {Int} skillId
   * @memberof Game_Actor
   */
  learnSkill(skillId: number): void;

  /**
   * Actor forgets the specified skill given the skill id from
   * the actor's usable skills.
   * @param {Int} skillId
   * @memberof Game_Actor
   */
  forgetSkill(skillId: number): void;

  /**
   * Returns true if the actor has learned the specified
   * skill given the specified skill id.
   * @param {Int} skillId
   * @returns {Bool}
   * @memberof Game_Actor
   */
  isLearnedSkill(skillId: number): boolean;

  /**
   * Changes the actor class; if keep is true, the actor
   * will retain their experience points.
   * @param {Int} classId
   * @param {Bool} keepExp
   * @memberof Game_Actor
   */
  changeClass(classId: number, keepExp: boolean): void;

  setCharacterImage(characterName: string, characterIndex: number): void;

  /**
   * Sets the face image of the actor given the face image (from database)
   * and face index within the iamge.
   *
   * @param {String} faceName
   * @param {Int} faceIndex
   * @memberof Game_Actor
   */
  setFaceImage(faceName: string, faceIndex: number): void;

  /**
   * Sets the battler image of the actor; this is the sprite displayed
   * in the side view mode.
   * @param {String} battlerName
   * @memberof Game_Actor
   */
  setBattlerImage(battlerName: string): void;

  /**
   * Returns true if the actor sprite is visible.
   *
   * @returns {Bool}
   * @memberof Game_Actor
   */
  isSpriteVisible(): boolean;

  /**
   * Starts the animation on the actor given the specified animation id;
   * if  mirror is set to true, the animation will be mirrored. If a delay is enter,
   * the animation will be delayed.
   * @param {Int} animationId
   * @param {Bool} mirror
   * @param {Int} delay
   * @memberof Game_Actor
   */
  startAnimation(animationId: number, mirror: boolean, delay: number): void;

  /**
   * Performs the attack motion for the actor.
   *
   * @memberof Game_Actor
   */
  performAttack(): void;

  /**
   * Perform the victory motion for the actor.
   *
   * @memberof Game_Actor
   */
  performVictory(): void;

  /**
   * Performs the escape motion for the actor.
   *
   * @memberof Game_Actor
   */
  performEscape(): void;

  /**
   * Creates the action list for the actor.
   *
   * @returns {Array<Game_Action>}
   * @memberof Game_Actor
   */
  makeActionList(): Game_Action[];

  /**
   * Creates the auto battle actions for the game actor.
   *
   * @memberof Game_Actor
   */
  makeAutoBattleActions(): void;

  makeConfusionActions(): void;

  /**
   * Handler for when the player walks on the map scene.
   *
   * @memberof Game_Actor
   */
  onPlayerWalk(): void;

  updateStateSteps(state: rm.types.State): void;

  /**
   * Shows the added states to the actor.
   *
   * @memberof Game_Actor
   */
  showAddedStates(): void;

  /**
   * Shows the removed states from the actor.
   *
   * @memberof Game_Actor
   */
  showRemovedStates(): void;

  stepsForTurn(): number;

  turnEndOnMap(): void;

  /**
   * Checks the effect of the floor on the actor.
   *
   * @memberof Game_Actor
   */
  checkFloorEffect(): void;

  /**
   * Executes the floor dmaage on the actor.
   *
   * @memberof Game_Actor
   */
  executeFloorDamage(): void;

  /**
   * Returns the basic floor damage.
   *
   * @returns {Int}
   * @memberof Game_Actor
   */
  basicFloorDamage(): number;

  /**
   * Returns the max floor damage.
   *
   * @returns {Int}
   * @memberof Game_Actor
   */
  maxFloorDamage(): number;

  /**
   * Perform damage to the actor on the map scene.
   *
   * @memberof Game_Actor
   */
  performMapDamage(): void;

  /**
   * Clears all of the actor's animations.
   *
   * @memberof Game_Actor
   */
  clearActions(): void;

  /**
   * Returns action the actor is inputting.
   *
   * @returns {Game_Action}
   * @memberof Game_Actor
   */
  inputtingAction(): Game_Action;

  selectNextCommand(): boolean;

  selectPreviousCommand(): boolean;

  /**
   * Returns the last menu skill of the actor.
   *
   * @returns {Skill}
   * @memberof Game_Actor
   */
  lastMenuSkill(): rm.types.Skill;

  setLastMenuSkill(skill: rm.types.Skill): void;

  /**
   * Returns the last battle skill of the actor.
   *
   * @returns {Skill}
   * @memberof Game_Actor
   */
  lastBattleSkill(): rm.types.Skill;

  setLastBattleSkill(skill: rm.types.Skill): void;

  /**
   * Returns the last command symbol that the actor used.
   *
   * @returns {String}
   * @memberof Game_Actor
   */
  lastCommandSymbol(): string;

  /**
   * Sets the last command symbol to the given symbol; this is the
   * selected command in the battle menu.
   * @param {String} symbol
   * @memberof Game_Actor
   */
  setLastCommandSymbol(symbol: string): void;

  /**
   * Returns true if the item effect  has a special effect from game action.
   * @param item
   * @return Bool
   */
  testEscape(item: rm.types.BaseItem): boolean;
}

declare class Game_Actors {
  constructor();

  /**
   * List of all Game_Actor in the database.
   */
  _data: Game_Actor[];

  initialize(): void;

  /**
   * Returns the actor with the specified id.
   *
   * @param {number} actorId
   * @returns {Game_Actor}
   * @memberof Game_Actors
   */
  actor(actorId: number): Game_Actor;
}

declare namespace rm.types {
  export const enum MoveSpeed {
    X8SLOWER = 1,
    X4SLOWER = 2,
    X2SLOWER = 3,
    NORMAL = 4,
    X2FASTER = 5,
    X4FASTER = 6,
  }
}

declare namespace rm.types {
  export const enum MoveFrequency {
    LOWEST = 1,
    LOWER = 2,
    NORMAL = 3,
    HIGHER = 4,
    HIGHEST = 5,
  }
}

declare namespace rm.types {
  export const enum Direction {
    base = -1,
  }
}

declare namespace rm.types {}

declare namespace rm.types {
  export const enum CharacterPriority {
    BELOW_CHARACTERS = 0,
    SAME_AS_CHARACTERS = 1,
    ABOVE_CHARACTERS = 2,
  }
}

declare namespace rm.types {
  export const enum BalloonId {
    EXCLAMATION = 0,
    QUESTION = 1,
    MUSIC_NOTE = 2,
    HEART = 3,
    ANGER = 4,
    SWEAT = 5,
    COBWEB = 6,
    SILENCE = 7,
    LIGHT_BULB = 8,
    ZZZ = 9,
  }
}

/**
 * The super class of Game Character. It handles basic
 * information such as coordinates, images, and more
 * shared by all characters.
 */
declare class Game_CharacterBase {
  constructor();

  _x: number;
  _y: number;
  _realX: number;
  _realY: number;
  _moveSpeed: rm.types.MoveSpeed;
  _moveFrequency: rm.types.MoveFrequency;
  _opacity: number;
  _blendMode: number;
  _direction: rm.types.Direction;
  _pattern: rm.types.CharacterPattern;
  _priorityType: rm.types.CharacterPriority;
  _tileId: number;
  _characterName: string;
  _characterIndex: number;
  _isObjectCharacter: boolean;
  _walkAnime: boolean;
  _stepAnime: boolean;
  _directionFix: boolean;
  _through: boolean;
  _transparent: boolean;
  _bushDepth: number;
  _animationId: rm.types.AnimationId;
  _ballonId: rm.types.BalloonId;
  _animationPlaying: boolean;
  _balloonPlaying: boolean;
  _animationCount: number;
  _stopCount: number;
  _jumpCount: number;
  _jumpPeak: number;
  _movementSuccess: boolean;

  initialize(): void;

  /**
   * [read-only]
   */
  x: number;
  /**
   * [read-only]
   */
  y: number;

  initMembers(): void;

  pos(x: number, y: number): boolean;

  posNt(x: number, y: number): boolean;

  /**
   * Returns the move speed of the game character.
   *
   * @returns {Int}
   * @memberof Game_CharacterBase
   */
  moveSpeed(): number;

  setMoveSpeed(moveSpeed: number): void;

  /**
   * Returns the move frequency of the character.
   *
   * @returns {Int}
   * @memberof Game_CharacterBase
   */
  moveFrequency(): number;

  setMoveFrequency(moveFrequency: number): void;

  opacity(): number;

  setOpacity(opacity: number): void;

  /**
   * Returns the blend mode of the character;
   * these are represented by Ints.
   * @returns {Int}
   * @memberof Game_CharacterBase
   */
  blendMode(): number;

  /**
   * Sets the blend mode of the character;
   * these are represented by Ints.
   * @param {Int} blendMode
   * @memberof Game_CharacterBase
   */
  setBlendMode(blendMode: number): void;

  /**
   * Returns true if the character is
   * normal priority; this means you can collide with them.
   * @returns {Bool}
   * @memberof Game_CharacterBase
   */
  isNormalPriority(): boolean;

  setPriorityType(priorityType: number): void;

  /**
   * Returns true if the character is moving.
   *
   * @returns {Bool}
   * @memberof Game_CharacterBase
   */
  isMoving(): boolean;

  /**
   * Returns true if the character is jumping.
   *
   * @returns {Bool}
   * @memberof Game_CharacterBase
   */
  isJumping(): boolean;

  /**
   * Returns the jump height of base character.
   *
   * @returns {Int}
   * @memberof Game_CharacterBase
   */
  jumpHeight(): number;

  /**
   * Returns true if the character is stopping.
   *
   * @returns {Bool}
   * @memberof Game_CharacterBase
   */
  isStopping(): boolean;

  checkStop(threshold: number): boolean;

  /**
   * Resets the step count of the character.
   *
   * @memberof Game_CharacterBase
   */
  resetStopCount(): void;

  realMoveSpeed(): number;

  distancePerFrame(): number;

  /**
   * Returns true if the character is dashing.
   *
   * @returns {Bool}
   * @memberof Game_CharacterBase
   */
  isDashing(): boolean;

  isDebugThrough(): boolean;

  /**
   * Straightens the character.
   *
   * @memberof Game_CharacterBase
   */
  straighten(): void;

  reverseDir(d: number): number;

  canPass(x: number, y: number, d: number): boolean;

  canPassDiagonally(x: number, y: number, horz: number, vert: number): boolean;

  isMapPassable(x: number, y: number, d: number): boolean;

  isCollidedWithCharacters(x: number, y: number): boolean;

  isCollidedWithEvents(x: number, y: number): boolean;

  isCollidedWithVehicles(x: number, y: number): boolean;

  setPosition(x: number, y: number): void;

  copyPosition(character: Game_Player): void;

  locate(x: number, y: number): void;

  direction(): number;

  /**
   * Sets the direction of the character based on numpad
   * directions.
   * @param {Int} d
   * @memberof Game_CharacterBase
   */
  setDirection(d: number): void;

  /**
   * Returns true if the character is a tile; these
   * are events without character sprites.
   * @returns {Bool}
   * @memberof Game_CharacterBase
   */
  isTile(): boolean;

  /**
   * Returns true if the character is an object character;
   * these are events with character sprites.
   * @returns {Bool}
   * @memberof Game_CharacterBase
   */
  isObjectCharacter(): boolean;

  shiftY(): number;

  scrolledX(): number;

  scrolledY(): number;

  /**
   * Returns the character's scrreen x position.
   *
   * @returns {Int}
   * @memberof Game_CharacterBase
   */
  screenX(): number;

  /**
   * Returns the character's screen y position.
   *
   * @returns {Int}
   * @memberof Game_CharacterBase
   */
  screenY(): number;

  /**
   * Returns the character's screen z position.
   *
   * @returns {Int}
   * @memberof Game_CharacterBase
   */
  screenZ(): number;

  isNearTheScreen(): boolean;

  update(): void;

  updateStop(): void;

  updateJump(): void;

  updateMove(): void;

  updateAnimation(): void;

  animationWait(): number;

  /**
   * Updates the character's animation count.
   *
   * @memberof Game_CharacterBase
   */
  updateAnimationCount(): void;

  updatePattern(): void;

  maxPattern(): number;

  /**
   * Returns the pattern of the character; these are the walking
   * patterns.
   * @returns {Int}
   * @memberof Game_CharacterBase
   */
  pattern(): number;

  /**
   * Sets the pattern of the character, given
   * a pattern Int.
   * @param {Int} pattern
   * @memberof Game_CharacterBase
   */
  setPattern(pattern: number): void;

  isOriginalPattern(): boolean;

  /**
   * Resets the pattern of the character.
   *
   * @memberof Game_CharacterBase
   */
  resetPattern(): void;

  refreshBushDepth(): void;

  isOnLadder(): boolean;

  isOnBush(): boolean;

  /**
   * Returns the terrain tag of the character.
   *
   * @returns {Int}
   * @memberof Game_CharacterBase
   */
  terrainTag(): number;

  /**
   * Returns the region id of the character.
   *
   * @returns {Int}
   * @memberof Game_CharacterBase
   */
  regionId(): number;

  /**
   * Increases the character steps.
   *
   * @memberof Game_CharacterBase
   */
  increaseSteps(): void;

  /**
   * Returns the tile id of character.
   *
   * @returns {Int}
   * @memberof Game_CharacterBase
   */
  tileId(): number;

  characterName(): string;

  characterIndex(): number;

  setImage(characterName: string, characterIndex: number): void;

  setTileImage(tileId: number): void;

  checkEventTriggerTouchFront(d: number): void;

  checkEventTriggerTouch(x: number, y: number): boolean;

  isMovementSucceeded(x: number, y: number): boolean;

  setMovementSuccess(success: boolean): void;

  moveStraight(d: number): void;

  moveDiagonally(horz: number, vert: number): void;

  jump(xPlus: number, yPlus: number): void;

  hasWalkAnime(): boolean;

  setWalkAnime(walkAnime: boolean): void;

  /**
   * Returns true if the character has step animation.
   *
   * @returns {Bool}
   * @memberof Game_CharacterBase
   */
  hasStepAnime(): boolean;

  setStepAnime(stepAnime: boolean): void;

  /**
   * Returns true if the character is set to a fixed direction.
   *
   * @returns {Bool}
   * @memberof Game_CharacterBase
   */
  isDirectionFixed(): boolean;

  /**
   * Sets the character to be fixed in a specified direction
   * given a Bool value.
   * @param {Bool} directionFix
   * @memberof Game_CharacterBase
   */
  setDirectionFix(directionFix: boolean): void;

  /**
   * Returns true if the character is set to pass through.
   *
   * @returns {Bool}
   * @memberof Game_CharacterBase
   */
  isThrough(): boolean;

  setThrough(through: boolean): void;

  isTransparent(): boolean;

  /**
   * Returns the bush depth around the character.
   *
   * @returns {Int}
   * @memberof Game_CharacterBase
   */
  bushDepth(): number;

  setTransparent(transparent: boolean): void;

  /**
   * Requests an animation given the animation id.
   *
   * @param {Int} animationId
   * @memberof Game_CharacterBase
   */
  requestAnimation(animationId: number): void;

  /**
   * Requests the balloon animation given the balloon id.
   *
   * @param {Int} balloonId
   * @memberof Game_CharacterBase
   */
  requestBalloon(balloonId: number): void;

  /**
   * Returns the animation id.
   *
   * @returns {Int}
   * @memberof Game_CharacterBase
   */
  animationId(): number;

  /**
   * Returns the id of the balloon animation.
   *
   * @returns {Int}
   * @memberof Game_CharacterBase
   */
  balloonId(): number;

  /**
   * Starts the requested animation.
   *
   * @memberof Game_CharacterBase
   */
  startAnimation(): void;

  /**
   * Stars a balloon animation.
   *
   * @memberof Game_CharacterBase
   */
  startBalloon(): void;

  isAnimationPlaying(): boolean;

  /**
   * Returns true if a balloon animation is playing.
   *
   * @returns {Bool}
   * @memberof Game_CharacterBase
   */
  isBalloonPlaying(): boolean;

  endAnimation(): void;

  /**
   * Ends the balloon animation on the character.
   *
   * @memberof Game_CharacterBase
   */
  endBalloon(): void;
}

declare namespace rm.types {
  /**
   * The data class for the Move command.
   */
  export type MoveCommand = {
    /**
     * Move command code.
     */
    code: number;
    /**
     * An array containing the Move command's arguments. The contents vary for each command.
     */
    parameters: any[];
  };
}

declare namespace rm.types {
  /**
   * The data class for the Move route.
   */
  export type MoveRoute = {
    /**
     * Program contents. An RPG.MoveCommand array.
     */
    list: rm.types.MoveCommand[];
    /**
     * The truth value of the [Repeat Action] option.
     */
    repeat: boolean;
    /**
     * The truth value of the [Skip If Cannot Move] option.
     */
    skippable: boolean;
    /**
     * The truth value of the [Wait for Completion] option.
     */
    wait: boolean;
  };
}

/**
 * Super class of Game_Player, Game_Follower, Game_Vehicle,
 * and Game_Event.
 */
declare class Game_Character extends Game_CharacterBase {
  protected constructor();

  _moveRouteForcing: boolean;
  _moveRoute: rm.types.MoveRoute;
  _moveRouteIndex: number;
  _originalMoveRoute: rm.types.MoveRoute;
  _originalMoveRouteIndex: number;
  _waitCount: number;

  initMembers(): void;

  /**
   * Memorizes the movement route.
   *
   * @memberof Game_Character
   */
  memorizeMoveRoute(): void;

  /**
   * Restores the original movement route.
   *
   * @memberof Game_Character
   */
  restoreMoveRoute(): void;

  /**
   * Returns true if the move route is being forced.
   *
   * @returns {Bool}
   * @memberof Game_Character
   */
  isMoveRouteForcing(): boolean;

  /**
   * Sets the move route of the game character.
   *
   * @param {RPG.MoveRoute} moveRoute
   * @memberof Game_Character
   */
  setMoveRoute(moveRoute: rm.types.MoveRoute): void;

  /**
   * Forces the move route of the game character.
   *
   * @param {RPG.MoveRoute} moveRoute
   * @memberof Game_Character
   */
  forceMoveRoute(moveRoute: rm.types.MoveRoute): void;

  updateStop(): void;

  /**
   * Updates the game character's move routine.
   *
   * @memberof Game_Character
   */
  updateRoutineMove(): void;

  /**
   * Processes the given move commands.
   *
   * @param {RPG.MoveCommand} command
   * @memberof Game_Character
   */
  processMoveCommand(command: rm.types.MoveCommand): void;

  deltaXFrom(x: number): number;

  deltaYFrom(y: number): number;

  /**
   * Move's the game character at random.
   *
   * @memberof Game_Character
   */
  moveRandom(): void;

  /**
   * Moves the game character toward the other game character.
   *
   * @param {Game_Character} character
   * @memberof Game_Character
   */
  moveTowardCharacter(character: Game_Character): void;

  /**
   * Moves the game character away from the other game character.
   *
   * @param {Game_Character} character
   * @memberof Game_Character
   */
  moveAwayFromCharacter(character: Game_Character): void;

  /**
   * Turns the game character toward the other game character.
   *
   * @param {Game_Character} character
   * @memberof Game_Character
   */
  turnTowardCharacter(character: Game_Character): void;

  /**
   * Turns the game character away from the other game character.
   *
   * @param {Game_Character} character
   * @memberof Game_Character
   */
  turnAwayFromCharacter(character: Game_Character): void;

  /**
   * Turns the game character toward the player.
   *
   * @memberof Game_Character
   */
  turnTowardPlayer(): void;

  /**
   * Turns the game character away from the player.
   *
   * @memberof Game_Character
   */
  turnAwayFromPlayer(): void;

  /**
   * Moves the game character toward the player.
   *
   * @memberof Game_Character
   */
  moveTowardPlayer(): void;

  /**
   * Moves the game character away from the player.
   *
   * @memberof Game_Character
   */
  moveAwayFromPlayer(): void;

  /**
   * Moves the game character forward.
   *
   * @memberof Game_Character
   */
  moveForward(): void;

  /**
   * Moves the game character backward.
   *
   * @memberof Game_Character
   */
  moveBackward(): void;

  /**
   * Handles the end of the move route.
   *
   * @memberof Game_Character
   */
  processRouteEnd(): void;

  advanceMoveRouteIndex(): void;

  /**
   * Turns the game character right by 90 degrees.
   *
   * @memberof Game_Character
   */
  turnRight90(): void;

  /**
   * Turns the game character left by 90 degrees.
   *
   * @memberof Game_Character
   */
  turnLeft90(): void;

  turn180(): void;

  /**
   * Turns the game character or left by 90 degrees.
   *
   * @memberof Game_Character
   */
  turnRightOrLeft90(): void;

  /**
   * Turns the game character at random.
   *
   * @memberof Game_Character
   */
  turnRandom(): void;

  swap(character: Game_Character): void;

  findDirectionTo(goalX: number, goalY: number): number;

  /**
   * Returns the search limit for path finding.
   *
   * @returns {Int}
   * @memberof Game_Character
   */
  searchLimit(): number;

  static ROUTE_END: number;
  static ROUTE_MOVE_DOWN: number;
  static ROUTE_MOVE_LEFT: number;
  static ROUTE_MOVE_RIGHT: number;
  static ROUTE_MOVE_UP: number;
  static ROUTE_MOVE_LOWER_L: number;
  static ROUTE_MOVE_LOWER_R: number;
  static ROUTE_MOVE_UPPER_L: number;
  static ROUTE_MOVE_UPPER_R: number;
  static ROUTE_MOVE_RANDOM: number;
  static ROUTE_MOVE_TOWARD: number;
  static ROUTE_MOVE_AWAY: number;
  static ROUTE_MOVE_FORWARD: number;
  static ROUTE_MOVE_BACKWARD: number;
  static ROUTE_JUMP: number;
  static ROUTE_WAIT: number;
  static ROUTE_TURN_DOWN: number;
  static ROUTE_TURN_LEFT: number;
  static ROUTE_TURN_RIGHT: number;
  static ROUTE_TURN_UP: number;
  static ROUTE_TURN_90D_R: number;
  static ROUTE_TURN_90D_L: number;
  static ROUTE_TURN_180D: number;
  static ROUTE_TURN_90D_R_L: number;
  static ROUTE_TURN_RANDOM: number;
  static ROUTE_TURN_TOWARD: number;
  static ROUTE_TURN_AWAY: number;
  static ROUTE_SWITCH_ON: number;
  static ROUTE_SWITCH_OFF: number;
  static ROUTE_CHANGE_SPEED: number;
  static ROUTE_CHANGE_FREQ: number;
  static ROUTE_WALK_ANIME_ON: number;
  static ROUTE_WALK_ANIME_OFF: number;
  static ROUTE_STEP_ANIME_ON: number;
  static ROUTE_STEP_ANIME_OFF: number;
  static ROUTE_DIR_FIX_ON: number;
  static ROUTE_DIR_FIX_OFF: number;
  static ROUTE_THROUGH_ON: number;
  static ROUTE_THROUGH_OFF: number;
  static ROUTE_TRANSPARENT_ON: number;
  static ROUTE_TRANSPARENT_OFF: number;
  static ROUTE_CHANGE_IMAGE: number;
  static ROUTE_CHANGE_OPACITY: number;
  static ROUTE_CHANGE_BLEND_MODE: number;
  static ROUTE_PLAY_SE: number;
  static ROUTE_SCRIPT: number;
}

declare namespace rm.types {
  /**
   * The data class for the Event command.
   */
  export type EventCommand = {
    /**
     * The event code.
     */
    code: number;
    /**
     * The indent depth. Usually 0. The [Conditional Branch] command, among others, adds 1 with every step deeper.
     */
    indent: number;
    /**
     * An array containing the Event command's arguments. The contents vary for each command.
     */
    parameters: any[];
  };
}

declare namespace rm.types {
  /**
   * The data class for common events.
   */
  export type CommonEvent = {
    /**
     * The event ID.
     */
    id: number;
    /**
     * A list of event commands. An RPG.EventCommand array.
     */
    list: rm.types.EventCommand[];
    /**
     * The event name.
     */
    name: string;
    /**
     * The condition switch ID.
     */
    switchId: number;
    /**
     * The event trigger (0: none, 1: autorun; 2: parallel).
     */
    trigger: number;
  };
}

declare class Game_CommonEvent {
  constructor(commonEventId: number);

  _commonEventId: number;
  _interpreter: Game_Interpreter;

  initialize(commonEventId: number): void;

  /**
   * Returns the common event information from the database.
   *
   * @returns {CommonEvent}
   * @memberof Game_CommonEvent
   */
  event(): rm.types.CommonEvent;

  /**
   * Returns the common event's list of event commands.
   *
   * @returns {Array<EventCommand>}
   * @memberof Game_CommonEvent
   */
  list(): rm.types.EventCommand[];

  /**
   * Refreshes the common event.
   *
   * @memberof Game_CommonEvent
   */
  refresh(): void;

  /**
   * Returns true if the common event is active.
   *
   * @returns {boolean}
   * @memberof Game_CommonEvent
   */
  isActive(): boolean;

  /**
   * Updates the common event.
   *
   * @memberof Game_CommonEvent
   */
  update(): void;
}

declare namespace rm.types {
  export type EnemyDropItem = {
    /**
     * The ID of the data depending on the type of dropped item (item, weapon, or armor).
     */
    dataId: number;
    /**
     * N of the probability that the item will be dropped, 1/N.
     */
    denominator: number;
    /**
     * The type of dropped item.
     *
     * 0: None
     * 1: Item
     * 2: Weapon
     * 3: Armor
     */
    kind: number;
  };
}

declare namespace rm.types {
  export type Enemy = {
    /**
     * The enemy's action pattern. An array of RPG.Enemy.Action.
     */
    actions: rm.types.EnemyAction[];
    /**
     * The adjustment value for the battler graphic's hue (0..360).
     */
    battlerHue: number;
    /**
     * The file name of the enemy's battler graphic.
     */
    battlerName: string;
    doc: string;
    /**
     * The items the enemy drops. An RPG.Enemy.DropItem array.
     */
    dropItems: rm.types.EnemyDropItem[];
    /**
     * The enemy's experience.
     */
    exp: number;
    /**
     * The enemy's gold.
     */
    gold: number;
    /**
     * The ID of the enemy in the database.
     *
     * @type {number}
     * @memberof Enemy
     */
    id: number;
    internal: boolean;
    links: string[];
    /**
     * The name of the enemy in the database.
     *
     * @type {string}
     * @memberof Enemy
     */
    name: string;
    parameters: string[];
    /**
     * Parameters. An array of integers using the following IDs as subscripts:
     *
     * 0: Maximum hit points
     * 1: Maximum magic points
     * 2: Attack power
     * 3: Defense power
     * 4: Magic attack power
     * 5: Magic defense power
     * 6: Agility
     * 7: Luck
     */
    params: number[];
    platforms: haxe.display.Platform[];
    targets: haxe.display.MetadataTarget[];
  };
}

/**
 * -----------------------------------------------------------------------------
 * Game_Enemy
 *
 * The game object declare class for an enemy.
 * @class Game_Enemy
 */
declare class Game_Enemy extends Game_Battler {
  constructor(enemyId: number, x: number, y: number);

  _enemyId: number;
  _letter: string;
  _plural: boolean;
  _screenX: number;
  _screenY: number;

  initialize(enemyId: number, x: number, y: number): void;

  initMembers(): void;

  setup(enemyId: number, x: number, y: number): void;

  /**
   * Returns the other troops in the enemy team.
   *
   * @returns {Game_Troop}
   * @memberof Game_Enemy
   */
  friendsUnit(): Game_Temp;

  /**
   * Returns the game party.
   *
   * @returns {Game_Party}
   * @memberof Game_Enemy
   */
  opponentsUnit(): Game_Party;

  isBattleMember(): boolean;

  /**
   * Returns the enemy id.
   *
   * @returns {number}
   * @memberof Game_Enemy
   */
  enemyId(): number;

  /**
   * Returns the enemy information from the database.
   *
   * @returns {RPG.Enemy}
   * @memberof Game_Enemy
   */
  enemy(): rm.types.Enemy;

  /**
   * Returns the enemy's exp amount.
   *
   * @memberof Game_Enemy
   */
  exp(): void;

  /**
   * Returns enemy's given gold.
   *
   * @memberof Game_Enemy
   */
  gold(): void;

  /**
   * Creates the drop items for the enemy specified by the database.
   *
   * @returns {Array<RPG.BaseItem>}
   * @memberof Game_Enemy
   */
  makeDropItems(): rm.types.BaseItem[];

  /**
   * Returns the item drop rate of the enemy.
   *
   * @returns {number}
   * @memberof Game_Enemy
   */
  dropItemRate(): number;

  itemObject(kind: number, dataId: number): void;

  /**
   * Returns true if the enemy sprite is visible.
   *
   * @returns {boolean}
   * @memberof Game_Enemy
   */
  isSpriteVisible(): boolean;

  /**
   * Returns enemy screen X coordinate.
   *
   * @returns {number}
   * @memberof Game_Enemy
   */
  screenX(): number;

  /**
   * Returns enemy screen Y position.
   *
   * @returns {number}
   * @memberof Game_Enemy
   */
  screenY(): number;

  /**
   * Returns the hue of the enemy.
   *
   * @returns {number}
   * @memberof Game_Enemy
   */
  battlerHue(): number;

  /**
   * Returns the enemy original name.
   *
   * @returns {string}
   * @memberof Game_Enemy
   */
  originalName(): string;

  isLetterEmpty(): boolean;

  setLetter(letter: string): void;

  setPlural(plural: boolean): void;

  /**
   * Transform the enemy into another enemy based on id.
   *
   * @param {number} enemyId
   * @memberof Game_Enemy
   */
  transform(enemyId: number): void;

  meetsCondition(action: rm.types.EnemyAction): boolean;

  meetsTurnCondition(param1: number, param2: number): boolean;

  meetsHpCondition(param1: number, param2: number): boolean;

  meetsMpCondition(param1: number, param2: number): boolean;

  meetsStateCondition(param: number): boolean;

  meetsPartyLevelCondition(param: number): boolean;

  meetsSwitchCondition(param: number): boolean;

  /**
   * Returns true if the enemy action is valid.
   *
   * @param {RPG.Enemy.Action} action
   * @returns {boolean}
   * @memberof Game_Enemy
   */
  isActionValid(action: rm.types.EnemyAction): boolean;

  selectAction(actionList: rm.types.EnemyAction[], ratingZero: number): rm.types.EnemyAction;

  selectAllActions(actionList: rm.types.EnemyAction[]): void;
}

declare namespace rm.types {
  export const enum MoveType {
    FIXED = 0,
    RANDOM = 1,
    APPROACH = 2,
    CUSTOM = 3,
  }
}

declare namespace rm.types {
  export const enum EventTrigger {
    base = -1,
  }
}

declare namespace rm.types {
  /**
   * The data class for the event page conditions.
   */
  export type EventPageConditions = {
    /**
     * The ID of that actor if the [Actor] condition is valid.
     */
    actorId: string;
    /**
     * The truth value indicating whether the [Actor] condition is valid.
     */
    actorValid: boolean;
    /**
     * The ID of that item if the [Item] condition is valid.
     */
    itemId: string;
    /**
     * The truth value indicating whether the [Item] condition is valid.
     */
    itemValid: boolean;
    /**
     * The letter of that self switch ("A".."D") if the [Self Switch] condition is valid.
     */
    selfSwitchCh: string;
    /**
     * The truth value indicating whether the [Self Switch] condition is valid.
     */
    selfSwitchValid: boolean;
    /**
     * The ID of that switch if the first [Switch] condition is valid.
     */
    switch1Id: number;
    /**
     * The truth value indicating whether the first [Switch] condition is valid.
     */
    switch1Valid: boolean;
    /**
     * The ID of that switch if the second [Switch] condition is valid.
     */
    switch2Id: number;
    /**
     * The truth value indicating whether the second [Switch] condition is valid.
     */
    switch2Valid: boolean;
    /**
     * The ID of that variable if the [Variable] condition is valid.
     */
    variableId: number;
    /**
     * The truth value indicating whether the [Variable] condition is valid.
     */
    variableValid: boolean;
    /**
     * The standard value of that variable (x and greater) if the [Variable] condition is valid.
     */
    variableValue: number;
  };
}

declare namespace rm.types {
  /**
   * The data class for the Event page [Graphics].
   */
  export type EventPageImage = {
    /**
     * The index of the character's graphic file (0..7).
     */
    characterIndex: number;
    /**
     * The file name of the character's graphic.
     */
    characterName: string;
    /**
     * The direction in which the character is facing (2: down, 4: left, 6: right, 8: up).
     */
    direction: number;
    /**
     * The character's pattern (0..2).
     */
    pattern: number;
    /**
     * The tile ID. If the specified graphic is not a tile, this value is 0.
     */
    tileId: number;
  };
}

declare namespace rm.types {
  /**
   * The data class for the event page.
   */
  export type EventPage = {
    /**
     * The event condition (RPG.EventPage.Condition).
     */
    conditions: rm.types.EventPageConditions;
    /**
     * The truth value of the [Direction Fix] option.
     */
    directionFix: boolean;
    /**
     * The event graphic (RPG.EventPage.Image) .
     */
    image: rm.types.EventPageImage;
    /**
     * A list of event commands. An RPG.EventCommand array.
     */
    list: rm.types.EventCommand[];
    /**
     * The movement frequency (1: lowest, 2: lower, 3: normal, 4: higher, 5: highest).
     */
    moveFrequency: number;
    /**
     * The movement route (RPG.MoveRoute). Referenced only when the movement type is set to custom.
     */
    moveRoute: rm.types.MoveRoute[];
    /**
     * The movement speed (1: x8 slower, 2: x4 slower, 3: x2 slower, 4: normal, 5: x2 faster, 6: x4 faster).
     */
    moveSpeed: number;
    /**
     * The type of movement (0: fixed, 1: random, 2: approach, 3: custom).
     */
    moveType: number;
    /**
     * The priority type (0: below characters, 1: same as characters, 2: above characters).
     */
    priorityType: number;
    /**
     * The truth value of the [Stepping Animation] option.
     */
    stepAnime: boolean;
    /**
     * The truth value of the [Through] option.
     */
    through: boolean;
    /**
     * The event trigger (0: action button, 1: player touch, 2: event touch, 3: autorun, 4: parallel).
     */
    trigger: number;
    /**
     * The truth value of the [Walking Animation] option.
     */
    walkAnime: boolean;
  };
}

declare namespace rm.types {
  /**
   * The data class for map events.
   */
  export type Event = {
    /**
     * The event ID.
     */
    id: number;
    meta: Object;
    /**
     * The event name.
     */
    name: string;
    note: string;
    /**
     * The event pages. RPG.EventPage array.
     */
    pages: rm.types.EventPage[];
    /**
     * The event's x-coordinate on the map.
     */
    x: number;
    /**
     * The event's y-coordinate on the map.
     */
    y: number;
  };
}

/**
 * THe game object class for an event. It contains functionality
 * for event page switching and running parallel process events.
 */
declare class Game_Event extends Game_Character {
  constructor(mapId: number, eventId: number);

  _mapId: number;
  _eventId: number;
  _moveType: rm.types.MoveType;
  _trigger: rm.types.EventTrigger;
  _starting: boolean;
  _erased: boolean;
  _pageIndex: number;
  _originalPattern: rm.types.CharacterPattern;
  _originalDirection: rm.types.Direction;
  _prelockDirection: rm.types.Direction;
  _locked: boolean;

  initialize(mapId: number, eventId: number): void;

  initMembers(): void;

  /**
   * Returns the event id of the game event.
   *
   * @returns {Int}
   * @memberof Game_Event
   */
  eventId(): number;

  /**
   * Returns the event database information.
   *
   * @returns {RPG.Event}
   * @memberof Game_Event
   */
  event(): rm.types.Event;

  /**
   * Returns the event page created in the database.
   *
   * @returns {RPG.EventPage}
   * @memberof Game_Event
   */
  page(): rm.types.EventPage;

  /**
   * Returns the list of event commands on the current page of the game event.
   *
   * @returns {Array<RPG.EventCommand>}
   * @memberof Game_Event
   */
  list(): rm.types.EventCommand[];

  isCollidedWithCharacters(x: number, y: number): boolean;

  isCollidedWithEvents(x: number, y: number): boolean;

  /**
   * Returns true if the event has collided with the player character
   * at the specified x and y coordinates.
   * @param {Int} x
   * @param {Int} y
   * @returns {Bool}
   * @memberof Game_Event
   */
  isCollidedWithPlayerCharacters(x: number, y: number): boolean;

  /**
   * Locks the game event.
   *
   * @memberof Game_Event
   */
  lock(): void;

  /**
   * Unlocks the game eveent.
   *
   * @memberof Game_Event
   */
  unlock(): void;

  updateStop(): void;

  /**
   * Updates the self movement of the game event.
   *
   * @memberof Game_Event
   */
  updateSelfMovement(): void;

  stopCountThreshold(): number;

  moveTypeRandom(): void;

  moveTypeTowardPlayer(): void;

  /**
   * Returns true if the game event is near the player.
   *
   * @returns {Bool}
   * @memberof Game_Event
   */
  isNearThePlayer(): boolean;

  moveTypeCustom(): void;

  /**
   * Returns true if the event is staring.
   *
   * @returns {Bool}
   * @memberof Game_Event
   */
  isStarting(): boolean;

  /**
   * Clears the starting flag.
   *
   * @memberof Game_Event
   */
  clearStartingFlag(): void;

  isTriggerIn(triggers: number[]): boolean;

  /**
   * Starts the game event.
   *
   * @memberof Game_Event
   */
  start(): void;

  /**
   * Erases the event.
   *
   * @memberof Game_Event
   */
  erase(): void;

  /**
   * Refreshes the game event.
   *
   * @memberof Game_Event
   */
  refresh(): void;

  /**
   * Finds the proper page index of the game event for
   * event command processing.
   * @returns {Int}
   * @memberof Game_Event
   */
  findProperPageIndex(): number;

  meetsConditions(page: rm.types.EventPage): boolean;

  /**
   * Sets up the event page.
   *
   * @memberof Game_Event
   */
  setupPage(): void;

  /**
   * Clears the page settings of the game event.
   *
   * @memberof Game_Event
   */
  clearPageSettings(): void;

  /**
   * Sets up the event page settings.
   *
   * @memberof Game_Event
   */
  setupPageSettings(): void;

  isOriginalPattern(): boolean;

  /**
   * Resets the pattern of the game event.
   *
   * @memberof Game_Event
   */
  resetPattern(): void;

  checkEventTriggerAuto(): void;

  update(): void;

  updateParallel(): void;

  locate(x: number, y: number): void;

  /**
   * Forces the game event to move along the specified route.
   *
   * @param {RPG.MoveRoute} moveRoute
   * @memberof Game_Event
   */
  forceMoveRoute(moveRoute: rm.types.MoveRoute): void;
}

/**
 * -----------------------------------------------------------------------------
 * Game_Follower
 *
 * The game object declare class for a follower. A follower is an allied character,
 * other than the front character, displayed in the party.
 * @class Game_Follower
 */
declare class Game_Follower extends Game_Character {
  constructor(memberIndex: number);

  _memberIndex: number;

  refresh(): void;

  /**
   * Returns the current actor.
   *
   * @returns {Game_Actor}
   * @memberof Game_Follower
   */
  actor(): Game_Actor;

  /**
   * Returns true if the follower is visible.
   *
   * @returns {Bool}
   * @memberof Game_Follower
   */
  isVisible(): boolean;

  /**
   * Updates the game follower.
   *
   * @memberof Game_Follower
   */
  update(): void;

  /**
   * Has the follower chase a game actor.
   *
   * @param {Game_Character} character
   * @memberof Game_Follower
   */
  chaseCharacter(character: Game_Character): void;
}

/**
 * -----------------------------------------------------------------------------
 * Game_Followers
 *
 * The wrapper declare class for a follower array.
 * @class Game_Followers
 */
declare class Game_Followers {
  constructor();

  _visible: boolean;
  _gathering: boolean;
  _data: Game_Follower[];

  initialize(): void;

  /**
   * Returns true if the followers are visible.
   *
   * @returns {Bool}
   * @memberof Game_Followers
   */
  isVisible(): boolean;

  /**
   * Shows  the followers.
   *
   * @memberof Game_Followers
   */
  show(): void;

  /**
   * Hides the followers.
   *
   * @memberof Game_Followers
   */
  hide(): void;

  /**
   * Returns the followers at the specified index.
   *
   * @param {number} index
   * @returns {Game_Follower}
   * @memberof Game_Followers
   */
  follower(index: number): Game_Follower;

  forEach(callback: () => void, thisObject: any): void;

  reverseEach(callback: () => void, thisObject: any): void;

  /**
   * Refreshes the game followers.
   *
   * @memberof Game_Followers
   */
  refresh(): void;

  /**
   * Updates the game followers.
   *
   * @memberof Game_Followers
   */
  update(): void;

  /**
   * Updates the movement of game followers.
   *
   * @memberof Game_Followers
   */
  updateMove(): void;

  jumpAll(): void;

  synchronize(x: number, y: number, d: number): void;

  /**
   * Gathers the followers.
   *
   * @memberof Game_Followers
   */
  gather(): void;

  /**
   * Returns true if the followers are gathering.
   *
   * @returns {Bool}
   * @memberof Game_Followers
   */
  areGathering(): boolean;

  /**
   * Returns the visible followers.
   *
   * @returns {Array<Game_Follower>}
   * @memberof Game_Followers
   */
  visibleFollowers(): Game_Follower[];

  /**
   * Returns true if the followers are moving.
   *
   * @returns {Bool}
   * @memberof Game_Followers
   */
  areMoving(): boolean;

  /**
   * Returns true if the followers are gathered.
   *
   * @returns {Bool}
   * @memberof Game_Followers
   */
  areGathered(): boolean;

  /**
   * Returns true if the specified follower is collided at the specified x and y
   *coordinate.
   *
   * @param {number} x
   * @param {number} y
   * @returns {Bool}
   * @memberof Game_Followers
   */
  isSomeoneCollided(x: number, y: number): boolean;
}

/**
 * -----------------------------------------------------------------------------
 * Game_Interpreter
 *
 * The interpreter for running event commands.
 * @class Game_Interpreter
 */
declare class Game_Interpreter {
  constructor(depth: number);

  _depth: number;
  _branch: Object;
  _params: any[];
  _indent: number;
  _frameCount: number;
  _freezeChecker: number;
  _mapId: number;
  _eventId: number;
  _list: rm.types.EventCommand[];
  _index: number;
  _waitCount: number;
  _waitMode: string;
  _comments: string;
  _character: Game_Event;
  _childInterpreter: Game_Interpreter;

  checkOverflow(): void;

  /**
   * Clears the interpreter.
   *
   * @memberof Game_Interpreter
   */
  clear(): void;

  /**
   * Sets up the interpreter with the list of event commands, and the given
   * event Id.
   *
   * @param {Array<RPG.EventCommand>} list
   * @param {Int} eventId
   * @memberof Game_Interpreter
   */
  setup(list: rm.types.EventCommand[], eventId: number): void;

  /**
   * Returns the currrent eventId.
   *
   * @returns {Int}
   * @memberof Game_Interpreter
   */
  eventId(): number;

  /**
   * Returns true if the event is on the current map.
   *
   * @returns {Bool}
   * @memberof Game_Interpreter
   */
  isOnCurrentMap(): boolean;

  /**
   * Returns true after setting up the reserved common event.
   *
   * @returns {Bool}
   * @memberof Game_Interpreter
   */
  setupReservedCommonEvent(): boolean;

  /**
   * Returns true if the interpreter is running.
   *
   * @returns {Bool}
   * @memberof Game_Interpreter
   */
  isRunning(): boolean;

  update(): void;

  /**
   * Updates the child game interpreter.
   *
   * @returns {Bool}
   * @memberof Game_Interpreter
   */
  updateChild(): boolean;

  /**
   * Updates the wait of the game interpreter.
   *
   * @returns {Bool}
   * @memberof Game_Interpreter
   */
  updateWait(): boolean;

  updateWaitCount(): boolean;

  updateWaitMode(): boolean;

  /**
   * Sets the wait mode of the interpreter.
   *
   * @param {String} waitMode
   * @memberof Game_Interpreter
   */
  setWaitMode(waitMode: string): void;

  /**
   * sets a specified wait duration for the interpreter.
   *
   * @param {Int} duration
   * @memberof Game_Interpreter
   */
  wait(duration: number): void;

  fadeSpeed(): number;

  /**
   * Executes the event command;
   * returns true or false based on execution.
   * @returns {Bool}
   * @memberof Game_Interpreter
   */
  executeCommand(): boolean;

  /**
   * Checks if the interpreter has frozen.
   *
   * @returns {Bool}
   * @memberof Game_Interpreter
   */
  checkFreeze(): boolean;

  /**
   * Terminates the game interpreter.
   *
   * @memberof Game_Interpreter
   */
  terminate(): void;

  /**
   * Skips a conditional branch on the interpreter.
   *
   * @memberof Game_Interpreter
   */
  skipBranch(): void;

  /**
   * Returns the current event command.
   *
   * @returns {RPG.EventCommand}
   * @memberof Game_Interpreter
   */
  currentCommand(): rm.types.EventCommand;

  /**
   * Returns the next event code.
   *
   * @returns {Int}
   * @memberof Game_Interpreter
   */
  nextEventCode(): number;

  iterateActorId(param: number, callback: (actor: Game_Actor) => void): void;

  iterateActorEx(param1: number, param2: number, callback: (actor: Game_Actor) => void): void;

  iterateActorIndex(param: number, callback: (actor: Game_Actor) => void): void;

  iterateEnemyIndex(param: number, callback: (enemt: Game_Enemy) => void): void;

  iterateBattler(param1: number, param2: number, callback: (battler: Game_Battler) => void): void;

  character(param: number): Game_Character;

  operateValue(operation: number, operandType: number, operand: number): number;

  changeHp(target: number, value: number, allowDeath: boolean): void;

  /**
   * Show Text
   */
  command101(): boolean;

  /**
   * Show Choices
   */
  command102(): boolean;

  setupChoices(params: any[]): void;

  /**
   * When [**]
   */
  command402(): boolean;

  /**
   * When Cancel
   */
  command403(): boolean;

  /**
   * Input Number
   */
  command103(): boolean;

  /**
   *
   * @param params
   */
  setupNumInput(params: number[]): void;

  /**
   * Select Item
   */
  command104(): boolean;

  setupItemChoice(params: number[]): void;

  /**
   * Show Scrolling Text
   */
  command105(): boolean;

  /**
   * Comment
   */
  command108(): boolean;

  /**
   * Conditional Branch
   */
  command111(): boolean;

  /**
   * Else
   */
  command411(): boolean;

  /**
   * Loop
   */
  command112(): boolean;

  /**
   * Repeat Above
   */
  command413(): boolean;

  /**
   * Break Loop
   */
  command113(): boolean;

  /**
   * Exit Event Processing
   */
  command115(): boolean;

  /**
   * Common Event
   */
  command117(): boolean;

  setupChild(list: rm.types.EventCommand[], eventId: number): void;

  /**
   * Label
   */
  command118(): boolean;

  /**
   * Jump to Label
   */
  command119(): boolean;

  jumpTo(index: number): void;

  /**
   * Control Switches
   */
  command121(): boolean;

  /**
   * Control Variables
   */
  command122(): boolean;

  gameDataOperand(type: number, param1: number, param2: number): number;

  operateVariable(variableId: number, operationType: number, value: number): void;

  /**
   * Control Self Switch
   */
  command123(): boolean;

  /**
   * Control Timer
   */
  command124(): boolean;

  /**
   * Change Gold
   */
  command125(): boolean;

  /**
   * Change Items
   */
  command126(): boolean;

  /**
   * Change Weapons
   */
  command127(): boolean;

  /**
   * Change Armors
   */
  command128(): boolean;

  /**
   * Change Party Member
   */
  command129(): boolean;

  /**
   * Change Battle BGM
   */
  command132(): boolean;

  /**
   * Change Victory ME
   */
  command133(): boolean;

  /**
   * Change Save Access
   */
  command134(): boolean;

  /**
   * Change Menu Access
   */
  command135(): boolean;

  /**
   * Change Encounter Disable
   */
  command136(): boolean;

  /**
   * Change Formation Access
   */
  command137(): boolean;

  /**
   * Change Window Color
   */
  command138(): boolean;

  /**
   * Change Defeat ME
   */
  command139(): boolean;

  /**
   * Change Vehicle BGM
   */
  command140(): boolean;

  /**
   * Transfer Player
   */
  command201(): boolean;

  /**
   * Set Vehicle Location
   */
  command202(): boolean;

  /**
   * Set Event Location
   */
  command203(): boolean;

  /**
   * Scroll Map
   */
  command204(): boolean;

  /**
   * Set Movement Route
   */
  command205(): boolean;

  /**
   * Getting On and Off Vehicles
   */
  command206(): boolean;

  /**
   * Change Transparency
   */
  command211(): boolean;

  /**
   * Show Animation
   */
  command212(): boolean;

  /**
   * Show Balloon Icon
   */
  command213(): boolean;

  /**
   * Erase Event
   */
  command214(): boolean;

  /**
   * Change Player Followers
   */
  command216(): boolean;

  /**
   * Gather Followers
   */
  command217(): boolean;

  /**
   * Fadeout Screen
   */
  command221(): boolean;

  /**
   * Fadein Screen
   */
  command222(): boolean;

  /**
   * Tint Screen
   */
  command223(): boolean;

  /**
   * Flash Screen
   */
  command224(): boolean;

  /**
   * Shake Screen
   */
  command225(): boolean;

  /**
   * Wait
   */
  command230(): boolean;

  /**
   * Show Picture
   */
  command231(): boolean;

  /**
   * Move Picture
   */
  command232(): boolean;

  /**
   * Rotate Picture
   */
  command233(): boolean;

  /**
   * Tint Picture
   */
  command234(): boolean;

  /**
   * Erase Picture
   */
  command235(): boolean;

  /**
   * Set Weather Effect
   */
  command236(): boolean;

  /**
   * Play BGM
   */
  command241(): boolean;

  /**
   * Fadeout BGM
   */
  command242(): boolean;

  /**
   * Save BGM
   */
  command243(): boolean;

  /**
   * Resume BGM
   */
  command244(): boolean;

  /**
   * Play BGS
   */
  command245(): boolean;

  /**
   * Fadeout BGS
   */
  command246(): boolean;

  /**
   * Play ME
   */
  command249(): boolean;

  /**
   * Play SE
   */
  command250(): boolean;

  /**
   * Stop SE
   */
  command251(): boolean;

  /**
   * Play Movie
   */
  command261(): boolean;

  videoFileExt(): string;

  /**
   * Change Map Name Display
   */
  command281(): boolean;

  /**
   * Change Tileset
   */
  command282(): boolean;

  /**
   * Change Battle Back
   */
  command283(): boolean;

  /**
   * Change Parallax
   */
  command284(): boolean;

  /**
   * Get Location Info
   */
  command285(): boolean;

  /**
   * Battle Processing
   */
  command301(): boolean;

  /**
   * If Win
   */
  command601(): boolean;

  /**
   * If Escape
   */
  command602(): boolean;

  /**
   * If Lose
   */
  command603(): boolean;

  /**
   * Shop Processing
   */
  command302(): boolean;

  /**
   * Name Input Processing
   */
  command303(): boolean;

  /**
   * Change HP
   */
  command311(): boolean;

  /**
   * Change MP
   */
  command312(): boolean;

  /**
   * Change TP
   */
  command326(): boolean;

  /**
   * Change State
   */
  command313(): boolean;

  /**
   * Recover All
   */
  command314(): boolean;

  /**
   * Change EXP
   */
  command315(): boolean;

  /**
   * Change Level
   */
  command316(): boolean;

  /**
   * Change Parameter
   */
  command317(): boolean;

  /**
   * Change Skill
   */
  command318(): boolean;

  /**
   * Change Equipment
   */
  command319(): boolean;

  /**
   * Change Name
   */
  command320(): boolean;

  /**
   * Change Class
   */
  command321(): boolean;

  /**
   * Change Actor Images
   */
  command322(): boolean;

  /**
   * Change Vehicle Image
   */
  command323(): boolean;

  /**
   * Change Nickname
   */
  command324(): boolean;

  /**
   * Change Profile
   */
  command325(): boolean;

  /**
   * Change Enemy HP
   */
  command331(): boolean;

  /**
   * Change Enemy MP
   */
  command332(): boolean;

  /**
   * Change Enemy TP
   */
  command342(): boolean;

  /**
   * Change Enemy State
   */
  command333(): boolean;

  /**
   * Enemy Recover All
   */
  command334(): boolean;

  /**
   * Enemy Appear
   */
  command335(): boolean;

  /**
   * Enemy Transform
   */
  command336(): boolean;

  /**
   * Show Battle Animation
   */
  command337(): boolean;

  /**
   * Force Action
   */
  command339(): boolean;

  /**
   * Abort Battle
   */
  command340(): boolean;

  /**
   * Open Menu Screen
   */
  command351(): boolean;

  /**
   * Open Save Screen
   */
  command352(): boolean;

  /**
   * Game Over
   */
  command353(): boolean;

  /**
   * Return to Title Screen
   */
  command354(): boolean;

  /**
   * Script
   */
  command355(): boolean;

  /**
   * Plugin Command
   */
  command356(): boolean;

  pluginCommand(command: string, args: string[]): void;
}

/**
 * Game_Item
 * The game object class for handling skills, items, weapons. armor.
 * This is required, because the save data should not include the database object itself.
 * @class Game_Item
 */
declare class Game_Item {
  constructor(item: rm.types.BaseItem);

  _dataClass: string;
  _itemId: number;

  initialize(): void;

  /**
   * Returns true the item is a skill.
   *
   * @returns {boolean}
   * @memberof Game_Item
   */
  isSkill(): boolean;

  /**
   * Returns true if the item is an item.
   *
   * @returns {boolean}
   * @memberof Game_Item
   */
  isItem(): boolean;

  /**
   * Returns true if the item is a usable item, similar to a potion.
   *
   * @returns {boolean}
   * @memberof Game_Item
   */
  isUsableItem(): boolean;

  /**
   * Returns true if the item is a weapon.
   *
   * @returns {boolean}
   * @memberof Game_Item
   */
  isWeapon(): boolean;

  /**
   * Returns true if the item is an armor.
   *
   * @returns {boolean}
   * @memberof Game_Item
   */
  isArmor(): boolean;

  /**
   * Returns true if the item is equippable (weapon or armor).
   *
   * @returns {boolean}
   * @memberof Game_Item
   */
  isEquipItem(): boolean;

  /**
   * Returns true if the item is 'null'.
   *
   * @returns {boolean}
   * @memberof Game_Item
   */
  isNull(): boolean;

  /**
   * Returns the current itemId.
   *
   * @returns {number}
   * @memberof Game_Item
   */
  itemId(): number;

  object(): rm.types.BaseItem;

  /**
   * Sets the current item of the current Game_Item object.
   *
   * @param {RPG.BaseItem} item
   * @memberof Game_Item
   */
  setObject(item: rm.types.BaseItem): void;

  setEquip(isWeapon: boolean, itemId: number): void;
}

declare namespace rm.types {
  /**
   * The data class for tile sets.
   */
  export type Tileset = {
    doc: string;
    /**
     * The flags table. A 1-dimensional array containing a variety of flags (Table).
     *
     * Uses tile IDs as subscripts. The correspondence of each bit is as shown below:
     *
     * 0x0001: Impassable downward
     * 0x0002: Impassable leftward
     * 0x0004: Impassable rightward
     * 0x0008: Impassable upward
     * 0x0010: Display on normal character
     * 0x0020: Ladder
     * 0x0040: Bush
     * 0x0080: Counter
     * 0x0100: Damage floor
     * 0x0200: Impassable by boat
     * 0x0400: Impassable by ship
     * 0x0800: Airship cannot land
     * 0xF000: Terrain tag
     * This manual does not discuss bit operations, but they are similar to those in C.
     * We recommend an Internet search using keywords such as "hexadecimal bit operations" when necessary.
     */
    flags: number[];
    /**
     * The ID of the tile set.
     */
    id: number;
    internal: boolean;
    links: string[];
    /**
     * The mode of the tile set (0: Field type, 1: Area type, 2: VX compatible type).
     */
    mode: number;
    /**
     * The name of the tile set.
     */
    name: string;
    parameters: string[];
    platforms: haxe.display.Platform[];
    targets: haxe.display.MetadataTarget[];
    /**
     * The file name of the graphic used as the number index (0-8) tile set.
     *
     * The correspondence between numbers and sets is illustrated in the table below.
     *
     * 0 TileA1
     * 1 TileA2
     * 2 TileA3
     * 3 TileA4
     * 4 TileA5
     * 5 TileB
     * 6 TileC
     * 7 TileD
     * 8 TileE
     */
    tilesetNames: string[];
  };
}

declare namespace rm.types {
  export type MapEncounter = {
    /**
     * An array containing Region IDs.
     */
    regionSet: number[];
    /**
     * The enemy Troop ID.
     */
    troopId: number;
    /**
     * Weight/chance  among enemies on the map that you'll
     * meet this enemy.
     */
    weight: number;
  };
}

/**
 * The game object for the map. It contains scrolling and passage functions
 * along with the game interpreter.
 */
declare class Game_Map {
  constructor();

  _interpreter: Game_Interpreter;
  _mapId: number;
  _tilesetId: number;
  _events: Game_Event[];
  _commonEvents: Game_CommonEvent[];
  _vehicles: Game_Vehicle[];
  _displayX: number;
  _displayY: number;
  _nameDisplay: boolean;
  _scrollDirection: number;
  _scrollRest: number;
  _scrollSpeed: number;
  _parallaxName: string;
  _parallaxZero: boolean;
  _parallaxLoopX: boolean;
  _parallaxLoopY: boolean;
  _parallaxSx: number;
  _parallaxSy: number;
  _parallaxX: number;
  _parallaxY: number;
  _battleback1Name: string;
  _battleback2Name: string;
  _needsRefresh: boolean;

  initialize(): void;

  setup(mapId: number): void;

  /**
   * Returns true if an event is running.
   *
   * @returns {Bool}
   * @memberof Game_Map
   */
  isEventRunning(): boolean;

  /**
   * Returns tile width.
   *
   * @returns {Int}
   * @memberof Game_Map
   */
  tileWidth(): number;

  /**
   * Returns tile height.
   *
   * @returns {Int}
   * @memberof Game_Map
   */
  tileHeight(): number;

  /**
   * Returns map id.
   *
   * @returns {Int}
   * @memberof Game_Map
   */
  mapId(): number;

  /**
   * Returns the tileset id.
   *
   * @returns {Int}
   * @memberof Game_Map
   */
  tilesetId(): number;

  /**
   * Returns the display x coordinate.
   *
   * @returns {Int}
   * @memberof Game_Map
   */
  displayX(): number;

  /**
   * Returns the display y coordinate.
   *
   * @returns {Int}
   * @memberof Game_Map
   */
  displayY(): number;

  parallaxName(): string;

  /**
   * Returns the name of battle back 1.
   *
   * @returns {String}
   * @memberof Game_Map
   */
  battleback1Name(): string;

  /**
   * Returns the name of battle back 2.
   *
   * @returns {String}
   * @memberof Game_Map
   */
  battleback2Name(): string;

  requestRefresh(mapId: number): void;

  /**
   * Returns true if the name display is enabled.
   *
   * @returns {Bool}
   * @memberof Game_Map
   */
  isNameDisplayEnabled(): boolean;

  /**
   * Disables name display.
   *
   * @memberof Game_Map
   */
  disableNameDisplay(): void;

  /**
   * Enable name display.
   *
   * @memberof Game_Map
   */
  enableNameDisplay(): void;

  /**
   * Creates the vehicles for the game map.
   *
   * @memberof Game_Map
   */
  createVehicles(): void;

  /**
   * Refreshes the vehicles on the game map.
   *
   * @memberof Game_Map
   */
  refereshVehicles(): void;

  vehicles(): Game_Vehicle[];

  /**
   * Returns the game vehicle specified by type.
   *
   * @param {String} type
   * @returns {Game_Vehicle}
   * @memberof Game_Map
   */
  vehicle(type: string): Game_Vehicle;

  /**
   * Returns the game boat.
   *
   * @returns {Game_Vehicle}
   * @memberof Game_Map
   */
  boat(): Game_Vehicle;

  /**
   * Returns the game ship.
   *
   * @returns {Game_Vehicle}
   * @memberof Game_Map
   */
  ship(): Game_Vehicle;

  /**
   * Returns the game airship.
   *
   * @returns {Game_Vehicle}
   * @memberof Game_Map
   */
  airship(): Game_Vehicle;

  setupEvents(): void;

  /**
   * Returns all events on the game map.
   *
   * @returns {Array<Game_Event>}
   * @memberof Game_Map
   */
  events(): Game_Event[];

  /**
   * Returns an event, given the event id.
   *
   * @param {Int} eventId
   * @returns {Game_Event}
   * @memberof Game_Map
   */
  event(eventId: number): Game_Event;

  /**
   * Erases the event given the event id.
   *
   * @param {Int} eventId
   * @memberof Game_Map
   */
  eraseEvent(eventId: number): void;

  /**
   * Returns all the parallel running common events.
   *
   * @returns {Array<RPG.CommonEvent>}
   * @memberof Game_Map
   */
  parallelCommonEvents(): rm.types.CommonEvent[];

  setupScroll(): void;

  setupParallax(): void;

  setupBattleback(): void;

  setDisplayPos(x: number, y: number): void;

  parallaxOx(): number;

  parallaxOy(): number;

  /**
   * Returns the tilset of the game map.
   *
   * @returns {RPG.Tileset}
   * @memberof Game_Map
   */
  tileset(): rm.types.Tileset;

  /**
   * Returns the tileset flags of the game map.
   *
   * @returns {Array<Int>}
   * @memberof Game_Map
   */
  tilesetFlags(): number[];

  /**
   * Returns the display name of the game map.
   *
   * @returns {String}
   * @memberof Game_Map
   */
  displayName(): string;

  /**
   * Returns the map width.
   *
   * @returns {Int}
   * @memberof Game_Map
   */
  width(): number;

  /**
   * Returns the map height.
   *
   * @returns {Int}
   * @memberof Game_Map
   */
  height(): number;

  /**
   * Returns the map data.
   *
   * @returns {Array<Int>}
   * @memberof Game_Map
   */
  data(): number[];

  /**
   * Returns true if the map loops horizontally.
   *
   * @returns {Bool}
   * @memberof Game_Map
   */
  isLoopHorizontal(): boolean;

  /**
   * Returns true if the map loops vertically.
   *
   * @returns {Bool}
   * @memberof Game_Map
   */
  isLoopVertical(): boolean;

  /**
   * Returns true if dash is disabled on the map.
   *
   * @returns {Bool}
   * @memberof Game_Map
   */
  isDashDisabled(): boolean;

  /**
   * Returns the list of possible encounters on the current map.
   *
   * @returns {Array<RPG.Map.Encounter>}
   * @memberof Game_Map
   */
  encounterList(): rm.types.MapEncounter[];

  /**
   * Returns the Int of encounter steps on the map.
   *
   * @returns {Int}
   * @memberof Game_Map
   */
  encounterStep(): number;

  /**
   * Returns true if the map is an over world map.
   *
   * @returns {Bool}
   * @memberof Game_Map
   */
  isOverworld(): boolean;

  /**
   * Returns the screen tile x coordinate.
   *
   * @returns {Int}
   * @memberof Game_Map
   */
  screenTileX(): number;

  /**
   * Returns the screen tile y coordinate.
   *
   * @returns {Int}
   * @memberof Game_Map
   */
  screenTileY(): number;

  adjustX(x: number): number;

  adjustY(y: number): number;

  roundX(x: number): number;

  roundY(y: number): number;

  xWithDirection(x: number, d: number): number;

  yWithDirection(y: number, d: number): number;

  roundXWithDirection(x: number, d: number): number;

  roundYWithDirection(y: number, d: number): number;

  deltaX(x1: number, x2: number): number;

  deltaY(y1: number, y2: number): number;

  distance(x1: number, y1: number, x2: number, y2: number): number;

  /**
   * Converts the x coordinate from canvas to map coordinate x.
   *
   * @param {Int} x
   * @returns {Int}
   * @memberof Game_Map
   */
  canvasToMapX(x: number): number;

  /**
   * Converts the y coordinate from canvas to map y coordinate.
   *
   * @param {Int} y
   * @returns {Int}
   * @memberof Game_Map
   */
  canvasToMapY(y: number): number;

  /**
   * Auto plays the game map.
   *
   * @memberof Game_Map
   */
  autoplay(): void;

  refreshIfNeeded(): void;

  refresh(): void;

  refreshTileEvents(): void;

  /**
   * Returns the game events at the specified
   * x and y position.
   * @param {Int} x
   * @param {Int} y
   * @returns {Array<Game_Event>}
   * @memberof Game_Map
   */
  eventsXy(x: number, y: number): Game_Event[];

  eventsXyNt(x: number, y: number): Game_Event[];

  tileEventsXy(x: number, y: number): Game_Event[];

  eventIdXy(x: number, y: number): number;

  scrollDown(distance: number): void;

  scrollLeft(distance: number): void;

  scrollRight(distance: number): void;

  scrollUp(distance: number): void;

  /**
   * Returns true if the x and y coordinates are valid.
   *
   * @param {Int} x
   * @param {Int} y
   * @returns {Bool}
   * @memberof Game_Map
   */
  isValid(x: number, y: number): boolean;

  checkPassage(x: number, y: number, bit: number): boolean;

  /**
   * Returns the tile id at the specified x, y, and z coordinates.
   *
   * @param {Int} x
   * @param {Int} y
   * @param {Int} z
   * @returns {Int}
   * @memberof Game_Map
   */
  tileId(x: number, y: number, z: number): number;

  layeredTiles(x: number, y: number): number[];

  allTiles(x: number, y: number): number[];

  autotileType(x: number, y: number, z: number): number;

  isPassable(x: number, y: number, d: number): boolean;

  isBoatPassable(x: number, y: number): boolean;

  isShipPassable(x: number, y: number): boolean;

  isAirshipLandOk(x: number, y: number): boolean;

  checkLayeredTilesFlags(x: number, y: number, bit: number): boolean;

  /**
   * Returns true if the specified element at the given x and y coordinates
   * is a ladder.
   * @param {Int} x
   * @param {Int} y
   * @returns {Bool}
   * @memberof Game_Map
   */
  isLadder(x: number, y: number): boolean;

  isBush(x: number, y: number): boolean;

  isCounter(x: number, y: number): boolean;

  isDamageFloor(x: number, y: number): boolean;

  terrainTag(x: number, y: number): number;

  regionId(x: number, y: number): number;

  startScroll(direction: number, distance: number, speed: number): void;

  isScrolling(): boolean;

  /**
   * Updates the game map, given that the scene is active.
   *
   * @param {Bool} sceneActive
   * @memberof Game_Map
   */
  update(sceneActive: boolean): void;

  updateScroll(): void;

  scrollDistance(): number;

  doScroll(direction: number, distance: number): void;

  /**
   * Updates all events on the map.
   *
   * @memberof Game_Map
   */
  updateEvents(): void;

  /**
   * Updates all game vehicles on the map.
   *
   * @memberof Game_Map
   */
  updateVehicles(): void;

  /**
   * Updates the map parallaxes.
   *
   * @memberof Game_Map
   */
  updateParallax(): void;

  /**
   * Changes them ap tileset, given the tileset id.
   *
   * @param {Int} tilesetId
   * @memberof Game_Map
   */
  changeTileset(tilesetId: number): void;

  changeBattleback(battleback1Name: string, battleback2Name: string): void;

  changeParallax(name: string, loopX: boolean, loopY: boolean, sx: number, sy: number): void;

  /**
   * Updates the map's game interpreter.
   *
   * @memberof Game_Map
   */
  updateInterpreter(): void;

  /**
   * Unlocks an event on the map given the event id.
   *
   * @param {Int} eventId
   * @memberof Game_Map
   */
  unlockEvent(eventId: number): void;

  setupStartingEvent(): boolean;

  setupTestEvent(): boolean;

  setupStartingMapEvent(): boolean;

  /**
   * Sets up an auto run common event.
   *
   * @returns {Bool}
   * @memberof Game_Map
   */
  setupAutorunCommonEvent(): boolean;

  /**
   * Returns true if any event is starting on the map.
   *
   * @returns {Bool}
   * @memberof Game_Map
   */
  isAnyEventStarting(): boolean;
}

declare namespace rm.types {
  export const enum MessageBackgroundType {
    base = -1,
  }
}

declare namespace rm.types {
  export const enum MessagePositionType {
    base = -1,
  }
}

declare namespace rm.types {
  export const enum ChoiceDefaultType {
    base = -1,
  }
}

declare namespace rm.types {
  export const enum ChoiceCancelType {
    base = -1,
  }
}

declare namespace rm.types {
  export const enum ChocieBackgroundType {
    base = -1,
  }
}

declare namespace rm.types {
  export const enum ChoicePositionType {
    base = -1,
  }
}

declare class Game_Message {
  constructor();

  _texts: string[];
  _choices: string[];
  _faceName: string;
  _faceIndex: number;
  _background: rm.types.MessageBackgroundType;
  _positionType: rm.types.MessagePositionType;
  _choiceDefaultType: rm.types.ChoiceDefaultType;
  _choiceCancelType: rm.types.ChoiceCancelType;
  _choiceBackground: rm.types.ChocieBackgroundType;
  _choicePositionType: rm.types.ChoicePositionType;
  _numInputVariableId: number;
  _numInputMaxDigits: number;
  _itemChoiceVariableId: number;
  _itemChoiceItypeId: number;
  _scrollMode: boolean;
  _scrollSpeed: number;
  _scrollNoFast: boolean;
  _choiceCallback: (n: number) => void;

  initialize(): void;

  clear(): void;

  choices(): string[];

  /**
   * Returns the name of the face image used for the message.
   *
   * @returns {String}
   * @memberof Game_Message
   */
  faceName(): string;

  /**
   * Returns the face index within the face image to display the
   * correct face.
   * @returns {number}
   * @memberof Game_Message
   */
  faceIndex(): number;

  /**
   * Returns the background associated with the message;
   * this is the background
   * @returns {number}
   * @memberof Game_Message
   */
  background(): number;

  /**
   * Returns the current position type of the message window.
   *
   * @returns {number}
   * @memberof Game_Message
   */
  positionType(): number;

  choiceDefaultType(): number;

  choiceCancelType(): number;

  /**
   * Returns the background type of the choice window.
   *
   * @returns {number}
   * @memberof Game_Message
   */
  choiceBackground(): number;

  /**
   * Returns the position of the choice window.
   *
   * @returns {number}
   * @memberof Game_Message
   */
  choicePositionType(): number;

  /**
   * Returns the number input variable id.
   *
   * @returns {number}
   * @memberof Game_Message
   */
  numInputVariableId(): number;

  /**
   * Returns the number input maximum digits.
   *
   * @returns {number}
   * @memberof Game_Message
   */
  numInputMaxDigits(): number;

  /**
   * Returns the item choice variable id.
   *
   * @returns {number}
   * @memberof Game_Message
   */
  itemChoiceVariableId(): number;

  /**
   * Returns the item choice item type id.
   *
   * @returns {number}
   * @memberof Game_Message
   */
  itemChoiceItypeId(): number;

  /**
   * Returns true if the scroll mode is set to true.
   *
   * @returns {boolean}
   * @memberof Game_Message
   */
  scrollMode(): boolean;

  /**
   * Returns the scroll speed.
   *
   * @returns {number}
   * @memberof Game_Message
   */
  scrollSpeed(): number;

  /**
   * Returns true if the scroll is set to not being fast.
   *
   * @returns {boolean}
   * @memberof Game_Message
   */
  scrollNoFast(): boolean;

  /**
   * Adds text to the game message object.
   *
   * @param {String} text
   * @memberof Game_Message
   */
  add(text: string): void;

  setFaceImage(faceName: string, faceIndex: number): void;

  /**
   * Sets the background of the message window;
   * options are 0 (fully opaque), 1 (transparent), 2 (invisible background).
   * The default is 0.
   * @param {number} background
   * @memberof Game_Message
   */
  setBackground(background: number): void;

  /**
   * Sets the position of the message window;
   * default is 2.
   * @param {number} positionType
   * @memberof Game_Message
   */
  setPositionType(positionType: number): void;

  /**
   * Sets the choices within the choice window;
   * sets the default and cancel choices for the window.
   * @param {Array<String>} choices
   * @param {number} defaultType
   * @param {number} cancelType
   * @memberof Game_Message
   */
  setChoices(choices: string[], defaultType: number, cancelType: number): void;

  setChoiceBackground(background: number): void;

  /**
   * Sets the position of the choice window associated with the
   * message window.
   * @param {number} positionType
   * @memberof Game_Message
   */
  setChoicePositionType(positionType: number): void;

  /**
   * Sets the number input and associates it with a variable id;
   * the max number of digits can also be set.
   * @param {number} variableId
   * @param {number} maxDigits
   * @memberof Game_Message
   */
  setNumberInput(variableId: number, maxDigits: number): void;

  /**
   * Sets the choice and associates it with a variable id;
   * sets the itemtype id associated with the choice.
   * @param {number} variableId
   * @param {number} itemType
   * @memberof Game_Message
   */
  setItemChoice(variableId: number, itemType: number): void;

  /**
   * Sets the scroll speed of the message window;
   * disable fast movement if noFast is set to true.
   * @param {number} speed
   * @param {boolean} noFast
   * @memberof Game_Message
   */
  setScroll(speed: number, noFast: boolean): void;

  /**
   * Sets a callback to be associated with a specific choice;
   * a callback is a JavaScript function that will be run when the
   * choice is selected.
   * @param {((n: number) => Void)} callback
   * @memberof Game_Message
   */
  setChoiceCallback(callback: (n: number) => void): void;

  onChoice(n: number): void;

  /**
   * Returns true if the game message object has text.
   *
   * @returns {boolean}
   * @memberof Game_Message
   */
  hasText(): boolean;

  /**
   * Returns true if the game message object has a set of choices.
   *
   * @returns {boolean}
   * @memberof Game_Message
   */
  isChoice(): boolean;

  /**
   * Returns true if the game message object has a number input attached.
   *
   * @returns {boolean}
   * @memberof Game_Message
   */
  isNumberInput(): boolean;

  /**
   * Returns true if the game message object has an item choice attached.
   *
   * @returns {boolean}
   * @memberof Game_Message
   */
  isItemChoice(): boolean;

  /**
   * Returns true if the game message object has text, choices, number input,
   * or item choice.
   * @returns {boolean}
   * @memberof Game_Message
   */
  isBusy(): boolean;

  /**
   * Creates a new page for the text within the message window.
   *
   * @memberof Game_Message
   */
  newPage(): void;

  /**
   * Returns all of the text contained within the message.
   *
   * @returns {String}
   * @memberof Game_Message
   */
  allText(): string;
}

/**
 * Superclass of Game_Party & Game_Troop
 *
 * @class Game_Unit
 */
declare class Game_Unit {
  constructor();

  _inBattle: boolean;

  initialize(): void;

  /**
   * Returns true if unit is in battle.
   *
   * @returns {boolean}
   * @memberof Game_Unit
   */
  inBattle(): boolean;

  /**
   * Returns the list of battlers.
   *
   * @returns {Array<Game_Battler>}
   * @memberof Game_Unit
   */
  members(): Game_Battler[];

  /**
   * Returns the list of alive battlers.
   *
   * @returns {Array<Game_Battler>}
   * @memberof Game_Unit
   */
  aliveMembers(): Game_Battler[];

  /**
   * Returns the list of dead battlers.
   *
   * @returns {Array<Game_Battler>}
   * @memberof Game_Unit
   */
  deadMembers(): Game_Battler[];

  /**
   * Returns the list of movable members.
   *
   * @returns {Array<Game_Battler>}
   * @memberof Game_Unit
   */
  movableMembers(): Game_Battler[];

  /**
   * Clears the unit's actions.
   *
   * @memberof Game_Unit
   */
  clearActions(): void;

  /**
   * Returns the agility of the unit.
   *
   * @returns {Int}
   * @memberof Game_Unit
   */
  agility(): number;

  tgrSum(): number;

  /**
   * Returns a random target from the game unit.
   *
   * @returns {Game_Battler}
   * @memberof Game_Unit
   */
  randomTarget(): Game_Battler;

  /**
   * Returns a random dead target from the game unit.
   *
   * @returns {Game_Battler}
   * @memberof Game_Unit
   */
  randomDeadTarget(): Game_Battler;

  smoothTarget(index: number): Game_Battler;

  smoothDeadTarget(index: number): Game_Battler;

  /**
   * Clears the action results.
   *
   * @memberof Game_Unit
   */
  clearResults(): void;

  /**
   * Handler for when battle is started.
   *
   * @memberof Game_Unit
   */
  onBattleStart(): void;

  /**
   * Handler for when battle has ended.
   *
   * @memberof Game_Unit
   */
  onBattleEnd(): void;

  /**
   * Creates the action's of the game unit.
   *
   * @memberof Game_Unit
   */
  makeActions(): void;

  /**
   * Selects a member of the unit given a battler.
   *
   * @param {Game_Battler} activeMember
   * @memberof Game_Unit
   */
  select(activeMember: Game_Battler): void;

  /**
   * Returns true if all members of the unit are dead.
   *
   * @returns {Bool}
   * @memberof Game_Unit
   */
  isAllDead(): boolean;

  substituteBattler(): Game_Battler;
}

/**
 * The game object for the party. Contains information
 * such as gold and items.
 */
declare class Game_Party extends Game_Unit {
  protected constructor();

  _gold: number;
  _steps: number;
  _lastItem: Game_Item;
  _menuActorId: number;
  _targetActorId: number;
  _actors: Game_Actor[];
  /**
   * Data structure.
   * [ItemId:Int] : Int
   */
  _items: { [key: string]: any };
  /**
   * Data structure.
   * [weaponId:Int] : Int
   */
  _weapons: { [key: string]: any };
  /**
   * Data structure.
   * [armorId:Int] : Int
   */
  _armors: { [key: string]: any };

  /**
   * Returns all party members.
   *
   * @returns {Array<Game_Actor>}
   * @memberof Game_Party
   */
  members(): Game_Actor[];

  /**
   * Returns the living party members.
   *
   * @returns {Array<Game_Actor>}
   * @memberof Game_Party
   */
  aliveMembers(): Game_Actor[];

  /**
   * Returns the dead party members.
   *
   * @returns {Array<Game_Actor>}
   * @memberof Game_Party
   */
  deadMembers(): Game_Actor[];

  /**
   * Returns the movable members in the party.
   *
   * @returns {Array<Game_Actor>}
   * @memberof Game_Party
   */
  movableMembers(): Game_Actor[];

  /**
   * Returns the battle members in the party.
   *
   * @returns {Array<Game_Actor>}
   * @memberof Game_Party
   */
  battleMembers(): Game_Actor[];

  /**
   * Initialize all party items.
   *
   * @memberof Game_Party
   */
  initAllItems(): void;

  /**
   * Returns true if the game party exists.
   *
   * @returns {Bool}
   * @memberof Game_Party
   */
  exists(): boolean;

  /**
   * Returns the size of the party.
   *
   * @returns {Int}
   * @memberof Game_Party
   */
  size(): number;

  /**
   * Returns true if the game party is empty.
   *
   * @returns {Bool}
   * @memberof Game_Party
   */
  isEmpty(): boolean;

  /**
   * Returns the maximum battle members in the party.
   *
   * @returns {Int}
   * @memberof Game_Party
   */
  maxBattleMembers(): number;

  /**
   * Returns the leader of the party.
   *
   * @returns {Game_Actor}
   * @memberof Game_Party
   */
  leader(): Game_Actor;

  /**
   * Revive the battle members of the party.
   *
   * @memberof Game_Party
   */
  reviveBattleMembers(): void;

  items(): rm.types.Item[];

  /**
   * Returns the weapons of the party.
   *
   * @returns {Array<RPG.Weapon>}
   * @memberof Game_Party
   */
  weapons(): rm.types.Weapon[];

  /**
   * Returns the party's armor.
   *
   * @returns {Array<RPG.Armor>}
   * @memberof Game_Party
   */
  armors(): rm.types.Armor[];

  /**
   * Returns the party's equippable items.
   *
   * @returns {Array<RPG.EquipItem>}
   * @memberof Game_Party
   */
  equipItems(): rm.types.EquipItem[];

  /**
   * Returns all items within the party's posession.
   * Items can be of equip item, or item type.
   * @returns {Array<RPG.BaseItem>}
   * @memberof Game_Party
   */
  allItems(): rm.types.BaseItem[];

  itemContainer(item: rm.types.BaseItem): { ItemId: number };

  /**
   * Sets up the starting party members.
   *
   * @memberof Game_Party
   */
  setupStartingMembers(): void;

  name(): string;

  /**
   * Sets up a test battle with the party.
   *
   * @memberof Game_Party
   */
  setupBattleTest(): void;

  /**
   * Sets up the battle test members.
   *
   * @memberof Game_Party
   */
  setupBattleTestMembers(): void;

  /**
   * Sets up the battle test items.
   *
   * @memberof Game_Party
   */
  setupBattleTestItems(): void;

  /**
   * Returns the highest level in the party.
   *
   * @returns {Int}
   * @memberof Game_Party
   */
  highestLevel(): number;

  /**
   * Adds an actor to the party given the actor id.
   *
   * @param {Int} actorId
   * @memberof Game_Party
   */
  addActor(actorId: number): void;

  /**
   * Removes an actor from the party given the actor id.
   *
   * @param {Int} actorId
   * @memberof Game_Party
   */
  removeActor(actorId: number): void;

  /**
   * Returns party gold.
   *
   * @returns {Int}
   * @memberof Game_Party
   */
  gold(): number;

  /**
   * Increases the party gold given a specified amount.
   *
   * @param {Int} amount
   * @memberof Game_Party
   */
  gainGold(amount: number): void;

  /**
   * Decreases the party gold given a specified amount.
   *
   * @param {Int} amount
   * @memberof Game_Party
   */
  loseGold(amount: number): void;

  /**
   * Returns maximum gold of the party.
   *
   * @returns {Int}
   * @memberof Game_Party
   */
  maxGold(): number;

  /**
   * Returns the Int of steps the party has taken.
   *
   * @returns {Int}
   * @memberof Game_Party
   */
  steps(): number;

  /**
   * Increases the Int of steps the party has taken.
   *
   * @memberof Game_Party
   */
  increaseSteps(): void;

  /**
   * Returns the Int of items in the possession of the party of the
   * given item.
   * @param {RPG.BaseItem} item
   * @returns {Int}
   * @memberof Game_Party
   */
  numItems(item: rm.types.BaseItem): number;

  /**
   * Returns the maximum Int of items of the given item.
   *
   * @param {RPG.BaseItem} item
   * @returns {Int}
   * @memberof Game_Party
   */
  maxItems(item: rm.types.BaseItem): number;

  hasMaxItems(item: rm.types.BaseItem): boolean;

  /**
   * Returns true if the party has the given item;
   * if includeEquip is set to true, this will also check party equipment.
   * @param {RPG.BaseItem} item
   * @param {Bool} includeEquip
   * @returns {Bool}
   * @memberof Game_Party
   */
  hasItem(item: rm.types.BaseItem, includeEquip: boolean): boolean;

  /**
   * Returns true if any party member has the specified equip item.
   *
   * @param {RPG.EquipItem} item
   * @returns {Bool}
   * @memberof Game_Party
   */
  isAnyMemberEquipped(item: rm.types.EquipItem): boolean;

  gainItem(item: rm.types.BaseItem, amount: number, includeEquip: boolean): void;

  discardMembersEquip(item: rm.types.EquipItem, amount: number): void;

  loseItem(item: rm.types.BaseItem, amount: number, includeEquip: boolean): void;

  /**
   * Has the party consume the given item.
   *
   * @param {RPG.BaseItem} item
   * @memberof Game_Party
   */
  consumeItem(item: rm.types.BaseItem): void;

  /**
   * Returns true if the party can use the item.
   *
   * @param {RPG.BaseItem} item
   * @returns {Bool}
   * @memberof Game_Party
   */
  canUse(item: rm.types.BaseItem): boolean;

  canInput(): boolean;

  /**
   * Handler for when the player walks.
   *
   * @memberof Game_Party
   */
  onPlayerWalk(): void;

  /**
   * Returns the actor that will be used in the current menu;
   * this is for menu scenes that target one actor.
   * @returns {Game_Actor}
   * @memberof Game_Party
   */
  menuActor(): Game_Actor;

  setMenuActor(actor: Game_Actor): void;

  makeMenuActorNext(): void;

  makeMenuActorPrevious(): void;

  targetActor(): Game_Actor;

  setTargetActor(actor: Game_Actor): void;

  /**
   * Returns the last item selected by the game party.
   *
   * @returns {RPG.BaseItem}
   * @memberof Game_Party
   */
  lastItem(): rm.types.BaseItem;

  setLastItem(item: rm.types.BaseItem): void;

  swapOrder(index1: number, index2: number): void;

  /**
   * Returns the characters that go on the save life.
   *
   * @returns {Array<Array<any>>}
   * @memberof Game_Party
   */
  charactersForSavefile(): any[][];

  /**
   * Returns the actor faces for the save file.
   *
   * @returns {Array<Array<any>>}
   * @memberof Game_Party
   */
  facesForSavefile(): any[][];

  partyAbility(abilityId: number): boolean;

  /**
   * Returns true if the encounter rate is set to half.
   *
   * @returns {Bool}
   * @memberof Game_Party
   */
  hasEncounterHalf(): boolean;

  /**
   * Returns true if the encounter rate is set to none.
   *
   * @returns {Bool}
   * @memberof Game_Party
   */
  hasEncounterNone(): boolean;

  hasCancelSurprise(): boolean;

  /**
   * Returns true if the party has an increased chance of preemptive strike.
   *
   * @returns {Bool}
   * @memberof Game_Party
   */
  hasRaisePreemptive(): boolean;

  /**
   * Returns true if the party has double gold in effect.
   *
   * @returns {Bool}
   * @memberof Game_Party
   */
  hasGoldDouble(): boolean;

  hasDropItemDouble(): boolean;

  ratePreemptive(troopAgi: number): number;

  rateSurprise(troopAgi: number): number;

  /**
   * Performs victory motion for the entire party.
   *
   * @memberof Game_Party
   */
  performVictory(): void;

  /**
   * Performs escape motion for the entire party.
   *
   * @memberof Game_Party
   */
  performEscape(): void;

  /**
   * Remove battle states from all actors in the party.
   *
   * @memberof Game_Party
   */
  removeBattleStates(): void;

  /**
   * Refreshes the motion on all actors in the party.
   *
   * @memberof Game_Party
   */
  requestMotionRefresh(): void;

  static ABILITY_ENCOUNTER_HALF: number;
  static ABILITY_ENCOUNTER_NONE: number;
  static ABILITY_CANCEL_SURPRISE: number;
  static ABILITY_RAISE_PREEMPTIVE: number;
  static ABILITY_GOLD_DOUBLE: number;
  static ABILITY_DROP_ITEM_DOUBLE: number;
}

/**
 * -----------
 * The game object class for a picture.
 *
 */
declare class Game_Picture {
  constructor();

  /**
   * _name property of the current picture.
   *
   * @protected
   * @type {string}
   * @memberof Game_Picture
   */
  _name: string;
  _origin: number;
  _x: number;
  _y: number;
  _scaleX: number;
  _scaleY: number;
  _opacity: number;
  /**
   * Blend Mode, accepts an integer.
   */
  _blendMode: number;
  _targetX: number;
  _targetY: number;
  _targetScaleX: number;
  _targetScaleY: number;
  _targetOpacity: number;
  _duration: number;
  /**
   * Tone of the picture, in RGB format.
   * 0 - 255, for all three tone elements.
   */
  _tone: number[];
  _toneTarget: number[];
  _toneDuration: number;
  _angle: number;
  _rotationSpeed: number;

  initialize(): void;

  /**
   * Returns the name of the game picture.
   *
   * @returns {string}
   * @memberof Game_Picture
   */
  name(): string;

  /**
   * Returns the origin of the game picture.
   *
   * @returns {number}
   * @memberof Game_Picture
   */
  origin(): number;

  /**
   * Returns the picture x coordinate.
   *
   * @returns {number}
   * @memberof Game_Picture
   */
  x(): number;

  /**
   * Returns the picture y coordinate.
   *
   * @returns {number}
   * @memberof Game_Picture
   */
  y(): number;

  /**
   * Returns x scale of the game picture.
   *
   * @returns {number}
   * @memberof Game_Picture
   */
  scaleX(): number;

  /**
   * Returns the y scale of the game picture.
   *
   * @returns {number}
   * @memberof Game_Picture
   */
  scaleY(): number;

  /**
   * Returns the opacity of the game picture.
   *
   * @returns {number}
   * @memberof Game_Picture
   */
  opacity(): number;

  /**
   * Returns the blend mode of the game picture.
   *
   * @returns {number}
   * @memberof Game_Picture
   */
  blendMode(): number;

  /**
   * Returns the tone of the game picture.
   *
   * @returns {Array<number>}
   * @memberof Game_Picture
   */
  tone(): number[];

  /**
   * Returns the angle of the game picture.
   *
   * @returns {number}
   * @memberof Game_Picture
   */
  angle(): number;

  initBasic(): void;

  initTarget(): void;

  initTone(): void;

  initRotation(): void;

  show(
    name: string,
    origin: number,
    x: number,
    y: number,
    scaleX: number,
    scaleY: number,
    opacity: number,
    blendMode: number,
  ): void;

  move(
    origin: number,
    x: number,
    y: number,
    scaleX: number,
    scaleY: number,
    opacity: number,
    blendMode: number,
    duration: number,
  ): void;

  rotate(speed: number): void;

  tint(tone: number[], duration: number): void;

  /**
   * Erases the game picture.
   *
   * @memberof Game_Picture
   */
  erase(): void;

  /**
   * Updates the game picture.
   *
   * @memberof Game_Picture
   */
  update(): void;

  /**
   * Updates the movement of the game picture.
   *
   * @memberof Game_Picture
   */
  updateMove(): void;

  /**
   * Updates the tone of the game picture.
   *
   * @memberof Game_Picture
   */
  updateTone(): void;

  /**
   * Updates the rotation of the game picture.
   *
   * @memberof Game_Picture
   */
  updateRotation(): void;
}

/**
 * The game object class for the player. It contains,
 * encounter information, x, y, vehicle type, and whether
 * the player is dashing. The Game Player State is this
 * object.
 */
declare class Game_Player extends Game_Character {
  protected constructor();

  _vehicleType: string;
  _vehicleGettingOn: boolean;
  _vehicleGettingOff: boolean;
  _dashing: boolean;
  _needsMapReload: boolean;
  _transferring: boolean;
  _newX: number;
  _newY: number;
  _newDirection: rm.types.Direction;
  _fadeType: number;
  _followers: Game_Followers;
  _encounterCount: number;

  /**
   * Clears the transfer information for the player.
   *
   * @memberof Game_Player
   */
  clearTransferInfo(): void;

  /**
   * Returns the player followers (party members).
   *
   * @returns {Game_Followers}
   * @memberof Game_Player
   */
  followers(): Game_Followers;

  /**
   * Refreshes the game player.
   *
   * @memberof Game_Player
   */
  refresh(): void;

  /**
   * Returns true if the player is stopping.
   *
   * @returns {Bool}
   * @memberof Game_Player
   */
  isStopping(): boolean;

  /**
   * Reserves a transfer of the player to the specified map, at the given
   * x and y coordinates, facing the given direction (d). Using a specific fade.
   * @param {Int} mapId
   * @param {Int} x
   * @param {Int} y
   * @param {Int} [d]
   * @param {Int} [fadeType]
   * @memberof Game_Player
   */
  reserveTransfer(mapId: number, x: number, y: number, d?: number, fadeType?: number): void;

  requestMapReload(): void;

  isTransferring(): boolean;

  /**
   * Returns the new map id.
   *
   * @returns {Int}
   * @memberof Game_Player
   */
  newMapId(): number;

  fadeType(): number;

  /**
   * Performs a transfer of the player to a different area or map.
   *
   * @memberof Game_Player
   */
  performTransfer(): void;

  isMapPassable(x: number, y: number, d: number): boolean;

  /**
   * Returns the current vehicles the player is riding in.
   *
   * @returns {Game_Vehicle}
   * @memberof Game_Player
   */
  vehicle(): Game_Vehicle;

  /**
   * Returns true if the player is in a boat.
   *
   * @returns {Bool}
   * @memberof Game_Player
   */
  isInBoat(): boolean;

  /**
   * Returns true if the player is in a ship.
   *
   * @returns {Bool}
   * @memberof Game_Player
   */
  isInShip(): boolean;

  /**
   * Returns true if the player is in an airship.
   *
   * @returns {Bool}
   * @memberof Game_Player
   */
  isInAirship(): boolean;

  /**
   * Returns true if the player is in a vehicle.
   *
   * @returns {Bool}
   * @memberof Game_Player
   */
  isInVehicle(): boolean;

  /**
   * Returns true if the player is in their normal state.
   *
   * @returns {Bool}
   * @memberof Game_Player
   */
  isNormal(): boolean;

  /**
   * Returns true if the player is dashing.
   *
   * @returns {Bool}
   * @memberof Game_Player
   */
  isDashRing(): boolean;

  isDebugThrough(): boolean;

  isCollided(x: number, y: number): boolean;

  /**
   * Returns the player's center x coordinate.
   *
   * @returns {Int}
   * @memberof Game_Player
   */
  centerX(): number;

  /**
   * Returns the player's center y coordinate.
   *
   * @returns {Int}
   * @memberof Game_Player
   */
  centerY(): number;

  center(x: number, y: number): void;

  locate(x: number, y: number): void;

  increaseSteps(): void;

  /**
   * Creates the encounter count for the player.
   *
   * @memberof Game_Player
   */
  makeEncounterCount(): void;

  /**
   * Creates the encounter troop id and returns it.
   *
   * @returns {Int}
   * @memberof Game_Player
   */
  makeEncounterTroopId(): number;

  meetsEncounterConditions(encounter: rm.types.MapEncounter): boolean;

  /**
   * Executes an encounter.
   *
   * @returns {Bool}
   * @memberof Game_Player
   */
  executeEncounter(): boolean;

  /**
   * Has the player start a map event at the given x and y coordinates.
   * Also passing the triggers and whether the event start is normal.
   * @param {Int} x
   * @param {Int} y
   * @param {Array<Int>} triggers
   * @param {Bool} normal
   * @memberof Game_Player
   */
  startMapEvent(x: number, y: number, triggers: number[], normal: boolean): void;

  moveByInput(): void;

  /**
   * Returns true if the player can move.
   *
   * @returns {Bool}
   * @memberof Game_Player
   */
  canMove(): boolean;

  /**
   * Gets the input direction of the player as a Int.
   *
   * @returns {Int}
   * @memberof Game_Player
   */
  getInputDirection(): number;

  executeMove(direction: number): void;

  update(sceneActive?: boolean): void;

  /**
   * Updates the dashing of the player.
   *
   * @memberof Game_Player
   */
  updateDashing(): void;

  /**
   * Returns true if the dash button is pressed.
   *
   * @returns {Bool}
   * @memberof Game_Player
   */
  isDashButtonPressed(): boolean;

  updateScroll(lastScrolledX: number, lastScrolledY: number): void;

  /**
   * Updates the vehicle.
   *
   * @memberof Game_Player
   */
  updateVehicle(): void;

  /**
   * Updates the player getting on the vehicle.
   *
   * @memberof Game_Player
   */
  updateVehicleGetOn(): void;

  /**
   * Updates the player getting off the vehicle.
   *
   * @memberof Game_Player
   */
  updateVehicleGetOff(): void;

  updateNonmoving(wasMoving: boolean): void;

  triggerAction(): boolean;

  triggerButtonAction(): boolean;

  /**
   * Returns true if the player triggered a touch action.
   *
   * @returns {Bool}
   * @memberof Game_Player
   */
  triggerTouchAction(): boolean;

  triggerTouchActionD1(x1: number, y1: number): boolean;

  triggerTouchActionD2(x2: number, y2: number): boolean;

  triggerTouchActionD3(x2: number, y2: number): boolean;

  /**
   * Updates the player encounter count.
   *
   * @memberof Game_Player
   */
  updateEncounterCount(): void;

  /**
   * Returns true if the player can encounter enemies.
   *
   * @returns {Bool}
   * @memberof Game_Player
   */
  canEncounter(): boolean;

  /**
   * Returns the encounter progress value of the player.
   *
   * @returns {Int}
   * @memberof Game_Player
   */
  encounterProgressValue(): number;

  checkEventTriggerHere(triggers: number[]): void;

  checkEventTriggerThere(triggers: number[]): void;

  /**
   * Returns true if the player can start local events.
   *
   * @returns {Bool}
   * @memberof Game_Player
   */
  canStartLocalEvents(): boolean;

  /**
   * Returns true if the player is getting on/off a vehicle.
   *
   * @returns {Bool}
   * @memberof Game_Player
   */
  getOnOffVehicle(): boolean;

  /**
   * Returns true if the player is getting on a vehicle.
   *
   * @returns {Bool}
   * @memberof Game_Player
   */
  getOnVehicle(): boolean;

  /**
   * Returns true if the player is getting off a vehicle.
   *
   * @returns {Bool}
   * @memberof Game_Player
   */
  getOffVehicle(): boolean;

  /**
   * Forces the player to move forward.
   *
   * @memberof Game_Player
   */
  forceMoveForward(): void;

  /**
   * Returns true if the player is on a floor that does damage.
   *
   * @returns {Bool}
   * @memberof Game_Player
   */
  isOnDamageFloor(): boolean;

  /**
   * Moves the player straight, given a direction.
   *
   * @param {Int} d
   * @memberof Game_Player
   */
  moveStraight(d: number): void;

  /**
   * Moves the player diagonally, given a horizontal
   * and vertical direction. The numpad represents the directions.
   * @param {Int} horz
   * @param {Int} vert
   * @memberof Game_Player
   */
  moveDiagonally(horz: number, vert: number): void;

  /**
   * Has the player jump in the given direction at the specified
   * x and y coordinate. This x and y will be added to the player's current
   * position.
   * @param {Int} xPlus
   * @param {Int} yPlus
   * @memberof Game_Player
   */
  jump(xPlus: number, yPlus: number): void;

  /**
   * Shows the player followers.
   *
   * @memberof Game_Player
   */
  showFollowers(): void;

  /**
   * Hides the player followers.
   *
   * @memberof Game_Player
   */
  hideFollowers(): void;

  /**
   * Gather followers around the player.
   *
   * @memberof Game_Player
   */
  gatherFollowers(): void;

  /**
   * Returns true if the followers are currently gathering.
   *
   * @returns {Bool}
   * @memberof Game_Player
   */
  areFollowersGathering(): boolean;

  /**
   * Returns true if the followers are gathered.
   *
   * @returns {Bool}
   * @memberof Game_Player
   */
  areFollowersGathered(): boolean;
}

/**
 * The game object class for screen effect data, such as
 * changes in color tone and flashes.
 * aliased as $gameScreen.
 * @class Game_Screen
 */
declare class Game_Screen {
  constructor();

  initialize(): void;

  clear(): void;

  /**
   * Handler for when the battle starts in game; prepares
   * the screen for the battle scene.
   * @memberof Game_Screen
   */
  onBattleStart(): void;

  /**
   * Returns the brightness of the game screen.
   *
   * @returns {number}
   * @memberof Game_Screen
   */
  brightness(): number;

  tone(): number[];

  flashColor(): number[];

  shake(): number;

  /**
   * Returns the  zoom x coordinate of the screen.
   *
   * @returns {number}
   * @memberof Game_Screen
   */
  zoomX(): number;

  /**
   * Returns the zoom y coordiante of the screen.
   *
   * @returns {number}
   * @memberof Game_Screen
   */
  zoomY(): number;

  /**
   * Returns the zoom scale of the screen.
   *
   * @returns {number}
   * @memberof Game_Screen
   */
  zoomScale(): number;

  /**
   * Returns the current weather type.
   *
   * @returns {string}
   * @memberof Game_Screen
   */
  weatherType(): string;

  /**
   * Returns the weather power.
   *
   * @returns {number}
   * @memberof Game_Screen
   */
  weatherPower(): number;

  /**
   * Returns the specified picture given the picture id.
   *
   * @param {number} pictureId
   * @returns {Game_Picture}
   * @memberof Game_Screen
   */
  picture(pictureId: number): Game_Picture;

  /**
   * Returns the real picture id, given the picture id.
   *
   * @param {number} pictureId
   * @returns {number}
   * @memberof Game_Screen
   */
  realPictureId(pictureId: number): number;

  /**
   * Clears the screen fade.
   *
   * @memberof Game_Screen
   */
  clearFade(): void;

  /**
   * Clears the screen tone.
   *
   * @memberof Game_Screen
   */
  clearTone(): void;

  /**
   * Clears the screen flash.
   *
   * @memberof Game_Screen
   */
  clearFlash(): void;

  /**
   * Clears the screen shake.
   *
   * @memberof Game_Screen
   */
  clearShake(): void;

  /**
   * Clears the screen zoom.
   *
   * @memberof Game_Screen
   */
  clearZoom(): void;

  /**
   * Clears the screen weather.
   *
   * @memberof Game_Screen
   */
  clearWeather(): void;

  /**
   * Clears the pictures set on the game screen.
   *
   * @memberof Game_Screen
   */
  clearPictures(): void;

  /**
   * Erases the battle pictures.
   *
   * @memberof Game_Screen
   */
  eraseBattlePictures(): void;

  /**
   * Returns the maximum number of pictures set on the game screen.
   *
   * @returns {number}
   * @memberof Game_Screen
   */
  maxPictures(): number;

  startFadeOut(duration: number): void;

  startFadeIn(duration: number): void;

  startTint(tone: number[], duration: number): void;

  startFlash(color: number[], duration: number): void;

  startShake(power: number, speed: number, duration: number): void;

  startZoom(x: number, y: number, scale: number, duration: number): void;

  setZoom(x: number, y: number, scale: number): void;

  changeWeather(type: string, power: number, duration: number): void;

  /**
   * Updates the game screen.
   *
   * @memberof Game_Screen
   */
  update(): void;

  /**
   * Updates the screen fade out.
   *
   * @memberof Game_Screen
   */
  updateFadeOut(): void;

  /**
   * Updates the screen fade in.
   *
   * @memberof Game_Screen
   */
  updateFadeIn(): void;

  /**
   * Updates the screen tone.
   *
   * @memberof Game_Screen
   */
  updateTone(): void;

  /**
   * Update the screen flash.
   *
   * @memberof Game_Screen
   */
  updateFlash(): void;

  /**
   * Update the screen shake.
   *
   * @memberof Game_Screen
   */
  updateShake(): void;

  /**
   * Update the screen zoom.
   *
   * @memberof Game_Screen
   */
  updateZoom(): void;

  /**
   * Update the screen weather.
   *
   * @memberof Game_Screen
   */
  updateWeather(): void;

  /**
   * Update the screen pictures.
   *
   * @memberof Game_Screen
   */
  updatePictures(): void;

  startFlashForDamage(): void;

  showPicture(
    pictureId: number,
    name: string,
    origin: number,
    x: number,
    y: number,
    scaleX: number,
    scaleY: number,
    opacity: number,
    blendMode: number,
  ): void;

  movePicture(
    pictureId: number,
    origin: number,
    x: number,
    y: number,
    scaleX: number,
    scaleY: number,
    opacity: number,
    blendMode: number,
    duration: number,
  ): void;

  rotatePicture(pictureId: number, speed: number): void;

  tintPicture(pictureId: number, tone: number[], duration: number): void;

  /**
   * Erases a picture from the screen given the respected picture id.
   *
   * @param {number} pictureId
   * @memberof Game_Screen
   */
  erasePicture(pictureId: number): void;
}

/**
 *  The game object class for self switches.
 */
declare class Game_SelfSwitches {
  constructor();

  /**
   * {key: Array<Any>}
   */
  _data: { [key: string]: any };

  initialize(): void;

  /**
   * Clears the array of data for the game's self switches.
   *
   * @memberof Game_SelfSwitches
   */
  clear(): void;

  /**
   * Returns the value of the switch at the current key;
   * the value is a boolean (true or false).
   * @param {Array<any>} key
   * @returns {boolean}
   * @memberof Game_SelfSwitches
   */
  value(key: any[]): boolean;

  /**
   * Sets the value of the key of the respected self switch.
   *
   * @param {Array<any>} key
   * @param {boolean} value
   * @memberof Game_SelfSwitches
   */
  setValue(key: any[], value: boolean): void;

  onChange(): void;
}

/**
 * The game object class for game switches.
 *
 * @class Game_Switches
 */
declare class Game_Switches {
  constructor();

  _data: boolean[];

  initialize(): void;

  clear(): void;

  value(switchId: number): boolean;

  setValue(switchId: number, value: boolean): void;

  onChange(): void;
}

declare namespace rm.types {
  /**
   * The data class for audio file.
   */
  export type AudioFile = {
    /**
     * The sound file name.
     */
    name: string;
    /**
     * The pan.
     */
    pan: number;
    /**
     * The sound's pitch (50..150). The default value is 100.
     */
    pitch: number;
    /**
     * The sound's volume (0..100). The default values are 100 for BGM and ME and 80 for BGS and SE.
     */
    volume: number;
  };
}

/**
 * Game_System
 * The game object class representing system data.
 * @class Game_System
 */
declare class Game_System {
  protected constructor();

  initialize(): void;

  isJapanese(): boolean;

  isChinese(): boolean;

  isKorean(): boolean;

  isCJK(): boolean;

  isRussian(): boolean;

  isSideView(): boolean;

  isAutoSaveEnabled(): boolean;

  isSaveEnabled(): boolean;

  /**
   * Disables the ability to save the game.
   *
   * @memberof Game_System
   */
  disableSave(): void;

  /**
   * Enables the ability to save the game.
   *
   * @memberof Game_System
   */
  enableSave(): void;

  /**
   * Returns true if the menu is enabled.
   *
   * @returns {Bool}
   * @memberof Game_System
   */
  isMenuEnabled(): boolean;

  /**
   * Disables the menu from being accessed.
   *
   * @memberof Game_System
   */
  disableMenu(): void;

  /**
   * Enables the menu to be accessed.
   *
   * @memberof Game_System
   */
  enableMenu(): void;

  isEncounterEnabled(): boolean;

  /**
   * Returns true if the player can encounter enemies.
   *
   * @memberof Game_System
   */
  disableEncounter(): void;

  enableEncounter(): void;

  isFormationEnabled(): boolean;

  /**
   * Disables the use of the formation command in the menu.
   *
   * @memberof Game_System
   */
  disableFormation(): void;

  /**
   * Enables the use of the formation command in the menu.
   *
   * @memberof Game_System
   */
  enableFormation(): void;

  /**
   * Returns the number of battles the player has participated in.
   *
   * @returns {number}
   * @memberof Game_System
   */
  battleCount(): number;

  /**
   * Returns the number of the wins the player has gained in battle.
   *
   * @returns {number}
   * @memberof Game_System
   */
  winCount(): number;

  /**
   * Returns the number of battles the player has escaped from in battle.
   *
   * @returns {number}
   * @memberof Game_System
   */
  escapeCount(): number;

  /**
   * Returns the number of saves the player has made in game.
   *
   * @returns {number}
   * @memberof Game_System
   */
  saveCount(): number;

  /**
   * Returns the version id represented in the database.
   *
   * @returns {number}
   * @memberof Game_System
   */
  versionId(): number;

  /**
   * Returns the current save file id
   *
   * @returns {number}
   * @memberof Game_System
   */
  savefileId(): number;

  /**
   * Sets the save file id
   *
   * @returns {Void}
   * @memberof Game_System
   */
  setSavefileId(savefileId: number): void;

  /**
   * Returns the tone of the window in the database.
   *
   * @returns {Array<Int>}
   * @memberof Game_System
   */
  windowTone(): number[];

  /**
   * Sets the window tone, given an array
   * of rgb. Example:  [0, 255, 255].
   * @param {Array<Int>} value
   * @memberof Game_System
   */
  setWindowTone(value: number[]): void;

  /**
   * Returns the current battle background music.
   *
   * @returns {AudioFile}
   * @memberof Game_System
   */
  battleBgm(): rm.types.AudioFile;

  /**
   * Sets the battle background music.
   *
   * @param {AudioFile} value
   * @memberof Game_System
   */
  setBattleBgm(value: rm.types.AudioFile): void;

  /**
   * Returns the victory musical effect.
   *
   * @returns {AudioFile}
   * @memberof Game_System
   */
  victoryMe(): rm.types.AudioFile;

  /**
   * Sets the victory musical effect.
   *
   * @param {AudioFile} value
   * @memberof Game_System
   */
  setVictoryMe(value: rm.types.AudioFile): void;

  /**
   * Returns the defeat musical effect.
   *
   * @returns {AudioFile}
   * @memberof Game_System
   */
  defeatMe(): rm.types.AudioFile;

  onBattleStart(): void;

  onBattleWin(): void;

  onBattleEscape(): void;

  onBeforeSave(): void;

  onAfterLoad(): void;

  /**
   * Returns the total play time.
   *
   * @returns {number}
   * @memberof Game_System
   */
  playtime(): number;

  playtimeText(): string;

  /**
   * Saves background music to the game system object.
   *
   * @memberof Game_System
   */
  saveBgm(): void;

  /**
   * Replays the saved background music.
   *
   * @memberof Game_System
   */
  replayBgm(): void;

  /**
   * Saves the walking background music.
   *
   * @memberof Game_System
   */
  saveWalkingBgm(): void;

  /**
   * Replays the saved walking background music.
   *
   * @memberof Game_System
   */
  replayWalkingBgm(): void;

  /**
   * Saves the second walking bgm from the map data.
   *
   * @memberof Game_System
   */
  saveWalkingBgm2(): void;

  /**
   * Returns the main font face
   *
   * @returns {String}
   * @memberof Game_System
   */
  mainFontFace(): string;

  /**
   * Returns the font face for number
   *
   * @returns {String}
   * @memberof Game_System
   */
  numberFontFace(): string;

  /**
   * Returns the main font size
   *
   * @returns {Int}
   * @memberof Game_System
   */
  mainFontSize(): number;

  /**
   * Returns the window padding
   *
   * @returns {Int}
   * @memberof Game_System
   */
  windowPadding(): number;
}

/**
 * The game object class for temporary data that is not
 * included in the save data.
 * @class Game_Temp
 */
declare class Game_Temp {
  constructor();

  _isPlaytest: boolean;
  _commonEventId: number;
  _destinationX: number;
  _destinationY: number;

  initialize(): void;

  /**
   * Determines if the game is in Play Test mode.
   *
   * @returns {boolean}
   * @memberof Game_Temp
   */
  isPlaytest(): boolean;

  /**
   * Reserves the given commonEventId to be processed later.
   *
   * @param {number} commonEventId
   * @memberof Game_Temp
   */
  reserveCommonEvent(commonEventId: number): void;

  /**
   * Clears the current reserved common event.
   *
   * @memberof Game_Temp
   */
  clearCommonEvent(): void;

  /**
   * Returns true if a common event has been reserved.
   *
   * @returns {boolean}
   * @memberof Game_Temp
   */
  isCommonEventReserved(): boolean;

  /**
   * Returns common event data from the database.
   *
   * @returns {any}
   * @memberof Game_Temp
   */
  reservedCommonEvent(): any;

  /**
   * Sets the destination of the player for movement.
   *
   * @param {number} x
   * @param {number} y
   * @memberof Game_Temp
   */
  setDestination(x: number, y: number): void;

  /**
   * Clears the destination of movement for the player.
   *
   * @memberof Game_Temp
   */
  clearDestination(): void;

  /**
   * Determines if the destination for the player to move to is valid.
   *
   * @returns {boolean}
   * @memberof Game_Temp
   */
  isDestinationValid(): boolean;

  /**
   * Returns the x coordinate of the destination.
   *
   * @returns {number}
   * @memberof Game_Temp
   */
  destinationX(): number;

  /**
   * Returns the y coordinate of the destination.
   *
   * @returns {number}
   * @memberof Game_Temp
   */
  destinationY(): number;
}

/**
 * The game object class for the timer.
 *
 * @class Game_Timer
 */
declare class Game_Timer {
  constructor();

  _frames: number;
  _working: boolean;

  initialize(): void;

  /**
   * Updates the game timer.
   *
   * @param {boolean} sceneActive
   * @memberof Game_Timer
   */
  update(sceneActive: boolean): void;

  /**
   * Starts the timer with the specified number of frames as count.
   *
   * @param {number} count
   * @memberof Game_Timer
   */
  start(count: number): void;

  /**
   * Stops the timer.
   *
   * @memberof Game_Timer
   */
  stop(): void;

  /**
   * Returns true if the timer is working and counting down.
   *
   * @returns {boolean}
   * @memberof Game_Timer
   */
  isWorking(): void;

  /**
   * Returns the number of seconds on the timer.
   *
   * @returns {number}
   * @memberof Game_Timer
   */
  seconds(): number;

  /**
   * Handler for when the time expires on the timer.
   *
   * @memberof Game_Timer
   */
  onExpire(): void;
}

declare namespace rm.types {
  export type TroopMember = {
    /**
     * The enemy ID.
     */
    enemyId: number;
    /**
     * The truth value of the [Appear Halfway] option.
     */
    hidden: boolean;
    /**
     * The troop member's x-coordinate.
     */
    x: number;
    /**
     * The troop member's y-coordinate.
     */
    y: number;
  };
}

declare namespace rm.types {
  /**
   * The data class of battle event [Conditions].
   */
  export type PageCondition = {
    /**
     * The HP percentage specified in the [Actor] condition.
     */
    actorHp: number;
    /**
     * The actor ID specified in the [Actor] condition.
     */
    actorId: number;
    /**
     * The truth value indicating whether the [Actor] condition is valid.
     */
    actorValid: boolean;
    /**
     * The HP percentage specified in the [Enemy] condition.
     */
    enemyHp: number;
    /**
     * The troop member index specified in the [Enemy] condition (0..7).
     */
    enemyIndex: number;
    /**
     * The truth value indicating whether the [Enemy] condition is valid.
     */
    enemyValid: boolean;
    /**
     * The switch ID specified in the [Switch] condition.
     */
    switchId: number;
    /**
     * The truth value indicating whether the [Switch] condition is valid.
     */
    switchValid: boolean;
    /**
     * The a and b values specified in the [Turn No.] condition. To be input in the form A + B * X.
     */
    turnA: number;
    /**
     * The a and b values specified in the [Turn No.] condition. To be input in the form A + B * X.
     */
    turnB: number;
    /**
     * The truth value indicating whether the [At End of Turn] condition is valid.
     */
    turnEnding: boolean;
    /**
     * The truth value indicating whether the [Turn No.] condition is valid.
     */
    turnValid: boolean;
  };
}

declare namespace rm.types {
  /**
   * The data class for battle events (pages).
   */
  export type Page = {
    /**
     * Condition (RPG.Troop.Page.Condition).
     */
    condition: rm.types.PageCondition;
    /**
     * Program contents. An RPG.EventCommand array.
     */
    list: rm.types.EventCommand[];
    /**
     * Span (0: battle, 1: turn, 2: moment).
     */
    span: number;
  };
}

declare namespace rm.types {
  export type Troop = {
    /**
     * The troop ID.
     */
    id: number;
    /**
     * The troop members. An RPG.Troop.Member array.
     */
    members: rm.types.TroopMember[];
    /**
     * The troop name.
     */
    name: string;
    /**
     * The battle events. An RPG.Troop.Page array.
     */
    pages: rm.types.Page[];
  };
}

/**
 * -----------------------------------------------------------------------------
 * Game_Troop
 *
 * The game object declare class for a troop and the battle-related data.
 * @class Game_Troop
 */
declare class Game_Temp extends Game_Unit {
  constructor();

  _interpreter: Game_Interpreter;
  _troopId: number;

  /**
   * Returns all enemies in the battle.
   *
   * @returns {Array<Game_Enemy>}
   * @memberof Game_Troop
   */
  members(): Game_Enemy[];

  /**
   * Returns all alive enemies.
   *
   * @returns {Array<Game_Enemy>}
   * @memberof Game_Troop
   */
  aliveMembers(): Game_Enemy[];

  /**
   * Returns all dead enemies.
   *
   * @returns {Array<Game_Enemy>}
   * @memberof Game_Troop
   */
  deadMembers(): Game_Enemy[];

  /**
   * Returns movable enemies.
   *
   * @returns {Array<Game_Enemy>}
   * @memberof Game_Troop
   */
  movableMembers(): Game_Enemy[];

  /**
   * Returns true if event is running.
   *
   * @returns {boolean}
   * @memberof Game_Troop
   */
  isEventRunning(): boolean;

  /**
   * Updates the game interpreter.
   *
   * @memberof Game_Troop
   */
  updateInterpreter(): void;

  /**
   * Returns the turn count.
   *
   * @returns {number}
   * @memberof Game_Troop
   */
  turnCount(): number;

  clear(): void;

  /**
   * Returns troop information from the database.
   *
   * @returns {RPG.Troop}
   * @memberof Game_Troop
   */
  troop(): rm.types.Troop;

  setup(troopId: number): void;

  /**
   * Creates unique names for each enemy.
   *
   * @memberof Game_Troop
   */
  makeUniqueNames(): void;

  /**
   * Returns the letter table for enemy troops.
   *
   * @returns {Array<string>}
   * @memberof Game_Troop
   */
  letterTable(): string[];

  /**
   * Returns the name of enemies within the troop.
   *
   * @returns {Array<string>}
   * @memberof Game_Troop
   */
  enemyNames(): string[];

  meetsConditions(page: rm.types.EventPage): boolean;

  setupBattleEvent(): void;

  /**
   * Increases the turn number.
   *
   * @memberof Game_Troop
   */
  increaseTurn(): void;

  /**
   * Returns the total exp of all members of the enemy troop.
   *
   * @returns {number}
   * @memberof Game_Troop
   */
  expTotal(): number;

  /**
   * Return the total gold of all enemies.
   *
   * @returns {number}
   * @memberof Game_Troop
   */
  goldTotal(): number;

  /**
   * Returns the gold rate based on game party gold rate.
   *
   * @returns {number}
   * @memberof Game_Troop
   */
  goldRate(): number;

  /**
   * Creates the drop items for all members of the enemy troop, and
   * returns the item information.
   * @returns {Array<RPG.BaseItem>}
   * @memberof Game_Troop
   */
  makeDropItems(): rm.types.BaseItem[];

  static LETTER_TABLE_HALF: string[];
  static LETTER_TABLE_FULL: string[];
}

/**
 * The game object class for game variables.
 */
declare class Game_Variables {
  constructor();

  _data: number[];

  initialize(): void;

  clear(): void;

  value(variableId: number): number;

  setValue(variableId: number, value: number): void;

  onChange(): void;
}

declare namespace rm.types {
  /**
   * The data class for vehicles.
   */
  export type SystemVehicle = {
    /**
     * The vehicle's BGM (RPG.AudioFile).
     */
    bgm: rm.types.AudioFile;
    /**
     * The index of the vehicle's walking graphic (0..7).
     */
    characterIndex: number;
    /**
     * The file name of the vehicle's walking graphic.
     */
    characterName: string;
    /**
     * The map ID of the vehicle's initial position.
     */
    startMapId: number;
    /**
     * The map's x-coordinate of the vehicle's initial position.
     */
    startX: number;
    /**
     * The map's y-coordinate of the vehicle's initial position.
     */
    startY: number;
  };
}

/**
 * The game object class for a vehicle.
 */
declare class Game_Vehicle extends Game_Character {
  constructor(type: string);

  _type: string;
  _mapId: number;
  _altitude: number;
  _driving: boolean;
  _bgm: rm.types.AudioFile;

  initMembers(): void;

  /**
   * Returns true if the vehicle is a boat.
   *
   * @returns {Bool}
   * @memberof Game_Vehicle
   */
  isBoat(): boolean;

  /**
   * Returns true if the vehicle is a ship.
   *
   * @returns {Bool}
   * @memberof Game_Vehicle
   */
  isShip(): boolean;

  /**
   * Returns true if the vehicle is an airship.
   *
   * @returns {Bool}
   * @memberof Game_Vehicle
   */
  isAirship(): boolean;

  /**
   * Resets the direction of the vehicle.
   *
   * @memberof Game_Vehicle
   */
  resetDirection(): void;

  initMoveSpeed(): void;

  /**
   * Returns the current vehicle.
   *
   * @returns {RPG.System.Vehicle}
   * @memberof Game_Vehicle
   */
  vehicle(): rm.types.SystemVehicle;

  loadSystemSettings(): void;

  /**
   * Refreshes the game vehicle.
   *
   * @memberof Game_Vehicle
   */
  refresh(): void;

  setLocation(mapId: number, x: number, y: number): void;

  pos(x: number, y: number): boolean;

  isMapPassable(x: number, y: number, d: rm.types.Direction): boolean;

  /**
   * Gets on the vehicle.
   *
   * @memberof Game_Vehicle
   */
  getOn(): void;

  /**
   * Gets off the vehicle.
   *
   * @memberof Game_Vehicle
   */
  getOff(): void;

  /**
   * Sets the bgm associated with the vehicle.
   *
   * @param {RPG.AudioFile} bgm
   * @memberof Game_Vehicle
   */
  setBgm(bgm: rm.types.AudioFile): void;

  /**
   * Plays the bgm associated with the vehicle.
   *
   * @memberof Game_Vehicle
   */
  playBgm(): void;

  /**
   * Syncs the vehicle with the player.
   *
   * @memberof Game_Vehicle
   */
  syncWithPlayer(): void;

  /**
   * Returns the screen y position of the vehicle.
   *
   * @returns {number}
   * @memberof Game_Vehicle
   */
  screenY(): number;

  /**
   * Returns the shadow x position of the vehicle's shadow.
   *
   * @returns {number}
   * @memberof Game_Vehicle
   */
  shadowX(): number;

  /**
   * Returns the shadow y position of the vehicle's shadow.
   *
   * @returns {number}
   * @memberof Game_Vehicle
   */
  shadowY(): number;

  /**
   * Returns the shadow opacity of the vehicle's shadow.
   *
   * @returns {number}
   * @memberof Game_Vehicle
   */
  shadowOpacity(): number;

  /**
   * Returns true if the vehicle can move.
   *
   * @returns {Bool}
   * @memberof Game_Vehicle
   */
  canMove(): boolean;

  /**
   * Updates the vehicle.
   *
   * @memberof Game_Vehicle
   */
  update(): void;

  /**
   * Updates the airship.
   *
   * @memberof Game_Vehicle
   */
  updateAirship(): void;

  /**
   * Updates airship altitude.
   *
   * @memberof Game_Vehicle
   */
  updateAirshipAltitude(): void;

  /**
   * Returns the max altitude of the vehicle.
   *
   * @returns {number}
   * @memberof Game_Vehicle
   */
  maxAltitude(): number;

  /**
   * Returns true if the vehicle is at it's lowest altitude.
   *
   * @returns {Bool}
   * @memberof Game_Vehicle
   */
  isLowest(): boolean;

  /**
   * Returns true if the vehicle is at it's highest altitude.
   *
   * @returns {Bool}
   * @memberof Game_Vehicle
   */
  isHighest(): boolean;

  /**
   * Returns true if take off is ok.
   *
   * @returns {Bool}
   * @memberof Game_Vehicle
   */
  isTakeoffOk(): boolean;

  /**
   * Returns true if the vehicle is capable of landing at the specified
   * x, y coordinate, with the specified direction (d).
   * @param {number} x
   * @param {number} y
   * @param {number} d
   * @returns {Bool}
   * @memberof Game_Vehicle
   */
  isLandOk(x: number, y: number, d: rm.types.Direction): boolean;
}

declare class Scene_Base extends Stage {
  constructor();

  _windowLayer: WindowLayer;

  /**
   * Initializes the scene.
   * @return Void
   */
  initialize(): void;

  /**
   * Creates the scene's important properties.
   */
  create(): void;

  /**
   * Returns whether the scene is active or not.
   * @return Bool
   */
  isActive(): boolean;

  /**
   * Returns if the scene is ready or not.
   * @return Bool
   */
  isReady(): boolean;

  /**
   * Starts the scene.
   */
  start(): void;

  /**
   * Updates the scene.
   */
  update(): void;

  /**
   * Stops the scene.
   */
  stop(): void;

  /**
   * Checks if the Scene is busy processing an event
   * or other conditions.
   * @return Bool
   */
  isBusy(): boolean;

  isStarted(): boolean;

  isFading(): boolean;

  createColorFilter(): void;

  updateColorFilter(): void;

  scaleSprite(): void;

  centerSprite(): void;

  isBottomHelpMode(): boolean;

  isBottomButtonMode(): boolean;

  isRightInputMode(): boolean;

  mainCommandWidth(): number;

  buttonAreaTop(): number;

  buttonAreaBottom(): number;

  buttonAreaHeight(): number;

  buttonY(): number;

  calcWindowHeight(): number;

  requestAutosave(): void;

  isAutosaveEnabled(): boolean;

  executeAutosave(): void;

  onAutosaveSuccess(): void;

  onAutosaveFailure(): void;

  /**
   * Terminates/ends the scene.
   */
  terminate(): void;

  /**
   * Creates the window layer on the current scene
   * for displaying any and all windows.
   */
  createWindowLayer(): void;

  /**
   * Adds a child window to the window layer for processing.
   * @param window
   */
  addWindow(window: Window_Base): void;

  /**
   * Request a fadeIn screen process
   * @param duration  [duration=30] The time the process will take to fadeIn the screen.
   * @param white  [white=false] If true the fadeIn will process with a white color else it will be black.
   */
  startFadeIn(duration: number, white: boolean): void;

  /**
   * Request a fadeOut screen process
   * @param duration  [duration=30] The time the process will take to fadeOut the screen.
   * @param white  [white=false] If true the fadeOut will process with a white color else it will be black.
   */
  startFadeOut(duration: number, white: boolean): void;

  createFadeSprite(white: boolean): void;

  /**
   * Updates the scene's fade
   */
  updateFade(): void;

  updateChildren(): void;

  /**
   * Pops the current scene and returns to the previous scene if available.
   */
  popScene(): void;

  /**
   * Checks for game over.
   */
  checkGameOver(): void;

  fadeOutAll(): void;

  /**
   * Returns the fade speed.
   * @return Int
   */
  fadeSpeed(): number;

  /**
   * Returns the slow fade speed.
   * @return Int
   */
  slowFadeSpeed(): number;
}

declare class Scene_Message extends Scene_Base {
  protected constructor();

  isMessageWindowClosing: boolean;
  _choiceListWindow: Window_ChoiceList;
  _eventItemWindow: Window_EventItem;
  _nameBoxWindow: Window_NameBox;
  _numberInputWindow: Window_NumberInput;

  createAllWindows(): void;

  createMessageWindow(): void;

  messageWindowRect(): void;

  createScrollTextWindow(): void;

  scrollTextWindowRect(): void;

  createGoldWindow(): void;

  goldWindowRect(): void;

  createNameBoxWindow(): void;

  createChoiceListWindow(): void;

  createNumberInputWindow(): void;

  createEventItemWindow(): void;

  eventItemWindowRect(): void;

  associateWindows(): void;
}

/**
 * Scene class of the battle screen.
 */
declare class Scene_Battle extends Scene_Message {
  protected constructor();

  updateBattleProcess(): void;

  isAnyInputWindowActive(): boolean;

  changeInputWindow(): void;

  updateVisibility(): void;

  updateLogWindowVisibility(): void;

  updateStatusWindowVisibility(): void;

  updateInputWindowVisibility(): void;

  needsInputWindowChange(): boolean;

  isTimeActive(): boolean;

  shouldAutosave(): boolean;

  updateStatusWindowPosition(): void;

  statusWindowX(): number;

  shouldOpenStatusWindow(): boolean;

  logWindowRect(): Rectangle;

  statusWindowRect(): Rectangle;

  partyCommandWindowRect(): Rectangle;

  actorCommandWindowRect(): Rectangle;

  helpWindowRect(): Rectangle;

  skillWindowRect(): Rectangle;

  itemWindowRect(): Rectangle;

  actorWindowRect(): Rectangle;

  enemyWindowRect(): Rectangle;

  helpAreaTop(): number;

  helpAreaBottom(): number;

  helpAreaHeight(): number;

  mainAreaTop(): number;

  mainAreaBottom(): number;

  mainAreaHeight(): number;

  buttonAreaTop(): number;

  windowAreaHeight(): number;

  createButtons(): void;

  needsCancelButton(): void;

  createCancelButton(): void;

  needsPageButtons(): void;

  createPageButtons(): void;

  updatePageButtons(): void;

  arePageButtonsEnabled(): boolean;

  closeCommandWindows(): void;

  hideSubInputWindows(): void;

  startEnemySelection(): void;

  /**
   * Stops the battle scene.
   *
   * @memberof Scene_Battle
   */
  stop(): void;

  /**
   * Returns true if the battle needs a slow fade out.
   *
   * @returns {boolean}
   * @memberof Scene_Battle
   */
  needsSlowFadeOut(): boolean;

  /**
   * Updates the status window on the battle scene.
   *
   * @memberof Scene_Battle
   */
  updateStatusWindow(): void;

  /**
   * Updates the position of the battle scene windows.
   *
   * @memberof Scene_Battle
   */
  updateWindowPositions(): void;

  /**
   * Creates all the display objects including:
   * the spritesheet, window layer, windows, and more.
   *
   * @memberof Scene_Battle
   */
  createDisplayObjects(): void;

  /**
   * Creates the spriteset within
   * the battle scene. This includes
   * sprites for actors, enemies, etc.
   * @memberof Scene_Battle
   */
  createSpriteset(): void;

  /**
   * Creates all the windows within the
   * battle scene.
   * @memberof Scene_Battle
   */
  createAllWindows(): void;

  /**
   * Creates the log window.
   *
   * @memberof Scene_Battle
   */
  createLogWindow(): void;

  createStatusWindow(): void;

  createPartyCommandWindow(): void;

  /**
   * Creates the actor command window.
   *
   * @memberof Scene_Battle
   */
  createActorCommandWindow(): void;

  /**
   * Creates the help window.
   *
   * @memberof Scene_Battle
   */
  createHelpWindow(): void;

  /**
   * Creates the skill window.
   *
   * @memberof Scene_Battle
   */
  createSkillWindow(): void;

  /**
   * Creates the item window.
   *
   * @memberof Scene_Battle
   */
  createItemWindow(): void;

  /**
   * Creates the actor window.
   *
   * @memberof Scene_Battle
   */
  createActorWindow(): void;

  /**
   * Creates the enemy window.
   *
   * @memberof Scene_Battle
   */
  createEnemyWindow(): void;

  /**
   * Creates the message window on the battle scene.
   *
   * @memberof Scene_Battle
   */
  createMessageWindow(): void;

  /**
   * Creates the scroll text window.
   *
   * @memberof Scene_Battle
   */
  createScrollTextWindow(): void;

  refreshStatus(): void;

  startPartyCommandSelection(): void;

  /**
   * Handler for the fight command on battle start..
   *
   * @memberof Scene_Battle
   */
  commandFight(): void;

  /**
   * Handler for the escape command on battle start.
   *
   * @memberof Scene_Battle
   */
  commandEscape(): void;

  startActorCommandSelection(): void;

  /**
   * Handler for the attack command.
   *
   * @memberof Scene_Battle
   */
  commandAttack(): void;

  /**
   *
   * Handler for the skill command.
   *
   * @memberof Scene_Battle
   */
  commandSkill(): void;

  /**
   * Handler for the guard command.
   *
   * @memberof Scene_Battle
   */
  commandGuard(): void;

  /**
   * Handler for the item command.
   *
   * @memberof Scene_Battle
   */
  commandItem(): void;

  /**
   * Selects the next command in the battle scene.
   *
   * @memberof Scene_Battle
   */
  selectNextCommand(): void;

  /**
   * Selects the previous command in the battle scene.
   *
   * @memberof Scene_Battle
   */
  selectPreviousCommand(): void;

  selectActorSelection(): void;

  onActorOk(): void;

  onActorCancel(): void;

  selectEnemySelection(): void;

  /**
   * Handler for when an enemy is selected.
   *
   * @memberof Scene_Battle
   */
  onEnemyOk(): void;

  onEnemyCancel(): void;

  /**
   * Handler for when a skill is selected.
   *
   * @memberof Scene_Battle
   */
  onSkillOk(): void;

  onSkillCancel(): void;

  onItemOk(): void;

  onItemCancel(): void;

  onSelectAction(): void;

  endCommandSelection(): void;
}

declare class Scene_Boot extends Scene_Base {
  protected constructor();

  _databaseLoaded: boolean;

  onDatabaseLoaded(): void;

  setEncryptionInfo(): void;

  loadPlayerData(): void;

  loadGameFonts(): void;

  isPlayerDataLoaded(): void;

  startNormalGame(): void;

  resizeScreen(): void;

  adjustBoxSize(): void;

  adjustWindow(): void;

  /**
   * Loads the system images upon booting the game.
   */
  loadSystemImages(): void;

  /**
   * Loads the system window image for showing all windows in game.
   */
  loadSystemWindowImage(): void;

  /**
   * Returns true if the game font is loaded.
   * @return Bool
   */
  isGameFontLoaded(): boolean;

  updateDocumentTitle(): void;

  /**
   * Checks the player location upon booting the game.
   */
  checkPlayerLocation(): void;
}

/**
 * The super class of all menu-type scenes
 */
declare class Scene_MenuBase extends Scene_Base {
  protected constructor();

  /**
   * Returns the current game actor.
   *
   * @returns {Game_Actor}
   * @memberof Scene_MenuBase
   */
  actor(): Game_Actor;

  _cancelButton: Sprite_Button;
  _pageupButton: Sprite_Button;
  _pagedownButton: Sprite_Button;

  helpAreaTop(): void;

  helpAreaBottom(): void;

  helpAreaHeight(): void;

  mainAreaTop(): void;

  mainAreaBottom(): void;

  mainAreaHeight(): void;

  helpWindowRect(): Rectangle;

  createButtons(): void;

  needsCancelButton(): boolean;

  createCancelButton(): void;

  needsPageButtons(): boolean;

  createPageButtons(): void;

  updatePageButtons(): void;

  arePageButtonsEnabled(): boolean;

  /**
   * Updates the current actor.
   *
   * @memberof Scene_MenuBase
   */
  updateActor(): void;

  /**
   * Creates the background of the base menu scene.
   *
   * @memberof Scene_MenuBase
   */
  createBackground(): void;

  /**
   * Sets the background opacity of the base menu scene background.
   *
   * @param {number} opacity
   * @memberof Scene_MenuBase
   */
  setBackgroundOpacity(opacity: number): void;

  /**
   * Creates the help window.
   *
   * @memberof Scene_MenuBase
   */
  createHelpWindow(): void;

  /**
   * Moves to the next actor in the party.
   *
   * @memberof Scene_MenuBase
   */
  nextActor(): void;

  /**
   * Moves to the previous actor in the party.
   *
   * @memberof Scene_MenuBase
   */
  previousActor(): void;

  onActorChange(): void;
}

/**
 * Scene class of the debug menu screen.
 */
declare class Scene_Debug extends Scene_MenuBase {
  protected constructor();

  /**
   * Creates the range window.
   *
   * @memberof Scene_Debug
   */
  createRangeWindow(): void;

  rangeWindowRect(): Rectangle;

  editWindowRect(): Rectangle;

  debugHelpWindowRect(): Rectangle;

  /**
   * Creates the edit window.
   *
   * @memberof Scene_Debug
   */
  createEditWindow(): void;

  /**
   * Creates the debug help .window.
   *
   * @memberof Scene_Debug
   */
  createDebugHelpWindow(): void;

  onRangeOk(): void;

  onEditCancel(): void;

  refreshHelpWindow(): void;

  /**
   * Returns the help text within the
   * debug scene.
   * @returns {String}
   * @memberof Scene_Debug
   */
  helpText(): string;
}

/**
 * Scene class of the equipment screen.
 */
declare class Scene_Equip extends Scene_MenuBase {
  protected constructor();

  statusWindowRect(): Rectangle;

  commandWindowRect(): Rectangle;

  slotWindowRect(): Rectangle;

  itemWindowRect(): Rectangle;

  executeEquipChange(): void;

  /**
   * Creates the status window within the equip scene.
   *
   * @memberof Scene_Equip
   */
  createStatusWindow(): void;

  /**
   * Creates the status window within the equip scene.
   *
   * @memberof Scene_Equip
   */
  createCommandWindow(): void;

  /**
   * Creates the slot window within the  equip scene.
   *
   * @memberof Scene_Equip
   */
  createSlotWindow(): void;

  /**
   * Creates the item window within the equip scene.
   *
   * @memberof Scene_Equip
   */
  createItemWindow(): void;

  /**
   * Refreshes the actor within the equip scene.
   *
   * @memberof Scene_Equip
   */
  refreshActor(): void;

  /**
   * Handler for the equip command.
   *
   * @memberof Scene_Equip
   */
  commandEquip(): void;

  /**
   * Handler for the optimize command.
   *
   * @memberof Scene_Equip
   */
  commandOptimize(): void;

  commandClear(): void;

  onSlotOk(): void;

  onSlotCancel(): void;

  onItemOk(): void;

  onItemCancel(): void;

  onActorChange(): void;
}

/**
 *  Super class of Scene_Save and Scene_Load
 */
declare class Scene_File extends Scene_MenuBase {
  protected constructor();

  isSavefileEnabled(): boolean;

  listWindowRect(): Rectangle;

  needsAutosave(): boolean;

  firstSavefileId(): number;

  /**
   * Returns the current savefileId.
   *
   * @memberof Scene_File
   */
  savefileId(): number;

  createHelpWindow(): void;

  createListWindow(): void;

  /**
   * The current mode of the scene;
   * the modes are 'save' or 'load'.
   * @memberof Scene_File
   */
  mode(): void;

  activateListWindow(): void;

  helpWindowText(): string;

  /**
   * Handler for when a
   * save file is selected within the file scene.
   * @memberof Scene_File
   */
  onSavefileOk(): void;
}

/**
 * Scene class of the game end screen.
 */
declare class Scene_GameEnd extends Scene_MenuBase {
  protected constructor();

  commandWindowRect(): Rectangle;

  stop(): void;

  /**
   * Creates the background for
   * the game end scene.
   * @memberof Scene_GameEnd
   */
  createBackground(): void;

  /**
   * Creates the command window
   * for the game end screen.
   * @memberof Scene_GameEnd
   */
  createCommandWindow(): void;

  /**
   * Handler for when to title
   * is clicked within the game end screen.
   * @memberof Scene_GameEnd
   */
  commandToTitle(): void;
}

/**
 * Scene class of the game over screen.
 */
declare class Scene_Gameover extends Scene_Base {
  protected constructor();

  adjustBackground(): void;

  stop(): void;

  /**
   * Plays the game over music
   * within the game over scene.
   * @memberof Scene_Gameover
   */
  playGameoverMusic(): void;

  /**
   * Creates the background of
   * the game over scene.
   * @memberof Scene_Gameover
   */
  createBackground(): void;

  isTriggered(): boolean;

  /**
   * Returns to the title scene (Scene_Title).
   * @memberof Scene_Gameover
   */
  gotoTitle(): void;
}

/**
 * Super class of all public function item(skill, items, etc) screen scenes.
 * Super class of Scene_Item & Scene_Skill.
 */
declare class Scene_ItemBase extends Scene_Base {
  protected constructor();

  actorWindowRect(): Rectangle;

  /**
   * Creates the actor window within the base item scene.
   *
   * @memberof Scene_ItemBase
   */
  createActorWindow(): void;

  /**
   * Returns the item attached to the item window within the base item scene.
   *
   * @returns {*}
   * @memberof Scene_ItemBase
   */
  item(): any;

  /**
   * Returns the current game actor.
   *
   * @returns {Game_Actor}
   * @memberof Scene_ItemBase
   */
  user(): Game_Actor;

  /**
   * Returns true if the cursor is left.
   *
   * @returns {boolean}
   * @memberof Scene_ItemBase
   */
  isCursorLeft(): boolean;

  /**
   * Shows the sub window.
   *
   * @param {Window_Base} window
   * @memberof Scene_ItemBase
   */
  showSubWindow(window: Window_Base): void;

  /**
   * Hides the sub window.
   *
   * @param {Window_Base} window
   * @memberof Scene_ItemBase
   */
  hideSubWindow(window: Window_Base): void;

  onActorOk(): void;

  onActorCancel(): void;

  determineItem(): void;

  /**
   * Uses the current item.
   *
   * @memberof Scene_ItemBase
   */
  useItem(): void;

  /**
   * Activates the item window.
   *
   * @memberof Scene_ItemBase
   */
  activateItemWindow(): void;

  itemTargetActors(): Game_Actor;

  /**
   * Returns true if the user (game actor) can use the item.
   *
   * @returns {boolean}
   * @memberof Scene_ItemBase
   */
  canUse(): boolean;

  /**
   * Returns true if the item effects are valid on the public function target(the user).
   *
   * @returns {boolean}
   * @memberof Scene_ItemBase
   */
  isItemEffectsValid(): boolean;

  applyItem(): void;

  /**
   * Checks the common event set on the item.
   *
   * @memberof Scene_ItemBase
   */
  checkCommonEvent(): void;
}

/**
 * -----------------------------------------------------------------------------
 * Scene_Item
 *
 * The scene class of the item screen.
 * @class Scene_Item
 * @extends {Scene_ItemBase}
 */
declare class Scene_Item extends Scene_ItemBase {
  protected constructor();

  categoryWindowRect(): Rectangle;

  itemWindowRect(): Rectangle;

  /**
   * Creates the category window on the
   * item scene.
   * @memberof Scene_Item
   */
  createCategoryWindow(): void;

  /**
   * Creates the item window on the item
   * scene.
   * @memberof Scene_Item
   */
  createItemWindow(): void;

  /**
   * Handler for when a cataegory is selected
   * on the item scene.
   * @memberof Scene_Item
   */
  onCategoryOk(): void;

  /**
   * Handler for when an item is selected
   * on the item scene.
   * @memberof Scene_Item
   */
  onItemOk(): void;

  /**
   * Handler for when an item selection
   * is canceled on the item scene.
   * @memberof Scene_Item
   */
  onItemCancel(): void;

  /**
   * Plays a sound effect when the
   * item is confirmed.
   * @memberof Scene_Item
   */
  playSeForItem(): void;
}

/**
 * The scene class of the load game screen.
 */
declare class Scene_Load extends Scene_File {
  protected constructor();

  /**
   * Returns the mode of the
   * load scene.
   * @returns {string}
   * @memberof Scene_Load
   */
  mode(): string;

  /**
   * Returns the help window text on the
   * game load scene.
   * @returns {string}
   * @memberof Scene_Load
   */
  helpWindowText(): string;

  executeLoad(saveFileId: number): void;

  firstSavefileIndex(): number;

  onSavefileOk(): void;

  onLoadSuccess(): void;

  onLoadFailure(): void;

  reloadMapIfUpdated(): void;
}

/**
 * The scene class for the map screen.
 */
declare class Scene_Map extends Scene_Base {
  protected constructor();

  _lastMapWasNull: boolean;

  shouldAutosave(): boolean;

  onTransferEnd(): void;

  isPlayerActive(): boolean;

  updateMenuButton(): void;

  hideMenuButton(): void;

  updateMapNameWindow(): void;

  isAnyButtonPressed(): boolean;

  onMapTouch(): void;

  mapNameWindowRect(): Rectangle;

  createButtons(): void;

  createMenuButton(): void;

  menuCalling: boolean;

  /**
   * Handler for when the map scene is loaded.
   *
   * @memberof Scene_Map
   */
  onMapLoaded(): void;

  /**
   * Speeds up the updateMain if
   * the map scene is in fast forward mode.
   * @memberof Scene_Map
   */
  updateMainMultiply(): void;

  /**
   * Updates the main $game globals
   * if the map scene is active.
   * @memberof Scene_Map
   */
  updateMain(): void;

  /**
   * Returns true if the player
   * is holding down the confirm button to
   * fast forward through text.
   * @returns {boolean}
   * @memberof Scene_Map
   */
  isFastForward(): boolean;

  /**
   * Stops the map scene and prepares
   * for a new scene.
   * @memberof Scene_Map
   */
  stop(): void;

  /**
   * Returns true if the map scene needs a slow fade in.
   *
   * @returns {boolean}
   * @memberof Scene_Map
   */
  needsFadeIn(): boolean;

  /**
   * Returns true if the map scene needs a slow fade out.
   *
   * @returns {boolean}
   * @memberof Scene_Map
   */
  needsSlowFadeOut(): boolean;

  updateWaitCount(): boolean;

  /**
   * Constantly checks if the player
   * is touching the map, then processes
   * a map touch for mouse based player character movement.
   * @memberof Scene_Map
   */
  updateDestination(): void;

  /**
   * Returns true if the map scene is
   * active and the player can move. Used for
   * mouse movement on the map scene.
   * @returns {boolean}
   * @memberof Scene_Map
   */
  isMapTouchOk(): boolean;

  /**
   * Processes the map touch and turns it
   * into coordinates for the player character to move to.
   * @memberof Scene_Map
   */
  processMapTouch(): void;

  isSceneChangeOk(): boolean;

  /**
   * Updates the scene.
   *
   * @memberof Scene_Map
   */
  updateScene(): void;

  /**
   * Creates all the display objects on the map scene;
   * this includes the sprites, window layer, windows, and more.
   * @memberof Scene_Map
   */
  createDisplayObjects(): void;

  /**
   * Creates the spriteset on the map scene;
   * this shows all the characters and events on the map.
   * @memberof Scene_Map
   */
  createSpriteset(): void;

  /**
   * Creates all the windows on the map scene
   * contains other window creation methods.
   * @memberof Scene_Map
   */
  createAllWindows(): void;

  /**
   * Creates the map name window within
   * the map scene; display map name.
   * @memberof Scene_Map
   */
  createMapNameWindow(): void;

  /**
   * Creates the message window for displaying
   * text on the map scene. Commonly used with the
   * showText command.
   * @memberof Scene_Map
   */
  createMessageWindow(): void;

  /**
   * Creates a scrolling text window on the map scene.
   *
   * @memberof Scene_Map
   */
  createScrollTextWindow(): void;

  /**
   * Checks if the player is transferring;
   * if the player is transferring, move to a new map scene.
   * @memberof Scene_Map
   */
  updateTransferPlayer(): void;

  /**
   * Processes starting an encounter on the map scene.
   *
   * @memberof Scene_Map
   */
  updateEncounter(): void;

  /**
   * Processes calling the menu on the map scene.
   *
   * @memberof Scene_Map
   */
  updateCallMenu(): void;

  /**
   * Returns true if the menu is enabled
   * in the database.
   * @returns {boolean}
   * @memberof Scene_Map
   */
  isMenuEnabled(): boolean;

  /**
   * Returns true if the menu is called
   * via an input trigger.
   * @returns {boolean}
   * @memberof Scene_Map
   */
  isMenuCalled(): boolean;

  /**
   * Calls the standard RPGMakerMV menu scene.
   *
   * @memberof Scene_Map
   */
  callMenu(): void;

  /**
   * Checks if debug is called via input trigger
   * and starts the debug scene.
   *
   * @memberof Scene_Map
   */
  updateCallDebug(): void;

  /**
   * Returns true if the debug
   * scene is called via button press and in play test mode.
   * @returns {boolean}
   * @memberof Scene_Map
   */
  isDebugCalled(): void;

  fadeInForTransfer(): void;

  fadeOutForTransfer(): void;

  /**
   * Launches into the battle scene.
   *
   * @memberof Scene_Map
   */
  launchBattle(): void;

  /**
   * Stops all audio on battle start.
   *
   * @memberof Scene_Map
   */
  stopAudioOnBattleStart(): void;

  /**
   * Starts the encounter effect on the map scene.
   *
   * @memberof Scene_Map
   */
  startEncounterEffect(): void;

  updateEncounterEffect(): void;

  /**
   * Takes a snapshot of the map scene for displaying
   * on the battle scene if no battleback is present.
   * @memberof Scene_Map
   */
  snapForBattleBackground(): void;

  /**
   * Starts a flash encounter effect on the map scene
   * given a duration of the flash.
   * @param {number} duration
   * @memberof Scene_Map
   */
  startFlashForEncounter(duration: number): void;

  /**
   * Returns the speed of the encounter effect.
   *
   * @returns {number}
   * @memberof Scene_Map
   */
  encounterEffectSpeed(): number;
}

/**
 * The menu scene in your RPGMakerMV game.
 */
declare class Scene_Menu extends Scene_MenuBase {
  protected constructor();

  commandWindowRect(): Rectangle;

  goldWindowRect(): Rectangle;

  /**
   * Creates the main menu window on the
   * menu scene; contains the commands for
   * menu usage.
   * @memberof Scene_Menu
   */
  createCommandWindow(): void;

  /**
   * Creates the gold window on the menu scene.
   *
   * @memberof Scene_Menu
   */
  createGoldWindow(): void;

  /**
   * Creates the status window on the menu scene.
   *
   * @memberof Scene_Menu
   */
  createStatusWindow(): void;

  /**
   * Handler for what to do when the 'item'
   * command is clicked.
   * @memberof Scene_Menu
   */
  commandItem(): void;

  commandPersonal(): void;

  commandFormation(): void;

  /**
   * Handler for what to do when the 'option'
   * command is clicked.
   * @memberof Scene_Menu
   */
  commandOptions(): void;

  /**
   * Handler for what to do when the 'save'
   * command is clicked.
   * @memberof Scene_Menu
   */
  commandSave(): void;

  /**
   * Handler for what to do when the 'game end'
   * command is clicked.
   * @memberof Scene_Menu
   */
  commandGameEnd(): void;

  onPersonalOk(): void;

  onPersonalCancel(): void;

  onFormationOk(): void;

  onFormationCancel(): void;
}

/**
 * Scene class of the name input screen.
 */
declare class Scene_Name extends Scene_MenuBase {
  protected constructor();

  editWindowRect(): void;

  inputWindowRect(): Rectangle;

  /**
   * Prepares the name input scene for giving the specified
   * actor at actorId a name with a maximum number of characters
   * given by maxLength.
   * @param {number} actorId
   * @param {number} maxLength
   * @memberof Scene_Name
   */
  prepare(actorId: number, maxLength: number): void;

  /**
   * Creates the edit window.
   *
   * @memberof Scene_Name
   */
  createEditWindow(): void;

  /**
   * Creates the input window.
   *
   * @memberof Scene_Name
   */
  createInputWindow(): void;

  /**
   * Handler for when ok is processed on the name input scene.
   *
   * @memberof Scene_Name
   */
  onInputOk(): void;
}

/**
 * Scene class of the options screen.
 */
declare class Scene_Options extends Scene_MenuBase {
  protected constructor();

  createOptionsWindow(): void;

  optionsWindowRect(): Rectangle;

  maxCommands(): number;

  maxVisibleCommands(): number;
}

/**
 * Save game screen scene for RPGMakerMV.
 */
declare class Scene_Save extends Scene_File {
  protected constructor();

  /**
   * Returns the mode of the
   * save scene.
   * @returns {string}
   * @memberof Scene_Save
   */
  mode(): string;

  helpWindowText(): string;

  executeSave(saveFileId: number): void;

  /**
   * Returns the index of the first
   * save file within the save scene.
   * @returns {number}
   * @memberof Scene_Save
   */
  firstSavefileIndex(): number;

  /**
   * Handler for when a save file
   * is confirmed within the save scene.
   * @memberof Scene_Save
   */
  onSavefileOk(): void;

  /**
   * Handler for when save is a success.
   *
   * @memberof Scene_Save
   */
  onSaveSuccess(): void;

  /**
   * Handler for when save fails.
   *
   * @memberof Scene_Save
   */
  onSaveFailure(): void;
}

declare namespace rm.types {
  export const enum Money {
    base = -1,
  }
}

/**
 * Scene class of the shop screen.
 */
declare class Scene_Shop extends Scene_MenuBase {
  protected constructor();

  commandWindowRect(): Rectangle;

  dummyWindowRect(): Rectangle;

  numberWindowRect(): Rectangle;

  statusWindowRect(): Rectangle;

  buyWindowRect(): Rectangle;

  categoryWindowRect(): Rectangle;

  sellWindowRect(): Rectangle;

  prepare(goods: any[][], purchaseOnly: boolean): void;

  /**
   * Creates the gold window on the shop scene.
   *
   * @memberof Scene_Shop
   */
  createGoldWindow(): void;

  /**
   * Creates the command window for buying or selling.
   *
   * @memberof Scene_Shop
   */
  createCommandWindow(): void;

  createDummyWindow(): void;

  /**
   * Creates the number input window on the shop scene.
   *
   * @memberof Scene_Shop
   */
  createNumberWindow(): void;

  /**
   * Creates the status window.
   *
   * @memberof Scene_Shop
   */
  createStatusWindow(): void;

  /**
   * Creates the buy window.
   *
   * @memberof Scene_Shop
   */
  createBuyWindow(): void;

  /**
   * Creates the category window.
   *
   * @memberof Scene_Shop
   */
  createCategoryWindow(): void;

  /**
   * Creates the sell window.
   *
   * @memberof Scene_Shop
   */
  createSellWindow(): void;

  /**
   * Activates the buy window within the shop scene.
   *
   * @memberof Scene_Shop
   */
  activateBuyWindow(): void;

  /**
   * Activates the sell window within the shop scene.
   *
   * @memberof Scene_Shop
   */
  activateSellWindow(): void;

  /**
   * Handler for pressing buy within the shop scene.
   *
   * @memberof Scene_Shop
   */
  commandBuy(): void;

  /**
   * Handler for pressing sell within the shop scene.
   *
   * @memberof Scene_Shop
   */
  commandSell(): void;

  /**
   * Handler for when buying is confirmed.
   *
   * @memberof Scene_Shop
   */
  onBuyOk(): void;

  /**
   * Handler for when buying is cancelled.
   *
   * @memberof Scene_Shop
   */
  onBuyCancel(): void;

  onCategoryOk(): void;

  onCategoryCancel(): void;

  onSellOk(): void;

  onSellCancel(): void;

  onNumberOk(): void;

  onNumberCancel(): void;

  /**
   * Amount of item to buy.
   * @param number
   */
  doBuy(number: number): void;

  /**
   * Amount of item to sell.
   * @param number
   */
  doSell(number: number): void;

  endNumberInput(): void;

  /**
   * Returns the maximum number bought.
   *
   * @returns {number}
   * @memberof Scene_Shop
   */
  maxBuy(): number;

  /**
   * Returns the maximum number sold.
   *
   * @returns {number}
   * @memberof Scene_Shop
   */
  maxSell(): number;

  /**
   * Returns the player gold within
   * the shop scene.
   * @returns {number}
   * @memberof Scene_Shop
   */
  money(): rm.types.Money;

  /**
   * Returns the currency unit of the
   * game within the shop scene.
   * @returns {string}
   * @memberof Scene_Shop
   */
  currencyUnit(): string;

  /**
   * Returns the buying price for the current item.
   *
   * @returns {number}
   * @memberof Scene_Shop
   */
  buyingPrice(): rm.types.Money;

  /**
   * Returns the selling price for the current item.
   *
   * @returns {number}
   * @memberof Scene_Shop
   */
  sellingPrice(): rm.types.Money;
}

/**
 * The scene class of the skill screen.
 */
declare class Scene_Skill extends Scene_ItemBase {
  protected constructor();

  skillTypeWindowRect(): Rectangle;

  statusWindowRect(): Rectangle;

  itemWindowRect(): Rectangle;

  /**
   * Creates the window for skill types
   * within the skill scene.
   * @memberof Scene_Skill
   */
  createSkillTypeWindow(): void;

  /**
   * Creates the status window within
   * the skill scene.
   * @memberof Scene_Skill
   */
  createStatusWindow(): void;

  /**
   * Creates the item window
   * within the skill scene.
   * @memberof Scene_Skill
   */
  createItemWindow(): void;

  /**
   * Refreshes the current actor displayed in the Skill
   * window with updated information on changes.
   */
  refreshActor(): void;

  /**
   * Handler for when a skill is
   * selected/confirmed within the skill scene.
   * @memberof Scene_Skill
   */
  commandSkill(): void;

  /**
   * Function to run when selection is cancelled
   * on the skill scene.
   */
  onItemCancel(): void;

  /**
   * Plays a sound effect on item
   * confirmation withiin the skill scene.
   * @memberof Scene_Skill
   */
  playSeForItem(): void;

  /**
   * Handler for when an an actor is
   * changed within the skill scene.
   * @memberof Scene_Skill
   */
  onActorChange(): void;
}

/**
 * Scene class of the status screen in RPGMakerMV.
 */
declare class Scene_Status extends Scene_MenuBase {
  protected constructor();

  createProfileWindow(): void;

  createStatusWindow(): void;

  createStatusParamsWindow(): void;

  createStatusEquipWindow(): void;

  statusParamsWindowRect(): Rectangle;

  statusEquipWindowRect(): Rectangle;

  statusParamsWidth(): number;

  statusParamsHeight(): number;

  profileHeight(): number;

  /**
   * Refreshes the actor within the status scene.
   *
   * @memberof Scene_Status
   */
  refreshActor(): void;

  /**
   * Handler for when the actor is changed within the status scene.
   *
   * @memberof Scene_Status
   */
  onActorChange(): void;
}

declare class Scene_Title extends Scene_Base {
  protected constructor();

  /**
   * The command window for title scene commands like new game, continue, or options.
   */
  _commandWindow: Window_TitleCommand;
  /**
   * The background sprite which loads the $dataSystem.title1Name image.
   */
  _backSprite1: Sprite;
  /**
   * The background sprite which loads the $dataSystem.title2Name image.
   */
  _backSprite2: Sprite;
  /**
   * The sprite which holds the game title text.
   */
  _gameTitleSprite: Sprite;

  /**
   * Creates the title scene background.
   */
  createBackground(): void;

  /**
   * Creates the title screen foreground.
   */
  createForeground(): void;

  /**
   * Draws the game title.
   */
  drawGameTitle(): void;

  /**
   * Scales the background images and centers it.
   */
  adjustBackground(): void;

  /**
   * Creates the command window and sets it's handlers.
   */
  createCommandWindow(): void;

  commandWindowRect(): Rectangle;

  centerSprite(sprite: Sprite): void;

  /**
   * Handler for the new game command.
   */
  commandNewGame(): void;

  /**
   * Handler for the continue command.
   */
  commandContinue(): void;

  /**
   * Handler for the options command.
   */
  commandOptions(): void;

  /**
   * Plays the title screen music upon entering the scene.
   */
  playTitleMusic(): void;
}

declare namespace rm.types {
  /**
   * The data class for the timing of an animation's SE and flash effects.
   */
  export type AnimationTiming = {
    /**
     * The color of the flash (Color).
     */
    flashColor: number[];
    /**
     * The duration of the flash.
     */
    flashDuration: number;
    /**
     * The flash area (0: none, 1: target, 2: screen; 3: hide target).
     */
    flashScope: number;
    /**
     * The frame number. 1 less than the number displayed in RPG Maker.
     */
    frame: number;
    /**
     * The sound effect or SE (RPG.AudioFile).
     */
    se: rm.types.AudioFile;
  };
}

declare namespace rm.types {
  /**
   * The data class for animation.
   */
  export type Animation = {
    /**
     * The adjustment value for the hue of the first animation's graphic (0..360).
     */
    animation1Hue: string;
    /**
     * The file name of the first animation's graphic.
     */
    animation1Name: string;
    /**
     * The adjustment value for the hue of the second animation's graphic (0..360).
     */
    animation2Hue: number;
    /**
     * The file name of the second animation's graphic.
     */
    animation2Name: string;
    /**
     * Number of frames.
     */
    frameMax: number;
    /**
     * The three-dimensional array containing the frame contents.
     */
    frames: number[][][];
    /**
     * The animation ID.
     */
    id: number;
    /**
     * The animation name.
     */
    name: string;
    /**
     * The base position (0: head, 1: center, 2: feet, 3: screen).
     */
    position: number;
    /**
     * Timing for SE and flash effects. An RPG.Animation.Timing array.
     */
    timings: rm.types.AnimationTiming[];
  };
}

declare class Sprite_Base extends Sprite {
  constructor();

  /**
   * The animation sprites assigned to the
   * sprite object.
   * @protected
   * @type {Array<Sprite_Animation>}
   * @memberof Sprite_Base
   */
  _animationSprites: Sprite_Animation[];
  /**
   * The target that will have the animations applied
   * to it.
   * @protected
   * @type {Sprite_Base}
   * @memberof Sprite_Base
   */
  _effectTarget: Sprite_Base;
  /**
   * Determines sprite's visibility
   */
  _hiding: boolean;

  /**
   * Initializes the sprite.
   *
   * @memberof Sprite_Base
   */
  initialize(): void;

  /**
   * Hides the sprite.
   */
  hide(): void;

  /**
   * Shows the sprite.
   */
  show(): void;

  /**
   * Updadtes the visibility of the sprite based
   * on the _hiding property.
   * @memberof Sprite_Base
   */
  updateVisibility(): void;

  /**
   * Updates the animation sprites, cloning them;
   * if the sprite is playing, pushes the sprites into
   * animation sprites, otherwise the sprite is removed.
   * @memberof Sprite_Base
   */
  updateAnimationSprites(): void;

  /**
   * Starts a new animation on the current
   * sprite by assigning a new Sprite_Animation object to the
   * sprite's parent.
   * @param {RPG.Animation} animation
   * @param {boolean} mirror
   * @param {number} delay
   * @memberof Sprite_Base
   */
  startAnimation(animation: rm.types.Animation, mirror: boolean, delay: number): void;

  /**
   * Returns true if an animation is currently playing.
   * @returns {boolean}
   * @memberof Sprite_Base
   */
  isAnimationPlaying(): boolean;
}

declare class Sprite_Battler extends Sprite_Base {
  protected constructor();

  _battler: Game_Battler;
  _homeX: number;
  _homeY: number;
  _offsetX: number;
  _offsetY: number;
  _targetOffsetX: number;
  _targetOffsetY: number;
  _movementDuration: number;
  _selectionEffectCount: number;

  initMembers(): void;

  setHome(x: number, y: number): void;

  /**
   * Updates the main loop of the sprite battler.
   */
  updateMain(): void;

  /**
   * Updates the bitmap of the sprite battler.
   */
  updateBitmap(): void;

  /**
   * Updates movement on the sprite battler.
   */
  updateMove(): void;

  /**
   * Updates the position of the sprite battler.
   */
  updatePosition(): void;

  /**
   * Updates the sprite battler animation.
   */
  updateAnimation(): void;
}

declare namespace rm.types {
  export type Motion = {
    index: number;
    loop: boolean;
  };
}

/**
 * -----------------------------------------------------------------------------
 * Sprite_Actor
 *
 * The sprite for displaying an actor.
 */
declare class Sprite_Actor extends Sprite_Battler {
  /**
   * Creates an instance of Sprite_Actor; can be passed
   * a battler on creation.
   * @param {Game_Actor} [battler]
   * @memberof Sprite_Actor
   */
  constructor(battler?: Game_Actor);

  _battlerName: string;
  _motion: rm.types.Motion;
  _motionCount: number;
  _pattern: rm.types.CharacterPattern;
  _mainSprite: Sprite_Base;
  _shadowSprite: Sprite;
  _weaponSprite: Sprite_Weapon;
  _stateSprite: Sprite_StateOverlay;
  _actor: Game_Actor;

  initialize(battler?: Game_Actor): void;

  /**
   * Creates the main sprite of the sprite actor.
   *
   * @memberof Sprite_Actor
   */
  createMainSprite(): void;

  /**
   * Creates the shadow sprite of the sprite actor.
   *
   * @memberof Sprite_Actor
   */
  createShadowSprite(): void;

  /**
   * Sets the weapon sprite of the sprite actor.
   *
   * @memberof Sprite_Actor
   */
  createWeaponSprite(): void;

  /**
   * Creates the state sprite of the sprite actor.
   *
   * @memberof Sprite_Actor
   */
  createStateSprite(): void;

  /**
   * Sets the battler of the sprite actor.
   *
   * @param {Game_Actor} battler
   * @memberof Sprite_Actor
   */
  setBattler(battler: Game_Actor): void;

  /**
   * Moves the sprite actor to the start position.
   *
   * @memberof Sprite_Actor
   */
  moveToStartPosition(): void;

  setActorHome(index: number): void;

  updateShadow(): void;

  /**
   * Sets up motion on the sprite actor.
   *
   * @memberof Sprite_Actor
   */
  setupMotion(): void;

  /**
   * Sets up weapon animation on the sprite actor.
   *
   * @memberof Sprite_Actor
   */
  setupWeaponAnimation(): void;

  /**
   * Starts the motion given the specified motion
   * type.
   * @param {string} motionType
   * @memberof Sprite_Actor
   */
  startMotion(motionType: string): void;

  updateTargetPosition(): void;

  /**
   * Updates the sprite actor's movement.
   *
   * @memberof Sprite_Actor
   */
  updateMove(): void;

  /**
   * Updates the sprite actor's motion.
   *
   * @memberof Sprite_Actor
   */
  updateMotion(): void;

  updateMotionCount(): void;

  /**
   * Returns the speed of the motion for the sprite actor.
   *
   * @returns {Int}
   * @memberof Sprite_Actor
   */
  motionSpeed(): number;

  /**
   * Refreshes the motion of the sprite actor.
   *
   * @memberof Sprite_Actor
   */
  refreshMotion(): void;

  /**
   * Starts the entry motion of the sprite actor.
   *
   * @memberof Sprite_Actor
   */
  startEntryMotion(): void;

  /**
   * Has the sprite actor step forward.
   *
   * @memberof Sprite_Actor
   */
  stepForward(): void;

  /**
   * Has the sprite actor step back.
   *
   * @memberof Sprite_Actor
   */
  stepBack(): void;

  /**
   * Has the sprite actor retreat.
   *
   * @memberof Sprite_Actor
   */
  retreat(): void;

  damageOffsetX(): number;

  damageOffsetY(): number;

  static MOTIONS: {
    abnormal: rm.types.Motion;
    chant: rm.types.Motion;
    damage: rm.types.Motion;
    dead: rm.types.Motion;
    dying: rm.types.Motion;
    escape: rm.types.Motion;
    evade: rm.types.Motion;
    guard: rm.types.Motion;
    item: rm.types.Motion;
    missile: rm.types.Motion;
    skill: rm.types.Motion;
    sleep: rm.types.Motion;
    spell: rm.types.Motion;
    swing: rm.types.Motion;
    thrust: rm.types.Motion;
    victory: rm.types.Motion;
    wait: rm.types.Motion;
    walk: rm.types.Motion;
  };
}

/**
 * -----------------------------------------------------------------------------
 * Sprite_Animation
 *
 * The sprite for displaying an animation.
 */
declare class Sprite_Animation extends Sprite {
  protected constructor();

  /**
   * {
   * key: Animation
   * };
   */
  _checker1: { [key: string]: any };
  _target: Sprite_Base;
  _animation: rm.types.Animation;
  _mirror: boolean;
  _delay: number;
  _rate: number;
  _duration: number;
  _flashColor: number[];
  _flashDuration: number;
  _screenFlashDuration: number;
  _hidingDuration: number;
  _bitmap1: Bitmap;
  _bitmap2: Bitmap;
  _cellSprites: Sprite[];
  _screenFlashSprite: ScreenSprite;
  _duplicated: boolean;
  _reduceArtifacts: boolean;

  initMembers(): void;

  setup(target: Sprite_Base, animation: rm.types.Animation, mirror: boolean, delay: number): void;

  /**
   * Removes the sprite animation.
   *
   * @memberof Sprite_Animation
   */
  remove(): void;

  setupRate(): void;

  setupDuration(): void;

  /**
   * Updates the flash animation of the sprite animation.
   *
   * @memberof Sprite_Animation
   */
  updateFlash(): void;

  updateScreenFlash(): void;

  /**
   * Returns the absolute x position of the sprite animation.
   *
   * @returns {number}
   * @memberof Sprite_Animation
   */
  absoluteX(): number;

  /**
   * Returns the absolute y position of the sprite aniamtion.
   *
   * @returns {number}
   * @memberof Sprite_Animation
   */
  absoluteY(): number;

  /**
   * Updates the hiding of the sprite animation.
   *
   * @memberof Sprite_Animation
   */
  updateHiding(): void;

  /**
   * Returns true if the sprite animation is playing.
   *
   * @returns {boolean}
   * @memberof Sprite_Animation
   */
  isPlaying(): boolean;

  /**
   * Loads the bitmaps of the sprite animation.
   *
   * @memberof Sprite_Animation
   */
  loadBitmaps(): void;

  /**
   * Returns true if the sprite animation is ready.
   *
   * @returns {boolean}
   * @memberof Sprite_Animation
   */
  isReady(): boolean;

  /**
   * Create the sprites of the sprite animation.
   *
   * @memberof Sprite_Animation
   */
  createSprites(): void;

  /**
   * Create the cell sprites of the sprite animation.
   *
   * @memberof Sprite_Animation
   */
  createCellSprites(): void;

  /**
   * Create the screen flash sprite of the sprite animation.
   *
   * @memberof Sprite_Animation
   */
  createScreenFlashSprite(): void;

  /**
   * Updates the main loop of the sprite animation.
   *
   * @memberof Sprite_Animation
   */
  updateMain(): void;

  /**
   * Updates the position of the sprite animation.
   *
   * @memberof Sprite_Animation
   */
  updatePosition(): void;

  /**
   * Updates the frame of the sprite aniamtion.
   *
   * @memberof Sprite_Animation
   */
  updateFrame(): void;

  /**
   * Returns the current frame index of the sprite aniamtion.
   *
   * @returns {number}
   * @memberof Sprite_Animation
   */
  currentFrameIndex(): number;

  updateAllCellSprites(frame: number[][]): void;

  updateCellSprite(sprite: Sprite, cell: number[]): void;

  processTimingData(timing: rm.types.AnimationTiming): void;

  startFlash(color: number[], duration: number): void;

  startScreenFlash(color: number[], duration: number): void;

  /**
   * Starts hiding the sprite animation.
   *
   * @param {number} duration The duration of the hide.
   * @memberof Sprite_Animation
   */
  startHiding(duration: number): void;

  /**
   * Structure
   * {
   * key: Animation
   * };
   */
  static _checker2: { [key: string]: any };
}

/**
 * -----------------------------------------------------------------------------
 * Sprite_Balloon
 *
 * The sprite for displaying a balloon icon.
 */
declare class Sprite_Balloon extends Sprite_Base {
  protected constructor();

  _balloonId: rm.types.BalloonId;
  _duration: number;

  initMembers(): void;

  loadBitmap(): void;

  setup(balloonId: rm.types.BalloonId): void;

  /**
   * Updates the balloon sprite.
   *
   * @memberof Sprite_Balloon
   */
  update(): void;

  /**
   * Updates the balloon sprite frame.
   *
   * @memberof Sprite_Balloon
   */
  updateFrame(): void;

  /**
   * Returns the speed of the balloon animation.
   *
   * @returns {number}
   * @memberof Sprite_Balloon
   */
  speed(): number;

  /**
   * Returns the wait time.
   *
   * @returns {number}
   * @memberof Sprite_Balloon
   */
  waitTime(): number;

  /**
   * Returns the frame index of the balloon animation.
   *
   * @returns {number}
   * @memberof Sprite_Balloon
   */
  frameIndex(): number;

  /**
   * Returns true if the balloon animation is playing.
   *
   * @returns {boolean}
   * @memberof Sprite_Balloon
   */
  isPlaying(): boolean;
}

/**
 * -----------------------------------------------------------------------------
 * Sprite_Button
 *
 * The sprite for displaying a button.
 */
declare class Sprite_Button extends Sprite {
  protected constructor();

  _touching: boolean;
  _coldFrame: Rectangle;
  _hotFrame: Rectangle;
  _clickHandler: () => void;

  /**
   * Update method, which checks if the sprite is being touched and updates
   * the current frame.
   *
   * @memberof Sprite_Button
   */
  updateFrame(): void;

  /**
   * Set the button sprites cold frame.
   *
   * @param {Int} x
   * @param {Int} y
   * @param {Int} width
   * @param {Int} height
   *
   * @memberof Sprite_Button
   *
   */
  setColdFrame(x: number, y: number, width: number, height: number): void;

  /**
   * Set the button sprites hot frame
   *
   * @param {Int} x
   * @param {Int} y
   * @param {Int} width
   * @param {Int} height
   *
   * @memberof Sprite_Button
   *
   */
  setHotFrame(x: number, y: number, width: number, height: number): void;

  /**
   * Creates a new handler and binds it to the button.
   *
   * @param {function} height
   * @memberof Sprite_Button
   */
  setClickHandler(method: () => void): void;

  /**
   * Calls the handler method bound to the button.
   *
   * @param {function} height
   * @memberof Sprite_Button
   */
  callClickHandler(): void;

  /**
   * Processes weather or not the button is being touched and calls the handler
   * bound to the button.
   * @memberof Sprite_Button
   */
  processTouch(): void;

  /**
   * Returns true if the sprite button is currently active.
   * @returns {Bool}
   * @memberof Sprite_Button
   */
  isActive(): boolean;

  /**
   * Returns true is the button is presently being touched.
   * @returns {Bool}
   * @memberof Sprite_Button
   */
  isButtonTouched(): boolean;

  /**
   * Changes the x coordinate of the screen to local sprite x coordinate.
   * @param {Int} x
   * @returns {Int}
   * @memberof Sprite_Button
   */
  canvasToLocalX(x: number): number;

  /**
   * Changes the y coordinate of the screen
   * to local sprite y coordinate.
   * @param {Int} y
   * @returns {Int}
   * @memberof Sprite_Button
   */
  canvasToLocalY(y: number): number;
}

/**
 * -----------------------------------------------------------------------------
 * Sprite_Character
 *
 * The sprite for displaying a character.
 */
declare class Sprite_Character extends Sprite_Base {
  /**
   * Creates an instance of Sprite_Character.
   * @param {Game_Character} character
   * @memberof Sprite_Character
   */
  constructor(character: Game_Character);

  /**
   * The Game_Character object assigned
   * to the sprite.
   * @private var
   * @type {Game_Character}
   * @memberof Sprite_Character
   */
  _character: Game_Character;
  _balloonDuration: number;
  _tilesetId: number;
  _upperBody: Sprite;
  _lowerBody: Sprite;
  _bushDepth: number;
  /**
   * The current balloon sprite
   * assigned to the sprite.
   * @private var
   * @type {Sprite_Balloon}
   * @memberof Sprite_Character
   */
  _balloonSprite: Sprite_Balloon;

  initialize(character: Game_Character): void;

  initMembers(): void;

  /**
   * Sets the current Game_Character object
   * attached to the sprite.
   * @param {Game_Character} character
   * @memberof Sprite_Character
   */
  setCharacter(character: Game_Character): void;

  /**
   * Returns true if the Game_Character object
   * tileId is greater than 0.
   * @returns {Bool}
   * @memberof Sprite_Character
   */
  isTile(): boolean;

  tilesetBitmap(tileId: number): Bitmap;

  /**
   * Updates the bitmap of the sprite character.
   *
   * @memberof Sprite_Character
   */
  updateBitmap(): void;

  /**
   * Returns true if the sprite character image has changed.
   *
   * @returns {Bool}
   * @memberof Sprite_Character
   */
  isImageChanged(): boolean;

  setTileBitmap(): void;

  /**
   * Sets the sprite character bitmap.
   *
   * @memberof Sprite_Character
   */
  setCharacterBitmap(): void;

  /**
   * Updates the sprite character frame.
   *
   * @memberof Sprite_Character
   */
  updateFrame(): void;

  /**
   * Updates the sprite character tile frame.
   *
   * @memberof Sprite_Character
   */
  updateTileFrame(): void;

  /**
   * Updates the sprite character -- character frame.
   *
   * @memberof Sprite_Character
   */
  updateCharacterFrame(): void;

  characterBlockX(): number;

  characterBlockY(): number;

  /**
   * Returns the character x pattern.
   *
   * @returns {number}
   * @memberof Sprite_Character
   */
  characterPatternX(): rm.types.CharacterPattern;

  /**
   * Returns the character y pattern.
   *
   * @returns {number}
   * @memberof Sprite_Character
   */
  characterPatternY(): rm.types.CharacterPattern;

  /**
   * Returns the pattern width.
   *
   * @returns {number}
   * @memberof Sprite_Character
   */
  patternWidth(): number;

  /**
   * Returns the pattern height.
   *
   * @returns {number}
   * @memberof Sprite_Character
   */
  patternHeight(): number;

  updateHalfBodySprites(): void;

  createHalfBodySprites(): void;

  /**
   * Updates the position of the sprite character.
   *
   * @memberof Sprite_Character
   */
  updatePosition(): void;

  updateAnimation(): void;

  updateOther(): void;

  setupAnimation(): void;

  /**
   * Sets up the Game_Character object
   * balloon sprite, and calls the startBalloon method.
   * @memberof Sprite_Character
   */
  setupBalloon(): void;

  /**
   * Starts the balloon sprite on the
   * Game_Character object.
   * @memberof Sprite_Character
   */
  startBalloon(): void;

  /**
   * Processes the balloon sprite, calls
   * the endBaloon method if the balloon sprite is done playing.
   * @memberof Sprite_Character
   */
  updateBalloon(): void;

  /**
   * Ends the balloon sprite, removing it from
   * the Game_Character object sprite.
   * @memberof Sprite_Character
   */
  endBalloon(): void;

  /**
   * Returns true if a balloon animation
   * is playing on the character.
   * @returns {Bool}
   * @memberof Sprite_Character
   */
  isBalloonPlaying(): boolean;
}

/**
 * -----------------------------------------------------------------------------
 * Sprite_Damage
 *
 * The sprite for displaying a popup damage.
 */
declare class Sprite_Damage extends Sprite {
  protected constructor();

  __duration: number;
  /**
   * Array of 3 numbers of RGB
   */
  _flashColor: number[];
  _flashDuration: number;
  _damageBitmap: Bitmap;

  setup(target: Game_Actor): void;

  setupCriticalEffect(): void;

  /**
   * Returns the digit width of the sprite damage.
   *
   * @returns {number}
   * @memberof Sprite_Damage
   */
  digitWidth(): number;

  /**
   * Returns the digit height of the sprite damage.
   *
   * @returns {number}
   * @memberof Sprite_Damage
   */
  digitHeight(): number;

  /**
   * Creates the miss display of the damage sprite.
   *
   * @memberof Sprite_Damage
   */
  createMiss(): void;

  createDigits(baseRow: number, value: number): void;

  /**
   * Creates the child sprite of the damage sprite for displaying damage.
   *
   * @returns {Sprite}
   * @memberof Sprite_Damage
   */
  createChildSprite(): Sprite;

  updateChild(sprite: Sprite): void;

  /**
   * Updates the flash of the damage sprite.
   *
   * @memberof Sprite_Damage
   */
  updateFlash(): void;

  /**
   * Updates the opacity of the damage sprite.
   *
   * @memberof Sprite_Damage
   */
  updateOpacity(): void;

  /**
   * Returns true if the damage sprite is playing.
   *
   * @returns {boolean}
   * @memberof Sprite_Damage
   */
  isPlaying(): boolean;
}

/**
 * -----------------------------------------------------------------------------
 * Sprite_Destination
 *
 * The sprite for displaying the destination place of the touch input.
 */
declare class Sprite_Destination extends Sprite {
  protected constructor();

  _frameCount: number;

  /**
   * Creates the destination bitmap of the destination sprite.
   *
   * @memberof Sprite_Destination
   */
  createBitmap(): void;

  /**
   * Updates the position of the destination sprite.
   *
   * @memberof Sprite_Destination
   */
  updatePosition(): void;

  /**
   * Updates the destination sprite animation.
   *
   * @memberof Sprite_Destination
   */
  updateAnimation(): void;
}

/**
 * -----------------------------------------------------------------------------
 * Sprite_Enemy
 *
 * The sprite for displaying an enemy.
 */
declare class Sprite_Enemy extends Sprite_Battler {
  constructor(battler: Game_Enemy);

  _enemy: Game_Enemy;
  _appeared: boolean;
  _battlerName: string;
  _battlerHue: number;
  _effectType: string;
  _effectDuration: number;
  _shake: number;
  _stateIconSprite: Sprite_StateIcon;

  initialize(battler: Game_Enemy): void;

  createStateIconSprite(): void;

  /**
   * Sets the battler to an instance of game enemy.
   *
   * @param {Game_Enemy} battler Instance of game enemy.
   * @memberof Sprite_Enemy
   */
  setBattler(battler: Game_Enemy): void;

  loadBitmap(name: string, hue: number): void;

  /**
   * Updates the state sprite on the sprite enemy.
   *
   * @memberof Sprite_Enemy
   */
  updateStateSprite(): void;

  initVisibility(): void;

  setupEffect(): void;

  startEffect(effectType: string): void;

  /**
   * Starts the appearinig effect on the sprite enemy.
   *
   * @memberof Sprite_Enemy
   */
  startAppear(): void;

  /**
   * Starts the disappearing effect on the sprite enemy.
   *
   * @memberof Sprite_Enemy
   */
  startDisappear(): void;

  /**
   * Starts the whiten effect on the sprite enemy.
   *
   * @memberof Sprite_Enemy
   */
  startWhiten(): void;

  /**
   * Starts the blink effect on the sprite enemy.
   *
   * @memberof Sprite_Enemy
   */
  startBlink(): void;

  /**
   * Starts the collapse effect of the sprite
   * enemy.
   * @memberof Sprite_Enemy
   */
  startCollapse(): void;

  /**
   * Starts the boss collapse effect of the sprite
   * enemy.
   * @memberof Sprite_Enemy
   */
  startBossCollapse(): void;

  /**
   * Starts the instant collapse effect of the sprite enemy.
   *
   * @memberof Sprite_Enemy
   */
  startInstantCollapse(): void;

  updateEffect(): void;

  /**
   * Returns true if the effect type on the sprite enemy
   * is not null.
   * @returns {Bool}
   * @memberof Sprite_Enemy
   */
  isEffecting(): boolean;

  /**
   * Revers the sprite enemy to a normal state.
   *
   * @memberof Sprite_Enemy
   */
  revertToNormal(): void;

  /**
   * Updates the whiten effect on the sprite enemy.
   *
   * @memberof Sprite_Enemy
   */
  updateWhiten(): void;

  /**
   * Updates the blink effect on the sprite enemy.
   *
   * @memberof Sprite_Enemy
   */
  updateBlink(): void;

  /**
   * Updates the appear effect on the sprite enemy.
   *
   * @memberof Sprite_Enemy
   */
  updateAppear(): void;

  updateDisappear(): void;

  /**
   * Updates the collapse effect.
   *
   * @memberof Sprite_Enemy
   */
  updateCollapse(): void;

  /**
   * Updates the boss collapse effect.
   *
   * @memberof Sprite_Enemy
   */
  updateBossCollapse(): void;

  /**
   * Updates the instant collapse effect.
   *
   * @memberof Sprite_Enemy
   */
  updateInstantCollapse(): void;

  damageOffsetX(): number;

  damageOffsetY(): number;
}

/**
 * -----------------------------------------------------------------------------
 * Sprite_Picture
 *
 * The sprite for displaying a picture.
 */
declare class Sprite_Picture {
  constructor(pictureId: number);

  _pictureId: number;
  _pictureName: string;
  _isPicture: boolean;

  picture(): Game_Picture;

  updateBitmap(): void;

  updateOrigin(): void;

  updatePosition(): void;

  updateScale(): void;

  updateTone(): void;

  updateOther(): void;

  loadBitmap(): void;
}

/**
 * -----------------------------------------------------------------------------
 * Sprite_StateIcon
 *
 * The sprite for displaying state icons.
 */
declare class Sprite_StateIcon extends Sprite {
  protected constructor();

  _battler: Game_Battler;
  _iconIndex: number;
  _animationCount: number;
  _animationIndex: number;

  /**
   * Initializes the sprite state icon properties.
   *
   * @memberof Sprite_StateIcon
   */
  initMembers(): void;

  /**
   * Loads the bitmap of the sprite state icon.
   *
   * @memberof Sprite_StateIcon
   */
  loadBitmap(): void;

  setup(battler: Game_Battler): void;

  animationWait(): number;

  /**
   * Updates the icon displayed in the icon sprite.
   *
   * @memberof Sprite_StateIcon
   */
  updateIcon(): void;

  /**
   * Updates the state icon sprite frame.
   *
   * @memberof Sprite_StateIcon
   */
  updateFrame(): void;
}

/**
 * -----------------------------------------------------------------------------
 * Sprite_StateOverlay
 *
 * The sprite for displaying an overlay image for a state.
 */
declare class Sprite_StateOverlay extends Sprite_Base {
  protected constructor();

  _battler: Game_Battler;
  _overlayIndex: number;
  _animationCount: number;
  _pattern: number;

  /**
   * Initialize the overlay sprite properties.
   *
   * @memberof Sprite_StateOverlay
   */
  initMembers(): void;

  /**
   * Loads the bitmap of the overlay sprite.
   *
   * @memberof Sprite_StateOverlay
   */
  loadBitmap(): void;

  setup(battler: Game_Battler): void;

  animationWait(): number;

  /**
   * Updates the overlay sprite pattern.
   *
   * @memberof Sprite_StateOverlay
   */
  updatePattern(): void;

  /**
   * Updates the overlay sprite frame.
   *
   * @memberof Sprite_StateOverlay
   */
  updateFrame(): void;
}

/**
 * -----------------------------------------------------------------------------
 * Sprite_Timer
 *
 * The sprite for displaying the timer.
 */
declare class Sprite_Timer extends Sprite {
  protected constructor();

  _seconds: number;

  /**
   * Creates the bitmap of the sprite timer.
   *
   * @memberof Sprite_Timer
   */
  createBitmap(): void;

  /**
   * Updates the bitmap of the sprite timer.
   *
   * @memberof Sprite_Timer
   */
  updateBitmap(): void;

  /**
   * Redraws the sprite timer.
   *
   * @memberof Sprite_Timer
   */
  redraw(): void;

  /**
   * Returns the text of the timer.
   *
   * @returns {string} The text displayed on the timer.
   * @memberof Sprite_Timer
   */
  timerText(): string;

  /**
   * Updates the positon of the sprite timer.
   *
   * @memberof Sprite_Timer
   */
  updatePosition(): void;

  /**
   * Updates the visibility of the sprite timer.
   *
   * @memberof Sprite_Timer
   */
  updateVisibility(): void;
}

/**
 * -----------------------------------------------------------------------------
 * Sprite_Weapon
 *
 * The sprite for displaying a weapon image for attacking.
 */
declare class Sprite_Weapon extends Sprite_Base {
  protected constructor();

  _weaponImageId: rm.types.WeaponImageId;
  _animaationCount: number;
  _pattern: number;

  /**
   * Initializes the members of the weapon sprite object.
   *
   * @memberof Sprite_Weapon
   */
  initMembers(): void;

  setup(weaponImageId: rm.types.WeaponImageId): void;

  animationWait(): number;

  /**
   * Updates the pattern of the weapon sprite.
   *
   * @memberof Sprite_Weapon
   */
  updatePattern(): void;

  /**
   * Loads the bitmap of the weapon sprite.
   *
   * @memberof Sprite_Weapon
   */
  loadBitmap(): void;

  /**
   * Updates the weapon sprite frames.
   *
   * @memberof Sprite_Weapon
   */
  updateFrame(): void;

  /**
   * Returns true if the weapon sprite is playing.
   *
   * @returns {boolean}
   * @memberof Sprite_Weapon
   */
  isPlaying(): boolean;
}

/**
 * -----------------------------------------------------------------------------
 * Spriteset_Base
 *
 * The superdeclare class of Spriteset_Map and Spriteset_Battle.
 */
declare class Spriteset_Base extends Sprite {
  protected constructor();

  _tone: number[];
  _baseSprite: Sprite;
  _blackScreen: ScreenSprite;
  _toneFilter: ToneFilter;
  _toneSprite: ToneSprite;
  _pictureContainer: Sprite;
  _timerSprite: Sprite_Timer;
  _flashSprite: ScreenSprite;
  _fadeSprite: ScreenSprite;

  /**
   * Creates the lower layer including the base sprites.
   *
   * @memberof Spriteset_Base
   */
  createLowerLayer(): void;

  /**
   * Creates the upper layer including the pictures,
   * timer, and screen sprites.
   * @memberof Spriteset_Base
   */
  createUpperLayer(): void;

  /**
   * Create the base sprite.
   *
   * @memberof Spriteset_Base
   */
  createBaseSprite(): void;

  /**
   * Creates the tone changer sprite.
   *
   * @memberof Spriteset_Base
   */
  createToneChanger(): void;

  /**
   * Creates the WebGL toner.
   *
   * @memberof Spriteset_Base
   */
  createWebGLToneChanger(): void;

  /**
   * Creates the Canvas tone.
   *
   * @memberof Spriteset_Base
   */
  createCanvasToneChanger(): void;

  /**
   * Creates a new sprite picture on the spritesetb ase.
   *
   * @memberof Spriteset_Base
   */
  createPictures(): void;

  /**
   * Creates a new Sprite timer on the spriteset base.
   *
   * @memberof Spriteset_Base
   */
  createTimer(): void;

  /**
   * Creates the screen sprite.
   *
   * @memberof Spriteset_Base
   */
  createScreenSprites(): void;

  /**
   * Updates the screen sprites on the spriteset base.
   *
   * @memberof Spriteset_Base
   */
  updateScreenSprites(): void;

  updateToneChanger(): void;

  /**
   * Updates the WebGL tone changer.
   *
   * @memberof Spriteset_Base
   */
  updateWebGLToneChanger(): void;

  /**
   * Updates the Canvas tone changer.
   *
   * @memberof Spriteset_Base
   */
  updateCanvasToneChanger(): void;

  /**
   * Updates the position of spriteset base.
   *
   * @memberof Spriteset_Base
   */
  updatePosition(): void;
}

/**
 * -----------------------------------------------------------------------------
 * Spriteset_Battle
 *
 * The set of sprites on the battle screen.
 */
declare class Spriteset_Battle extends Spriteset_Base {
  protected constructor();

  _battlebackLocated: boolean;
  _backgroundSprite: Sprite;
  _battleField: Sprite;
  _back1Sprite: TilingSprite;
  _back2Sprite: TilingSprite;
  _enemySprites: Sprite_Enemy[];
  _actorSprites: Sprite_Actor[];

  /**
   * Creates the background of the battle spriteset.
   *
   * @memberof Spriteset_Battle
   */
  createBackground(): void;

  /**
   * Creates the battlefield of the battle spriteset.
   *
   * @memberof Spriteset_Battle
   */
  createBattleField(): void;

  /**
   * Creates the battleback of the battle spriteset.
   *
   * @memberof Spriteset_Battle
   */
  createBattleback(): void;

  /**
   * Updates the battleback of the battle spriteset.
   *
   * @memberof Spriteset_Battle
   */
  updateBattleback(): void;

  /**
   * Locates the battleback and adjusts the coordinates of the
   * battleback.
   * @memberof Spriteset_Battle
   */
  locateBattleback(): void;

  /**
   * Returns battleb ack 2 of the battle spriteset.
   *
   * @returns {Bitmap} Instance of the Bitmap class.
   * @memberof Spriteset_Battle
   */
  battleback1Bitmap(): Bitmap;

  /**
   * Returns battleback 2 of the battle spriteset.
   *
   * @returns {Bitmap} Instance of the Bitmap class.
   * @memberof Spriteset_Battle
   */
  battleback2Bitmap(): Bitmap;

  /**
   *
   *
   * @returns {string} Name of battleback 1 bitmap.
   * @memberof Spriteset_Battle
   */
  battleback1Name(): string;

  /**
   *
   *
   * @returns {string} Name of battleback 2 bitmap.
   * @memberof Spriteset_Battle
   */
  battleback2Name(): string;

  /**
   * Returns the battleback 1 name as a string.
   *
   * @returns {string} Name of overworld battleback 1 bitmap.
   * @memberof Spriteset_Battle
   */
  overworldBattleback1Name(): string;

  /**
   * Returns the battleback 2 name as a string.
   *
   * @returns {string} Name of overworld battleback 2 bitmap.
   * @memberof Spriteset_Battle
   */
  overworldBattleback2Name(): string;

  /**
   *
   *
   * @returns {string} Name of the normal battleback 1 bitmap.
   * @memberof Spriteset_Battle
   */
  normalBattleback1Name(): string;

  /**
   *
   *
   * @returns {string} Name of the normal battleback 2 bitmap.
   * @memberof Spriteset_Battle
   */
  normalBattleback2Name(): string;

  /**
   * Given the specified terrtain type, return
   * the battleback 1 name.
   * @param {number} type Terrain type.
   * @returns {string} Name of the terrtain battleback 1 bitmap.
   * @memberof Spriteset_Battle
   */
  terrainBattleback1Name(type: number): string;

  /**
   * Given the specified terrain type, return
   * the battleback 2 name.
   * @param {number} type Terrain type.
   * @returns {string} Name of the terrain battleback 2 bitmap.
   * @memberof Spriteset_Battle
   */
  terrainBattleback2Name(type: number): string;

  /**
   *
   *
   * @returns {string} Name of the default battleback 1 name.
   * @memberof Spriteset_Battle
   */
  defaultBattleback1Name(): string;

  /**
   *
   *
   * @returns {string} Name of the default battleback 2 name.
   * @memberof Spriteset_Battle
   */
  defaultBattleback2Name(): string;

  /**
   *
   *
   * @returns {string} Name of the ship battleback 1  bitmap.
   * @memberof Spriteset_Battle
   */
  shipBattleback1Name(): string;

  /**
   *
   *
   * @returns {string} Name of the ship battleback 2 bitmap.
   * @memberof Spriteset_Battle
   */
  shipBattleback2Name(): string;

  autotileType(z: number): number;

  /**
   * Creates sprite enemies for the battle spriteset.
   *
   * @memberof Spriteset_Battle
   */
  createEnemies(): void;

  compareEnemySprite(a: Sprite_Enemy, b: Sprite_Enemy): number;

  /**
   * Creates sprite actors for the battle spriteset.
   *
   * @memberof Spriteset_Battle
   */
  createActors(): void;

  /**
   * Updates the actor sprites on the battle spriteset.
   *
   * @memberof Spriteset_Battle
   */
  updateActors(): void;

  /**
   * Returns all battler sprites on the battle spriteset.
   *
   * @returns {Array<Sprite_Battler>}
   * @memberof Spriteset_Battle
   */
  battlerSprites(): Sprite_Battler[];

  /**
   * Returns true if animation is playing on the battle spriteset.
   *
   * @returns {boolean}
   * @memberof Spriteset_Battle
   */
  isAnimationPlaying(): boolean;

  isEffecting(): boolean;

  /**
   * Returns true if any sprite actor or enemy is moving.
   *
   * @returns {boolean} Representing whether any battle participants are moving.
   * @memberof Spriteset_Battle
   */
  isAnyoneMoving(): boolean;

  /**
   * Returns true if the battle spriteset is busy.
   *
   * @returns {boolean}
   * @memberof Spriteset_Battle
   */
  isBusy(): boolean;
}

/**
 * -----------------------------------------------------------------------------
 * Spriteset_Map
 *
 * The set of sprites on the map screen.
 */
declare class Spriteset_Map extends Spriteset_Base {
  protected constructor();

  _parallax: TilingSprite;
  _tilemap: Tilemap | ShaderTilemap;
  _tileset: rm.types.Tileset;
  _characterSprites: Sprite_Character[];
  _shadowSprite: Sprite;
  _destinationSprite: Sprite_Destination;
  _weather: Weather;
  _parallaxName: string;

  /**
   * Hides the map spriteset character sprites.
   *
   * @memberof Spriteset_Map
   */
  hideCharacters(): void;

  /**
   * Creates the map spriteset parallax.
   *
   * @memberof Spriteset_Map
   */
  createParallax(): void;

  /**
   * Creates the map spriteset tile map.
   *
   * @memberof Spriteset_Map
   */
  createTilemap(): void;

  /**
   * Loads the map spriteset tileset.
   *
   * @memberof Spriteset_Map
   */
  loadTileset(): void;

  /**
   * Creates the map spriteset character sprite.
   *
   * @memberof Spriteset_Map
   */
  createCharacters(): void;

  /**
   * Creates the map spriteset shadow sprite.
   *
   * @memberof Spriteset_Map
   */
  createShadow(): void;

  /**
   * Creates the map spriteset destination sprite.
   *
   * @memberof Spriteset_Map
   */
  createDestination(): void;

  /**
   * Creates the map spriteset weather.
   *
   * @memberof Spriteset_Map
   */
  createWeather(): void;

  /**
   * Updates the map spriteset tileset.
   *
   * @memberof Spriteset_Map
   */
  updateTileset(): void;

  updateParallax(): void;

  /**
   * Updates the map spriteset tile map.
   *
   * @memberof Spriteset_Map
   */
  updateTilemap(): void;

  /**
   * Updates the map spriteset shadow.
   *
   * @memberof Spriteset_Map
   */
  updateShadow(): void;

  /**
   * Updates the map spriteset weather.
   *
   * @memberof Spriteset_Map
   */
  updateWeather(): void;
}

declare namespace rm.types {
  export type TextState = {
    buffer: string;
    drawing: boolean;
    height: number;
    index: number;
    outputHeight: number;
    outputWidth: number;
    rtl: boolean;
    startX: number;
    startY: number;
    text: string;
    width: number;
  };
}

declare class Window_Base extends Window {
  constructor(rect: Rectangle);

  initialize(rect: Rectangle): void;

  /**
   * Returns the standard line height of the current window;
   * default is 36.
   * @returns {number}
   * @memberof Window_Base
   */
  lineHeight(): number;

  /**
   * Returns the standard font face of the
   * game based on what language the game is in.
   * @returns {String}
   * @memberof Window_Base
   */
  standardFontFace(): string;

  /**
   * Returns the standard font size of the text
   * in window; default is 28.
   * @returns {number}
   * @memberof Window_Base
   */
  standardFontSize(): number;

  /**
   * Returns the standard padding of the window;
   * default is 18.
   * @returns {number}
   * @memberof Window_Base
   */
  standardPadding(): number;

  /**
   * Returns the text padding of the window;
   * default is 6.
   * @returns {number}
   * @memberof Window_Base
   */
  textPadding(): number;

  /**
   * Returns the standard back opacity of the window; this is the
   * opacity of the area behind the window's text content.
   * Default is 192.
   * @returns {number}
   * @memberof Window_Base
   */
  standardBackOpacity(): number;

  /**
   * Loads the window skin from the img/system directory.
   *
   * @memberof Window_Base
   */
  loadWindowSkin(): void;

  /**
   * Updates the window padding based on the
   * standardPadding method.
   * @memberof Window_Base
   */
  updatePadding(): void;

  /**
   * Updates the back opacity of the window
   * based on the standardBackOpacity method.
   * @memberof Window_Base
   */
  updateBackOpacity(): void;

  /**
   * Returns the inner content width of the window.
   *
   * @returns {number}
   * @memberof Window_Base
   */
  contentsWidth(): number;

  /**
   * Returns the inner content height of the window.
   *
   * @returns {number}
   * @memberof Window_Base
   */
  contentsHeight(): number;

  /**
   * Returns the fitting height given a number of lines based on
   * the line height plus standard padding of the window.
   * Default formula: numLines * lineHeight + standardPadding * 2
   *
   * @param {number} numLines
   * @returns {number}
   * @memberof Window_Base
   */
  fittingHeight(numLines: number): number;

  /**
   * Updates the tone of the window based on the
   * game system window tone defined in the database.
   * @memberof Window_Base
   */
  updateTone(): void;

  /**
   * Creates the contents of the window; this is the area
   * of the window which text is drawn to.
   * @memberof Window_Base
   */
  createContents(): void;

  /**
   * Resets the font settings of the window back to the
   * default.
   * @memberof Window_Base
   */
  resetFontSettings(): void;

  /**
   * Resets the text color of the window back to the
   * default.
   * @memberof Window_Base
   */
  resetTextColor(): void;

  /**
   * The update method of the window; this is
   * run every frame to do logic processing for the window.
   * @memberof Window_Base
   */
  update(): void;

  /**
   * Updates the openness of the window when the
   * _opening property is set to true.
   * Openness is increased.
   * @memberof Window_Base
   */
  updateOpen(): void;

  /**
   * Updates the openness of the window when the
   * _closing property is set to true.
   * Openness is decreased.
   * @memberof Window_Base
   */
  updateClose(): void;

  /**
   * Opens the window.
   *
   * @memberof Window_Base
   */
  open(): void;

  /**
   * Closes the window.
   *
   * @memberof Window_Base
   */
  close(): void;

  /**
   * Returns true if the window is currently opening.
   *
   * @returns {boolean}
   * @memberof Window_Base
   */
  isOpening(): boolean;

  /**
   * Returns true if the window is currently closing.
   *
   * @returns {boolean}
   * @memberof Window_Base
   */
  isClosing(): boolean;

  /**
   * Shows the window, making it visible.
   *
   * @memberof Window_Base
   */
  show(): void;

  /**
   * Hides the window, making it invisible;
   * the window is not closed when hidden.
   *
   * @memberof Window_Base
   */
  hide(): void;

  /**
   * Activates the window, allowing it to be processed
   * and to update.
   * @memberof Window_Base
   */
  activate(): void;

  /**
   * Deactives the window, preventing further processing.
   *
   * @memberof Window_Base
   */
  deactivate(): void;

  /**
   * Returns a text color given a numbered index
   * as a css color String; this index maps
   * directly to the img/system/window.png colors
   * by default.
   * @param {number} n
   * @returns {*}
   * @memberof Window_Base
   */
  textColor(n: number): string;

  /**
   * Returns the normal color as a css
   * color String.
   * @returns {String}
   * @memberof Window_Base
   */
  normalColor(): string;

  /**
   * Returns the system color as a
   * css color String.
   * @returns {String}
   * @memberof Window_Base
   */
  systemColor(): string;

  /**
   * Returns the crisis color as a
   * css color String.
   * @returns {String}
   * @memberof Window_Base
   */
  crisisColor(): string;

  /**
   * Returns the death color as a
   * css color String.
   * @returns {String}
   * @memberof Window_Base
   */
  deathColor(): string;

  /**
   * Returns the gauage back color as
   * a css color String.
   * @returns {String}
   * @memberof Window_Base
   */
  gaugeBackColor(): string;

  /**
   * Returns the hp gauge color 1
   * as a css color String.
   * @returns {String}
   * @memberof Window_Base
   */
  hpGaugeColor1(): string;

  /**
   * Returns the hp gauge color 2
   * as a css color String.
   * @returns {String}
   * @memberof Window_Base
   */
  hpGaugeColor2(): string;

  /**
   * Returns the mp gauge color 1
   * as a css color String.
   * @returns {String}
   * @memberof Window_Base
   */
  mpGaugeColor1(): string;

  /**
   * Returns the mp gauge color 2
   * as a css color String.
   * @returns {String}
   * @memberof Window_Base
   */
  mpGaugeColor2(): string;

  /**
   * Returns the mp cost color as a
   * css color String.
   * @returns {String}
   * @memberof Window_Base
   */
  mpCostColor(): string;

  /**
   * Returns the power up color as a
   * css color String.
   * @returns {String}
   * @memberof Window_Base
   */
  powerUpColor(): string;

  /**
   * Returns the power down color as a
   * css color String.
   * @returns {String}
   * @memberof Window_Base
   */
  powerDownColor(): string;

  /**
   * Returns the tp gauge color 1 as a
   * css color String.
   * @returns {String}
   * @memberof Window_Base
   */
  tpGaugeColor1(): string;

  /**
   * Returns tp gauge color 2 as a
   * css color String.
   * @returns {String}
   * @memberof Window_Base
   */
  tpGaugeColor2(): string;

  /**
   * Returns the tp cost color as a
   * css color String.
   * @returns {String}
   * @memberof Window_Base
   */
  tpCostColor(): string;

  /**
   * Returns the pending color as a
   * css color String.
   * @returns {String}
   * @memberof Window_Base
   */
  pendingColor(): string;

  /**
   * Returns the translucentOpacity for the window;
   * The default is 160.
   *
   * @returns {number}
   * @memberof Window_Base
   */
  translucentOpacity(): number;

  /**
   * Changes the text color property given a css color String.
   *
   * @param {String} color
   * @memberof Window_Base
   */
  changeTextColor(color: string): void;

  /**
   * Changes the paintOpacity (the opacity of the text drawn to the window);
   * if true the opacity is set to 255, otherwise the opacity is set to 160.
   * @param {boolean} enabled
   * @memberof Window_Base
   */
  changePaintOpacity(enabled: boolean): void;

  /**
   * Given text or a number, draws the content to the window's contents
   * layer at the specified x and y coordinate within the max width.
   * The text content can also be aligned with the align property.
   * The possible alignments are: "left", "center", "right".
   * @param {(String | number)} text
   * @param {number} x
   * @param {number} y
   * @param {number} maxWidth
   * @param {String} align
   * @memberof Window_Base
   */
  drawText(text: string | number, x: number, y: number, maxWidth: number, align: string): void;

  /**
   * Calculates the width of a text String and
   * returns a number.
   * @param {String} text
   * @returns {number}
   * @memberof Window_Base
   */
  textWidth(text: string): number;

  /**
   * Draws text with text codes included; this will draw
   * icons, increase text height, and more.
   * @param text
   * @param x
   * @param y
   * @param width
   * @returns Int
   * @memberof Window_Base
   */
  drawTextEx(text: string, x: number, y: number, width: number): number;

  /**
   * Returns the text Size of drawTextEx.
   * @param text
   * @returns {width:Int, height:Int}
   */
  textSizeEx(text: string): { height: number; width: number };

  /**
   * Processes all the text in the window, then
   * flushes the text state.
   * @param textState
   */
  processAllText(textState: rm.types.TextState): void;

  /**
   * Flushes the text state.
   * @param textState
   */
  flushTextState(textState: rm.types.TextState): void;

  /**
   * Create a text buffer and determines whether
   * to use right to left embedding (U+202B).
   * @param rtl
   * @return String
   */
  createTextBuffer(rtl: boolean): string;

  processControlCharacter(extState: rm.types.TextState, character: string): void;

  /**
   * Converts the escape characters and returns the text content
   * after processing the characters.
   * @param {String} text
   * @returns {String}
   * @memberof Window_Base
   */
  convertEscapeCharacters(text: string): string;

  /**
   * Returns the actor name given an index;
   * the index starts from 1.
   * @param {number} actorIndex
   * @returns {String}
   * @memberof Window_Base
   */
  actorName(actorIndex: number): string;

  /**
   * Returns a party member name given an index;
   * the index starts from 1.
   * @param {number} partyMemberIndex
   * @returns {String}
   * @memberof Window_Base
   */
  partyMemberName(partyMemberIndex: number): string;

  /**
   * Process each character in the text when drawTextEx
   * is used to draw text.
   * @param {MV.TextState} textState
   * @memberof Window_Base
   */
  processCharacter(textState: rm.types.TextState): void;

  /**
   * Processes the normal characters in the text
   * when drawTextEx is used to draw text.
   * Normal characters are letters and numbers.
   * @param {MV.TextState} textState
   * @memberof Window_Base
   */
  processNormalCharacter(textState: rm.types.TextState): void;

  /**
   * Processes new line when drawTextEx is used to draw text.
   *
   * @param {MV.TextState} textState
   * @memberof Window_Base
   */
  processNewLine(textState: rm.types.TextState): void;

  /**
   * Processes new page when drawTexttEx is used to draw text.
   *
   * @param {MV.TextState} textState
   * @memberof Window_Base
   */
  processNewPage(textState: rm.types.TextState): void;

  obtainEscapeCode(textState: rm.types.TextState): string;

  /**
   * Obtains the escape parameters from text codes in the text state
   * when drawTextEx is used to draw text.
   * @param {MV.TextState} textState
   * @returns {(number | String)}
   * @memberof Window_Base
   */
  obtainEscapeParam(textState: string | rm.types.TextState): number | string;

  /**
   * Processes escape characters when drawTextEx is used
   * for drawing text.
   * @param {String} code
   * @param {MV.TextState} textState
   * @memberof Window_Base
   */
  processEscapeCharacter(code: string, textState: rm.types.TextState): void;

  /**
   * Processes drawing an icon when drawTextEx is used for
   * drawing text.
   * @param {number} iconIndex
   * @param {MV.TextState} textState
   * @memberof Window_Base
   */
  processDrawIcon(iconIndex: number, textState: rm.types.TextState): void;

  /**
   * Makes the font bigger by a value of 12.
   *
   * @memberof Window_Base
   */
  makeFontBigger(): void;

  /**
   * Makes the font smaller by a value of 12.
   *
   * @memberof Window_Base
   */
  makeFontSmaller(): void;

  /**
   * Calculates the text height of the textState (when using drawTextEx);
   * if all is set to true, all lines of text are calculated, otherwise
   * only a single line is processed.
   * @param {rm.TextState} textState
   * @param {Bool} all
   * @returns Int
   * @memberof Window_Base
   */
  calcTextHeight(textState: rm.types.TextState, all: boolean): number;

  /**
   * Draws an icon given the specified iconIndex at the specified
   * x and y coordinates. The Width and Height of the icon is based on the
   * _iconWidth and _iconHeight properties.
   * @param {number} iconIndex
   * @param {number} x
   * @param {number} y
   * @memberof Window_Base
   */
  drawIcon(iconIndex: number, x: number, y: number): void;

  drawFace(faceName: string, faceIndex: number, x: number, y: number, width: number, height: number): void;

  /**
   * Draws a character (map sprites) at the specified x and y coordinate.
   * CharacterName refers to character spritesheet, and characterIndex refers
   * to the characterIndex on the spritesheet.
   * @param {String} characterName
   * @param {number} characterIndex
   * @param {number} x
   * @param {number} y
   * @memberof Window_Base
   */
  drawCharacter(characterName: string, characterIndex: number, x: number, y: number): void;

  /**
   * Draws a gauge at the specified x and y coordinates within the given width.
   * Color1 and Color2 represent the gradient as css color Strings of the gauge.
   *
   * @param {number} x
   * @param {number} y
   * @param {number} width
   * @param {number} rate
   * @param {String} color1
   * @param {String} color2
   * @memberof Window_Base
   */
  drawGauge(x: number, y: number, width: number, rate: number, color1: string, color2: string): void;

  /**
   * Returns the hp color as a css String.
   *
   * @param {Game_Actor} actor
   * @returns {String}
   * @memberof Window_Base
   */
  hpColor(actor: Game_Actor): string;

  /**
   * Returns the mp color as a css color String.
   *
   * @param {Game_Actor} actor
   * @returns {String}
   * @memberof Window_Base
   */
  mpColor(actor: Game_Actor): string;

  /**
   * Returns the tp color as a css color String.
   *
   * @param {Game_Actor} actor
   * @returns {String}
   * @memberof Window_Base
   */
  tpColor(actor: Game_Actor): string;

  drawActorCharacter(actor: Game_Actor, x: number, y: number): void;

  /**
   * Draws the actor face at the specified x and y coordinates within
   * the given width.
   * @param {Game_Actor} actor
   * @param {number} x
   * @param {number} y
   * @param {number} width
   * @param {number} height
   * @memberof Window_Base
   */
  drawActorFace(actor: Game_Actor, x: number, y: number, width: number, height: number): void;

  /**
   * Draws the actor name at the specified x and y coordinates within
   * the given width.
   * @param {Game_Actor} actor
   * @param {number} x
   * @param {number} y
   * @param {number} width
   * @memberof Window_Base
   */
  drawActorName(actor: Game_Actor, x: number, y: number, width: number): void;

  /**
   * Draws the actor class at the specified x and y coordinates
   * within the given width.
   * @param {Game_Actor} actor
   * @param {number} x
   * @param {number} y
   * @param {number} width
   * @memberof Window_Base
   */
  drawActorClass(actor: Game_Actor, x: number, y: number, width: number): void;

  /**
   * Draws the actor nickname at the specified x and y coordinates
   * within the given width.
   * @param {Game_Actor} actor
   * @param {Int} x
   * @param {Int} y
   * @param {Int} width
   * @memberof Window_Base
   */
  drawActorNickname(actor: Game_Actor, x: number, y: number, width: number): void;

  /**
   * Draws the actor level at the specified x and y coordinates.
   *
   * @param {Game_Actor} actor
   * @param {Int} x
   * @param {Int} y
   * @memberof Window_Base
   */
  drawActorLevel(actor: Game_Actor, x: number, y: number): void;

  /**
   * Draws the actor icons at the specified x and y coordinates
   * within the given width.
   * @param {Game_Actor} actor
   * @param {number} x
   * @param {number} y
   * @param {number} width
   * @memberof Window_Base
   */
  drawActorIcons(actor: Game_Actor, x: number, y: number, width: number): void;

  /**
   * Draws the current and max number at the specified x and y coordinate
   * within the given width. Color1 represents the current number and color2
   * represents the max number when the text is drawn.
   * @param {number} current
   * @param {number} max
   * @param {number} x
   * @param {number} y
   * @param {number} width
   * @param {String} color1
   * @param {String} color2
   * @memberof Window_Base
   */
  drawCurrentAndMax(
    current: number,
    max: number,
    x: number,
    y: number,
    width: number,
    color1: string,
    color2: string,
  ): void;

  /**
   * Draws the actor hp at the specified x and y coordinates within
   * the given width.
   * @param {Game_Actor} actor
   * @param {number} x
   * @param {number} y
   * @param {number} width
   * @memberof Window_Base
   */
  drawActorHp(actor: Game_Actor, x: number, y: number, width: number): void;

  /**
   * Draws the actor mp at the specified x and y coordinates within
   * the given width.
   * @param {Game_Actor} actor
   * @param {number} x
   * @param {number} y
   * @param {number} width
   * @memberof Window_Base
   */
  drawActorMp(actor: Game_Actor, x: number, y: number, width: number): void;

  /**
   * Draws the actor tp at the specified x and y coordinates within the
   * given width.
   * @param {Game_Actor} actor
   * @param {number} x
   * @param {number} y
   * @param {number} width
   * @memberof Window_Base
   */
  drawActorTp(actor: Game_Actor, x: number, y: number, width: number): void;

  /**
   * Draws a simple status for the game actor passed into the method at the
   * specified x and y coordinates within the given width.
   *
   * @param {Game_Actor} actor
   * @param {number} x
   * @param {number} y
   * @param {number} width
   * @memberof Window_Base
   */
  drawActorSimpleStatus(actor: Game_Actor, x: number, y: number, width: number): void;

  /**
   * Draws the item name at the specified x and y coordinates within
   * the given width.
   * @param {RPG.BaseItem} item
   * @param {number} x
   * @param {number} y
   * @param {number} width
   * @memberof Window_Base
   */
  drawItemName(item: rm.types.BaseItem, x: number, y: number, width: number): void;

  /**
   * Draws the currency value given at the specified x and y coordinates within
   * the width given. Useful if you want to write your own custom currency value.
   * @param {number} value
   * @param {String} unit
   * @param {number} x
   * @param {number} y
   * @param {number} width
   * @memberof Window_Base
   */
  drawCurrencyValue(value: number, unit: string, x: number, y: number, width: number): void;

  /**
   * Changes the text color based on the powerUpColor, powerDownColor
   * and normal color. powerUpColor is any number greater than 0, powerDownColor
   * is any color less than 0, otherwise normal color is returned.
   * @param {number} change
   * @memberof Window_Base
   */
  paramchangeTextColor(change: number): rm.types.Color;

  /**
   * Sets the background type of the window.
   * 0 is 255 window opacity (standard).
   * 1 is the window with background dimmer.
   * Any other number changes the opacity
   * to 0.
   * @param {number} type
   * @memberof Window_Base
   */
  setBackgroundType(type: number): void;

  /**
   * Shows the background dimmer sprite.
   *
   * @memberof Window_Base
   */
  showBackgroundDimmer(): void;

  /**
   * Hides the background dimmer sprite.
   *
   * @memberof Window_Base
   */
  hideBackgroundDimmer(): void;

  /**
   * Updates the background dimmer sprite opacity based on the openness
   * of the window.
   * @memberof Window_Base
   */
  updateBackgroundDimmer(): void;

  /**
   * Refreshes the bitmap attached to the dimmer sprite
   * based on the window dimensions.
   * @memberof Window_Base
   */
  refreshDimmerBitmap(): void;

  /**
   * Color 1 of the dimmer sprite bitmap.
   * for the gradient.
   * @returns {String}
   * @memberof Window_Base
   */
  dimColor1(): string;

  /**
   * Color 2 of the dimmer sprite bitmap
   * for the gradient.
   * @returns {String}
   * @memberof Window_Base
   */
  dimColor2(): string;

  /**
   * Returns the x coordinate of the mouse to
   * a local window x coordinate.
   * @param {number} x
   * @returns {number}
   * @memberof Window_Base
   */
  canvasToLocalX(x: number): number;

  /**
   * Returns the y coordinate of the mouse
   * to a local window y coordinate.
   * @param {number} y
   * @returns {number}
   * @memberof Window_Base
   */
  canvasToLocalY(y: number): number;

  /**
   * Reverses the face images of the
   * game party members.
   * @memberof Window_Base
   */
  reserveFaceImages(): void;

  /**
   * Checks if object passed in is a rectangle..
   * Error is usually thrown when an MV plugin is used.
   * @param rect
   */
  checkRectObject(rect: Rectangle): void;

  /**
   * Returns a new rectangle used for base text.
   * Width and height are based off the innerWidth
   * and innerHeight properties.
   * @return Rectangle
   */
  baseTextRect(): Rectangle;

  changeOutlineColor(color: string): void;

  /**
   * Draws a rectangle using the outline color and
   * main text color of the window.
   * @param x
   * @param y
   * @param width
   * @param height
   */
  drawRect(x: number, y: number, width: number, height: number): void;

  /**
   * Destroys the window contents.
   * Also destroys the window back contents.
   */
  destroyContents(): void;

  /**
   * Plays a sound effect when okay is processed.
   *
   */
  playOkSound(): void;

  /**
   * Plays the buzzer sound effect when input is
   * incorrect.
   */
  playBuzzerSound(): void;

  /**
   * Plays the cursor sound from SoundManager;
   */
  playCursorSound(): void;

  /**
   * Changes the color based on the color index.
   * @param colorIndex
   */
  processColorChange(colorIndex: number): void;

  /**
   * Returns the maximum font size for a single line in rm.
   * @param line
   * @return Int
   *
   */
  maxFontSizeInLine(line: string): number;
}

declare class Window_Scrollable extends Window_Base {
  /**
   * Constructor for Window_Scrollable
   * @param rect
   */
  constructor(rect: Rectangle);

  /**
   * Initializes the scrollable window with a rectangle
   * called internally by new operator.
   * @param rect
   */
  initialize(rect: Rectangle): void;

  clearScrollStatus(): void;

  scrollX(): number;

  scrollY(): number;

  scrollBaseX(): number;

  scrollBaseY(): number;

  scrollTo(x: number, y: number): void;

  scrollBy(x: number, y: number): void;

  smoothScrollTo(x: number, y: number): void;

  setScrollAccel(x: number, y: number): void;

  /**
   *
   * Overall width of the window.
   * @return Int
   */
  overallWidth(): number;

  /**
   * Overall height of the window.
   * @return Int
   */
  overallHeight(): number;

  maxScrollX(): number;

  maxScrollY(): number;

  scrollBlockWidth(): number;

  scrollBlockHeight(): number;

  /**
   * Smoothly scrolls down to the number by the
   * height of each item.
   * @param num
   * @return Int
   */
  smoothScrollDown(num: number): number;

  /**
   * Smoothly scrolls up to the number by the height
   * of each item.
   * @param num
   * @return Int
   */
  smoothScrollUp(num: number): number;

  /**
   * Process wheel scrolling with the TouchInput wheel.
   * @return Int
   */
  processWheelScroll(): void;

  /**
   * Processes touch input scrolling.
   */
  processTouchScroll(): void;

  /**
   * Defaults to isScrollEnabled();
   * Determines if wheel scroll is enabled.
   * @return Bool
   */
  isWheelScrollEnabled(): boolean;

  /**
   * Defaults to isScrollEnabled();
   * Determines if touch scroll is enabled.
   * @return Bool
   */
  isTouchScrollEnabled(): boolean;

  /**
   * Whether scrolling is enabled; default to true.
   * @return Bool
   */
  isScrollEnable(): boolean;

  /**
   * Returns true if the window is touched
   * within the frame.
   * @return Bool
   */
  isTouchedInsideFrame(): boolean;

  /**
   * Handler for when touch scroll is started.
   */
  onTouchScrollStart(): void;

  /**
   * Handler for when touch scrolling.
   */
  onTouchScroll(): void;

  /**
   * Handler for when touch scroll is ending.
   */
  onTouchScrollEnd(): void;

  /**
   * Update function for smooth scrolling.
   */
  updateSmoothScroll(): void;

  /**
   * Update function for scroll accelleration.
   */
  updateScrollAccel(): void;

  /**
   * Update function for the scrolling arrows.
   */
  updateArrows(): void;

  /**
   * Update function for the origin when scrolling.
   */
  updateOrigin(): void;

  /**
   * Update function for the Scroll Base.
   */
  updateScrollBase(): void;

  /**
   * Function to be overriden for your personal use.
   */
  paint(): void;
}

declare class Window_Selectable extends Window_Scrollable {
  constructor(rect: Rectangle);

  initialize(rect: Rectangle): void;

  /**
   * Forcefully selects the index and cursor
   * visibility is set to false.
   * @param index
   */
  forceSelect(index: number): void;

  /**
   * Smoothly scrolls to the index.
   * Cursor visibility is set to true.
   * @param index
   */
  smoothSelect(index: number): void;

  itemRectWithPadding(index: number): Rectangle;

  /**
   * Return the rectangle for the line item with padding.
   * @param index
   */
  itemLineRect(index: number): Rectangle;

  /**
   * Overwrite this or add to it to add your own draw functions.
   */
  paint(): void;

  /**
   * Refreshes the cursor.
   */
  refreshCursor(): void;

  /**
   * Refreshes cursor for selecting all items.
   */
  refreshCursorForAll(): void;

  /**
   * Returns true by default.
   * Determines if hover support is enabled.
   * @return Bool
   */
  isHoverEnabled(): boolean;

  /**
   * Handler for Touching okay in the window.
   */
  onTouchOk(): void;

  /**
   * Handler for selecting in the window.
   */
  onTouchSelect(): void;

  /**
   * Handler for touching and processing cancel in
   * the window.
   */
  onTouchCancel(): void;

  /**
   * Returns the current position of the _index property.
   *
   * @returns {Int}
   * @memberof Window_Selectable
   */
  index(): number;

  /**
   * Returns true if the _cursorFixed property is true;
   * this means the cursor is locked to a position.
   * @returns {boolean}
   * @memberof Window_Selectable
   */
  cursorFixed(): boolean;

  /**
   * Sets the _cursorFixed property of the
   * window.
   * @param {boolean} cursorFixed
   * @memberof Window_Selectable
   */
  setCursorFixed(cursorFixed: boolean): void;

  cursorAll(): boolean;

  setCursorAll(cursorAll: boolean): void;

  /**
   * Returns the maximum number of columns
   * for the window.
   * @returns {number}
   * @memberof Window_Selectable
   */
  maxCols(): number;

  /**
   * Returns the maximum number of items within the window;
   * useful to overwrite when creating a new window.
   * This method is used to calculate the number of rows and more.
   * @returns {number}
   * @memberof Window_Selectable
   */
  maxItems(): number;

  spacing(): number;

  /**
   * Returns the width of an item within the window;
   * determines the width of a column.
   * @returns {number}
   * @memberof Window_Selectable
   */
  itemWidth(): number;

  /**
   * Returns the height of an item within the window;
   * determines the height of a row.
   * @returns {number}
   * @memberof Window_Selectable
   */
  itemHeight(): number;

  /**
   * Selects the current index within the window given a number.
   *
   * @param {number} index
   * @memberof Window_Selectable
   */
  select(index: number): void;

  /**
   * Deselects the currently selected index.
   *
   * @memberof Window_Selectable
   */
  deselect(): void;

  /**
   * Reselects the index based on the window's _index property.
   *
   * @memberof Window_Selectable
   */
  reselect(): void;

  row(): number;

  topRow(): number;

  maxTopRow(): number;

  /**
   * Sets the current top row of the given a number.
   * The top row will then be moved to an index
   * of the window.
   * @param {number} row
   * @memberof Window_Selectable
   */
  setTopRow(row: number): void;

  resetScroll(): void;

  maxPageRows(): number;

  maxPageItems(): number;

  /**
   * Returns true if the window is horizontal;
   * means the window only has a single row.
   * @returns {boolean}
   * @memberof Window_Selectable
   */
  isHorizontal(): boolean;

  bottomRow(): number;

  setBottomRow(row: number): void;

  /**
   * Creates a new rectangle based on itemWidth and itemHeight.
   * The rectangle is mainly used for positioning items within
   * the selectable window.
   * @param {number} index
   * @returns {Rectangle}
   * @memberof Window_Selectable
   */
  itemRect(index: number): Rectangle;

  /**
   * Creates a new rectangle based on itemWidth and itemHeight
   * The rectangle is used for positioning text within
   * the selectable window.
   * @param {number} index
   * @returns {Rectangle}
   * @memberof Window_Selectable
   */
  itemRectForText(index: number): Rectangle;

  setHelpWindow(helpWindow: Window_Help): void;

  /**
   * Shows the attached help window.
   *
   * @memberof Window_Selectable
   */
  showHelpWindow(): void;

  /**
   * Hides the attached help window.
   *
   * @memberof Window_Selectable
   */
  hideHelpWindow(): void;

  /**
   * Creates a new handler with the symbol as the handler name
   * and a method (JS function) bound to it.
   * @param {string} symbol
   * @param {*} method
   * @memberof Window_Selectable
   */
  setHandler(symbol: string, method: any): void;

  isHandled(symbol: string): boolean;

  callHandler(symbol: string): void;

  isOpenAndActive(): boolean;

  isCursorMovable(): boolean;

  /**
   * Moves the cursor down; if wrap is passed
   * as true, then it will return to the top when
   * at the end of the list.
   * @param {boolean} wrap
   * @memberof Window_Selectable
   */
  cursorDown(wrap: boolean): void;

  /**
   * Moves the cursor up; if wrap is passed
   * as true, then it will return to the bottom
   * when at the top of the list.
   * @param {boolean} wrap
   * @memberof Window_Selectable
   */
  cursorUp(wrap: boolean): void;

  cursorRight(wrap: boolean): void;

  cursorLeft(wrap: boolean): void;

  cursorPagedown(): void;

  cursorPageup(): void;

  scrollDown(): void;

  scrollUp(): void;

  updateArrows(): void;

  /**
   * Handles the processing of cursor movement.
   *
   * @memberof Window_Selectable
   */
  processCursorMove(): void;

  /**
   * Handles the process of attached handlers.
   *
   * @memberof Window_Selectable
   */
  processHandling(): void;

  /**
   * Handles the processing of the scroll wheel within
   * the window.
   * @memberof Window_Selectable
   */
  processWheel(): void;

  /**
   * Handles the processing of touch input.
   *
   * @memberof Window_Selectable
   */
  processTouch(): void;

  isTouchedInsideFrame(): boolean;

  onTouch(triggered: boolean): void;

  hitTest(x: number, y: number): number;

  isContentsArea(x: number, y: number): boolean;

  /**
   * Determines if touch ok is enabled as an option;
   * this means whether you can confirm the selection
   * of an item within the window with touch input.
   * @returns {boolean}
   * @memberof Window_Selectable
   */
  isTouchOkEnabled(): boolean;

  /**
   * Determines if ok is enabled as an option;
   * this means whether you can confirm selection
   * of an item within the window.
   * @returns {boolean}
   * @memberof Window_Selectable
   */
  isOkEnabled(): boolean;

  isCancelEnabled(): boolean;

  isOkTriggered(): boolean;

  isCancelTriggered(): boolean;

  processOk(): void;

  callOkHandler(): void;

  processCancel(): void;

  callCancelHandler(): void;

  processPageup(): void;

  processPagedown(): void;

  updateInputData(): void;

  updateCursor(): void;

  /**
   * Determines if the cursor is visible within
   * the window.
   * @returns {boolean}
   * @memberof Window_Selectable
   */
  isCursorVisible(): boolean;

  ensureCursorVisible(): void;

  callUpdateHelp(): void;

  updateHelp(): void;

  setHelpWindowItem(item: any): void;

  isCurrentItemEnabled(): boolean;

  /**
   * Draws all items within the window; this method
   * cals drawItem multiple times.
   * @memberof Window_Selectable
   */
  drawAllItems(): void;

  drawItem(index: number): void;

  clearItem(index: number): void;

  redrawItem(index: number): void;

  redrawCurrentItem(): void;

  /**
   * Refreshes the window contents.
   *
   * @memberof Window_Selectable
   */
  refresh(): void;
}

declare class Window_Command extends Window_Selectable {
  constructor(rect: Rectangle);

  initialize(rect: Rectangle): void;

  /**
   * Returns the width of the window;
   * default is 240.
   * @returns {number}
   * @memberof Window_Command
   */
  windowWidth(): number;

  /**
   * Returns the height of the window;
   * takes the visible rows and passes it to the fittingHeight method.
   * @returns {number}
   * @memberof Window_Command
   */
  windowHeight(): number;

  /**
   * Returns the number of visible rows within the window.
   *
   * @returns {number}
   * @memberof Window_Command
   */
  numVisibleRows(): number;

  /**
   * Returns the maximum number of items within the window.
   *
   * @returns {number}
   * @memberof Window_Command
   */
  maxItems(): number;

  /**
   * Clears the list of commands from the window;
   * this is useful for refreshing changing commands.
   * @memberof Window_Command
   */
  clearCommandList(): void;

  /**
   * Convenient method for overwriting and adding
   * commands with the addCommand method.
   * @memberof Window_Command
   */
  makeCommandList(): void;

  /**
   * Adds commands to the window list with the specified
   * parameters. The actual command can be found as an object.
   * @param {String} name
   * @param {String} symbol
   * @param {boolean} enabled
   * @param {(any | object)} [ext]
   * @memberof Window_Command
   */
  addCommand(name: string, symbol: string, enabled: boolean, ext?: any): void;

  /**
   * Returns the command name given an index.
   *
   * @param {number} index
   * @returns {String}
   * @memberof Window_Command
   */
  commandName(index: number): string;

  /**
   * Returns the command symbol given an index.
   *
   * @param {number} index
   * @returns {String}
   * @memberof Window_Command
   */
  commandSymbol(index: number): string;

  /**
   * Determines if the command is enabled;
   * checks the enabled property of the command.
   * @param {number} index
   * @returns {boolean}
   * @memberof Window_Command
   */
  isCommandEnabled(index: number): boolean;

  /**
   * Returns the command object at the current index.
   *
   * @returns {object}
   * @memberof Window_Command
   */
  currentData(): any;

  /**
   * Returns the command symbol at the current index.
   *
   * @returns {String}
   * @memberof Window_Command
   */
  currentSymbol(): string;

  /**
   * Returns the ext property of the command at the current index.
   *
   * @returns {(any | object)}
   * @memberof Window_Command
   */
  currentExt(): any;

  /**
   * Finds a command object and returns the index number based
   * on the symbol property.
   * @param {String} symbol
   * @returns {number}
   * @memberof Window_Command
   */
  findSymbol(symbol: string): boolean;

  /**
   * Selects a command object based on the symbol property.
   *
   * @param {String} symbol
   * @memberof Window_Command
   */
  selectSymbol(symbol: string): void;

  /**
   * Finds a command object and returns the index number
   * based on the ext property.
   * @param {(any | object)} ext
   * @returns {number}
   * @memberof Window_Command
   */
  findExt(ext: any): number;

  /**
   * Selects a command object based on the ext property.
   *
   * @param {(any | object)} ext
   * @memberof Window_Command
   */
  selectExt(ext: any): void;

  /**
   * Returns the text align of the commands;
   * possible values are: 'left', 'center', 'right'.
   * @returns {String}
   * @memberof Window_Command
   */
  itemTextAlign(): string;
}

/**
 * -----------------------------------------------------------------------------
 * Window_ActorCommand
 *
 * The window for selecting an actor's action on the battle screen.
 * @class Window_ActorCommand
 */
declare class Window_ActorCommand extends Window_Command {
  constructor();

  /**
   * Adds the attack command to the actor command window.
   *
   * @memberof Window_ActorCommand
   */
  addAttackCommand(): void;

  /**
   * Adds the skill command to the actor command window.
   *
   * @memberof Window_ActorCommand
   */
  addSkillCommands(): void;

  /**
   * Adds the guard command to the actor command window.
   *
   * @memberof Window_ActorCommand
   */
  addGuardCommand(): void;

  /**
   * Adds the item command to the actor command window.
   *
   * @memberof Window_ActorCommand
   */
  addItemCommand(): void;

  /**
   * Sets up the actor command window with a specified actor.
   *
   * @param {Game_Actor} actor
   * @memberof Window_ActorCommand
   */
  setup(actor: Game_Actor): void;

  selectLast(): void;
}

declare namespace rm.types {
  export const enum GaugeType {
    HP = 'hp',
    MP = 'mp',
    TP = 'tp',
    TIME = 'time',
  }
}

declare class Window_StatusBase extends Window_Selectable {
  constructor(rect: Rectangle);

  initialize(rect: Rectangle): void;

  /**
   * Loads all face images in MZ for party members.
   */
  loadFaceImages(): void;

  /**
   * Refreshes the window.
   */
  refesh(): void;

  hideAdditionalSprites(): void;

  /**
   * Places the game actor within the window.
   * @param actor
   * @param x
   * @param y
   */
  placeActorName(actor: Game_Actor, x: number, y: number): void;

  /**
   * Places the actor name within the status window.
   * @param actor
   * @param x
   * @param y
   */
  placeStateIcon(actor: Game_Actor, x: number, y: number): void;

  placeGauage(actor: Game_Actor, gaugeType: rm.types.GaugeType, x: number, y: number): void;

  /**
   * Creates an inner sprite using a sprite class of your choosing..
   * Should pass in a class without actually instantiating it.
   * @param key
   * @param spriteClass
   */
  createInnerSprite<T>(key: string, spriteClass: T): T;

  /**
   * Places the time gauages within the status window.
   * @param actor
   * @param x
   * @param y
   */
  placeTimeGauge(actor: Game_Actor, x: number, y: number): void;

  /**
   * Places the basic gauges on the window.
   * @param actor
   * @param x
   * @param y
   */
  placeBasicGauges(actor: Game_Actor, x: number, y: number): void;

  /**
   * Line height of the guage.
   * By default set to 24.
   * @return Int
   */
  guageLineHeight(): number;

  drawActorCharacter(actor: Game_Actor, x: number, y: number): void;

  drawActorFace(actor: Game_Actor, x: number, y: number, width: number, height: number): void;

  /**
   * Draws the actor name if the width isn't passed in
   * defaults to 168.
   * @param actor
   * @param x
   * @param y
   * @param width
   */
  drawActorName(actor: Game_Actor, x: number, y: number, width?: number): void;

  /**
   * Draws the actor class name.
   * If width isn't passed in, defaults to 168px.
   * @param actor
   * @param x
   * @param y
   * @param width
   */
  drawActorClass(actor: Game_Actor, x: number, y: number, width?: number): void;

  /**
   * Draws the actor nickname.
   * If width isn't passed, defaults to 270px.
   * @param actor
   * @param x
   * @param y
   * @param width
   */
  drawActorNickname(actor: Game_Actor, x: number, y: number, width: number): void;

  /**
   * Draws the actor level.
   * @param actor
   * @param x
   * @param y
   */
  drawActorLevel(actor: Game_Actor, x: number, y: number): void;

  /**
   * Draws the actor icons.
   * If width isn't passed in defaults to 144px.
   * @param actor
   * @param x
   * @param y
   * @param width
   */
  drawActorIcons(actor: Game_Actor, x: number, y: number, width?: number): void;

  /**
   * Draws the actor simple status.
   * Using all of the drawing methods above.
   * @param actor
   * @param x
   * @param y
   */
  drawActorSimpleStatus(actor: Game_Actor, x: number, y: number): void;

  /**
   * Returns the name of the actor equip slot.
   * @param actor
   * @param index
   */
  actorSlotName(actor: Game_Actor, index: number): string;
}

/**
 * -----------------------------------------------------------------------------
 * Window_BattleStatus
 *
 * The window for displaying the status of party members on the battle screen.
 * @class Window_BattleStatus
 */
declare class Window_BattleStatus extends Window_StatusBase {
  constructor(rect: Rectangle);

  initialize(rect: Rectangle): void;
}

/**
 * -----------------------------------------------------------------------------
 * Window_BattleActor
 *
 * The window for selecting a target actor on the battle screen.
 * @class Window_BattleActor
 */
declare class Window_BattleActor extends Window_BattleStatus {
  constructor(rect: Rectangle);

  initialize(rect: Rectangle): void;

  /**
   * Selects an actor within the battle actor window.
   *
   * @param {number} index
   * @memberof Window_BattleActor
   */
  select(index: number): void;

  /**
   * Returns the current selected actor.
   *
   * @returns {Game_Actor}
   * @memberof Window_BattleActor
   */
  actor(): Game_Actor;
}

/**
 * -----------------------------------------------------------------------------
 * Window_BattleEnemy
 *
 * The window for selecting a target enemy on the battle screen.
 * @class Window_BattleEnemy
 */
declare class Window_BattleEnemy extends Window_Selectable {
  constructor(rect: Rectangle);

  initialize(rect: Rectangle): void;

  /**
   * Returns the window width.
   *
   * @returns {number}
   * @memberof Window_BattleEnemy
   */
  windowWidth(): number;

  /**
   * Returns the window height.
   *
   * @returns {number}
   * @memberof Window_BattleEnemy
   */
  windowHeight(): number;

  /**
   * Returns the number of visible rows.
   *
   * @returns {number}
   * @memberof Window_BattleEnemy
   */
  numVisibleRows(): number;

  /**
   * Returns the current enemy.
   *
   * @returns {Game_Enemy}
   * @memberof Window_BattleEnemy
   */
  enemy(): Game_Enemy;

  /**
   * Returns the current index selected.
   *
   * @returns {number}
   * @memberof Window_BattleEnemy
   */
  enemyIndex(): number;

  /**
   * Selects a specified enemy using the index.
   *
   * @param {number} index
   * @memberof Window_BattleEnemy
   */
  select(index: number): void;
}

declare class Window_ItemList extends Window_Selectable {
  constructor(rect: Rectangle);

  initialize(rect: Rectangle): void;

  /**
   * Returns the item at the current index of the window.
   * @return Item
   */
  item(): rm.types.Item;

  /**
   * Returns item at the specific index.
   * @return Null<Item>
   */
  itemAt(): rm.types.Item;

  needsNumber(): boolean;

  selectLast(): void;

  makeItemList(): void;

  numberWidth(): number;
}

/**
 * -----------------------------------------------------------------------------
 * Window_BattleItem
 *
 * The window for selecting an item to use on the battle screen.
 * @class Window_BattleItem
 */
declare class Window_BattleItem extends Window_ItemList {
  constructor(rect: Rectangle);

  initialize(rect: Rectangle): void;

  includes(item: rm.types.UsableItem): boolean;
}

/**
 * -----------------------------------------------------------------------------
 * Window_BattleLog
 *
 * The window for displaying battle progress. No frame is displayed, but it is
 * handled as a window for convenience.
 * @class Window_BattleLog
 */
declare class Window_BattleLog extends Window_Base {
  constructor(rect: Rectangle);

  initialize(rect: Rectangle): void;

  setSpriteset(spriteset: Spriteset_Battle): void;

  windowWidth(): number;

  windowHeight(): number;

  maxLines(): number;

  createBackBitmap(): void;

  createBackSprite(): void;

  numLines(): number;

  messageSpeed(): number;

  isBusy(): boolean;

  updateWait(): boolean;

  updateWaitCount(): boolean;

  updateWaitMode(): boolean;

  setWaitMode(waitMode: string): void;

  callNextMethod(): void;

  isFastForward(): boolean;

  push(methodName: string, args: any[]): void;

  clear(): void;

  wait(): void;

  waitForEffect(): void;

  waitForMovement(): void;

  addText(text: string): void;

  pushBaseLine(): void;

  popBaseLine(): void;

  waitForNewLine(): void;

  popupDamage(target: Game_Battler): void;

  performActionStart(subject: Game_Battler, action: Game_Action): void;

  performAction(subject: Game_Battler, action: Game_Action): void;

  performActionEnd(subject: Game_Battler): void;

  performDamage(target: Game_Battler): void;

  performMiss(target: Game_Battler): void;

  performRecovery(target: Game_Battler): void;

  performEvasion(target: Game_Battler): void;

  performMagicEvasion(target: Game_Battler): void;

  performCounter(target: Game_Battler): void;

  performReflection(target: Game_Battler): void;

  performSubstitute(substitute: Game_Battler, target: Game_Battler): void;

  performCollapse(target: Game_Battler): void;

  showAnimation(subject: Game_Battler, targets: Game_Battler, animationId: number): void;

  showAttackAnimation(subject: Game_Battler, targets: Game_Battler): void;

  showActorAttackAnimation(subject: Game_Battler, targets: Game_Battler): void;

  showEnemyAttackAnimation(subject: Game_Battler, targets: Game_Battler): void;

  showNormalAnimation(targets: Game_Battler, animationId: number, mirror: boolean): void;

  animationBaseDelay(): number;

  animationNextDelay(): number;

  drawBackground(): void;

  backRect(): Rectangle;

  backColor(): string;

  backPaintOpacity(): number;

  drawLineText(index: number): void;

  startTurn(): void;

  startAction(subject: Game_Battler, action: Game_Action, targets: Game_Battler[]): void;

  endAction(subject: Game_Battler): void;

  displayCurrentState(subject: Game_Battler): void;

  displayRegeneration(subject: Game_Battler): void;

  displayAction(subject: Game_Battler, item: rm.types.UsableItem): void;

  displayCounter(target: Game_Battler): void;

  displayReflection(target: Game_Battler): void;

  displaySubstitute(substitute: Game_Battler, target: Game_Battler): void;

  displayActionResults(subject: Game_Battler, targt: Game_Battler): void;

  displayFailure(target: Game_Battler): void;

  displayCritical(target: Game_Battler): void;

  displayDamage(target: Game_Battler): void;

  displayMiss(target: Game_Battler): void;

  displayEvasion(target: Game_Battler): void;

  displayHpDamage(target: Game_Battler): void;

  displayMpDamage(target: Game_Battler): void;

  displayTpDamage(target: Game_Battler): void;

  displayAffectedStatus(target: Game_Battler): void;

  displayAutoAffectedStatus(target: Game_Battler): void;

  displayChangedStates(target: Game_Battler): void;

  displayAddedStates(target: Game_Battler): void;

  displayRemovedStates(target: Game_Battler): void;

  displayChangedBuffs(target: Game_Battler): void;

  /**
   *
   * @param target
   * @param buffs list of integers representing buff Ids
   * @param fmt
   */
  displayBuffs(target: Game_Battler, buffs: number[], fmt: string): void;

  makeHpDamageText(target: Game_Battler): void;

  makeMpDamageText(target: Game_Battler): string;

  makeTpDamageText(target: Game_Battler): string;
}

declare namespace rm.types {
  export const enum SkillTypeIdA {
    base = -1,
  }
}

/**
 * -----------------------------------------------------------------------------
 * Window_SkillList
 *
 * The window for selecting a skill on the skill screen.
 * @class Window_SkillList
 */
declare class Window_SkillList extends Window_Selectable {
  constructor(rect: Rectangle);

  _actor: Game_Actor;
  /**
   * Returns the Skill Type Id, which is an Int.
   */
  _stypeId: rm.types.SkillTypeIdA;
  _data: rm.types.Skill[];

  initialize(rect: Rectangle): void;

  /**
   * Returns skill at the specified index.
   * @param index
   * @return Null<Skill>
   */
  itemAt(index: number): rm.types.Skill;

  /**
   * Sets the current actor of the skill list window.
   *
   * @param {Game_Actor} actor
   * @memberof Window_SkillList
   */
  setActor(actor: Game_Actor): void;

  /**
   * Sets the skill type id of the skill list window.
   *
   * @param {number} stypeId - Integer
   * @memberof Window_SkillList
   */
  setStypeId(stypeId: rm.types.SkillTypeIdA): void;

  /**
   * Returns the current skill at the window index
   * loaded from the databse.
   *
   * @returns {RPG.Skill}
   * @memberof Window_SkillList
   */
  item(): rm.types.Skill;

  /**
   * Returns true if the given skill is included.
   *
   * @param {RPG.Skill} item
   * @returns {boolean}
   * @memberof Window_SkillList
   */
  includes(item: rm.types.Skill): boolean;

  /**
   * Returns true if the given skill is enabled.
   *
   * @param {RPG.Skill} item
   * @returns {boolean}
   * @memberof Window_SkillList
   */
  isEnabled(item: rm.types.Skill): boolean;

  /**
   * Creates the item list.
   *
   * @memberof Window_SkillList
   */
  makeItemList(): void;

  selectLast(): void;

  costWidth(): number;

  drawSkillCost(skill: rm.types.Skill, x: number, y: number, width: number): void;
}

/**
 * -----------------------------------------------------------------------------
 * Window_BattleSkill
 *
 * The window for selecting a skill to use on the battle screen.
 * @class Window_BattleSkill
 */
declare class Window_BattleSkill extends Window_SkillList {
  constructor(rect: Rectangle);

  initialize(rect: Rectangle): void;
}

/**
 * -----------------------------------------------------------------------------
 * Window_ChoiceList
 *
 * The window used for the event command [Show Choices].
 * @class Window_ChoiceList
 */
declare class Window_ChoiceList extends Window_Command {
  constructor();

  initialize(): void;

  setMessageWindow(messageWindow: Window_Message): void;

  windowY(): number;

  windowX(): number;

  needsCancelButton(): boolean;

  maxLines(): number;

  start(): void;

  selectDefault(): void;

  updatePlacement(): void;

  updateBackground(): void;

  maxChoiceWidth(): number;

  textWidthEx(text: string): number;
}

/**
 * -----------------------------------------------------------------------------
 * Window_DebugEdit
 *
 * The window for displaying switches and variables on the debug screen.
 * @class Window_DebugEdit
 */
declare class Window_DebugEdit extends Window_Selectable {
  constructor(x: number, y: number, width: number);

  itemName(dataId: number): string;

  itemStatus(dataId: string): string;

  setMode(mode: string): void;

  setTopId(id: number): void;

  currentId(): number;

  updateSwitch(): void;

  updateVariable(): void;
}

/**
 * -----------------------------------------------------------------------------
 * Window_DebugRange
 *
 * The window for selecting a block of switches/variables on the debug screen.
 * @class Window_DebugRange
 */
declare class Window_DebugRange extends Window_Selectable {
  constructor(x: number, y: number);

  windowWidth(): number;

  windowHeight(): number;

  mode(): string;

  topId(): number;

  setEditWindow(editWindow: Window_DebugEdit): void;
}

declare class Window_HorzCommand extends Window_Command {
  constructor(x: number, y: number);
}

declare class Window_EquipCommand extends Window_HorzCommand {
  protected constructor();
}

/**
 * -----------------------------------------------------------------------------
 * Window_EquipItem
 *
 * The window for selecting an equipment item on the equipment screen.
 * @class Window_EquipItem
 */
declare class Window_EquipItem {
  constructor(x: number, y: number, width: number, height: number);

  setActor(actor: Game_Actor): void;

  setSlotId(slotId: number): void;

  includes(item: rm.types.EquipItem): boolean;

  isEnabled(item: rm.types.EquipItem): boolean;

  setStatusWindow(statusWindow: Window_EquipStatus): void;
}

/**
 * -----------------------------------------------------------------------------
 * Window_EquipSlot
 *
 * The window for selecting an equipment slot on the equipment screen.
 * @class Window_EquipSlot
 */
declare class Window_EquipSlot extends Window_Selectable {
  constructor(x: number, y: number, width: number, height: number);

  initialize(x: number, y: number, width: number, height: number): void;

  /**
   * Sets the current game actor.
   *
   * @param {Game_Actor} actor
   * @memberof Window_EquipSlot
   */
  setActor(actor: Game_Actor): void;

  /**
   * Returns the current equip item.
   *
   * @returns {RPG.EquipItem}
   * @memberof Window_EquipSlot
   */
  item(): rm.types.EquipItem;

  /**
   * Returns the name of the slot at the specified index.
   *
   * @param {number} index
   * @returns {string}
   * @memberof Window_EquipSlot
   */
  slotName(index: number): string;

  /**
   * Returns true if the current slot is enabled.
   *
   * @param {number} index
   * @returns {boolean}
   * @memberof Window_EquipSlot
   */
  isEnabled(index: number): boolean;

  /**
   * Sets the status window within the equip slot window.
   *
   * @param {Window_EquipStatus} statusWindow
   * @memberof Window_EquipSlot
   */
  setStatusWindow(statusWindow: Window_EquipStatus): void;

  /**
   * Sets the item window within the equip slot window.
   *
   * @param {Window_EquipItem} itemWindow
   * @memberof Window_EquipSlot
   */
  setItemWindow(itemWindow: Window_EquipItem): void;
}

declare class Window_EquipStatus extends Window_Base {
  protected constructor();
}

/**
 * -----------------------------------------------------------------------------
 * Window_EventItem
 *
 * The window used for the event command [Select Item].
 * @class Window_EventItem
 */
declare class Window_EventItem extends Window_ItemList {
  constructor(messageWindow: Window_Message);

  /**
   * Returns the height off the window.
   *
   * @returns {number}
   * @memberof Window_EventItem
   */
  windowHeight(): number;

  /**
   * Returns the number of visible rows.
   *
   * @returns {number}
   * @memberof Window_EventItem
   */
  numVisibleRows(): number;

  /**
   * Starts the event item window.
   *
   * @memberof Window_EventItem
   */
  start(): void;

  updatePlacement(): void;

  includes(item: rm.types.BaseItem): boolean;

  isEnabled(item: rm.types.BaseItem): boolean;

  onOk(): void;

  onCancel(): void;
}

/**
 * -----------------------------------------------------------------------------
 * Window_GameEnd
 *
 * The window for selecting "Go to Title" on the game end screen.
 * @class Window_GameEnd
 */
declare class Window_GameEnd extends Window_Base {
  constructor();

  updatePlacement(): void;
}

declare class Window_Gold extends Window_Base {
  constructor(x: number, y: number);

  /**
   * Returns the $gameParty gold as a number.
   *
   * @returns {number}
   * @memberof Window_Gold
   */
  value(): number;

  /**
   * Returns the RPGMakerMV database currency
   * as a string.
   * @returns {string}
   * @memberof Window_Gold
   */
  currencyUnit(): string;
}

declare class Window_Help extends Window_Base {
  constructor(rect: Rectangle);

  /**
   * Sets the _text property of the window;
   * this text will be displayed within the window.
   * @param {string} text
   * @memberof Window_Help
   */
  setText(text: string): void;

  clear(): void;

  /**
   * Sets the current item of the help window.
   *
   * @param {RPG.BaseItem} item
   * @memberof Window_Help
   */
  setItem(item: rm.types.BaseItem): void;
}

declare class Window_ItemCategory extends Window_HorzCommand {
  protected constructor();
}

/**
 * -----------------------------------------------------------------------------
 * Window_MapName
 *
 * The window for displaying the map name on the map screen.
 * @class Window_MapName
 */
declare class Window_MapName extends Window_Base {
  constructor();

  /**
   * Returns the window width.
   *
   * @returns {number}
   * @memberof Window_MapName
   */
  windowWidth(): number;

  /**
   * Returns the window height.
   *
   * @returns {number}
   * @memberof Window_MapName
   */
  windowHeight(): number;

  updateFadeIn(): void;

  updateFadeOut(): void;

  /**
   * Windows the map name window.
   *
   * @memberof Window_MapName
   */
  refresh(): void;

  /**
   * Draws the background of the map name window.
   *
   * @param {number} x
   * @param {number} y
   * @param {number} width
   * @param {number} height
   * @memberof Window_MapName
   */
  drawBackground(x: number, y: number, width: number, height: number): void;
}

/**
 * -----------------------------------------------------------------------------
 * Window_MenuStatus
 *
 * The window for displaying party member status on the menu screen.
 * @class Window_MenuStatus
 * @extends {Window_Selectable}
 */
declare class Window_MenuStatus extends Window_Selectable {
  /**
   * Creates an instance of Window_MenuStatus.
   * @param {number} x
   * @param {number} y
   * @memberof Window_MenuStatus
   */
  constructor(x: number, y: number);

  /**
   * Window width.
   * @return Int
   */
  windowWidth(): number;

  /**
   * Window height.
   * @return Int
   */
  windowHeight(): number;

  /**
   * Returns the height of each item (actor status) in the main menu
   * window.
   * @returns {number}
   * @memberof Window_MenuStatus
   */
  itemHeight(): number;

  /**
   * Returns the number of visible rows in
   * menu status.
   * @returns {number}
   * @memberof Window_MenuStatus
   */
  numVisibleRows(): number;

  /**
   * Loads the images for the main menu status window.
   *
   * @memberof Window_MenuStatus
   */
  loadImages(): void;

  /**
   * Draws the item background at the given index.
   *
   * @param {number} index
   * @memberof Window_MenuStatus
   */
  drawItemBackground(index: number): void;

  drawItemImage(index: number): void;

  drawItemStatus(index: number): void;

  selectLast(): void;

  /**
   * Determines if the window is in formation mode;
   * if true, the player can select characters to swap
   * positions with.
   * @returns {boolean}
   * @memberof Window_MenuStatus
   */
  formationMode(): boolean;

  /**
   * Sets the formation mode to true or false.
   *
   * @param {boolean} formationMode
   * @memberof Window_MenuStatus
   */
  setFormationMode(formationMode: boolean): void;

  pendingIndex(): number;

  setPendingIndex(index: number): void;
}

declare class Window_MenuActor extends Window_MenuStatus {
  constructor();

  initialize(): void;

  selectForItem(item: rm.types.BaseItem): void;
}

declare class Window_MenuCommand extends Window_Command {
  constructor(x: number, y: number);

  /**
   * Adds the standard game commands to the
   * RPGMakerMV main menu.
   * @memberof Window_MenuCommand
   */
  addMainCommands(): void;

  /**
   * Adds the standard formation command to the
   * RPGMakerMV main menu.
   * @memberof Window_MenuCommand
   */
  addFormationCommand(): void;

  /**
   * Adds any user created commands to the
   * RPGMakerMV main menu.
   * @memberof Window_MenuCommand
   */
  addOriginalCommands(): void;

  /**
   * Adds the save command to the
   * RPGMakerMV main menu.
   * @memberof Window_MenuCommand
   */
  addSaveCommand(): void;

  addGameEndCommand(): void;

  /**
   * Checks if the standard game commands for the menu
   * are needed based on database system options.
   * @param {string} name
   * @returns {boolean}
   * @memberof Window_MenuCommand
   */
  needsCommand(name: string): boolean;

  /**
   * Determines if the main commands are enabled;
   * this is based on the settings in the database.
   * @returns {boolean}
   * @memberof Window_MenuCommand
   */
  areMainCommandsEnabled(): boolean;

  /**
   * Determines if the option command is enabled;
   * based on the setting in the database.
   * @returns {boolean}
   * @memberof Window_MenuCommand
   */
  isOptionsEnabled(): boolean;

  /**
   * Determines if the save command is enabled;
   * based on the setting in the database.
   * @returns {boolean}
   * @memberof Window_MenuCommand
   */
  isSaveEnabled(): boolean;

  /**
   * Selects the last command in menu.
   *
   * @memberof Window_MenuCommand
   */
  selectLast(): void;

  static initCommandPosition(): void;

  static _lastCommandSymbol: any;
}

declare class Window_Message extends Window_Base {
  protected constructor();

  /**
   * Returns the sub windows attached to the message window.
   *
   * @returns {Array<Window_Base>}
   * @memberof Window_Message
   */
  subWindows(): Window_Base[];

  /**
   * Creates the sub windows for the message window.
   *
   * @memberof Window_Message
   */
  createSubWindows(): void;

  /**
   * Returns the width of the window.
   *
   * @returns {number}
   * @memberof Window_Message
   */
  windowWidth(): number;

  /**
   * Returns the height of the window.
   *
   * @returns {number}
   * @memberof Window_Message
   */
  windowHeight(): number;

  /**
   * Returns the number of visible rows within the message window.
   *
   * @returns {number}
   * @memberof Window_Message
   */
  numVisibleRows(): number;

  checkToNotClose(): void;

  /**
   * Returns true if the message window can start.
   *
   * @returns {boolean}
   * @memberof Window_Message
   */
  canStart(): boolean;

  /**
   * Starts the displaying of the message within the message window.
   *
   * @memberof Window_Message
   */
  startMessage(): void;

  /**
   * Updates the placement of the message window.
   *
   * @memberof Window_Message
   */
  updatePlacement(): void;

  /**
   * Clears the message window flags for
   * fast text, pause skip, and line show fast
   */
  clearFlags(): void;

  /**
   * Sets the background type of the window in terms
   * of transparency.
   * 0,
   * 1,
   * 2
   */
  setBackgroundType(backgroundType: number): void;

  /**
   * Processes the escape characters in the message window.
   * @param code
   * @param textState
   */
  processEscapeCharacter(code: string, textState: string): void;

  /**
   * Starts the wait count for the message window.
   * @param count
   */
  startWait(count: number): void;

  /**
   * Starts the pause for the window.
   * Sets the wait count to 10 and pauses
   * the window.
   */
  startPause(): void;

  /**
   * Updates the background of the message window.
   *
   * @memberof Window_Message
   */
  updateBackground(): void;

  /**
   * Terminates the message and closes the gold and message window.
   *
   * @memberof Window_Message
   */
  terminateMessage(): void;

  /**
   * Updates the wait of the message window.
   *
   * @returns {Bool}
   * @memberof Window_Message
   */
  updateWait(): boolean;

  updateLoading(): boolean;

  /**
   * Updates input when the message window is processing.
   *
   * @returns {Bool}
   * @memberof Window_Message
   */
  updateInput(): boolean;

  /**
   * Returns true if any sub window is active.
   *
   * @returns {Bool}
   * @memberof Window_Message
   */
  isAnySubWindowActive(): boolean;

  /**
   * Updates the message.
   *
   * @returns {Bool}
   * @memberof Window_Message
   */
  updateMessage(): boolean;

  /**
   * Handler for when there is no text left to display within
   * the message window.
   * @memberof Window_Message
   */
  onEndOfText(): void;

  startInput(): boolean;

  /**
   * Returns true if the ok or cancel inputs have been triggered
   * multiple times.
   * @returns {Bool}
   * @memberof Window_Message
   */
  isTriggered(): boolean;

  /**
   * Returns true if the message window still has text
   * and settings have not changed.
   * @returns {Bool}
   * @memberof Window_Message
   */
  doesContinue(): boolean;

  /**
   * Returns true if the message window settings have been changed.
   *
   * @returns {Bool}
   * @memberof Window_Message
   */
  areSettingsChanged(): boolean;

  updateShowFast(): void;

  newPage(textState: rm.types.TextState): void;

  loadMessageFace(): void;

  drawMessageFace(): void;

  newLineX(): number;

  processNewLine(textState: rm.types.TextState): void;

  processNewPage(textState: rm.types.TextState): void;
}

/**
 * The window for displaying a speaker name above
 * the message window.
 */
declare class Window_NameBox extends Window_Base {
  protected constructor();

  /**
   * Sets the current message window.
   * @param messageWindow
   */
  setMessageWindow(messageWindow: Window_Message): void;

  /**
   * Sets the name in the message window box.
   * @param name
   */
  setName(name: string): void;

  /**
   * Clears the name in the message box.
   */
  clear(): void;

  /**
   * Starts the name box window.
   */
  start(): void;

  /**
   * Updates the placement of the name box window.
   */
  updatePlacement(): void;

  /**
   * Updates the name box background
   * to match game message background.
   * ```js
   *  this.setBackgroundType($gameMessage.background());
   * ```
   */
  updateBackground(): void;

  /**
   * Returns the window width based on the text size.
   * @returns Int
   */
  windowWidth(): number;

  /**
   * Returns the window height based on
   * the fitting height.
   * @return Int
   */
  windowHeight(): number;

  /**
   * Refreshes the window and
   * redraws the text.
   * ```js
   * const rect = this.baseTextRect();
   * this.contents.clear();
   * this.drawTextEx(this._name, rect.x, rect.y, rect.width);
   * ```
   * @return Int
   */
  refresh(): void;
}

/**
 * -----------------------------------------------------------------------------
 * Window_NameEdit
 *
 * The window for editing an actor's name on the name input screen.
 * @class Window_NameEdit
 */
declare class Window_NameEdit {
  /**
   * Creates an instance of Window_NameEdit.
   * @param {Game_Actor} actor
   * @param {number} maxLength
   * @memberof Window_NameEdit
   */
  constructor(actor: Game_Actor, maxLength: number);

  /**
   * Returns the window width.
   *
   * @returns {number}
   * @memberof Window_NameEdit
   */
  windowWidth(): number;

  /**
   * Returns the window height.
   *
   * @returns {number}
   * @memberof Window_NameEdit
   */
  windowHeight(): number;

  restoreDefault(): boolean;

  add(ch: string): boolean;

  back(): boolean;

  /**
   * Returns the width of the character face.
   *
   * @returns {number}
   * @memberof Window_NameEdit
   */
  faceWidth(): number;

  /**
   * Returns the width of a character.
   *
   * @returns {number}
   * @memberof Window_NameEdit
   */
  charWidth(): number;

  left(): number;

  underlineRect(index: number): Rectangle;

  /**
   * Returns the color of the underline as a css color String.
   *
   * @returns {String}
   * @memberof Window_NameEdit
   */
  underlineColor(): string;

  /**
   * Draws the underline at the given index of the window.
   *
   * @param {number} index
   * @memberof Window_NameEdit
   */
  drawUnderline(index: number): void;

  /**
   * Draws a character within the window at the specified index.
   *
   * @param {number} index
   * @memberof Window_NameEdit
   */
  drawChar(index: number): void;

  /**
   * Refreshes the window contents.
   *
   * @memberof Window_NameEdit
   */
  refresh(): void;
}

/**
 * -----------------------------------------------------------------------------
 * Window_NameInput
 *
 * The window for selecting text characters on the name input screen.
 * @class Window_NameInputt
 */
declare class Window_NameInput extends Window_Selectable {
  constructor(editWindow: Window_NameEdit);

  /**
   * Returns the window height.
   *
   * @returns {number}
   * @memberof Window_NameInput
   */
  windowHeight(): number;

  /**
   * Returns the table of characters to input.
   *
   * @returns {Array<Array<String>>}
   * @memberof Window_NameInput
   */
  table(): string[][];

  character(): string;

  isPageChange(): boolean;

  isOk(): boolean;

  processJump(): void;

  processBack(): void;

  onNameAdd(): void;

  onNameOk(): void;

  static LATIN1: string[];
  static LATIN2: string[];
  static RUSSIA: string[];
  static JAPAN1: string[];
  static JAPAN2: string[];
  static JAPAN3: string[];
}

declare class Window_NumberInput extends Window_Selectable {
  constructor(messageWindow: Window_Message);

  start(): void;

  updatePlacement(): void;

  /**
   * Returns the window width.
   *
   * @returns {number}
   * @memberof Window_NumberInput
   */
  windowWidth(): number;

  /**
   * Returns the window height.
   *
   * @returns {number}
   * @memberof Window_NumberInput
   */
  windowHeight(): number;

  /**
   * Returns the item width.
   *
   * @returns {number}
   * @memberof Window_NumberInput
   */
  itemWidth(): number;

  /**
   * Creates the number input window buttons.
   *
   * @memberof Window_NumberInput
   */
  createButtons(): void;

  /**
   * Places the number input window buttons.
   *
   * @memberof Window_NumberInput
   */
  placeButtons(): void;

  updateButtonsVisiblity(): void;

  showButtons(): void;

  hideButtons(): void;

  buttonY(): number;

  processDigitChange(): void;

  changeDigit(up: boolean): void;

  onButtonUp(): void;

  onButtonDown(): void;

  onButtonOk(): void;
}

declare class Window_Options extends Window_Command {
  constructor();

  updatePlacement(): void;

  addGeneralOptions(): void;

  addVolumeOptions(): void;

  statusWidth(): number;

  statusText(index: number): string;

  isVolumeSymbol(symbol: string): boolean;

  booleanStatusText(value: boolean): string;

  volumeStatusText(value: number): string;

  volumeOffset(): number;

  changeValue(symbol: string, value: boolean | number): void;

  getConfigValue(symbol: string): boolean;

  setConfigValue(symbol: string, volume: boolean | number): void;
}

/**
 * -----------------------------------------------------------------------------
 * Window_PartyCommand
 *
 * The window for selecting whether to fight or escape on the battle screen.
 * @class Window_PartyCommand
 */
declare class Window_PartyCommand extends Window_Command {
  constructor();

  setup(): void;
}

declare namespace rm.windows {
  export type Info = {
    key: string;
  };
}

declare class Window_SavefileList extends Window_Selectable {
  constructor(rect: Rectangle);

  /**
   * Whether ornot the auto save feature is enabled
   *
   * @default {Bool} false
   * @memberof Window_SavefileList
   */
  _autosave: boolean;

  /**
   * Sets the mode of the save file window.
   *
   * @param {string} mode
   * @memberof Window_SavefileList
   */
  setMode(mode: string): void;

  /**
   * Returns the maximum number of visible items.
   *
   * @returns {number}
   * @memberof Window_SavefileList
   */
  maxVisibleItems(): number;

  itemHeight(): number;

  /**
   * Draws the file id at the specified x and y coordinates.
   *
   * @param {number} id
   * @param {number} x
   * @param {number} y
   * @memberof Window_SavefileList
   */
  drawFileId(id: number, x: number, y: number): void;

  drawContents(info: rm.windows.Info, rect: Rectangle, valid: boolean): void;

  drawGameTitle(info: rm.windows.Info, x: number, y: number, width: number): void;

  drawPartyCharacters(info: rm.windows.Info, x: number, y: number): void;

  /**
   * Draws the current playtime at the specified x and y coordinates within the given
   * width.
   * @param {{key: string}} info
   * @param {number} x
   * @param {number} y
   * @param {number} width
   * @memberof Window_SavefileList
   */
  drawPlaytime(info: rm.windows.Info, x: number, y: number, width: number): void;
}

/**
 * -----------------------------------------------------------------------------
 * Window_ScrollText
 *
 * The window for displaying scrolling text. No frame is displayed, but it
 * is handled as a window for convenience.
 * @class Window_ScrollText
 */
declare class Window_ScrollText extends Window_Base {
  constructor(rect: Rectangle);

  initialize(rect: Rectangle): void;

  /**
   * Starts the displaying of a message in the scroll text window.
   *
   * @memberof Window_ScrollText
   */
  startMessage(): void;

  refresh(): void;

  updateMessage(): void;

  scrollSpeed(): number;

  /**
   * Returns true if the scene is in fast forward mode.
   *
   * @returns {boolean}
   * @memberof Window_ScrollText
   */
  isFastForward(): boolean;

  /**
   * Returns the fast forward rate of the scroll text window.
   *
   * @returns {number}
   * @memberof Window_ScrollText
   */
  fastForwardRate(): number;

  /**
   * Terminates the message and the scroll text window is hidden.
   *
   * @memberof Window_ScrollText
   */
  terminateMessage(): void;
}

/**
 * -----------------------------------------------------------------------------
 * Window_ShopBuy
 *
 * The window for selecting an item to buy on the shop screen.
 * @class Window_ShopBuy
 */
declare class Window_ShopBuy extends Window_Selectable {
  /**
   * Creates an instance of Window_ShopBuy.
   * @param {number} x
   * @param {number} y
   * @param {number} height
   * @param {Array<Array<any>>} shopGoods
   * @memberof Window_ShopBuy
   */
  constructor(x: number, y: number, height: number, shopGoods: any[][]);

  /**
   * Returns the width of the window.
   *
   * @returns {number}
   * @memberof Window_ShopBuy
   */
  windowWidth(): number;

  /**
   * Returns the current item of the window.
   *
   * @returns {RPG.BaseItem}
   * @memberof Window_ShopBuy
   */
  item(): rm.types.BaseItem;

  setMoney(money: number): void;

  /**
   * Returns the p rice of an item
   * @param item
   * @return Int
   */
  price(item: rm.types.BaseItem): number;

  /**
   * Checks if the current item is enabled (can be bought/sold).
   *
   * @param {RPG.BaseItem} item
   * @returns {boolean}
   * @memberof Window_ShopBuy
   */
  isEnabled(item: rm.types.BaseItem): boolean;

  /**
   * Creates a list of items for the shop window.
   *
   * @memberof Window_ShopBuy
   */
  makeItemList(): void;

  /**
   * Sets the status window for the shop buy window.
   * @param statusWindow
   */
  setStatusWindow(statusWindow: Window_ShopStatus): void;
}

/**
 * -----------------------------------------------------------------------------
 * Window_ShopCommand
 *
 * The window for selecting buy/sell on the shop screen.
 * @class Window_ShopCommand
 */
declare class Window_ShopCommand extends Window_HorzCommand {
  protected constructor();

  _purchaseOnly: boolean;

  /**
   * Determines the width of the shop buy/sell window;
   * also determines if the shop is purchase only.
   * @param {number} width
   * @param {boolean} purchaseOnly
   * @memberof Window_ShopCommand
   */
  initialize(width: number, purchaseOnly: boolean): void;
}

/**
 * -----------------------------------------------------------------------------
 * Window_ShopNumber
 *
 * The window for inputting quantity of items to buy or sell on the shop
 * screen.
 * @class Window_ShopNumber
 */
declare class Window_ShopNumber extends Window_Selectable {
  /**
   * Creates an instance of Window_ShopNumber.
   * @param {number} x
   * @param {number} y
   * @param {number} height
   * @memberof Window_ShopNumber
   */
  constructor(x: number, y: number, height: number);

  /**
   * Returns the width of the window.
   *
   * @returns {number}
   * @memberof Window_ShopNumber
   */
  windowWidth(): number;

  number(): number;

  setup(item: rm.types.BaseItem, max: number, price: number): void;

  setCurrencyUnit(currencyUnit: string): void;

  createButtons(): void;

  placeButtons(): void;

  updateButtonsVisiblity(): void;

  /**
   * Shows the quantity input buttons.
   *
   * @memberof Window_ShopNumber
   */
  showButtons(): void;

  /**
   * Hides the quantity input buttons.
   *
   * @memberof Window_ShopNumber
   */
  hideButtons(): void;

  drawMultiplicationSign(): void;

  /**
   * Draws the number.
   */
  drawNumber(): void;

  /**
   * Draws the total price of the selected
   * quantity of item.
   * @memberof Window_ShopNumber
   */
  drawTotalPrice(): void;

  /**
   * Item y position.
   * @return Int
   */
  itemY(): number;

  /**
   * Y position of the price.
   * @return Int
   */
  priceY(): number;

  /**
   * Y position of the button.
   * @return Int
   */
  buttonY(): number;

  /**
   * Returns the width of the cursor.
   *
   * @returns {number}
   * @memberof Window_ShopNumber
   */
  cursorWidth(): number;

  /**
   * Cursor x position.
   * @return Int
   */
  cursorX(): number;

  /**
   * The max number of digits for the shop number display
   * window.
   */
  maxDigits(): number;

  /**
   * Processes the change in quantity.
   *
   * @memberof Window_ShopNumber
   */
  processNumberChange(): void;

  /**
   * Changes the quantity, given a number.
   *
   * @param {number} amount
   * @memberof Window_ShopNumber
   */
  changeNumber(amount: number): void;

  /**
   * Handle for button up event.
   */
  onButtonUp(): void;

  /**
   * Handler for button up event 2.
   */
  onButtonUp2(): void;

  /**
   * Handler for button down event.
   */
  onButtonDown(): void;

  /**
   * Handler for button down event.
   */
  onButtonDown2(): void;

  /**
   * Handler for button ok event.
   */
  onButtonOk(): void;
}

/**
 * -----------------------------------------------------------------------------
 * Window_ShopSell
 *
 * The window for selecting an item to sell on the shop screen.
 * @class Window_ShopSell
 */
declare class Window_ShopSell extends Window_ItemList {
  constructor(x: number, y: number, width: number, height: number);

  /**
   * Determines if the item is sellable, otherwise, greyed out.
   *
   * @param {RPG.BaseItem} item
   * @returns {boolean}
   * @memberof Window_ShopSell
   */
  isEnabled(item: rm.types.BaseItem): boolean;
}

declare namespace rm.types {
  export const enum EquipTypeId {
    base = -1,
  }
}

/**
 * -----------------------------------------------------------------------------
 * Window_ShopStatus
 *
 * The window for displaying number of items in possession and the actor's
 * equipment on the shop screen.
 * @class Window_ShopStatus
 */
declare class Window_ShopStatus extends Window_Base {
  /**
   * Creates an instance of Window_ShopStatus.
   * @param {number} x
   * @param {number} y
   * @param {number} width
   * @param {number} height
   * @memberof Window_ShopStatus
   */
  constructor(x: number, y: number, width: number, height: number);

  /**
   * Refreshes the window contents.
   *
   * @memberof Window_ShopStatus
   */
  refresh(): void;

  /**
   * Sets the item in the window shop status for display.
   */
  setItem(item: rm.types.BaseItem): void;

  /**
   * Returns true if the item in the
   * shop status window is an equippable item.
   */
  isEquipItem(): boolean;

  drawPossession(x: number, y: number): void;

  /**
   * Draw Equip Information.
   */
  drawEquipInfo(x: number, y: number): void;

  statusMembers(): Game_Actor[];

  /**
   * Returns the page size.
   *
   * @returns {number}
   * @memberof Window_ShopStatus
   */
  pageSize(): number;

  /**
   * Returns the max number of pages.
   *
   * @returns {number}
   * @memberof Window_ShopStatus
   */
  maxPages(): number;

  drawActorEquipInfo(x: number, y: number, actor: Game_Actor): void;

  drawActorParamChange(x: number, y: number, actor: Game_Actor, item1: rm.types.EquipItem): void;

  /**
   * Returns the parameter id.
   *
   * @returns {number}
   * @memberof Window_ShopStatus
   */
  paramId(): number;

  /**
   * Returns the current item equiped by the given actor when
   * the respective equipment Id is passed.
   * @param {Game_Actor} actor
   * @param {number} etypeId
   * @returns {RPG.EquipItem}
   * @memberof Window_ShopStatus
   */
  currentEquippedItem(actor: Game_Actor, etypeId: rm.types.EquipTypeId): rm.types.EquipItem;

  /**
   * Updates the current page.
   *
   * @memberof Window_ShopStatus
   */
  updatePage(): void;

  /**
   * Determines if page can be changed.
   *
   * @returns {boolean}
   * @memberof Window_ShopStatus
   */
  isPageChangeEnabled(): boolean;

  isPageChangeRequested(): boolean;

  /**
   * Determines if the window is touched within it's frame.
   *
   * @returns {boolean}
   * @memberof Window_ShopStatus
   */
  isTouchedInsideFrame(): boolean;

  /**
   * Changes the current page.
   *
   * @memberof Window_ShopStatus
   */
  changePage(): void;
}

/**
 * -----------------------------------------------------------------------------
 * Window_SkillStatus
 *
 * The window for displaying the skill user's status on the skill screen.
 * @class Window_SkillStatus
 */
declare class Window_SkillStatus extends Window_Base {
  /**
   * Creates an instance of Window_SkillStatus.
   * @param x
   * @param y
   * @param width
   * @param height
   * @memberof Window_SkillStatus
   */
  constructor(x: number, y: number, width: number, height: number);

  /**
   * Sets the current actor for the window.
   *
   * @param {Game_Actor} actor
   * @memberof Window_SkillStatus
   */
  setActor(actor: Game_Actor): void;

  /**
   * Refreshes the window contents.
   *
   * @memberof Window_SkillStatus
   */
  refresh(): void;
}

/**
 * -----------------------------------------------------------------------------
 * Window_SkillType
 *
 * The window for selecting a skill type on the skill screen.
 * @class Window_SkillType
 */
declare class Window_SkillType extends Window_Command {
  /**
   * Creates an instance of Window_SkillType.
   * @param  x
   * @param  y
   * @memberof Window_SkillType
   */
  constructor(x: number, y: number);

  /**
   * Sets the current actor for the skill type window.
   *
   * @param {Game_Actor} actor
   * @memberof Window_SkillType
   */
  setActor(actor: Game_Actor): void;

  /**
   * Sets the skill window for the current skill type.
   *
   * @param {Window_SkillList} skillWindow
   * @memberof Window_SkillType
   */
  setSkillWindow(skillWindow: Window_SkillList): void;

  /**
   * Selects the last command in the window.
   *
   * @memberof Window_SkillType
   */
  selectLast(): void;
}

declare class Window_Status extends Window_StatusBase {
  constructor(rect: Rectangle);

  initialize(rect: Rectangle): void;

  /**
   * Returns the Y value for block1.
   * Defaults to 0.
   * @return Int
   */
  block1Y(): number;

  /**
   * Draws the basic info and exp info as a block.
   * ```js
   * const y = this.block2Y();
   * this.drawActorFace(this._actor, 12, y);
   * this.drawBasicInfo(204, y);
   * this.drawExpInfo(456, y);
   * ```
   */
  drawBlock2(): void;

  /**
   * Returns the Y value for block2.
   * This is based on the lineHeight.
   * ```js
   * const lineHeight = this.lineHeight();
   * const min = lineHeight;
   * const max = this.innerHeight - lineHeight * 4;
   * return Math.floor((lineHeight * 1.4).clamp(min, max));
   * ```
   * @return Int
   */
  block2Y(): number;

  /**
   * Draws the basic info such as
   * actor level, icons and guages
   * at the specified x and y coordinates.
   * @param x
   * @param y
   */
  drawBasicInfo(x: number, y: number): void;

  /**
   * Draws the experience info
   * at the specified x and y coordinates.
   * @param x
   * @param y
   */
  drawExpInfo(x: number, y: number): void;

  /**
   * Returns the exp total value as a string otherwise
   * returns "-------".
   * @return String
   */
  expTotalValue(): string;

  /**
   * Returns the exp next value as a string
   * otherise returns "-------".
   * @return String
   */
  expNextValue(): string;

  /**
   * Sets the current actor of the skill list window.
   *
   * @param {Game_Actor} actor
   * @memberof Window_SkillList
   */
  setActor(actor: Game_Actor): void;
}

/**
 * -----------------------------------------------------------------------------
 * Window_TitleCommand
 *
 * The window for selecting New Game/Continue on the title screen.
 * @class Window_TitleCommand
 */
declare class Window_TitleCommand extends Window_Command {
  protected constructor();

  updatePlacement(): void;

  isContinueEnabled(): boolean;

  initCommandPosition(): void;

  selectLast(): void;
}

interface AttackMotion {
  /**
   * The type of the motion.
   */
  type: number;

  /**
   * The ID of the weapon image.
   */
  weaponImageId: number;
}

interface Terms {
  /**
   * The basic status. A string array with the following subscripts:
   *
   * 0: Level
   * 1: Level (short)
   * 2: HP
   * 3: HP (short)
   * 4: MP
   * 5: MP (short)
   * 6: TP
   * 7: TP (short)
   * 8: EXP
   * 9: EXP (short)
   */
  basic: Array<string>;

  /**
   * Parameters. A string array with the following subscripts:
   *
   * 0: Maximum hit points
   * 1: Maximum magic points
   * 2: Attack power
   * 3: Defense power
   * 4: Magic attack power
   * 5: Magic defense power
   * 6: Agility
   * 7: Luck
   * 8: Hit
   * 9: Evasion
   */
  params: Array<string>;

  /**
   * 0: Fight
   * 1: Escape
   * 2: Attack
   * 3: Defend
   * 4: Item
   * 5: Skill
   * 6: Equip
   * 7: Status
   * 8: Sort
   * 9: Save
   * 10: Exit Game
   * 11: Option
   * 12: Weapon
   * 13: Armor
   * 14: Key Item
   * 15: Change Equipment
   * 16: Ultimate Equipment
   * 17: Remove All
   * 18: New Game
   * 19: Continue
   * 20: (not used)
   * 21: Go to Title
   * 22: Cancel
   * 23: (not used)
   * 24: Buy
   * 25: Sell
   */
  commands: Array<string>;

  /**
   * The messages.
   */
  messages: { [key: string]: string };
}

interface TestBattler {
  /**
   * The actor ID.
   */
  actorId: number;

  /**
   * The actor's level.
   */
  level: number;

  /**
   * The actor's equipment. An array of weapon IDs or armor IDs with the following subscripts:
   *
   * 0: Weapon
   * 1: Shield
   * 2: Head
   * 3: Body
   * 4: Accessory
   */
  equips: Array<number>;
}

type MapInfo = {
  /**
   * The map name.
   */
  name: string;

  /**
   * The parent map ID.
   */
  parentId: number;

  /**
   * The map tree display order, which is used internally.
   */
  order: number;
};

interface MetaData {
  /**
   * The text of the note.
   */
  note: string;

  /**
   * The Meta.
   */
  meta: { [key: string]: any };
}

/**
 * The data class for maps.
 */
interface Map extends MetaData {
  /**
   * The map's display name.
   */
  displayName: string;

  /**
   * The map's tile set.
   */
  tilesetId: number;

  /**
   * The map's width.
   */
  width: number;

  /**
   * The map's height.
   */
  height: number;

  /**
   * The scroll type (0: No Loop, 1: Vertical Loop, 2: Horizontal Loop, 3: Both Loop).
   */
  scrollType: number;

  /**
   * The truth value indicating whether the battle background specification is enabled.
   */
  specifyBattleback: boolean;

  /**
   * The file name of the floor graphic if the battle background specification is enabled.
   */
  battleback1Name: string;

  /**
   * The file name of the wall graphic if the battle background specification is enabled.
   */
  battleback2_name: string;

  /**
   * The truth value indicating whether BGM autoswitching is enabled.
   */
  autoplayBgm: boolean;

  /**
   * The name of that BGM (RPG.AudioFile) if BGM autoswitching is enabled.
   */
  bgm: rm.types.AudioFile;

  /**
   * The truth value indicating whether BGS autoswitching is enabled.
   */
  autoplayBgs: boolean;

  /**
   * The name of that BGS (RPG.AudioFile) if BGS autoswitching is enabled.
   */
  bgs: rm.types.AudioFile;

  /**
   * The truth value of the [Disable Dashing] option.
   */
  disableDashing: boolean;

  /**
   * An encounter list. A RPG.Map.Encounter ID array.
   */
  encounterList: Array<rm.types.MapEncounter>;

  /**
   * The average number of steps between encounters.
   */
  encounterStep: number;

  /**
   * The file name of the parallax background's graphic.
   */
  parallaxName: string;

  /**
   * The truth value of the [Loop Horizontal] option for the parallax background.
   */
  parallaxLoopX: boolean;

  /**
   * The truth value of the [Loop Vertical] option for the parallax background.
   */
  parallaxLoopY: boolean;

  /**
   * The automatic x-axis scrolling speed for the parallax background.
   */
  parallaxSx: number;

  /**
   * The automatic y-axis scrolling speed for the parallax background.
   */
  parallaxSy: number;

  /**
   * The truth value of the [Show in the Editor] option for the parallax background.
   */
  parallaxShow: boolean;

  /**
   * The map data. A 3-dimensional tile ID array (Table).
   */
  data: Array<number>;

  /**
   * The array of RPG.Event data.
   */
  events: Array<Event>;
}

interface System {
  /**
   * The game title.
   */
  gameTitle: string;

  /**
   * A random number used for update checks. The number changes every time data is saved in RPG Maker.
   */
  versionId: number;

  /**
   * The locale string such as "ja_JP" and "en_US".
   */
  locale: string;

  /**
   * The initial party. An array of actor IDs.
   */
  partyMembers: Array<number>;

  /**
   * The unit of currency.
   */
  currencyUnit: string;

  /**
   * The window color.
   */
  windowTone: Array<number>;

  /**
   * The array of System.AttackMotion data.
   */
  attackMotions: Array<rm.types.Motion>;

  /**
   * A list of elements. A string array using element IDs as subscripts, with the element in the 0 position being nil.
   */
  elements: Array<string>;

  /**
   * he equipment type. A string array with the following subscripts:
   * 1: Weapon
   * 2: Shield
   * 3: Head
   * 4: Body
   * 5: Accessory
   */
  equipTypes: Array<string>;

  /**
   * A list of skill types. A string array using skill type IDs as subscripts, with the element in the 0 position being nil.
   */
  skillTypes: Array<string>;

  /**
   * A list of weapon types. A string array using weapon type IDs as subscripts, with the element in the 0 position being nil.
   */
  weaponTypes: Array<string>;

  /**
   * A list of armor types. A string array using armor type IDs as subscripts, with the element in the 0 position being nil.
   */
  armorTypes: Array<string>;

  /**
   * A switch name list. A string array using switch IDs as subscripts, with the element in the 0 position being nil.
   */
  switches: Array<string>;

  /**
   * A variable name list. A string array using variable IDs as subscripts, with the element in the 0 position being nil.
   */
  variables: Array<string>;

  /**
   * Boat settings (RPG.System.Vehicle).
   */
  boat: rm.types.SystemVehicle;

  /**
   * Ship settings (RPG.System.Vehicle).
   */
  ship: rm.types.SystemVehicle;

  /**
   * Airship settings (RPG.System.Vehicle).
   */
  airship: rm.types.SystemVehicle;

  /**
   * The file name of the title (background) graphic.
   */
  title1Name: string;

  /**
   * The file name of the title (frame) graphic.
   */
  title2Name: string;

  /**
   * The truth value of the [Draw Game Title] option.
   */
  optDrawTitle: boolean;

  /**
   * The truth value of the [Start Transparent] option.
   */
  optTransparent: boolean;

  /**
   * The truth value of the [Show Player Followers] option.
   */
  optFollowers: boolean;

  /**
   * The truth value of the [K.O. by Slip Damage] option.
   */
  optSlipDeath: boolean;

  /**
   * The truth value of the [K.O. by Floor Damage] option.
   */
  optFloorDeath: boolean;

  /**
   * The truth value of the [Display TP in Battle] option.
   */
  optDisplayTp: boolean;

  /**
   * The truth value of the [Reserve Members' EXP] option.
   */
  optExtraExp: boolean;

  /**
   * The truth value of the [use side-view battle] option.
   */
  optSideView: boolean;

  /**
   * The title BGM (RPG.AudioFile).
   */
  titleBgm: rm.types.AudioFile;

  /**
   * The battle BGM (RPG.AudioFile).
   */
  battleBgm: rm.types.AudioFile;

  /**
   * The battle end ME (RPG.AudioFile).
   */
  battleEndMe: rm.types.AudioFile;

  /**
   * The gameover ME (RPG.AudioFile).
   */
  gameoverMe: rm.types.AudioFile;

  /**
   * Sound effects. An RPG.SE array.
   */
  sounds: Array<rm.types.AudioFile>;

  /**
   * The map ID of the player's initial position.
   */
  startMapId: number;

  /**
   * The map's x-coordinate of the player's initial position.
   */
  startX: number;

  /**
   * The map's y-coordinate of the player's initial position.
   */
  startY: number;

  /**
   * Terms (RPG.System.Terms).
   */
  terms: Terms;

  /**
   * Party settings for battle tests. An RPG.System.TestBattler array.
   */
  testBattlers: Array<TestBattler>;

  /**
   * The enemy troop ID for battle tests.
   */
  testTroopId: number;

  /**
   * The file name of the battle background (floor) graphic for use in editing enemy troops and battle tests.
   */
  battleback1Name: string;

  /**
   * The file name of the battle background (wall) graphic for use in editing enemy troops and battle tests.
   */
  battleback2Name: string;

  /**
   * The battler graphic file name for use in editing animations.
   */
  battlerName: string;

  /**
   * The adjustment value for the battler graphic's hue (0..360) for use in editing animations.
   */
  battlerHue: number;

  /**
   * The ID of the map currently being edited. For internal use.
   */
  editMapId: number;
}

/** @global RPGMaker Plugin's Object */
declare let $plugins: Array<rm.types.PluginSettings>;
/** @global RPGMakerMV Actor data. */
declare let $dataActors: Array<rm.types.Actor>;
/** @global RPGMakerMV Class data. */
declare let $dataClasses: Array<rm.types.RPGClass>;
/** @global RPGMakerMV Skill data. */
declare let $dataSkills: Array<rm.types.Skill>;
/** @global RPGMakerMV Item data. */
declare let $dataItems: Array<rm.types.Item>;
/** @global RPGMakerMV Weapon data. */
declare let $dataWeapons: Array<rm.types.Weapon>;
/** @global RPGMakerMV Armor data. */
declare let $dataArmors: Array<rm.types.Armor>;
/** @global RPGMakerMV Enemy data. */
declare let $dataEnemies: Array<rm.types.Enemy>;
/** @global RPGMakerMV Troop data. */
declare let $dataTroops: Array<rm.types.Troop>;
/** @global RPGMakerMV State data. */
declare let $dataStates: Array<rm.types.State>;
/** @global RPGMakerMV Animation data. */
declare let $dataAnimations: Array<rm.types.Animation>;
/** @global RPGMakerMV Tileset data. */
declare let $dataTilesets: Array<rm.types.Tileset>;
/** @global RPGMakerMV CommonEvent data. */
declare let $dataCommonEvents: Array<rm.types.CommonEvent>;
/** @global RPGMakerMV System data. */
declare let $dataSystem: System;
/** @global RPGMakerMV MapInfo data. */
declare let $dataMapInfos: Array<MapInfo>;
/** @global RPGMakerMV Map data for the current map. */
declare let $dataMap: Map;
/** @global RPGMakerMV Temporary game data; not saved with the game. */
declare let $gameTemp: Game_Temp;
/** @global RPGMakerMV Game System data; saved with the game.
 * @type {Game_Temp}
 */
declare let $gameSystem: Game_System;
/** @global RPGMakerMV Game Screen; contains properties and methods
 * for adjusting the game screen.
 * @type {Game_Screen}
 */
declare let $gameScreen: Game_Screen;
declare let $gameTimer: Game_Timer;
/** @global RPGMakerMV Game Message; contains properties and methods
 * for displaying messages in the game message window.
 * @type {Game_Message}
 */
declare let $gameMessage: Game_Message;
/** @global RPGMakerMV Game Switches; contains properties and methods
 * for modifying in game switches while the game is running.
 * These are boolean values: true or false.
 * @type {Game_Switches}
 */
declare let $gameSwitches: Game_Switches;
/** @global RPGMakerMV Game Variables; contains properties and methods
 * for modifying the values of game variables.
 * The variables can contain anything.
 * @type {Game_Variables}
 */
declare let $gameVariables: Game_Variables;
declare let $gameSelfSwitches: Game_SelfSwitches;
declare let $gameActors: Game_Actors;
/** @global RPGmakerMV Game Party; contains properties and methods
 * for interacting with the game party. Some of the methods include
 * number of party members, etc.
 * @type {Game_Party}
 */
declare let $gameParty: Game_Party;
/** @global RPGMakerMV Game Troop; contains properties and methods
 * for interacting with the game troops. Some of the methods include
 * enemy data, enemy names, etc.
 * @type {Game_Troop}
 */
declare let $gameTroop: Game_Troop;
/** @global RPGMakerMV Game Map; contains properties and methods
 * for interacting with the game map. Some of these methods include
 * interacting with the map's game_interpreter, and event information.
 * @type {Game_Map}
 */
declare let $gameMap: Game_Map;
/** @global RPGMakerMV Game Player; contains properties and methods
 * for interacting with the game player. Some of these methods
 * include interacting with the player's position and move route.
 * @type {Game_Player}
 */
declare let $gamePlayer: Game_Player;
declare let $testEvent: Array<rm.types.EventCommand>;
